/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.BasicTreeItem'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web', 'BasicTreeItemMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.ui.tree.AbstractItem'); //$NON-NLS-1$

(function() {
	/**
	 * A node that works with a `jazz.ui.Tree`. Similar to IconItem, but supports
	 * progress (load) icons and more customization points. 
	 */
	dojo.declare(
		'com.ibm.relm.common.web.BasicTreeItem', //$NON-NLS-1$
		[ jazz.ui.tree.AbstractItem, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'templates/BasicTreeItem.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web', 'BasicTreeItemMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			_spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web','ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			label: '',		// The label for the node //$NON-NLS-1$
			iconClass: '',	// The class that gives the background image to use for the icon //$NON-NLS-1$

			/**
			 * Optional URL. If set, the labels of the node will appears as links
			 * with this as the target HREF. When click, the link will actually trigger
			 * the tree's "onEnter" callback and set the selection to the clicked node.
			 * @type {string}
			 */
			href: null,

			postCreate: function() {
				this.inherited(arguments);
			},

			_setLoadingAttr: function(isLoading) {
				this._set('loading', isLoading); //$NON-NLS-1$

				dijit.setWaiState(this.domNode, 'busy', isLoading); //$NON-NLS-1$

				this._loadingImg && dojo.destroy(this._loadingImg);
				delete this._loadingImg;

				this._icon.style.display = isLoading ? 'none' : ''; //$NON-NLS-1$ //$NON-NLS-2$

				if (!isLoading) return;

				this._loadingImg = dojo.create('img', { //$NON-NLS-1$
					alt: this.messages.loadingAltText,
					src: this._spinnerSrc,
					className: 'ple-icon' //$NON-NLS-1$
				});
				dojo.place(this._loadingImg, this.domNode, 'first'); //$NON-NLS-1$
			},

			_setHrefAttr: function(href) {
				this._set('href', href); //$NON-NLS-1$

				if (!this._label) return;

				var link = dojo.create('a', { //$NON-NLS-1$
					href: href,
					tabIndex: -1
				});

				this.connect(link, 'onclick', function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					this.tree.setSelection(this);
					this.tree.onEnter(e);
				});

				link.appendChild(this._label);
				dojo.place(link, this._icon, 'after'); //$NON-NLS-1$
			},

			getFocusNode: function() {
				return this._label;
			}
		}
	);

})();