/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	 /**summary:
	  * Implement Relm's own actions for the Impact Analysis Diagram context-menu
	  * 
	  * description:
	  * Provider for RELM-based context-menu items to Impact Analysis. */
	dojo.provide("relm.iahost.DiagramMenuProvider"); //$NON-NLS-1$
	
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu"); //$NON-NLS-1$
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram"); //$NON-NLS-1$
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition"); //$NON-NLS-1$

	dojo.require('jazz.ui.Menu'); //$NON-NLS-1$

	//Overrides dojo.i18n.getLocalization to allow for renaming of term product
	dojo.require('com.ibm.relm.common.web.ui.ProductNameCache'); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("relm.iahost", "DiagramMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	var messages = dojo.i18n.getLocalization("relm.iahost", "DiagramMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var nodeMenuPrototype = dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu.prototype;
	var _oldNodeMenuInit = nodeMenuPrototype._initMenuItems;

	var iaDiagramPrototype = dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram.prototype;
	var _oldToolbarLoader = iaDiagramPrototype._loadToolButtons;

	var traversalDefPrototype = dm.analysis.app.ui.internal.impactanalysis.profile.TraversalDefinition.prototype;
	var _oldBoundaryTypeButtonsLoader = traversalDefPrototype._addBoundaryTypeButtons;

	//FIND USE:
	/** Determine if the Find Use action is to be disabled. 
	 *  The find use can only be performed if there is a URI for the resource.
	 *
	 * @return true (disable) if the resource does not have a URI
	 */
	var _disableFindUseCallback = function(graphElement) {
		var nodeResourceUri = graphElement.getData()._resourceLink.uri;
		
		// Disable Find Use if:
		// No Uri
		// OR Uri is an internal Product Definition type
		// OR Uri represents a primitive type defined by open-services or w3.org
		var disable = !nodeResourceUri ||
						nodeResourceUri.indexOf('http://jazz.net/ns/pd#') === 0 || //$NON-NLS-1$
						nodeResourceUri.indexOf('http://open-services.net') === 0 || //$NON-NLS-1$
						nodeResourceUri.indexOf('http://www.w3.org') === 0; //$NON-NLS-1$
		return disable;
	};
	var enableForNodes_findUse = function(nodes){
		if (nodes.length !== 1) return false; //can only operate on a single node

		return !_disableFindUseCallback(nodes[0]);
	};
	var findUseOnSelection = function(){
		var selection = this.impactDiagram.getSelection().get();

		if (selection.count === 1){
			var node = this.impactDiagram.asGraphElement(selection.item(0));
			if(!node._isIBMDiagramLink){
				_findUseCallback(node);
			}
		}
	};

	/** 
	 * Callback to perform Find Use on a resource.
	 */
	var _findUseCallback = function(graphElement, event) {
		var data = graphElement.getData();
		var nodeResourceUri = data._resourceLink.uri;

		var regexProductResourceMatch = /\/productdef\/resource\/[^\/]+_\d+/.exec(nodeResourceUri);
		var isItemVersionResource = !!regexProductResourceMatch;

		if (nodeResourceUri) {
			query = {
				action: 'ple.find_use', //$NON-NLS-1$
				time: new Date().getTime()
			};

			if (isItemVersionResource)
				query.product = nodeResourceUri;
			else
				query.artifact = nodeResourceUri;

			window.location.href = '#' + dojo.objectToQuery(query); //$NON-NLS-1$
		}
	};


	var menuItem_findUse = function() {
		var self = this;
		return new dijit.MenuItem({
			label: messages.actionFindUse,
			iconClass: 'ia-icon-findUse', //$NON-NLS-1$ 
			disabled: false,
			onClick: function(e) {
				var node = self.diagram.impactDiagram.lastMenuTarget;
				dojo.hitch(this, _findUseCallback, node)();
			}
		});
	};

	/** 
	 * Prototype injection for DiagramNodeMenu
	 */
	nodeMenuPrototype._initMenuItems = function() {
		dojo.hitch(this, _oldNodeMenuInit)();

		//Add 'Find Use' action
		this._menuItems.addNodeMenuItem({
			menuItem: dojo.hitch(this, menuItem_findUse)(),
			disableCallback: dojo.hitch(this, _disableFindUseCallback)
		});
	};
	
	/** 
	 * Hide the Open Legend button.
	 *
	 * ALSO add the 'Find Use' action to the Node Toolbar
	 *
	 * We do this because RELM has nothing interesting in the Legend.
	 * (Since RELM lists all artifactas as "All Domains")
	 */
	iaDiagramPrototype._loadToolButtons = function() {
		dojo.hitch(this, _oldToolbarLoader)();

		//Remove the last 2 items on the toolbar (the separator and the Open Legend button)
		this.toolbar.removeItem(this.toolbar.items.length-1);
		this.toolbar.removeItem(this.toolbar.items.length-1);

		//Add the 'Find Use' action to the Node Toolbar
		this.nodeToolbar.addSeparator();

		this.findUseButton = this.nodeToolbar.addButton(this, findUseOnSelection, {
			imgSrc: dojo.moduleUrl('relm.iahost', 'graphics/icons/findUse.gif'),  //$NON-NLS-1$ //$NON-NLS-2$
			label:  messages.actionFindUse
		});
		this.findUseButton.enableForNodes = enableForNodes_findUse;
		this.nodeToolbar.disableButton(this.findUseButton);
		this.nodeToolbarButtons.push(this.findUseButton);
	};

	/**
	 * Show the 'add boundary type' buttons.
	 * 
	 * Currently only available in RELM because resource types aren't provided in DM yet
	 */
	traversalDefPrototype._addBoundaryTypeButtons = function(){
		dojo.hitch(this, _oldBoundaryTypeButtonsLoader)();

		dojo.removeClass(this._upstreamActionToolbar.domNode, 'hidden'); //$NON-NLS-1$
		dojo.removeClass(this._downstreamActionToolbar.domNode, 'hidden'); //$NON-NLS-1$
		dojo.removeClass(this._fullTraceActionToolbar.domNode, 'hidden'); //$NON-NLS-1$
	 };

}());