/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	/**
	 * A simple status to show in dialog with a spinner image and a text message.
	 * By default, this will mirror any message sent to StatusMessage.show.
	 * This is an easy way to add feedback to a dialog without needing
	 * to publish multiple messages.
	 *
	 * It will show itself only when a messages is being displayed and will
	 * be hidden along with the StatusMessage messsage.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.DialogStatusMessage', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templateString: '<div class="relm-DialogStatusMessage" style="display: none;"> <img src="${spinnerSrc}"> <span dojoAttachPoint="_messageNode" class="j-hint"></span> </div>', //$NON-NLS-1$
			spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * If true, will automatically hide and show to match the messages of StatusMessage.
			 * This also implicitly hides the page level (green bar) status message. So that messages
			 * aren't being shown in two places.
			 * @type {Boolean} true to mirror status messages. False to control yourself.
			 */
			mirrorStatusMessage: true,

			postCreate: function() {
				this.inherited(arguments);

				if (this.mirrorStatusMessage) {
					this.subscribe('relm.status_message.show', this.show); //$NON-NLS-1$
					this.subscribe('relm.status_message.hide', this.hide); //$NON-NLS-1$

					this._pageStatus = dojo.query('#content-cell .status-message'); //$NON-NLS-1$
					this._pageStatus.addClass('ple-DialogStatusMessage-hiddenPageStatus'); //$NON-NLS-1$
				}
			},

			/**
			 * Show this widget (with spinner graphic) and show the given message.
			 * The last message, is any, is lost.
			 * @param  {string} message the message to show.
			 */
			show: function(message) {
				dojo.empty(this._messageNode);
				// Adding ... is not right, but it's what PageTemplate.js already does
				this._messageNode.appendChild(document.createTextNode(message + '...')); //$NON-NLS-1$

				this.domNode.style.display = ''; //$NON-NLS-1$
			},

			/**
			 * Hide this entire widget.
			 */
			hide: function() {
				this.domNode.style.display = 'none'; //$NON-NLS-1$
			},

			uninitialize: function() {
				if (this._pageStatus) {
					this._pageStatus.removeClass('ple-DialogStatusMessage-hiddenPageStatus'); //$NON-NLS-1$
				}
			}
		}
	);

})();