/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	// summary:
	//	Implement required diagram utilities for Impact Analysis
	//
	// description:
	//	Provider a RELM-based compact rendering services to the impact analysis UI.
	dojo.provide('relm.iahost.HostDiagramUtilityProvider'); //$NON-NLS-1$
	
	dojo.require('dm.analysis.app.ui.DiagramUtilityMethods'); //$NON-NLS-1$
	
	var ACTION_PREFIX = "#action="; //$NON-NLS-1$
	var IA_PREFIX = "dm.app.analysis."; //$NON-NLS-1$

	var DIAGRAM_RESOURCE = "impactanalysis"; //$NON-NLS-1$
	var FILTER_RESOURCE = "impactconfig"; //$NON-NLS-1$
	var PROFILE_RESOURCE = "impactprofile"; //$NON-NLS-1$

	var EDIT_DIAGRAM_ACTION = "impactanalysis_impactAnalysis"; //$NON-NLS-1$
	var EDIT_FILTER_ACTION = "impactanalysis_iaConfig"; //$NON-NLS-1$
	var EDIT_PROFILE_ACTION = "impactanalysis_iaProfile"; //$NON-NLS-1$

	/*
	 * In RELM, there is only one context, so just the resource URI should be enough to get a compact rendering.
	 */
	getCompactRenderingUri = function(resourceUri){
		return resourceUri;
	};

	getResourceEditUri = function(resourceUri){
		var action = _getActionForResource(resourceUri);
		if (!action)
			return resourceUri;

		action = ACTION_PREFIX + IA_PREFIX + action;

		var resource = '&' + dojo.objectToQuery({"resourceUri": resourceUri}); //$NON-NLS-1$ //$NON-NLS-2$

		var editorUri = action + resource;

		return editorUri;
	};

	_isRELMResource = function (resourceUri) {
		return (resourceUri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === 0);
	};

	_getActionForResource = function (resourceUri){
		var action = ""; //$NON-NLS-1$

		if (_isRELMResource(resourceUri)){
			var resource = resourceUri.replace(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL, ""); //$NON-NLS-1$
			resource = resource.split('/'); //$NON-NLS-1$

			if(resource.length > 1){
				switch(resource[1]){
					case DIAGRAM_RESOURCE:
						action = EDIT_DIAGRAM_ACTION;
						break;
					case FILTER_RESOURCE:
						action = EDIT_FILTER_ACTION;
						break;
					case PROFILE_RESOURCE:
						action = EDIT_PROFILE_ACTION;
						break;
					default:
						break;
				}
			}
		}

		return action;
	};

	// Register the utility methods with the common IA
	dm.analysis.app.ui.DiagramUtilityMethods.set_getCompactRenderingUri(getCompactRenderingUri);
	dm.analysis.app.ui.DiagramUtilityMethods.set_getResourceEditUri(getResourceEditUri);
	dm.analysis.app.ui.DiagramUtilityMethods.enableFullTrace(true);

}());