/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
	// summary:
	//	Implement a resource picker for Impact Analysis
	//
	// description:
	//	Provider a RELM-based picker to the impact analysis engine.
	dojo.provide('relm.iahost.IAResourcePickerProvider'); //$NON-NLS-1$

	dojo.require('com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker'); //$NON-NLS-1$
	dojo.require('dm.analysis.app.ui.picker'); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("relm.iahost", "IAResourcePickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox"); //$NON-NLS-1$
	dojo.require("dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox"); //$NON-NLS-1$

	var messages = dojo.i18n.getLocalization("relm.iahost", "IAResourcePickerMessages"), //$NON-NLS-1$ //$NON-NLS-2$
		pickerFunction;

	dojo.declare('relm.iahost._IAResourcePicker', com.ibm.relm.ple.web.ui.picker.LqeArtifactPicker, { //$NON-NLS-1$
		_setResourceCallback: null,

		constructor: function (args) {
			this._setResourceCallback = args.setResourceCallback || function () {};
		},

		onOk: function (resource) {
			this._setResourceCallback.call(null, resource.uri);
			this.close();
		}
	});

	// listen for picker requests from IA, and show the dialog...
	pickerFunction = function (setResourceCallback) {
		var picker = new relm.iahost._IAResourcePicker({
			dialogTitle: messages.pickerTitle,
			setResourceCallback: setResourceCallback,
			hideBaselines: true
		});
		picker.startup();
	};

	dm.analysis.app.ui.picker.setPicker(pickerFunction);

	var iaSearchBoxPrototype = dm.analysis.app.ui.internal.impactanalysis.picker.IASearchBox.prototype;
	var iaConfigurationSearchBoxPrototype = dm.analysis.app.ui.internal.impactanalysis.picker.IAConfigurationSearchBox.prototype;

	function _formatResults(searchResults) {
		var formattedResults = [];
		var hits = searchResults.hits;
	
		for (var i = 0; i < hits.length; i++) {
			var hit = hits[i];

			var label =
				'<span class="IASearchBox-search-result">' + //$NON-NLS-1$
					'<img height="16" width="16" src="' + hit.iconUrl +'">' + //$NON-NLS-1$ //$NON-NLS-2$
					hit.text +
				'<span>'; //$NON-NLS-1$
			
			var singleResult = {
				label: label,
				handler: this.getHandler(hit.uri, label)
			};

			formattedResults.push(singleResult);
		}
		return formattedResults;
	}
	
	function performUpdate(query, scope, callback, errorCallback) {
		if(this._previousQuery != query) {
			this._previousQuery = query;
		}
		
		var self = this;

		if (this._prevXhr){
			this._prevXhr.cancel();
		}
		
		var content =  {
			decorateProductTitles: true,
			pageSize: 25,
			pageNumber: 1,
			q: query,
			// No paging of these results, so always use a new `timestamp`
			// to disable caching.
			timestamp: new Date().getTime(),
		};
		
		if (scope)
		{
			content['filterTypesIgnore'] = scope.filterTypesIgnore; //$NON-NLS-1$
			content['filteredTypes'] = scope.filteredTypes; //$NON-NLS-1$
		}
		
		this._prevXhr = jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
			handleAs: 'json', //$NON-NLS-1$
			content: content,
			load: function(data, ioArgs){
				var queryResults = dojo.hitch(self, _formatResults)(data);
				callback(queryResults);
			},
			error: function(data, ioArgs){
				var errorSummary = dojo.fromJson(ioArgs.xhr.responseText).summary;
				errorCallback(errorSummary);
			},
			handle: function(){
				self._prevXhr = null;
			}
		});
	}

	iaSearchBoxPrototype.performUpdate = function(query, scope, callback, errorCallback) {
		dojo.hitch(this, performUpdate)(query, scope, callback, errorCallback);
	};
	iaConfigurationSearchBoxPrototype.performUpdate = function(query, scope, callback, errorCallback) {
		var scope = {
			filterTypesIgnore : false,
			filteredTypes : "['http://jazz.net/ns/pd#Configuration']", 	 //$NON-NLS-1$
		};
		dojo.hitch(this, performUpdate)(query, scope, callback, errorCallback);
	};
}());