/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.common.web.ProgressBarDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.common.web", "ProgressBarDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization("com.ibm.relm.common.web", "ProgressBarDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.declare(
		"com.ibm.relm.common.web.ProgressBarDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'templates/ProgressBarDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: messages,

			primaryTitle: '', //$NON-NLS-1$
			explanation: '', //$NON-NLS-1$
			width: '350px', //$NON-NLS-1$
			value: 0,
			iconClass: null,
			closeTimeout: 150,
			progressMessage: '', //$NON-NLS-1$

			constructor: function(args, node) {
				if (args.progressMessage) {
					this.progressMessage = dojo.replace(this.messages.progressMessage, [args.progressMessage]);
				}
			},

			postCreate: function() {
				this.inherited(arguments);

				if (!this.progressMessage) {
					this.progressMessageArea.style.display = "none"; //$NON-NLS-1$
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.primaryTitle || messages.defaultTitle,
					width: this.width,
					closable: false,
					destroyContents: true,
					contentNode: this.domNode
				});
			},

			_setValueAttr: function(value) {
				this.progressBar.set('value', value); //$NON-NLS-1$
			},

			/**
			 * Notifies the dialog that the operation in progress is complete. Waits for a 
			 * specified timeout before closing the dialog. 
			 * 
			 * @param  {integer} wait How long to wait before the dialog closes. If nothing
			 *                        is provided, then the default is used.
			 */
			complete: function(wait) {
				window.setTimeout(dojo.hitch(this, function() {
					this._dialog.close();
				}), wait || this.closeTimeout);
			}
		});

})();