dojo.provide('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$

dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$

(function(){

	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	var messageStack_ = [];
	var nextId_ = 0;

	/**
	 * Removes the message from the stack, whether its on top or not.
	 * If it was showing (on top) it is hidden, and the new top messages
	 * is shown. If it was the last message, then the status message area
	 * is hidden completely.
	 * 
	 * @param  {number} messageId ID of message to hide.
	 */
	function popMessageId(messageId) {
		// Walk backwards so we don't modify indices we haven't seen, plus
		// a message is more likely near the end when removed.
		for (var i = messageStack_.length - 1; i >= 0; i--) {
			var info = messageStack_[i];
			if (info.id === messageId) {
				window.clearTimeout(info.timeoutId);
				messageStack_.splice(i, 1);
			}
		}

		var remaining = messageStack_.length;

		if (remaining) {
			var topMessage = messageStack_[remaining - 1].message;
			PlatformUI.getWorkbench().showMessage(topMessage);
			dojo.publish('relm.status_message.show', [topMessage]); //$NON-NLS-1$
		}
		else {
			dojo.publish('relm.status_message.hide'); //$NON-NLS-1$
			PlatformUI.getWorkbench().hideMessageArea();
			dojo.removeClass(document.body, 'relm-busy'); //$NON-NLS-1$
		}
	}

	/**
	 * Creates a new MessageInfo and adds it to the stack.
	 * Shows the message.
	 * @param  {string} message the message
	 * @return {MessageInfo}         use to .hide() the message later.
	 */
	function pushMessage(message) {
		var info = new MessageInfo(nextId_++, message);
		messageStack_.push(info);
		PlatformUI.getWorkbench().showMessage(message);
		dojo.addClass(document.body, 'relm-busy'); //$NON-NLS-1$
		var timeoutId = window.setTimeout(function() {
			dojo.publish('relm.status_message.show', [message]); //$NON-NLS-1$
		}, 300);
		info.timeoutId = timeoutId;
		return info;
	}
	
	/**
	 * The handle to the message returned to the caller and 
	 * used to hide the message. Also added to the stack of messages
	 * to track the IDs.
	 * @param id unique ID for this message.
	 * @param message the string to show. A single line.
	 */
	function MessageInfo(id, message) {
		this.id = id;
		this.message = message;
	}

	/**
	 * Hide the associated message. May close the status message
	 * area if there are not other messages, otherwise, shows the
	 * next most recent message.
	 */
	MessageInfo.prototype.hide = function() {
		popMessageId(this.id);
	};

	/**
	 * Provides an improved interface to PlatformUI.getWorkbench().showMessage.
	 * Using .showMessage directly leads to problems since there is only
	 * a single method to hide the current message, whatever that message is.
	 * This module maintains a stack of messages so that several can be be stacked
	 * on top of each other. When a new message is shown, it covers the current message,
	 * but when the latest message is hidden, the previous message will be shown again.
	 * Each message has its own unique ID, so that when hidden, it will only hide that specific message
	 * (whether visible or not currently), and will never hide unrelated messages.
	 * Only after all messages have been hidden will the status line go away.
	 */
	dojo.mixin(com.ibm.relm.common.web.StatusMessage, {

		/**
		 * Shows a messages in the top (green) status message line.
		 * The message can only be hidden with the returned message handle.
		 * @param  {string} message the message to show. Do NOT include a trailing ellipses; one
		 * is added automatically by Jazz.
		 * @return {MessageInfo}         an opaque handle to the message. Exposes a .hide() method
		 * that must be used to hide the message.
		 */
		show: pushMessage

	});

})();
