/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.TabGroup'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){
	/**
	 * Used internally by a TabGroup when new tabs are added. 
	 * Other classes should not create directly, but can use instances
	 * returned by `addTab` to manually activate a tab.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.Tab', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'templates/Tab.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,

			name: '', //$NON-NLS-1$

			// Either a DOM node to show and hide or an object
			// of two functions (.show and .hide) to call.
			content: null,
			
			/**
			 * TabGroup instance that owns this tab.
			 * @type {[type]}
			 */
			tabGroup: null,

			_onActivate: function(e) {
				dojo.stopEvent(e);
				this.activate();
			},

			/**
			 * Activates a tab. Brings if forward and implicitly deactivates
			 * the previous active tab.
			 */
			activate: function () {
				if (this.tabGroup.activeTab === this) return;
				
				dojo.addClass(this.domNode, 'relm-Tab-active'); //$NON-NLS-1$
				if (this.content) {
					if (this.content.show) {
						this.content.show();
					}
					else this.content.style.display = ''; //$NON-NLS-1$
				}
				dojo.attr(this._nameNode, 'tabindex', '0'); //$NON-NLS-1$ //$NON-NLS-2$
				this._nameNode.focus();
				this.tabGroup._tabDidActivate(this);
			},

			/**
			 * Deactivates a tab. Implicitly called when another tab is activated.
			 * @return {[type]} [description]
			 */
			deactivate: function() {
				dojo.removeClass(this.domNode, 'relm-Tab-active'); //$NON-NLS-1$
				if (this.content) {
					if (this.content.hide) {
						this.content.hide();
					}
					else this.content.style.display = 'none'; //$NON-NLS-1$
				}
				dojo.attr(this._nameNode, 'tabindex', '-1'); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	);
	
	/**
	 * Controls a set of related tabs used to switch between several content view.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.TabGroup', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'templates/TabGroup.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,

			activeTab: null,

			constructor: function() {
				this.tabs = [];
				this.activeTab = null;
			},

			_tabDidActivate: function(tab) {
				if (tab === this.activeTab) return;

				this.activeTab && this.activeTab.deactivate();

				this.set('activeTab', tab); //$NON-NLS-1$
			},

			_onKeyDown: function(e) {
				var k = e.keyCode;

				if (!this.activeTab) return;
				var activeIndex = dojo.indexOf(this.tabs, this.activeTab);

				if (k === dojo.keys.LEFT_ARROW) {
					activeIndex -= 1;
				}
				else if (k === dojo.keys.RIGHT_ARROW) {
					activeIndex += 1;	
				}

				activeIndex = Math.max(0, activeIndex);
				activeIndex = Math.min(this.tabs.length - 1, activeIndex);
				var tab = this.tabs[activeIndex];
				tab.activate();
			},

			/**
			 * Used to add a tab programatically to this tab group.
			 * There is no declarative way to do this.
			 * @param {string} name        the name to show on the tab.
			 * @param {DOM node OR object} content an _optional_ node to use as the content of this tab. Will be
			 * shown and hidden as tabs are clicked. Instead of a node, you can pass an object
			 * with a .show and .hide method. Instead of the tab showing and hiding content,
			 * these functions (with no scope, so use hitch if needed) will be called.
			 * If null, then no content is shown for this tab.
			 */
			addTab: function(name, content) {

				var tab = new com.ibm.relm.common.web.Tab({
					name: name,
					content: content,
					tabGroup: this
				});

				this.own(tab);

				this.tabs.push(tab);
				if (content && content.nodeName) {
					content.style.display = 'none'; //$NON-NLS-1$
					this._contentNode.appendChild(content);
				}
				tab.placeAt(this._tabsNode);

				if (this.tabs.length === 1) {
					tab.activate();
				}

				return tab;
			}
		}
	);

})();
