/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.AdvancedSettings"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "AdvancedSettingsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require("com.ibm.relm.common.web.ui.EasyMessageBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.FloatingMessageBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$
dojo.require("relm.app.Permissions"); //$NON-NLS-1$

(function(){

	var isJazzAdmin = relm.app.Permissions.isJazzAdmin();

	dojo.declare(
		"com.ibm.relm.ple.web.admin.AdvancedSettings", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/AdvancedSettings.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "AdvancedSettingsMessages"), //$NON-NLS-1$ //$NON-NLS-2$
			
			postCreate: function() {
				this.inherited(arguments);

				if (!isJazzAdmin) {
					this.variablityCheckbox.disabled = true;
					this.productExportCheckbox.disabled = true;
					this.advancedImportCheckbox.disabled = true;
				}
			},

			onChange: function() {
				this._saveButton.disabled = false;
			},

			onSave: function(e) {
				this._saveButton.disabled = true;
				this._errorBox.hideMessage();

				var settings = {
					variable_include: this.variablityCheckbox.checked,
					product_export: this.productExportCheckbox.checked,
					advanced_import: this.advancedImportCheckbox.checked
				};

				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/advanced_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: { 
						settings: dojo.toJson(settings)
					},
				
					load: function () {
						self._successBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.OK,
							summary: self.messages.saveSucceeded
						});
						self._successBox.hideLater();
						dojo.mixin(com.ibm.relm.ple.web.admin.AdvancedSettings, settings);
					},
				
					error: function(error, ioArgs) {
						self._saveButton.disabled = false;
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			refresh: function() {
				this._errorBox.hideMessage();
				this._saveButton.disabled = true;

				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/advanced_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
		
					handle: function() {
						if (isJazzAdmin) {
							self._saveButton.disabled = false;
							self._stateGroupField.disabled = false;
						}
					},

					load: function(settings) {
						dojo.mixin(com.ibm.relm.ple.web.admin.AdvancedSettings, settings);
						self.variablityCheckbox.checked = settings.variable_include;
						self.productExportCheckbox.checked = settings.product_export;
						self.advancedImportCheckbox.checked = settings.advanced_import;
					},

					error: function(error, ioArgs) {
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			}
		});

	jazz.client.xhrGet({
		url: net.jazz.ajax._contextRoot + '/ple/advanced_settings', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		failOk: true,
		load: function(settings) {
			dojo.mixin(com.ibm.relm.ple.web.admin.AdvancedSettings, settings);
		},

		error: function(error, ioArgs) {
			//Do nothing
		}
	});
})();