/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.ImportPage"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.ple.web.admin.ImportHistoryRow"); //$NON-NLS-1$

dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "ImportPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$

dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require('jazz.ui.HoverPopup');//$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$

dojo.require("relm.app.Permissions"); //$NON-NLS-1$

dojo.require("com.ibm.relm.ple.web.admin.ImportUploadDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.FloatingMessageBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.EasyMessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "ImportPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportPage", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			constructor: function(args, node) {
				this.messages = messages;
				this.rows = {};
			},

			postCreate: function() {
				this.inherited(arguments);

				var columns = [ {label: this.messages.fileName, width: "15%"}, //$NON-NLS-1$
								{label: this.messages.percentComplete, width: "13%"}, //$NON-NLS-1$
								{label: this.messages.status, width: "10%"}, //$NON-NLS-1$
								{label: this.messages.creator, width: "6%"}, //$NON-NLS-1$
								{label: this.messages.created, width: "13%"}, //$NON-NLS-1$
								{label: this.messages.baselinesCreated, width: "12%"}, //$NON-NLS-1$
								{label: this.messages.numberCreated, width: "12%"} //$NON-NLS-1$
							  ];

				this.table = new jazz.ui.StyledTable({
					columns: columns,
					trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_FIXED,
					heading: this.messages.importHistory
				}, this._importProgressTable);

				var restoreLink = dojo.create('a', { //$NON-NLS-1$
					innerHTML: this.messages.clearHistoryTitle,
					title: this.messages.clearHistoryDescription,
					href: '#', //$NON-NLS-1$
					style: 'float: right' //$NON-NLS-1$
				}, this.table._globalHeader);

				this.connect(restoreLink, 'onclick', this.clearHistory); //$NON-NLS-1$

				var startImportLink = dojo.query('a.relm-Import-importLink', this.noHistoryMessage)[0]; //$NON-NLS-1$
				this.connect(startImportLink, 'onclick', this.onImport); //$NON-NLS-1$
			},

			clearHistory: function(e) {
				if (confirm(this.messages.clearHistoryConfirmation))
				{
					dojo.stopEvent(e);
					var self = this;
					jazz.client.xhrDelete({
						url: net.jazz.ajax._contextRoot + '/ple/import', //$NON-NLS-1$
						load: function() {
							self._clearTable();
						},
						error: function(error, ioArgs) {
							self._importErrorBox.showXhrError(ioArgs.xhr);
						}
					});
				}
			},

			_clearTable: function() {
				this._importErrorBox.hide();
				this.table.clearBody();
				for (var key in this.rows) {
				  if (this.rows.hasOwnProperty(key)) {
				    this.rows[key].destroy();
				    delete this.rows[key];
				  }
				}
				dojo.removeClass(this.noHistoryMessage, 'relm-Import-yesHistory'); //$NON-NLS-1$
			},

			uninitialize: function() {
				for (var prop in this.rows) {
					this.rows[prop].distory();
				}
			},

			onImport: function(e) {
				dojo.stopEvent(e);
				var d = new com.ibm.relm.ple.web.admin.ImportUploadDialog({
					onClose: dojo.hitch(this, this.checkImportProgress)
				});
				d.startup();
			},

			refresh: function() {
				this.checkImportProgress();
			},

			checkImportProgress: function() {
				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/import', //$NON-NLS-1$,
					handleAs: "json", //$NON-NLS-1$
					load: function(allProgress) {
						dojo.toggleClass(self.noHistoryMessage, 'relm-Import-yesHistory', allProgress && allProgress.length); //$NON-NLS-1$
						var checkAgain = false;
						self.table.clearBody();
						dojo.forEach(allProgress, function(progress) {
							if (progress.status != "Complete" && progress.status != "Error") { //$NON-NLS-1$ //$NON-NLS-2$
								checkAgain = true;
							}
							self.drawHistoryRow(progress);
						});
						if (checkAgain) {
							window.setTimeout(dojo.hitch(self, self.checkImportProgress), 1000);
						}
					},
					error: function(error, ioArgs) {
						self._importErrorBox.showXhrError(ioArgs.xhr, true);
					}
				});
			},

			drawHistoryRow: function(args) {
				var rowInfo = {};
				rowInfo.id = args.id;
				var fileName;
				if (args.fileName.lastIndexOf("\\") > 0) { //$NON-NLS-1$
					fileName = args.fileName.substring(args.fileName.lastIndexOf("\\") + 1); //$NON-NLS-1$
				} else {
					fileName = args.fileName;
				}
				rowInfo.fileName = fileName;
				rowInfo.creator = args.creator;
				rowInfo.created = args.created;
				rowInfo.userReadableStatus = args.userReadableStatus;
				rowInfo.status = args.status;
				if (rowInfo.status == "Error") { //$NON-NLS-1$
					rowInfo.error = args.error;
				}
				rowInfo.percentComplete = args.percentComplete;

				//Need to do this check rather than checking individual values since 0 is valid values
				if (args.statistics) {
					rowInfo.statistics = args.statistics;
					rowInfo.numRootCreated = args.statistics.importedConfigurationVersionUris.length;
					rowInfo.numCreated = args.statistics.numberOfProductsCreated;
					// rowInfo.numSkipped = args.statistics.numberOfProductsSkppped;
				} else {
					rowInfo.numRootCreated = ""; //$NON-NLS-1$
					rowInfo.numCreated = ""; //$NON-NLS-1$
					// rowInfo.numSkipped = ""; //$NON-NLS-1$
				}
				if (this.rows[rowInfo.id]) {
					var existingRow = this.rows[rowInfo.id];
					existingRow.update(rowInfo);
					this.table.tbody.appendChild(existingRow.domNode);
				} else {
					var newRow = new com.ibm.relm.ple.web.admin.ImportHistoryRow(rowInfo, this._importErrorBox);
					newRow.startup();
					this.rows[rowInfo.id] = newRow;
					this.table.tbody.appendChild(newRow.domNode);
				}
			}
		});

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportHistoryRow", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportHistoryRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			percentComplete: 0,

			/**
			 * This is to keep track of the different pieces of information we store about an import. 
			 * The properties cannot be stored directly in the widget because it would conflict with 
			 * dojoAttachPoints
			 */
			//importInfo = {}

			/**
			 * Takes import data and converts it into a table row.
			 * 
			 * @param  {Object} args Takes 7 fields and applies them directly to the widget template:
			 *                       fileName, status, creator, created, percentComplete, numRootCreated, numCreated
			 * @return {Widget}      The row widget constructed
			 */
			constructor: function(args, messageBox) {
				this.messages = messages;
				this.importInfo = args;
				this.messageBox = messageBox;
			},

			postCreate: function() {
				this.progressBar.set('value', this.percentComplete); //$NON-NLS-1$
				this.created.innerHTML = this.formattedDateTime(this.importInfo.created);

				this.activateLinks();
			},

			formattedDateTime: function(dt) {
				return dojo.date.locale.format(
					new Date(dt), {
						formatLength: 'medium' //$NON-NLS-1$
					}
				)
			},

			activateLinks: function() {
				var hasError = this.importInfo.status == "Error" && this.importInfo.error; //$NON-NLS-1$
				var hasStats = this.importInfo.status == "Complete" && this.importInfo.statistics && this.importInfo.statistics.importedConfigurationVersionUris && this.importInfo.statistics.importedConfigurationVersionUris.length; //$NON-NLS-1$
				dojo.toggleClass(this.toggleErrorLink, 'relm-ImportHistory-noDetails', !hasError); //$NON-NLS-1$
				this.hoverPopup = new jazz.ui.HoverPopup({
					around: this.toggleErrorLink,
					createContents: dojo.hitch(this, function() {
						this._errorBox.showErrorObject(this.importInfo.error);
						return this._errorBox.domNode; }),
					openDelay: 300,
					closeDelay: 200
				});
				if (hasStats && (this.importInfo.statistics.importedConfigurationVersionUris.length > 0) && (!this._resourceLink))
				{
					var icon = dojo.create('img', { //$NON-NLS-1$
						height: '16', //$NON-NLS-1$
						width: '16', //$NON-NLS-1$
						className: 'ple-icon', //$NON-NLS-1$
						src: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/baseline.gif') //$NON-NLS-1$ //$NON-NLS-2$
					}, this.baselineResourceLink);
					
					var a = dojo.create('a', { //$NON-NLS-1$
						href: this.importInfo.statistics.importedConfigurationVersionUris[0], //$NON-NLS-1$
					}, this.baselineResourceLink);

					this._resourceLink = new jazz.ui.ResourceLink({
						iconNode: icon
					}, a);
				}
			},

			toggleError: function(e) {
				dojo.stopEvent(e);
			},

			/**
			 * Takes the same args as the constructor and updates the properties of the row
			 * 
			 * @param  {[type]} args [description]
			 */
			update: function(args) {
				var properties = ["fileName", "userReadableStatus", "creator", "created", "percentComplete", "numRootCreated", "numCreated"]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
				dojo.forEach(properties, function(prop) {
					if (prop == "percentComplete") { //$NON-NLS-1$
						this.progressBar.set('value', args.percentComplete); //$NON-NLS-1$
					} else if (prop == "created") { //$NON-NLS-1$
						this[prop].innerHTML = this.formattedDateTime(args.created);
					} else {
						this[prop].innerHTML = args[prop];
					}
				}, this);

				this.importInfo = args;
				this.activateLinks();
			},

			uninitialize: function() {
				this.hoverPopup.destroyRecursive();
				delete this.hoverPopup;
			}
		});
})();
