/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.ImportUploadDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "ImportUploadDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require("dojox.form.Uploader"); //$NON-NLS-1$
dojo.require("dojox.form.uploader.plugins.IFrame"); //$NON-NLS-1$
dojo.require("dojox.form.uploader.plugins.HTML5"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.LabeledRadio"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.AdvancedSettings"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader');//$NON-NLS-1$

//Overrides dojo.i18n.getLocalization to allow for renaming of term product
dojo.require('com.ibm.relm.common.web.ui.ProductNameCache'); //$NON-NLS-1$

(function(){

	var advancedSettings = com.ibm.relm.ple.web.admin.AdvancedSettings;

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportUploadDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportUploadDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "ImportUploadDialogMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			width: '400px', //$NON-NLS-1$
			closeTimeout: 150,

			/**
			 * If true, the dialog will remain open after the file upload is complete and the 
			 * dialog will remain open until the import operations completes. 
			 * @type {Boolean}
			 */
			showImportProgress: false,
			
			jazzProjectId: "", //$NON-NLS-1$

			postMixInProperties: function() {
				this.uploadUrl = net.jazz.ajax._contextRoot + '/ple/import'; //$NON-NLS-1$
				this.advancedImportEnabled = advancedSettings.advanced_import;
				this.jazzProjectId = jazz.app.context.get().itemId || ""; //$NON-NLS-1$
				this.selectedSiteJson = (this.site) ? dojo.toJson(this.site) : ""; //$NON-NLS-1$
			},

			postCreate: function() {
				this.usingIframe = dojo.isIE < 10;
				this.returnHtmlInput.value = this.usingIframe;

				if (this.advancedImportEnabled) {
					this.advancedImportSection.style.display = ""; //$NON-NLS-1$
				} 

				if (this.showImportProgress) {
					this.importProgressArea.style.display = ""; //$NON-NLS-1$
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.primaryTitle || this.messages.dialogTitle,
					width: this.width,
					closable: true,
					destroyContents: true,
					contentNode: this.domNode
				});
				this.setUpUploader();
			},

			setUpUploader: function() {
				this.uploadContainer.style.display = 'none'; //$NON-NLS-1$

				this.connect(this.uploader, 'onError', this.onError); //$NON-NLS-1$
				this.connect(this.uploader, 'onComplete', this.onComplete); //$NON-NLS-1$
				this.connect(this.uploader, 'onProgress', this.onProgress); //$NON-NLS-1$
				this.connect(this.uploader, 'onChange', this.onChange); //$NON-NLS-1$
				this.connect(this.uploader, 'onBegin', this.onBegin); //$NON-NLS-1$


				dojo.place(this.uploader.inputNode, this.pickerHolder);
				dojo.attr(this.uploader.inputNode, 'id', this.id + '_picker'); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.uploader.inputNode, {
					position:"", //$NON-NLS-1$
					fontSize:"", //$NON-NLS-1$
					top:"", //$NON-NLS-1$
					right:"", //$NON-NLS-1$
					opacity: 1
				});
			},

			onCancel: function() {
				this._dialog.close();
			},

			onBegin: function() {
				dojo.addClass(this.formHolder, 'relm-ImportUploadDialog-hide'); //$NON-NLS-1$
				dojo.removeClass(this.progressArea, 'relm-ImportUploadDialog-hide'); //$NON-NLS-1$
				if (this.usingIframe) {
					this.uploadProgressBar.set('value', Infinity); //$NON-NLS-1$
				} else {
					this.uploadProgressBar.set('value', 0); //$NON-NLS-1$
				}

				// this.progressMessageArea.innerHTML = dojo.replace(this.messages.uploadingX, [this.currentFileName]);
			},

			onChange: function(files) {
				if (files && files[0]) {
					this.currentFileName = files[0].name;
					this.importButton.disabled = false;
				} else {
					this.importButton.disabled = true;
				}
			},

			onProgress: function(args) {
				if (!this.usingIframe) {
					var percent = args.bytesLoaded/args.bytesTotal*100;
					this.uploadProgressBar.set('value', percent); //$NON-NLS-1$
				}
			},

			onError: function(args) {
				this.messageBox.showErrorObject({
					type: "ERROR", //$NON-NLS-1$
					summary: this.messages.uploadError,
					explanation: this.messages.uploadError$explanation,
					userAction: this.messages.uploadError$useraction
				});  
			},

			/**
			 * Placeholder function to connect to or override. Used by import page
			 */
			onClose: function() {
				//No-op
			},

			onComplete: function(args) {
				if (args && dojo.isObject(args) && args.type && args.summary) {
					this.messageBox.showErrorObject(args);
					return;
				}
				if (this.showImportProgress) {
					this.trackImportProgress(args.importId);
				}
				else {
					this.onClose();
					this.complete(200);
				}
			},

			trackImportProgress: function(importId) {
				// this.progressMessageArea.innerHTML = dojo.replace(this.messages.importingXProducts, [this.currentFileName]);
				this._checkImportProgress(importId);
			},

			_checkImportProgress: function(importId) {
				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/import', //$NON-NLS-1$,
					handleAs: "json", //$NON-NLS-1$
					content: {importId: importId, progress: true},

					load: function(progress) {
						var checkAgain = false;

						if (progress.status != "Complete" && progress.status != "Error") { //$NON-NLS-1$ //$NON-NLS-2$
							checkAgain = true;
						}
						self.importProgressBar.set('value', progress.percentComplete); //$NON-NLS-1$
						
						if (checkAgain) {
							window.setTimeout(dojo.hitch(self, self._checkImportProgress, importId), 500);
						}
						else {
							dojo.publish("ple.recent_imports_did_change", [progress.importedBaselinePathSegment]); //$NON-NLS-1$
							self.onClose();
							self.complete(500);
						}
					},

					error: function(error, ioArgs) {
						self.messageBox.showXhrError(ioArgs.xhr, false);
					}
				});
			},

			_setValueAttr: function(value) {
				this.uploadProgressBar.set('value', value); //$NON-NLS-1$
			},

			/**
			 * Notifies the dialog that the operation in progress is complete. Waits for a 
			 * specified timeout before closing the dialog. 
			 * 
			 * @param  {integer} wait How long to wait before the dialog closes. If nothing
			 *                        is provided, then the default is used.
			 */
			complete: function(wait) {
				window.setTimeout(dojo.hitch(this, function() {
					this._dialog.close();
				}), wait || this.closeTimeout);
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);
				if (this.usingIframe) {
					this.onBegin();
				}
				this.uploader.submit(this.importForm);	
			}
		});

})();
