/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.admin.ImportedProductsList"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.admin", "ImportedProductsListMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.common.web.ui.FloatingMessageBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.EasyMessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.Breadcrumbs"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.StyledHeader"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.table.HoverRow"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.LoadingMessage"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.formatting"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.productapi"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.Goto"); //$NON-NLS-1$

//Overrides dojo.i18n.getLocalization to allow for renaming of term product
dojo.require('com.ibm.relm.common.web.ui.ProductNameCache'); //$NON-NLS-1$

(function(){
	var formatting = com.ibm.relm.ple.web.ui.formatting;

	dojo.declare(
		"com.ibm.relm.ple.web.admin.ImportedProductsList", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/ImportedProductsList.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.admin", "ImportedProductsListMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			importId: '', //$NON-NLS-1$

			constructor: function(args, node) {
				this._rowWidgets = [];
				this._productLinkHandles = [];
			},

			postCreate: function() {
				this.breadcrumbs.segments=[{
					label: this.messages.importProducts,
					href: "#action=relm.admin.import_products" //$NON-NLS-1$
				}];
				this.breadcrumbs.refresh();
			},

			getTopLevelProducts: function() {
				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/import?importId=' + self.importId, //$NON-NLS-1$,
					handleAs: "json", //$NON-NLS-1$
					load: function(topLevelProducts) {
						self.topLevelProducts = topLevelProducts;
						self.drawProducts(topLevelProducts);
						self._loadingMessage.hide();
					},
					error: function(error, ioArgs) {
						self._errorBox.showXhrError(ioArgs.xhr, true);
					}
				});
			},

			drawProducts: function(topLevelProducts) {
				dojo.empty(this._resultsTbody);

				var frag = document.createDocumentFragment();

				for (var i = 0; i < topLevelProducts.length; i++) {
					var product = topLevelProducts[i];
					var row = this.newHoverRowForProduct(product);
					dojo.place(row.domNode, frag);
				}

				this._resultsTbody.appendChild(frag);
			},

			newHoverRowForProduct: function(product) {
				var row = new com.ibm.relm.common.web.ui.table.HoverRow();
				this._rowWidgets.push(row);

				var td = dojo.create('td'); //$NON-NLS-1$
				row.addCell(td);

				var classAndAlt = formatting.getIconClassAndAltTextForItem(product);
				var alt = com.ibm.relm.common.web.ui.ProductNameCache.substituteString(classAndAlt[1]);
				var iconNode = dojo.create('img', { //$NON-NLS-1$
					src: this._blankGif,
					className: classAndAlt[0],
					title: alt,
					alt: alt
				}, td );

				var title = product.name;

				var link = dojo.create('a', { //$NON-NLS-1$
					href: product.uri,
					innerHTML: title,
					target: '_blank' //$NON-NLS-1$
				}, td);

				var jazzLink = new jazz.ui.ResourceLink({
					lazyFetch: true,
					retainLinkText: false,
					iconNode: iconNode
				}, link);

				this._rowWidgets.push(jazzLink);

				return row;
			},

			refresh: function(importId) {
				if (!importId) {
					throw new Error("No importId sent to ImportedProductsList.refresh"); //$NON-NLS-1$
				}
				this.importId = importId;
				this._errorBox.hide();
				this.getTopLevelProducts();
			},

			_destroyWidgets: function() {
				dojo.forEach(this._rowWidgets, function(w) { w.destroyRecursive(); });
				dojo.forEach(this._productLinkHandles, function(w) { w.destroyRecursive(); });
			},

			uninitialize: function() {
				this._destroyWidgets();
			}
		});

})();