/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.admin.RqmIntegrationSettings'); //$NON-NLS-1$

dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.FloatingMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("relm.app.Permissions"); //$NON-NLS-1$

(function(){
	var util = com.ibm.relm.common.web.ui.util;
	var html = jazz.util.html;
	var isJazzAdmin = relm.app.Permissions.isJazzAdmin();

	dojo.declare(
		'com.ibm.relm.ple.web.admin.RqmIntegrationSettings', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/RqmIntegrationSettings.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function() {
				this._stateGroupId = dijit.getUniqueId('RqmIntegrationSettings'); //$NON-NLS-1$
				this._enabledId = dijit.getUniqueId('RqmIntegrationSettings'); //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);
				this.connect(this._stateGroupField, "onKeyUp",  "onChange"); //$NON-NLS-1$ //$NON-NLS-2$

				if (!isJazzAdmin) {
					this._enabledCheckbox.disabled = true;
					this._stateGroupField.disabled = true;
				}
			},

			onChange: function() {
				this._saveButton.disabled = false;
			},

			onSave: function(e) {
				this._stateGroupField.set("disabled", true); //$NON-NLS-1$
				this._saveButton.disabled = true;
				this._errorBox.hideMessage();


				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/rqm_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						enabled: this._enabledCheckbox.checked,
						baselineStateGroup: dojo.trim(this._stateGroupField.value)
					},
				
					load: function () {
						self._saveButton.disabled = true;
						self._stateGroupField.set("disabled", false); //$NON-NLS-1$
						self._successBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.OK,
							summary: self.messages.saveSucceeded
						});
						self._successBox.hideLater();
					},
				
					error: function(error, ioArgs) {
						self._saveButton.disabled = false;
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			_onClickEnabled: function(e) {
				this._saveButton.disabled = false;
				this._stateGroupField.set("disabled", !this._enabledCheckbox.checked); //$NON-NLS-1$
			},

			refresh: function() {
				this._errorBox.hideMessage();
				this._saveButton.disabled = true;
				this._stateGroupField.set("disabled", true); //$NON-NLS-1$

				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/rqm_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
		
					handle: function() {
						if (isJazzAdmin) {
							self._saveButton.disabled = true;
							self._enabledCheckbox.disabled = false;
						}
					},

					load: function(settings) {
						self._enabledCheckbox.checked = settings.enabled;
						self._stateGroupField.set("disabled", !self._enabledCheckbox.checked); //$NON-NLS-1$
						self._stateGroupField.set("value", dojo.trim(settings.baselineStateGroup)); //$NON-NLS-1$
					},

					error: function(error, ioArgs) {
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			}
		}
	);

})();