/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.admin.RtcIntegrationSettings'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.FloatingMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.admin.RtcServerRow'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("relm.app.Permissions"); //$NON-NLS-1$

(function(){
	var util = com.ibm.relm.common.web.ui.util;
	var html = jazz.util.html;
	
	var isJazzAdmin = relm.app.Permissions.isJazzAdmin();
	
	dojo.declare(
		'com.ibm.relm.ple.web.admin.RtcIntegrationSettings', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/RtcIntegrationSettings.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function() {
				this._serverRows = [];
			},

			postCreate: function() {
				this.inherited(arguments);

				var addLink = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					innerHTML: html.escape(this.messages.addUrl)
				});
				if (isJazzAdmin)
				{
					this._section.setControls(addLink);
				}

				// start with a dummy one shown until the initial load (refresh)
				this.addUrl(false);

				this.connect(addLink, 'onclick', this.addUrlEvent); //$NON-NLS-1$
			},

			addUrl: function(focus) {
				var row = new com.ibm.relm.ple.web.admin.RtcServerRow();
				this._serverRows.push(row);
				row.placeAt(this._serversTable);
				if (isJazzAdmin)
				{
					row.enable(focus);
				}
				else
				{
					row.disable();
				}
				return row;
			},

			addUrlEvent: function(e) {
				dojo.stopEvent(e);
				this.addUrl(true);
			},
			
			enableSave: function()
			{
				if (isJazzAdmin)
				{
					this._saveButton.disabled = false;
				}
				else
				{
					this._saveButton.disabled = true;
				}
			},
			
			onSave: function(e) {
				this._saveButton.disabled = true;
				this._errorBox.hideMessage();

				var rtcUrls = dojo.map(this._serverRows, function(row) {
					return row.get('url'); //$NON-NLS-1$
				});

				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/rtc_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						rtcUrls: dojo.toJson(rtcUrls)
					},
				
					load: function (normalUrls) {
						self.enableSave();
						self._successBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.OK,
							summary: self.messages.saveSucceeded
						});
						self._successBox.hideLater();

						self._drawUrls(normalUrls);
						self.testConnections();
					},
				
					error: function(error, ioArgs) {
						self._saveButton.disabled = false;
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			testConnections: function() {
				for (var i = 0; i < this._serverRows.length; i++) {
					var row = this._serverRows[i];
					row.testConnection();
				}
			},

			_drawUrls: function(rtcUrls) {
				dojo.forEach(this._serverRows, function(oldRow) {
					oldRow.destroyRecursive();
				});
				this._serverRows = [];

				dojo.forEach(rtcUrls, function(rtcUrl) {
					var row = this.addUrl();
					row.set('url', rtcUrl); //$NON-NLS-1$
				}, this);

				if (rtcUrls.length === 0) {
					this.addUrl();
				}
			},

			refresh: function() {
				this._errorBox.hideMessage();
				this._saveButton.disabled = true;

				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/rtc_settings', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
		
					handle: function() {
						self.enableSave();
					},

					load: function(settings) {
						self._drawUrls(settings.rtcUrls);
						self.testConnections();
					},

					error: function(error, ioArgs) {
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			}
		}
	);

})();