/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.admin.RtcServerRow'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){
	var util = com.ibm.relm.common.web.ui.util;

	dojo.declare(
		'com.ibm.relm.ple.web.admin.RtcServerRow', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','admin/templates/RtcServerRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.admin', 'RtcIntegrationSettingsMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			_setUrlAttr: function(url) {
				this._urlField.value = url;
			},

			_getUrlAttr: function() {
				return dojo.trim(this._urlField.value);
			},

			enable: function(focus) {
				this._urlField.disabled = false;
				if (focus) this._urlField.select();
			},

			disable: function() {
				this._urlField.disabled = true;
			},

			testConnection: function() {
				var rtcUrl = this.get('url'); //$NON-NLS-1$

				if (!rtcUrl.length) {
					this.hideStaus();
					return;
				}

				// We want to hit a real service that needs OAuth to really test this, so just
				// do a dummy search that won't fine anything (or ignore if it does).
				var servicePart = '/service/com.ibm.team.rtc.relm.IRelmService?method=findStreams&name=PingRtcServerToTestConnection'; //$NON-NLS-1$

				var self = this;
				jazz.client.xhrGet({
					url: rtcUrl + servicePart,
					handleAs: 'text', //$NON-NLS-1$
					failOk: true,
				
					load: function () {
						self.showSuccess(true);
					},
				
					error: function(error, ioArgs) {
						// Strip the extra args from the URL in the error
						// message. It's an implementation detail that user
						// shouldn't see.
						var text = ioArgs.xhr.statusText.replace(servicePart, ''); //$NON-NLS-1$
						self.showWarning(self.messages.connectionFailed, text);
					}
				});
			},

			hideStaus: function() {
				this._statusNode.style.display = 'none'; //$NON-NLS-1$
			},

			showSuccess: function() {
				dojo.removeAttr(this._statusNode, 'title'); //$NON-NLS-1$
				this._statusIcon.className = 'ple-icon-okSmall'; //$NON-NLS-1$
				dojo.removeClass(this._statusMessage, 'jazz-app-ua-TextHoverHelp'); //$NON-NLS-1$
				util.setText(this._statusMessage, this.messages.enabled);
				this._statusNode.style.display = ''; //$NON-NLS-1$
			},

			showWarning: function(message, tooltip) {
				dojo.attr(this._statusNode, 'title', tooltip); //$NON-NLS-1$
				this._statusIcon.className = 'ple-icon-warningSmall'; //$NON-NLS-1$
				dojo.addClass(this._statusMessage, 'jazz-app-ua-TextHoverHelp'); //$NON-NLS-1$
				util.setText(this._statusMessage, message);
				this._statusNode.style.display = ''; //$NON-NLS-1$
			}
		}
	);

})();