/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.admin.SettingsPage'); //$NON-NLS-1$

dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.FloatingMessageBox'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.ConfirmDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('jazz.ui.SideNavigationLayout'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.admin.RtcIntegrationSettings'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.admin.RqmIntegrationSettings'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.ImportPage"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.ImportedProductsList"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.AdvancedSettings"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.admin.DoorsIntegrationSettings"); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.admin', 'SettingsPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$

(function(){

	var html = jazz.util.html;

	var KEYS = ['Product', 'product', 'Products', 'products', 'Aproduct', 'aProduct', 'aproduct', 'Theproduct', 'theProduct', 'theproduct', 'Theproducts','theProducts', 'theproducts']; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$

	var isJazzAdmin = relm.app.Permissions.isJazzAdmin();

	dojo.declare(
		'com.ibm.relm.ple.web.admin.SettingsPage', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web', 'admin/templates/SettingsPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.admin', 'SettingsPageMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			termInputs: null,
			constructor: function(args, node)
			{
				this._inited = false;
				this.termInputs = [];
			},

			postCreate: function() {
				this.inherited(arguments);

				var PlatformUI = net.jazz.ajax.ui.PlatformUI;
				var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
				ActionRegistry.registerAction('relm.admin.rename_products', this, '_showRenameProductsActions'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.import_products', this, '_showImportProductsActions'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.view_imported', this, '_showImportedProducts'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.stream_management', this, '_showStreamManagement'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.doors_baselines', this, '_showDoorsBaseline'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.rqm_baselines', this, '_showRqmBaseline'); //$NON-NLS-1$ //$NON-NLS-2$
				ActionRegistry.registerAction('relm.admin.advanced', this, '_showAdvancedSettings'); //$NON-NLS-1$ //$NON-NLS-2$

				var columns = [
					{
						label: this.messages.originalTerm
					},
					{
						label: this.messages.yourTerm,
						width: '9999px' //$NON-NLS-1$
					}
				];

				this._nameTable = new jazz.ui.StyledTable({
					heading: this.messages.customProductTerms,
					columns: columns,
					trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_FIXED
				}, this._nameTable);

				dojo.attr(this._nameTable.domNode, 'width', '100%'); //$NON-NLS-1$ //$NON-NLS-2$

				if (isJazzAdmin) {

					var restoreLink = dojo.create('a', { //$NON-NLS-1$
						innerHTML: html.escape(this.messages.restoreOriginalTerms),
						title: this.messages.restoreOriginalTermsTitle,
						href: '#', //$NON-NLS-1$
						style: 'float: right' //$NON-NLS-1$
					}, this._nameTable._globalHeader);

					this.connect(restoreLink, 'onclick', this._onRestoreDefaultProductsNames); //$NON-NLS-1$
				}

				this._nav = new jazz.ui.SideNavigationLayout();

				// Pull content off the page until Nav section loads it.
				new dojo.NodeList(this._renameProductsContent).orphan();

				//Product settings
				var productNavSection = this._nav.addNavigationSection(dojo.create('div'), this.messages.products, true); //$NON-NLS-1$
				//Rename
				this._renameProductsRow = productNavSection.addSectionRow(this.messages.renameProductsTitle, '#action=relm.admin.rename_products'); //$NON-NLS-1$
				productNavSection.addArtifactCommand(this._renameProductsRow, this.messages.refresh, dojo.hitch(this, this._confirmRefreshProductNames), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$
				//Import
				this._importProductsRow = productNavSection.addSectionRow(this.messages.importProducts, '#action=relm.admin.import_products'); //$NON-NLS-1$
				productNavSection.addArtifactCommand(this._importProductsRow, this.messages.refresh, dojo.hitch(this, this._refreshImportsProgress), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$
				
				//Integrations
				var integrationsNavSection = this._nav.addNavigationSection(dojo.create('div'), this.messages.integrations, true); //$NON-NLS-1$
				//RTC
				this._streamMangementRow = integrationsNavSection.addSectionRow(this.messages.teamConcertIntegration, '#action=relm.admin.stream_management'); //$NON-NLS-1$
				integrationsNavSection.addArtifactCommand(this._streamMangementRow, this.messages.refresh, dojo.hitch(this, this._refreshStreamMangement), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$


				//Is advanced settings?
				var query = window.location.search.substring(1, window.location.length);
				query = dojo.queryToObject(query);
				var advanced = query.advanced && query.advanced.toLowerCase() === 'true'; //$NON-NLS-1$ 
				if (advanced) {
					//DOORS
					this._doorsIntegrationRow = integrationsNavSection.addSectionRow(this.messages.doorsIntegration, '#action=relm.admin.doors_baselines'); //$NON-NLS-1$
					integrationsNavSection.addArtifactCommand(this._doorsIntegrationRow, this.messages.refresh, dojo.hitch(this, this._refreshDoorsBaseline), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$//DOORS
					//RQM
					this._rqmIntegrationRow = integrationsNavSection.addSectionRow(this.messages.rqmIntegration, '#action=relm.admin.rqm_baselines'); //$NON-NLS-1$
					integrationsNavSection.addArtifactCommand(this._rqmIntegrationRow, this.messages.refresh, dojo.hitch(this, this._refreshRqmBaseline), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$

					//Advanced Settings
					this._advancedSettingsRow = integrationsNavSection.addSectionRow(this.messages.advancedSection, '#action=relm.admin.advanced'); //$NON-NLS-1$
					integrationsNavSection.addArtifactCommand(this._advancedSettingsRow, this.messages.refresh, dojo.hitch(this, this._refreshAdvanced), 'ple-icon-nosprite-refresh'); //$NON-NLS-1$					
				}

				this._nav.placeAt(this.domNode);
				this._nav.startup();
			},

			_onRestoreDefaultProductsNames: function(e) {
				dojo.stopEvent(e);
				dojo.forEach(this.termInputs, function(input) {
					input.value = input.oldValue = ''; //$NON-NLS-1$
				});
				this._toggleSave(true);
				this._saveButton.focus();
			},

			_confirmRefreshProductNames: function(e) {
				e && dojo.stopEvent(e);

				// TODO: does not check for dirty an navigate away
				if (!this.isDirty()) {
					this._refreshProductNames();
					return;
				}

				var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
					dialogTitle: this.messages.confirmRefreshSettingsTitle,
					prompt: this.messages.confirmRefreshSettingsPrompt,
					details: this.messages.confirmRefreshSettingsDetails
				});

				var self = this;
				d.addButton(this.messages.refresh, function() {
					self._refreshProductNames();
					var deferred = new dojo.Deferred();
					deferred.callback();
					return deferred;
				}, true);

				d.startup();
			},

			isDirty: function() {
				return !this._saveButton.disabled;
			},

			getRtcIntegrationWidget: function() {
				if (!this._rtcIntegrationSettings) {
					this._rtcIntegrationSettings = new com.ibm.relm.ple.web.admin.RtcIntegrationSettings();
				}
				return this._rtcIntegrationSettings;
			},

			getRqmIntegrationWidget: function() {
				if (!this._rqmIntegrationSettings) {
					this._rqmIntegrationSettings = new com.ibm.relm.ple.web.admin.RqmIntegrationSettings();
				}
				return this._rqmIntegrationSettings;
			},

			getAdvancedSettignsWidget: function() {
				if (!this._advancedSettings) {
					this._advancedSettings = new com.ibm.relm.ple.web.admin.AdvancedSettings();
				}
				return this._advancedSettings;
			},

			getDoorsIntegrationSettingsWidget: function() {
				if (!this._doorsIntegrationSettings) {
					this._doorsIntegrationSettings = new com.ibm.relm.ple.web.admin.DoorsIntegrationSettings();
				}
				return this._doorsIntegrationSettings;
			},

			_refreshDoorsBaseline: function(e) {
				e && dojo.stopEvent(e);
				this.getDoorsIntegrationSettingsWidget().refresh();
			},

			_refreshRqmBaseline: function(e) {
				e && dojo.stopEvent(e);
				this.getRqmIntegrationWidget().refresh();
			},

			_refreshAdvanced: function(e) {
				e && dojo.stopEvent(e);
				this.getAdvancedSettignsWidget().refresh();
			},

			_refreshImportsProgress: function(e) {
				e && dojo.stopEvent(e);
				window.location.hash="action=relm.admin.import_products"; //$NON-NLS-1$
				this._showImportProductsActions();
			},

			_refreshStreamMangement: function(e) {
				e && dojo.stopEvent(e);
				this.getRtcIntegrationWidget().refresh();
			},

			_refreshProductNames: function() {
				this._errorBox.hideMessage();
				this._successBox.hideMessage();
				this._toggleSave(false);

				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/configure/productName', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						includeDefaultNames: false
					},
				
					load: function(nameMap) {
						var isDefault = nameMap.isDefault === 'true'; //$NON-NLS-1$
						var fields = dojo.query('input[type="text"]', self._nameTable.domNode); //$NON-NLS-1$

						for (var i = 0; i < KEYS.length; i++) {
							var key = KEYS[i];
							var value = nameMap[key];
							fields[i].value = isDefault ? '' : value; //$NON-NLS-1$
						}
					},
				
					error: function(error, ioArgs) {
						self._successBox.hideMessage();
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			_initNameTable: function(nameMap) {
				// Yes! That's string true
				var defaultNames = nameMap.defaultNames;
				var frag = document.createDocumentFragment();
				this.termInputs = [];

				var disabled = !isJazzAdmin;

				for (var i = 0; i < KEYS.length; i++) {
					var key = KEYS[i];
					var yourName = nameMap[key];
					var defaultName = defaultNames[key];
					var htmlDefaultName = html.escape(defaultName);

					var tr = dojo.create('tr', {}, frag); //$NON-NLS-1$
					
					var originalCell = dojo.create('td', { //$NON-NLS-1$
						className: 'relm-SettingsPage-originalCell' //$NON-NLS-1$
					}, tr);

					var inputId = dijit.getUniqueId('relm-SettingsPage-term'); //$NON-NLS-1$

					dojo.create('label', { //$NON-NLS-1$
						innerHTML: htmlDefaultName,
						htmlFor: inputId
					}, originalCell);

					var yourCell = dojo.create('td', {}, tr); //$NON-NLS-1$
					var yourInput = dojo.create('input', { //$NON-NLS-1$
						type: 'text', //$NON-NLS-1$
						name: key,
						placeholder: defaultName,
						id: inputId,
						disabled: disabled
					}, yourCell);
					this.termInputs.push(yourInput);

					// On blur of the first four terms [Product, product, Products, products] replace all occurances of the 
					// term in a blind way.
					if (i<4) {
						this.connect(yourInput, 'onblur', dojo.hitch(this, this._smartTermAutoFill, yourInput)); //$NON-NLS-1$
					}

					var value = !yourName ? '' : yourName; //$NON-NLS-1$
					yourInput.value = value;
					yourInput.originalValue = value;

					this.connect(yourInput, 'onkeyup', this._checkForDirtyNames); //$NON-NLS-1$
				}

				dojo.empty(this._nameTable.tbody);
				dojo.place(frag, this._nameTable.tbody);
			},

			/**
			 * This method monitors the blur of the first four product terms [Product, product, Products, products].
			 * It attempts to do a string replace and auto fill the new term, if nothing has been filled in yet. 
			 * 
			 * @param  {DOM Element} input - The input field that was blurred
			 */
			_smartTermAutoFill: function(input) {
				if (!input.value.length || input.oldValue == input.value)
					return;
				
				var placeholder = input.placeholder;
				var tester = new RegExp('\\b' + input.placeholder + '\\b'); //$NON-NLS-1$ //$NON-NLS-2$
				input.oldValue = input.value;
				dojo.forEach(this.termInputs, function(currentInput) {
					//Don't do any replacement if the field already has a value
					if (currentInput.value.length) return;

					var crntPlaceholder = currentInput.placeholder;
					if (tester.test(crntPlaceholder)) {
						var newString = crntPlaceholder.replace(tester, input.value);
						currentInput.value = newString;
						this._checkForDirtyNames(); //changing the input value doesn't trigger 'onkeyup', so we have to call this manually
					}
				}, this);
			},

			_toggleSave: function(canSave) {
				this._saveButton.disabled = !canSave;// || !isJazzAdmin;
			},

			_checkForDirtyNames: function() {
				var inputList = this.termInputs;

				var allHaveNewValues = dojo.every(inputList, function(input) {
					return input.value.length !== 0;
				});

				if (allHaveNewValues) {
					this._toggleSave(true);
					return;
				}

				var allCleared = dojo.every(inputList, function(input) {
					return input.value.length === 0 && input.value !== input.originalValue;
				});

				if (allCleared) {
					this._toggleSave(true);
					return;
				}

				this._toggleSave(false);
			},

			_loadAndInitProductNames: function() {
				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/configure/productName', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						includeDefaultNames: true,
						fromSettingsPage: true
					},
				
					load: function(nameMap) {
						self._initNameTable(nameMap);
					},
				
					error: function(error, ioArgs) {
						self._errorBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			onSave: function(e) {
				var nameMap = {};
				var hasCustomNames = false;

				dojo.forEach(this.termInputs, function(input) {
					var value = dojo.trim(input.value);
					if (value.length) {
						hasCustomNames = true;
					}

					nameMap[input.name] = value;
				});

				var restoreDefaults = !hasCustomNames;
				this._saveProductName(nameMap, restoreDefaults);
			},

			_saveProductName: function(nameMap, restoreDefaults) {
				this._errorBox.hideMessage();
				var self = this;
				self._toggleSave(false);

				if (restoreDefaults) {
					// ignore any names if restoring. This is a little weird since we're overloading how
					//  this data is used in a pointless attempt to be RESTful.
					nameMap = {
						// Yes. This stupid value is a String
						isDefault: 'true' //$NON-NLS-1$
					};
				}

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/configure/productName', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					headers: {
						'Content-Type': 'application/json' //$NON-NLS-1$ //$NON-NLS-2$
					},
					postData: dojo.toJson(nameMap),
				
					load: function () {
						self._toggleSave(false);
						self._successBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.OK,
							summary: self.messages.saveSucceeded
						});
						self._successBox.hideLater();
					},
				
					error: function(error, ioArgs) {
						self._toggleSave(true);
						self._errorBox.showXhrError(ioArgs.xhr);
						self._successBox.hideMessage();
					}
				});
			},


			/**
			 * SideNavigationLayout will use dojo.empty to clean the main content.
			 * This can ruins the nodes if they are from a widget in IE. Track
			 * the main content here to remove it safely before setting it.
			 * @param {[type]} mainContentNode [description]
			 * @param {[type]} navRow The row in the navigation pain to select and all other rows will be deselected
			 */
			_setMainContent: function(mainContentNode, navRow) {
				// Don't let SideNavigationLayout remove old content, since it uses dojo.remove
				// which breaks widget content in IE.
				if (this._mainContentNode) {
					new dojo.NodeList(this._mainContentNode).orphan();
				}
				if (this._selectedNavRow) {
					dojo.removeClass(this._selectedNavRow, 'selected'); //$NON-NLS-1$
				}

				this._mainContentNode = mainContentNode;
				this._nav.setMainContent(mainContentNode);				

				this._selectedNavRow = navRow;
				if (this._selectedNavRow) {
					dojo.addClass(this._selectedNavRow, 'selected'); //$NON-NLS-1$
				}
			},

			_showImportProductsActions: function() {
				var importPage = this._getImportPageWidget();
				this._setMainContent(importPage.domNode, this._importProductsRow);
				importPage.refresh();
			},

			_getImportPageWidget: function(args) {
				if (!this._importPage) {
					this._importPage = new com.ibm.relm.ple.web.admin.ImportPage();
					this._importPage.startup();
				} 
				return this._importPage;
			},

			_showImportedProducts: function(actionArgs) {
				var importedList = this._getImportedProductsList();
				this._setMainContent(importedList.domNode, this._importProductsRow);
				importedList.refresh(actionArgs && actionArgs.importId);
			},

			_getImportedProductsList: function(args) {
				if (!this._importedList) {
					this._importedList = new com.ibm.relm.ple.web.admin.ImportedProductsList();
					this._importedList.startup();
				} 
				return this._importedList;
			},

			_showRenameProductsActions: function() {
				this._setMainContent(this._renameProductsContent, this._renameProductsRow);

				if (!this._inited) {
					this._loadAndInitProductNames();
					this._inited = true;
				}
				else {
					if (!this.isDirty()) this._refreshProductNames();
				}
			},

			_showStreamManagement: function() {
				this._setMainContent(this.getRtcIntegrationWidget().domNode, this._streamMangementRow);
				this.getRtcIntegrationWidget().refresh();
			},

			_showDoorsBaseline: function() {
				this._setMainContent(this.getDoorsIntegrationSettingsWidget().domNode, this._doorsIntegrationRow);
				this.getDoorsIntegrationSettingsWidget().refresh();
			},

			_showRqmBaseline: function() {
				this._setMainContent(this.getRqmIntegrationWidget().domNode, this._rqmIntegrationRow);
				this.getRqmIntegrationWidget().refresh();
			},

			_showAdvancedSettings: function() {
				this._setMainContent(this.getAdvancedSettignsWidget().domNode, this._advancedSettingsRow);
				this.getAdvancedSettignsWidget().refresh();
			},

			uninitialize: function() {
				this._nameTable.destroyRecursive();
				this._rtcIntegrationSettings && this._rtcIntegrationSettings.destroyRecursive();
				this._rqmIntegrationSettings && this._rqmIntegrationSettings.destroyRecursive();
			}
		}
	);

})();
