/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.configgroupings.EditConfigGrouping"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.configgroupings", "EditConfigGroupingMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.AutoHeightBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.RequiredMarker"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.editor.PageHelper");  //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.configgroupings.ConfigGroupingsEditorInput"); //$NON-NLS-1$
dojo.require("dijit.form.Textarea"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.formatting"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.ProductNameCache'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin"); //$NON-NLS-1$

(function(){
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var JazzUIWorkbench = PlatformUI.getWorkbench();
	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var utils = com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil;

	var templateQuery = "\n" + //$NON-NLS-1$
		"# Sample configuration grouping to return all top-level configurations (i.e., configurations without a parent configuration) and a commented-out\n" + //$NON-NLS-1$
		"# clause to show you how to query on a property\n" + //$NON-NLS-1$
		"PREFIX dcterms: <http://purl.org/dc/terms/>\n" + //$NON-NLS-1$
		"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" + //$NON-NLS-1$
		"PREFIX pd: <http://jazz.net/ns/pd#>\n" + //$NON-NLS-1$
		"PREFIX pd_ext: <http://jazz.net/ns/pd/extensions#>\n" + //$NON-NLS-1$
		"\n" + //$NON-NLS-1$
		"SELECT ?configurationVersion\n" + //$NON-NLS-1$
		"WHERE {\n" + //$NON-NLS-1$
		"   graph ?configurationVersion\n" + //$NON-NLS-1$
		"   {\n" + //$NON-NLS-1$
		"      ?configuration a pd:Configuration.\n" + //$NON-NLS-1$
		"      #?configuration <{serverUrl}/property/_ConfigurationProperty_> \"PropertyValue\".\n" + //$NON-NLS-1$
		"   }\n" + //$NON-NLS-1$
		"   filter not exists { ?somePart pd:binding ?configurationVersion }\n" + //$NON-NLS-1$
		"}"; //$NON-NLS-1$

	var orionEditor = null, globalSetText = null;

	var PAGE_ID = "com.ibm.relm.ple.web.configgroupings.EditConfigGrouping"; //$NON-NLS-1$
	var PAGE_REMOVAL_ID = "com.ibm.relm.ple.web.configgroupings.EditConfigGrouping.removePage"; //$NON-NLS-1$

	dojo.declare(
		"com.ibm.relm.ple.web.configgroupings.EditConfigGrouping", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','configgroupings/templates/EditConfigGrouping.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.configgroupings", "EditConfigGroupingMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			pageHelper: null,
			_editorInputs: {},

			constructor: function() {
				this._nameId = dijit.getUniqueId('EditConfigGrouping_name'); //$NON-NLS-1$
				this._descriptionId = dijit.getUniqueId('EditConfigGrouping_description'); //$NON-NLS-1$
				this._sparqlId = dijit.getUniqueId('EditConfigGrouping_sparql'); //$NON-NLS-1$
				this._statusId = dijit.getUniqueId('EditConfigGrouping_sparqlStatus'); //$NON-NLS-1$
				this._previewId = dijit.getUniqueId('EditConfigGrouping_preview'); //$NON-NLS-1$

				this.resourceLinks = [];

				orionEditor = null;
				new com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin({initializer: dojo.hitch(this, this.initOrionEditor)});
			},

			postCreate: function() {
				/* Declare get and set methods to support Orion Editor */
				this.sparqlText.get = function(property) {
					if(orionEditor === null) //$NON-NLS-1$
						return ""; //$NON-NLS-1$
					else
						return orionEditor.getText();
				};
				
				this.sparqlText.set = function(property,value) {
					if(orionEditor === null) //$NON-NLS-1$
						globalSetText = value;
					else {
						setTimeout(function() { // Defer setting of text to allow DOM to load completely
							orionEditor.setText(value);
						}, 100);
					}
				};
				
				var ActionRegistry = JazzUIWorkbench.getActionRegistry();
				ActionRegistry.registerAction("ple.edit_config_grouping", this, 'openGrouping'); //$NON-NLS-1$ //$NON-NLS-2$

				this.connect(this.nameNode, "onkeyup",  this.onContentChange); //$NON-NLS-1$
				this.connect(this.descriptionText, "onkeyup", this.onContentChange); //$NON-NLS-1$
				//this.connect(this.sparqlText, "onkeyup", this.onContentChange); //$NON-NLS-1$

				this._breadcrumbs.segments.push({
					label: this.messages.configGroupings,
					href: "#action=ple.manage_config_groupings" //$NON-NLS-1$
				});
				this._breadcrumbs.refresh();

				if (!this.navigateAwayHandle && dojo.isFF !== undefined) {
					this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
				}
				if (!this.onBeforeUnloadHandle) {
					this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);
				}
			
				this._autoScrollHeight = new com.ibm.relm.common.web.ui.AutoHeightBox(this._contentNode, {
					listenOnPage: 'com.ibm.relm.ple.web.configgroupings.EditConfigGrouping', //$NON-NLS-1$
					fillParentWidth: true,
					priority: 10
				});

				//TODO: call resize on open and close of message box
				this._addShowPreview();
				this._initPrefixSelector();

				this.pageHelper= com.ibm.relm.common.web.ui.editor.PageHelper.getInstance(this.getPageId());
				this.pageHelper.init(this);
			},
			
			initOrionEditor: function() {
				var self = this;
				
				// Instantiate an instance of the Orion Editor and initialize it
				require(["orion/editor/edit", "orion/editor/textMateStyler", "orion/editor/sparqlContentAssist", "orion/editor/sparqlGrammar"], function(edit, mTextMateStyler, mSPARQLContentAssist, mSparqlGrammar) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					
					// Remove active selection to avoid bug in IE
					if (window.getSelection) {
						if (window.getSelection().empty) {
							window.getSelection().empty();
						}
						else if (window.getSelection().removeAllRanges) {
							window.getSelection().removeAllRanges();
						}
					}
					
					// Line and column number status bar
					var status = ""; //$NON-NLS-1$
					var statusReporter = function(message, isError) {
						if (isError) {
							status =  "ERROR: " + message; //$NON-NLS-1$
						} else {
							status = message;
						}
						self.sparqlTextStatus.innerHTML = status;
					};
					
					// Instantiate the editor
					orionEditor = edit({
						parent: self.sparqlText,
						statusReporter: statusReporter,
						autoPairParentheses: true,
						autoPairBraces: true,
						autoPairSquareBrackets: true,
						autoPairAngleBrackets: true,
						autoPairQuotations: true,
						smartIndentation: true
					});
					
					// Add SPARQL content assist to the editor
					var contentAssist = orionEditor.getContentAssist ? orionEditor.getContentAssist() : orionEditor._contentAssist;
					contentAssist.providers.push(new mSPARQLContentAssist.SPARQLContentAssistProvider());
					
					// Add Grammar based syntax highlighter for SPARQL to the editor
					var syntaxHighlighter = {
						styler: null,
						highlight: function(editor) {
							if (this.styler) {
								this.styler.destroy();
								this.styler = null;
							}
							var textView = editor.getTextView();
							this.styler = new mTextMateStyler.TextMateStyler(textView, mSparqlGrammar.SparqlGrammar());
						}
					};

					syntaxHighlighter.highlight(orionEditor);
					if (globalSetText !== null)
					{
						orionEditor.setText(globalSetText);
						globalSetText = null;
					}
					
					var textView = orionEditor.getTextView();
					textView.addEventListener("KeyUp", dojo.hitch(self,self.onContentChange)); //$NON-NLS-1$
					
					// To avoid incorrect setting of the editor height by Orion
					dojo.removeAttr(self.sparqlText, "style"); //$NON-NLS-1$
				});
			},

			_initPrefixSelector: function() {
				var addPrefix = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					innerHTML: this.messages.AddPrefix,
					title: this.messages.AddPrefixTooltip,
					className: 'relm-Section-link' //$NON-NLS-1$
				});
				this.connect(addPrefix, 'onclick', dojo.hitch(this, function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					this._handleAddPrefix();
				}));
				this._sparqlPane.setControls(addPrefix);
			},

			_handleAddPrefix: function() {
				var self = this;
				var prefixDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog({
					okCallback: dojo.hitch(this,this._okPrefixDialogCallback),
					cancelCallback: dojo.hitch(this,this._cancelPrefixDialogCallback)
				});

				self._prefixDialog = new jazz.ui.Dialog({
					contentNode: prefixDialog.domNode,
					primaryTitle: self.messages.AddPredifinedPrefix,
					width: "600px", //$NON-NLS-1$
					isClosable: true
				});
			},

			_okPrefixDialogCallback: function(args){	
				this._prefixDialog.close();
				this._prefixDialog.destroy();
				// refresh sparql
				var sparql = this.sparqlText.get("value"); //$NON-NLS-1$
				sparql = args.prefixText + sparql;
				this.sparqlText.set("value",sparql); //$NON-NLS-1$
				// Orion editor seems to take a short amount of time to "process" a change.
				// Even when it's visually there, the value can't be programtically read immediately.
				// Dumb. A short delay works around this.
				setTimeout(function() {
					this.onContentChange();	
				}.bind(this), 100);
			},

			_cancelPrefixDialogCallback: function(args){
				this._prefixDialog.close();
				this._prefixDialog.destroyRecursive();
			},

			onContentChange: function(evt) {
				this.messageBox.hideMessage();
				var groupingInfo = this._getGroupingInfo();
				var isDirty = this._isDirty(groupingInfo);
				this._enableSaveButtons(
					isDirty
					&& groupingInfo.name
					&& groupingInfo.sparql);
				JazzUIWorkbench.setTitle(groupingInfo.name || "");//$NON-NLS-1$
				this.dirtyStateChanged(isDirty);
			},
			
			isDirty: function() {
				return false;
			},
			
			_isDirty: function(groupingInfo) {
				groupingInfo = groupingInfo || this._getGroupingInfo();
				var original = this.grouping;

				var nameDirty = groupingInfo.name != original.name;
				var descriptionDirty = groupingInfo.description != original.description;
				var sparqlDirty = groupingInfo.sparql != original.sparql;

				return (nameDirty || descriptionDirty || sparqlDirty);
			},

			_initializeBlankGrouping: function() {
				this.grouping = {
					name: "", //$NON-NLS-1$
					uri: "", //$NON-NLS-1$
					description: "", //$NON-NLS-1$
					// parameters: [],
					sparql: "" //$NON-NLS-1$
				};
			},

			_enableSaveButtons: function(enable){
				//TODO: permission checking
				if (enable) {
					this.saveButton.disabled = ""; //$NON-NLS-1$
				} else {
					this.saveButton.disabled = "true"; //$NON-NLS-1$
				}
				if (this.mode != "create") { //$NON-NLS-1$
					this.saveCopyButton.disabled = ""; //$NON-NLS-1$
				} else {
					this.saveCopyButton.disabled = "true"; //$NON-NLS-1$
				}
			},

			_getGroupingInfo: function() {
		        var queryInfo = {
		        	uri: (this.grouping.uri) || "", //$NON-NLS-1$
		        	name: this.nameNode.value,
		        	description: this.descriptionText.value,
		        	sparql: this.sparqlText.get("value") //$NON-NLS-1$
			    };
		        return queryInfo;
			},

			_addSampleQuery: function() {
				// Disable spell check to avoid red squiggly lines in the editor in Firefox
				if(dojo.isFF !== undefined)
					dojo.body().setAttribute("spellCheck","false"); //$NON-NLS-1$ //$NON-NLS-2$
				
				var substitutedSample = templateQuery.replace(/\{serverUrl\}/,net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
				this.sparqlText.set("value",substitutedSample); //$NON-NLS-1$
				this.grouping.sparql = substitutedSample;
			},

			_addShowPreview: function() {
				var showPreview = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					title: this.messages.showPreviewTooltip,
					innerHTML: this.messages.showPreview,
					className: 'relm-Section-link' //$NON-NLS-1$
				});
				this.connect(showPreview, 'onclick', dojo.hitch(this, function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					this.showPreview();
				}));
				this._previewPane.setControls(showPreview);
			},
			
			_getEditorInput: function(args) {
				var pgUri = args.uri || ""; //$NON-NLS-1$
				var pgName = args.name || ""; //$NON-NLS-1$
				var pgDescription = args.description || ""; //$NON-NLS-1$
				var editorInput = this._editorInputs[pgUri];
				if(!editorInput) {
					editorInput = new com.ibm.relm.ple.web.configgroupings.ConfigGroupingsEditorInput({id: pgUri.split("/").pop(), //$NON-NLS-1$ 
																						uri: pgUri,
																						name: pgName,
																						description: pgDescription }); 
					editorInput.editor = this;
					this._editorInputs[pgUri] = editorInput;
				} else {
					editorInput.configGroupingUpdated(args);
				}
				return editorInput;
			},
			

			clearEditor: function() {
				this.nameNode.value = ""; //$NON-NLS-1$
				this.descriptionText.value = ""; //$NON-NLS-1$
				this.sparqlText.set("value",""); //$NON-NLS-1$ //$NON-NLS-2$
			},

			openGrouping: function(args) {
				this.saveButton.disabled = true;
				this.saveCopyButton.disabled = true;
				dojo.empty(this.previewContents);
				this.clearEditor();
				this._loadingMessage.hide();
				this.messageBox.hide();
				this.mode = args.mode;

				if (args.mode == "edit") { //$NON-NLS-1$
					this.editorInput = this._getEditorInput(args); // store current EditorInput
					this.editGrouping(args);
				} else if (args.mode == "create") { //$NON-NLS-1$
					this.createGrouping(args);					
				} else {
					throw new Error('Invalid mode passed to edit grouping.'); //$NON-NLS-1$
				}
			},

			editGrouping: function(args) {
				// Disable spell check to avoid red squiggly lines in the editor in Firefox
				if(dojo.isFF !== undefined) {
					dojo.body().setAttribute("spellCheck","false"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				this.saveButton.innerHTML = this.messages.save;
				this.saveButton.title = this.messages.saveTitle;

				jazz.client.xhrGet({
						url: net.jazz.ajax._contextRoot + "/ple/config_grouping", //$NON-NLS-1$
						handleAs: "json", //$NON-NLS-1$
						content: {
							method: "getGrouping", //$NON-NLS-1$
							queryUri: args.uri
						},
						load: dojo.hitch(this, function(grouping) {
							this.grouping = grouping;
							this.nameNode.value = grouping.name;
							this.sparqlText.set("value", grouping.sparql); //$NON-NLS-1$
							this.descriptionText.value = grouping.description;
							this.editorInput.configGroupingLoaded(grouping);
							this.getPageHelper().openEditor(this);
							this.onContentChange();
						}),
						error: dojo.hitch(this, function(error, args) {
							this._initializeBlankGrouping();
							this.messageBox.showXhrError(args.xhr, true);
						})
				});
			},

			createGrouping: function(args) {
				this.saveButton.innerHTML = this.messages.create;
				this.saveButton.title = this.messages.createTitle;
				this._initializeBlankGrouping();
				this._addSampleQuery();
			},

			onSave: function() {
				this._saveGrouping(this._getGroupingInfo());
			},

			onSaveCopy: function() {
				this.nameNode.value = dojo.string.substitute(this.messages.CopyOf, [this.nameNode.value]);
				var groupingInfo = this._getGroupingInfo();
				delete groupingInfo.uri;
				this._saveGrouping(groupingInfo);
			},

			_saveGrouping: function(grouping) {
				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + "/ple/config_grouping", //$NON-NLS-1$
					handleAs: "json", //$NON-NLS-1$
					postData: dojo.toJson(grouping),
					headers: { 'Content-Type': 'application/json' }, //$NON-NLS-1$ //$NON-NLS-2$
					load: dojo.hitch(this, function(grouping) {
						this.grouping = grouping;
						var message;
						if (this.mode == "create") { //$NON-NLS-1$
							this.saveButton.innerHTML = this.messages.save;
							this.saveButton.title = this.messages.saveTitle;
							this.mode = "modify"; //$NON-NLS-1$
							message = this.messages.createSuccess;
						}
						else {
							message = this.messages.saveSuccess;
						}
						this.editorInput = this._getEditorInput(grouping); // store current EditorInput
						this.getPageHelper().openEditor(this);
						this.onContentChange();
						this._enableSaveButtons(false);
						var status = {
								type : jazz.ui.MessageBox.STATUS_TYPES.OK,
								summary : message,
								closable : true
							};
						this.messageBox.showMessage(status);
						dojo.publish('ple.config_groupings_did_change'); //$NON-NLS-1$
					}),
					error: function(error, args) {
						self.messageBox.showXhrError(args.xhr, true);
					},
					handle: function() {
						self._autoScrollHeight.resize();
					}
				});
			},

			showPreview: function() {
				dojo.empty(this.previewContents);
				this._loadingMessage.show();
				var sparql = this.sparqlText.get("value"); //$NON-NLS-1$
				if (!sparql) {
					this._loadingMessage.showPlaceholder();
					return;
				}
				var self = this;
				jazz.client.xhrGet({
						url: net.jazz.ajax._contextRoot + "/ple/config_grouping", //$NON-NLS-1$
						handleAs: "json", //$NON-NLS-1$
						content: {
							sparql: sparql,
							method: "previewGrouping", //$NON-NLS-1$
							parameters: dojo.toJson(utils.getSystemParameterValues())
						},
						load: function(products) {
							if (!products || !products.length) {
								self._loadingMessage.showPlaceholder();
								return;
							} else {
								self._loadingMessage.hide();
							}
							dojo.forEach(products, function(product) {
								var productEntry  = this._drawProductEntry(product);
								dojo.place(productEntry, this.previewContents);
							}, self);
						},
						error: function(error, args) {
							self._loadingMessage.hide();
							self.messageBox.showXhrError(args.xhr, true);
						}
				});
			},

			_drawProductEntry: function(product) {
				var productEntry = dojo.create("div", {"class": "ple-EditConfigGrouping-productPreview"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var iconClassAndAltText = formatting.getIconClassAndAltTextForItem(product);
				var iconNode = dojo.create('img', { //$NON-NLS-1$
					height: '16', //$NON-NLS-1$
					width: '16', //$NON-NLS-1$
					className: iconClassAndAltText[0] + " r-valign-middle", //$NON-NLS-1$
					alt: iconClassAndAltText[1],
					src: this._blankGif
				}, productEntry);
				var formattedTitle = formatting.formatProductTitleForItem(product);
				var link = dojo.create('a', {innerHTML: formattedTitle, "class":"ple-EditConfigGrouping-previewTitle"}, productEntry, "last"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				var resourceLink = new jazz.ui.ResourceLink({
					uri: product.uri,
					lazyFetch: true,
					iconNode: iconNode
				}, link);
				this.resourceLinks.push(resourceLink);
				return productEntry;				
			},

			onCancel: function() {
				// Remove spell check attribute
				if(dojo.isFF !== undefined)
					dojo.body().removeAttribute("spellCheck"); //$NON-NLS-1$
				window.location.href = "#action=ple.manage_config_groupings"; //$NON-NLS-1$
			},

			uninitialize: function() {
				this._autoScrollHeight.destroy();
				dojo.forEach(this.resourceLinks, function(link) {link.destroyRecursive();}, this);
				delete this.resourceLinks;
			},
			
			_handleNavigateAway: function(url, e) {
				// Allow help links to open in a new window without ado
				if (e && e.target && (e.target.tagName == "A") && (e.target.target == "jazzhelp")) { //$NON-NLS-1$ //$NON-NLS-2$
					return false; // allow navigation
				}
				
				var currentLocation = window.location.href;
				var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1); //$NON-NLS-1$
				if (url == badLink) {
					return true;
				}
				
				if (this._isDirty()) {
					if (!window.confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) { //$NON-NLS-1$ //$NON-NLS-2$
						return true; // don't navigate away
					}
				}
				
				// Remove spell check attribute
				if(dojo.isFF !== undefined)
					dojo.body().removeAttribute("spellCheck"); //$NON-NLS-1$

				return false; // allow navigation
			},

			_handleOnBeforeUnload: function() {
				return (this._isDirty()) ? this._messages.unsavedChanges + "\n\n" : false; //$NON-NLS-1$
			},

			destroy: function(){
				// Remove spell check attribute
				if(dojo.isFF !== undefined)
					dojo.body().removeAttribute("spellCheck"); //$NON-NLS-1$
			
				if(orionEditor !== null)
					orionEditor.destroy();

				if (this.navigateAwayHandle) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
					this.navigateAwayHandle = null;
				}
				if (this.onBeforeUnloadHandle) {
					PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
					this.onBeforeUnloadHandle = null;
				}

				this.inherited(arguments);
			},
			
			getEditorInput: function() {
				return this.editorInput;
			},
			
			getPageRemovalId: function(){
				return PAGE_REMOVAL_ID;
			},
			
			getPageId: function(){
				return PAGE_ID;
			},

			getPageHelper: function(){
				return this.pageHelper;
			},
			
			getEditorView: function() {
				return this.pageHelper.getEditorView();
			},
			
			getEditorTrackerClass: function(){
				return com.ibm.relm.common.web.ui.editor.EditorMRU;
			},
			
			getDefaultUri: function(){
				return null;
			}
		});
	})();
