/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.configgroupings.ManageConfigGroupings"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.configgroupings", "ManageConfigGroupingsMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Separator"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.Toolbar"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.table.HoverRow"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.EasyMessageBox"); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.pages.ConfirmDialog"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.LoadingMessage"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util');//$NON-NLS-1$

(function(){
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var StyledTable = jazz.ui.StyledTable;
	var HTML = jazz.util.html;

	dojo.declare(
		"com.ibm.relm.ple.web.configgroupings.ManageConfigGroupings", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','configgroupings/templates/ManageConfigGroupings.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			// widgets: [],
			//groupings: null,

			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.configgroupings", "ManageConfigGroupingsMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function() {
				this.widgets = [];
			},

			postCreate: function() {
				var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
				ActionRegistry.registerAction("ple.manage_config_groupings", this, 'loadGroupings'); //$NON-NLS-1$ //$NON-NLS-2$

				this._table = new StyledTable({
					columns: [
						{label: this.messages.name, width: "9999px", sortable: false}, //$NON-NLS-1$
						{label: this.messages.createdBy, width: "1px", sortable: false}, //$NON-NLS-1$
						{label: this.messages.lastUpdated, width: "1px", sortable: false}, //$NON-NLS-1$
						{label: this.messages.actions, width: "1px", sortable: false} //$NON-NLS-1$
						],
					trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
					showHover: true}, this._tableHolder);
				this.widgets.push(this._table);
			},

			onShow: function() {
				PlatformUI.getWorkbench().setTitle(this.messages.pageTitle);
			},

			loadGroupings: function() {
				dojo.query('input[type="checkbox"]', this._table.domNode).attr('checked', false); //$NON-NLS-1$ //$NON-NLS-2$
				this._table.clearBody();
				this._updateExportButton();
				
				if (this.loading) return;

				var self = this;
				this.loading = jazz.client.xhrGet({
						url: net.jazz.ajax._contextRoot + "/ple/config_grouping", //$NON-NLS-1$
						handleAs: "json", //$NON-NLS-1$
						content: {
							method: "loadGroupings" //$NON-NLS-1$
						},
						load: function(groupings) {
							self.groupings = groupings;
							self.drawGroupings(groupings);
						},
						error: function(error, args) {
							self.groupings = null;
							self._messageBox.showXhrError(args.xhr, true);
						},
						handle: function() {
							self._loadingMessage.hide();
							self.loading = false;
						}
				});
			},

			drawGroupings: function(groupings) {
				var frag = document.createDocumentFragment();
				dojo.forEach(groupings, function(entry) {
					var hoverRow = this._createRowFromGrouping(entry);
					frag.appendChild(hoverRow.domNode);
				}, this);
				this._table.tbody.appendChild(frag);
			},

			_getSelectedRows: function() {
				return dojo.query('input[type="checkbox"]', this._table.domNode). //$NON-NLS-1$
				filter(function(checkbox) {
					// only get checked rows
					return checkbox.checked;
				}).
				map(function(checkbox) {
					// get parent widgets ( should be the HoverRow)
					return dijit.getEnclosingWidget(checkbox);
				});
			},

			_onSelectRow: function(e) {
				this._updateExportButton();
			},

			_updateExportButton: function() {
				var rows = this._getSelectedRows();
				this._exportButton.set('enabled', rows.length > 0); //$NON-NLS-1$	
			},

			_onExportSelected: function(e) {
				dojo.stopEvent(e);

				var queryUris = dojo.map(this._getSelectedRows(), function(row) {
					return row.queryUri;
				});

				com.ibm.relm.common.web.ui.util.downloadFile("/ple/content_package", 'configGroupingUri', queryUris, true); //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			* Takes in data about a product grouping and returns a hover row. This does not place the row.
			*
			* @return {HoverRow} A populated instance of a hover row
			*/
			_createRowFromGrouping: function(grouping) {
				var hoverRow = new com.ibm.relm.common.web.ui.table.HoverRow();
				this.widgets.push(hoverRow);
				hoverRow.queryUri = grouping.uri || ""; //$NON-NLS-1$
				hoverRow.name = grouping.name || ""; //$NON-NLS-1$

				var icon = dojo.replace('<img alt="" src="{0}" class="ple-icon-grouping ple-icon">', [this._blankGif]); //$NON-NLS-1$
				var name = HTML.escape(grouping.name);
				var creator = grouping.creator.name;
				var date = new Date(grouping.modified);
				var modified = dojo.date.locale.format(date, {formatLength: "medium"}); //$NON-NLS-1$

				var nameCell = dojo.create('td', { //$NON-NLS-1$
				});

				var checkbox = dojo.create('input', { //$NON-NLS-1$
					type: 'checkbox', //$NON-NLS-1$
					title: this.messages.exportCheckboxTooltip,
					className: 'ple-HoverRow-leadingCheckbox' //$NON-NLS-1$
				}, nameCell);
				this.connect(checkbox, 'onclick', this._onSelectRow); //$NON-NLS-1$

				dojo.create('a', { //$NON-NLS-1$
					href: "#action=ple.browse_products&show=" + encodeURIComponent(grouping.uri), //$NON-NLS-1$
					innerHTML: icon + name,
					title: dojo.replace(this.messages.showXInTree, [HTML.escape(name)])
				}, nameCell);

				hoverRow.addCell(nameCell);

				var creatorCell = dojo.create('td', { //$NON-NLS-1$
					className: 'ple-HoverRow-cinch' //$NON-NLS-1$
					//innerHTML: creator
					});
				
				 var creatorLink = dojo.create('a', { //$NON-NLS-1$
				 	href: grouping.creator.uri,
				 	innerHTML: grouping.creator.name
				 }, creatorCell);

				 var creatorResouceLink = new jazz.ui.ResourceLink({
				 	uri: grouping.creator.uri
				 }, creatorLink);

				hoverRow.addCell(creatorCell);

				var modifiedCell = dojo.create('td', { //$NON-NLS-1$
					className: 'ple-HoverRow-cinch', //$NON-NLS-1$
					innerHTML: modified});
				hoverRow.addCell(modifiedCell);

				var toolbar = hoverRow.toolbar;
				toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-showInTree', //$NON-NLS-1$
					title: this.messages.showInTree,
					onClick: dojo.hitch(this, this.showInTree, grouping)
				}));
				toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-ConfigGrouping-edit', //$NON-NLS-1$
					title: this.messages.edit,
					onClick: dojo.hitch(this, this.editGrouping, grouping)
				}));
				toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-ConfigGrouping-delete', //$NON-NLS-1$
					title: this.messages.deleteGrouping,
					onClick: dojo.hitch(this, this.deleteGrouping, hoverRow)
				}));
				return hoverRow;
			},

			showInTree: function(grouping) {
				window.location.href = "#action=ple.browse_products&show=" + encodeURIComponent(grouping.uri); //$NON-NLS-1$
			},

			editGrouping: function(grouping) {
				var args = {
					mode: "edit", //$NON-NLS-1$
					uri: grouping.uri,
					name: grouping.name
				};
				window.location.href = "#action=ple.edit_config_grouping&" + dojo.objectToQuery(args); //$NON-NLS-1$
			},

			deleteGrouping: function(hoverRow) {
				var self = this;

				var escapedName = jazz.util.html.escape(hoverRow.name);
				
				var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
					dialogTitle: this.messages.confirmDelete,
					prompt: this.messages.deletePrompt,
					details: dojo.replace(this.messages.deleteDetailsX, [escapedName]),
					iconPath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/delete.gif') //$NON-NLS-1$ //$NON-NLS-2$
				});

				d.addButton(this.messages.deleteButton, function() {
					return self._performDelete(hoverRow);
				}, true);

				d.startup();
			},

			_performDelete: function(hoverRow) {
				var deferred = new dojo.Deferred();
				var self = this;
				jazz.client.xhrDelete({
					url: net.jazz.ajax._contextRoot + "/ple/config_grouping", //$NON-NLS-1$
					content: {
						queryUri: hoverRow.queryUri
					},
					handle: function() {
						self._updateExportButton();
					},
					load: function() {
						self._animateDelete(hoverRow);
						dojo.publish('ple.config_groupings_did_change'); //$NON-NLS-1$
						deferred.callback();
					},
					error: function(error, args) {
						self._messageBox.showXhrError(args.xhr, true);
					}
				});
				return deferred;
			},

			_animateDelete: function(hoverRow) {
				dojo.fadeOut({node: hoverRow.domNode, duration: 150, onEnd: function() {
					hoverRow.destroyRecursive();
				}}).play();
			},

			newGrouping: function() {
				window.location.href = "#action=ple.edit_config_grouping&mode=create"; //$NON-NLS-1$
			},

			uninitialize: function() {
				dojo.forEach(this.widgets, function(widget) {
					widget.destroyRecursive();
				});
			}
		});

	})();