/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

/**
 * Copied from jazz.app.internal.OAuthResponseInterceptor
 */
(function() {
dojo.provide("relm.app.core.internal.AuthInterceptor"); //$NON-NLS-1$

dojo.require("relm.app.core.internal.AuthDialog"); //$NON-NLS-1$

var win = null;  // The oauth popup window
var winReq = null;  // The request associated with the current popup window
var winResp = null;  // The response associated with the current popup window
var winReqMgr = null;  // The request manager associated with the current popup window
var pendingUserAction = false;
var onPopupCloseInterval = null;

var queue = {};  // keeps track of a list of requests we need to open popups for 
var callbackQueue = [];  // keeps track of a list of requests that should be replayed when the current popup closes

var skipInterceptor= function(request){
	if(dojo.isArray(request.args.skipInterceptors)) {
		for(var i=0; i<request.args.skipInterceptors.length; i++) {
			// allow caller to handle OAuth directly
			if(request.args.skipInterceptors[i] === "jazz.oauth") { //$NON-NLS-1$
				return false;
			}
		}
	}
};

relm.app.core.internal.AuthInterceptor = function(request, response, reqMgr) {
	debugger;
	var oauthUrlStr = response.getHeader("X-jazz-web-oauth-url"); //$NON-NLS-1$
	if(!oauthUrlStr|| skipInterceptor(request)|| !detectPopupBlocker()) {
		return false;
	}
	
	var rewrittenOAuthUrl = getRewrittenOAuthAuthorizationUrl(oauthUrlStr);
	var dialog = new relm.app.core.internal.AuthDialog({
		width: "340px", //$NON-NLS-1$
		popupUrl: rewrittenOAuthUrl,
		onClose: function () {
			if (pendingUserAction) {
				triggerCancel();
			}
		}
	});
	
	if (!win && !pendingUserAction) {
		function triggerListen() {
			onPopupCloseInterval = setInterval(onPopupClose, 100);
		}
		
		function onPopupClose() {
			if(win && win.closed) {
				// cancel the request associated with this popup and all
				// of the queued requests associated with the same server
				triggerCancel();
			}
		}
		
		function triggerCancel() {
			winReq.clientDeferred.cancel();
			if (onPopupCloseInterval) {
				clearInterval(onPopupCloseInterval);
			}			
			while (callbackQueue.length > 0) {
				callbackQueue.shift()[0].clientDeferred.cancel();
			}
			win = null;
			pendingUserAction = false;
			triggerNextInQueue();
		}
		
		function triggerNextInQueue() {
			setTimeout(dojo.hitch(this, function () {
				for (var key in queue) {
					var nextItem = queue[key].shift();
					while (queue[key].length > 0) {
						callbackQueue.push(queue[key].shift());
					}
					delete queue[key];
					relm.app.core.internal.AuthInterceptor.apply(this, nextItem);
					break;
				}
			}), 0);
		}
		
		jazz.__oauthCallback = function(token, secret) {
			clearInterval(onPopupCloseInterval);
			pendingUserAction = false;
			
			// resend the original request
			var resendUrl = getResendUrl(winReq.args.url, token, secret);
			winReq.args.url = resendUrl;
			winReqMgr.forceRequest(winReq);
			
			function replayRequest (req) {
				// Delete the handle, load, and error methods from req so they don't run twice.
				// These methods are already registered as callbacks on req.clientDeferred
				dojo.forEach(["handle", "load", "error"], dojo.hitch(req, function (cbName) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					delete this.args[cbName];
				}));
				var dfd = jazz.client.xhr(req.method, req.args);
				dfd.addCallback(dojo.hitch(req, function (dfd, response) {
					this.remoteDeferred.ioArgs = dfd.ioArgs;
					this.clientDeferred.callback(response);
				}, dfd));
				dfd.addErrback(dojo.hitch(req, function (dfd, err) {
					this.remoteDeferred.ioArgs = dfd.ioArgs;
					this.clientDeferred.errback(err);
				}, dfd));
			}
			
			// resend all of the queued requests for this server
			while (callbackQueue.length > 0) {
				var curReq = callbackQueue.shift()[0];
				// we can't send the queued requests until we get a response from the first one
				// because it is the response from the first request that will set the cookies
				// that we need for the proxy.
				winReq.clientDeferred.addCallback(dojo.hitch(curReq, function () {
					replayRequest(this);				
				}));
				winReq.clientDeferred.addErrback(dojo.hitch(curReq, function (err) {
					if (err.status === 500) {
						// If the user didn't allow communication to the server,
						// cancel all the queued requests for this server
						this.clientDeferred.cancel();
					} else {
						replayRequest(this);
					}									
				}));
			}
			win.close();
			win = null;			
			triggerNextInQueue();
		};
		
		// if there is not already an oauth popup open, then launch a new window
		// for the user to accept or reject the authorization request
		winReq = request;
		winResp = response;
		winReqMgr = reqMgr;
		
		pendingUserAction = true;
		dojo.connect(dialog, "setWindow", this, function(winObj) { //$NON-NLS-1$
			win = winObj;
			pendingUserAction = false;
			dialog.close();
			triggerListen();
		});
	} else {
		// if there is already a popup window open, queue up the request
		var oauthUrlBase = response.getHeader("X-jazz-web-oauth-url").split("?")[0]; //$NON-NLS-1$ //$NON-NLS-2$
		if (oauthUrlBase === winResp.getHeader("X-jazz-web-oauth-url").split("?")[0]) { //$NON-NLS-1$ //$NON-NLS-2$
			// add it to the callback queue for this window if it is a request to the same server
			callbackQueue.push([request, response, reqMgr]);
		} else {
			// add it to the queue for a new popup window if it is not a request to the same server
			if (queue[oauthUrlBase]) {
				queue[oauthUrlBase].push([request, response, reqMgr]);
			} else {
				queue[oauthUrlBase] = [[request, response, reqMgr]];
			}
		}
	}
	
	return false;
};


function detectPopupBlocker() {
  return !window.open("about:blank","","directories=no,height=100,width=100,menubar=no,resizable=no,scrollbars=no,status=no,titlebar=no,top=0,location=no"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
}

/**
 * Rewrite the OAuth authorization URL to redirect to the canned popup rather
 * than to the originally requested URL. This is necessary because we're dealing
 * in XHR requests rather than page transitions.
 * rewrite OAuth URL to have callback to the popup rather than the original page
 * 
 * @param {Object} oauthUrlStr the original OAuth URL (as a string)
 */
function getRewrittenOAuthAuthorizationUrl(oauthUrlStr) {	
	var oauthUrl = new dojo._Url(oauthUrlStr);
	var newOauthUrl = oauthUrl.scheme + "://" + oauthUrl.host + (oauthUrl.port ? ":" + oauthUrl.port : "" ) + oauthUrl.path; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	var reqParms = getReqParms(oauthUrl);
	for(var i=0; i<reqParms.length; i++) {
		var nvp = reqParms[i];
		newOauthUrl += (i === 0 ? "?" : "&") + nvp.n + "=";  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		//console.warn("nvp.n: " + nvp.n + ", nvp.v: " + nvp.v);
		if(nvp.n === "oauth_callback") { //$NON-NLS-1$
			var winUrl = new dojo._Url(window.location.href);
			var popupUrl = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" ) + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					net.jazz.ajax._contextRoot + "/web/net.jazz.ajax.service/oauth/internal/OAuthRedirectPopup.html"; //$NON-NLS-1$
			var encodedUrlNvps = getReqParms(new dojo._Url(decodeURIComponent(nvp.v)));
			for(var j=0; j<encodedUrlNvps.length; j++) {
				if(encodedUrlNvps[j].n === "request_token_secret") { //$NON-NLS-1$
					popupUrl += "?request_token_secret=" + encodedUrlNvps[j].v; //$NON-NLS-1$
					break;
				}
			}
			newOauthUrl += encodeURIComponent(popupUrl);
		} else {
			newOauthUrl += nvp.v;
		}
	}
	return newOauthUrl;
}

/**
 * Modify the original URL to be resend by adding the "oauth_token" request
 * parameter.
 * 
 * @param {Object} origUrlStr the original URL that we are resending
 * @param {Object} authToken the new auth token to append as a query parameter
 */
function getResendUrl(origUrlStr, token, secret) {
	var origUrl = new dojo._Url(origUrlStr);
	var newUrl = ""; //$NON-NLS-1$
	if(origUrl.scheme && origUrl.host) {
		newUrl = origUrl.scheme + "://" + origUrl.host + (origUrl.port ? ":" + origUrl.port : "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	newUrl += origUrl.path;
	var reqParms = getReqParms(new dojo._Url(origUrlStr));
	reqParms.push({n: "oauth_token", v: token}); //$NON-NLS-1$
	reqParms.push({n: "request_token_secret", v: secret}); //$NON-NLS-1$
	for(var i=0; i<reqParms.length; i++) {
		newUrl += (i === 0 ? "?" : "&"); //$NON-NLS-1$ //$NON-NLS-2$
		newUrl += reqParms[i].n + "=" + reqParms[i].v; //$NON-NLS-1$
	}
	
	return newUrl;
}

function getReqParms(/* dojo._Url */url) {
	var nvps = [];
	var rawNvps = url.query ? url.query.split("&") : []; //$NON-NLS-1$
	for(var i=0; i<rawNvps.length; i++) {
		var nvp = rawNvps[i].split("="); //$NON-NLS-1$
		nvps.push({n: nvp[0], v: nvp[1]});
	}
	return nvps;
}

})();
