/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.util");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.client.xhr");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");

var ReviewEditorUtil= rmps.app.ui.ReviewEditorUtil;
var DesignUtil= rmps.app.ui.internal.pages.design.DesignUtil;

rmps.app.core.util = {
	
	MODELING_PAGE: "rmps.app.modeling",
	
	/*We keep this one to be used by all methods in the util, as getBasicPrefix is designed only to work for two methods getViewResourceUri and getSearchUri*/
	_basicPrefix: "#action=rmps.app.",
	
	adminBasicPrefix: "#action=dm.app.admin.",
	
	reviewBasicPrefix: "#action=dm.app.review.",
	
	_getBasicPrefix: function(resourceUri){
		/*
		 * Workaround till resolving:
		 * 
		 * 22213: rmps.app.core.util#getViewResourceUri always makes assumption that the view links belong to the models page
		 * 
		 * We assume tha the pageId would be passed, and if the page id is anything else other than dm.app.review, then, we will return the default _basicPrefix
		 * 
		 */
		var prefix= this._basicPrefix;
		if (resourceUri && resourceUri.indexOf(ReviewEditorUtil.RES_REVIEW_PREFIX)>-1){
			prefix= this.reviewBasicPrefix;
		}
		return prefix;
	},
	
	getWelcomeUri: function() {
		return this._basicPrefix + "welcome";
	},
	
	// TODO: Should call getDesignViewResourceUri once there are no clients of the modeling page left.
	getViewResourceUri: function(resourceUri)  {
		var obj = {"resourceUri": resourceUri};
		return this._getBasicPrefix(resourceUri) + "viewResource&" + dojo.objectToQuery(obj);
	},
 
	getDesignViewResourceUri: function(resourceUri)  {
		var prefix = this._getBasicPrefix(resourceUri);
		// Send everything to the design page, with the exception of reviews.
		if (prefix === this.reviewBasicPrefix) {
			var obj = {"resourceUri": resourceUri};
			return prefix + "viewResource&" + dojo.objectToQuery(obj);			
		}
		return DesignUtil.getEditResourceUri(resourceUri);
	},

	getSearchUri: function(queryString)  {
		var obj = {query: queryString};
		return this._getBasicPrefix() + "search&" + dojo.objectToQuery(obj);
	},
		
	getProxyRequestUri: function(uri){
		var obj = {
			"uri": uri
		};
		return rmps.core.RootServices.getUri(rmps.core.Namespaces.FP, "processProxy")+ "?"+ dojo.objectToQuery(obj);
	},

	getShowViewpointUri: function(viewpointUri) {
		if (viewpointUri) {
			var obj = {"viewpoint": viewpointUri};
			return this._basicPrefix + "design.explore&" + dojo.objectToQuery(obj);			
		}
		return this._basicPrefix + "design.explore";
	},

	logout: function() {
		if (!net.jazz.ajax.ui.PlatformUI.getWorkbench().shouldVetoNavigation()) {
			var dfd = jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + "/logout",
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				sync: true,
				load: function(res, ioArgs){
					jazz.client.xhrPost({
						url: ioArgs.xhr.getResponseHeader("X-rmps-logout-url"),
						headers: {
							"X-ibm-rmps-internal": "true"
						},
						sync: true,
						load: function(res, ioArgs){
							/*Required procedure to prevent the o auth interceptor from refreshing the auth information before the log out
							* process is done
							* Block popups first before reloading the page
							* 
							*IE onload behavior is different on onload way. So, we have to do it like that way 
							*/
							var frame = dojo.doc.createElement(dojo.isIE ? '<iframe title="Logout frame" onload="window.frameLoadCallback()">' : 'iframe');
					        frame.onload = window.frameLoadCallback = function() {
					           	/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
								dojo.doc.location.reload();
					        };
							/*We MUST refresh after making sure that the logout dialog is closed.
							 * So, we do that in the unload method of its window*/
							/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
					        frame.src= net.jazz.ajax._sharedJtsServerUrl + "/auth/logout";
					        //frame.src= net.jazz.ajax._dataServerRoot + "/auth/logout";
							frame.style.display= "none";
							
							dojo.body().appendChild(frame);
						}
					});
				}
			});
		}
		return false;
	},
	
	closeNavigationPanel: function(){
		var wb = dojo.query(".show-hide-arrow");
		var b = wb[0];
	
		var clickevent=document.createEvent("MouseEvents");
		clickevent.initEvent("click", true, true);
		b.dispatchEvent(clickevent);
	},
	
	checkIfMobile: function(){
		
		var mobile = false;
		
		var uagent = navigator.userAgent.toLowerCase();
		
		if( uagent.search( 'ipad' ) > -1 ){
			mobile = true;
		}
		
		return mobile;
	},
	
	/* 
	 * Load and run a given clazz asynchronously, and pass in the "args" in the constructor, and then make a callback for the given "func"
	 * 
	 * What's it used for? Make sure to avoid using dojo.require for the classes that we do not use right away (such as dialogs)
	 * Why's is it required to be here? I order to make sure that the class will be constructed at the scope of review controller, so all places will have a unified scope for that clazz
	 *   
	 */
	loadAndRun: function(clazz, args, func){
		var loader= function(classConstructor){
			var load = new classConstructor(args);
			if(dojo.isFunction(func)){
				func(load);
			}
		};
		var classConstructor= dojo.getObject(clazz);
		if(classConstructor){
			loader(classConstructor);
		}else{
			jazz.core.loader.load_async(clazz, dojo.hitch(this, function(){
				loader(dojo.getObject(clazz));									
			}));			
		}
	}
};

}());