/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.dnd.Avatar'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){

	/**
	 * A representation of something that's being dragged, but hasn't actually been moved yet.
	 * The icon you see while dragging.
	 * Currently, the actual content must be set as innerHTML on .contentNode.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.dnd.Avatar', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','dnd/templates/Avatar.html'), //$NON-NLS-1$ //$NON-NLS-2$

			hintIconClass: '', //$NON-NLS-1$
			hintText: '', //$NON-NLS-1$

			/**
			 * Set the initial start positions (absolute position on page).
			 * All movement is relative to this.
			 * @type {Number}
			 */
			startX: 0,
			startY: 0,

			postCreate: function() {
				this.inherited(arguments);
				this.moveTo(this.startX, this.startY );
			},

			moveBy: function(diffX, diffY) {
				this.moveTo(this.startX + diffX, this.startY + diffY);
			},

			moveTo: function(x, y) {
				dojo.style(this.domNode, {
					top: y + 'px', //$NON-NLS-1$
					left: x + 'px' //$NON-NLS-1$
				});
			},

			/**
			 * Moves the avatar back to its starting position.
			 * Used to "snap back" after canceling a drag. This
			 * does not destroy the avatar yet, just moves it.
			 * @param  {number} vOffset how far to shift the vertical start position.
			 * Used to compensate for the start position having been scrolled while
			 * dragging. Can be undefined.
			 */
			moveToStartPosition: function(vOffset) {
				vOffset = vOffset || 0;
				var deferred = new dojo.Deferred();

				dojo.animateProperty({
					node: this.domNode,
					duration: 200,
					properties: {
						top: this.startY + vOffset,
						left: this.startX
					},
					onEnd: function() {
						deferred.callback();
					}
				}).play();

				return deferred;
			},

			_hideIfEmpty: function() {
				if (!this.hintIconClass && !this.hintText) {
					this.hintNode.style.display = 'none'; //$NON-NLS-1$
				}
				else {
					this.hintNode.style.display = ''; //$NON-NLS-1$	
					this.hintTextNode.style.display = this.hintText ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
					this.hintImgNode.style.display = this.hintIconClass ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
				}
			},

			_setHintIconClassAttr: function(iconClass) {
				this.hintIconClass = iconClass;
				this.hintImgNode.className = iconClass || 'ple-icon-none'; //$NON-NLS-1$
				this._hideIfEmpty();
			},

			_setHintTextAttr: function(hintText) {
				this.hintText = hintText;
				com.ibm.relm.common.web.ui.util.setText(this.hintTextNode, hintText || ''); //$NON-NLS-1$
				this._hideIfEmpty();
			}
		}
	);

})();