/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 dojo.provide("com.ibm.relm.ple.web.doors.DoorsUtils"); //$NON-NLS-1$

 dojo.require('jazz.util.html'); //$NON-NLS-1$
 dojo.require('dojo.i18n'); //$NON-NLS-1$
 dojo.requireLocalization("com.ibm.relm.ple.web.doors", "DoorsUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$
 dojo.require('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$
 
 (function() {
 	var html = jazz.util.html;
 	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.doors", "DoorsUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

 	dojo.mixin(com.ibm.relm.ple.web.doors.DoorsUtils, {

 		DOORS_BASELINE_ICON: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/doorsBaseline.png'), //$NON-NLS-1$ //$NON-NLS-2$
 		DOORS_CONTRIBUTION_ICON: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/contribution.gif'), //$NON-NLS-1$ //$NON-NLS-2$

 		/**
 		 * Gets what we are using to represent the title of the baseline. For now that is just going to
 		 * be the version number.
 		 * 
 		 * @param  {Object} baseline the baseline for which we want the title
 		 * @return {String}          the title of the baseline
 		 */
 		getBaselineTitleHtml: function(baseline) {
 			var toReplace = (baseline.suffix) ? messages.baselineNameSuffix : messages.baselineNameNoSuffix;
 			return dojo.replace(toReplace, function(_, key) {
 				return html.escape(baseline[key]);
 			});
 		},

 		/**
 		 * Returns the arguments to be passed to the server to perform a save artifact link on 
 		 * a DOORS baseline. Some of the arguments needed for DOORS baselines are collected differently
 		 * and additional arguments may be needed.
 		 *
 		 * Designed to be called from LqeArtifactPicker only
 		 * 
 		 * @param  {Object} baseline         the json representation of the DOORS baseline
 		 * @return {Object}                  arguments that can be sent to the server to save the link
 		 */
 		saveLinkArgs: function(baseline) {
 			return {
				artifactUri: baseline.module,
				privateBaseline: baseline.uri,
				artifactTitle: this.getBaselineTitleHtml(baseline),
				artifactDescription: baseline.description,
				conceptTitle: baseline.moduleName
			};
 		},

 		/**
 		 * Does the XHR load for DOORs baselines of a specific module and returns a specific baseline
 		 * @param  {String} baselineUri    the url for the baseline to load
 		 * @return {Deferred}              A dojo.Deferred that callbacks can be attached to for when
 		 *                                   the baseline is loaded
 		 */
 		fetchBaseline: function(baselineUri) {
 			var deferred = new dojo.Deferred();

 			// The request to load the baselines must be formatted as XML. It's all boilerplate
 			// except the URI to the module, so load from a template.
 			var xmlRequest = dojo.cache('com.ibm.relm.ple.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
 			xmlRequest = dojo.replace(xmlRequest, [baselineUri]);

 			// Need to figure out where the DOORS REST API lives based on the
 			// selected requirement. Get the base URL for DOORS server by stripping
 			// off the requirement-specific URN bit and everything after.
 			var doorsBaseUrl = baselineUri.replace(/\/urn:.*/, ''); //$NON-NLS-1$

 			var self = this;
 			jazz.client.xhrPut({
 				url: doorsBaseUrl + '/dxl/baselineInfo', //$NON-NLS-1$
 				headers: {
 					'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
 					'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
 				},
 				handleAs: 'xml', //$NON-NLS-1$

 				putData: xmlRequest,
 			
 				load: function (resultDoc) {
 					var resultNode = resultDoc.getElementsByTagNameNS('http://jazz.net/doors/xmlns/prod/jazz/doors/2.0/', 'result')[0]; //$NON-NLS-1$ //$NON-NLS-2$
 					if (!resultNode) {
 						deferred.errback();
 						return;
 					}

 					var json = resultNode.textContent;
 					var baseline = dojo.fromJson(json).result;

 					// Adjust from seconds to millis
 					baseline.date *= 1000;
 					if (baseline) {
 						deferred.callback(baseline);
 					} else {
 						deferred.errback();
 					}
 				},
 			
 				error: function(error, ioArgs) {
 					deferred.errback(self.couldNotLoadBaselines(error, ioArgs));
 				}
 			});			
			return deferred;
 		},

 		/**
 		 * The error and ioArgs return from an xhr call. This wraps them in an error message to be shown
 		 * by a message box. 
 		 *
 		 * Written as a method so message files can be shared.
 		 * 
 		 * @param  {Object} error  the error returned from an xhr call
 		 * @param  {Object} ioArgs the ioArgs returned form an xhr call
 		 * @return {Object}        An error object that can be passed to the EasyMessageBox
 		 */
 		couldNotLoadBaselines: function(error, ioArgs) {
 			var xhr = ioArgs.xhr;
 			var errorObject = {
 				summary: messages.couldNotGetDoorsBaselines,
 				explanation: dojo.replace(messages.couldNotGetDoorsBaselines$explanation, [xhr.responseText || xhr.statusText]),
 				userAction: messages.couldNotGetDoorsBaselines$useraction
 			};
 			return errorObject;
 		},

 		/**
 		 * Generates a preview for the baseline since DOORS doesn't have previews for baselines, but
 		 * only for modules. 
 		 * 
 		 * @param  {Object} baseline the baseline to generate the preview for
 		 * @param  {boolean} includeName If true, then the baseline name is included in the preview table
 		 * @return {HTML}          returns a DOM element that can be placed directly on the page
 		 */
 		createBaselinePreview: function(baseline, includeName, conceptUri) {
 			var preview = dojo.create('div'); //$NON-NLS-1$

 			var table = dojo.place('<table class="ple-LqeArtifactPicker-previewTable ple-ArtifactView-previewTable"><col width="0"><col width="999"></table>', preview); //$NON-NLS-1$
 			
 			if (includeName) {
 				if (!com.ibm.relm.ple.web.rtc.RtcSettings.prototype50) {
	 				var baselineNameRow = dojo.create('tr', null, table); //$NON-NLS-1$
	 				dojo.create('th', { //$NON-NLS-1$
	 					className: 'j-label', //$NON-NLS-1$
	 					innerHTML: html.escape(messages.baselineName)
	 				}, baselineNameRow);
	 				dojo.create('td', { //$NON-NLS-1$
	 					innerHTML: dojo.replace(messages.baselineNameNoModule, function(_, key) {
	 						return html.escape(baseline[key]);
	 					})
	 				}, baselineNameRow);
 				}
 				else {
	 				var conceptRow = dojo.create('tr', null, table); //$NON-NLS-1$
	 				dojo.create('th', { //$NON-NLS-1$
	 					className: 'j-label', //$NON-NLS-1$
	 					innerHTML: html.escape(messages.conceptResource)
	 				}, conceptRow);
	 				var conceptTd = dojo.create('td', null, conceptRow); //$NON-NLS-1$
	 				
	 				var a = dojo.create('a', { //$NON-NLS-1$
						href: conceptUri,
						innerHTML: '...' //$NON-NLS-1$
					}, conceptTd);
	
					new jazz.ui.ResourceLink({}, a);	
	 				
	 				var configurationRow = dojo.create('tr', null, table); //$NON-NLS-1$
	 				dojo.create('th', { //$NON-NLS-1$
	 					className: 'j-label', //$NON-NLS-1$
	 					innerHTML: html.escape(messages.configuration)
	 				}, configurationRow);
	 				var configurationTd = dojo.create('td', null, configurationRow); //$NON-NLS-1$
	 				
	 				a = dojo.create('a', { //$NON-NLS-1$
						href: baseline.uri,
						innerHTML: dojo.replace(messages.baselineNameNoModule, function(_, key) {
							return html.escape(baseline[key]);
						})
					}, configurationTd);
	
					new jazz.ui.ResourceLink({retainLinkText: true}, a);
 				}
 			}

 			var createdByRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.createdBy)
 			}, createdByRow);
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: html.escape(baseline.user)
 			}, createdByRow);


 			var createdOnRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.createdOn)
 			}, createdOnRow);
 			var createdOnDateString = dojo.date.locale.format(new Date(baseline.date), {formatLength: "medium"}); //$NON-NLS-1$
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: createdOnDateString
 			}, createdOnRow);

 			var descriptionRow = dojo.create('tr', null, table); //$NON-NLS-1$
 			dojo.create('th', { //$NON-NLS-1$
 				style: {verticalAlign: 'top'}, //$NON-NLS-1$
 				className: 'j-label', //$NON-NLS-1$
 				innerHTML: html.escape(messages.description)
 			}, descriptionRow);
 			dojo.create('td', { //$NON-NLS-1$
 				innerHTML: html.escape(baseline.description)
 			}, descriptionRow);

 			return preview;
 		},

 		// TODO: this code is nearly identical to that in LqeArtifactPicker. We need to unit these (that should probably call this).
 		loadBaselinesForModule: function(doorsModuleUrl, doorsModuleName) {
 			// The request to load the baselines must be formatted as XML. It's all boilerplate
 			// except the URI to the module, so load from a template.
 			var xmlRequest = dojo.cache('com.ibm.relm.ple.web', 'ui/picker/requests/doorsListOfBaselinesForModuleX.xml'); //$NON-NLS-1$ //$NON-NLS-2$
 			xmlRequest = dojo.replace(xmlRequest, [doorsModuleUrl]);

 			// Need to figure out where the DOORS REST API lives based on the
 			// selected requirement. Get the base URL for DOORS server by stripping
 			// off the requirement-specific URN bit and everything after.
 			var doorsBaseUrl = doorsModuleUrl.replace(/\/urn:.*/, ''); //$NON-NLS-1$

 			var deferred = new dojo.Deferred();

 			var self = this;
 			jazz.client.xhrPut({
 				url: doorsBaseUrl + '/dxl/listOfBaselines', //$NON-NLS-1$
 				headers: {
 					'Accept': 'application/rdf+xml', //$NON-NLS-1$ //$NON-NLS-2$
 					'Content-Type': 'application/rdf+xml' //$NON-NLS-1$ //$NON-NLS-2$
 				},
 				handleAs: 'xml', //$NON-NLS-1$

 				putData: xmlRequest,
 			
 				load: function (resultDoc) {
 					var baselines = [];

 					var resultNode = resultDoc.getElementsByTagNameNS('http://jazz.net/doors/xmlns/prod/jazz/doors/2.0/', 'result')[0]; //$NON-NLS-1$ //$NON-NLS-2$
 					if (resultNode) {
 						var json = resultNode.textContent;
 						baselines = dojo.fromJson(json).result;
 					}

 					var relevance = 0;

					dojo.forEach(baselines, function(baseline) {
						// convert from secs to millis
						baseline.date *= 1000;
						baseline.isDoorsBaseline = true;
						baseline.module = doorsModuleUrl;
						baseline.moduleName = doorsModuleName;

						baseline.relevance = relevance++;
						baseline.text = self.getBaselineTitleHtml(baseline);
						baseline.createdTime = baseline.date;
						baseline.contributorName = baseline.user;
						baseline.iconClass = 'ple-icon ple-icon-doorBaseline'; //$NON-NLS-1$
						// OK, version is not ID, but there's no exact match. It's close.
						baseline.id = baseline.version;
					});

 					deferred.callback(baselines);
 				},

 				error: function(error, ioArgs) {
 					var errorObject = self.couldNotLoadBaselines(error, ioArgs);
 					deferred.errback(errorObject);
 				}
 			});
	
			return deferred;
 		}

 	});
	

	jazz.client.xhrGet({
		url: net.jazz.ajax._contextRoot + '/ple/doors_settings', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		failOk: true,

		load: function(settings) {
			dojo.mixin(com.ibm.relm.ple.web.doors.DoorsUtils, settings);
		},

		error: function(error, ioArgs) {
			//Do nothing
		}
	});
 }());