/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.common.web.enums.EnumManagerDialog'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.enums.EnumProperties'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.enums.ExportEnumsDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.enums.ExternalEnumProperties'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.ComboButton'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.MenuItem'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('dojox.widget.Standby'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.MenuButton'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Separator'); //$NON-NLS-1$


dojo.requireLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
	/**
	 * Dialog for managing local and external enumerations.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.enums.EnumManagerDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'enums/templates/EnumManagerDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			
			width: '750', //$NON-NLS-1$
			//This is passed directly to the dialog constructor to prevent the X in the corner, which causes strange interactions 
			//for the parameter table
			closable: false,
			_enumListbox: null, // the UI control (select) showing the available enums
			_displayArea: null, // the place to show the enum details and values
			_enumToWidgetMap: null, // uri -> EnumProperties / ExternalEnumProperties
			_enumStore: null, // backing data source
			standby: null,

			/**
			 * @param args.onSave Optional callback that receives a new store with the
			 * 		latest enumeration changes upon a successful save. If the save was
			 * 		a no-op then the store will be null.
			 */
			constructor: function(args, node) {
				this._currentWidget = null;
				this._enumToWidgetMap = {};
				this._enumStore = new com.ibm.relm.common.web.enums.EnumStore();
				
				// Track the deletion of both local and external enums, and the
				// addition of external enums. The creation and modification of
				// local enums will be computed later (need to compare what's
				// in the widgets to the store).
				this.enumChanges = {
					//enumsToUpdate: [],
					enumUrisToDelete: [],
					externalEnumUrisToAdd: [],
					externalEnumUrisToDelete: []
				};
			},

			postCreate: function() {
				this.inherited(arguments);

				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.loadingStatus);
				
				// Link up labels to fields
				var listboxId = dijit.getUniqueId('listboxId'); //$NON-NLS-1$
				var enumTitle = dojo.query('.relm-Section-title', this.domNode)[0]; //$NON-NLS-1$
				dojo.attr(this._enumListbox, 'id', listboxId); //$NON-NLS-1$
				dojo.attr(enumTitle, 'for', listboxId); //$NON-NLS-1$

				this.standby = new dojox.widget.Standby({target: this.domNode});
				this.domNode.appendChild(this.standby.domNode);
				this.standby.startup();
				
				this.buildAddComboButton();
				
				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					iconPath: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/addorderedlist_enabled.gif'), //$NON-NLS-1$ //$NON-NLS-2$
					width: this.width + 'px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					closable: this.closable
				});
				this._dialog.startup();
				
				// Retrieve the enums from the server
				this._enumStore.load().then(dojo.hitch(this, function() {
					this._populateEnumListbox();
					message.hide();
					this._showNoEnumSelected();
					this.addButton.set('enabled', true); //$NON-NLS-1$
					
					if (this._enumListbox.length > 0) {
						this.submitButton.disabled = false;
					}
					
				}),
				dojo.hitch(this,  function(error) {
					message.hide();
					this._messageBox.showErrorObject(error);
				}));
			},
			
			/**
			 * Creates the add combo button.
			 */
			buildAddComboButton: function() {
				var menuItems = [
  				{
  					label: this.messages.addNewEnumMenuItem,
  					title: this.messages.addNewEnumMenuItemTooltip,
  					iconClass: 'relm-EnumManagerDialog-createIcon', //$NON-NLS-1$
  					onClick: dojo.hitch(this, this._onCreateEnum)
  					
  				},
  				{
  					label: this.messages.addExistingEnumMenuItem,
  					title: this.messages.addExistingEnumMenuItemTooltip,
  					iconClass: 'relm-EnumManagerDialog-linkIcon', //$NON-NLS-1$
  					onClick: dojo.hitch(this, this._onAddExternalEnum)
  				}];

 				this.addButton = new com.ibm.relm.common.web.ui.ComboButton({
 					text: this.messages.addEnumButton,
 					title: this.messages.addEnumButtonTooltip,
 					menuItems: menuItems,
 					iconClass: 'relm-EnumManagerDialog-createIcon', //$NON-NLS-1$
 					enabled: false,
 					onClick: dojo.hitch(this, this._onCreateEnum)
 				}, this.addButton);
 				this.addButton.startup();
			},

			/**
			 * This function is available to be overridden by callers or used with dojo.connect.
			 */
			onSave: function(updatedStore) {
				//No-op	
			},

			/**
			 * 	This function is available to be overridden by callers or used with dojo.connect.
			 */
			onCancel: function() {
				//No-op
			},
			
			/**
			 * Populate the listbox of enums from the backing store. The
			 * value of each option is the enum's URI. Note that this
			 * call triggers the backing store to re-sort itself.
			 * 
			 * @param {string} uri Optional uri that should be selected.
			 */
			_populateEnumListbox: function(uriToSelect) {
				removeChildren(this._enumListbox);
				
				dojo.forEach(this._enumStore.getEnums(), function(enumeration){
					var option = document.createElement('option'); //$NON-NLS-1$
					option.text = enumeration.label;
					option.value = enumeration.uri;
					if (enumeration.uri == uriToSelect) {
						option.selected = true;
					}
					this._enumListbox.add(option, null);
				}, this);
			},

			_showNoEnumSelected: function() {
				this._hideEnumPropertiesWidget();
				this._addUriForm.style.display = 'none'; //$NON-NLS-1$
				this._nothingSelectedMessage.style.display = 'block'; //$NON-NLS-1$
				this._detailsSection.domNode.style.display = 'block'; //$NON-NLS-1$
			},

			_showEnumPropertiesWidget: function(widget, isNew) {
				this._hideEnumPropertiesWidget();
				this._detailsSection.domNode.style.display = 'none'; //$NON-NLS-1$
				this._displayArea.appendChild(widget.domNode);
				this._currentWidget = widget;
				
				if (isNew) {
					widget.startup();
				}
			},

			_hideEnumPropertiesWidget: function() {
				if (this._currentWidget) {
					this._displayArea.removeChild(this._currentWidget.domNode);
					this._currentWidget = null;
				}
			},

			_showAddExternal: function() {
				this._hideEnumPropertiesWidget();
				this._loadButton.disabled = this._uriInput.value.length === 0;
				this._addUriForm.style.display = 'block'; //$NON-NLS-1$
				this._nothingSelectedMessage.style.display = 'none'; //$NON-NLS-1$
				this._detailsSection.domNode.style.display = 'block'; //$NON-NLS-1$
			},
			
			/**
			 * Swap in the appropriate widget when a different enum is
			 * selected by the user.
			 */
			_onEnumChange: function(e) {
				var index = this._enumListbox.selectedIndex;
				var uri = this._enumListbox[index].value;
				var widget = this._enumToWidgetMap[uri];
				var isNew = false;
				
				if (!widget) { // create a widget for the enum if one doesn't exist
					isNew = true;
					var enumeration = this._enumStore.getEnum(uri);
					var args = { enumeration: enumeration };
					
					if (enumeration.type == 'external') { //$NON-NLS-1$
						widget = new com.ibm.relm.common.web.enums.ExternalEnumProperties(args);
					}
					else {
						widget = new com.ibm.relm.common.web.enums.EnumProperties(args);
						this.connect(widget, 'onNameChange', dojo.hitch(this, this._handleNameChange)); //$NON-NLS-1$
					}
					
					this._enumToWidgetMap[uri] = widget;
				}
				
				this._showEnumPropertiesWidget(widget, isNew);
				
				this.deleteButton.set('enabled', true); //$NON-NLS-1$
			},

			/**
			 * Create an enum.
			 */
			_onCreateEnum: function(e) {
				var initialName = this.messages.initialNewEnumName;
				var enumeration = this._enumStore.addLocalEnum(initialName);
				enumeration.members = [com.ibm.relm.common.web.enums.EnumStore.newEnumMember()];
				
				this._populateEnumListbox(enumeration.uri);
				this._onEnumChange();
				this._currentWidget.selectName();
				this.submitButton.disabled = false;
			},
			
			/**
			 * Add an external enum by URI.
			 */
			_onAddExternalEnum: function(e) {
				this.deleteButton.set('enabled', false); //$NON-NLS-1$
				this._showAddExternal();
				this._enumListbox.selectedIndex = -1;
			},

			_onExport: function(e) {
				dojo.stopEvent(e);

				var submitDisabled = this.submitButton.disabled;
				this.submitButton.disabled = true;
				var cancelDisabled = this.cancelButton.disabled;
				this.cancelButton.disabled = true;

				// Nesting model dialogs looks ugly. This tries to make the rear
				// one look less harsh. A bit of a shortcut for a feature that's not used too much.
				dojo.style(this._dialog.domNode, 'opacity', 0.8); //$NON-NLS-1$
				this._statusWrapper.style.display = 'none'; //$NON-NLS-1$

				var self = this;
				new com.ibm.relm.common.web.enums.ExportEnumsDialog({
					enumStore: this._enumStore,
					onClose: function() {
						self.submitButton.disabled = submitDisabled;
						self.cancelButton.disabled = cancelDisabled;
						dojo.style(self._dialog.domNode, 'opacity', 1.0); //$NON-NLS-1$
						self._statusWrapper.style.display = ''; //$NON-NLS-1$
					}
				});
			},
			
			_loadExternalUri: function(e) {
				e && dojo.stopEvent(e);
				
				this._messageBox.hideMessage();
				var uri = dojo.trim(this._uriInput.value);
				
				if (this._enumStore.getEnum(uri)) {
					this._messageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: dojo.replace(this.messages.externalUriAlreadyAdded, [uri]),
						closable: true
					});
					return;
				}
				
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.loadingStatus);
				
				this._enumStore.addExternalEnum(uri).then(dojo.hitch(this, function(enumeration) {
					this._populateEnumListbox(enumeration.uri);
					this._onEnumChange();
					this._uriInput.value = ''; //$NON-NLS-1$
					
					var removeArrayIndex = dojo.indexOf(this.enumChanges.externalEnumUrisToDelete, uri);
					if (removeArrayIndex == -1) { // haven't seen this one before; add it
						this.enumChanges.externalEnumUrisToAdd.push(enumeration.uri);
					} else { // existing one we soft-deleted; just take off the delete list
						this.enumChanges.externalEnumUrisToDelete.splice(removeArrayIndex, 1);
					}
					
					message.hide();
					this.submitButton.disabled = false;
				}),
				dojo.hitch(this,  function(error) {
					message.hide();
					this._messageBox.showErrorObject(error, true);
				}));
			},
			
			_uriKeyUp: function(e) {
				this._loadButton.disabled = this._uriInput.value.length === 0;
			},
			
			/**
			 * Delete an enum.
			 */
			_onDeleteEnum: function(e) {
				this.deleteButton.set('enabled', false); //$NON-NLS-1$
				this._showNoEnumSelected();

				var index = this._enumListbox.selectedIndex;
				var uri = this._enumListbox[index].value;
				var widget = this._enumToWidgetMap[uri];
				var enumeration = this._enumStore.removeEnum(uri);
				
				if (enumeration.type == 'external') { //$NON-NLS-1$
					var addArrayIndex = dojo.indexOf(this.enumChanges.externalEnumUrisToAdd, uri);
					if (addArrayIndex == -1) { // not found, and thus an existing external enum
						this.enumChanges.externalEnumUrisToDelete.push(uri);
					} else { // new external enum that hasn't been committed
						this.enumChanges.externalEnumUrisToAdd.splice(addArrayIndex, 1);
					}
				} else {
					if (!com.ibm.relm.common.web.enums.EnumStore.uriIsNew(uri)) {
						this.enumChanges.enumUrisToDelete.push(uri);
					}
				}

				delete this._enumToWidgetMap[uri];
				this._enumListbox.remove(index);
				widget.destroyRecursive();
			},
			
			_handleNameChange: function(uri, newValue) {
				this._enumStore.relabelEnum(uri, newValue);
				this._populateEnumListbox(uri);
			},
			
			/**
			 * Identify the local enums that are dirty (need to be saved).
			 * This is done by interrogating the widgets, as only enums
			 * having a widget could have been modified (or created).
			 * 
			 * @throws exception if any of the enums fail to validate
			 */
			_getEnumsToUpdateCreate: function() {
				var enumsToUpdate = [];
				
				for (var uri in this._enumToWidgetMap) {
					var widget = this._enumToWidgetMap[uri];
					
					if (widget.getCurrentRepresentationIfDirty) {
						var enumeration = widget.getCurrentRepresentationIfDirty();
						if (enumeration) {
							enumsToUpdate.push(enumeration);
						}
					}
				}
				
				return enumsToUpdate;
			},

			cancel: function(e) {
				this._dialog.close();
				this.onCancel();
			},
			
			onSubmit: function(e) {
				e && dojo.stopEvent(e);
				this.standby.show();
				
				this._messageBox.hideMessage();
				
				try {
					this.enumChanges.enumsToUpdate = this._getEnumsToUpdateCreate();
				} catch (e) {
					this._messageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: e.message,
						closable: true
					});
					this.standby.hide();
					return false;
				}
				
				var numChanges = 0;
				for (var list in this.enumChanges) {
					numChanges += this.enumChanges[list].length;
				}
				
				if (numChanges > 0) {
					console.debug('There were %d changes to the enumerations.', numChanges); //$NON-NLS-1$
					var self = this;
					
					this.saveChanges(this.enumChanges).then(function(store) {
						self._dialog.close();
						self.onSave(store);
					},
					function(error) {
						self._messageBox.showErrorObject(error);
						self.standby.hide();
					});
				} else {
					console.debug('There were no changes to the enumerations--nothing to save.'); //$NON-NLS-1$
					this._dialog.close();
					this.onSave(null);
				}
				
				return false;
			},
			
			/**
			 * Persist changes to the enums on the server and retrieve the
			 * latest enumerations for the callback.
			 * 
			 * @param enumChanges.enumsToUpdate List of local enums to update.
			 * @param enumChanges.enumUrisToDelete List of URIs of local enums to delete.
			 * @param enumChanges.externalEnumUrisToAdd List of URIs of external enums to add.
			 * @param enumChanges.externalEnumUrisToDelete List of URIs of external enums to delete.
			 * @return {Deferred}
			 */
			saveChanges: function(enumChanges) {
				console.debug('enumChanges', enumChanges); //$NON-NLS-1$
				var deferred = new dojo.Deferred();
				
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/enum', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					failOk: true,
					content: {
						enumsToUpdate: dojo.toJson(enumChanges.enumsToUpdate),
						enumUrisToDelete: dojo.toJson(enumChanges.enumUrisToDelete),
						externalEnumUrisToAdd: dojo.toJson(enumChanges.externalEnumUrisToAdd),
						externalEnumUrisToDelete: dojo.toJson(enumChanges.externalEnumUrisToDelete)
					},

					load: function(enums) {
						var freshEnumStore = new com.ibm.relm.common.web.enums.EnumStore(enums);
						deferred.callback(freshEnumStore);
					},

					error: dojo.hitch(this, function(error, ioArgs) {
						deferred.errback(dojo.fromJson(ioArgs.xhr.responseText));
					})
				});

				return deferred;
			},
			
			uninitialize: function() {
				for (var uri in this._enumToWidgetMap) {
					var widget = this._enumToWidgetMap[uri];
					widget.destroyRecursive();
				}
				
				this.standby.destroyRecursive();
				this.addButton.destroyRecursive();
			}
		}); // declare

	/**
	 * IE 9 is not working with dojo.empty. This does the same thing logically--
	 * removes all child nodes from a parent node.
	 * @param  {DOM node} parent node to remove all children from.
	 */
	function removeChildren(parent) {
		var child = parent.lastChild;

		while (child) {
			parent.removeChild(child);
			child = parent.lastChild;
		}
	}
})();
