/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.require('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$
dojo.provide('com.ibm.relm.common.web.enums.EnumValueRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
	/**
	 * A table row with 3 columns: name (label), query substitution value, and
	 * an actions toolbar. A row can be moved up, down, or deleted. Leverages
	 * the styles defined in HoverRow.css.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.enums.EnumValueRow', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'enums/templates/EnumValueRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			
			/**
			 * Create a new enum value row.
			 * 
			 * @param args.enumValue An enumeration value (member): { uri, label, substitution }
			 * 		A URI is assigned if not specified (e.g., for new members).
			 * @param node Optional node to replace.
			 */
			constructor: function(args, node) {
				this.enumValue = args.enumValue;
				
				if (!this.enumValue.uri) {
					this.enumValue.uri = com.ibm.relm.common.web.enums.EnumStore.generateUniqueTempUri();
				}
			},
			
			/**
			 * Return an enum member (enum value) representing the current state
			 * of the row.
			 */
			getCurrentEnumValue: function() {
				return { uri: this.enumValue.uri,
					label: this._nameInput.value,
					substitution: this._substitutionInput.value
				};
			},
			
			/**
			 * A new row with neither a name or substitution value defined. For
			 * validation purposes, ignore such rows.
			 */
			isNewAndEmpty: function() {
				return com.ibm.relm.common.web.enums.EnumStore.uriIsNew(this.enumValue.uri)
					&& !this._nameInput.value && !this._substitutionInput.value;
			},
			
			/**
			 * A name (label) field is required. New rows that are completely
			 * empty are ignored and thus considered complete.
			 */
			isComplete: function() {
				return isNewAndEmpty() || this._nameInput.value.length > 0;
			},
			
			/**
			 * Move a row up by one, wrapping if necessary.
			 */
			moveUp: function(e) {
				var previous = this.domNode.previousSibling;
				if (previous == null) { // wrap to bottom
					dojo.place(this.domNode, this.domNode.parentNode, 'last'); //$NON-NLS-1$
				} else {
					dojo.place(this.domNode, previous, 'before'); //$NON-NLS-1$
				}
				
				e.target.focus();
				com.ibm.relm.common.web.ui.util.flashNode(this.domNode);
			},
			
			/**
			 * Move a row down by one, wrapping if necessary.
			 */
			moveDown: function(e) {
				var next = this.domNode.nextSibling;
				if (next == null) { // wrap to top
					dojo.place(this.domNode, this.domNode.parentNode, 'first'); //$NON-NLS-1$
				} else {
					dojo.place(this.domNode, next, 'after'); //$NON-NLS-1$
				}
				
				e.target.focus();
				com.ibm.relm.common.web.ui.util.flashNode(this.domNode);
			},
			
			/**
			 * Remove the row from the table.
			 */
			remove: function(e) {
				this.destroyRecursive();
			},
			
			focus: function() {
				this._nameInput.focus();
			},
			
			/**
			 * Default the substitution string to the name value. The user can
			 * still delete or change the substitution string.
			 */
			_nameChange: function(e) {
				if (this._nameInput.value && !this._substitutionInput.value) {
					this._substitutionInput.value = this._nameInput.value;
				}
			},
			
			// Copied from HoverRow.js
			_onRowFocus: function() {
				dojo.addClass(this.domNode, 'ple-HoverRow-focused'); //$NON-NLS-1$
			},

			// Copied from HoverRow.js
			_onRowBlur: function() {
				dojo.removeClass(this.domNode, 'ple-HoverRow-focused'); //$NON-NLS-1$
			}
		}); // declare
	})();