/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.enums.ExportEnumsDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.enums', 'ExportEnumsDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Separator'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$

(function(){
	var html = jazz.util.html;
	var StatusMessage = com.ibm.relm.common.web.StatusMessage;
	var EnumStore = com.ibm.relm.common.web.enums.EnumStore;

	dojo.declare(
		'com.ibm.relm.common.web.enums.ExportEnumsDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','enums/templates/ExportEnumsDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'ExportEnumsDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			// passed from parent widget
			enumStore: null,

			/**
			 * Callback when this dialog is closed.	
			 * @type {function}
			 */	
			onClose: null,

			postCreate: function() {
				this.inherited(arguments);

				var columns = [
					{
						label: this.messages.exportHeader,
						width: '1px' //$NON-NLS-1$
					},
					{
						label: this.messages.enumerationHeader,
						width: '9999px' //$NON-NLS-1$
					}
				];

				// Copy enums so they can't change on us
				this._enums = this.enumStore.getEnums().slice(0);
				var self = this;
				this._enums = dojo.filter(this._enums, function(enumeration) {
					console.debug(enumeration.uri);
					console.debug(self.enumStore.NEW_URI_PREFIX);
					console.debug(enumeration.uri.indexOf(self.enumStore.NEW_URI_PREFIX));
					return enumeration.uri.indexOf(EnumStore.NEW_URI_PREFIX) !== 0;
				});

				this._enumTable = new jazz.ui.StyledTable({
					columns: columns
				}, this._enumTable);

				this._initTable();

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: '600px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					closable: true,
					onClose: this.onClose,
					iconClass: 'ple-icon-download' //$NON-NLS-1$
				});
			},

			_initTable: function() {
				var enums = this._enums;
				var frag = document.createDocumentFragment();

				for (var i = 0; i < enums.length; i++) {
					var enumeration = enums[i];

					var row = dojo.create('tr', null, frag); //$NON-NLS-1$

					var id = dijit.getUniqueId('ExportEnumsDialogCheck'); //$NON-NLS-1$

					var checkCell = dojo.create('td', { //$NON-NLS-1$
						className: 'relm-EnumManager-exportCheckCell' //$NON-NLS-1$
					}, row);

					var checkbox = dojo.create('input', { //$NON-NLS-1$
						type: 'checkbox', //$NON-NLS-1$
						title: this.messages.clickToSelectTooltip,
						id: id,
						checked: true
					}, checkCell);

					this.connect(checkbox, 'onclick', this._onChecked); //$NON-NLS-1$

					var labelCell = dojo.create('td', null, row); //$NON-NLS-1$

					var label = dojo.create('label', { //$NON-NLS-1$
						htmlFor: id,
						innerHTML: html.escape(enumeration.label)
					}, labelCell);
				}

				this._enumTable.tbody.appendChild(frag);
			},

			_onChecked: function(e) {
				var checks = dojo.query('input[type="checkbox"]', this._enumTable.domNode); //$NON-NLS-1$

				var checkedCount = 0;
				var uncheckedCount = 0;

				for (var i = 0; i < checks.length; i++) {
					var check = checks[i];
					if (check.checked) {
						checkedCount++;
					}
					else {
						uncheckedCount++;
					}
				}

				this._exportButton.disabled = checkedCount === 0;
			},

			_getSelectedEnumerationUris: function() {
				var checks = dojo.query('input[type="checkbox"]', this._enumTable.domNode); //$NON-NLS-1$

				var selectedUris = [];

				for (var i = 0; i < checks.length; i++) {
					var checkbox = checks[i];
					if (checkbox.checked) {
						selectedUris.push(this._enums[i].uri);
					}
				}

				return selectedUris;
			},

			onExport: function(e) {
				dojo.stopEvent(e);

				dojo.stopEvent(e);

				var enumerationUris = this._getSelectedEnumerationUris();

				com.ibm.relm.common.web.ui.util.downloadFile('/ple/content_package', "enumerationUri", enumerationUris, true); //$NON-NLS-1$ //$NON-NLS-2$

				var message = StatusMessage.show(this.messages.exportProgess);
				this._exportButton.disabled = true;

				// delay closing the window so it looks like something is happening
				var self = this;
				var timeout = setTimeout(function() {
					message.hide();
					self._dialog.close();
				}, 500);
			},

			onCancel: function(e) {
				this._dialog.close();
			},

			uninitialize: function() {
				this.inherited(arguments);
			}
		}
	);

})();