/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.common.web.enums.ExternalEnumProperties'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

	/**
	 * ExternalEnumProperties is a widget for displaying the read-only
	 * properties of an external enumeration.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.enums.ExternalEnumProperties', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','enums/templates/ExternalEnumProperties.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			enumeration: null,
			_table: null,
			
			/**
			 * 
			 * @param args.enumeration Required. An enum object of type 'external':
			 * 		{uri, label, type, members: [{uri, label, substitution}, ...]}.
			 * @param args.dirtyFlag Optional. 'unchanged', 'added', or 'deleted'
			 * @param node Optional node to replace.
			 */
			constructor: function(args, node) {
				this.enumeration = args.enumeration;
			},

			postCreate: function() {
				this.inherited(arguments);
				
				var args = {
					columns: [
						{label: this.messages.nameColumnHeading},
						{label: this.messages.substitutionColumnHeading},
					]
				};
				
				this._table = new jazz.ui.StyledTable(args);
				dojo.addClass(this._table.domNode, 'relm-EnumProperties-table'); //$NON-NLS-1$
				
				dojo.forEach(this.enumeration.members, function(member){
					var tr = dojo.create('tr'); //$NON-NLS-1$
					var td1 = dojo.create('td', {innerHTML: jazz.util.html.escape(member.label)}); //$NON-NLS-1$
					var td2 = dojo.create('td', {innerHTML: jazz.util.html.escape(member.substitution)}); //$NON-NLS-1$
					tr.appendChild(td1);
					tr.appendChild(td2);
					this._table.tbody.appendChild(tr);
				}, this);
				
				this._table.placeAt(this._enumProperties);
			},

			uninitialize: function() {
				this._table.destroyRecursive();
			}

		}); // declare

	})();