/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.common.web.error"); //$NON-NLS-1$

dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("dojox.xml.DomParser"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.common.web", "errorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("com.ibm.relm.common.web", "errorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

//private factory that creates com.ibm.relm.common.web.Error objects
var _ErrorFactory = { 
  create: function(message, action){
        var result;
        if(dojo.isString(message)){
            return new com.ibm.relm.common.web.Error({
                message: message,
                detailedMessage: null,
                action: action
            });
        }else if(message instanceof Error){
            return this.createFromError(message, action);
        }else{
            return this.createFromError(new Error(), action);
        }    
  },
  
  createFromError: function(error, action){
		return new com.ibm.relm.common.web.Error({
            error: error,
			severity: error.severity,
			message: error.message,
			// only add a stack trace while in debug mode
			detailedMessage: dojo.config.isDebug && error.stack ? error.stack : null,
			status: error.status ? error.status : error.number ? error.number.toString() : null,
			action: action  
		});
  }  
};

/*
 * This class is under review.  Please see Task 26323  
 */

com.ibm.relm.common.web.error = {
	
	Channel: "com.ibm.relm.common.web.Error.event", //$NON-NLS-1$
	
	Severity: {
		INFO: "INFO",	// Not used //$NON-NLS-1$
		WARNING: "WARNING", //$NON-NLS-1$
		ERROR: "ERROR", //$NON-NLS-1$
		CRITICAL: "CRITICAL" //does not have it's own error box (jazz.ui.messagebox) //$NON-NLS-1$
	},
	
	// General API for errors: (* indicates a required field)
	// Date*: Time stamp of when the error occurred
	// Message*: Title of the error, "Could not find document X"
	// Details*: Details of the error, "While searching document server Y, the document X could not be found"
	// Action*: What you were trying to do (in the code), "Automatically load last saved document"
	// Caused By: If this error was caused by another error, pass that error object in
	// Solution: If there is a well-known solution, provide a link to that document
	
	create: function(/*Error or String*/message, /*String*/action) {
		if( dojo.isDebug ){
		    console.warn("com.ibm.relm.common.web.error.create is depricated, use com.ibm.relm.common.web.createError instead"); //$NON-NLS-1$
		}
		return this.createError(message, action);
	},
	
	createCritical: function(/*Error or String*/message, /*String*/action) {
        var result = _ErrorFactory.create(message, action);
        result.severity = com.ibm.relm.common.web.error.Severity.CRITICAL;
        return result;
    },
	   
    createError: function(/*Error or String*/message, /*String*/action) {
        var result = _ErrorFactory.create(message, action);
        result.severity = com.ibm.relm.common.web.error.Severity.ERROR;
        return result;
    },

	createWarning: function(/*Error or String*/message, /*String*/action) {
        var result = _ErrorFactory.create(message, action);
        result.severity = com.ibm.relm.common.web.error.Severity.WARNING;
        return result;
	},
	
	createInfo: function(/*Error or String*/message, /*String*/action) {
        var result = _ErrorFactory.create(message, action);
        result.severity = com.ibm.relm.common.web.error.Severity.INFO;
        return result;
    },
	
	createXhrError: function(ioArgs, action) {
		var contentType = ioArgs.xhr.getResponseHeader("Content-Type"); //$NON-NLS-1$
		if (ioArgs.xhr.responseText && contentType && contentType.toLowerCase() === "application/rdf+xml") { //$NON-NLS-1$
			var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText);
			return new com.ibm.relm.common.web._RdfError(ioArgs, rdf.getElementsByTagName("rdf:Description")[0], action); //$NON-NLS-1$
		}
		
		var message;
		if (ioArgs.xhr.statusText) {
			message = ioArgs.xhr.statusText + " - " + ioArgs.url; //$NON-NLS-1$
		} else {
			message = dojo.string.substitute(messages.defaultXhrErrorMessage, [ioArgs.url]);
		}
		
		var details = null;
		if (ioArgs.xhr.responseText) {
			contentType = ioArgs.xhr.getResponseHeader("Content-Type"); //$NON-NLS-1$
			if ((contentType && contentType.toLowerCase() === "text/html") || //$NON-NLS-1$
					ioArgs.xhr.responseText.search(/^<html>/i) !== -1) {
				details = dojo.doc.createElement("div"); //$NON-NLS-1$
				try {
					details.innerHTML = ioArgs.xhr.responseText;
				} catch (e) {
					details = ioArgs.xhr.responseText;
				}
			} else {
				details = ioArgs.xhr.responseText;
			}
		}
		
		var date = ioArgs.xhr.getResponseHeader("Date"); //$NON-NLS-1$
		return new com.ibm.relm.common.web.Error({
			status: ioArgs.xhr.status || null,
			severity: this.Severity.ERROR,
			"message": message, //$NON-NLS-1$
			detailedMessage: details,
			"date": date ? new Date(date) : null, //$NON-NLS-1$
			action: action
		});
	}
};

dojo.declare("com.ibm.relm.common.web.Error", Error, { //$NON-NLS-1$
    
		
	error: null,
	status: null,
	subStatus: null,
	severity: null,
	message: null,
	detailedMessage: null, // string or dom node
	seeAlso: null,
	cause: null,
	date: null,
	dispatched: null,
	
	action: null, // the action is a localized string which is combined with
	              // a generic error message to provide context
	              
	_isGeneric: true, //true by default              
	
	constructor: function(params) {
		if (params) {
			dojo.mixin(this, params);
		}
		if (!this.date) {
			this.date = new Date();
		}
		if (!this.severity) {
			this.severity = com.ibm.relm.common.web.error.Severity.ERROR;
		}
		
		this.dispatched = false;
	},
	
	preamble: function() {
		return [];
	},
	
	/**
	 * <p>
	 * Dispatches the error for the error view controllers to handle.  If the error has already been dispatched
	 * then the default action is to not dispatch it again (don't view multiple errors.)  However, the <code>force</code>
	 * boolean parameter can override that.
	 * </p>
	 * 
	 * @param force <code>TRUE</code> to ignore dispatched status, <code>FALSE</code> to abide by it.
	 */
	dispatch: function(/*boolean*/ force) {
		if(!this.dispatched || force){
			dojo.publish(com.ibm.relm.common.web.error.Channel, [this._isGeneric ? this.getGenericError() : this]);
			this.dispatched = true;
		}
	},
	
	/**
	 * <p>
	 * Sets the <code>detailedMessage</code> parameter which is shown to the user under the heading "Details:", 
	 * and returns <code>this</code> object. It is meant to show the details of the error, for example: 
	 * <ul><li>"While searching document server Y, the document X could not be found"</li></ul>
	 * It can be used as a part of method chain.
	 * </p>
	 * 
	 * @param detailedMessage a <code>String</code> representing the  .
	 *  */
    details: function(/*String*/ detailedMessage){
      this.detailedMessage = detailedMessage;
      this._isGeneric = false;
      return this;
    },
    
     /**
     * <p>
     * Sets the <code>cause</code> parameter which is shown to the user under the heading "Caused By:", and returns <code>this</code>.
     * 
     * It can be used as a part of method chain.
     * </p>
     * 
     * @param cause the root <code>com.ibm.relm.common.web.Error</code> cause of the problem that has trickled down to here.
     *  */
	causedBy: function(/*com.ibm.relm.common.web.Error*/ cause){
	    this.cause = cause;
	    return this;
	},
	/**
     * <p>
     * Sets the <code>seeAlso</code> parameter, and returns <code>this</code>.
     * It can be used as part of a method chain.
     * </p>
     * 
     * @param{URI} a URI <code>String</code> to a resource that may contain information such as how to resolve this error.
     * */
    see: function(/*String*/ URI){
      this.seeAlso = URI;
      return this;  
    },
	
	
	getGenericError: function() {
		// Construct a generic error message including the action
		// and set the cause of the generic error to be the real error.
		// Some portions of the error may not be blank, we should test each one to see if it already has a value before going with defaults.
		return new com.ibm.relm.common.web.Error({
			status: this.status,
			subStatus: this.subStatus,
			severity: this.severity,
			message: this.message ? this.message : (this.action ? ((this.severity === com.ibm.relm.common.web.error.Severity.INFO || this.severity === com.ibm.relm.common.web.error.Severity.WARNING) ? dojo.string.substitute(messages.genericWarning_message_action, [this.action]) : dojo.string.substitute(messages.genericError_message_action, [this.action])) : messages.genericError_message),
			detailedMessage: this.detailedMessage ? this.detailedMessage : (this.severity === com.ibm.relm.common.web.error.Severity.INFO ? null : messages.genericError_detailedMessage),
			seeAlso: this.seeAlso,
			date: this.date,
			cause: this.cause ? this.cause : this,
			_isGeneric: true // mark as generic
		});
	}
});

//err:Report    err:errorStatus      No.     The major status code, usually the corresponding HTTP status code.
//err:Report    err:errorMessage     Yes.    Some textual response.
//err:Report    err:errorSubStatus   No.     The minor status code, usually a Foundation specified value.
//err:Report    err:errorSeverity    No.     One of the values: info, warning, error, critical.
//err:Report    err:detailedMessage  Yes.    Any additional information the application needs to convey.
//err:Report    err:seeAlso          No.     A URL to a resource that may contain information such as how to resolve this error.
//err:Report    err:errorCause       No.     Another err:Report that denotes a condition that caused this error report to be generated.

dojo.declare("com.ibm.relm.common.web._RdfError", com.ibm.relm.common.web.Error, { //$NON-NLS-1$
	
	constructor: function(ioArgs, rdfDescription, action) {
		this.action = action;
		var date = ioArgs.xhr.getResponseHeader("Date"); //$NON-NLS-1$
		if (date) {
			this.date = new Date(date);
		} else {
			this.date = new Date();
		}
		for (var i = 0; i < rdfDescription.childNodes.length; ++i) {
			var childNode = rdfDescription.childNodes[i];
			if (childNode.nodeType === 1 && childNode.childNodes.length > 0 && childNode.childNodes[0].nodeValue) {
				switch (String(childNode.nodeName)) {
					case 'err:errorStatus': //$NON-NLS-1$
						this.status = childNode.childNodes[0].nodeValue;
						break;
					case 'err:errorMessage': //$NON-NLS-1$
						this.message = childNode.childNodes[0].nodeValue;
						break;
					case 'err:errorSubStatus': //$NON-NLS-1$
						this.subStatus = childNode.childNodes[0].nodeValue;
						break;
					case 'err:errorSeverity': //$NON-NLS-1$
						this.severity = childNode.childNodes[0].nodeValue;
						break;
					case 'err:detailedMessage': //$NON-NLS-1$
						var value = childNode.childNodes[0].nodeValue;
						if (value.search(/^<html>/i) !== -1) {
							this.detailedMessage = dojo.doc.createElement("div"); //$NON-NLS-1$
							try {
								this.detailedMessage.innerHTML = value;
							} catch (e) {
								this.detailedMessage = value;
							}
						} else {
							this.detailedMessage = value;
						}
						break;
					case 'err:seeAlso': //$NON-NLS-1$
						this.seeAlso = childNode.childNodes[0].nodeValue;
						break;
					case 'err:errorCause': //$NON-NLS-1$
						this.cause = new com.ibm.relm.common.web._RdfError(ioArgs, childNode.getElementsByTagName("rdf:Description")[0]); //$NON-NLS-1$
						break;
				}
			}
		}
	},
	
	preamble: function() {
		return [];
	}
	
});
}());