/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.explorer.Explorer"); //$NON-NLS-1$

dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.Dialog"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("dijit.layout._LayoutWidget"); //$NON-NLS-1$
dojo.require("dijit.layout.BorderContainer"); //$NON-NLS-1$
dojo.require("dijit.layout.StackContainer"); //$NON-NLS-1$
dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
dojo.require("dijit.TitlePane"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.QueryResult"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.QueryEditor"); //$NON-NLS-1$

(function(){

var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
	
var _defaultSparql = nodeUtils.INITIAL_SPARQL_CONTENT;

dojo.declare("com.ibm.relm.explorer.web.explorer.Explorer",  //$NON-NLS-1$
	[dijit._Widget,dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/templates/Explorer.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	queryPage: null,
	
	_editorInput: null,

	
    layout: function(){
    },
    
    postCreate: function(){
		this.inherited(arguments);
        this.subscribe("OPEN_QUERY_EDITOR", "onEditQuery"); //$NON-NLS-1$ //$NON-NLS-2$
		this.subscribe("CREATE_QUERY", "createQuery"); //$NON-NLS-1$ //$NON-NLS-2$
    },
    
    resize: function(size){
    	if(this.queryEditor && this.queryEditor.domNode){
    		this.queryEditor.resize(size);
    	}
    	this._size = size; // keep size for QueryEditor creation
    },
    
	setMainContent: function(node){
        this.handleQueryEditor(node);
        var me = this.containerNode;
        for(var c; c = me.lastChild;){ // intentional assignment
            me.removeChild(c);
        }
        me.appendChild(node);
	},

    handleQueryEditor: function(node) {
        var widgetOfNode = dijit.getEnclosingWidget(node);
        var isQueryEditor = widgetOfNode.declaredClass == "com.ibm.relm.explorer.web.explorer.page.QueryEditor"; //$NON-NLS-1$
        dojo.toggleClass(dojo.body(), 'queryEditor-noMargin', isQueryEditor); //$NON-NLS-1$
    },
    
    createQuery: function(){
    	this.onEditQuery({
    		sparql: _defaultSparql,
    		mode: "create" //$NON-NLS-1$
    	});
    },
    
    open: function(message){
    	if(message.queryUri) { //$NON-NLS-1$
        	this._editorInput = message._editorInput;
        	if(this._editorInput) this._editorInput.editor = this;
    		this.queryPage.pageHelper.openEditor(this);
    	}
		if(message.edit){
			this.onEditQuery(message);
		}else{
			this.onExecuteQuery(message);				
		}    	
    },
    	
    onEditQuery: function(message){
        var hashObj = dojo.queryToObject(window.location.hash.substring(1));
        if (hashObj.edit == "true" || hashObj.mode == "create") { //$NON-NLS-1$ //$NON-NLS-2$
            message.edit = true;
        }
        if(!this.queryEditor || !this.queryEditor.domNode){ 
            var mode = message.mode || "modify"; //$NON-NLS-1$
            var editor = new com.ibm.relm.explorer.web.explorer.page.QueryEditor({mode: mode});
            this.queryEditor = editor;
            editor.queryPage = this.queryPage;
            this.setMainContent(this.queryEditor.domNode);
            editor.startup();
            editor.resize(this._size);
        }
        if(message){
        	this.queryEditor.setQueryInfo(message);
        }
        this.setMainContent(this.queryEditor.domNode);

		// Disable spell check to avoid red squiggly lines in the editor in Firefox
		if(dojo.isFF !== undefined)
			dojo.body().setAttribute("spellCheck","false"); //$NON-NLS-1$ //$NON-NLS-2$
    },
    
    onExecuteQuery: function(message){
    	if(!message.queryUri && (message.sparql===undefined || message.sparql===null) && !message.resourceUri && !message.query && !message.artifactElementId){
			// open editor
			this.createQuery();
			return;
    	}
        else if(message.sparql!==undefined && message.sparql!==null && dojo.trim(message.sparql) === ''){  //$NON-NLS-1$
                //empty query will not run
            return;
        }

    	this.queryResult.setQueryInfo(message);
    	this.setMainContent(this.queryResult.domNode);
    }    ,
    
	getEditorInput: function() {
		return this._editorInput;
	},
	
	setEditorInput: function(editorInput) {
		this._editorInput = editorInput;
	},
	
	isDirty: function() {
		return false;
	}

});
	
})();