/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

(function()  {
dojo.provide("com.ibm.relm.explorer.web.explorer.matrix.SelectRelationStep"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup");//$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.matrix.SelectRelationshipDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$


dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var WizardStep = jazz.ui.wizard.WizardStep;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;

var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

dojo.declare("com.ibm.relm.explorer.web.explorer.matrix.SelectRelationStep",  //$NON-NLS-1$
		[WizardStep, dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/matrix/templates/SelectRelationStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	
	wizard: null,
	
	postMixInProperties: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.relations = [];
		this.inherited(arguments);
		this._popups = [];
	},

	postCreate: function(){
		this.inherited(arguments);
		this.header = new WizardStepHeader ({stepTitle: ""}); //$NON-NLS-1$
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.header.helpArea.innerHTML = this.messages.selectRelationshipsDescription;

		var columns = [
           {id: "relationName", name: this.messages.titleRelation, editable: false}, //$NON-NLS-1$
           {id: "display", name: this.messages.titleDisplay, editable: true, callback: this.displayChanged},//$NON-NLS-1$
           {id: "actions", name: this.messages.titleAction, isAction: true}//$NON-NLS-1$
		];
		this.relationsTable.setupColumns(tableUtils.createTableColumns(columns, this));
		var self = this;
		dojo.create("a", {  //$NON-NLS-1$
			href: "#",  //$NON-NLS-1$
			innerHTML: this.messages.addRelationship,
			title: this.messages.addRelationship,
			onclick: function(e){
				dojo.stopEvent(e);
				self.openAddRelationDialog();
			}
		}, this.toolbar);
	},
	
	destroy: function(){
		this.inherited(arguments);		
		dojo.forEach(this._popups, function(popup){popup.destroy();});
	},
	
	displayChanged: function(data, id, value, widget){
		data._rel.display = value;
	},

	getRelations: function(){
		return this.relations;
	},
	
	canFlipToNextPage: function() {
		return true;
	},
	
	canFinish: function() {
		return this.relations.length > 0;
	},
	
	canCancel: function() {
		return true;
	},
	
	openAddRelationDialog: function(){
		var contentWidget = new com.ibm.relm.explorer.web.explorer.matrix.SelectRelationshipDialog({
			permissions: this.wizard.permissions
		});
		var self = this;
		new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
			headerContent: {
				mainMessage: this.messages.selectRelationStep,
				mainHtmlMessage: this.messages.descriptionForToolbar || ""//$NON-NLS-1$
			},
			dialogContent: contentWidget,
			primaryTitle: this.messages.selectRelationStep,
			onOpen: function(){
				dijit.focus(contentWidget.relationSelect);
			},
			onOK: function(values){
				var v = contentWidget.getValue();
				if(v){
					var rel = {
						name: v.text || v.value,
						relation: v.value,
						display: "X"//$NON-NLS-1$
					};
					self.relations.push(rel);
					self.refreshRelations();
				}
			}
		});
	},
	
	refreshRelations: function() {

		var table = this.relationsTable;
		table.clearItems();

		var self = this;
		dojo.forEach(this.relations, function(relation){
			var row = table.addItem({
				_rel: relation,
				relationName : relation.name || relation.relation, 
				display: relation.display, 
				actions: [{
					iconClass: "delete-command",//$NON-NLS-1$
					title: self.messages.removeRelationship,
					onClick: function(e){
						var tobeRemoved = this.data._rel;
						if(tobeRemoved){
							for(var i=0; i<self.relations.length; i++){
								if(self.relations[i] == tobeRemoved){
									self.relations.splice(i, 1);
									break;
								}
							}
							self.refreshRelations();
						}
					}
				}]
			});
			var td = row.getCellByColumnId("relationName");//$NON-NLS-1$
			if(td) {
				self._popups.push(new jazz.ui.HoverPopup({around: td, html: jazz.util.html.escape(relation.relation)}));
			}
		});
		
		this.triggerButtonStateUpdate();			
	},
	
	_createAddButton: function(label, callback, context, buttonLabel){
		callback = dojo.hitch(context || this, callback);
		return dojo.create("a", {  //$NON-NLS-1$
			href: "#",  //$NON-NLS-1$
			innerHTML: buttonLabel || messages.Add,
			className: CLASS_ADD_COMMAND,
			title: label,
			style: {
				verticalAlign: "middle",  //$NON-NLS-1$
				marginLeft: "8px"  //$NON-NLS-1$
			},
			onclick: function(e){
				dojo.stopEvent(e);
				callback();
			}
		});
	}
});

}) ();