/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

(function()  {
dojo.provide("com.ibm.relm.explorer.web.explorer.matrix.SelectRelationshipDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Textarea"); //$NON-NLS-1$

dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.matrix.EditRelationshipDialog");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils");  //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.relm.explorer.web.explorer.matrix.SelectRelationshipDialog",  //$NON-NLS-1$
		[dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true, 
	templateString: 
		"<div class='com-ibm-relm-explorer-web-matrix-dialog'>" +//$NON-NLS-1$
		"<div dojotype='jazz.ui.Toolbar' class='toolbar' dojoAttachPoint='toolbar'>" +//$NON-NLS-1$
		"	<button dojoType='jazz.ui.toolbar.Button' dojoAttachPoint='addRelation' imgClass='add-command' title='${messages.addRelation}'></button>"+//$NON-NLS-1$ 
		"	<button dojoType='jazz.ui.toolbar.Button' dojoAttachPoint='editRelation' enabled='false' imgClass='edit-command' title='${messages.editRelation}'></button>"+//$NON-NLS-1$ 
		"	<button dojoType='jazz.ui.toolbar.Button' dojoAttachPoint='removeRelation' enabled='false' imgClass='delete-command' title='${messages.removeRelation}'></button>"+//$NON-NLS-1$ 
		"</div>" + //$NON-NLS-1$
		"<select dojoAttachPoint='relationSelect' style='width: 100%;' size='10'></select>" + //$NON-NLS-1$
		"</div>",//$NON-NLS-1$
		
	postMixInProperties: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this._fillPredefinedRelationshipTypes();
		if(this.permissions && this.permissions.deletePermission){
			dojo.style(this.toolbar, "display", ""); //$NON-NLS-1$//$NON-NLS-2$
			this.connect(this.addRelation, "onClick", this._onAddRelation);//$NON-NLS-1$
			this.connect(this.editRelation, "onClick", this._onEditRelation);//$NON-NLS-1$
			this.connect(this.removeRelation, "onClick", this._onRemoveRelation);//$NON-NLS-1$
		}else{
			dojo.style(this.toolbar.domNode, "display", "none"); //$NON-NLS-1$//$NON-NLS-2$		
		}
	},
	
	getValue: function(){
		return this.relationSelect.options[this.relationSelect.selectedIndex];
	},
	
	_onAddRelation: function(ev){
		this._openAddPropertyPathDialog(true);
	},
	
	_onEditRelation: function(ev){
		this._openAddPropertyPathDialog(false);		
	},
	
	_onRemoveRelation: function(ev){
		var selectedIndex = this.relationSelect.selectedIndex;
		if(selectedIndex >= 0){
			this.relationSelect.options[selectedIndex] = null;
			this.saveRelationships();
		}
	},

	_fillPredefinedRelationshipTypes: function(){
		// fill relation selection box
		this.relationSelect.options.length = 0;
		var self = this;
		jazz.client.xhrGet({
			url: com.ibm.relm.explorer.web.ui.matrixServiceUri,
			preventCache: true,
			handleAs: "json", //$NON-NLS-1$
			load: function(result, ioArgs){
				self.eTag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
				dojo.forEach(result, function(rel, i){
					self.relationSelect.options[i] = new Option(rel.name, rel.propertyPath);
				}, self);
			}
		});

		this.connect(this.relationSelect, "onchange", function(){//$NON-NLS-1$
			var enabled = this.relationSelect.selectedIndex >= 0;
			this.editRelation.set("enabled", enabled);//$NON-NLS-1$
			this.removeRelation.set("enabled", enabled);//$NON-NLS-1$
		});				
	},
	
	_openAddPropertyPathDialog: function(createNew){
		var args = {};
		if(!createNew){
			var opt = this.getValue();
			args.relationName = opt.label;
			args.propertyPath = opt.value;
		}
		var contentWidget = new com.ibm.relm.explorer.web.explorer.matrix.EditRelationshipDialog(args);
		var self = this;
		new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
			dialogContent: contentWidget,
			primaryTitle: this.messages.addRelationshipTitle,
			validate: function(values, callback){
				if(dojo.trim(values.relationName) && values.propertyPath){
					callback();
				}else{
					alert(self.messages.needFillNameAndValue);
				}
			},
			onOpen: function(){
			},
			onOK: function(values){
				if(createNew){
					var length = self.relationSelect.options.length;
					self.relationSelect.options[length] = new Option(values.relationName, values.propertyPath);
				}else{
					var selected = self.relationSelect.options[self.relationSelect.selectedIndex];
					selected.text = values.relationName;
					selected.value = values.propertyPath;
				}
				self.saveRelationships();
			}
		});
	},
	
	saveRelationships: function(){
		var json = [];
		var length = this.relationSelect.options.length;
		for(var i=0; i<length; i++){
			var opt = this.relationSelect.options[i];
			json.push({name: opt.text, propertyPath: opt.value});
		}
		var self = this;
		jazz.client.xhrPut({
			url: com.ibm.relm.explorer.web.ui.matrixServiceUri,
			headers: {
				ETag: this.eTag, 
				"Content-Type": "text/json; charset=utf-8"  //$NON-NLS-1$ //$NON-NLS-2$
			},
			putData: dojo.toJson(json),
			load: function(result, ioArgs){
				self.eTag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
			},
			hasBody: true
		});
	}
});


}) ();