/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

(function()  {
dojo.provide("com.ibm.relm.explorer.web.explorer.matrix.SetAxisesStep"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("dojox.layout.TableContainer"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var WizardStep = jazz.ui.wizard.WizardStep;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;

dojo.declare("com.ibm.relm.explorer.web.explorer.matrix.SetAxisesStep", //$NON-NLS-1$
		[WizardStep, dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/matrix/templates/SetAxisesStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	
	wizard: null,
	queries: null, 
	
	postMixInProperties: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.internal", "CreateMatrixWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		//
		this.header = new WizardStepHeader ({stepTitle: ""}); //$NON-NLS-1$
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.header.helpArea.innerHTML = this.messages.setRowAndColumnDescription;
		// set loading image
		this._rowResourceLoading.src = dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/matrix/templates/loading.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		this._columnResourceLoading.src = dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/matrix/templates/loading.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		// fill query selection box 
		this._fillSelectForQuery(this._rowQuery, 0);
		this._onChangeRowQuery();
		this._fillSelectForQuery(this._columnQuery, 1);
		this._onChangeColumnQuery();
		// event handlers
		this.connect(this._rowQuery, "onchange", this._onChangeRowQuery); //$NON-NLS-1$
		this.connect(this._columnQuery, "onchange", this._onChangeColumnQuery); //$NON-NLS-1$
	},
	
	_fillSelectForQuery: function(select, index){
		select.options.length = 0;
		dojo.forEach(this.queries, function(query, i){
			select.options[i] = new Option(query.name, query.id);
			if(i == index){
				select.options[i].selected = true;
			}
		});		
	},
	
	_onChangeRowQuery: function(){
		this._updateResourceSelect(this.getRowQueryURI(), this._rowResource, this._rowResourceLoading);
	},

	_onChangeColumnQuery: function(){
		this._updateResourceSelect(this.getColumnQueryURI(), this._columnResource, this._columnResourceLoading);
	},
	
	_updateResourceSelect: function(queryURI, select, icon){
		dojo.style(icon, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
		var self = this;
		self._messageBox.hideMessage();
		jazz.client.xhrGet({
			url: queryURI, // + "?format=false", 
			headers: {"Accept":"application/sparql-results+json"}, //$NON-NLS-1$ //$NON-NLS-2$
			handleAs: "json", //$NON-NLS-1$
			handle: function(){
				dojo.style(icon, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
				self.triggerButtonStateUpdate();
			},
			load: function(data){
				var options = [];
				var bindings = data.results.bindings;
				if(bindings.length > 0){
					select.bindings = bindings; // keep it
					for(var varName in bindings[0]){
						var b = bindings[0][varName];
						if(b.type == "uri"){ //$NON-NLS-1$
							options.push(varName);
						}
					}
				}
				select.options.length = 0;
				dojo.forEach(options, function(o, i){
					select.options[i] = new Option(o, o);
				});
				dojo.style(select, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
			},
			error: function(ioargs){
				self._messageBox.showXhrError(ioargs.xhr);
			}
		});
	},
	
	getRowVar: function(){
		return this._rowResource.value;
	},
	
	getRowQueryURI: function(){
		return this._rowQuery.options[this._rowQuery.selectedIndex].value;
	},
	
	getRowValues: function(){
		var uris = [];
		var titles = [];
		var rowVar = this._rowResource.value;
		if(this._rowResource.bindings){
			dojo.forEach(this._rowResource.bindings, function(b){
				var uri = b[rowVar].value;
				if(dojo.indexOf(uris, uri) < 0){
					uris.push(uri);
					titles.push(b[rowVar].label);
				}
			});
		}
		var ret = {uris: uris};
		if(!dojo.some(titles, function(t){ return t===undefined; })){
			ret.titles = titles;
		}
		return ret;
	},
	
	getColumnVar: function(){
		return this._columnResource.value;
	},
	
	getColumnQueryURI:function(){
		return this._columnQuery.options[this._columnQuery.selectedIndex].value;
	},
	
	getColumnValues: function(){
		var uris = [];
		var titles = [];
		var colVar = this._columnResource.value;
		if(this._columnResource.bindings){
			dojo.forEach(this._columnResource.bindings, function(b){
				var uri = b[colVar].value;
				if(dojo.indexOf(uris, uri) < 0){
					uris.push(uri);
					titles.push(b[colVar].label);
				}
			});
		}
		var ret = {uris: uris};
		if(!dojo.some(titles, function(t){ return t===undefined; })){
			ret.titles = titles;
		}
		return ret;
	},

	canFlipToNextPage: function() {
		return this._rowResource.options.length > 0 && this._columnResource.options.length > 0;
	},
	
	canFinish: function() {
		return false;
	},
	
	canCancel: function() {
		return true;
	}
	
	
});

}) ();