/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.explorer.page.ExplorerClient"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util");//$NON-NLS-1$
dojo.require("jazz.app.context"); //$NON-NLS-1$

(function() { // module container
	var ServiceRequest = com.ibm.relm.explorer.web.explorer.page.ServiceRequest;
	
	dojo.declare("com.ibm.relm.explorer.web.explorer.page.ServiceClient", null, {//$NON-NLS-1$
			constructor: function(args) {
				dojo.safeMixin(this, args);
			},
	
			/**
			 * Fetch all visible root folder descriptors. The client does not need
			 * to supply arguments. This method will supply the
			 * currently-authenticated user and current projectArea.
			 */
			getSystemRootFolderDescriptors : function(responseHandler) {
				var args = {};
				var currentProjectArea = jazz.app.context.get();
				args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				this._invokeFolderService(responseHandler,
						"getRootFolderDescriptors", args); //$NON-NLS-1$
			},
			/**
			 * Fetch folders. This method will supply the projectAreaUUID.
			 */
			getFolderDescriptors : function(responseHandler, args) {
				if (args.projectAreaUUID == undefined) {
					var currentProjectArea = jazz.app.context.get();
					args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				}
				this._invokeFolderService(responseHandler, "getFolderDescriptors", //$NON-NLS-1$
						args);
			},
			
			/**
			 * Fetch folders under null project area root folder
			 */
			getFolderDescriptorsUnderNonProjectAreaRootFolder : function(responseHandler, args) {
				if (args == undefined) {
					args = {};
				}
				args.projectAreaUUID = ""; //$NON-NLS-1$
				this._invokeFolderService(responseHandler, "getFolderDescriptors", //$NON-NLS-1$
						args);
			},
			/**
			 * Fetch a single folder.
			 */
			getFolderDescriptor : function(responseHandler, folderUUID, args) {
				this._invokeFolderService(responseHandler, "getFolderDescriptor", //$NON-NLS-1$
						args, folderUUID);
			},
			/**
			 * Helper function. ProcessClient doesn't provide this.
			 */
			getProcessAreaByUUID : function(responseHandler, args) {
				return;
				if (args.projectAreaUUID == undefined) {
					var currentProjectArea = jazz.app.context.get();
					args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				}
				this._invokeFolderService(responseHandler, "getProcessAreaByUUID", //$NON-NLS-1$
						args);
			},
			/**
			 * Delete the given folder.
			 */
			postDeleteFolderDescriptor : function(responseHandler, folderUuid, args, headers) {
				this._invokeFolderService(responseHandler,
						"postDeleteFolderDescriptor", args, folderUuid, headers); //$NON-NLS-1$
			},
			/**
			 * Delete the given folders.
			 */
			postDeleteFolderDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postDeleteFolderDescriptors", args); //$NON-NLS-1$
			},
			/**
			 * Rename the given folder.
			 */
			postRenameFolderDescriptor : function(responseHandler, folderUuid, args) {
				this._invokeFolderService(responseHandler,
						"postRenameFolderDescriptor", args, folderUuid); //$NON-NLS-1$
			},

			/**
			 * Save a folder with the given arguments. This could be creating a new
			 * one or saving an existing one.
			 */
			postSaveFolderDescriptor : function(responseHandler, folderUuid, args, headers) {
				if (args.projectAreaUUID == undefined) {
					var currentProjectArea = jazz.app.context.get();
					args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				}
				this._invokeFolderService(responseHandler,
						"postSaveFolderDescriptor", args, folderUuid, headers); //$NON-NLS-1$
			},
			postMoveFolderDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postMoveFolderDescriptors", args); //$NON-NLS-1$
			},
			postCopyFolderDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postCopyFolderDescriptors", args); //$NON-NLS-1$
			},
			/**
			 * Invokes the requested method on the Reports REST service.
			 */
			_invokeFolderService : function(responseHandler, method, args , resourceId, headers) {
				var self = this;
//				args is optional, so test whether or not it was sent
				var serviceRequest;
				if (!args) {
					if(!resourceId) {
						serviceRequest = new ServiceRequest(self.SERVICE_URI, method);
					}
					else {
						serviceRequest = new ServiceRequest(self.SERVICE_URI+resourceId, method);
					}
				} else {
					if(!resourceId) {
						serviceRequest = new ServiceRequest(self.SERVICE_URI, method, args);
					} else {
						serviceRequest = new ServiceRequest(self.SERVICE_URI+resourceId, method, args);
					}
				}
				var isPOST = serviceRequest.getMethodType() === "POST"; //$NON-NLS-1$
				var xhrArgs = {
						url: self._assembleURL(serviceRequest),
						preventCache: true,
						headers: dojo.mixin(headers, {"Accept":"application/json"}), //$NON-NLS-1$ //$NON-NLS-2$
						handleAs: "json", //$NON-NLS-1$
						handle: function(data, ioArgs) {
							 self.handleCallback(data, ioArgs, responseHandler);
						}
				};
//				fill in customized arguments, such as skipInterceptor
				if(args) {
					xhrArgs.content = args;
				}

				if(isPOST) {
					// post
					jazz.client.xhrPost(xhrArgs);
				} else {
					// get
					jazz.client.xhrGet(xhrArgs);
				}
			},
			
			/*
			 * Assembles a URL, appending a query string to GET queries. Handles both
			 * non-secure and secure contexts.
			 */
			_assembleURL: function(serviceRequest) {
				var method = serviceRequest.getMethod();
				var methodType = serviceRequest.getMethodType();
				var urlResource =
					method.substring(methodType.length, methodType.length + 1).toLowerCase() +
					method.substring(methodType.length + 1);
				//				assemble basic url
				var url = ""; //$NON-NLS-1$
				if(!serviceRequest.getResourceId()) {
					url = /*self.getServicePrefix() + */ serviceRequest.getServiceInterface() +"?method=" + urlResource; //$NON-NLS-1$
				} else {
					url = /*self.getServicePrefix() + */ serviceRequest.getServiceInterface() + serviceRequest.getResourceId() +"?method=" + urlResource; //$NON-NLS-1$
				}
				//				if this is a GET request, append the query string
				if (methodType === "GET") { //$NON-NLS-1$
					var tmpQueryString = serviceRequest.getQueryString();
					var queryString = tmpQueryString ? ("&" + tmpQueryString) : ""; //$NON-NLS-1$ //$NON-NLS-2$
					url += queryString;
				}
				return url;
			},
			
			handleCallback: function(response, ioArgs, serviceResponseHandler) {
				var self = this;
				if(!(response instanceof Error)) {
					net.jazz.ajax.core.internal.util.callContributedCode(function() {
						serviceResponseHandler.doCallback(response, ioArgs);
					});
				} else {
//					error callback
					if (dojo.isObject(response) && response.dojoType && response.dojoType === "cancel") { //$NON-NLS-1$
//						cancellation case
						return; // early return - see Defect 71631
					}
//					remote error case
					var errorObj = {
							ioArgs: ioArgs,
							status: ioArgs.xhr.status
					};
					if(errorObj.status === 403) {
						errorObj = handlePermissionDeniedError(errorObj);
					} else if (response.dojoType === "popupBlocked") { //$NON-NLS-1$
						dojo.mixin(errorObj, {
							message: response.message,
							exception: []
						});
					} else {
//						in most other cases we expect that the Jazz server will have sent
//						a response body containing a JSON representation of the problem
//						that occurred
//						manually demarshall JSON since Dojo doesn't on error paths
						var responseObj = self._parseJSON(response.responseText);
						try {
							dojo.mixin(errorObj, {
								errorClass: responseObj.errorClass,
								message: responseObj.errorMessage || responseObj.explanation,
								exception: responseObj.errorTrace || responseObj.stackTrace,
								errorData: responseObj.errorData,
								nestedError: responseObj.nestedError
							});
						}
						catch (/* Error */ e) {
//							for non-JSON responses, include it as a message
							errorObj.message = response.responseText;
						}
					}
//					send error callback
					serviceResponseHandler.doErrorCallback(errorObj);
				}
			},
			/*
			 * Parse JSON string into JavaScript Object
			 * Use the browser's native JSON.parse (if available) to do the parsing instead of dojo.fromJSON (which uses eval).
			 * NOTE: This will not be needed for Dojo 1.7+ since it will be implemented in dojo.fromJSON.
			 * For more information, see http://bugs.dojotoolkit.org/ticket/8111
			 */
			_parseJSON: function(response) {
				if(typeof JSON != "undefined" && JSON.parse) { //$NON-NLS-1$
					return JSON.parse(response);
				} else {
					return dojo.fromJson(response);
				}
			}

	});

	// Query Client
	com.ibm.relm.explorer.web.explorer.page.ExplorerClient = new com.ibm.relm.explorer.web.explorer.page.ServiceClient({
			SERVICE_URI : com.ibm.relm.explorer.web.ui.queryServiceUri,
			/**
			 * Fetch descriptors. This method will supply the
			 * projectAreaUUID.
			 */
			getQueryDescriptors : function(responseHandler, args) {
				return this.getResourceDescriptors(responseHandler, args);
			},
			
			getResourceDescriptors : function(responseHandler, args) {
				if (args == undefined) {
					args = {};
				}
				if (args.projectAreaUUID == undefined) {
					var currentProjectArea = jazz.app.context.get();
					args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				}
				this._invokeFolderService(responseHandler, "getQueryDescriptors",  //$NON-NLS-1$
						args);
			},
			/**
			 * Fetch descriptors under null project area root folder
			 * projectAreaUUID.
			 */
			getResourceDescriptorsUnderNonProjectAreaRootFolder : function(responseHandler, args) {
				if (args == undefined) {
					args = {};
				}
				args.projectAreaUUID = ""; //$NON-NLS-1$
				this._invokeFolderService(responseHandler, "getQueryDescriptorsUnderNonProjectAreaRootFolder",  //$NON-NLS-1$
						args);
			},
			getQueryInfo: function(responseHandler, args) {
				if (args == undefined) {
					args = {};
				}
				if (args.projectAreaUUID == undefined) {
					var currentProjectArea = jazz.app.context.get();
					args.projectAreaUUID = currentProjectArea.itemId || ""; //$NON-NLS-1$
				}
				this._invokeFolderService(responseHandler, "getQueryInfo",  //$NON-NLS-1$
						args);
			},
			/**
			 * Delete the given query descriptor.
			 */
			deleteResourceDescriptor : function(responseHandler, queryId, args) {
				this._invokeFolderService(responseHandler,
						"postDeleteQueryDescriptor", queryId, args); //$NON-NLS-1$
			},
			/**
			 * Delete the given query descriptors.
			 */
			deleteResourceDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postDeleteQueryDescriptors", args); //$NON-NLS-1$
			},

			/**
			 * Move/Copy the given query descriptors.
			 */
			moveResourceDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postMoveQueryDescriptors", args); //$NON-NLS-1$
			},
			copyResourceDescriptors : function(responseHandler, args) {
				this._invokeFolderService(responseHandler,
						"postCopyQueryDescriptors", args); //$NON-NLS-1$
			}
	});
	
})();
