/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.explorer.web.explorer.page.ScopeMenuButton'); //$NON-NLS-1$

dojo.require('jazz.ui.toolbar.MenuButton'); //$NON-NLS-1$
dojo.require('jazz.ui.MegaMenu'); //$NON-NLS-1$
dojo.require('jazz.ui.menu.MenuItem'); //$NON-NLS-1$

(function(){

	/**
	 * A drop down menu button that looks just like the one in the banner, but
	 * not tied to the search box widget. Handles tracking the current scope.
	 * Callers must query the 'currentScope' when they do a search.
	 *
	 * Implementation note: this is an extension of MenuButton, but should probably have
	 * just delegated to a MenuButton.
	 */
	dojo.declare(
		'com.ibm.relm.explorer.web.explorer.page.ScopeMenuButton', //$NON-NLS-1$
		[ jazz.ui.toolbar.MenuButton ],
		{
			imgClass: 'relm-SearchResults-artifactsScope', //$NON-NLS-1$
			title: 'Search All Indexed Artifacts', //$NON-NLS-1$

			/**
			 * Array of scope objects. Each object has:
			 *
			 * id: a ID to find the scope by. Used to match the scope in the banner to a scope here.
			 * imgClass: CSS class for the image
			 * label: text for the menu
			 * title: tooltip for selected scope
			 * groupName: the "group" to show the scope under (e.g., "Products" for all products searches).
			 * 
			 * @type {Array}
			 */
			scopes: [],

			/**
			 * Set this to change the current scope. Should be a full scope
			 * object as passed to 'scopes'.
			 * Can use .watch to see changes.
			 */
			currentScope: null,

			_setCurrentScopeAttr: function(scope) {
				this._set('currentScope', scope); //$NON-NLS-1$

				this.set('title', scope.title); //$NON-NLS-1$
				this.set('imgClass', scope.imgClass); //$NON-NLS-1$
			},

			/**
			 * Sets the scope to a currently available scope
			 * with the same ID as given.
			 * @param  {string} scopeId the ID of the scope to select.
			 * @return {boolean}         true if a new scope was selected.
			 *     false if the scope was already selected, no change.
			 *     undefined if the scope ID was invalid.
			 */
			selectScopeById: function(scopeId) {
				for (var i = 0; i < this.scopes.length; i++) {
					var scope = this.scopes[i];
					if (scope.id === scopeId) {
						if (this.currentScope === scope) return false;

						this.set('currentScope', scope); //$NON-NLS-1$
						return true;
					}
				}

				return undefined;
			},

			postCreate: function() {
				this.inherited(arguments);

				this.set('menuProvider', dojo.hitch(this, this.newScopeMenu)); //$NON-NLS-1$
				this.set('destroyOnClose', true); //$NON-NLS-1$
			},

			newScopeMenu: function() {
				// Use mega menu since it support adding a bullet to a selected item.
				var menu = new jazz.ui.MegaMenu();

				dojo.addClass(menu.domNode, 'selectable'); //$NON-NLS-1$

				var currentGroupName = null;

				for (var i = 0; i < this.scopes.length; i++) {
					var scope = this.scopes[i];

					if (currentGroupName !== scope.groupName) {
						currentGroupName = scope.groupName;
						if (scope.groupName) menu.addTitle(scope.groupName);
					}

					var self = this;
					var item = new jazz.ui.menu.MenuItem({
						label: scope.label,
						iconClass: scope.imgClass,
						onClick: dojo.hitch(this, function(scope) {
							self.set('currentScope', scope); //$NON-NLS-1$
						}, scope)
					});

					if (scope == this.currentScope) {
						menu.setSelected(item);
					}

					menu.add(item);
				}

				return menu;
			},

			uninitialize: function() {
			
				this.inherited(arguments);
			}
		}
	);

})();