/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$
dojo.declare("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler", null, { //$NON-NLS-1$
//	-------------------------------------------------------------------------
//	Constructor
//	-------------------------------------------------------------------------
	/**
	 * An container for the object and functions required to handle an
	 * asyncronous service response.
	 * <p>
	 * Example usage:
	 * </p>
	 * <pre>
	 * var obj= {
	 * mySuccessFunction: function(returnedStuff) {
	 * // do something
	 * },
	 * myErrorFunction: function(errorType, errorMessage) {
	 * // do something with the error type and message
	 * }
	 * };
	 * var myServiceResponseHandler=
	 * new com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler(
	 * obj,
	 * "mySuccessFunction",
	 * "myErrorFunction");
	 * </pre>
	 *
	 * @param contextObject a valid object containing the functions named by
	 * <code>callbackName</code> and <code>errorCallbackName</code>.
	 * @param callbackName the name of a function existing in the context of
	 * <code>contextObject</code>, represented as a string. This function will
	 * be invoked upon an successful service request.
	 * @param errorCallbackName the name of a function existing in the context
	 * of <code>contextObject</code>, represented as a string. This function
	 * will be invoked upon an unsuccessful service request.
	 *
	 * @throws Exception if parameters count and types do not conform to the
	 * specification above.
	 *
	 * @since 0.5
	 */
	constructor: function(contextObject, callback, errorCallback) {
		if (arguments.length != 3) {
			throw new Error("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler(): invalid number of parameters: " + arguments.length);  //$NON-NLS-1$
		}
		if (!dojo.isObject(contextObject) ||
				!dojo.isFunction(callback) && !dojo.isFunction(contextObject[callback]) ||
				!dojo.isFunction(errorCallback) && !dojo.isFunction(contextObject[errorCallback])) {
			console.error(arguments);
			throw new Error("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler(): invalid parameter types. Expected (object contextObject, string successFunctionName, string errorFunctionName)."); //$NON-NLS-1$
		}
		this._contextObject= contextObject;
		this._callback= dojo.isFunction(callback) ? callback : this._contextObject[callback];
		this._errorCallback= dojo.isFunction(errorCallback) ? errorCallback : this._contextObject[errorCallback];
	},
	/**
	 * Performs a callback to this <code>ServiceReponseHandler</code>'s success
	 * method. This method is generally intended to be called by the web client
	 * framework, but it is sometimes useful to intercept, inspect, and perhaps
	 * filter responses and then call back to the original caller.
	 *
	 * @since 0.5
	 */
	doCallback: function(returnVal, ioArgs) {
		var obj= this._contextObject;
		this._callback.call(obj, returnVal, ioArgs);
	},
	/**
	 * Performs a callback to this <code>ServiceReponseHandler</code>'s error
	 * method. This method is generally intended to be called by the web client
	 * framework, but it is sometimes useful to intercept, inspect, and perhaps
	 * filter responses and then call back to the original caller.
	 *
	 * @since 0.5
	 */
	doErrorCallback: function(errorObj, ioArgs) {
		var obj= this._contextObject;
//		obj[this._errorCallbackName].call(obj, errorObj, ioArgs);
		this._errorCallback.call(obj, errorObj, ioArgs);
	}
});
