/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.explorer.web.explorer.page.SparqlQueryDialog'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin"); //$NON-NLS-1$

dojo.requireLocalization('com.ibm.relm.explorer.web.explorer.page', 'SparqlQueryDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
	var orionEditor = null, globalSetText = null;
	
	/**
	 * Dialog for displaying the SPARQL query string.
	 */
	dojo.declare(
		'com.ibm.relm.explorer.web.ui.internal.view.SparqlQueryDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			widgetsInTemplate: true,
			templatePath: dojo.moduleUrl('com.ibm.relm.explorer.web', 'explorer/page/templates/SparqlQueryDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			messages: dojo.i18n.getLocalization('com.ibm.relm.explorer.web.explorer.page', 'SparqlQueryDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * @param args.onSave Optional callback that receives a new store with the
			 * 		latest enumeration changes upon a successful save. If the save was
			 * 		a no-op then the store will be null.
			 */
			constructor: function(args, node) {
				orionEditor = null;
				new com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin({initializer: dojo.hitch(this, this.initOrionEditor)});
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this.textbox.set = function(property,value) {
					if(orionEditor === null) //$NON-NLS-1$
						globalSetText = value;
					else {
						setTimeout(function() { // Defer setting of text to allow DOM to load completely
							orionEditor.setText(value);
						},100);
					}
				};
				
				this.textbox.set("value",this.query || ""); //$NON-NLS-1$ //$NON-NLS-2$
				
				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					iconPath: dojo.moduleUrl('com.ibm.relm.explorer.web', 'explorer/images/show_query.gif'), //$NON-NLS-1$ //$NON-NLS-2$
					width: '', // see textarea in template for sizing //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					onClose: function() {
						if(orionEditor !== null)
							orionEditor.destroy();
					}
				});
				this._dialog.startup();
			},
			
			initOrionEditor: function() {
				var self = this;
				
				// Instantiate an instance of the Orion Editor and initialize it
				require(["orion/editor/edit", "orion/editor/textMateStyler", "orion/editor/sparqlGrammar"], function(edit, mTextMateStyler, mSparqlGrammar) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

					// Remove active selection to avoid bug in IE (occurs when parameters prompt dialog is opened before Show Query)
					if (window.getSelection) {
						if (window.getSelection().empty) {
							window.getSelection().empty();
						}
						else if (window.getSelection().removeAllRanges) {
							window.getSelection().removeAllRanges();
						}
					}
					
					// Line and column number status bar
					var status = ""; //$NON-NLS-1$
					var statusReporter = function(message, isError) {
						if (isError) {
							status =  "ERROR: " + message; //$NON-NLS-1$
						} else {
							status = message;
						}
						self.textboxStatus.innerHTML = status;
					};
					
					// Instantiate the editor
					orionEditor = edit({
						parent: self.textbox,
						statusReporter: statusReporter,
						readonly: true
					});
					
					// Add Grammar based syntax highlighter for SPARQL to the editor
					var syntaxHighlighter = {
						styler: null,
						highlight: function(editor) {
							if (this.styler) {
								this.styler.destroy();
								this.styler = null;
							}
							var textView = editor.getTextView();
							this.styler = new mTextMateStyler.TextMateStyler(textView, mSparqlGrammar.SparqlGrammar());
						}
					};

					syntaxHighlighter.highlight(orionEditor);
					if(globalSetText != null)
					{
						orionEditor.setText(globalSetText);
						globalSetText = null;
					}
				});
			},

			close: function(e) {
				if(orionEditor !== null)
					orionEditor.destroy();
				this._dialog.close();
			},
		}); // declare

})();
