/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.ParameterTable"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.ParameterRow"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.enums.EnumManagerDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.enums.EnumStore"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var EnumStore = com.ibm.relm.common.web.enums.EnumStore;
	var EnumManagerDialog = com.ibm.relm.common.web.enums.EnumManagerDialog;
	var HTML = jazz.util.html;
	var commonUtil = com.ibm.relm.common.web.ui.util;

	/**
	* A styled table extended to work with parameters
	*/
	dojo.declare(
		"com.ibm.relm.explorer.web.explorer.queryparams.ParameterTable", //$NON-NLS-1$
		[jazz.ui.StyledTable] ,
		{
			// _parameterRows: [],

			/**
			* This will be a list of data in the table
			*/
			_cachedData: null, //[]

			constructor: function(args) {
				this._cachedData = [];
				this._parameterRows = [];
				this._headerIds = [];
				this._hoverToDestroy = [];
			},

			uninitialize: function() {
				this.destroyParameterRows();
				dojo.forEach(this._hoverToDestroy, function(w) { w.destroy(); });

			},

			destroyParameterRows: function() {
				dojo.forEach(this._parameterRows, function(w) { 
					w.destroyRecursive();
				});
				this._parameterRows = [];
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this.loadingEnumsFailed = false;
				this.deferredLoading = (new EnumStore()).load();
				
				// Adding hover help into a styled table isn't really supported. We'll guess where
				// the title is and wrap it in a span to add the hover help to
				//Also adding ids to be used with aria-labelledby
				var uaKeys = ['com.ibm.relm.explorer.web.text_ParametersType001', 'com.ibm.relm.explorer.web.text_ParametersLabel001'];  //$NON-NLS-1$ //$NON-NLS-2$

				dojo.query('div', this._headerRow).slice(1).forEach(function(titleDiv, i) {  //$NON-NLS-1$
					this._headerIds[i] = dijit.getUniqueId('headerId');//$NON-NLS-1$
					
					var titleSpan = dojo.create('span', {  //$NON-NLS-1$
						innerHTML: titleDiv.innerHTML,
						
						id: this._headerIds[i]
					});		

					dojo.empty(titleDiv);
					dojo.place(titleSpan, titleDiv);

					this._hoverToDestroy.push(new jazz.app.ua.TextHoverHelp({
						uaKey: uaKeys[i],
						contentNode: titleSpan
					}));

				}, this);
			},

			/**
			* Clears the body of the table and destorys the row widgets in the table
			* 
			* @Override clearBody of jazz.ui.StyledTable
			*/
			clearBody: function() {
				this.inherited(arguments);
				this.destroyParameterRows();
			},

			/**
			 * Callback after a value on this table changes. Used to enable Save button
			 * on parent page. This shouldn't be overridden without calling inherited.
			 */
			onChange: function() {
				this._refreshCachedData(this.getData());
			},

			/**
			 * Draws the parameter table, given an array of parameter data
			 *
			 * @param {Parameters [{name: , parameterClass: , type: , label: }]}
			 * @param errback Error handler if enumerations can't be loaded.
			 */
			draw: function(parameters, errback) {
				// console.debug("Parameters passed to draw", parameters); //$NON-NLS-1$
				// allow retries if prior attempt failed
				if (this.loadingEnumsFailed) {
					this.deferredLoading = (new EnumStore()).load();
				}
			 	
			 	//If there is no deferredLoading, then we can assume the enums are ready to use
				if (this.deferredLoading) {
					this.deferredLoading.then(dojo.hitch(this, function(enums) {
						this.enums = enums;
						this.loadingEnumsFailed = false;
						delete this.deferredLoading;
						this._performDraw(parameters);
					}),	errback);
				} else {
					this._performDraw(parameters);
				}
			},

			_performDraw: function(parameters) {
				this.clearBody();
				if (!parameters || parameters.length == 0) return;

				parameters.sort(function(param1, param2) {
					var name1 = param1.name; var name2 = param2.name;
					if (name1 < name2)
						return -1;
					else if (name1 > name2)
						return 1;
					else return 0;
				});
		 		
		 		parameters = dojo.map(parameters, function(parameter) {
		 			parameter = this.checkForCahcedParameter(parameter);
		 			var crntRow = new com.ibm.relm.explorer.web.explorer.queryparams.ParameterRow({
		 				parameter: parameter,
		 				enums: this.enums,
		 				headerIds: this._headerIds,
		 			});
		 			this.connect(crntRow, 'onEnumRefresh', dojo.hitch(this, this.updateEnums)); //$NON-NLS-1$
		 			// Track the parent table so rows can emit onChange events for the table as a whole.
		 			crntRow.parentTable = this;
		 			this.tbody.appendChild(crntRow.domNode);
		 			this._parameterRows.push(crntRow);
		 			return parameter;
		 		}, this);

		 		// console.debug("Parameters being passed to updateCache", parameters); //$NON-NLS-1$
				this._refreshCachedData(parameters);
			},

			/**
			* Reloads the enums of each row in the table from a new store.
			* 
			* @param {updatedStore EnumStore} the updated store 
			*/
			updateEnums: function(updatedStore) {
				this.enums = updatedStore.getEnums();
				dojo.forEach(this._parameterRows, function(row) {
					row.redrawEnumValues(this.enums);
				}, this);
			},

			/**
			* Gets the data in the table and stores it by parameter name in a cache.
			*
			* If data is newer than cached, then replace data, but if cached data contains
			* something that wasn't in data, keep cached copy. 
			*/
			_refreshCachedData: function(data) {
				//Doing two loops over the array seems wrong, but still best option. 
				//This runs in 2n time so O(n) while other way would be O(n^2), loop inside a loop
				var newDataIndexed = {};
				dojo.forEach(data, function(newParam) {
					newDataIndexed[newParam.name] = newParam;
				});

				this._cachedData = dojo.map(this._cachedData, function(oldParam) {
					if (newDataIndexed[oldParam.name]) {
						return newDataIndexed[oldParam.name];
					} else {
						return oldParam;
					}
				}, this);
			},

			/**
			* Checks if a parameter exists in the cache and replaces the parameter with that. 
			* Only replaces with the cached data if the old data only contains a name and nothing else.
			* 
			* @param {parameter {name: , parameterClass: , type: , label: }}
			*/
			checkForCahcedParameter: function(parameter) {
				var foundCachedParam = dojo.some(this._cachedData, function(cachedParam) {
					if (parameter.name == cachedParam.name) {
						if (!parameter.parameterClass && !parameter.type && !parameter.label) {
							parameter = cachedParam;
							// console.debug("Replacing:", parameter, "with:", cachedParam); //$NON-NLS-1$ //$NON-NLS-2$
						} else {
							// console.debug("Not replacing:", parameter, "with:", cachedParam); //$NON-NLS-1$ //$NON-NLS-2$
						}
						return true;
					}
				}, this);

				//If cache didn't contain this parameter, add it to cache
				if (!foundCachedParam) {
					this._cachedData.push(parameter);
				}

				return parameter;
			},

			/**
			* Returns an array with the data stored in the table
			*
			* @return {parameter[]}
			*/
			getData: function() {
				var parameters = [];
					dojo.forEach(dijit.findWidgets(this.tbody), function(row) {
						try {
							var paramInfo = row.getParameter();
							parameters.push(paramInfo);
						} catch (e) {
							if (e.type && e.type == "paramPromptValidation") { //$NON-NLS-1$
								row.domNode.focus();
								commonUtil.flashNode(row.domNode);
							}
							throw e;
						};
					});
				return parameters;
			}
		}
	);

	dojo.declare(
		"com.ibm.relm.explorer.web.explorer.queryparams.ParameterRow", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.explorer.web','explorer/queryparams/templates/ParameterRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			name: "", //$NON-NLS-1$
			parameterClass: "", //$NON-NLS-1$
			type: "", //$NON-NLS-1$
			label: "", //$NON-NLS-1$
			enums: null,

			/*
			 * @args.name {String} Name of the parameter, without surrounding syntax
			 * @args.parameterClass {String} The class of the parameter.
			 * @args.type {String} The type of the parameter. Either simple, list, or complex. A simple parameter
			 		is of class boolean, date, number or string. A list is populated from the database and a user selects a single
			 		value. A system parameter is automatically filled in and the user is not prompted for it.
			 * @args.label {String} Display label of parameter for prompting
			 * @args.enums {Array} Existing enums to populate dropdown
			*/
			constructor: function(args, node) {
				this.messages = messages;
				var parameter = args.parameter;
				this.name = parameter.name;
				this.parameterClass = parameter.parameterClass || ''; //$NON-NLS-1$
				this.type = parameter.type || ''; //$NON-NLS-1$
				this.label = parameter.label || ''; //$NON-NLS-1$
				this.currentLabel = this.label;
				this.enums = args.enums;
				this.enumId = dijit.getUniqueId('enumObj');//$NON-NLS-1$
				this.typeId = args.headerIds[0];
				this.labelId = args.headerIds[1];
			},

			addEnumsToSecondarySelect: function(enums) {
				dojo.forEach(enums, function(enumeration) {
					dojo.create("option", {value: enumeration.uri, innerHTML: HTML.escape(enumeration.label)}, this.enumMoreOption, "before"); //$NON-NLS-1$ //$NON-NLS-2$
				}, this);
			},

			postCreate: function() {
				//This allows parameter rows to be queried by name
				this.domNode.parameterName = this.name;

				this.enumDefaultOption = dojo.create("option", {value: "", innerHTML: HTML.escape(this.messages.Select)}, this.secondarySelect, "first"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this.secondarySelect.selectedIndex = 0;
				this.addEnumsToSecondarySelect(this.enums);

				if (this.parameterClass == "simple") { //$NON-NLS-1$
					this.primarySelect.value = "simple:" + this.type; //$NON-NLS-1$
				} else if (this.parameterClass == "enum") { //$NON-NLS-1$
					this.showSecondarySelect();
					this.primarySelect.value = this.parameterClass;
					this.secondarySelect.value = this.type;
				} else if (this.parameterClass == "system") { //$NON-NLS-1$
					this.primarySelect.value = "system:" + this.type; //$NON-NLS-1$
				} else
					this.primarySelect.value = "simple:string"; //$NON-NLS-1$
			},

			getParameter: function() {
				//name, parameterClass, type, label
				var parameter = {
					name: this.name,
					label: dojo.trim(this.labelField.value) 
				};
				dojo.mixin(parameter, this.getTypeInfo());
				return parameter;
			},


			showSecondarySelect: function() {
				dojo.removeClass(this.secondarySelect, "relm-query-param-subtype-hidden"); //$NON-NLS-1$
				this.secondarySelect.selectedIndex = 0;
			},
			
			hideSecondarySelect: function() {
				dojo.addClass(this.secondarySelect, "relm-query-param-subtype-hidden"); //$NON-NLS-1$
			},

			onPrimaryChange: function() {
				var selected = this.primarySelect.options[this.primarySelect.selectedIndex];
				if (selected.value === "enum") { //$NON-NLS-1$
					this.showSecondarySelect();
				} else {
					this.hideSecondarySelect();
					if (selected.value.split(":")[0] == "system") { //$NON-NLS-1$ //$NON-NLS-2$
						this._setLabelIfNotCustom(selected.text);
					} else {
						this._setLabelIfNotCustom(""); //$NON-NLS-1$
					}
				}

				this._onChange();
			},

			/**
			* Takes a new set of enumeration values and populates the dropdown list with them
			* while keeping track of the currently selected option
			* 
			* @param {enums Array} - An array of all the enumerations present with minimally 
			* names and uris for the enumeration.  
			*/
			redrawEnumValues: function(enums) {
				this.enums = enums;
				//Save currently selected
				var currentlySelected = this.secondarySelect.value;
				//Remove default option and more option from select
				var frag = document.createDocumentFragment();
				frag.appendChild(this.enumDefaultOption);
				frag.appendChild(this.enumMoreOption);
				//Delete remaining elements in select
				dojo.empty(this.secondarySelect);
				//Add default option and more option back to select
				this.secondarySelect.appendChild(frag);
				//Add back the updated enums
				this.addEnumsToSecondarySelect(this.enums);
				//Select the correct value
				this.secondarySelect.value = ""; //$NON-NLS-1$
				if (currentlySelected != "more") { //$NON-NLS-1$
					this.secondarySelect.value = currentlySelected;
				}
			},

			onSecondaryChange: function() {
				var selected = this.secondarySelect.options[this.secondarySelect.selectedIndex];
				if (selected.value === "more") { //$NON-NLS-1$
					var d = new EnumManagerDialog();
					this.connect(d, 'onSave', dojo.hitch(this, this._onEnumManagerSave)); //$NON-NLS-1$
					this.connect(d, 'onCancel', dojo.hitch(this, this._onEnumManagerCancel)); //$NON-NLS-1$
        			d.startup();
				} else if (selected.value){
					this._setLabelIfNotCustom(selected.text);
				}
				
				this._onChange();
			},
			
			_onEnumManagerSave: function(updatedStore) {
				if (updatedStore) {
					this.onEnumRefresh(updatedStore);
				} else {
					this.secondarySelect.value = ""; //$NON-NLS-1$
				}
			},
			
			_onEnumManagerCancel: function() {
				this.secondarySelect.value = ""; //$NON-NLS-1$
			},
			
			/**
			 * Callback indicating that changes in the Enumeration Manager dialog mean that
			 * the table should be updated.
			 * This function is available to be overridden by callers or used with dojo.connect.
			 */
			onEnumRefresh: function(updatedStore) {
				//No-op
			},

			/**
			 * Emits on change event for *parent* table.
			 */
			_onChange: function() {
				this.parentTable.onChange();
			},

			/**
			 * Checks if the label change changed after a keyup to enable Save immediately.
			 */
			_checkForChange: function() {
				if (this.currentLabel !== this.labelField.value) {
					this.currentLabel = this.labelField.value;
					this._onChange();
				}
			},
			
			/**
			 * A custom label is one that is not
			 * 1. blank
			 * 2. the same as the default system properties
			 * 3. the same as an enumeration
			 */
			_setLabelIfNotCustom: function(label) {
				var oldLabel = this.labelField.value;
				
				if (oldLabel) {
					var isSystemLabel = dojo.query('option', this.systemProperties).some(function(option){ //$NON-NLS-1$
						return option.text == oldLabel;
					});
					
					if (!isSystemLabel) {
						var isEnumLabel = dojo.some(this.enums, function(enumeration){
							return enumeration.label == oldLabel;
						});
						
						if (!isEnumLabel) return;
					}
				}
				
				this.labelField.value = label;
			},

			/**
			* @return {Object{parameterClass: [, type: ]}}
			*/
			getTypeInfo: function() {
				var rawType = this.primarySelect.value.split(":"); //$NON-NLS-1$
				var parameterClass = rawType[0];
				var typeInfo = {parameterClass : parameterClass};
				if (parameterClass == "system") //$NON-NLS-1$
					typeInfo.type = rawType[1];
				else if (parameterClass == "simple") //$NON-NLS-1$
					typeInfo.type = rawType[1];
				else if (parameterClass == "enum") { //$NON-NLS-1$ 
					var type = this.secondarySelect.value;	
					if (type != "more") { //$NON-NLS-1$
						typeInfo.type = type;
					} else {
						typeInfo.type = ""; //$NON-NLS-1$
					}
				}
				return typeInfo;
			}
		});
	})();
