/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.SparqlQueryParameters"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.explorer.queryparams.SparqlCommentStripper');//$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var PARAM_START 	= "\\$"; //Opening $ //$NON-NLS-1$
var PARAM_NAME 		= "([^ \\$ \\s \\. ;]+)"; // Param name. Accept almost anything, then validate later for better error message. //$NON-NLS-1$
var SUB_PARAMETER	= "(:?[.;][^\\$]+)?"; // Optional sub-parameter parsed by ST, but doesn't contribute to the name. Like .name in user.name. //$NON-NLS-1$
var PARAM_END		= "\\$"; //Closing $ //$NON-NLS-1$

var PARAMETER_RE = new RegExp(PARAM_START+PARAM_NAME+SUB_PARAMETER+PARAM_END, "g"); //$NON-NLS-1$
var VALID_NAME_RE = /^[a-zA-Z_][a-zA-Z0-9_-]*$/;

function endsWith(str, suffix) {
    return str.indexOf(suffix, str.length - suffix.length) !== -1;
}

function startsWith(str, prefix) {
	return str.indexOf(prefix) === 0;
}

function InvalidParameterNames(message) {
    this.name = "InvalidParameterNames"; //$NON-NLS-1$
    this.message = (message || ""); //$NON-NLS-1$
    this.summary = this.message;
}
InvalidParameterNames.prototype = Error.prototype;

/**
 * Extracts String Template style place holders from a SPARQL query of the form
 * "$name$". This is done to know which parameters are in a query and prompt the
 * user to fill in values for each parameters. Parameters within SPARQL comments are ignored,
 * which string template doesn't natively support. RELM strips the comments before handing
 * a query to String Template.
 * 
 * @since 5.0.1
 */
dojo.mixin(com.ibm.relm.explorer.web.explorer.queryparams.SparqlQueryParameters, {
	InvalidParameterNames: InvalidParameterNames,
	
	/**
	 * Returns all the query parameters names in the given SPARQL query.
	 * Ignores parameters that have been commented out.
	 * 
	 * @param sparql
	 * @return
	 * 		array of parameter names. Empty if there are none.
	 */
	parseQueryParameterNames: function(sparql) {
		var sparqlNoComments = new com.ibm.relm.explorer.web.explorer.queryparams.SparqlCommentStripper(sparql).strip();
		var paramNames = {};
		var invalidParamNames = {};
		
		var results;
		var name;
		while ((results = PARAMETER_RE.exec(sparqlNoComments)) !== null) {
			name = results[1];
			
			// Check for ST keywords
			if (name == "endif" || name == "else") continue; //$NON-NLS-1$ //$NON-NLS-2$

			// Check for ST functions
			if (endsWith(name, ")")) { //$NON-NLS-1$
				if (startsWith(name, "if(") || startsWith(name, "elseif(")) { //$NON-NLS-1$ //$NON-NLS-2$
					continue;
				}
			}

			// Check for system parameter
			if (name == "system") continue; //$NON-NLS-1$

			// Check valid name
			if (!VALID_NAME_RE.test(name)) {
				invalidParamNames[name] = true;
				continue;
			}

			paramNames[name] = true;
		}

		var invalidParamNamesList = Object.keys(invalidParamNames);
		if (invalidParamNamesList && invalidParamNamesList.length) {
			var invalidParamNamesString = invalidParamNamesList.join(", "); //$NON-NLS-1$
			throw new InvalidParameterNames(dojo.replace(messages.invalidParamNames, [invalidParamNamesString]));
		}

		return Object.keys(paramNames);
	},

	parameterNamesAsObjects: function(names) {
		return names.map(function (name) {
			return {name: name};
		});
	}
});
})();