/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.EnumPrompt"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.DataPrompt"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

(function(){
	var HTML = jazz.util.html;

	dojo.declare(
		"com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.EnumPrompt", //$NON-NLS-1$
		[com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.DataPrompt], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.explorer.web','explorer/queryparams/paramprompt/templates/EnumPrompt.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,

			type: '', //$NON-NLS-1$

			postCreate: function() {
				this.inherited(arguments);

				dojo.create('option', {innerHTML: HTML.escape(this.messages.selectValue), value: ""}, this.input, 'first'); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var enumOrDeferred = this.getOrLoadEnum(this.type);

				var callback = function(enumeration) {
					var options = enumeration.members;
					dojo.forEach(options, function(option) {
						var name = HTML.escape(option.label);
						dojo.create('option', {innerHTML: name, value: dojo.toJson(option)}, this.input, 'last'); //$NON-NLS-1$ //$NON-NLS-2$
					}, this);
					this.input.value = ""; //$NON-NLS-1$
				};
				var errback = function(error) {
					this.messageBox.showErrorObject(error, true);
					dojo.empty(this.inputContainer);
					this.input = dojo.create('input', {placeholder: this.messages.enterValue}, this.inputContainer); //$NON-NLS-1$
				};

				dojo.when(enumOrDeferred, dojo.hitch(this, callback), dojo.hitch(this,errback));
			},

			/**
			* Get the raw enum for the given URI. If the enum is not found, then an XHR call is made to retrieve it.
			* The return will either be an enum object or a deferred, so consumers should use dojo.when.
			*
			* @param {string} uri The enum URI.
			* @return {enumObject} The raw enum if available or a deferred. 
			*/
			getOrLoadEnum: function(uri) {
				var enums = this.enumStore.getEnums();
				for (var i = 0; i < enums.length; i++) {
					var enumeration = enums[i];
					if (enumeration.uri == uri)
						return enumeration;
				}

				//Getting to this code means enumeration not found
				var deferred = new dojo.Deferred();

				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/enum', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						uri: uri,
						enumType: "internal" //$NON-NLS-1$
					},
					load: function(enumeration) {
						deferred.callback(enumeration);		
					},
					error: function(error, ioArgs) {
						console.error('Failed to load enumeration.', ioArgs); //$NON-NLS-1$
						deferred.errback(dojo.fromJson(ioArgs.xhr.responseText));
					}
				});

				return deferred;
			},
			
			fillValue: function(value) {
				this.input.value = dojo.toJson(value.value);
			},
			
			getValue: function() {
				var value;
				var rawValue = this.input.value;
				if (rawValue) {
					value = dojo.fromJson(rawValue);
				} else {
					value = rawValue;
				}
				return {
					type: "enum", //$NON-NLS-1$
					value: value
				};
			}
		});

	})();