/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor"); //$NON-NLS-1$

dojo.require("dojox.gfx"); //$NON-NLS-1$
//dojo.require("com.ibm.relm.explorer.web.base.internal.gdc._gfxPatch"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.Editor"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.CreateNodeConnectionTracker"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.Editor,
{
	surface: null, // surface to add gfx shapes
	group: null, // gfx group object
	_delayedLayoutTimeout: 10, // default layout timeout
	
	highlightState: true,
	
	delayedUpdate: true,
	
	_updateTimerId: null,

	reconnectTrackerClass: "com.ibm.relm.explorer.web.base.internal.canvas.tool.ReconnectTracker", //$NON-NLS-1$
	createConnectionTrackerClass: "com.ibm.relm.explorer.web.base.internal.canvas.tool.CreateNodeConnectionTracker", //$NON-NLS-1$

	setHighlightState:function(state) {
		this.highlightState = state;
	},
	// override Viewer#initCanvas
	initCanvas: function(){
		if(!dojo.isIE || this._calledOnce){
			// reinitialize
			this.initSurface();
		}else{
			// Here we need setTimeout, since we want to use calculated dimension of this.domNode.
			var _this = this;
			setTimeout(function(){
				_this.initSurface();
			}, 0);
		}
		if(!this._calledOnce){
			this.connect(this, "addChild", function(w){w.onAdded && w.onAdded();}); //$NON-NLS-1$
			this.connect(this, "removeChild", function(w){w.onRemoved && w.onRemoved();}); //$NON-NLS-1$

			this._updateNodeTargets = [];

			var self = this;
			// setup for delayed loading of node contents
			if (this.delayedUpdate) {
				this.subscribe("com.ibm.relm.explorer.web.VIEW_LAYOUT_COMPLETED", function(message) { //$NON-NLS-1$
					if (message && message == self) {
						self._updateAllContents();
					}
				});
				this.subscribe("com.ibm.relm.explorer.web.CONTAINER_LAYOUT_COMPLETED", function(message) { //$NON-NLS-1$
					if (message && message.owner == self) {
						self._updateAllContents();
					}
				});
			}
		}
		this._calledOnce = true;
	},

	initSurface: function(){
		if(!this.surface) {
			this.surface = dojox.gfx.createSurface(this.domNode, this.domNode.offsetWidth, this.domNode.offsetHeight);
		}
		// Surface.clipNode is no longer supported in the latest Dojo 
//		if(dojo.isIE && this.domNode.childNodes.length > 1){
//			// make the clipNode the first child of domNode so that HTML elements
//			// of child widgets can be shown on top of gfx nodes.
//			this.domNode.removeChild(this.surface.clipNode);
//			this.domNode.insertBefore(this.surface.clipNode, this.domNode.firstChild);
//		}
		if(!this.group) {
			this.group = this.surface.createGroup();
		}
		if(dojo.isIE < 8){
			// bgNode, i.e. <v:rect> needs to be a little smaller than domNode.
			// Otherwise, unnecessary scroll bars appear.
			var r = this.group.bgNode;
			r.style.width  = this.domNode.offsetWidth - 1 + "px"; //$NON-NLS-1$
			r.style.height = this.domNode.offsetHeight - 1 + "px"; //$NON-NLS-1$
		}
		this.adjustCanvasSize();
	},

	destroy: function(){
		if(this.surface){
			this.surface.destroy();
			delete this.surface;
		}
		if (this._clearCachedResult) {
			this._clearCachedResult();
		}
		this._updateNodeTargets = null;
		this.inherited(arguments);
	},
	
	XXfindRightBottom: function(/*Widget?*/widget){
		if(widget){
			return dojo.marginBox(widget.domNode);
		}
		var maxRight = 0, maxBottom = 0;
		dojo.forEach(this.getDescendants(), function(w){
			var dim = dojo.marginBox(w.domNode);
			maxRight = Math.max(maxRight, dim.l + dim.w);
			maxBottom = Math.max(maxBottom, dim.t + dim.h);
		});
		return {w: maxRight, h: maxBottom};
	},

	adjustCanvasSize: function(){
		// summary: resize the surface
		if(!this.domNode){
			// already destroyed
			return;
		}
		if(!this.surface){
			// surface is not created yet ... 84506
			return;
		}
		var needAdjust = !this.surface.rawNode.getAttribute("width")||  //$NON-NLS-1$
						!this.surface.rawNode.getAttribute("height"); //$NON-NLS-1$
		if(!needAdjust){
			var d1 = this.surface.getDimensions();
			var d2 = dojo.marginBox(this.domNode);
			needAdjust = d1.width != d2.w || d1.height != d2.h;
		}
		if(needAdjust){
			this.surface.setDimensions(this.domNode.offsetWidth, this.domNode.offsetHeight);
		}
	},
	
	// override com.ibm.relm.explorer.web.base.internal.canvas.Editor#setupWidgetForEdit
	setupWidgetForEdit: function(widget){
		this.inherited(arguments);
		com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager.createConnectionAnchors(widget);
		if(!widget.isGfxWidget && widget.nodeMoved){
			widget.subscribe("btp:PropertyChanged", "nodeMoved"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},

	// override
	XXaddChild: function(/*Widget*/widget, /*int?*/insertIndex){
		this.inherited(arguments);
		widget.onAdded && widget.onAdded();
	},

	// override
	XXremoveChild: function(/*Widget*/widget){
		this.inherited(arguments);
		widget.onRemoved && widget.onRemoved();
	},

	_updateAllContents: function() {
		this._cancelPreviousRequest();
		
		var allResourceNodes = this.getAllResourceNodes();
		this._updateNodeTargets = dojo.filter(allResourceNodes, function(node) {
			return node && !node._contentsCreated && node.updateNodeContents;
		});
		
		this.updateResorceNodes();
	},
	
	/*
	 * Load node contents which have not been unloaded. Only nodes displayed within visible location are targeted.
	 */
	updateResorceNodes: function() {
		if (this._updateNodeTargets.length == 0) {
			return;
		}
		
		this._cancelPreviousRequest();

		// Find nodes exposed on window
		var domNode = this.domNode;
		if (!domNode) {
			return;
		}
		
		var viewX = domNode.clientLeft;
		var viewY = domNode.clientTop;
		var viewWidth = domNode.clientWidth;
		var viewHeight = domNode.clientHeight;

		var visibleResourceNodes = dojo.filter(this._updateNodeTargets, function(node) {
			return !node._contentsCreated && node.updateNodeContents && 
			this._isExposed(viewX, viewY, viewWidth, viewHeight, node);
		}, this);

		// Do nothing if no nodes to be updated
		if (visibleResourceNodes.length == 0) {
			return;
		}
		
		this._doUpdateResourceNode(visibleResourceNodes, 100);
	},
	
	/*
	 * Check if node is located within box.
	 */
	_isExposed: function(viewX, viewY, viewWidth, viewHeight, node) {
		var domNode = node.domNode;
		if (!domNode) {
			return false;
		}

		// Check if node is inside the window
		var boxes = node.shape.getTransformedBoundingBox();
		var topLeft = boxes[0];
		var bottomRight = boxes[2];
		if (viewX > bottomRight.x || topLeft.x > viewWidth || viewY > bottomRight.y || topLeft.y > viewHeight) {
			return false;
		}
		return true;
	},
	
	/*
	 * Cancel unprocessed request exists if needed
	 */ 
	_cancelPreviousRequest: function() {
		if (this._updateTimerId) {
			clearTimeout(this._updateTimerId);
			delete this._updateTimerId;
			this._updateTimerId = null;
		}
	},

	/*
	 * Load node contents asynchronously. The first 'maxCount' of targetNodes are rendered.
	 */
	_doUpdateResourceNode: function(targetNodes, maxCount) {
		this._updateTimerId = setTimeout(dojo.hitch(this, function(nodes) {
			var count = 0;
			this._updateTimerId = null;
			dojo.some(nodes, function(node){
				if (node.updateNodeContents()) {
					// Remove rendered node once it is proceeded
					var index = dojo.indexOf(this._updateNodeTargets, node);
					if (index >= 0) {
						this._updateNodeTargets.splice(index, 1);
					}
				}
				
				return ++count >= maxCount;
			}, this);		

			if (nodes.length <= count) {
				return;
			}
			
			nodes.splice(0, count);
			if (nodes.length > 0) {
				this._doUpdateResourceNode(nodes, maxCount);
			}
		}, targetNodes), 10);
	}

});
