/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.Viewer"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Container"); //$NON-NLS-1$
dojo.require("dijit._Contained"); //$NON-NLS-1$
dojo.require("dojo.colors"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.Viewer", //$NON-NLS-1$
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	src: "", // url to be loaded in the canvas //$NON-NLS-1$
	label: "", // to be displayed on the tab panel //$NON-NLS-1$
	extensions: [], // array of class names to be mixed in
	extPrefix: "com.ibm.relm.explorer.web.base.internal.canvas.extension.", // the default extension prefix //$NON-NLS-1$

	isEditor: true, // indicates this is an editor

	_editArea: null,

	constructor: function() {
		this._deferredExtensionLoad = new dojo.Deferred();
	},

	postMixInProperties: function(){
		// load extensions
		var extensionsToLoad = [];
		for(var i = 0; i < this.extensions.length; i++){
			if(this.extensions[i].indexOf('.') == -1){ //$NON-NLS-1$
				this.extensions[i] = this.extPrefix + this.extensions[i];
			}
			extensionsToLoad.push(this.extensions[i]);
		}

		var self = this;

		jazz.core.loader.batch_load_async(extensionsToLoad, function() {
			for(var i = 0; i < self.extensions.length; i++){
				var obj = dojo.getObject(self.extensions[i]);
				if(!obj){ console.warn(self.declaredClass+": "+self.extensions[i]+" not found."); } //$NON-NLS-1$ //$NON-NLS-2$
				dojo.extend(com.ibm.relm.explorer.web.base.internal.canvas.Viewer, obj);
				if(obj.initialize){ obj.initialize.call(self); }
			}
			
			self._deferredExtensionLoad.callback();
		});
	},

	postCreate: function(){
		this.domNode.className = "btpCanvas"; //$NON-NLS-1$
		this._editArea = this;
		this.initCanvas();

		if (this.src){
			var self = this;
			this._deferredExtensionLoad.then(function() {
				self.load(self.src);
			});
		}
	},

	// subclass may want to implement
	initCanvas: function(){
	},

	loadComponents: function(/*Array*/list){
		// summary:
		//		Loads component classes.
		// description:
		//		Load may be performed asynchronously.
		//		If list is not specified, find component names from this.domNode.
		if(!list){
			list = dojo.map(dojo.query('[dojoType]', this.domNode), function(node){ //$NON-NLS-1$
				return  node.getAttribute("dojoType"); //$NON-NLS-1$
			});
		}
		for(var i = 0, len = list.length; i < len; i++){
			var className = list[i];
			if(className.indexOf(".") == -1){console.warn("Viewer#loadComponents: className is required: "+className);} // TODO:assert //$NON-NLS-1$ //$NON-NLS-2$
			jazz.core.loader.load_async(className, function() {});
		}
		return list;
	},

	// this.loaderClass extension overrides this
    load: function(url){
		return false;
	},

	isShowing: function(widget){
		for(var w = widget; w; w = w.getParent()){
			if(dojo.style(w.domNode, "display") == "none"){ //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}
		return true;
	},

	fireMessagePosted: function(text){
		var message = {
			type: "MessagePostedNotification", //$NON-NLS-1$
			src: this,
			text: text
		};
		dojo.publish("btp:MessagePosted", [message]); //$NON-NLS-1$
	},

	fireInputChanged: function(){
		var message = {
			type: "InputChangedNotification", //$NON-NLS-1$
			src: this
		};
		dojo.publish("btp:InputChanged", [message]); //$NON-NLS-1$
	},

	setFocus: function(){
	},

	setFocusFrame: function(){
	},

	setFocusFrameRelated: function(){
	},

	removeFocus: function(){
	},

	removeFocusFrame: function(){
	},

	refreshFocusFrame: function(){
	},

	setHalfFocus: function(){
	},

	setHalfFocusFrame: function(){
	},

	removeHalfFocus: function(){
	},

	removeHalfFocusFrame: function(){
	},

	refreshHalfFocusFrame: function(){
	},

	adjustCanvasSize: function(){
	},

	setCanvasSize: function(){
	},

	snap: function(pos, force){
		return pos;
	},

	generate: function(widget){
	}
});

dojo.extend(dijit._Widget, {
	sourceAnchorClass: '', //$NON-NLS-1$
	targetAnchorClass: '', //$NON-NLS-1$
	componentInfoId: '' //$NON-NLS-1$
});
