/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand"); //$NON-NLS-1$

// ----------------------------------------------------------------------------
// Command classes which encapsulate a request as an object
// ----------------------------------------------------------------------------

com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand = {

	/*
	 * appendChild Command
	 */
	AppendChildCommand: function(parent, child){
		this._parent = parent;
		this._child = child;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._child);
		};
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._child);
		};
	},

	/*
	 * insertBefore Command
	 */
	InsertBeforeCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;

		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
		};
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._newChild);
		};
	},

	/*
	 * removeChild Command
	 */
	ReparentCommand: function(child, parent, refChild){
		this._child = child;
		this._newParent = parent;
		this._newRefChild = refChild;
		this._oldParent = child.getParent();
		this._oldRefChild = child.getNextSibling();
		
		this.execute = function(){
			this._newParent.hlp_reparentChild(this._child, this._newRefChild);
		};
	
		this.undo = function(){
			this._oldParent.hlp_reparentChild(this._child, this._oldRefChild);
		};
	},
	/*
	 * removeChild Command
	 */
	RemoveChildCommand: function(oldChild){
		this._oldChild = oldChild;
	
		this.execute = function(){
			if (this._oldChild.domNode) {
				this._parent = this._oldChild.getParent();
				this._refChild = this._oldChild.getNextSibling();
			}
			if(!this._parent){
				// maybe already removed
				console.debug("RemoveChildCommand: parent is null. " + this._oldChild); //$NON-NLS-1$
			}else{
				this._parent.hlp_removeChild(this._oldChild);
			}
		};
	
		this.undo = function(){
			if(this._parent && this._oldChild.domNode){
				if(this._refChild){
					this._parent.hlp_addChild(this._oldChild, this._refChild);
				}else{
					this._parent.hlp_addChild(this._oldChild);
				}
			}
		};
	},

	/*
	 * replaceChild Command
	 */
	ReplaceChildCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
			this._parent.hlp_removeChild(this._refChild);
		};
	
		this.undo = function(){
			this._parent.hlp_addChild(this._refChild, this._newChild);
			this._parent.hlp_removeChild(this._newChild);
		};
	},

	/*
	 * changeName Command (for change operator)
	 */
	ChangeNameCommand: function(target, newName){
		this._target = target;
		this._newName = newName;
	
		this.execute = function(){
			this._oldName = this._target.nodeName;
			this._target.nodeName = this._newName;
		};
	
		this.undo = function(){
			this._target.nodeName = this._oldName;
		};
	},

	/*
	 * move Command
	 */
	MoveCommand: function(widget, /*Object*/toDim, /*Object?*/fromDim){
		// Type of toDim/fromDim: {top:"#px", left:"#px"} //$NON-NLS-1$
		this._widget = widget;
		this._toDim = toDim;
		this._fromDim = fromDim;
	
		this.execute = function(){
			if(!this._fromDim){
				this._fromDim = {
					top: this._widget.hlp_getStyle("top") + "px", //$NON-NLS-1$ //$NON-NLS-2$
					left: this._widget.hlp_getStyle("left") + "px" //$NON-NLS-1$ //$NON-NLS-2$
				};
			}
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top; //$NON-NLS-1$
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left; //$NON-NLS-1$
			}
			this._widget.hlp_setStyles(props);
		};
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top; //$NON-NLS-1$
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left; //$NON-NLS-1$
			}
			this._widget.hlp_setStyles(props);
		};
	},

	/*
	 * resize Command
	 */
	ResizeCommand: function(widget, toDim){
		this._widget = widget;
		this._toDim = toDim;
	
		this.execute = function(){
			this._fromDim = {
				top: this._widget.hlp_getStyle("top") + "px", //$NON-NLS-1$ //$NON-NLS-2$
				left: this._widget.hlp_getStyle("left") + "px", //$NON-NLS-1$ //$NON-NLS-2$
				width: this._widget.hlp_getStyle("width") + "px", //$NON-NLS-1$ //$NON-NLS-2$
				height: this._widget.hlp_getStyle("height") + "px" //$NON-NLS-1$ //$NON-NLS-2$
			};
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top; //$NON-NLS-1$
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left; //$NON-NLS-1$
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._toDim.width; //$NON-NLS-1$
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._toDim.height; //$NON-NLS-1$
			}
			this._widget.hlp_setStyles(props);
		};
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top; //$NON-NLS-1$
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left; //$NON-NLS-1$
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._fromDim.width; //$NON-NLS-1$
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._fromDim.height; //$NON-NLS-1$
			}
			this._widget.hlp_setStyles(props);
		};
	},

	/*
	 * setPropertyValue Command
	 */
	SetPropertyValueCommand: function(widget, /*String*/attribute, value){
		this._widget = widget;
		this._attribute = attribute;
		this._value = value;
	
		this.execute = function(){
			this._oldValue = this._widget.hlp_getPropertyValue(this._attribute);
			this._widget.hlp_setPropertyValue(this._attribute, this._value);
		};
	
		this.undo = function(){
			this._widget.hlp_setPropertyValue(this._attribute, this._oldValue);
		};
	},

	/*
	 * setStyle Command
	 */
	SetStyleCommand: function(widget, styleName, value){
		this._widget = widget;
		this._styleName = styleName;
		this._value = value;
	
		this.execute = function(){
			if(this._styleName == "top" || this._styleName == "left" //$NON-NLS-1$ //$NON-NLS-2$
				|| this._styleName == "width" || this._styleName == "height"){ //$NON-NLS-1$ //$NON-NLS-2$
				this._oldValue = this._widget.hlp_getStyle(this._styleName) + "px"; //$NON-NLS-1$
			}else{
				this._oldValue = dojo.style(this._widget.domNode, this._styleName);
			}
			this._widget.hlp_setStyle(this._styleName, this._value);
		};
	
		this.undo = function(){
			this._widget.hlp_setStyle(this._styleName, this._oldValue);
		};
	},

	/*
	 * update Command
	 */
	UpdateCommand: function(widget, /*String*/updater){
		this._widget = widget;
		this._updater = updater;
	
		this.execute = function(){
			this._widget[this._updater||"update"](); //$NON-NLS-1$
		};
	
		this.undo = this.execute;
	}
};
