/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.extension.connectableEndpoints"); //$NON-NLS-1$

// summary:
//		Creates draggable end points for a gfx line widget.
//		'this' points to the Editor.

dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

com.ibm.relm.explorer.web.base.internal.canvas.extension.connectableEndpoints = {

	// initializer
	initialize: function(){
		this.connect(this, "setHalfFocusFrame", "addConnectableEndpointsRequest"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this, "removeHalfFocusFrame", "removeConnectableEndpointsRequest"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	_targetWidget: null,

	_getConnectableWidget: function(widget) {
		while (widget != null && !widget.connectable) {
			widget = widget.getParent();
		}
		return widget;
	},
	
	/**
	 * Post a request to create connection handles 
	 */
	addConnectableEndpointsRequest: function(widget) {
		this._calcelRequest();
		
		if (this.hlp_getEditor().movingWidget) {
			return;
		}

		widget = this._getConnectableWidget(widget);
		if (!widget || !widget.connectable || widget == this._targetWidget) {
			return;
		}
		
		this._removeEndPoints();
		
		this._showTimer = setTimeout(dojo.hitch(this, function() {
			delete this._showTimer;
			this._addEndPoints(widget);
		}), 500);
	},

	/**
	 * Post a request to remove connection handles 
	 */
	removeConnectableEndpointsRequest: function(widget) {
		this._calcelRequest();
		
		if (!this._targetWidget || widget.hlp_getEditor().movingWidget) {
			return;
		}
		
		this._hideTimer = setTimeout(dojo.hitch(this, function() {
			delete this._hideTimer;
			this._removeEndPoints();
		}), 1000);
	},
	
	startConnectiong: function() {
		this.hlp_getEditor().movingWidget = true;
	},
	
	stopConnectiong: function() {
		this.hlp_getEditor().movingWidget = false;
		this._removeEndPoints();
	},
	
	_addEndPoints: function(widget) {
		if (!widget || this._targetWidget) return;
		
		this.addConnectableEndpoints(widget);
		this._targetWidget = widget;

		this._connectingWidgetHandle = this.connect(this._targetWidget, "onRemoved", dojo.hitch(this, function() { //$NON-NLS-1$
			this._removeEndPoints();
		}));
	},
	
	_removeEndPoints: function() {
		if (!this._targetWidget) return;

		this.removeConnectableEndpoints(this._targetWidget);
		this._targetWidget = null;

		if (this._connectingWidgetHandle) {
			this.disconnect(this._connectingWidgetHandle);
			delete this._connectingWidgetHandle;
		}
	},
	
	_calcelRequest: function() {
		if (this._showTimer) {
			clearTimeout(this._showTimer);
			delete this._showTimer;
		}
		if (this._hideTimer) {
			clearTimeout(this._hideTimer);
			delete this._hideTimer;
		}
	},
	
	/*
	 *              0
	 *        +----------+
	 *        |          |
	 *      3 |          | 1
	 *        |          |
	 *        +----------+
	 *              2
	 */       
	addConnectableEndpoints: function(widget) {
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget.connectable) { return; }
		
		if(widget.__showingConnectableEndpoints){
			; // already showing
		}else if(!widget.__connectableEndpoints){
			this.createConnectableEndpoints(widget);
		}else{
			for(var i = 0, len = widget.__connectableEndpoints.length; i < len; i++){
				this.domNode.appendChild(widget.__connectableEndpoints[i]);
			}
		}
		widget.__showingConnectableEndpoints = true;

		this._updateKnobPosition(widget);
		this._addCallbacks(widget);

		if (dojo.isFunction(widget.onConnectableEndpointsAdded))
			widget.onConnectableEndpointsAdded(widget.__connectableEndpoints);
		this.onConnectableEndpointsAdded(widget, widget.__connectableEndpoints);
	},
	
	_updateKnobPosition: function(widget) {
		if (!widget.shape) return;
		var tbb = widget.shape.getTransformedBoundingBox();
		var points = [{x: (tbb[0].x + tbb[1].x) / 2, y: tbb[0].y},
		              {x: tbb[1].x, y: (tbb[1].y + tbb[2].y) / 2},
		              {x: (tbb[2].x + tbb[3].x) / 2, y: tbb[2].y},
		              {x: tbb[3].x, y: (tbb[3].y + tbb[0].y) / 2}];
	
		dojo.forEach(points, function(point, index) {
			var knob = widget.__connectableEndpoints[index];
			var offset = {x:0, y:0};
			var width = knob.offsetWidth > 0 ? knob.offsetWidth : 16;
			var height = knob.offsetHeight > 0 ? knob.offsetHeight : 16;
			switch(index) {
			case 0: offset.x -= Math.round(width / 2); offset.y -= height + 1; break;
			case 1: offset.y -= Math.round(height / 2); break;
			case 2: offset.x -= Math.round(width / 2); break;
			case 3: offset.x -= width + 1; offset.y -= Math.round(height / 2); break;
			}
			knob.style.left = point.x + offset.x + "px"; //$NON-NLS-1$
			knob.style.top = point.y + offset.y + "px"; //$NON-NLS-1$
		});
	},
	
	removeConnectableEndpoints: function(widget) {
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget.__showingConnectableEndpoints){ return; }
		dojo.forEach(widget.__connectableEndpoints, function(knob) {
			this.domNode.removeChild(knob);
			knob.widget = null;
		}, this);
		delete widget.__connectableEndpoints;

		this._removeCallbacks(widget);
		widget.__showingConnectableEndpoints = false;
		
		if (dojo.isFunction(widget.onConnectableEndpointsRemoved))
			widget.onConnectableEndpointsRemoved();
		this.onConnectableEndpointsRemoved(widget);
	},
	
	createConnectableEndpoints: function(widget){
		widget.__connectableEndpoints = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 4; i++){
			knob = this._createKnob(i);
			if(!knob){ continue; }
			this.domNode.appendChild(knob);

			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){
				return _this.createConnectionTrackerClass;
			};
//			if(!knob.style.cursor){
//				knob.style.cursor = "crosshair"; //$NON-NLS-1$
//			}
			
			widget.__connectableEndpoints.push(knob);
			knob.widget = widget;

			var ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource(knob, "connectable"); //$NON-NLS-1$
			ds.operation = "connect"; //$NON-NLS-1$
			ds.setDragTarget(null); // No need to move anything.
		}
	},
	
	onConnectableEndpointsAdded: function(widget) {
	},
	
	onConnectableEndpointsRemoved: function(widget) {
	},

	_createKnob: function(index) {
		var type = "internal/images/"; //$NON-NLS-1$
		switch(index) {
		case 0: type += "up.png"; break; //$NON-NLS-1$
		case 1: type += "right.png"; break; //$NON-NLS-1$
		case 2: type += "down.png"; break; //$NON-NLS-1$
		case 3: type += "left.png"; break; //$NON-NLS-1$
		default: return null;
		}
		var img = dojo.doc.createElement("IMG"); //$NON-NLS-1$
		img.className = "resizeKnob"; //$NON-NLS-1$
		img.alt = ""; //$NON-NLS-1$
		img.src = dojo.moduleUrl("com.ibm.relm.explorer.web.base", type)+""; //$NON-NLS-1$ //$NON-NLS-2$
		img.style.zIndex = "1100"; //$NON-NLS-1$
		img.style.position = "absolute"; //$NON-NLS-1$
		
		return img;
	},

	_addCallbacks: function(widget) {
		this.__connectingHandles = [];
		
		this.__connectingHandles.push(this.connect(widget, "setPosition", function() { //$NON-NLS-1$
			if (widget.__showingConnectableEndpoints)
				this._updateKnobPosition(widget);
		}));
		this.__connectingHandles.push(this.connect(widget._viewer, "updateResorceNodes", function() { //$NON-NLS-1$
			if (widget.__showingConnectableEndpoints)
				this._updateKnobPosition(widget);
		}));

		var self = this;
		dojo.forEach(widget.__connectableEndpoints, function(knob) {
			this.__connectingHandles.push(this.connect(knob, "onmouseenter", function() { //$NON-NLS-1$
				self._calcelRequest();
				dojo.setStyle(knob, "border", "dotted 1px #999999"); //$NON-NLS-1$ //$NON-NLS-2$
			}));
			this.__connectingHandles.push(this.connect(knob, "onmouseout", function(e) { //$NON-NLS-1$
				self.removeConnectableEndpointsRequest(widget);
				dojo.setStyle(knob, "border", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			}));
		}, this);
	},
	
	_removeCallbacks: function(widget) {
		dojo.forEach(this.__connectingHandles, this.disconnect);
		delete this.__connectingHandles;
	},
	
	XgetNodeType: function(nodeWidget, connectionWidget){
		var nodeInfo = this.hlp_getComponentInfo(nodeWidget);
		var nodeType = nodeInfo && nodeInfo.nodeType || ""; //$NON-NLS-1$
		var connInfo = this.hlp_getComponentInfo(connectionWidget);
		var connType = connInfo && connInfo.connectionType || ""; //$NON-NLS-1$
		return nodeType + (connType ? ":" + connType : "") || "connection"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
};