/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.extension.connectionEndpoints"); //$NON-NLS-1$

// summary:
//		Creates draggable end points for a gfx line widget.
//		'this' points to the Editor.

dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

com.ibm.relm.explorer.web.base.internal.canvas.extension.connectionEndpoints = {
	sourceEndpointKnobType: "blackSquare", //$NON-NLS-1$
	targetEndpointKnobType: "blackSquare", //$NON-NLS-1$

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	// initializer
	initialize: function(){
		this.connect(this, "setFocusFrame", "addConnectionEndpoints"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this, "removeFocusFrame", "removeConnectionEndpoints"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	/*    SOURCE           TARGET
	 *       0 ------------> 1
	 */
	addConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget.isGfxLineWidget) { return; }
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingConnectionEndpoints){
			; // already showing
		}else if(!widget.__connectionEndpoints){
			this.createConnectionEndpoints(widget);
		}else{
			for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
				if(widget.__connectionEndpoints[i].endpointKnobType != "decoration"){ //$NON-NLS-1$
					this.domNode.appendChild(widget.__connectionEndpoints[i]);
				}
			}
		}
		widget.__showingConnectionEndpoints = true;

		// SOURCE
		knob = widget.__connectionEndpoints[0];
		if(knob.endpointKnobType == "decoration"){ //$NON-NLS-1$
			/* nop */;
		}else if(widget.sourceDecorationObj && widget.sourceDecorationObj.getDimensions){
			var dim = widget.sourceDecorationObj.getDimensions();
			knob.style.width = dim.width + "px"; //$NON-NLS-1$
			knob.style.height = dim.height + "px"; //$NON-NLS-1$
			knob.style.top = dim.top + "px"; //$NON-NLS-1$
			knob.style.left = dim.left + "px"; //$NON-NLS-1$
		}else{
			var pt0 = widget.points[0];
			knob.style.top = pt0.y - Math.round(knob.offsetHeight/2) + "px"; //$NON-NLS-1$
			knob.style.left = pt0.x - Math.round(knob.offsetWidth/2) + "px"; //$NON-NLS-1$
		}

		// TARGET
		knob = widget.__connectionEndpoints[1];
		if(knob.endpointKnobType == "decoration"){ //$NON-NLS-1$
			/* nop */;
		}else if(widget.targetDecorationObj && widget.targetDecorationObj.getDimensions){
			var dim = widget.targetDecorationObj.getDimensions();
			knob.style.width = dim.width + "px"; //$NON-NLS-1$
			knob.style.height = dim.height + "px"; //$NON-NLS-1$
			knob.style.top = dim.top + "px"; //$NON-NLS-1$
			knob.style.left = dim.left + "px"; //$NON-NLS-1$
		}else{
			var pt1 = widget.points[widget.points.length - 1];
			knob.style.top = pt1.y - Math.round(knob.offsetHeight/2) + "px"; //$NON-NLS-1$
			knob.style.left = pt1.x - Math.round(knob.offsetWidth/2) + "px"; //$NON-NLS-1$
		}
	},

	removeConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget.__showingConnectionEndpoints){ return; }
		for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
			if(widget.__connectionEndpoints[i].endpointKnobType != "decoration"){ //$NON-NLS-1$
				this.domNode.removeChild(widget.__connectionEndpoints[i]);
			}
		}
		widget.__showingConnectionEndpoints = false;
	},

	createConnectionEndpoints: function(widget){
		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|"); //$NON-NLS-1$
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "SOURCE"){ //$NON-NLS-1$
					dirs |= this._SOURCE;
				}else if(arr[i] == "TARGET"){ //$NON-NLS-1$
					dirs |= this._TARGET;
				}else{
					console.debug("unrecognized direction: " + arr[i]); //$NON-NLS-1$
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		var endpointKnobTypes = [];
		endpointKnobTypes[0] = widget.getSourceEndpointKnobType && widget.getSourceEndpointKnobType();
		if(!endpointKnobTypes[0]){
			var srcInfo = this.getAttributeInfo(widget, "sourceEndpointKnobType"); //$NON-NLS-1$
			endpointKnobTypes[0] = srcInfo && srcInfo.value;
		}
		if(!endpointKnobTypes[0]){
			endpointKnobTypes[0] = this.sourceEndpointKnobType;
		}

		endpointKnobTypes[1] = widget.getTargetEndpointKnobType && widget.getTargetEndpointKnobType();
		if(!endpointKnobTypes[1]){
			var tgtInfo = this.getAttributeInfo(widget, "targetEndpointKnobType"); //$NON-NLS-1$
			endpointKnobTypes[1] = tgtInfo && tgtInfo.value;
		}
		if(!endpointKnobTypes[1]){
			endpointKnobTypes[1] = this.targetEndpointKnobType;
		}

		widget.__connectionEndpoints = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 2; i++){
			if(endpointKnobTypes[i] == "decoration"){ //$NON-NLS-1$
				knob = (i == 0) ? widget.getSourceDecoration().shape.rawNode
								: widget.getTargetDecoration().shape.rawNode;
			}else{
				knob = this.createKnob(endpointKnobTypes[i]);
				this.domNode.appendChild(knob);
			}
			if(!knob){ continue; }
			knob.knobPosition = i; // [custom property]
			knob.endpointKnobType = endpointKnobTypes[i]; // [custom property]
			knob.getDragTracker = function(){
				_this.setFocus(widget);
				return _this.reconnectTrackerClass;
			};
			if(!knob.style.cursor){
				knob.style.cursor = "crosshair"; //$NON-NLS-1$
			}
			widget.__connectionEndpoints.push(knob);

			var nodeWidget = (i == 0) ? widget.getTargetNodeWidget() : widget.getSourceNodeWidget();
			var type = nodeWidget.hlp_getNodeType(widget);
			var ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource(knob, type);
			ds.operation = "connect"; //$NON-NLS-1$
			ds.setDragTarget(null); // No need to move anything.

			if(i == 0 && !(dirs & this._SOURCE)){
				knob.style.display = "none"; //$NON-NLS-1$
			}else if(i == 1 && !(dirs & this._TARGET)){
				knob.style.display = "none"; //$NON-NLS-1$
			}
		}
	},

	XgetNodeType: function(nodeWidget, connectionWidget){
		var nodeInfo = this.hlp_getComponentInfo(nodeWidget);
		var nodeType = nodeInfo && nodeInfo.nodeType || ""; //$NON-NLS-1$
		var connInfo = this.hlp_getComponentInfo(connectionWidget);
		var connType = connInfo && connInfo.connectionType || ""; //$NON-NLS-1$
		return nodeType + (connType ? ":" + connType : "") || "connection"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
};