/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.extension.focus"); //$NON-NLS-1$

// summary:
//		Handles focus for components on Editor.
//		'this' points to the Editor.

com.ibm.relm.explorer.web.base.internal.canvas.extension.focus = {
	setFocus: function(widget){
		if(!this.focusWidgets){
			this.focusWidgets = [];
		}
		if(widget && this.focusWidgets[0] == widget){ 
			if(!dojo.isIE){
				this.domNode.focus();
			}
			if(widget.focus){
				widget.focus();
			}else{
				try {
					widget.domNode.focus();
				} catch(e){}
			}
			return; 
		}
		if(widget == this){
			widget = null;
		}
		if(!widget){
			this.removeFocus();
			this.domNode.focus();
			return;
		}
		if(this.focusWidgets[0]){
			// Hopefully, this removes focus from the currently focused widget without firing
			// the selectionChanged event even if the clicked widget has no focusable elements.
			this.removeFocus(this.focusWidgets);
		}

		if(!dojo.isIE){
			// Put focus on the EditArea.
			// FF needs this to respond to keys such as DEL, Ctrl+Z, etc.
			// However, IE scrolls up the canvas if you do this. IE doesn't need this to respond to keys.
			this.domNode.focus();
		}

		this.setFocusFrame(widget);
		if(widget.focus){
			widget.focus();
		}else{
			try {
				widget.domNode.focus();
			} catch(e){}
		}

		this.focusWidgets = [widget];
		if(!this._focusConns){
			this._focusConns = [];
		}
		this._focusConns.push(this.connect(widget, "layout", function(){//$NON-NLS-1$
			this.refreshFocusFrame(widget);
		}));
		if(this.highlightState) {
			this.setRelationFocus(widget);
		}
		this.fireSelectionChanged(widget);
	},
	
	toggleFocus: function(widget){
		var existingFocusIndex = dojo.indexOf(this.focusWidgets, widget);
		if (existingFocusIndex >= 0) {
			this.removeFocus([widget]);
			this.focusWidgets.splice(existingFocusIndex, 1);
			this.fireSelectionChanged(this.focusWidgets.length == 1 ? this.focusWidgets[0] : this.focusWidgets);
			return;
		}
			
		if(!this.focusWidgets){
			this.focusWidgets = [];
		}
		if(widget == this){
			widget = null;
		}
		if(!widget){
			return;
		}

		if(!dojo.isIE){
			// Put focus on the EditArea.
			// FF needs this to respond to keys such as DEL, Ctrl+Z, etc.
			// However, IE scrolls up the canvas if you do this. IE doesn't need this to respond to keys.
			this.domNode.focus();
		}

		this.setFocusFrame(widget);
		if(widget.focus){
			widget.focus();
		}else{
			try {
				widget.domNode.focus();
			} catch(e){}
		}

		this.focusWidgets.splice(0, 0, widget);
		if(!this._focusConns){
			this._focusConns = [];
		}
		this._focusConns.push(this.connect(widget, "layout", function(){//$NON-NLS-1$
			this.refreshFocusFrame(widget);
		}));
		if(this.highlightState) {
			this.setRelationFocus(widget);
		}
		this.fireSelectionChanged(this.focusWidgets.length == 1 ? this.focusWidgets[0] : this.focusWidgets);
	},

	setFocusFrame: function(widget){
		if(widget.hlp_setFocusFrame){
			widget.hlp_setFocusFrame();
		}
	},
	
	setRelationFocus: function(widget){
		if(widget.isGfxLineWidget) {
			// connection clicked
			var nodes = [];
			if(widget.sourceAnchorObj) {
				nodes.push(widget.sourceAnchorObj.owner);
			}
			if(widget.targetAnchorObj) {
				nodes.push(widget.targetAnchorObj.owner);
			}
			dojo.forEach(nodes, dojo.hitch(this, function(node) {
				node.hlp_setRelationFocusFrame();
				this.focusWidgets.push(node);
			}));
		} else {
			// node clicked
			if(dojo.isFunction(this.findConnections)){
				var links = this.findConnections(widget);
				links = links.concat(this.findConnections(null, widget));
				dojo.forEach(links, dojo.hitch(this, function(link) {
					link.hlp_setRelationFocusFrame();
					this.focusWidgets.push(link);
				}));
				var relatedNodes = this.findRelatedNodes(widget, links);
				dojo.forEach(relatedNodes, dojo.hitch(this, function(node) {
					node.hlp_setRelationFocusFrame();
					this.focusWidgets.push(node);
				}));
				
			}
		}
	},
	
	findRelatedNodes: function(node, links) {
		var targetNodes =[];
		dojo.forEach(links, function(link) {
			if(link.sourceAnchorObj.owner == node) targetNodes.push(link.targetAnchorObj.owner);
			if(link.targetAnchorObj.owner == node) targetNodes.push(link.sourceAnchorObj.owner);
		});
		return targetNodes;
	},
	
	// If from is given, remove focus from it.
	// Otherwise, remove focus from the this.focusWidgets and fire the event.
	removeFocus: function(from){
		if((!this.focusWidgets || this.focusWidgets.length == 0) && !from){ return; }

		var widgets = from ? from : this.focusWidgets;

		dojo.forEach(widgets, dojo.hitch(this, function(widget){
			this.removeFocusFrame(widget);

			if(widget.blur){
				widget.blur();
			}else if(widget.domNode){
				widget.domNode.blur();
			}
		}));
		dojo.forEach(this._focusConns, this.disconnect, this);
		
		if(!from){
			this.focusWidgets = [];
			this.fireSelectionChanged(null);
		}
	},

	removeFocusFrame: function(widget){
		if(widget.hlp_removeFocusFrame){
			widget.hlp_removeFocusFrame();
		}
	},

	refreshFocusFrame: function(widget){
		if(!widget && this.focusWidgets && this.focusWidgets[0]){
			widget = this.focusWidgets[0];
		}
		if(!widget){ return; }
		this.removeFocusFrame(widget);
		this.setFocusFrame(widget);
	},

	setHalfFocus: function(widget){
		if(!this.halfFocusWidgets){
			this.halfFocusWidgets = [];
		}
		if(widget && this.halfFocusWidgets[0] == widget){ return; }
		if(widget == this){
			widget = null;
		}
		if(!widget){
			this.removeHalfFocus();
			return;
		}
		if(this.halfFocusWidgets[0]){
			// Hopefully, this removes focus from the currently focused widget without firing
			// the selectionChanged event even if the clicked widget has no focusable elements.
			this.removeHalfFocus(this.halfFocusWidgets);
		}

		this.setHalfFocusFrame(widget);

		this.halfFocusWidgets = [widget];
	},

	setHalfFocusFrame: function(widget){
		if(widget.hlp_setHalfFocusFrame){
			widget.hlp_setHalfFocusFrame();
		}
	},

	// If from is given, remove focus from it.
	// Otherwise, remove focus from the this.focusWidgets and fire the event.
	removeHalfFocus: function(from){
		if((!this.halfFocusWidgets || this.halfFocusWidgets.length == 0) && !from){ return; }
		var widget = from ? from[0] : this.halfFocusWidgets[0];

		this.removeHalfFocusFrame(widget);

		if(!from){
			this.halfFocusWidgets = [];
		}
	},

	removeHalfFocusFrame: function(widget){
		if(widget.hlp_removeHalfFocusFrame){
			widget.hlp_removeHalfFocusFrame();
		}
	},

	deleteFocusWidget: function(){
		if(!this.focusWidgets || this.focusWidgets.length == 0){ return; }

		var focusWidgets = this.focusWidgets;
		this.removeFocus();

		// TODO: What if two widgets have parent-child relationship?
		var cc = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
		var cmdprovider = this.hlp_getEditor().getCommandProvider();
		var commandType = "delete"; //$NON-NLS-1$
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			if (focusWidget.notDeletable)
				continue;
			var parent = focusWidget.getParent();
			if(parent && parent.update){
				cc.append(new ec.UpdateCommand(parent));
				cc.append(cmdprovider.createCommand({type: commandType, widget: focusWidget}));
				cc.append(new ec.UpdateCommand(parent));
			}else{
				cc.append(cmdprovider.createCommand({type: commandType, widget: focusWidget}));
			}

			// remove connections
			if(this.surface){ // DiagramEditor
				var mgr = com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager;
				var connections = mgr.getSourceConnections(focusWidget).concat(mgr.getTargetConnections(focusWidget));
				for(var j = 0, len2 = connections.length; j < len2; j++){
					var connectionWidget = connections[j];
					if(dojo.indexOf(focusWidgets, connectionWidget) < 0){
						cc.append(cmdprovider.createCommand({type: commandType, widget: connectionWidget}));
					}
				}
			}
		}
		this.hlp_getEditor().executeCommand(cc);

		// Ensure that the focus is on the EditArea so that the user can use keys.
		// Note: don't know why but IE needs setTimeout.
		var _this = this;
		setTimeout(function(){
			_this.domNode.focus();
		}, 0);
	},
	
	copyFocusWidget: function(){
		if (!this.clipboard || !this.focusWidgets || !dojo.isFunction(this.generate))
			return;

		if (this.focusWidgets.length != 1 || this.focusWidgets[0].preventSave)
			return;

		var cmdprovider = this.hlp_getEditor().getCommandProvider();
		var config = cmdprovider.createClipbardData("config", this.getConfig()); //$NON-NLS-1$
		var copyText = cmdprovider.createClipbardData("layout", this.generate(this.focusWidgets[0])); //$NON-NLS-1$
		this.clipboard.setData("layout", copyText); //$NON-NLS-1$
		this.clipboard.setData("config", config); //$NON-NLS-1$
	},
	
	pasteWidget: function() {
		if (!this.clipboard || !this.focusWidgets)
			return;

		var dataFormats = this.clipboard.getAvailableDataFormats(["layout"]); //$NON-NLS-1$
		if (dataFormats.length == 0)
			return;
		
		var pasteText = this.clipboard.getData("layout"); //$NON-NLS-1$
		if (!pasteText)
			return;

		var config = this.clipboard.getData("config"); //$NON-NLS-1$

		var targetWidget = null;
		var refWidget = null;
		if (this.focusWidgets.length == 1) {
			var widget = this.focusWidgets[0];
			while (widget) {
				if (!widget.preventSave) {
					targetWidget = widget;
					break;
				}
				refWidget = widget;
				widget = widget.getParent ? widget.getParent() : null;
			}
		}
		if (!targetWidget) targetWidget = this;
		
		var commandType = "paste"; //$NON-NLS-1$
		var cmdprovider = this.hlp_getEditor().getCommandProvider();
		var command = cmdprovider.createCommand({type: commandType, target: targetWidget, ref: refWidget, 
												layout: pasteText, config: config});
		if (command) {
			this.hlp_getEditor().executeCommand(command);
		}
	}
};
