/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.extension.selectionHandles"); //$NON-NLS-1$

// summary:
//		Creates selection handles for a component on the Editor.
//		'this' points to the Editor.

dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

com.ibm.relm.explorer.web.base.internal.canvas.extension.selectionHandles = {
	selectionKnobType: "blackSquare", //$NON-NLS-1$

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	// initializer
	initialize: function(){
		this.connect(this, "setFocusFrame", "addSelectionHandles"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this, "removeFocusFrame", "removeSelectionHandles"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	/*              NORTH
	 *       0 ------ 1 ------ 2
	 *       |                 |
	 *  WEST 7                 3 EAST
	 *       |                 |
	 *       6 ------ 5 ------ 4
	 *              SOUTH
	 */
	addSelectionHandles: function(widget){
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget || widget.isGfxLineWidget){ return; } // gfx line widgets do not need selection handles
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingSelectionHandles){
			; // already showing
		}else if(!widget.__selectionHandles){
			this.createSelectionHandles(widget);
		}else{
			for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
				this.domNode.appendChild(widget.__selectionHandles[i]);
			}
		}
		widget.__showingSelectionHandles = true;
		for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
			// Copy widgetId to the root node of this shape
			// so that the widget can be found using dijit.getEnclosingWidget.
			// Note that the root node of this shape is not a child of this.domNode.
			widget.__selectionHandles[i].setAttribute("_widgetId", widget.domNode.getAttribute("widgetId")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	
		var dim = widget.hlp_getStyleDimensions(widget, this);
		var north = dim.top - 4 + "px"; //$NON-NLS-1$
		var south = dim.top + dim.height - 2 + "px"; //$NON-NLS-1$
		var west = dim.left - 4 + "px"; //$NON-NLS-1$
		var east = dim.left + dim.width - 2 + "px"; //$NON-NLS-1$
		var center = dim.left + Math.round(dim.width/2) - 3 + "px"; //$NON-NLS-1$
		var middle = dim.top + Math.round(dim.height/2) - 3 + "px"; //$NON-NLS-1$

		var knob;
		var i = 0;

		// NORTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = west;

		// NORTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = center;

		// NORTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = east;

		// EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = east;

		// SOUTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = east;

		// SOUTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = center;

		// SOUTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = west;

		// WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = west;
	},

	removeSelectionHandles: function(widget){
		if(!widget){
			widget = this.focusWidgets[0];
		}
		if(!widget){
			return; // removeFocus() was called when nothing is focused.
		}
		if(!widget.__showingSelectionHandles){
			return;
		}
		for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
			this.domNode.removeChild(widget.__selectionHandles[i]);
		}
		widget.__showingSelectionHandles = false;
	},

	createSelectionHandles: function(widget){
		var knobElem = this.createKnob(this.selectionKnobType);

		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|"); //$NON-NLS-1$
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "NORTH"){ //$NON-NLS-1$
					dirs |= this._NORTH;
				}else if(arr[i] == "SOUTH"){ //$NON-NLS-1$
					dirs |= this._SOUTH;
				}else if(arr[i] == "WEST"){ //$NON-NLS-1$
					dirs |= this._WEST;
				}else if(arr[i] == "EAST"){ //$NON-NLS-1$
					dirs |= this._EAST;
				}else{
					console.debug("unrecognized direction: " + arr[i]); //$NON-NLS-1$
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		widget.__selectionHandles = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 8; i++){
			knob = knobElem.cloneNode(true);
			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){ return _this.resizeTrackerClass; };
			widget.__selectionHandles.push(knob);
			this.domNode.appendChild(knob);

			var ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource(knob);
			ds.setDragTarget(null); // No need to move anything.
			
			switch (i){
				case 0:
					knob.style.cursor = "nw-resize"; break; //$NON-NLS-1$
				case 1:
					knob.style.cursor = "n-resize"; //$NON-NLS-1$
					if(!(dirs & this._NORTH)){
						knob.style.display = "none"; //$NON-NLS-1$
					}
					break;
				case 2:
					knob.style.cursor = "ne-resize"; break; //$NON-NLS-1$
				case 3:
					knob.style.cursor = "e-resize"; //$NON-NLS-1$
					if(!(dirs & this._EAST)){
						knob.style.display = "none"; //$NON-NLS-1$
					}
					break;
				case 4:
					knob.style.cursor = "se-resize"; break; //$NON-NLS-1$
				case 5:
					knob.style.cursor = "s-resize"; //$NON-NLS-1$
					if(!(dirs & this._SOUTH)){
						knob.style.display = "none"; //$NON-NLS-1$
					}
					break;
				case 6:
					knob.style.cursor = "sw-resize"; break; //$NON-NLS-1$
				case 7:
					knob.style.cursor = "w-resize"; //$NON-NLS-1$
					if(!(dirs & this._WEST)){
						knob.style.display = "none"; //$NON-NLS-1$
					}
					break;
			}
		}
	}
};
