/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractConnectionCreationTool"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractConnectionCreationTool", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	sourceWidget: null,
	targetWidget: null,
	newLine: null, // temporally line
	lineWidget: null, // established line
	connectionWidgetInfoId: null,
	connectionWidgetProps: null,

	// When the button is first pressed, the source node is determined and locked in.
	// After that time, the tool will be looking for the target node to complete the connection.
	handleMouseDown: function(e){
		var widget = this.sourceWidget = dijit.getEnclosingWidget(e.target);
		this.getEditor().setFocus(widget);
		if(!widget){ return false; }
		this.lineWidget = null;

		// store the clicked position
		var pos = dojo.position(widget.domNode);
		widget._clickedPos = {
			x: e.clientX - pos.x,
			y: e.clientY - pos.y
		};

		// find the source connection anchor
		this.sourceConnectionAnchor = com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager.getSourceConnectionAnchor(widget, e.target);
		if(!this.sourceConnectionAnchor){
			console.warn("AbstractConnectionCreationTool#handleMouseDown: Source Connection Anchor not found."); //$NON-NLS-1$
			return false;
		}

		if(widget.hlp_canStartConnection && !widget.hlp_canStartConnection(this.sourceConnectionAnchor)){
			return false;
		}

		// get connection widget info id
		this.connectionWidgetInfoId = null;
		this.connectionWidgetProps = null;
		if(widget && widget.hlp_getConnectionInfoId){
			this.connectionWidgetInfoId = widget.hlp_getConnectionInfoId(e.target); // from the node widget
			if(widget.hlp_getConnectionProps){
				this.connectionWidgetProps = widget.hlp_getConnectionProps(e.target); // from the node widget
			}
		}
		if(!this.connectionWidgetInfoId){
			var spec = this.getEditor().componentSpec;
			this.connectionWidgetInfoId = spec && spec.componentInfoId; // from the selected palette item
			this.connectionWidgetProps = spec && spec.componentProps; // from the selected palette item
		}
		this.getEditor().componentSpec = null; // clear componentSpec
		if(this.connectionWidgetInfoId.indexOf(".") != -1){ //$NON-NLS-1$
			console.warn("AbstractConnectionCreationTool#handleMouseDown: infoId is required: " + this.connectionWidgetInfoId); //$NON-NLS-1$
		} // TODO:assert

		// check if this operation satisfies the dragSources condition
		var nodeInfo = widget.hlp_getComponentInfo();
		if(nodeInfo.dragSources){
			var connInfo = this.getEditor().hlp_getComponentInfo(this.connectionWidgetInfoId);
			if(!connInfo.connectionType){ return false; }
			var arr = nodeInfo.dragSources.split(","); //$NON-NLS-1$
			var match = false;
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == connInfo.connectionType){
					match = true;
					break;
				}
			}
			if(!match){ return false; }
		}
		this._state = "ready"; //$NON-NLS-1$
		return true;
	},

	createNewLine: function(e){
		var spec = this.getEditor().componentSpec;
		var infoId = spec ? spec.componentInfoId : this.connectionWidgetInfoId;
		var props = spec ? spec.props : this.connectionWidgetProps;
		this.newLine = this.getEditor().createComponent(infoId, props);
		this.getEditor().addChild(this.newLine);
		this.newLine.moveToFront();
		this.newLine.setSourceAnchor(this.sourceConnectionAnchor);
		var router = this.getEditor().getAttributeInfo(this.newLine, "router"); //$NON-NLS-1$
		if(router && router.value){
			this.newLine.setRouter(router.value);
		}
		return this.newLine;
	},

	removeNewLine: function(e){
		if(!this.newLine){ return; }
		this.newLine.setEnd(this.newLine.getStart());
		this.newLine.setSourceAnchor(null); // remove the anchorListener
		this.newLine.setTargetAnchor(null); // remove the anchorListener if any
		this.getEditor().removeChild(this.newLine);
		this.newLine.destroy();
	},

	// Execute a command to create a connection
	handleCreateConnection: function(e){
		var sa = this.sourceConnectionAnchor;
		var ta = this.targetConnectionAnchor;
		var sw = this.sourceWidget;
		var tw = this.targetWidget;
		var lw = this.newLine;

		if(sa && ta){
			var props = {
				e: e,
				srcWidget: sw,
				tgtWidget: tw,
				srcAnchor: sa,
				tgtAnchor: ta,
				lineWidget: lw,
				tool: this
			};

			if(sw.onSourceBeforeConnect){
				if(sw.onSourceBeforeConnect(props) === false){ //L#105
					return false;
				}
			}
			if(tw.onTargetBeforeConnect){
				if(tw.onTargetBeforeConnect(props) === false){ //L#205
					return false;
				}
			}
			if(lw.onBeforeConnect){
				if(lw.onBeforeConnect(props) === false){ //L#305
					return false;
				}
			}

			// reuse the ghost line as a real line
			this.lineWidget = this.newLine;
			this.newLine = null;

			this.createConnection(sw, tw, sa, ta, lw);

			if(sw.onSourceConnected){
				sw.onSourceConnected(props); //L#106
			}
			if(tw.onTargetConnected){
				tw.onTargetConnected(props); //L#206
			}
			if(lw.onConnected){
				lw.onConnected(props); //L#306
			}
		}
		return true;
	},

	createConnection: function(sw, tw, sa, ta, lw){
		lw.setSourceAnchor(sa.owner.id+":"+sa.anchorName); //$NON-NLS-1$
		lw.setTargetAnchor(ta.owner.id+":"+ta.anchorName); //$NON-NLS-1$
		var offsetParent = this.getOffsetParent(sw, tw);
		if(!offsetParent){ offsetParent = this.getEditor(); }
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.AppendChildCommand(offsetParent, lw);
		this.getEditor().executeCommand(command);
	},

	finishCreateConnection: function(e){
		var sw = this.sourceWidget;
		var tw = this.targetWidget;
		var lw = this.lineWidget;

		if(!sw){ return; } // already finished
		var props = {
			e: e,
			srcWidget: sw,
			srcAnchor: this.sourceConnectionAnchor,
			tgtWidget: tw,
			tgtAnchor: this.targetConnectionAnchor,
			curWidget: tw,
			lineWidget: lw,
			tool: this
		};
		if(sw.onSourceConnectionEnd){
			sw.onSourceConnectionEnd(props); //L#107
		}
		if(tw && tw.onTargetConnectionEnd){
			tw.onTargetConnectionEnd(props); //L#207
		}
		if(lw && lw.onConnectionEnd){
			lw.onConnectionEnd(props); //L#307
		}

		this.removeNewLine(e);
		this.getEditor().loadDefaultTool();
		this.sourceWidget = this.targetWidget = null;
		this.sourceConnectionAnchor = this.targetConnectionAnchor = null;

		this.fireToolOperationFinished();
	},

	handleKeyDown: function(e){
		if(e.keyCode == 27){ // ESC
			this.finishCreateConnection(e);
		}
	},

	XgetNodeType: function(nodeWidget, connectionWidget){
		var nodeInfo = this.getEditor().hlp_getComponentInfo(nodeWidget);
		var nodeType = nodeInfo && nodeInfo.nodeType || ""; //$NON-NLS-1$
		var connInfo = this.getEditor().hlp_getComponentInfo(connectionWidget);
		var connType = connInfo && connInfo.connectionType || ""; //$NON-NLS-1$
		return nodeType + (connType ? ":" + connType : "") || "connection"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	// Return nearest parent widget that has both widget1 and widget2 as ancestors
	getOffsetParent: function(widget1, widget2){
		// first pass through the widget tree
		var temp1 = widget1;
		var temp2 = widget2;
		while(temp1){
			temp2 = widget2;
			while(temp2){
				if(temp1 == temp2){
					return temp1;
				}
				temp2 = temp2.getParent();
			}
			temp1 = temp1.getParent();
		}
		
		// second pass through the widget tree
		temp1 = widget1;
		temp2 = widget2;
		while(temp2){
			temp1 = widget1;
			while(temp1){
				if(temp1 == temp2){
					return temp1;
				}
				temp1 = temp1.getParent();
			}
			temp2 = temp2.getParent();
		}
		
		return null;
	}
});
