/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractDragTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractDragTool", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	_dropTargetWidget: null,
	_nodeUnderMouse: null,
	_conn: null,
	_type: "component", //$NON-NLS-1$

	handleMouseDown: function(e){
		if (com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils.isRightClick(e)) return;
		this.storeMouseDownInfo(e);
		this.findDropTarget(e);

		// In order to catch all the move events of the entire page, we do not use handleMouseMove.
		this._conn = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	onMouseMove: function(e){
		if (com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils.isRightClick(e)) return;
		e.preventDefault(); // this disables selection
	},

	handleMouseUp: function(e){
		// disconnect the mouseMove handler
		dojo.disconnect(this._conn);

		// to deselect the palette
		this.fireToolOperationFinished();

		// reset the tool
		this.getEditor().loadDefaultTool();
	},

	storeMouseDownInfo: function(e){
		this.layerX = e.layerX;
		this.layerY = e.layerY;
		this.pageX = e.pageX;
		this.pageY = e.pageY;
		this.coords = com.ibm.relm.explorer.web.base.internal.util.getEventCoords(e);
		var pos = dojo.position(this.getEditor().domNode, false);
		this.offsetX = pos.x;
		this.offsetY = pos.y;
		this.startX = this.pageX - pos.x;
		this.startY = this.pageY - pos.y;
	},

	findDropTarget: function(e){
		var dt = com.ibm.relm.explorer.web.base.internal.dnd.dragManager.findDropTarget(e.target, {type:this._type, operation:"drop"}); //$NON-NLS-1$
		if(dt){
			this._nodeUnderMouse = dt.nodeUnderMouse;
			this._dropTargetWidget = dijit.getEnclosingWidget(dt.domNode);
		}else{
			this._nodeUnderMouse = null;
			this._dropTargetWidget = null;
		}
		return dt;
	},

	// override (AbstractTool.js)
	findRefWidget: function(e){
		return this._nodeUnderMouse;
	},

	// override (AbstractTool.js)
	getDropTargetWidget: function(e){
		return this._dropTargetWidget;
	},

	// override (AbstractTool.js)
	getNodeUnderMouse: function(e){
		return this._dropTargetWidget.domNode;
	},

	// override (AbstractTool.js)
	getOrientation: function(e){
		var node = this._dropTargetWidget.domNode;
		var display = dojo.style(node, "display"); //$NON-NLS-1$
		if(display == "block"){ //$NON-NLS-1$
			return false;
		}else{
			return true;
		}
	}
});
