/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand"); //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool", null, { //$NON-NLS-1$
	_editor: null,
	_allowNegetivePosition: false,
	
	activate: function(){
	},
	
	deactivate: function(){
	},
	
	getEditor: function(){
		return this._editor;
	},

	setEditor: function(editor){
		this._editor = editor;
	},


	createInsertComponentCommand: function(e, refWidget, targetWidget, widget){
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		if(!refWidget || refWidget == targetWidget){
			command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.AppendChildCommand(targetWidget, widget));
		}else{
			command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.InsertBeforeCommand(targetWidget, widget, refWidget));
		}
		return command;
	},

	insertComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.createDroppedComponent(e);
		var command = this.createInsertComponentCommand(e, refWidget, targetWidget, widget);
		this.getEditor().executeCommand(command);
		this.getEditor().setFocus(widget);
		return true;
	},

	createReparentComponentCommand: function(e, refWidget, targetWidget, widget){
		var ds = e.dragSource;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var fromParent = dojo.position(widget.getParent().domNode, false);
		var toParent = dojo.position(targetWidget.domNode, false);
		var pdx = fromParent.x - toParent.x;
		var pdy = fromParent.y - toParent.y;
		var toDim = {
			top: ds.dragStartCssPosition.y + dy + pdy + "px", //$NON-NLS-1$
			left: ds.dragStartCssPosition.x + dx + pdx + "px" //$NON-NLS-1$
		};

		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		if(dojo.style(widget.domNode, "position") != ds.positioning){ //$NON-NLS-1$
			command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning)); //$NON-NLS-1$
		}
		if(refWidget == targetWidget){
			refWidget = null;
		}
		command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.ReparentCommand(widget, targetWidget, refWidget));
		command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.MoveCommand(widget, toDim));
		return command;
	},

	reparentComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.getDragSourceWidget(e); //TODO: what happens if null is returned?
		if(refWidget == widget){ return; }
		var command = this.createReparentComponentCommand(e, refWidget, targetWidget, widget);
		this.getEditor().executeCommand(command);
		return true;
	},
	
	getTransformation: function(widget){
		var top = 0, left = 0;
		var zoom = 1.0;
		for(; widget && widget.group != null; widget = widget.getParent()){
			var transform = widget.group.getTransform();
			if(transform){
				top += transform.dy;
				left += transform.dx;
				zoom = transform.xx;
			}
		}
		return {left: left, top: top, zoom: zoom};
	},


	createDroppedComponent: function(e){
		var targetWidget = this.getDropTargetWidget(e);
		var targetDomNode = targetWidget.domNode; // domNode of EditArea or parent container
		var dim = com.ibm.relm.explorer.web.base.internal.util.getDimensions(targetDomNode);
		var spec = this.getComponentSpec(e);
		var props = {};
		if(spec.props){
			props = dojo.clone(spec.props);
		}
		var x, y;
		if(e.dragObject && e.dragObject.dragClone){
			var dragClone = e.dragObject.dragClone;
			y = dragClone.offsetTop - dim.top + targetDomNode.parentNode.scrollTop;
			x = dragClone.offsetLeft - dim.left + targetDomNode.parentNode.scrollLeft;
		}else{ // click and release
			var coords;
			if(props.coords){ // ex. ResizeCreationTool stores the drag start position in props.coords
				coords = props.coords;
			}else{
				coords = com.ibm.relm.explorer.web.base.internal.util.getEventCoords(e);
			}
			y = coords.y - dim.top + targetDomNode.scrollTop;// - 4; // for some reason, both IE and FF need -4
			x = coords.x - dim.left + targetDomNode.scrollLeft;// - 4;
		}
		
		var transform = this.getTransformation(targetWidget);
		x -= transform.left;
		y -= transform.top;
		x /= transform.zoom;
		y /= transform.zoom;
		if(!this._allowNegetivePosition){
			if(x < 0) x = 0;
			if(y < 0) y = 0;
		}

		if(this.getEditor().snapObjectsToGridFlag){
			y = this.getEditor().snap(y);
			x = this.getEditor().snap(x);
		}
		if(this.getEditor().fireMessagePosted){
			this.getEditor().fireMessagePosted("("+x+","+y+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		var spec = this.getComponentSpec(e);
		var infoId = spec && spec.componentInfoId;
		if(this.getEditor().freeLayoutMode == true){//TODO: should allow customization with config and widget property
			if(!props.__style){
				props.__style = {};
			}
			if(!props.__style.top){
				props.__style.top = y + "px"; //$NON-NLS-1$
			}
			if(!props.__style.left){
				props.__style.left = x + "px"; //$NON-NLS-1$
			}
			if(!props.__style.position){
				props.__style.position = "absolute"; //$NON-NLS-1$
			}
		}
		var info = this.getEditor().hlp_getComponentInfo(infoId);
		if(infoId === "container"){ //$NON-NLS-1$
			props._containerId = dojo.isFunction(this.getEditor().createUniqueContainerId) ? this.getEditor().createUniqueContainerId("container") : "containerId"; //$NON-NLS-1$ //$NON-NLS-2$
		}
		var widget = this.getEditor().createComponent(infoId, props);
		this.addChildWidgets(widget, info);
		
		return widget;
	},

	addChildWidgets: function(widget, info){
		if(!info || !info.children){ return; }
		var childNodes = com.ibm.relm.explorer.web.base.internal.getChildNodes(info.children, "child"); //$NON-NLS-1$

		for(var i = 0, len = childNodes.length; i < len; i++){
			var childNode = childNodes[i]; // <child>
			if(childNode.attribute && typeof(childNode.attribute.length) == "undefined"){ //$NON-NLS-1$
				childNode.attribute = [childNode.attribute]; // make it an array
			}

			// set default attribute values if any
			var opt = {};
			for(var j = 0, len2 = childNode.attribute.length; j < len2; j++){
				var attr = childNode.attribute[j];
				if(attr.value){
					opt[attr.name] = attr.value;
				}
			}
			var childName = childNode.name;
			var childWidget = this.getEditor().createComponent(childName, opt);

			// repeat for 1-level deeper depth
			this.addChildWidgets(childWidget, childNode);

			widget.addChild(childWidget);
		}
	},

	findRefWidget: function(e){
		var refWidget = null;
		if(e.type == "mouseup"){ // Dropped with D&D //$NON-NLS-1$
			//TODO: what happens when the target is absolute positioning?
			var node = this.getNodeUnderMouse(e);
			if(node){
				refWidget = dijit.getEnclosingWidget(node);
				var isVertical = this.getOrientation(e);
				if(dojo.html && dojo.html.gravity){
				if(isVertical){
					if(dojo.html.gravity(node, e) & dojo.html.gravity.EAST){
						refWidget = refWidget.getNextSibling();
					}
				}else{
					if(dojo.html.gravity(node, e) & dojo.html.gravity.SOUTH){
						refWidget = refWidget.getNextSibling();
					}
				}
				}
			}
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release 
			// nop
		}
		return refWidget;
	},

	getDragSourceWidget: function(e){
		if(e.type == "mouseup"){ // Dropped with D&D //$NON-NLS-1$
			return e.dragSource.data;
		}
		return null;
	},

	getDropTargetWidget: function(e){
		var targetWidget; // EditArea or parent container
		if(e.type == "mouseup"){ // Dropped with D&D //$NON-NLS-1$
			targetWidget = e.dropTarget && e.dropTarget.data;
		}else /*if(e.type == "click")*/ { // Dropped with click and release 
			targetWidget = dijit.getEnclosingWidget(e.target);
		}
		return targetWidget;
	},

	getNodeUnderMouse: function(e){
		var dt = e.dropTarget;
		return dt.getNodeUnderMouse();
	},

	// true: orientation is vertical      (caret shape is like |)
	// false: orientation is horizontal   (caret shape is like -)
	getOrientation: function(e){
		var dt = e.dropTarget;
		return dt.vertical;
	},

	fireToolOperationFinished: function(){
		// Typically used to deselect the selected palette item
		var message = {
			type: "ToolOperationFinishedNotification", //$NON-NLS-1$
			src: this
		};
		dojo.publish("btp:ToolOperationFinished", [message]); //$NON-NLS-1$
	},

	getComponentSpec: function(e) {
		return this.getEditor().componentSpec;
	},
	
	// -------------------------------------------------------------------
	// Default Event Handlers
	// -------------------------------------------------------------------

	// Called when the mouse button has been pressed.
	handleMouseDown: function(e){
		return false;
	},

	// Called when the mouse button has been released.
	handleMouseUp: function(e){
		return false;
	},

	// Called when the mouse is moved.
	handleMouseMove: function(e){
		return false;
	},

	// Called when the mouse pointer is moved onto an element.
	handleMouseOver: function(e){
		return false;
	},

	// Called when the mouse pointer is moved out of an element.
	handleMouseOut: function(e){
		return false;
	},

	// Called when the mouse pointer enters into an element.
	handleMouseEnter: function(e){
		return false;
	},

	// Called when the mouse pointer leaves an element.
	handleMouseLeave: function(e){
		return false;
	},

	// Called when a mouse click occurs.
	handleClick: function(e){
		return false;
	},

	// Called when a mouse double-click occurs.
	handleDblClick: function(e){
		return false;
	},

	// Called when the key is pressed.
	handleKeyDown: function(e){
		return false;
	},

	// Called when the key is released.
	handleKeyUp: function(e){
		return false;
	},

	// Called only once during a drag when the drag threshold has been exceeded.
	handleDragStart: function(e){
		return false;
	},

	// Called when the mouse hovers over an element during dragging.
	handleDragOver: function(e){
		return false;
	},

	// Called whenever the mouse is being dragged.
	handleDragMove: function(e){
		return false;
	},

	// Called when the drag is finished. handleDropXX will be called soon.
	handleDragEnd: function(e){
		return false;
	},

	// Called before checking whether the drop is allowed.
	handleDropStart: function(e){
		return false;
	},

	// Called when a drop operation is performed.
	handleDrop: function(e){
		return false;
	},

	// Called when the drop operation has been finished. e.dragStatus has the result.
	handleDropEnd: function(e){
		return false;
	},

	// Called when the mouse pointer moves out of an element during dragging.
	handleDragOut: function(e){
		return false;
	}
});
