/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.ConnectionCreationTool"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractConnectionCreationTool"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.ConnectionCreationTool", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractConnectionCreationTool,
{
	_ds: null,

	handleMouseDown: function(e){
		e.preventDefault(); // to prevent FF3 from dragging an object
		if(this._state != "moving"){ //$NON-NLS-1$
			if (!this.inherited(arguments)){ return false; }
		}
		return true;
	},

	handleMouseMove: function(e){
		if(this._state != "ready" && this._state != "moving"){ return false; } //$NON-NLS-1$ //$NON-NLS-2$
		e.preventDefault(); // this disables selection

		var props;
		var sw = this.sourceWidget;
		var lw = this.newLine;
		if (this._state == "ready"){ //$NON-NLS-1$
			// This must be the first move event after mousedown. Let's create a new line.
			this.createNewLine(e);
			lw = this.newLine;
			props = {e:e, srcWidget:sw, srcAnchor:this.sourceConnectionAnchor, lineWidget:lw, tool:this};
			if(sw.onSourceConnectionStart){
				if(sw.onSourceConnectionStart(props) === false){ //L#101
					this.removeNewLine(e);
					this._state = "end"; //$NON-NLS-1$
					return false;
				}
			}
			if(lw.onConnectionStart){
				if(lw.onConnectionStart(props) === false){ //L#301
					this.removeNewLine(e);
					this._state = "end"; //$NON-NLS-1$
					return false;
				}
			}
			this._state = "moving"; //$NON-NLS-1$
		}

		props = {e:e, srcWidget:sw, srcAnchor:this.sourceConnectionAnchor, lineWidget:this.newLine, tool:this};
		if(sw.onSourceConnectionMove){
			sw.onSourceConnectionMove(props); //L#102
		}
		if(lw.onConnectionMove){
			lw.onConnectionMove(props); //L#302
		}

		if(dijit.getEnclosingWidget(e.target) == this.newLine){ return true; }

		var tw = this.targetWidget;
		var anchor = tw && com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager.getTargetConnectionAnchor(tw, e.target);
		if(anchor && this.targetConnectionAnchor == anchor){ return false; } // already anchored
		if(this.targetConnectionAnchor){ // release the old target anchor
			props.tgtWidget = props.curWidget = this.targetConnectionAnchor.owner;
			props.tgtAnchor = this.targetConnectionAnchor;
			if(sw.onSourceConnectionDetach){
				if(sw.onSourceConnectionDetach(props) === false){ return false; } //L#104
			}
			if(props.tgtWidget.onTargetConnectionDetach){ // old target widget
				if(props.tgtWidget.onTargetConnectionDetach(props) === false){ return false; } //L#204
			}
			if(lw.onConnectionDetach){
				if(lw.onConnectionDetach(props) === false){ return false; } //L#304
			}

			this.targetConnectionAnchor = null;
			this.newLine.setTargetAnchor(null);
		}
		if(anchor){ // attach
			props.tgtWidget = props.curWidget = tw;
			props.tgtAnchor = anchor;
			if(sw.onSourceConnectionAttach){
				if(sw.onSourceConnectionAttach(props) === false){ return false; } //L#103
			}
			if(tw.onTargetConnectionAttach){
				if(tw.onTargetConnectionAttach(props) === false){ return false; } //L#203
			}
			if(lw.onConnectionAttach){
				if(lw.onConnectionAttach(props) === false){ return false; } //L#303
			}

			this.targetConnectionAnchor = anchor;

			// store the clicked position
			var pos = dojo.position(tw.domNode);
			tw._clickedPos = {
				x: e.clientX - pos.x,
				y: e.clientY - pos.y
			};

			this.newLine.setTargetAnchor(anchor);
		}else{ // move
			// See also ReconnectTracker#handleMouseMove
			var info = this.newLine.hlp_getComponentInfo();
			var offsetH = info && info.draggingLineOffsetH || 5;
			var offsetV = info && info.draggingLineOffsetV || 5;
			var pos = dojo.position(this.getEditor().domNode, false);
			var x = e.pageX - pos.x + this.getEditor().domNode.scrollLeft - offsetH;
			var y = e.pageY - pos.y + this.getEditor().domNode.scrollTop - offsetV;
			this.newLine.setEnd({x:x, y:y});
		}
		this.sourceConnectionAnchor.anchorMoved();
		return true;
	},

	handleMouseOver: function(e){
		if(this._state != "moving"){ return false; } //$NON-NLS-1$
		if(dijit.getEnclosingWidget(e.relatedTarget) == this.newLine || 
			dijit.getEnclosingWidget(e.target) == this.newLine){ // already anchored and moved to/from newLine
			return true;
		}
		var w = dijit.getEnclosingWidget(e.target); // drop target widget
		if(this.sourceWidget == w){ return false; }
		var dt = this._dt = w && w.hlp_getConnectionTarget && w.hlp_getConnectionTarget(e.target);
		if(!dt){ return false; }
		var type = this.sourceWidget.hlp_getNodeType(this.newLine);
		if(dt.accepts({type: type, operation: "connect"})){ //$NON-NLS-1$
			this.targetWidget = w;
			dt.showFeedback();
		}else{
			this.targetWidget = null;
		}
		return true;
	},

	handleMouseOut: function(e){
		if(this._state != "moving" || !this.targetWidget){ return false; } //$NON-NLS-1$
		var to = dijit.getEnclosingWidget(e.relatedTarget);
		var from = dijit.getEnclosingWidget(e.target);
		if((to == this.newLine && from == this.targetWidget) || // from anchor to line
			(from == this.newLine && to == this.targetWidget)){ // from line to anchor
			return true;
		}
		this.targetWidget = null;
		var dt = this._dt;
		if(!dt){ return false; }
		dt.hideFeedback();
		return true;
	},

	handleMouseUp: function(e){
		if(this._state != "moving"){ return false; } //$NON-NLS-1$
		if(this.targetWidget){
			var dt = this._dt;
			dt.hideFeedback();
		}
		this.handleCreateConnection(e);
		this.finishCreateConnection(e);
		this._state = "end"; //$NON-NLS-1$
		return true;
	}

});
