/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.CreateNodeConnectionTracker"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$

// summary:
//		A DragTracker for creating a link between nodes.

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.CreateNodeConnectionTracker", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	sourceWidget: null,
	targetWidget: null,
	lineWidget: null,
	_operationStarted: false,

	_sourceKnob: null,

	handleMouseDown: function(e){
		e.preventDefault(); // this disables selection

		if(this._operationStarted) {
			this.stopConnecting(e);
			return;
		}

		// create a feedback line
		var lineprops = {
				stroke: {color:"#999999", style:"Dash", width:1, cap:"butt"} //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
		};
		this.lineWidget = this.getEditor().createComponent("lineConnection", lineprops); //$NON-NLS-1$
		this.getEditor().addChild(this.lineWidget);
		this.lineWidget.moveToFront();

		// store dragging info
		this._sourceKnob = e.target;
		this.sourceWidget = e.target.widget;
		this.targetWidget = null;
		if (this.getEditor().startConnectiong) this.getEditor().startConnectiong();
		this.getEditor().setFocus(this.sourceWidget);
	},
	
	handleDragStart: function(e){
		this._operationStarted = true;
//		this.getEditor().domNode.style.cursor = "crosshair"; //$NON-NLS-1$
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e){
		e.preventDefault(); // this disables selection
		if(!this._operationStarted) return; // drag is not started yet or perhaps was canceled with ESC

		
		if (dojo.hasClass(e.target, "resizeKnob")) { //$NON-NLS-1$
			return;
		}
		
		if (this.targetWidget)
			this.getEditor().removeHalfFocusFrame(this.targetWidget);
		var widget = this.getDropTargetWidget(e);
		this.targetWidget = this._getConnectableWidget(this.sourceWidget, widget); 
		if (this.targetWidget) {
			this.getEditor().setHalfFocusFrame(this.targetWidget);
			this.getEditor().domNode.style.cursor = "pointer"; //$NON-NLS-1$
		} else if (this._hasConnectableType(widget)) {
			this.getEditor().domNode.style.cursor = "no-drop"; //$NON-NLS-1$
		} else {
			this.getEditor().domNode.style.cursor = "default"; //$NON-NLS-1$
		}

		// redraw feedback line
		var knob = this._sourceKnob;
		var m = dojox.gfx.matrix.invert(this.getEditor().group.getTransform());
		
		var srcX = knob.offsetLeft + knob.offsetWidth / 2;
		var srcY = knob.offsetTop + knob.offsetHeight / 2;
		var dstX = e.layerX;
		var dstY = e.layerY;

		if (srcX < dstX) dstX - 1;
		else if (srcX > dstX) dstX + 1;
		if (srcY < dstY) dstY - 1;
		else if (srcY > dstY) dstY + 1;
		
		var sourcePos = dojox.gfx.matrix.multiplyPoint(m, srcX, srcY);
		var targetPos = dojox.gfx.matrix.multiplyPoint(m, dstX, dstY);
		
		this.lineWidget.points = [sourcePos, targetPos];
		this.lineWidget.update();
		return true;
	},

	handleMouseOver: function(e){
		if (this._operationStarted)
			e.preventDefault(); // this disables selection
	},
	
	handleMouseUp: function(e){
		e.preventDefault(); // this disables selection
		if(!this._operationStarted) {
			this.stopConnecting(e);
			return; // drag has not been started yet or perhaps was canceled with ESC
		}

		var source = this.sourceWidget;
		var target = this.targetWidget;

		this.stopConnecting(e);
		
		if (source != null && target != null)
			this.createConnection(source, target);
	},
	
	handleKeyDown: function(e){
		if(e.keyCode == 27) // ESC
			this.stopConnecting(e);
	},

	stopConnecting: function(e){
		if (this.getEditor().stopConnectiong) this.getEditor().stopConnectiong();
		if(!this._operationStarted) return; // already canceled
		delete this._operationStarted;
		
		this.getEditor().domNode.style.cursor = ""; //$NON-NLS-1$
		if(this._dt)
			this._dt.disabled = false;
		
		if (this.targetWidget)
			this.getEditor().removeHalfFocusFrame(this.targetWidget);

		this.getEditor().removeChild(this.lineWidget);
		this.lineWidget.destroyRecursive();
		this.getEditor().setFocus(this.sourceWidget);
		
		this.sourceWidget = this.targetWidget = this.lineWidget = this.sourceKnob = null;
	},

	_getConnectableWidget: function(source, target) {
		return null;
	},
	
	createConnection: function(source, target) {
	}
});
