/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultSelectionTool"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils'); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.base.internal.canvas.tool.DragGhostMoveTracker'); //$NON-NLS-1$

(function() {

var NodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultSelectionTool", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	defaultDragTracker: "com.ibm.relm.explorer.web.base.internal.canvas.tool.DragGhostMoveTracker", //$NON-NLS-1$
	cursor: "default", //$NON-NLS-1$
	_draggingWidget: null,

	// -------------------------------------------------------------------
	// Event Handlers
	// -------------------------------------------------------------------
	handleMouseDown: function(e){
		var widget = dijit.getEnclosingWidget(e.target);

		if (NodeUtils.isRightClick(e) && widget.showContextMenu) {
			// Ignore this event since the oncontextmenu event will fire as well.
			dojo.stopEvent(e);
			return null;
		}

		var dragTracker = this.findDragTracker(e.target);
		this.setDragTracker(dragTracker);
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseDown(e);
		}

		return null;
	},

	handleContextMenu: function(e) {
		var widget = dijit.getEnclosingWidget(e.target);
		var editor = widget.hlp_getEditor();
		if (editor.focusWidgets && editor.focusWidgets.length > 0)
			widget = editor.focusWidgets[0];
		
		if (widget.showContextMenu) {
			if(widget.showContextMenu(e.clientX, e.clientY)){
				dojo.stopEvent(e);				
			}
		} else if (widget instanceof com.ibm.relm.explorer.web.base.internal.canvas.Editor) {
			// If called by the Menu key, the event is only over the editor, not a resource node.
			// Assume this applies to the first selected node, if any.
			var firstResourceNode = widget.focusWidgets && widget.focusWidgets[0];
			if (firstResourceNode && firstResourceNode.showContextMenu) {
				if(firstResourceNode.showContextMenu()){
					dojo.stopEvent(e);					
				}
			}
		}
	},

	handleMouseUp: function(e){
		var widget = dijit.getEnclosingWidget(e.target);
		if (!widget.primaryMenuTarget && (e.ctrlKey || e.shiftKey) && this.getEditor().toggleFocus) {
			if (e.button < 2)
				this.getEditor().toggleFocus(widget);
		} else {
			this.getEditor().setFocus(widget);
		}
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseUp(e);
		}
	},
	
	handleMouseMove: function(e){
		if (NodeUtils.isRightClick(e)) return null;
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseMove(e);
		}
	},
	
	handleMouseOver: function(e){
		if(!e.target._trackerLoaded){
			var dragTracker = this.findDragTracker(e.target);
			if(!dojo.getObject(dragTracker)){
				dojo["require"](dragTracker); // may be async //$NON-NLS-1$
			}
			e.target._trackerLoaded = true; // [custom property]
		}
		if (this.getDragTracker()){
			this.getDragTracker().handleMouseOver(e);
			if (e.defaultPrevented) 
				return;
		}
		this.getEditor().domNode.style.cursor = this.cursor;
		if (!dojo.hasClass(e.target, "resizeKnob")) {//$NON-NLS-1$
			this.getEditor().setHalfFocus(dijit.getEnclosingWidget(e.target));
		}
	},
	
	handleMouseOut: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseOut(e);
		}
	},
	
	handleMouseEnter: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseEnter(e);
		}
	},
	
	handleMouseLeave: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleMouseLeave(e);
		}
	},
	
	handleClick: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleClick(e);
		}
		return null;
	},

	handleKeyDown: function(e){
		if (this.getDragTracker()){
			if(this.getDragTracker().handleKeyDown(e) === true){ return true; }
		}
		var command = null, arg = null;
		if(e.keyCode == 46){ // Del key
			command = "del"; //$NON-NLS-1$
		}
		else if(e.ctrlKey && e.keyCode == 123){ // Ctrl + F12
			console.open();
		}
		else if(e.ctrlKey && e.keyCode == 90){ // Ctrl + Z
			command = "undo"; //$NON-NLS-1$
		}
		else if(e.ctrlKey && e.keyCode == 89){ // Ctrl + Y
			command = "redo"; //$NON-NLS-1$
		}
		else if(e.ctrlKey && e.keyCode == 67){ // Ctrl + C
			command = "copy"; //$NON-NLS-1$
		}
		else if(e.ctrlKey && e.keyCode == 86){ // Ctrl + V
			command = "paste"; //$NON-NLS-1$
		}
		else if(e.keyCode == 27){ // ESC
			this.setDragTracker(null);
		}
		
		if(this.getEditor()._editMode && command){
			var message = {
				type: "ActionInvokedNotification", //$NON-NLS-1$
				src: this,
				editor: this.getEditor(),
				command: command,
				arg: arg
			};
			dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
		}
		
		if(e.keyCode >= 33 && e.keyCode <= 40 ){
			var message = {
					type: "MoveViewportByKey", //$NON-NLS-1$
					stc: this,
					keyCode: e.keyCode
			};
			dojo.publish("btp:KeyCommandInvoked", [message]); //$NON-NLS-1$
		}
		return false;
	},
	
	// -------------------------------------------------------------------
	// Drop Target Handlers
	// -------------------------------------------------------------------
	handleDragStart: function(e){
		if(e.dropTarget.data != this.getEditor()){ return null; } // dicard events for components on the editor
		if (this.getDragTracker()){
			return this.getDragTracker().handleDragStart(e);
		}
		return null;
	},

	handleDragEnd: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleDragEnd(e);
		}
	},

	handleDropStart: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleDropStart(e);
		}
	},

	handleDrop: function(e){
		if (this.getDragTracker()){
			return this.getDragTracker().handleDrop(e);
		}
		var targetWidget = this.getDropTargetWidget(e);
		var type = e.dragSource.type;
		var ret = null;
		if(targetWidget[type+"DropHandler"]){ //$NON-NLS-1$
			ret = targetWidget[type+"DropHandler"](e, this); //$NON-NLS-1$
		}else if(this[type+"DropHandler"]){ //$NON-NLS-1$
			ret = this[type+"DropHandler"](e); //$NON-NLS-1$
		}
		this.getEditor().loadDefaultTool();
		e.dragStatus = ret ? "dropSuccess" : "dropFailure"; // See DragManager.onMouseUp() //$NON-NLS-1$ //$NON-NLS-2$
		return null;
	},

	handleDropEnd: function(e){
		com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDropTarget = null;
		this.setDragTracker(null);
		var widget = this.getDragSourceWidget(e);
		if(widget instanceof com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode){
			this.getEditor().setFocus(widget);
		}
	},

	createBindingDropCommand: function(e){
		var ds = e.dragSource;
		var treeNodeWidget = ds.treeNode; // TreeNode widget
		var targetWidget = this.getDropTargetWidget(e);

		var model = null;
		var models = dojo.widget.manager.getWidgetsByType("Model"); // Hmm.. prefix doesn't seem to be allowed //$NON-NLS-1$
		var ids = {};
		for(var i = 0, len = models.length; i < len; i++){
			var m = models[i];
			if(m.src == treeNodeWidget.data){
				model = m;
				break;
			}
			ids[m.id] = true;
		}

		var ref = treeNodeWidget.ref ? treeNodeWidget.ref : "."; //$NON-NLS-1$
		if(model && targetWidget.modelId == model.id && targetWidget.ref == ref){
			return null; // nothing has been changed
		}

		var compoundCommand = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();

		compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.UpdateCommand(targetWidget));
		if(targetWidget.setTitle){
			var titleInfo = this.getEditor().getAttributeInfo(targetWidget, "title"); //$NON-NLS-1$
			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetPropertyValueCommand(targetWidget, titleInfo, treeNodeWidget.title));
		}

		var modelIdInfo = this.getEditor().getAttributeInfo(targetWidget, "modelId"); //$NON-NLS-1$
		var refInfo = this.getEditor().getAttributeInfo(targetWidget, "ref"); //$NON-NLS-1$
		if(model){
			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, model.id));
			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
		}else{
			// <btp:model id="model1" src="some.xml"></btp:model>
			var modelId;
			for(var j = 1; ; j++){
				if(!ids["model"+j]){ //$NON-NLS-1$
					modelId = "model"+i; //$NON-NLS-1$
					break;
				}
			}
			var opt = {
				id: modelId,
				src: treeNodeWidget.data,
				format: treeNodeWidget.format
			};
			var widget = this.getEditor().createLiveWidget("btp:model", opt); //$NON-NLS-1$

			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, modelId));
			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
			compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.InsertBeforeCommand(this.getEditor(), widget, this.getEditor().getChildren()[0]));

		}
		compoundCommand.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.UpdateCommand(targetWidget));
		return compoundCommand;
	},

	bindingDropHandler: function(e){
		var command = this.createBindingDropCommand(e);
		if(command){
			this.getEditor().executeCommand(command);
		}
		return true;
	},

	findDragTracker: function(/*DomNode*/target){
		var dragTracker;

		// Look for a DragTracker from the clicked DOM node
		if(target.getDragTracker){ // probably resize knob
			dragTracker = target.getDragTracker();
			if(dragTracker){
				return dragTracker;
			}
		}

		// Look for a DragTracker from the clicked widget
		var widget = dijit.getEnclosingWidget(target);
		if(widget && widget.hlp_getDragTracker){
			dragTracker = widget.hlp_getDragTracker(target);
			if(dragTracker){
				return dragTracker;
			}
		}

		// Look for a DragTracker from the EditArea
		dragTracker = this.getEditor().hlp_getDragTracker && this.getEditor().hlp_getDragTracker();
		if(dragTracker){
			return dragTracker;
		}

		// Use the default DragTracker
		return this.defaultDragTracker;
	},

	getDragTracker: function(){
		return this.dragTracker;
	},

	setDragTracker: function(/*String|DragTracker*/newDragTracker){
		if(typeof(newDragTracker) == "string"){ //$NON-NLS-1$
			if(!this._dragTrackerCache){ this._dragTrackerCache = []; }
			if(this._dragTrackerCache[newDragTracker]){
				newDragTracker = this._dragTrackerCache[newDragTracker];
			}else{
				console.log("newDragTracker="+newDragTracker); //$NON-NLS-1$
				var impl = dojo.getObject(newDragTracker);
				newDragTracker = this._dragTrackerCache[newDragTracker] = new impl();
				newDragTracker.setEditor(this.getEditor());
			}
		}
		this.dragTracker = newDragTracker;
	}
});
})();