/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.DragMoveTracker"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource"); //$NON-NLS-1$

// summary:
//		A DragTracker that moves components on the editor with drag and drop operation.

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.DragMoveTracker", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	_dragGhost: false, // true: drag a dashed frame, false: drag an actual component
	_connectors: null,
	_moveConn: null,
	_allowNegetivePosition: false,

	handleMouseDown: function(e){
		if(this._draggingWidget){
			// if already in drag mode by some reason, finish drag-move
			this.disableDragMove(this._draggingWidget);
			this._draggingWidget = null;
			return true;
		}
		// find target widget
		var widget = dijit.getEnclosingWidget(e.target);
		while(true){
			if(!widget || this.getEditor() == widget) return false;
			var nodeLock = this.getEditor().getConfig().nodeLock;
			if((this.getEditor()._editMode || nodeLock !== true)
			&& widget.movable !== false) break;
			if(!dojo.isFunction(widget.getParent)) return false;
			widget = widget.getParent();
		}
		this._draggingWidget = widget;
		this.enableDragMove(widget);
		return true;
	},

	handleClick: function(e){
		if(this._draggingWidget){
			this.disableDragMove(this._draggingWidget);
			this._draggingWidget = null;
		}
		return true;
	},

	enableDragMove: function(widget){
		this.getEditor().movingWidget = true;
		var ds = widget.__ds && widget.__ds[this.declaredClass];
		if(!ds){
			var node = widget.isGfxWidget ? widget.group.rawNode : widget.domNode;
			if(this._dragGhost){
				ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragGhostMoveSource(node, "component", widget); //$NON-NLS-1$
			}else{
				ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource(node, "component", widget); //$NON-NLS-1$
			}
			if(!widget.__ds){ widget.__ds = []; }
			widget.__ds[this.declaredClass] = ds;
			ds.offsetParent = this.getEditor().domNode;
			if(widget.isGfxWidget){
				ds.positioning = "absolute"; //$NON-NLS-1$
				ds.useCurrentPositioning = false; // always absolute
				ds.refNode = widget.domNode; // FF cannot get dimension info from an SVG node
				ds.setPos = function(top, left){
					var widget = this.data;
					widget.doRefresh = false;
					widget.setTop(top);
					widget.setLeft(left);
					widget.doRefresh = true;
				};
			}else{
				ds.targetShouldShowCaret = true;
			}
		}else{
			ds.reregister();
		}
		this._connectors = [];
		this._connectors.push(dojo.connect(ds, "onDragStart", this, "onDragStart")); //$NON-NLS-1$ //$NON-NLS-2$
		this._connectors.push(dojo.connect(ds, "onDragEnd", this, "onDragEnd")); //$NON-NLS-1$ //$NON-NLS-2$
	},

	disableDragMove: function(widget){
		this.getEditor().movingWidget = false;
		widget.__ds[this.declaredClass].unregister();
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	onDragStart: function(e){
		if(!this._dragGhost){
			this.getEditor().removeFocus();
		}
		this._connectors.push(dojo.connect(com.ibm.relm.explorer.web.base.internal.dnd.dragManager, "onMouseMove", this, "onDragMove")); //$NON-NLS-1$ //$NON-NLS-2$
		var widget = this._draggingWidget;
		if(widget && widget.hlp_onWidgetDragStart){
			widget.hlp_onWidgetDragStart(e);
		}
	},

	onDragMove: function(e){
		var ds = com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDragSource;
		var y = ds.dragObject.offsetTop;
		var x = ds.dragObject.offsetLeft;
		if(!this._allowNegetivePosition){
			if(x < 0) x = 0;
			if(y < 0) y = 0;
		}
		if(this.snapObjectsToGridFlag){
			x = this.snap(x);
			y = this.snap(y);
			ds.dragObject.style.top = y + "px"; //$NON-NLS-1$
			ds.dragObject.style.left = x + "px"; //$NON-NLS-1$
		}
		if(this.getEditor().fireMessagePosted){
			this.getEditor().fireMessagePosted("("+x+","+y+")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		var widget = this._draggingWidget;
		if(!this._dragGhost && widget.isGfxWidget){
			widget.updatePosition();
		}
		if(widget && widget.hlp_onWidgetDragMove){
			widget.hlp_onWidgetDragMove(e);
		}
	},

	onDragEnd: function(e){
		var widget = this._draggingWidget;
		if(!widget){ return; }
		var targetWidget = this.getDropTargetWidget(e);
		var handled = false;
		if(targetWidget && targetWidget.componentMoveHandler){
			handled = targetWidget.componentMoveHandler(e, this, widget);
		}
		if(!handled){
			this.componentMoveHandler(e);
		}
		this.disableDragMove(this._draggingWidget);
		this._draggingWidget = null;

		if(widget.hlp_onWidgetDragEnd){
			widget.hlp_onWidgetDragEnd(e);
		}
	},

	componentMoveHandler: function(e){
		var sourceWidget = this._draggingWidget;
		var targetWidget = this.getDropTargetWidget(e) || this.getEditor();
		if(sourceWidget.getParent() != targetWidget // moved to the different level
			&& sourceWidget != targetWidget){ // sourceWidget was dropped on other than itself
			var refWidget = this.findRefWidget(e);
			return this.reparentComponent(e, refWidget);
		}else{
			return this.moveComponent(e);
		}
	},

	moveComponent: function(e){
		var widget = this._draggingWidget;
		var command = this.createMoveComponentCommand(e, widget);
		this.getEditor().executeCommand(command);
		this.getEditor().setFocus(widget);
		return true;
	},

	createMoveComponentCommand: function(e, widget){
		var ds = e.dragSource;
		var zoom = this.getEditor().group.getTransform().xx;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var x = ds.dragStartCssPosition.x + dx/zoom;
		var y = ds.dragStartCssPosition.y + dy/zoom;
		if(!this._allowNegetivePosition){
			if(x < 0) x = 0;
			if(y < 0) y = 0;
		}
		if(this.getEditor().snapObjectsToGridFlag){
			x = this.getEditor().snap(x);
			y = this.getEditor().snap(y);
		}
		var toDim = {
			top: y + "px", //$NON-NLS-1$
			left: x + "px" //$NON-NLS-1$
		};
		var fromDim;
		if(!this._dragGhost){
			fromDim = {
				top: ds.dragStartCssPosition.y + "px", //$NON-NLS-1$
				left: ds.dragStartCssPosition.x + "px" //$NON-NLS-1$
			};
		}
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		if(ds.positioning && dojo.style(widget.domNode, "position") != ds.positioning){ //$NON-NLS-1$
			command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning)); //$NON-NLS-1$
		}
		command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.MoveCommand(widget, toDim, fromDim));
		return command;
	}
});
