/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.ReconnectTracker"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager"); //$NON-NLS-1$

// summary:
//		A DragTracker for dragging a connection endpoint to reconnect.

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.ReconnectTracker", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	sourceWidget: null,
	targetWidget: null,
	lineWidget: null,
	_operationStarted: false,

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	_draggingKnob: null,
	_oppositeKnob: null,
	_knobPosition: null, // 0: source, 1: target
	_idx0: -1, // index of the dragging end point
	_idx1: -1, // index of the opposite end point
	_draggingSideAnchor: null,
	_oppositeSideAnchor: null,
	_draggingSideWidget: null,
	_oppositeSideWidget: null,

	handleMouseDown: function(e){
		var line = dijit.getEnclosingWidget(e.target); // if found, maybe a decoration has been clicked.
		if(!line.isGfxLineWidget){
			line = this.getEditor().focusWidgets[0];
		}
		if(!line.isGfxLineWidget){ return; }
		line.moveToFront();
		this.lineWidget = line;
		this._draggingKnob = this.getEndpointKnobByElement(e.target);
		this._oppositeKnob = (this._draggingKnob == line.__connectionEndpoints[0])
			? line.__connectionEndpoints[1] : line.__connectionEndpoints[0];
		this._knobPosition = this._draggingKnob.knobPosition; // [custom property]
		this._idx0 = (this._knobPosition === 0) ? 0 : line.points.length - 1;
		this._idx1 = (this._knobPosition === 0) ? line.points.length - 1 : 0;
		this._draggingSideAnchor = (this._knobPosition === 0) ?
			line.sourceAnchorObj : line.targetAnchorObj;
		this._oppositeSideAnchor = (this._knobPosition === 0) ?
			line.targetAnchorObj : line.sourceAnchorObj;
		this._draggingSideWidget = this._draggingSideAnchor.owner;
		this._oppositeSideWidget = this._oppositeSideAnchor.owner;
		this._startDir = line.startDir;
		this._endDir = line.endDir;
		this.sourceConnectionAnchor = (this._knobPosition === 0) ? line.sourceAnchorObj : null;
		this.targetConnectionAnchor = (this._knobPosition === 0) ? null : line.targetAnchorObj;
		var sw = this.sourceWidget = line.sourceAnchorObj.owner;
		var tw = this.targetWidget = line.targetAnchorObj.owner;
		var lw = this.lineWidget;

		var props = {
			e: e,
			srcWidget: sw,
			tgtWidget: tw,
			curWidget: this._draggingSideWidget,
			srcAnchor: lw.sourceAnchorObj,
			tgtAnchor: lw.targetAnchorObj,
			lineWidget: lw
		};
		if(sw.onSourceReconnectionStart){
			sw.onSourceReconnectionStart(props); //L#121
		}
		if(tw.onTargetReconnectionStart){
			tw.onTargetReconnectionStart(props); //L#221
		}
		if(lw.onReconnectionStart){
			lw.onReconnectionStart(props); //L#321
		}
	},

	handleDragStart: function(e){
		this._operationStarted = true;
		this.getEditor().domNode.style.cursor = "crosshair"; //$NON-NLS-1$
		this._dt = this._oppositeSideWidget.hlp_getConnectionTarget(e.target);
		if(this._dt){
			this._dt.disabled = true;
		}
		this.getEditor().removeFocusFrame(this.lineWidget); // to remove the shadow frame
	},

	_findAnchor: function(targetWidget, clickedElement){
		if(!targetWidget){ return null; }
		if(this._knobPosition === 0){
			return com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager.getSourceConnectionAnchor(targetWidget, clickedElement);
		}else if(this._knobPosition == 1){
			return com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager.getTargetConnectionAnchor(targetWidget, clickedElement);
		}
		return null;
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e){
		e.preventDefault(); // this disables selection
		if(!this._operationStarted){ return; } // drag is not started yet or perhaps was canceled with ESC

		var sw = this.sourceWidget;
		var tw = this.targetWidget;
		var lw = this.lineWidget;
		var props = {
			e: e,
			srcWidget: sw,
			tgtWidget: tw,
			curWidget: this._draggingSideWidget,
			srcAnchor: lw.sourceAnchorObj,
			tgtAnchor: lw.targetAnchorObj,
			lineWidget: lw
		};
		if(sw.onSourceConnectionMove){
			sw.onSourceConnectionMove(props);
		}
		if(tw.onTargetConnectionMove){
			tw.onTargetConnectionMove(props);
		}

		var widget = dijit.getEnclosingWidget(e.target); // a widget to which e.target belongs
		var anchor;
		if(widget && widget.isGfxLineWidget){ // mouse is on the line or decoration
			anchor = (this._knobPosition === 0) ?
				this.sourceConnectionAnchor : this.targetConnectionAnchor;
		}else{
			var dt = com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDropTarget;
			var dropTargetWidget = dt && dt.data; // drop target widget (e.g. EditArea)
			anchor = this._findAnchor(dropTargetWidget, e.target);
		}
		if(anchor && (this.sourceConnectionAnchor == anchor
					|| this.targetConnectionAnchor == anchor)){ return; } // already anchored
		if(this.sourceConnectionAnchor){ // release the old source anchor
			if(sw.onSourceConnectionDetach){
				if(sw.onSourceConnectionDetach(props) === false){ return false; } //L#122
			}
			if(tw.onTargetConnectionDetach){
				if(tw.onTargetConnectionDetach(props) === false){ return false; } //L#222
			}
			if(lw.onConnectionDetach){
				if(lw.onConnectionDetach(props) === false){ return false; } //L#322
			}
			this.sourceConnectionAnchor = null;
			lw.setSourceAnchor(null);
		}
		if(this.targetConnectionAnchor){ // release the old target anchor
			if(sw.onSourceConnectionDetach){
				if(sw.onSourceConnectionDetach(props) === false){ return false; } //L#113
			}
			if(tw.onTargetConnectionDetach){
				if(tw.onTargetConnectionDetach(props) === false){ return false; } //L#114
			}
			this.targetConnectionAnchor = null;
			lw.setTargetAnchor(null);
		}
		if(anchor){ // attach
			var nw = anchor.owner; // new target widget
			props = {
				e: e,
				srcWidget: sw,
				tgtWidget: tw,
				curWidget: this._draggingSideWidget,
				srcAnchor: lw.sourceAnchorObj,
				tgtAnchor: lw.targetAnchorObj,
				lineWidget: lw,
				newWidget: nw
			};
			if(sw.onSourceConnectionAttach){
				sw.onSourceConnectionAttach(props); //L#123
			}
			if(nw.onTargetConnectionAttach){
				nw.onTargetConnectionAttach(props); //L#223
			}
			if(lw.onConnectionAttach){
				lw.onConnectionAttach(props); //L#323
			}

			if(this._knobPosition === 0){	// dragging the source-side knob
				// store the clicked position
				var pos = dojo.position(sw.domNode);
				sw._clickedPos = {
					x: e.clientX - pos.x,
					y: e.clientY - pos.y
				};

				this.sourceConnectionAnchor = anchor;
				lw.setSourceAnchor(anchor);
			}else{							// dragging the target-side knob
				// store the clicked position
				var pos = dojo.position(tw.domNode);
				tw._clickedPos = {
					x: e.clientX - pos.x,
					y: e.clientY - pos.y
				};

				this.targetConnectionAnchor = anchor;
				lw.setTargetAnchor(anchor);
			}
		}else{ // move
			// See also ConnectionDragCreationTool#handleMouseMove
			var info = lw.hlp_getComponentInfo();
			var offsetH = info && info.draggingLineOffsetH || 5;
			var offsetV = info && info.draggingLineOffsetV || 5;
			var pos = dojo.position(this.getEditor().domNode, false);
			var x = e.pageX - pos.x + this.getEditor().domNode.scrollLeft - offsetH;
			var y = e.pageY - pos.y + this.getEditor().domNode.scrollTop - offsetV;
			lw.setPoint({x:x, y:y}, this._idx0);
		}

		// Update the connection (and decorations if any)
		// There are two anchors, but here we just need to update either of them.
		if(lw.sourceAnchorObj){
			lw.sourceAnchorObj.anchorMoved();
		}else if(lw.targetAnchorObj){
			lw.targetAnchorObj.anchorMoved();
		}
		if(this.getEditor().addConnectionEndpoints){
			this.getEditor().addConnectionEndpoints(lw);
		}
		return true;
	},

	handleMouseUp: function(e){
		if(!this._operationStarted){ return; } // drag has not been started yet or perhaps was canceled with ESC

		var sw = this.sourceWidget;
		var tw = this.targetWidget;
		var lw = this.lineWidget;
		var props = {
			e: e,
			srcWidget: sw,
			srcAnchor: this.sourceConnectionAnchor,
			tgtWidget: tw,
			tgtAnchor: this.targetConnectionAnchor,
			curWidget: this._draggingSideWidget,
			lineWidget: lw
		};

		// Restore the anchor before invoking SetPropertyValueCommand so that undo works.
		// Do not use setSourceAnchor/setTargetAnchor, because we don't want anchorMoved to be called.
		lw.setAnchor(this._draggingSideAnchor, (this._knobPosition === 0) ? "source" : "target", true); //$NON-NLS-1$ //$NON-NLS-2$

		// Restore startDir and endDir before invoking SetPropertyValueCommand so that undo works.
		var newStartDir, newEndDir;
		if(lw.startDir != this._startDir){
			newStartDir = lw.startDir;
			lw.startDir = this._startDir;
		}
		if(lw.endDir != this._endDir){
			newEndDir = lw.endDir;
			lw.endDir = this._endDir;
		}

		var val, cmds = [], nw = this._draggingSideWidget;
		var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
		if(this.sourceConnectionAnchor && this.sourceConnectionAnchor != this._draggingSideAnchor){
			nw = this.sourceConnectionAnchor.owner; // new source widget
			val = this.sourceConnectionAnchor.owner.id+":"+this.sourceConnectionAnchor.anchorName; //$NON-NLS-1$
			cmds.push(new ec.SetPropertyValueCommand(lw, "sourceAnchor", val)); //$NON-NLS-1$
		}else if(this.targetConnectionAnchor && this.targetConnectionAnchor != this._draggingSideAnchor){
			nw = this.targetConnectionAnchor.owner; // new target widget
			val = nw.id+":"+this.targetConnectionAnchor.anchorName; //$NON-NLS-1$
			cmds.push(new ec.SetPropertyValueCommand(lw, "targetAnchor", val)); //$NON-NLS-1$
		}

		if(newStartDir){
			cmds.push(new ec.SetPropertyValueCommand(lw, "startDir", newStartDir)); //$NON-NLS-1$
		}else if(newEndDir){
			cmds.push(new ec.SetPropertyValueCommand(lw, "endDir", newEndDir)); //$NON-NLS-1$
		}


		if(cmds.length > 0){
			var cc = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
			cc.append(new ec.UpdateCommand(lw, "anchorMoved")); //$NON-NLS-1$
			for(var i = 0; i < cmds.length; i++){
				cc.append(cmds[i]);
			}
			cc.append(new ec.UpdateCommand(lw, "anchorMoved")); //$NON-NLS-1$

			this.getEditor().executeCommand(cc);

			if(this._knobPosition == 0){ // 0: source, 1: target
				if(nw.onSourceConnected){
					nw.onSourceConnected(props); //L#125 (reconnection at source-side)
				}
				if(tw.onTargetReconnected){
					tw.onTargetReconnected(props); //L#225 (reconnection at source-side)
				}
			}else{
				if(sw.onSourceReconnected){
					sw.onSourceReconnected(props); //L#124
				}
				if(nw.onTargetConnected){
					nw.onTargetConnected(props); //L#224
				}
			}
			if(lw.onConnected){
				lw.onReconnected(props); //L#324
			}
		}

		this.stopReconnecting(e);

		if(sw.onSourceConnectionEnd){
			sw.onSourceConnectionEnd(props); //L#126
		}
		if(tw && tw.onTargetConnectionEnd){
			tw.onTargetConnectionEnd(props); //L#226
		}
		if(nw && nw.onTargetConnectionEnd){
			nw.onTargetConnectionEnd(props); //L#226
		}
		if(lw && lw.onConnectionEnd){
			lw.onConnectionEnd(props); //L#326
		}
	},

	handleKeyDown: function(e){
		if(e.keyCode == 27){ // ESC
			this.stopReconnecting(e, true);
		}
	},

	stopReconnecting: function(e, /*Boolean*/restore){
		if(!this._operationStarted){ return; } // already canceled
		this._operationStarted = false;
		this.getEditor().domNode.style.cursor = ""; //$NON-NLS-1$
		if(this._dt){
			this._dt.disabled = false;
		}

		if(!this.sourceConnectionAnchor && !this.targetConnectionAnchor){
			this.onDropAborted(e, restore);
		}else{
			this.lineWidget.anchorMoved();
		}
	},

	onDropAborted: function(e, /*Boolean*/restore){
		var sw = this.sourceWidget;
		var tw = this.targetWidget;
		var lw = this.lineWidget;
		if(!restore){ // disconnect from the nodes
			// remove the connection
			this.getEditor().removeFocus();
			var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.RemoveChildCommand(lw);
			this.getEditor().executeCommand(command);
			var props = {
				e: e,
				srcWidget: sw,
				tgtWidget: tw,
				curWidget: this._draggingSideWidget,
				srcAnchor: lw.sourceAnchorObj,
				tgtAnchor: lw.targetAnchorObj,
				lineWidget: lw
			};
			if(sw.onSourceDisconnected){
				sw.onSourceDisconnected(props); //L#127
			}
			if(tw.onTargetDisconnected){
				tw.onTargetDisconnected(props); //L#227
			}
			if(lw.onDisconnected){
				lw.onDisconnected(props); //L#327
			}
		}else{ // restore the connection
			// recover the dragging end point
			if(this._knobPosition === 0){	// dragging the source-side knob
				lw.setSourceAnchor(this._draggingSideAnchor);
			}else{							// dragging the target-side knob
				lw.setTargetAnchor(this._draggingSideAnchor);
			}
			this.getEditor().setFocusFrame(lw); // to reset the endpoints position
		}
	},

	getEndpointKnobByElement: function(elem){
		while(elem){
			if(elem.nodeType != 1){ return null; }
			if(elem.endpointKnobType){
				return elem;
			}
			elem = elem.parentNode;
			if(!elem || elem.tagName == "BODY"){ //$NON-NLS-1$
				return null;
			}
		}
		return null;
	}
});
