/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.canvas.tool.ResizeTracker"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragMoveSource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

// summary:
//		A DragTracker for dragging a resize handle.

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.canvas.tool.ResizeTracker", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.AbstractTool,
{
	_rubberBand: null,

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	stopResizing: function(){
		var rubberBand = this.getEditor()._rubberBand;
		if(rubberBand.parentNode != this.getEditor().domNode){ return; } // already canceled

		this.getEditor().domNode.removeChild(rubberBand);
		this.getEditor().domNode.style.cursor = ""; //$NON-NLS-1$
	},

	// ======== Event Handlers ====================================
	handleMouseDown: function(e){
	},

	handleDragStart: function(e){
		var movingKnob = com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDragSource.domNode;
		var widget = this.getEditor().focusWidgets[0];

		var rubberBand = this.getEditor()._rubberBand;
		if(!rubberBand){
			rubberBand = this.getEditor()._rubberBand = dojo.doc.createElement("DIV"); //$NON-NLS-1$
			rubberBand.style.fontSize = "1px"; //$NON-NLS-1$
			rubberBand.style.position = "absolute"; //$NON-NLS-1$
			rubberBand.style.border = "1px dashed black"; //$NON-NLS-1$
		}
		this.getEditor().domNode.appendChild(rubberBand);

		var dim = widget.__dim = widget.hlp_getStyleDimensions(widget, this.getEditor());
		rubberBand.style.top = dim.top + "px"; //$NON-NLS-1$
		rubberBand.style.left = dim.left + "px"; //$NON-NLS-1$
		rubberBand.style.width = dim.width + "px"; //$NON-NLS-1$
		rubberBand.style.height = dim.height + "px"; //$NON-NLS-1$

		this.getEditor().domNode.style.cursor = "crosshair"; //$NON-NLS-1$
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e){
		var rubberBand = this.getEditor()._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this.getEditor().domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var movingKnob = com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDragSource.domNode;
		var widget = this.getEditor().focusWidgets[0];
		var dim = widget.__dim;
		var rubberBand = this.getEditor()._rubberBand;
		var knobPosition = movingKnob.knobPosition; // [custom property]
		var info = widget.hlp_getComponentInfo();

		var mouseX = e.pageX - com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDragSource.parentCoords.x;
		var mouseY = e.pageY - com.ibm.relm.explorer.web.base.internal.dnd.dragManager.currentDragSource.parentCoords.y;
		if(dojo.isMozilla){
			mouseX -= 2;
			mouseY -= 2;
		}

		var keep = info.resizable.keepAspectRatio;
		var dirs = widget.__resizeDirections;
		if(!e.shiftKey && !e.ctrlKey && !keep){ // resize freely
			if(dirs & this._NORTH
				&& (knobPosition == 0 || knobPosition == 1 || knobPosition == 2)){ // NORTH
				var h = dim.top - mouseY + dim.height;
				if(h > 0){
					rubberBand.style.top = this.getEditor().snap(mouseY) + "px"; //$NON-NLS-1$
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._SOUTH
				&& (knobPosition == 4 || knobPosition == 5 || knobPosition == 6)){ // SOUTH
				var h = mouseY - dim.top;
				if(h > 0){
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}
	
			if(dirs & this._EAST
				&& (knobPosition == 2 || knobPosition == 3 || knobPosition == 4)){ // EAST
				var w = mouseX - dim.left;
				if(w > 0){
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._WEST
				&& (knobPosition == 0 || knobPosition == 6 || knobPosition == 7)){ // WEST
				var w = dim.left - mouseX + dim.width;
				if(w > 0){
					rubberBand.style.left = this.getEditor().snap(mouseX) + "px"; //$NON-NLS-1$
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}
		}else if(e.shiftKey && !e.ctrlKey || keep){ // resize keeping the aspect ratio
			var x, y;
			if(dirs & this._NORTH && knobPosition == 1 && !keep){ // NORTH
				var h = dim.top - mouseY + dim.height;
				if(h > 0){
					rubberBand.style.top = this.getEditor().snap(mouseY) + "px"; //$NON-NLS-1$
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._EAST && knobPosition == 3 && !keep){ // EAST
				var w = mouseX - dim.left;
				if(w > 0){
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._SOUTH && knobPosition == 5 && !keep){ // SOUTH
				var h = mouseY - dim.top;
				if(h > 0){
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._WEST && knobPosition == 7 && !keep){ // WEST
				var w = dim.left - mouseX + dim.width;
				if(w > 0){
					rubberBand.style.left = this.getEditor().snap(mouseX) + "px"; //$NON-NLS-1$
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}else if((dirs & this._NORTH) && (dirs & this._EAST) && knobPosition == 2){ // top-right
				var w = mouseX - dim.left;
				var h = dim.top - mouseY + dim.height;
				if(w <= 0 || h <= 0){ return; }
				var ratioX = w / dim.width;
				var ratioY = h / dim.height;
				if(ratioX > ratioY){
					h = Math.round(dim.height * ratioX);
					y = dim.top - (h - dim.height);
				}else{
					w = Math.round(dim.width * ratioY);
					y = mouseY;
				}
				rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				rubberBand.style.top = this.getEditor().snap(y) + "px"; //$NON-NLS-1$
			}else if((dirs & this._EAST) && (dirs & this._SOUTH) && knobPosition == 4){ // bottom-right
				var w = mouseX - dim.left;
				var h = mouseY - dim.top;
				if(w <= 0 || h <= 0){ return; }
				var ratioX = w / dim.width;
				var ratioY = h / dim.height;
				if(ratioX > ratioY){
					h = Math.round(dim.height * ratioX);
					y = dim.top - (h - dim.height);
				}else{
					w = Math.round(dim.width * ratioY);
					y = mouseY;
				}
				rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
			}else if((dirs & this._SOUTH) && (dirs & this._WEST) && knobPosition == 6){ // bottom-left
				var w = dim.left - mouseX + dim.width;
				var h = mouseY - dim.top;
				if(w <= 0 || h <= 0){ return; }
				var ratioX = w / dim.width;
				var ratioY = h / dim.height;
				if(ratioX > ratioY){
					w = dim.left - mouseX + dim.width;
					h = Math.round(dim.height * ratioX);
					x = mouseX;
				}else{
					w = Math.round(dim.width * ratioY);
					h = mouseY - dim.top;
					x = dim.left - (w - dim.width);
				}
				rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				rubberBand.style.left = this.getEditor().snap(x) + "px"; //$NON-NLS-1$
			}else if((dirs & this._WEST) && (dirs & this._NORTH) && knobPosition == 0){ // top-left
				var w = dim.left - mouseX + dim.width;
				var h = dim.top - mouseY + dim.height;
				if(w <= 0 || h <= 0){ return; }
				var ratioX = w / dim.width;
				var ratioY = h / dim.height;
				if(ratioX > ratioY){
					h = Math.round(dim.height * ratioX);
					y = dim.top - (h - dim.height);
					x = mouseX;
				}else{
					w = Math.round(dim.width * ratioY);
					y = mouseY;
					x = dim.left - (w - dim.width);
				}
				rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				rubberBand.style.left = this.getEditor().snap(x) + "px"; //$NON-NLS-1$
				rubberBand.style.top = this.getEditor().snap(y) + "px"; //$NON-NLS-1$
			}
		}else if(!e.shiftKey && e.ctrlKey){ // resize keeping the center
			var x, y;
			if(dirs & this._NORTH
				&& (knobPosition == 0 || knobPosition == 1 || knobPosition == 2)){ // NORTH
				var h = (dim.top - mouseY) * 2 + dim.height;
				if(h > 0){
					rubberBand.style.top = this.getEditor().snap(mouseY) + "px"; //$NON-NLS-1$
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._SOUTH
				&& (knobPosition == 4 || knobPosition == 5 || knobPosition == 6)){ // SOUTH
				var h = (mouseY - dim.top - dim.height) * 2 + dim.height;
				y = dim.top - (mouseY - dim.top - dim.height);
				if(h > 0){
					rubberBand.style.top = this.getEditor().snap(y) + "px"; //$NON-NLS-1$
					rubberBand.style.height = this.getEditor().snap(h) + "px"; //$NON-NLS-1$
				}
			}

			if(dirs & this._EAST
				&& (knobPosition == 2 || knobPosition == 3 || knobPosition == 4)){ // EAST
				var w = (mouseX - dim.left - dim.width) * 2 + dim.width;
				x = dim.left - (mouseX - dim.left - dim.width);
				if(w > 0){
					rubberBand.style.left = this.getEditor().snap(x) + "px"; //$NON-NLS-1$
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}else if(dirs & this._WEST
				&& (knobPosition == 0 || knobPosition == 6 || knobPosition == 7)){ // WEST
				var w = (dim.left - mouseX) * 2 + dim.width;
				if(w > 0){
					rubberBand.style.left = this.getEditor().snap(mouseX) + "px"; //$NON-NLS-1$
					rubberBand.style.width = this.getEditor().snap(w) + "px"; //$NON-NLS-1$
				}
			}
		}else if(e.shiftKey && e.ctrlKey){ // resize keeping the aspect ratio and the center
		}
	},

	handleMouseUp: function(e){
		var rubberBand = this.getEditor()._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this.getEditor().domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var widget = this.getEditor().focusWidgets[0];
		var parentDim = com.ibm.relm.explorer.web.base.internal.util.getDimensions(widget.getParent().domNode);
		var editAreaDim = com.ibm.relm.explorer.web.base.internal.util.getDimensions(this.getEditor().domNode);
		var toBox = dojo.contentBox(rubberBand);
		var toDim = {
			top: (rubberBand.offsetTop - parentDim.top + editAreaDim.top) + "px", //$NON-NLS-1$
			left: (rubberBand.offsetLeft - parentDim.left + editAreaDim.left) + "px", //$NON-NLS-1$
			width: toBox.w + "px", //$NON-NLS-1$
			height: toBox.h + "px" //$NON-NLS-1$
		};

		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.ResizeCommand(widget, toDim);
		this.getEditor().executeCommand(command);

		this.stopResizing();
		this.getEditor().fireSelectionChanged(widget);
	},

	handleKeyDown: function(e){
		if(e.keyCode == 27){ // ESC
			this.stopResizing();
		}
	}
});
