/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.base.internal.dnd.DragManager", null, { //$NON-NLS-1$
	disabled: false, // disable all D&D operations
	currentDragSource: null,
	currentDropTarget: null,
	dropAcceptable: false,
	dropTargets: null,
	dragTriggered: false,
	currentDomElement: null,
	_connectors: null,
	_moveConnector: null,

	_ABSOLUTE_THRESHOLD: 3,
	_STATIC_THRESHOLD: 40,

	constructor: function(){
		this.dropTargets = [];
		this._connectors = [];
		this._connectors.push(dojo.connect(dojo.doc, "onkeydown", this, "onKeyDown")); //$NON-NLS-1$ //$NON-NLS-2$
		this._connectors.push(dojo.connect(dojo.doc, "onmouseover", this, "onMouseOver")); //$NON-NLS-1$ //$NON-NLS-2$
		this._connectors.push(dojo.connect(dojo.doc, "onmouseout", this, "onMouseOut")); //$NON-NLS-1$ //$NON-NLS-2$
		this._connectors.push(dojo.connect(dojo.doc, "onmousedown", this, "onMouseDown")); //$NON-NLS-1$ //$NON-NLS-2$
		this._connectors.push(dojo.connect(dojo.doc, "onmouseup", this, "onMouseUp")); //$NON-NLS-1$ //$NON-NLS-2$
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	onKeyDown: function(e){
		if(e.keyCode == 27){ // ESC
			if(this.currentDragSource){
				this.dropAcceptable = false;
				this.onMouseUp(e);
			}
		}
	},

	onMouseDown: function(e, dragSource){
		if(this.disabled){ return; }
		if(this.dragTriggered){ return; } // still dragging
		if(this._moveConnector){ return; }
//		if(dojo.isIE){
//			e.pageX -= 2;
//			e.pageY -= 2;
//		}
		var ds = this.currentDragSource = dragSource ? dragSource : this.findDragSource(e);
		if(!ds){ return; }
		ds.onDragDown(e);
		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			var dt = this.dropTargets[i];
			if(!dt.disabled){
				dt.onDragDown(e);
			}
		}

		e.preventDefault();
		this._moveConnector = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	onMouseMove: function(e){
		e.preventDefault(); // this disables selection
		if(dojo.isIE){
			e.pageX -= 2;
			e.pageY -= 2;
		}
		var ds = e.dragSource = this.currentDragSource;
		if(!this.dragTriggered){
			var dx = e.pageX - ds.pageX;
			var dy = e.pageY - ds.pageY;
			var d = Math.sqrt(dx*dx + dy*dy);
			if(ds.positioning == "absolute"){ //$NON-NLS-1$
				if(d < this._ABSOLUTE_THRESHOLD){ return; }
			}else{
				if(d < this._STATIC_THRESHOLD){ return; }
			}

			ds.onDragStart(e);
			for(var i = 0, len = this.dropTargets.length; i < len; i++){
				var dt = this.dropTargets[i];
				if(!dt.disabled){
					e.dropTarget = dt;
					dt.onDragStart(e);
				}
			}

			this.dragTriggered = true;

			// we need a mouseover event here to setup drop target
			this.onMouseOver(e);
		}


		ds.onDragMove(e);

		var dt = this.currentDropTarget;
		if(dt){
			if(this.dropAcceptable){
				e.dropTarget = dt;
				dt.onDragMove(e, this.dragObjects);
			}
		}
		else{
			this.dropAcceptable = false;
		}
	},

	onMouseOver: function(e){
		this.currentDomElement = e.target;

		if(!this.dragTriggered){ return; }

		// find acceptable drop target
		var ds = this.currentDragSource;
		if(!ds){ return; }
		e.dragSource = ds;
		var dt = this.findDropTarget(this.currentDomElement, ds);
		if(dt && ds.domNode == dt.domNode){ return; } // source and target are the same
		if(dt != this.currentDropTarget){
			if(this.currentDropTarget){
				e.dropTarget = this.currentDropTarget;
				this.currentDropTarget.onDragOut(e);
			}
			if(dt){
				e.dropTarget = dt;
				this.dropAcceptable = dt.onDragOver(e);
			}
		}
		this.currentDropTarget = dt;

		/*
		// switch the positioning mode
		if(dt && ds && dt.nodeUnderMouse == ds.domNode){
			if(ds.positioning == "absolute"){
				ds.positioning = "static";
			}else{
				ds.positioning = "absolute";
			}
		}
		*/
	},

	onMouseOut: function(e){
	},

	onMouseUp: function(e){
		if(this.ignoreMouseUp){ return; }
		dojo.disconnect(this._moveConnector);
		this._moveConnector = null;
		if(!this.dragTriggered){ // we are not dragging
			this._resetStatus();
			return;
		}

		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			var _dt = this.dropTargets[i];
			if(!_dt.disabled){
				e.dropTarget = _dt;
				_dt.onDragEnd(e);
			}
		}

		var ds = this.currentDragSource;
		var dt = this.currentDropTarget;
		e.dragSource = ds;
		e.dropTarget = dt;

		if(!ds){
			this._resetStatus();
			return;
		}

		if(dt){
			dt.onDropStart(e);
			if(this.dropAcceptable){
				var ret = dt.onDrop(e);
				dt.onDragOut(e);
				if(!e.dragStatus){
					e.dragStatus = ret ? "dropSuccess" : "dropFailure"; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}else{
				dt.onDragOut(e);
				e.dragStatus = "dropFailure"; //$NON-NLS-1$
			}
		}
		else{
			e.dragStatus = "dropFailure"; //$NON-NLS-1$
		}

		ds.onDragEnd(e);
		if(dt){
			dt.onDropEnd(e);
		}
		this._resetStatus();
	},

	_resetStatus: function(){
		this.currentDropTarget = null;
		this.currentDragSource = null;
		this.dropAcceptable = false;
		this.dragTriggered = false;
		this.currentDomElement = null;
	},

	findDragSource: function(e){
		for(var node = e.target; node && node != dojo.doc.body; node = node.parentNode){
			if(node.dragSource && !node.dragSource.disabled){
				return node.dragSource;
			}
		}
		return null;
	},

	findDropTarget: function(domElement, ds){
		var dt = null;
		for(var elem = domElement, prev = null; elem && elem.tagName != "BODY";){ //$NON-NLS-1$
			if(elem.dropTargets){ // cf. com.ibm.relm.explorer.web.base.internal.dnd.DropTarget
				for(var i = 0; i < elem.dropTargets.length; i++){
					var obj = elem.dropTargets[i];
					if(!obj.disabled && obj.accepts(ds)){
						dt = obj; // found the DropTarget
						dt.nodeUnderMouse = prev; // direct child of the drop target
						return dt;
					}
				}
			}
			if(elem.dragSource && !elem.dragSource.disabled){
				prev = elem;
			}
			if (!elem.getAttribute)	return null;
			var id = elem.getAttribute("_widgetId"); // widgetId might be copied to the root of a shape. //$NON-NLS-1$
			elem = id ? dijit.byId(id).domNode : elem.parentNode;
		}
		return null;
	},

	registerDropTarget: function(dt){
		this.dropTargets.push(dt);
	},

	unregisterDropTarget: function(dt){
		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			if(this.dropTargets[i] == dt){
				this.dropTargets.splice(i, 1);
				break;
			}
		}
	}
});

com.ibm.relm.explorer.web.base.internal.dnd.dragManager = new com.ibm.relm.explorer.web.base.internal.dnd.DragManager();
