/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.dnd.DragSource"); //$NON-NLS-1$
dojo.require("dojo.fx"); //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.base.internal.dnd.DragSource", null, { //$NON-NLS-1$
	disabled: false, // disable this drag source
	dragObject: null,
	dragClone: null,
	offsetParent: null,
	offsetParentCoords: null,
	parentCoords: null,
	refNode: null, // get information (position,top,left) from this node instead of domNode
	positioning: null, // CSS position property of this drag source. "absolute" or "static"
	useCurrentPositioning: true,
	targetShouldShowCaret: true,
	draggingMousePosition: "TopLeft", // "TopLeft" or "ClickedPosition" //$NON-NLS-1$
	data: null, // any user data associated with this ds. (widget, etc.)
	operation: "drop", // default operation //$NON-NLS-1$

	constructor: function(node, type, data){
		this.data = data;
		var node = dojo.byId(node);
		node.dragSource = this; // [custom property]
		this.domNode = node;
		this.dragObject = node;
		this.refNode = node;
		this.type = type;
		this.offsetParent = dojo.doc.body;
	},

	onDragDown: function(e){
		if(this.useCurrentPositioning){
			this.positioning = dojo.style(this.refNode, "position"); //$NON-NLS-1$
		}
		this.storeMouseDownInfo(e);
		this.initDrag(e);
	},

	onDragStart: function(e){
		this.createDragClone(e);
		return this.dragClone;
	},

	onDragMove: function(e){
		if(this.draggingMousePosition == "TopLeft"){ //$NON-NLS-1$
			this.dragClone.style.top = e.pageY + 4 + "px"; //$NON-NLS-1$
			this.dragClone.style.left = e.pageX + 4 + "px"; //$NON-NLS-1$
		}else if(this.draggingMousePosition == "ClickedPosition"){ //$NON-NLS-1$
			this.dragClone.style.top = e.pageY - this.dragOffset.y + "px"; //$NON-NLS-1$
			this.dragClone.style.left = e.pageX - this.dragOffset.x + "px"; //$NON-NLS-1$
		}
		if(this.dragClone.style.display == "none"){ //$NON-NLS-1$
			this.dragClone.style.display = ""; //$NON-NLS-1$
		}
	},

	onDragEnd: function(e){
		if(e.dragStatus == "dropSuccess"){ //$NON-NLS-1$
			this.removeClone();
		}else if(e.dragStatus == "dropFailure"){ //$NON-NLS-1$
			this.returnClone();
		}else{
			console.warn("DragSource#onDragEnd: internal error: e.dragStatus is not set"); //$NON-NLS-1$
		}
		this.cleanupDrag();
	},

	isRegistered: function(){
		return !!this.domNode.dragSource;
	},

	unregister: function(){
		if(!this.domNode.dragSource){ console.warn("already unregistered"); return; } // @@ already unregistered //$NON-NLS-1$
		this.domNode.dragSource = null; // [custom property]
		this.cleanupDrag();
	},

	reregister: function(){
		if(this.domNode.dragSource){ console.warn("already registered"); return; } // @@ already registered //$NON-NLS-1$
		this.domNode.dragSource = this; // [custom property]
		this.initDrag();
	},


	storeMouseDownInfo: function(e){
		if(dojo.isIE < 8){
			// take border width into account
			this.layerX = e.layerX + dojo._getBorderExtents(this.domNode).l; //TODO: is this really correct?
			this.layerY = e.layerY + dojo._getBorderExtents(this.domNode).t; //TODO: is this really correct?
		}else{
			this.layerX = e.layerX - 1;
			this.layerY = e.layerY - 1;
		}
		this.pageX = e.pageX;
		this.pageY = e.pageY;
		this.scrollOffset = dojo._docScroll();
		this.dragStartPosition = dojo.position(this.refNode, true); // {x:#, y:#}
		this.dragStartCssPosition = {
			y: this.refNode.offsetTop,
			x: this.refNode.offsetLeft
		};
		this.dragOffset = {
			y: e.pageY - this.dragStartPosition.y,
			x: e.pageX - this.dragStartPosition.x
		};
		this.offsetParentCoords = dojo.position(this.offsetParent, true); // {x:#, y:#}

		var parentPosition = dojo.position(this.domNode.parentNode, true);
		this.parentCoords = {
			y: parentPosition.y - this.offsetParentCoords.y,
			x: parentPosition.x - this.offsetParentCoords.x
		};

		// Users of the DragSource may want to fix the mouse down info
		if(this.fixMouseDownInfo){
			this.fixMouseDownInfo(this);
		}

		var widget = this.data; 
		var top = 0, left = 0;
		var zoom = 1.0;
		for(; widget && widget.group != null; widget = widget.getParent()){
			var transform = widget.group.getTransform();
			if(transform){
				zoom = transform.xx;
				top = top*zoom + transform.dy;
				left = left*zoom + transform.dx;
			}
		}
		this.transformX = left;
		this.transformY = top;
		this.zoom = zoom;
	},

	initDrag: function(){
		if(dojo.isIE){
			this.cleanupDrag();
			this._ieDragStartHandler = dojo.connect(this.domNode, "ondragstart", this.cancelEvent); //$NON-NLS-1$
		}
	},

	cleanupDrag: function(){
		if(dojo.isIE){
			this._ieDragStartHandler && dojo.disconnect(this._ieDragStartHandler);
			delete this._ieDragStartHandler;
		}
	},

	createDragClone: function(e){
		var dragClone = this.dragClone = this.dragObject.cloneNode(true);
		dojo.style(dragClone, 'opacity', 0.5); //$NON-NLS-1$
		dragClone.style.zIndex = 1000;

		var box = dojo.contentBox(this.dragObject);
		dragClone.style.width = box.w + "px"; //$NON-NLS-1$
		dragClone.style.height = box.h + "px"; //$NON-NLS-1$
		dragClone.style.top = this.dragObject.offsetTop + 4 + "px"; //$NON-NLS-1$
		dragClone.style.left = this.dragObject.offsetLeft + 4 + "px"; //$NON-NLS-1$
		dragClone.style.position = "absolute"; //$NON-NLS-1$
		dragClone.style.clear = "both"; //$NON-NLS-1$
		dragClone.style.display = "none"; //$NON-NLS-1$
		this.offsetParent.appendChild(dragClone);
	},

	setDragTarget: function(node){
		this.dragObject = node;
	},

	setRefNode: function(node){
		this.refNode = node;
	},

	cancelEvent: function(e){
		e.stopPropagation();
		e.preventDefault();
	},

	removeClone: function(){
		// Use setTime to ensure that any methods that are connected to onDragEnd
		// can safely access the dragClone before the actual removal is performed.
		setTimeout(dojo.hitch(this, "removeClone_1"), 0); //$NON-NLS-1$
	},

	removeClone_1: function(){
		if(this.dragClone){
			this.dragClone.parentNode.removeChild(this.dragClone);
			this.dragClone = null;
		}
	},

	returnClone: function(){
		var startCoords = dojo.position(this.dragClone, true); // {x, y}
		var endCoords = this.dragStartPosition; // {x, y}
//		var endCoords = {
//			left: this.dragStartPosition.x,
//			top: this.dragStartPosition.y
//		};
		var overrunCoords = {
			x: startCoords.x > endCoords.x ? endCoords.x - 10 : endCoords.x + 10,
			y: startCoords.y > endCoords.y ? endCoords.y - 10 : endCoords.y + 10
		};

		var _this = this;
		dojo.fx.slideTo({ node: this.dragClone, top: overrunCoords.y, left: overrunCoords.x, duration: 400,
			onEnd: function(){
				dojo.fx.slideTo({ node: _this.dragClone, top: endCoords.y, left: endCoords.x, duration: 100,
					onEnd: function(){
						_this.dragClone.parentNode.removeChild(_this.dragClone);
						_this.dragClone = null;
						_this.onCloneReturned();
					}
				}).play();
			}
		}).play();
	},

	onCloneReturned: function(){
	},

	globalChangeCursor: function(cursor){
		var sheet = dojo.doc.styleSheets[0];
		if(!sheet){ return; }
		if(cursor){
			if(this._showingCursor){
				if(this._showingCursor == cursor){ return; } // already showing
				this.globalChangeCursor(null);
			}
			this._showingCursor = cursor;
			if(dojo.isIE){
				this._ruleIndex = sheet.rules.length;
				sheet.addRule("*", "{cursor:"+cursor+" ! important;}", sheet.rules.length); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this._prevCursor = dojo.doc.body.style.cursor;
				dojo.doc.body.style.cursor = null; // workaround to update cursor immediately
			}else {
				this._ruleIndex = sheet.cssRules.length;
				sheet.insertRule("*" + "{cursor:"+cursor+" ! important;}", sheet.cssRules.length); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		}else{
			if(!this._showingCursor){ return; }
			this._showingCursor = null;
			if(dojo.isIE){
				sheet.removeRule(this._ruleIndex);
				dojo.doc.body.style.cursol = this._prevCursor != null ? this._prevCursor : 'auto'; //$NON-NLS-1$
			}else {
				sheet.deleteRule(this._ruleIndex);
			}
		}
	}
});
