/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.dnd.DropTarget"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager"); //$NON-NLS-1$

/*
 * Created by dropHelper or connectionHelper for each component.
 */
dojo.declare("com.ibm.relm.explorer.web.base.internal.dnd.DropTarget", null, { //$NON-NLS-1$
	disabled: false, // disable this drop target
	offsetParent: null,
	showingFeedback: false,
	acceptedTypes: null,
	nodeUnderMouse: null,
	caret: null,
	vertical: false,
	data: null, // any user data associated with this dt. (widget, etc.)

	constructor: function(node, types, data){
		this.data = data;
		var node = this.domNode = dojo.byId(node);

		// one dom node can have multiple DropTargets (ex. for drop and for connect)
		if(!node.dropTargets){
			node.dropTargets = [this]; // [custom property]
		}else{
			node.dropTargets.push(this); // [custom property]
		}

		if(!types){
			this.acceptedTypes = [];
		}else if(typeof(types) == "string"){ //$NON-NLS-1$
			this.acceptedTypes = [types];
		}else{
			this.acceptedTypes = types;
		}
		com.ibm.relm.explorer.web.base.internal.dnd.dragManager.registerDropTarget(this);
		this._isAcceptableMap = {};
	},

	destroy: function() {
		com.ibm.relm.explorer.web.base.internal.dnd.dragManager.unregisterDropTarget(this);
		this.data = null;
		this.caret = null;
		this.offsetParent = null;
		this.inherited(arguments);
	},
	
	// The onmousedown event has fired, but it doesn't mean D&D has started.
	onDragDown: function(e){
	},

	// D&D operation has been started
	onDragStart: function(e){
	},

	onDragOver: function(e){
		if(!this.accepts(e.dragSource)){ return false; }
		this.showFeedback(this);
		this.showingFeedback = true;
		return true;
	},

	onDragMove: function(e){
		if(this.shouldShowCaret(e)){
			this.showCaret(e);
		}
	},

	// D&D operation has been ended
	onDragEnd: function(e){
	},

	onDropStart: function(e){
	},

	onDrop: function(e){
	},

	onDropEnd: function(){
	},

	onDragOut: function(e){
		if(this.showingFeedback){
			this.hideFeedback(this);
			this.showingFeedback = false;
		}
		this.hideCaret();
	},

	// You may want to override this
	isAcceptable: function(ds, dt){
		return true;
	},

	registerIsAcceptable: function(/*String*/operation, /*Function*/f){
		this._isAcceptableMap[operation] = f;
	},

	unregisterIsAcceptable: function(/*String*/operation){
		delete this._isAcceptableMap[operation];
	},

	accepts: function(ds){
		if(!ds.operation || !this._isAcceptableMap[ds.operation]){ return false; }
		if(!this._isAcceptableMap[ds.operation](ds, this)){ return false; }
		for(var i = 0, len = this.acceptedTypes.length; i < len; i++){
			if(this.acceptedTypes[i] == ds.type || this.acceptedTypes[i] == "*"){ return true; } //$NON-NLS-1$
		}
		return false;
	},

	showFeedback: function(dt){
	},

	hideFeedback: function(dt){
	},

	getNodeUnderMouse: function(){
		return this.nodeUnderMouse;
	},

	shouldShowCaret: function(e){
		// check the drag source
		var ds = e.dragSource;
		if(!ds.targetShouldShowCaret){ 
			return false;
		}
		return true;
	},

	showCaret: function(e){
		if(!this.data || !this.data.getCaretRect) 
			return;
		var caretRect = this.data.getCaretRect(e);
		if(!caretRect) 
			return;
		this.createCaret();
		var caret = this.caret;
		caret.style.width = caretRect.width + "px";//$NON-NLS-1$
		caret.style.height = caretRect.height + "px";//$NON-NLS-1$
		caret.style.top = caretRect.top + "px";//$NON-NLS-1$
		caret.style.left = caretRect.left + "px";//$NON-NLS-1$
		caret.style.display = ""; //$NON-NLS-1$
	},

	hideCaret: function(){
		if(this.caret){
			this.caret.style.display = "none"; //$NON-NLS-1$
		}
	},

	createCaret: function(){
		if(!this.caret){
			var caret = this.caret = dojo.create("div"); //$NON-NLS-1$
			caret.className = "lwCaret"; //$NON-NLS-1$
			caret.style.zIndex = 1000; 
			caret.style.position = "absolute"; //$NON-NLS-1$
			caret.style.display = "none"; //$NON-NLS-1$
			caret.style.backgroundColor = "#888888"; //$NON-NLS-1$
			this.offsetParent.appendChild(this.caret);
		}
	}
});
