/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget"); //$NON-NLS-1$
dojo.require("dijit._Container"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget,
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]" //$NON-NLS-1$
	stroke: "black", // String //$NON-NLS-1$
	startDir: "right", //$NON-NLS-1$
	endDir: "left", //$NON-NLS-1$

	sourceDecorationClass: "", // decoration class name //$NON-NLS-1$
	sourceDecorationProps: null, // properties for the source decoration
	targetDecorationClass: "", // decoration class name //$NON-NLS-1$
	targetDecorationProps: null, // properties for the target decoration

	sourceLabelClass: "", // label class name //$NON-NLS-1$
	sourceLabelProps: null, // properties for the source label
	targetLabelClass: "", // label class name //$NON-NLS-1$
	targetLabelProps: null, // properties for the target label
	connectionLabelClass: "", // label class name //$NON-NLS-1$
	connectionLabelProps: null, // properties for the target label

	sourceDecorationObj: null, // decoration object
	targetDecorationObj: null, // decoration object
	sourceLabelObj: null, // label object
	targetLabelObj: null, // label object
	connectionLabelObj: null, // label object

	isGfxLineWidget: true,
	movable: false, // disable drag move
	
	_accs: ["sourceDecoration", "targetDecoration", "sourceLabel", "targetLabel", "connectionLabel"], //accessories //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	postMixInProperties: function(){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){ //$NON-NLS-1$
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (typeof(this.stroke) == "string" && this.stroke.charAt(0) == "{"){ //$NON-NLS-1$ //$NON-NLS-2$
			// ex. "{color:[255, 0, 0]}"
			this.stroke = eval('('+this.stroke+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		this._createdObjects = [];
	},

	postCreate: function(){
		this.domNode.isGfxLineWidget = true; // [custom property]
		this.domNode.style.position = "absolute"; //$NON-NLS-1$
		this.inherited(arguments);
	},

	destroy: function() {
		if (this._createdObjects) {
			dojo.forEach(this._createdObjects, function(item) {
				item.destroy();
			});
			delete this._createdObjects;
		}
		this.inherited(arguments);
	},
	
	startup: function(){
		if(this._started){ return; }

		dojo.forEach(this._accs, function(a){
			var cap = a.charAt(0).toUpperCase() + a.substring(1);
			this["set"+cap](this[a+"Class"]); //$NON-NLS-1$ //$NON-NLS-2$
		}, this);

		this.inherited(arguments);
	},

	createTargetShape: function(){
		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].createShape(this.group); //$NON-NLS-1$
			}
		}, this);
	},

	update: function(props){
		this.inherited(arguments);
		if(!this.shape || this.points.length < 2){ return; }
		this.setStroke(this.stroke);

		if(this._comparePoints(this.points, this._prevPoints) != 0){
			this.onPointsChanged();
			this.setShape();

			var copy = [];
			for(var i = 0, len = this.points.length; i < len - 1; i++){
				copy.push({x:this.points[i].x, y:this.points[i].y});
			}
			this._prevPoints = copy;
		}

		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].update(); //$NON-NLS-1$
				var pt;
				if(a.indexOf("source") != -1){ //$NON-NLS-1$
					pt = this.points[0]; // start
				}else if(a.indexOf("target") != -1){ //$NON-NLS-1$
					pt = this.points[this.points.length - 1]; // end
				}else /* if(a.indexOf("connection") != -1)*/{ 
					var p0 = this.points[0];
					var p1 = this.points[this.points.length - 1];
					pt = {x:Math.round((p0.x+p1.x)/2), y:Math.round((p0.y+p1.y)/2)}; // center
				}
				this[a+"Obj"].setLocation(pt); //$NON-NLS-1$
				if(this[a+"Obj"].setRotation){ //$NON-NLS-1$
					var angle = this.calculateAngle(this.points[(a.indexOf("source") != -1) ? 1 : this.points.length - 2], pt); //$NON-NLS-1$
					this[a+"Obj"].setRotation(angle); //$NON-NLS-1$
				}
			}
		}, this);
	},
	
	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	},

	removeShape: function(){
		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].removeShape(); //$NON-NLS-1$
			}
		}, this);
		this.inherited(arguments);
	},

	restoreShape: function(){
		this.inherited(arguments);
		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].restoreShape(); //$NON-NLS-1$
			}
		}, this);
	},

	// subclass must implement
	setShape: function(){
	},

	// summary: moves a shape to front of its parent's list of shapes
	moveToFront: function(){
		this.group.moveToFront();
		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].moveToFront(); //$NON-NLS-1$
			}
		}, this);
	},

	moveToBack: function(){
		this.group.moveToBack();
		dojo.forEach(this._accs, function(a){
			if(this[a+"Obj"]){ //$NON-NLS-1$
				this[a+"Obj"].moveToBack(); //$NON-NLS-1$
			}
		}, this);
	},

	calculateAngle: function(pt0, pt1){
		return 0;
	},

	// Called when points are added or changed. You may want to connect to this.
	onPointsChanged: function(){
	},

	_createAccessory: function(/*String|Object*/cls, /*String*/acc){
		if(!cls){
			this[acc] = ""; //$NON-NLS-1$
			this[acc+"Obj"] = null; //$NON-NLS-1$
			return;
		}
		var newInstance;
		if(typeof(cls) == "string"){ //$NON-NLS-1$
			this[acc] = cls;
			var impl = dojo.getObject(cls);
			if(!impl){ alert("com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine: "+cls+" not loaded."); } //$NON-NLS-1$ //$NON-NLS-2$
			var props = this[acc+"Props"]; //$NON-NLS-1$
			if(props && typeof(props) == "string"){ //$NON-NLS-1$
				props = eval('('+props+')'); //$NON-NLS-1$ //$NON-NLS-2$
			}
			if(props){ props.type = acc; }
			newInstance = new impl(this, props);
			this._createdObjects.push(newInstance);
		}else{
			this[acc] = cls.declaredClass;
			newInstance = cls;
		}
//		var parent = this.getParent() && this.getParent().group; // parent group
// shima
		var parent = this.group; // parent group
		if(parent){
			newInstance.createShape(parent);
		}
		if(this[acc+"Obj"]){ //$NON-NLS-1$
			this._createdObjects = dojo.filter(this._createdObjects, function(item) {
				return item != this[acc+"Obj"]; //$NON-NLS-1$
			}, this);
			this[acc+"Obj"].destroy(); // destroy the old instance //$NON-NLS-1$
		}
		this[acc+"Obj"] = newInstance; //$NON-NLS-1$
		this.update();
	},

	_setAccessoryProps: function(/*String|Object*/props, /*String*/acc){
		if(props && typeof(props) == "string"){ //$NON-NLS-1$
			props = eval('('+props+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this[acc+"Props"] = props; //$NON-NLS-1$
		if(this[acc+"Obj"]){ //$NON-NLS-1$
			this[acc+"Obj"].setProps(props); //$NON-NLS-1$
		}
		this.update();
	},


	// source decoration
	getSourceDecoration: function(){
		return this.sourceDecorationObj;
	},

	setSourceDecoration: function(/*String|Decoration*/dec){
		this._createAccessory(dec, "sourceDecoration"); //$NON-NLS-1$
	},

	setSourceDecorationProps: function(/*String|Object*/props){
		this._setAccessoryProps(props, "sourceDecoration"); //$NON-NLS-1$
	},

	// target decoration
	getTargetDecoration: function(){
		return this.targetDecorationObj;
	},

	setTargetDecoration: function(dec){
		this._createAccessory(dec, "targetDecoration"); //$NON-NLS-1$
	},

	setTargetDecorationProps: function(/*String|Object*/props){
		this._setAccessoryProps(props, "targetDecoration"); //$NON-NLS-1$
	},

	// source label
	getSourceLabel: function(){
		return this.sourceLabelObj;
	},

	setSourceLabel: function(/*String|Label*/label){
		this._createAccessory(label, "sourceLabel"); //$NON-NLS-1$
	},

	setSourceLabelProps: function(/*String|Object*/props){
		this._setAccessoryProps(props, "sourceLabel"); //$NON-NLS-1$
	},

	// target label
	getTargetLabel: function(){
		return this.targetLabelObj;
	},

	setTargetLabel: function(label){
		this._createAccessory(label, "targetLabel"); //$NON-NLS-1$
	},

	setTargetLabelProps: function(/*String|Object*/props){
		this._setAccessoryProps(props, "targetLabel"); //$NON-NLS-1$
	},

	// connection label
	getConnectionLabel: function(){
		return this.connectionLabelObj;
	},

	setConnectionLabel: function(label){
		this._createAccessory(label, "connectionLabel"); //$NON-NLS-1$
	},

	setConnectionLabelProps: function(/*String|Object*/props){
		this._setAccessoryProps(props, "connectionLabel"); //$NON-NLS-1$
	},

	// Adds the passed point to the Polyline.
	addPoint: function(/*{x:#,y:#}*/pt){
		this.points.push(pt);
	},

	// Inserts a given point at a specified index in the Polyline.
	insertPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points.splice(index, 0, pt);
	},

	// Erases the Polyline and removes all of its points.
	removeAllPoints: function(){
		this.points = [];
	},

	// Removes a point from the Polyline.
	removePoint: function(index){
		this.points.splice(index, 1);
	},

	// Sets the points at both extremes of the Polyline.
	setEndpoints: function(/*{x:#,y:#}*/start, /*{x:#,y:#}*/end){
		this.setStart(start);
		this.setEnd(end);
	},

	// Sets the point at index to the point pt.
	setPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points[index] = pt;
	},


// ======================= Setters/Getters =================================
	getStroke: function(){
		return this.stroke;
	},

	setStroke: function(/*String|Object*/stroke){
		this.stroke = stroke; // see dojox.gfx.defaultStroke
		if(!this.shape){ return; }
		if (stroke && typeof(stroke) == "string" && stroke.charAt(0) == "{"){ //$NON-NLS-1$ //$NON-NLS-2$
			// ex. "{color:[255, 0, 0]}"
			this.stroke = eval('('+stroke+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this.shape.setStroke(this.stroke);
	},


	// Returns the first point in the Polyline.
	getStart: function(){
		if(this.points.length == 0){ return null; }
		return this.points[0];
	},

	// Sets the start point of the Polyline.
	setStart: function(/*{x:#,y:#}*/start){
		if (this.points.length == 0){
			this.addPoint(start);
		}else{
			this.setPoint(start, 0);
		}
	},


	// Returns the last point in the Polyline.
	getEnd: function(){
		if(this.points.length == 0){ return null; }
		return this.points[this.points.length - 1];
	},

	// Sets the end point of the Polyline.
	setEnd: function(/*{x:#,y:#}*/end){
		if (this.points.length < 2){
			this.addPoint(end);
		}else{
			this.setPoint(end, this.points.length - 1);
		}
	},


	// Returns the points in this Polyline.
	getPoints: function(){
		return this.points;
	},

	// Sets the list of points to be used by this polyline.
	// Removes any previously existing points.
	setPoints: function(/*Array|String*/points){
		if (!points){
			this.points = [];
		}else if (typeof(points) == "string"){ //$NON-NLS-1$
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+points+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			this.points = points;
		}
	}
});
