/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.Curvyline"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.Curvyline", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine,
{
	curveType: "QuadraticBezier", // "QuadraticBezier" or "CubicBezier" //$NON-NLS-1$

	createTargetShape: function(){
		this.onPointsChanged();
		var shape = this.group.createPath();
		this.inherited(arguments);
		return shape;
	},

	setShape: function(){
		var pathStr = this._points2PathStr();
		if(pathStr){
			this.shape.setShape(pathStr);
		}
	},

	_points2PathStr: function(){
		if(this.curveType == "QuadraticBezier"){ return this._points2QuadraticBezier(); } //$NON-NLS-1$
		else if(this.curveType == "CubicBezier"){ return this._points2CubicBezier(); } //$NON-NLS-1$
		return null;
	},

	_points2QuadraticBezier: function(){
		if(this.points.length == 2){
			// if there are only two points, just draw a straight line.
			var p0 = this.points[0];
			var p1 = this.points[1];
			return "M" + p0.x + "," + p0.y + " L" + p1.x + "," + p1.y; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		var pathStr = null;
		for(var i = 0, len = this.points.length; i < len - 1; i++){
			var p0 = this.points[i];
			var p1 = this.points[i + 1];
			if(i == 0){
				pathStr = "M" + p0.x + "," + p0.y; //$NON-NLS-1$ //$NON-NLS-2$
			}else if(i == len - 2){
				pathStr += " Q" + p0.x + "," + p0.y + " " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					+ p1.x + "," + p1.y; //$NON-NLS-1$
			}else{
				pathStr += " Q" + p0.x + "," + p0.y + " " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					+ Math.round((p0.x+p1.x)/2) + "," + Math.round((p0.y+p1.y)/2); //$NON-NLS-1$
			}
		}
		return pathStr;
	},

	_points2CubicBezier: function(){
		// This algorithm uses only the start point and the end point.
		// That means, use of a router does not affect this algorithm.
		if(this.points.length < 2){ return null; }
		var p0 = this.points[0];
		var p1 = this.points[this.points.length - 1];
		// distance between a point and a control point is 100.
		// however, if distance between p0 and p1 is less than 100,
		// use the distance * 0.8 instead of 100.
		var d = Math.min(100, Math.round(Math.sqrt((p0.x - p1.x)*(p0.x - p1.x) + (p0.y - p1.y)*(p0.y - p1.y))*0.8));
		var dxStart = 0, dxEnd = 0, dyStart = 0, dyEnd = 0;

		var startDir = p0.dir || this.startDir;
		var endDir = p1.dir || this.endDir;

		if(startDir == "left"){ dxStart = -d; } //$NON-NLS-1$
		else if(startDir == "right"){ dxStart = d; } //$NON-NLS-1$
		if(endDir == "left"){ dxEnd = -d; } //$NON-NLS-1$
		else if(endDir == "right"){ dxEnd = d; } //$NON-NLS-1$

		if(startDir == "top"){ dyStart = -d; } //$NON-NLS-1$
		else if(startDir == "bottom"){ dyStart = d; } //$NON-NLS-1$
		if(endDir == "top"){ dyEnd = -d; } //$NON-NLS-1$
		else if(endDir == "bottom"){ dyEnd = d; } //$NON-NLS-1$

		var pathStr = "M" + p0.x + "," + p0.y //$NON-NLS-1$ //$NON-NLS-2$
			+ " C" + (p0.x + dxStart) + "," + (p0.y + dyStart) //$NON-NLS-1$ //$NON-NLS-2$
			+ " "  + (p1.x + dxEnd) + "," + (p1.y + dyEnd) //$NON-NLS-1$ //$NON-NLS-2$
			+ " "  + p1.x + "," + p1.y; //$NON-NLS-1$ //$NON-NLS-2$
		return pathStr;
	},

// ======================= Setters/Getters =================================
	getCurveType: function(){
		return this.curveType;
	},

	setCurveType: function(type){
		this.curveType = type;
		this.update();
	}
});
