/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Contained"); //$NON-NLS-1$
dojo.require("dojox.gfx"); //$NON-NLS-1$
dojo.require("rmpw.common.policy.Host"); //$NON-NLS-1$

// Basic Structure:
// +-----------------+
// | group           | <- this.group
// |    +-------+    |
// |    | rect  | <------ this.shape
// |    +-------+    |
// |    (child1)     |
// |    (child2)     |
// |    ...          |
// +-----------------+

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget", //$NON-NLS-1$
	[rmpw.common.policy.Host, dijit._Widget, dijit._Contained],
{
	group: null, // gfx group object
	shape: null, // gfx shape object, such as rect, circle, etc.

	createShapeCallbacks: null, // array of functions to be called at the end of createShape()
	restoreShapeCallbacks: null, // array of functions to be called at the end of restoreShape()
	updateCallbacks: null, // array of functions to be called at the end of update()
	removeShapeCallbacks: null, // array of functions to be called at the end of removeShape()

	isGfxWidget: true,

	postCreate: function(){
		this.domNode.isGfxWidget = true; // [custom property]
		for(var n = this.domNode; n; n = n.parentNode){
			if(n.tagName == "BODY"){ //$NON-NLS-1$
				// BODY found means this widget is online, i.e. this widget
				// was instantiated with markup, not by the new operator.
				this.createShape();
				this.update();
				break;
			}
		}
		if(this.addChild){
			this.connect(this, "addChild", function(w){w.onAdded && w.onAdded();}); //$NON-NLS-1$
			this.connect(this, "removeChild", function(w){w.onRemoved && w.onRemoved();}); //$NON-NLS-1$
		}
	},

	createShape: function(){
		var parent = this.getParent().group; // parent group
		if(!this.group){ // create new
			this.group = parent.createGroup();
			this.shape = this.createTargetShape();

			// Copy widgetId to the root node of this shape
			// so that the widget can be found using dijit.getEnclosingWidget.
			// Note that the root node of this shape is not a child of this.domNode.
			this.group.rawNode.setAttribute("_widgetId", this.domNode.getAttribute("widgetId")); //$NON-NLS-1$ //$NON-NLS-2$

			// There may be pending shapes, which are created offline.
			dojo.forEach(this.getChildren(), function(w){
				if(w instanceof com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget){
					w.createShape();
				}
			});
		}else if(!this.group.getParent()){
			this.restoreShape();
		}/*else{
			// shape exists and it is online
		}*/
		this._callCallbacks(this.createShapeCallbacks);
	},

	// subclass should implement
	update: function(props){
		if(props){
			dojo.mixin(this, props);
			var s = this.domNode.style;
			if(props.top){ s.top = props.top + "px"; } //$NON-NLS-1$
			if(props.left){ s.left = props.left + "px"; } //$NON-NLS-1$
			if(props.width){ s.width = props.width + "px"; } //$NON-NLS-1$
			if(props.height){ s.height = props.height + "px"; } //$NON-NLS-1$
		}
		this._callCallbacks(this.updateCallbacks);
	},

	updateRecursively: function(){
		this.update();
		dojo.forEach(this.getChildren(), function(w){
			if(w instanceof com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget){
				w.updateRecursively();
			}
		});
	},

	removeShape: function(){
		if(this.group){
			this.group.removeShape();
		}
		this._callCallbacks(this.removeShapeCallbacks);
	},

	// restore the removed shape
	restoreShape: function(){
		var parent = this.getParent().group; // parent group
		parent.add(this.group);
		this.recoverLostInfo(this.group);
		this._callCallbacks(this.restoreShapeCallbacks);
	},

	// subclass must implement
	createTargetShape: function(){
		return null;
	},

	recoverLostInfo: function(shape){
		if(dojo.isIE <= 8){
			// to recover the lost information. VML's bug.
			dojo.forEach(shape.children || [], function(c){
				this.recoverLostInfo(c);			
			},this);
			shape.setShape(shape.shape);
			shape.setTransform(shape.matrix);
			shape.setFill(shape.fillStyle);
			shape.setStroke(shape.strokeStyle);
		}
	},

	// event must be either "createShape", "restoreShape", "update", or "removeShape"
	registerCallback: function(/*String*/event, /*Function*/method){
		if(!this[event+"Callbacks"]){ //$NON-NLS-1$
			this[event+"Callbacks"] = []; //$NON-NLS-1$
		}
		if(dojo.indexOf(this[event+"Callbacks"], method) != -1){ return; } // already registered //$NON-NLS-1$
		this[event+"Callbacks"].push(method); //$NON-NLS-1$
	},

	_callCallbacks: function(callbacks){
		if(!callbacks){ return; }
		for(var i = 0, len = callbacks.length; i < len; i++){
			callbacks[i].call(this);
		}
	},

	onAdded: function(){
		var parent = this.getParent() && this.getParent().group; // parent group
		if(parent){
			this.createShape();
			this.updateRecursively();
		}
	},

	onRemoved: function(){
		this.removeShape();
		// refresh policies
		dojo.forEach(this.policies, function(p){
			if(p.refresh) p.refresh();
		});
	},
	
	destroy: function() {
		// destroy policies
		dojo.forEach(this.policies, function(p){
			if(p.destroy) p.destroy();
		});
		delete this.policies;
		//
		this.removeShape();
		if(this.shape){
			delete this.shape;
		}
		if(this.group){
			this.group.destroy();
			delete this.group;
		}
		
		// call super
		this.inherited(arguments);
	}	
});

//override dijit._base.manager
dijit.getEnclosingWidget = function(/*DOMNode*/node){
	while(node){
		var id = node.getAttribute
			&& (node.getAttribute("widgetId") || node.getAttribute("_widgetId")); //$NON-NLS-1$ //$NON-NLS-2$
		if(id){
			return dijit.byId(id);
		}
		node = node.parentNode;
	}
	return null;
};
