/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.LabeledImage"); //$NON-NLS-1$

// summary:
//		A gfx image node widget that can have a label text.

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.LabeledImage", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode,
{
	src: "", //$NON-NLS-1$
	header: "", // header label text //$NON-NLS-1$
	footer: "", // footer label text //$NON-NLS-1$
	headerFont: "normal", //$NON-NLS-1$
	footerFont: "normal", //$NON-NLS-1$
	isHeaderEditable: false,
	isFooterEditable: false,

	imageShape: null,
	headerShape: null,
	footerShape: null,

	postMixInProperties: function(){
		if (typeof(this.headerFont) == "string" && this.headerFont.charAt(0) == "{"){ //$NON-NLS-1$ //$NON-NLS-2$
			// ex. "{'weight':'bold'}"
			this.headerFont = eval('('+this.headerFont+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (typeof(this.footerFont) == "string" && this.footerFont.charAt(0) == "{"){ //$NON-NLS-1$ //$NON-NLS-2$
			// ex. "{'weight':'bold'}"
			this.footerFont = eval('('+this.footerFont+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},

	createTargetShape: function(){
		if(!this.width && !this.height){
			// this may not work sometimes. need to wait for the image to load.
			var img = dojo.create("img", {src:dojo.moduleUrl("com.ibm.relm.explorer.web.base.internal", this.src)+"",style:"visibility:hidden;"}, dojo.body()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			this.width = img.offsetWidth;
			this.height = img.offsetHeight;
			dojo.body().removeChild(img);
		}

		this.imageShape = this.group.createImage({width:this.getWidth(), height:this.getHeight(), src:dojo.moduleUrl("com.ibm.relm.explorer.web.base.internal", this.src)+""}); //$NON-NLS-1$ //$NON-NLS-2$
		this.headerShape = this.group.createText();
		this.footerShape = this.group.createText();
		this.setHeader(this.header);
		this.setFooter(this.footer);

		if(this.isHeaderEditable){
			this.connect(this.headerShape.rawNode, "onclick", "headerClickHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.headerInput = dojo.create("INPUT"); //$NON-NLS-1$
			this.connect(this.headerInput, "onblur", "headerInputBlurHandler"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(this.isFooterEditable){
			this.connect(this.footerShape.rawNode, "onclick", "footerClickHandler"); //$NON-NLS-1$ //$NON-NLS-2$
			this.footerInput = dojo.create("INPUT"); //$NON-NLS-1$
			this.connect(this.footerInput, "onblur", "footerInputBlurHandler"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		return null;
	},

	// Override gdc.widget.AbstractNode
	update: function(props){
		this.inherited(arguments);
		this.headerShape.setShape({text:this.header});
		this.headerShape.setFont(this.headerFont).setFill("black"); //$NON-NLS-1$
		this.footerShape.setShape({text:this.footer});
		this.footerShape.setFont(this.footerFont).setFill("black"); //$NON-NLS-1$
	},

	// restore the removed shape
	restoreShape: function(){
		this.inherited(arguments);
		this.headerShape.setFont(this.headerFont).setFill("black"); //$NON-NLS-1$
		this.footerShape.setFont(this.footerFont).setFill("black"); //$NON-NLS-1$
	},

	headerClickHandler: function(e){
		this.headerInput.value = this.header;
		this.domNode.appendChild(this.headerInput);
		this.headerInput.focus();
	},

	headerInputBlurHandler: function(e){
		this.setHeader(this.headerInput.value);
		this.domNode.removeChild(this.headerInput);
		this.headerInput.value = ""; //$NON-NLS-1$
	},

	footerClickHandler: function(e){
		this.footerInput.value = this.footer;
		this.domNode.appendChild(this.footerInput);
		this.footerInput.focus();
	},

	footerInputBlurHandler: function(e){
		this.setFooter(this.footerInput.value);
		this.domNode.removeChild(this.footerInput);
		this.footerInput.value = ""; //$NON-NLS-1$
	},

// ======================= Setters/Getters =================================
	setHeader: function(/*String*/header){
		this.header = header;
		this.headerShape.setShape({text:header});
		var dim = this.getBB(this.header, this.headerFont);
		this.headerShape.setShape({
			x: (this.getWidth()-dim.w) / 2,
			y: -4
		});
	},

	setFooter: function(/*String*/footer){
		this.footer = footer;
		this.footerShape.setShape({text:footer});
		var dim = this.getBB(this.footer, this.footerFont);
		this.footerShape.setShape({
			x: (this.getWidth()-dim.w) / 2,
			y: this.getHeight()+dim.h
		});
	},

	setSrc: function(src){
		this.src = src;
		if(this.doRefresh && this.imageShape){
			this.imageShape.setShape({src:dojo.moduleUrl("com.ibm.relm.explorer.web.base.internal", this.src)+""}); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
});
