/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.Polyline"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.Polyline", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine,
{
	r: 5,
	createTargetShape: function(){
		this.onPointsChanged();
		var shape;
		this.r = Number(this.r);
		if(this.r != NaN && this.r > 0){
			shape = this.group.createPath();
		}else{
			shape = this.group.createPolyline();
		}
		this.inherited(arguments);
		return shape;
	},

	setShape: function(){
		this.applyPath(this.shape, this.points, Number(this.r));
	},

	calculateAngle: function(pt0, pt1){
		var dx = Math.abs(pt1.x - pt0.x);
		var dy = pt1.y - pt0.y;
		if(dx == 0 && dy == 0){ return 0; }
		var angle = Math.atan(dy/dx)/Math.PI*180;
		if(pt1.x - pt0.x < 0){
			angle = 180 - angle;
		}
		return angle;
	},
	
    applyPath: function(path,points,r){
    	var calcD = function(p1, p2, r){
			var d = p2 - p1;
			if(d > r){
				d -= r;
			}else if(d < -r){
				d += r;
			}
			return d;
    	};
    	var calcD2 = function(p1, p2, r){
			var d = p2 - p1;
			if(d > r){
				d = r;
			}else if(d < -r){
				d = -r;
			}else{
				d = 0;
			}
			return d;
    	};
    	
        if (path instanceof dojox.gfx.Polyline) {
            path.setShape(points);
			path.bbox = null; // patch for GFX bug...
        } else {
            path.setShape("");//$NON-NLS-1$
            if (points && points.length > 0) {
                path.moveTo(points[0].x, points[0].y);
				if (r > 0 && points.length > 2) {
					for (var i = 0; i < points.length - 2; i++) {
						var x0 = points[i].x;
						var y0 = points[i].y;
						// initial point:
						var p1 = points[i + 1];
						var x1 = x0 + calcD(x0, p1.x, r);
						var y1 = y0 + calcD(y0, p1.y, r);
						path.lineTo(x1, y1);
						// last point
						var p2 = points[i + 2];
						var x2 = p1.x + calcD2(p1.x, p2.x, r);
						var y2 = p1.y + calcD2(p1.y, p2.y, r);
						path.qCurveTo(p1.x, p1.y, x2, y2);
					}
					path.lineTo(points[points.length-1].x, points[points.length-1].y);
				}
				else {
					// polyline:
					for (var i = 1; i < points.length; i++) {
						path.lineTo(points[i].x, points[i].y);
					}
				}
            }
        }
	}
});
