/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc._connection"); //$NON-NLS-1$

// summary:
//		Make a gfx line widget connection.
//		This class is intended to be mixed into a gfx line widget such as Polyline.
//		'this' points to a gfx line widget.

com.ibm.relm.explorer.web.base.internal.gdc._connection = {
	sourceAnchor: "", // "nodeId" or "nodeId:anchorName" //$NON-NLS-1$
	targetAnchor: "", // "nodeId" or "nodeId:anchorName" //$NON-NLS-1$

	sourceAnchorObj: null, // anchor object
	targetAnchorObj: null, // anchor object
	routerObj: null, // router object

	processParams: function(){
		this.setRouter(this.router);
		if(this.sourceAnchor && !this.sourceAnchorObj){
			var anchor = this.sourceAnchor;
			this.sourceAnchor = null; // to force the setter to work
			this.setSourceAnchor(anchor, true);
		}
		if(this.targetAnchor && !this.targetAnchorObj){
			var anchor = this.targetAnchor;
			this.targetAnchor = null; // to force the setter to work
			this.setTargetAnchor(anchor, true);
		}
		this.registerCallback("removeShape", this.disconnect); //$NON-NLS-1$
		this.registerCallback("restoreShape", this.reconnect); //$NON-NLS-1$
		this.anchorMoved();
	},

	cleanup: function(){
		this.disconnect();
	},

	disconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.removeAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.removeAnchorListener(this);
		}
	},

	reconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.addAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.addAnchorListener(this);
		}
	},

	// override com.ibm.relm.explorer.web.base.internal.gdc.AbstractLine#onPointsChanged
	onPointsChanged: function(){
		if(this.routerObj){
			this.routerObj.update();
		}
	},


	setAnchor: function(/*String|Anchor*/anchor, /*String*/which, /*boolean*/noRefresh){
		if(anchor && (this[which+"Anchor"] == anchor || this[which+"AnchorObj"] == anchor)){ return; } //$NON-NLS-1$ //$NON-NLS-2$
		if(!anchor && (!this[which+"Anchor"] && !this[which+"AnchorObj"])){ return; } //$NON-NLS-1$ //$NON-NLS-2$
		if(this[which+"AnchorObj"]){ //$NON-NLS-1$
			this[which+"AnchorObj"].removeAnchorListener(this); // remove the old anchor //$NON-NLS-1$
		}
		if(!anchor){
			this[which+"Anchor"] = ""; //$NON-NLS-1$ //$NON-NLS-2$
			this[which+"AnchorObj"] = null; //$NON-NLS-1$
			return;
		}
		var anchorObj;
		if(typeof(anchor) == "string"){ //$NON-NLS-1$
			var arr = anchor.split(":"); // "nodeId:anchorName" //$NON-NLS-1$
			var id =  arr[0];
			var anchorName = arr[1] || "base"; // default anchor name is "base" //$NON-NLS-1$
			var nodeWidget = dijit.byId(id);
			var cap = which.charAt(0).toUpperCase() + which.substring(1);
			anchorObj = nodeWidget && com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager["get"+cap+"ConnectionAnchorByName"](nodeWidget, anchorName); //$NON-NLS-1$ //$NON-NLS-2$
			this[which+"Anchor"] = anchor; //$NON-NLS-1$
		}else{
			this[which+"Anchor"] = anchor.owner && anchor.owner.id; //$NON-NLS-1$
			anchorObj = anchor;
		}
		anchorObj.addAnchorListener(this); // add the new anchor
		this[which+"AnchorObj"] = anchorObj; //$NON-NLS-1$
		if(!noRefresh){ this.anchorMoved(); }
	},

	setSourceAnchor: function(/*String|Anchor*/anchor, /* boolean */noRefresh){
		this.setAnchor(anchor, "source", noRefresh); //$NON-NLS-1$
	},

	setTargetAnchor: function(/*String|Anchor*/anchor, /* boolean */noRefresh){
		this.setAnchor(anchor, "target", noRefresh); //$NON-NLS-1$
	},

	getSourceNodeWidget: function(){
		return this.sourceAnchorObj && this.sourceAnchorObj.owner;
	},

	getTargetNodeWidget: function(){
		return this.targetAnchorObj && this.targetAnchorObj.owner;
	},

	setRouter: function(/*String|Router*/router){
		if(!router){
			if(this.router && this.router.destroy){
				this.router.destroy();
			}
			this.router = this.routerObj = null;
			return;
		}
		if(typeof(router) == "string"){ //$NON-NLS-1$
			try{
				dojo["require"](router); //TODO: async support //$NON-NLS-1$
				var routerImpl = dojo.getObject(router);
				this.routerObj = new routerImpl(this);
			}catch(e){
				console.debug("Specified router was not found: '"+router+"'"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}else if(typeof(router) == "object"){ //$NON-NLS-1$
			this.routerObj = router;
		}
		this.anchorMoved();
	},

	// Your widget may want to override
	getSourceEndpointKnobType: function(){
		return null;
	},

	// Your widget may want to override
	getTargetEndpointKnobType: function(){
		return null;
	},

	_timeout: null,
	anchorMoved: function(){
		if(this._timeout){
			clearTimeout(this._timeout);
		}
		this._timeout = setTimeout(dojo.hitch(this, this.anchorMovedCore), 50);		
	},
	anchorMovedCore: function(){
		this._timeout = null;
		if(!this.domNode){
			// already destroyed
			return;
		}
		if(this.sourceAnchorObj){
			var ref = null;
			if(this.sourceAnchorObj.needRefPoint !== false){
				if(this.targetAnchorObj){
					ref = this.targetAnchorObj.getReferencePoint();
					var offset = this._getNodeOffset(this.sourceAnchorObj, this.targetAnchorObj);
					ref.x += offset.x;
					ref.y += offset.y;
				}else{
					ref = this.getEnd();
				}
			}
			var pt = this.sourceAnchorObj.getLocation(ref, this);
			var widget = this.sourceAnchorObj.owner.getParent && this.sourceAnchorObj.owner.getParent();
			if(!widget){
				widget = dijit._WidgetBase.prototype.getParent.call(this.sourceAnchorObj.owner);
			}
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			pt.dir = this.sourceAnchorObj.getDirection(pt);
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
			this.setStart(pt);
		}

		if(this.targetAnchorObj){
			var ref = null;
			if(this.targetAnchorObj.needRefPoint !== false){
				if(this.sourceAnchorObj){
					ref = this.sourceAnchorObj.getReferencePoint();
					var offset = this._getNodeOffset(this.targetAnchorObj, this.sourceAnchorObj);
					ref.x += offset.x;
					ref.y += offset.y;
				}else{
					ref = this.getStart();
				}
			}
			var pt = this.targetAnchorObj.getLocation(ref, this);
			var widget = this.targetAnchorObj.owner.getParent && this.targetAnchorObj.owner.getParent();
			if(!widget){
				widget = dijit._WidgetBase.prototype.getParent.call(this.targetAnchorObj.owner);
			}
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			pt.dir = this.targetAnchorObj.getDirection(pt);
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
			this.setEnd(pt);
		}
		this.update();
	},

	_getOffsetPos: function(elem, parent){
		var top = 0, left = 0;
//		for(; elem != null; elem = elem.offsetParent){
		for(; elem != null; elem = elem.parentNode){
			if(elem == parent){ break; }
			top += elem.offsetTop;
			left += elem.offsetLeft;
		}
		return {top:top, left:left};
	},
	
	_getNodeOffset: function(source, target) {
		var parent = this.getParent();
		if(!source || !target || !parent)
			return {x: 0, y: 0};
		
		var dimS = this._getOffsetPos(source.owner.getParent().domNode, parent.domNode);
		var dimT = this._getOffsetPos(target.owner.getParent().domNode, parent.domNode);
		return {x: dimT.left - dimS.left, y: dimT.top - dimS.top};
	}
};
