/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager"); //$NON-NLS-1$

// summary:
//		Manages anchors for a gfx node widget or an html node widget.
//		com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager points to a singleton object of this class.

dojo.declare("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager", null, { //$NON-NLS-1$
	// The default implementation of anchor creation.
	createConnectionAnchors: function(widget){
		if(widget.sourceAnchorClass){
			var anchorImpl = dojo.getObject(widget.sourceAnchorClass);
			widget.sourceAnchorObj = new anchorImpl(widget);
			this.registerSourceConnectionAnchor(widget, widget.domNode, widget.sourceAnchorObj, widget.sourceAnchorName);
		}
		if(widget.targetAnchorClass){
			var anchorImpl = dojo.getObject(widget.targetAnchorClass);
			widget.targetAnchorObj = new anchorImpl(widget);
			this.registerTargetConnectionAnchor(widget, widget.domNode, widget.targetAnchorObj, widget.targetAnchorName);
		}
	},

	getConnections: function(/*Widget*/widget){
		// summary: returns an array of all the connected gfx line widgets.
		return this.getSourceConnections(widget).concat(this.getTargetConnections(widget));
	},


	// ----------------------------------------------------
	// Source Anchor Management
	// ----------------------------------------------------

	registerSourceConnectionAnchor: function(/*Widget*/nodeWidget, /*Element*/elem, /*Anchor*/anchor, /*String?*/anchorName){
		var name = anchorName ? anchorName : "base"; //$NON-NLS-1$
		if(!nodeWidget.sourceConnectionAnchors){ nodeWidget.sourceConnectionAnchors = {}; }
		nodeWidget.sourceConnectionAnchors[name] = anchor;
		anchor.anchorName = name;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(name); // [custom property]
	},

	getSourceConnectionAnchor: function(/*Widget*/nodeWidget, /*Element*/clickedElement){
		if(!nodeWidget || !nodeWidget.sourceConnectionAnchors){ return null; }
		for(var elem = clickedElement; elem;){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(nodeWidget.sourceConnectionAnchors[elem.anchorNames[i]]){
						return nodeWidget.sourceConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == nodeWidget.domNode || elem.tagName == "BODY"){ return null; } //$NON-NLS-1$
			var id = elem.getAttribute("_widgetId"); // widgetId might be copied to the root of a shape. //$NON-NLS-1$
			elem = id ? dijit.byId(id).domNode : elem.parentNode;
		}
		return null;
	},

	getSourceConnectionAnchorByName: function(/*Widget*/nodeWidget, /*String*/anchorName){
		return nodeWidget.sourceConnectionAnchors[anchorName];
	},

	getSourceConnectionAnchors: function(/*Widget*/nodeWidget){
		return nodeWidget.sourceConnectionAnchors;
	},

	getSourceConnections: function(/*Widget*/nodeWidget){
		if(!nodeWidget.sourceConnectionAnchors){ return []; }
		var connections = [];
		for (var name in nodeWidget.sourceConnectionAnchors){
			var anchor = nodeWidget.sourceConnectionAnchors[name];
			connections = connections.concat(anchor.getConnections());
		}
		return connections;
	},


	// ----------------------------------------------------
	// Target Anchor Management
	// ----------------------------------------------------

	registerTargetConnectionAnchor: function(/*Widget*/nodeWidget, /*Element*/elem, /*Anchor*/anchor, /*String?*/anchorName){
		var name = anchorName ? anchorName : "base"; //$NON-NLS-1$
		if(!nodeWidget.targetConnectionAnchors){ nodeWidget.targetConnectionAnchors = {}; }
		nodeWidget.targetConnectionAnchors[name] = anchor;
		anchor.anchorName = name;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(name); // [custom property]
	},

	getTargetConnectionAnchor: function(/*Widget*/nodeWidget, /*Element*/clickedElement){
		if(!nodeWidget || !nodeWidget.targetConnectionAnchors){ return null; }
		for(var elem = clickedElement; elem;){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(nodeWidget.targetConnectionAnchors[elem.anchorNames[i]]){
						return nodeWidget.targetConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == nodeWidget.domNode || elem.tagName == "BODY"){ return null; } //$NON-NLS-1$
			var id = elem.getAttribute("_widgetId"); // widgetId might be copied to the root of a shape. //$NON-NLS-1$
			elem = id ? dijit.byId(id).domNode : elem.parentNode;
		}
		return null;
	},

	getTargetConnectionAnchorByName: function(/*Widget*/nodeWidget, /*String*/anchorName){
		return nodeWidget.targetConnectionAnchors[anchorName];
	},

	getTargetConnectionAnchors: function(/*Widget*/nodeWidget){
		return nodeWidget.targetConnectionAnchors;
	},

	getTargetConnections: function(/*Widget*/nodeWidget){
		if(!nodeWidget.sourceConnectionAnchors){ return []; }
		var connections = [];
		for (var name in nodeWidget.targetConnectionAnchors){
			var anchor = nodeWidget.targetConnectionAnchors[name];
			connections = connections.concat(anchor.getConnections());
		}
		return connections;
	}
});

com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager = new com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager();
