/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.decoration.PolylineDecoration"); //$NON-NLS-1$

// Summary:
//		A decorative Figure intended to be placed on a Polyline.
//		Note that this is not a widget.

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.decoration.AbstractDecoration"); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.decoration.PolylineDecoration", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.decoration.AbstractDecoration,
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]" //$NON-NLS-1$
	stroke: "black", // String //$NON-NLS-1$
	fill: "", //$NON-NLS-1$
	scale: 1, // Integer

	parentGroupBox: null, // dojox.gfx.Group to add this shape

	constructor: function(owner, props){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){ //$NON-NLS-1$
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')'); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},

	destroy: function(){
		if (this.group) {
			this.group.removeShape().destroy();
			delete this.group;
		} 
		this.inherited(arguments);
	},

	processPreset: function(){
		var presets = com.ibm.relm.explorer.web.base.internal.gdc.decoration.presets;
		if(!presets || !this.preset){ return; }
		this.stroke = this.ownerColor;
		var spec = presets[this.preset];
		if(spec){
			this.points = spec.points;
			this.fill = spec.fill == undefined ? this.ownerColor : spec.fill;
		}
	},

	setProps: function(props){
		var prev = this.preset;
		this.inherited(arguments);
		if(this.preset != prev){
			this.processPreset();
		}
	},

	createShape: function(parent){
		this.parentGroupBox = parent;
		this.group = parent.createGroup();
		this.shape = this.group.createPolyline();
		this.inherited(arguments);
	},

	update: function(props){
		if(!this.shape){ return; }

		if(this._comparePoints(this.points, this.shape.shape.points) != 0){
			this.shape.setShape(this.points);
		}

		if((props && props.forceUpdate) || this.stroke != this._prevStroke){
			this.shape.setStroke(this.stroke);
			this._prevStroke = this.stroke;
		}

		if((props && props.forceUpdate) || this.fill != this._prevFill){
			this.shape.setFill(this.fill);
			this._prevFill = this.fill;
		}
	},

	removeShape: function(){
		var ret = this.group ? this.group.removeShape() : null;
		return ret;
	},

	// restore the removed shape
	restoreShape: function(){
		this.parentGroupBox.add(this.group);
		this.owner.recoverLostInfo(this.shape);
	},

	setLocation: function(/*{x:#,y:#}*/pt){
		if(!this.shape) return;
		this.shape.setTransform({dx:pt.x, dy:pt.y});
		if(!this._prevScale && this.scale != 1 || this.scale != this._prevScale){
			var matrix = [dojox.gfx.matrix.scaleAt(this.scale, this.scale, pt.x, pt.y)];
			this.group.setTransform(matrix);
			this._prevScale = this.scale;
		}
	},

	setRotation: function(/*float*/angle){
		if(!this.shape) return;
		var rotate = dojox.gfx.matrix.rotategAt(angle, 0, 0);
		this.shape.applyTransform(rotate);
	},

	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	}
});
