/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.router.ManhattanConnectionRouter"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.router.AbstractRouter"); //$NON-NLS-1$

// summary:
//		Update points with an orthogonal route between the source and target anchors.
//		'this' points to a connection widget.

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.router.ManhattanConnectionRouter", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.router.AbstractRouter,
{
	delta: 20,

	getPoints: function(){
		var pt0 = this.owner.points[0];
		var pt = this.owner.points[this.owner.points.length-1];
		var x1 = pt0.x;
		var y1 = pt0.y;
		var startdir = pt0.dir;
		var x2 = pt.x;
		var y2 = pt.y;
		var enddir = pt.dir;		
		var delta = this.delta;

		// top-bottom combination
		var points = [];
		if((startdir=="top" && enddir=="bottom" && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="top" && y1<y2)){ //$NON-NLS-1$ //$NON-NLS-2$
			points = [{x:x1,y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
			
		}else if((startdir=="top" && enddir=="bottom" && y1<y2)){ //$NON-NLS-1$ //$NON-NLS-2$
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if((startdir=="bottom" && enddir=="top" && y1>=y2)) { //$NON-NLS-1$ //$NON-NLS-2$
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];

		// left-right combination
		}else if((startdir=="left" && enddir=="right" && x1<x2)){ //$NON-NLS-1$ //$NON-NLS-2$
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if((startdir=="right" && enddir=="left" && x1>=x2)){ //$NON-NLS-1$ //$NON-NLS-2$
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];

		// left-top|bottom, right-top|bottom combination
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="right" && enddir=="top" && x1<x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x2,y:y1}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="right" && enddir=="top" && x1<x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];

		// top-left|right, bottom-left|right combination
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="top" && enddir=="right" && x1>=x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1<y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="top" && enddir=="right" && x1>=x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1>=y2) || //$NON-NLS-1$ //$NON-NLS-2$
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2-delta),y:(y1-delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2+delta),y:(y1-delta)},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2-delta),y:(y1+delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2+delta),y:(y1+delta)},{x:(x2+delta),y:y2}];

		// same direction
		}else if(
			(startdir=="left" && enddir=="left" && x1>=x2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x2-delta),y:y1},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="left" && enddir=="left" && x1<x2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1>=x2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1<x2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:(x2+delta),y:y1},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="top" && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="top" && enddir=="top" && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1-delta)},{x:x2,y:(y1-delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1>=y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y1+delta)},{x:x2,y:(y1+delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1<y2) //$NON-NLS-1$ //$NON-NLS-2$
			){
			points = [{x:x1,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else{ // includes left-right, right-left
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:y2}];
		}		
		points.unshift(this.owner.points[0]);
		points.push(this.owner.points[this.owner.points.length - 1]);
		return points;
	},
	
	update: function(){
		if(this.owner.points.length < 2){ return; }
		this.owner.points = this.getPoints();
	}
});