/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.helper.baseHelper"); //$NON-NLS-1$

// summary:
//		The base helper methods that are always mixed-in a widget
//		or an editor, and thus they are always available.
//		You can assume the editor itself and all the components on the editor
//		have these methods.
//		'this' points to the editor or a component.

com.ibm.relm.explorer.web.base.internal.helper.baseHelper = {
	hlp_getEditor: function(){
		if(this instanceof com.ibm.relm.explorer.web.base.internal.canvas.Editor){
			return this;
		}
		return this.__editArea;
	},

	hlp_setEditArea: function(editArea){
		this.__editArea = editArea;
	},

	hlp_getComponentInfo: function(/*String|Widget*/widget){
		var ed = this.hlp_getEditor();
		return ed.getComponentInfo && ed.getComponentInfo(widget || this);
	},

	hlp_getAttributeInfo: function(/*String|Widget*/widget, /*String*/attrName){
		var ed = this.hlp_getEditor();
		return ed.getAttributeInfo && ed.getAttributeInfo(widget || this, attrName);
	},

	hlp_getAttributes: function(/*String|Widget*/widget){
		var ed = this.hlp_getEditor();
		return ed.getAttributes && ed.getAttributes(widget || this);
	},

	hlp_getDragTracker: function(/*DomNode*/node){
		return this.dragMoveTrackerClass;
	},

	hlp_initLiveWidget: function(editArea){
		var info = this.hlp_getComponentInfo();
		if(info && info.dropTargets){
			if(this.hlp_dropTargets){
				this.hlp_dropTargets = this.hlp_dropTargets.concat(info.dropTargets.split(",")); //$NON-NLS-1$
			}else{
				this.hlp_dropTargets = info.dropTargets.split(","); //$NON-NLS-1$
			}
		}
		if(info && info.connectionTargets){
			if(this.hlp_connectionTargets){
				this.hlp_connectionTargets = this.hlp_connectionTargets.concat(info.connectionTargets.split(",")); //$NON-NLS-1$
			}else{
				this.hlp_connectionTargets = info.connectionTargets.split(","); //$NON-NLS-1$
			}
		}
//x		if(this.hlp_setupDropTarget){
//x			this.hlp_setupDropTarget();
//x		}
	},

	hlp_beforeAddChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_afterAddChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_beforeRemoveChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_afterRemoveChild: function(parent, widget){
		// Your widget may want to override this.
	},
	
	_getIndexOfChild: function(/*int|Widget?*/insertIndex){
		if(insertIndex && typeof(insertIndex) != "number"){ //$NON-NLS-1$
			// convert refWidget to insertIndex  
			insertIndex = dojo.indexOf(this.getChildren(), insertIndex);
			if(insertIndex == -1){ 
				dojo.error("Internal error: reference widget was not found");  //$NON-NLS-1$
			}
		}
		return insertIndex;		
	},

	hlp_addChild: function(/*Widget*/widget, /*int|Widget?*/insertIndex){
		insertIndex = this._getIndexOfChild(insertIndex);
		widget.hlp_beforeAddChild(this, widget);
		this.addChild(widget, insertIndex);
		if(!com.ibm.relm.explorer.web.base.internal.util.isOrphan(widget.domNode)){
			var hint = {
				parent: this,
				widget: widget,
				insertIndex: insertIndex
			};
			this.hlp_getEditor().firePropertyChanged("@AddChild", null, null, hint); //$NON-NLS-1$
			this.hlp_getEditor().adjustCanvasSize(widget);
		}
		widget.hlp_afterAddChild(this, widget);
	},

	hlp_reparentChild: function(/*Widget*/widget, /*int|Widget?*/insertIndex){
		var oldParent = widget.getParent();
		if(oldParent == this){
			var refIndex = insertIndex === undefined ? this.getChildren().length : this._getIndexOfChild(insertIndex);
			var currentIndex = this._getIndexOfChild(widget);
			if (refIndex === currentIndex || refIndex === currentIndex + 1) {
				// do nothing when the widget is being inserted at the left or the right of itself.
				return;
			}
		}
		if(oldParent){
			oldParent.removeChild(widget);
			if(dojo.isFunction(oldParent.layout)){
				oldParent.layout();
			}
		}
		insertIndex = this._getIndexOfChild(insertIndex);
		this.addChild(widget, insertIndex);		
		if(dojo.isFunction(this.layout)){
			this.layout();
		}
	},
	
	hlp_removeChild: function(/*Widget*/widget){
		widget.hlp_beforeRemoveChild(this, widget);
		var isOrphan = com.ibm.relm.explorer.web.base.internal.util.isOrphan(widget.domNode);
		this.removeChild(widget);
		if(!isOrphan){
			var hint = {
				parent: this,
				widget: widget
			};
			this.hlp_getEditor().firePropertyChanged("@RemoveChild", null, null, hint); //$NON-NLS-1$
		}
		widget.hlp_afterRemoveChild(this, widget);
	},

	hlp_getPropertyValue: function(name, onlyExplicit){
		var f = "get" + name.charAt(0).toUpperCase() + name.substring(1); //$NON-NLS-1$
		if(this[f]){
			return this[f](onlyExplicit);
		}
		return this[name];
	},

	hlp_setPropertyValue: function(name, value){
		var oldValue = this.hlp_getPropertyValue(name);
		var f = "set" + name.charAt(0).toUpperCase() + name.substring(1); //$NON-NLS-1$
		if(this[f]){
			this[f](value);
		}else{
			this[name] = value;
		}
		this.hlp_getEditor().firePropertyChanged(name, oldValue, value, {widget:this});
	},

	hlp_getNodeType: function(connectionWidget){
		var nodeType = this.nodeType || ""; //$NON-NLS-1$
		if(!nodeType){
			var nodeInfo = this.hlp_getComponentInfo();
			nodeType = nodeInfo && nodeInfo.nodeType || ""; //$NON-NLS-1$
		}

		var connType = connectionWidget.connectionType || ""; //$NON-NLS-1$
		if(!connType){
			var connInfo = this.hlp_getComponentInfo(connectionWidget);
			var connType = connInfo && connInfo.connectionType || ""; //$NON-NLS-1$
		}

		return nodeType + (connType ? ":" + connType : "") || "connection"; //$NON-NLS-1$ //$NON-NLS-2$  //$NON-NLS-3$
	}
};
