/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.helper.dropHelper"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DropTarget"); //$NON-NLS-1$

// summary:
//		Make a widget (component or Editor) "droppable".
//		'this' points to the Editor or a container type widget.

com.ibm.relm.explorer.web.base.internal.helper.dropHelper = {
	initialize: function(){
		this.hlp_setupDropTarget();
	},

	cleanup: function() {
		this._hlp_destroy();
	},
	
	hlp_dropTargets: ["component"], // each component's helper may want to override this //$NON-NLS-1$

	hlp_setupDropTarget: function(){
		if(!this.hlp_dropTargets || this.hlp_dropTargets.length == 0){ return false; }
		if (this._dt) {
			return true;
		}
		var editor = this.hlp_getEditor();
		var dt = this._dt = new com.ibm.relm.explorer.web.base.internal.dnd.DropTarget(this.domNode, this.hlp_dropTargets, this);
		dt.offsetParent = editor.domNode;
		var f = function(ds, dt){
			// If the drag source is an ancestor of the drop target, do not accept.
			var dsWidget = ds.data;
			var dtWidget = dt.data;
			if(!dsWidget && !dtWidget){ return true; }
			for(var w = dtWidget; w; w = dijit._WidgetBase.prototype.getParent.call(w)){
				if(w == dsWidget){
					return false;
				}
			}
			return true;
		};
		dt.registerIsAcceptable("drop", f); //$NON-NLS-1$
		editor.connect(dt, "onDragStart", "handleDragStart"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDragOver", "handleDragOver"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDragMove", "handleDragMove"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDragEnd", "handleDragEnd"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDropStart", "handleDropStart"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDrop", "handleDrop"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDropEnd", "handleDropEnd"); //$NON-NLS-1$ //$NON-NLS-2$
		editor.connect(dt, "onDragOut", "handleDragOut"); //$NON-NLS-1$ //$NON-NLS-2$

		if(this.hlp_showAllowDropFeedback){
			this.connect(dt, "showFeedback", "hlp_showAllowDropFeedback"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(this.hlp_hideAllowDropFeedback){
			this.connect(dt, "hideFeedback", "hlp_hideAllowDropFeedback"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		editor.connect(this, "destroy", dojo.hitch(this, function() { //$NON-NLS-1$
			this._hlp_destroy();
		}));
	},

	hlp_getDropTarget: function(){
		return this._dt;
	},
	
	_hlp_destroy: function() {
		if (this._dt) {
			this._dt.destroy();
			this._dt = null;
		}
	}
};
