/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.helper.focusFrameHelper"); //$NON-NLS-1$

// summary:
//		Shows a focus frame when a widget is focused.
//		'this' points to a node widget or a line widget.
(function(){
	
var showFocus = function (focusFrame, show){
	if(focusFrame && focusFrame.rawNode){
		focusFrame.rawNode.style.display = show ? "" : "none"; //$NON-NLS-1$//$NON-NLS-2$
	} 
};

com.ibm.relm.explorer.web.base.internal.helper.focusFrameHelper = {
	hlp_focusNodeColor: "#008ABF", // dark blue //$NON-NLS-1$
	hlp_focusLineColor: "#FFA500", // orange //$NON-NLS-1$
	hlp_focusRelatedNodeColor: "#FFA500", // orange //$NON-NLS-1$
	hlp_focusRelatedLineColor: "#FFA500", // orange //$NON-NLS-1$
	hlp_focusNodeWidth: 3,
	hlp_focusLineWidth: 3,
	hlp_focusRelatedNodeWidth: 3,
	hlp_focusRelatedLineWidth: 3,
	
	hlp_decorationIcon: null,

	hlp_focusNodeColorHalf: "#FFA500", // orange //$NON-NLS-1$
	hlp_focusLineColorHalf: "#FFA500", // orange //$NON-NLS-1$
	hlp_focusNodeWidthHalf: 5,
	hlp_focusLineWidthHalf: 5,

	hlp_dropNodeColor: "#ff00ff", // magenta //$NON-NLS-1$
	hlp_dropNodeWidth: 3,

	hlp_connectNodeColor: "#00ffff", // cyan //$NON-NLS-1$
	hlp_connectNodeWidth: 3,

	// ------------- Decoration ----------------------------
	hlp_createDecorationIcon: function(widget, iconUrl){
		if (!widget || !iconUrl) return null;
		var icon = dojo.doc.createElement("img");//$NON-NLS-1$
		icon.src = iconUrl;
		icon.alt = "";//$NON-NLS-1$
		icon.style.position = "absolute"; //$NON-NLS-1$
		icon.style.display = "none"; //$NON-NLS-1$

		return icon;
	},
	
	hlp_showDecoration: function(widget, show) {
		if(!this.hlp_decorationIcon) {
			if (!widget.getDecorationIcon || !widget.getDecorationIcon())
				return;
			
			this.hlp_decorationIcon = this.hlp_createDecorationIcon(widget, widget.getDecorationIcon());
			if(!this.hlp_decorationIcon)
				return;
			
			widget.hlp_getEditor().domNode.appendChild(this.hlp_decorationIcon);
		}
		
		if (show) {
			if (!widget.shape) return;
			var tbb = widget.shape.getTransformedBoundingBox();
			this.hlp_decorationIcon.style.left = tbb[0].x + "px"; //$NON-NLS-1$
			this.hlp_decorationIcon.style.top = tbb[0].y - (this.hlp_decorationIcon.height || 16) + "px"; //$NON-NLS-1$
		}

		if (show) {
			if (!this.__decorationHandles)
				this.__decorationHandles = [];
			this.__decorationHandles.push(this.connect(widget, "setPosition", function() { //$NON-NLS-1$
				this.hlp_updateDecorationPosition(widget);
			}));
			this.__decorationHandles.push(this.connect(widget._viewer, "updateResorceNodes", function() { //$NON-NLS-1$
				this.hlp_updateDecorationPosition(widget);
			}));

			this.hlp_decorationIcon.style.zIndex = "1100"; //$NON-NLS-1$
			this.hlp_decorationIcon.style.display = ""; //$NON-NLS-1$
		} else {
			dojo.forEach(this.__decorationHandles, this.disconnect);
			this.hlp_decorationIcon.style.zIndex = "0"; //$NON-NLS-1$
			this.hlp_decorationIcon.style.display = "none"; //$NON-NLS-1$
		}
	},

	hlp_updateDecorationPosition: function(widget) {
		if (!widget.shape || !this.hlp_decorationIcon) return;
		var tbb = widget.shape.getTransformedBoundingBox();
		this.hlp_decorationIcon.style.left = tbb[0].x + "px"; //$NON-NLS-1$
		this.hlp_decorationIcon.style.top = tbb[0].y - this.hlp_decorationIcon.height + "px"; //$NON-NLS-1$
	},
	
	// ------------- Focus Frame ----------------------------
	hlp_setFocusFrame: function(){
		if(this.isGfxLineWidget){
			this.hlp_showLineFrame(this.hlp_focusLineColor, this.hlp_focusLineWidth);
		}else{
			this.hlp_showNodeFrame(this.hlp_focusNodeColor, this.hlp_focusNodeWidth, true);
		}
	},

	hlp_removeFocusFrame: function(){
		if(this.isGfxLineWidget){
			this.hlp_hideLineFrame();
		}else{
			this.hlp_hideNodeFrame();
		}
	},
	
	hlp_setRelationFocusFrame: function() {
		if(this.isGfxLineWidget){
			this.hlp_showLineFrame(this.hlp_focusRelatedLineColor, this.hlp_focusRelatedLineWidth);
		}else{
			this.hlp_showNodeFrame(this.hlp_focusRelatedNodeColor, this.hlp_focusRelatedNodeWidth);
		}		
	},

	// ------------- Half Focus Frame ----------------------------
	hlp_setHalfFocusFrame: function(){
		if(this.isGfxLineWidget){
			this.hlp_showHalfLineFrame(this.hlp_focusLineColorHalf, this.hlp_focusLineWidthHalf);
		}else{
			this.hlp_showHalfNodeFrame(this.hlp_focusNodeColorHalf, this.hlp_focusNodeWidthHalf);
			this.hlp_showDecoration(this, true);
		}
	},

	hlp_removeHalfFocusFrame: function(){
		if(this.isGfxLineWidget){
			this.hlp_hideHalfLineFrame();
		}else{
			this.hlp_hideHalfNodeFrame();
			this.hlp_showDecoration(this, false);
		}
	},

	// ------------- Drop Feedback Frame ----------------------------
	hlp_showAllowDropFeedback: function(){
		if(!this.isGfxLineWidget){
			this.hlp_showNodeFrame(this.hlp_dropNodeColor, this.hlp_dropNodeWidth);
		}
	},

	hlp_hideAllowDropFeedback: function(){
		if(!this.isGfxLineWidget){
			this.hlp_hideNodeFrame();
		}
	},

	// ------------- Connect Feedback Frame ----------------------------
	hlp_showAllowConnectFeedback: function(){
		if(!this.isGfxLineWidget){
			this.hlp_showNodeFrame(this.hlp_connectNodeColor, this.hlp_connectNodeWidth);
		}
	},

	hlp_hideAllowConnectFeedback: function(){
		if(!this.isGfxLineWidget){
			this.hlp_hideNodeFrame();
		}
	},

	// -------- Show/Hide a frame around the node widget ----------
	hlp_showNodeFrame: function(/*String?*/color, /*Integer?*/width, /*boolean*/focusFrame){
		if(this == this.hlp_getEditor()){ return; }
		this._g1 = this._hlp_setupFocusFrameShape(this._g1, color, width, focusFrame);
		showFocus(this._g1, true);
	},

	hlp_hideNodeFrame: function(){
		if(this == this.hlp_getEditor()){ return; }
		showFocus(this._g1, false);
	},

	// -------- Show/Hide a frame around the line widget ----------
	hlp_showLineFrame: function(/*String?*/color, /*Integer?*/width){
		this._g2 = this._hlp_setupFocusLineShape(this._g2, color, width);
		showFocus(this._g2, true);
	},

	hlp_hideLineFrame: function(){
		if(this == this.hlp_getEditor()){ return; }
		showFocus(this._g2, false);
	},
	
	// -------- Show/Hide a half frame around the node widget ----------
	hlp_showHalfNodeFrame: function(/*String?*/color, /*Integer?*/width){
		if(this == this.hlp_getEditor()){ return; }
		this._g3 = this._hlp_setupFocusFrameShape(this._g3, color, width);
		showFocus(this._g3, true);
	},

	hlp_hideHalfNodeFrame: function(){
		if(this == this.hlp_getEditor()){ return; }
		showFocus(this._g3, false);
	},

	// -------- Show/Hide a frame around the line widget ----------
	hlp_showHalfLineFrame: function(/*String?*/color, /*Integer?*/width){
		if(this == this.hlp_getEditor()){ return; }
		this._g4 = this._hlp_setupFocusLineShape(this._g4, color, width);
		showFocus(this._g4, true);
	},

	hlp_hideHalfLineFrame: function(){
		if(this == this.hlp_getEditor()){ return; }
		showFocus(this._g4, false);
	},
	
	// --------- internal -----------
	_hlp_setupFocusFrameShape: function(focusShape, color, width, focusFrame){
		if(!focusShape){
			focusShape = this.group.createGroup();
		}
		width = (width || 3);
		var numShapes = Math.floor(width/2)+1;
		var numChild = focusShape.children.length;
		if(numChild < numShapes){
			for(var i = numChild; i < numShapes; i++){
				focusShape.createRect();
			}						
		}else if(numChild > numShapes){
			for(var i = numChild - 1; i >= numShapes; i--){
				var c = focusShape.children[i];
				c.removeShape().destroy();
			}
		}
		focusShape._moveToFront();
		
		var r = this.r || 4;
		var col = new dojo.Color(color || "yellow"); //$NON-NLS-1$
		var c = col.r <= col.g ? "g" : "r"; //$NON-NLS-1$ //$NON-NLS-2$
		c = col[c] <= col.b ? "b" : c; //$NON-NLS-1$

		var w = this.getWidth();
		var h = this.getHeight();

		var d = Math.round(col[c] * 0.1);
		for(var i = 0; i < numShapes; i++){
			var ww = i*2;
			col.a = 0.5 - 0.1*(width-ww-1);
			focusShape.children[i].setShape({x:0, y:0, width:w, height:h, r:r})
				.setStroke({color: col.toRgba(), width: width-ww, join: "round" }); //$NON-NLS-1$
			col.c -= d;
		}
		if(focusFrame){
			var padding = numShapes+1;
			focusShape.createRect({x:-padding, y:-padding, width:w-0+padding*2, height:h-0+padding*2, r:r-1+padding})
				.setStroke({color: "black", width: 1, join: "round", style: "ShortDot"});//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$			
		}
		
		return focusShape;
	},

	_hlp_setupFocusLineShape: function(focusLine, color, width){
		width = (width || 3);
		var numShapes = Math.floor(width/2)+1;
		var rawShape = this.shape.getShape();
		if(!focusLine){
			focusLine = this.group.createGroup();
			for(var i = 0; i < numShapes; i++){
				if(com.ibm.relm.explorer.web.base.internal.gdc.Polyline 
					&& this instanceof com.ibm.relm.explorer.web.base.internal.gdc.Polyline
					&& rawShape.type != "path") { //$NON-NLS-1$
					focusLine.createPolyline();
				}else{
					focusLine.createPath();
				}
			}
			focusLine._moveToFront();
		}

		var col = new dojo.Color(color || "yellow"); //$NON-NLS-1$
		var c = col.r <= col.g ? "g" : "r"; //$NON-NLS-1$ //$NON-NLS-2$
		c = col[c] <= col.b ? "b" : c; //$NON-NLS-1$

		var d = Math.round(col[c] * 0.1);
		for(var i = 0; i < numShapes; i++){
			var ww = i*2;
			col.a = 0.5 - 0.1*(width-ww-1);
			focusLine.children[i].setShape(rawShape)
				.setStroke({width: width-ww, color: col.toRgba()});
			col.c -= d;
		}
		
		return focusLine;
	}
};

})();