/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.helper.styleHelper"); //$NON-NLS-1$

com.ibm.relm.explorer.web.base.internal.helper.styleHelper = {
	// summary: get style value from an accessor if available or from domNode.
	// property: "top", "left", "width", "height", or "borderWidth"
	// return: integer value
	hlp_getStyle: function(/*String*/property){
		var info = this.hlp_getComponentInfo();
		if(info && info.accessors){
			var ac = info.accessors;
			var cap = com.ibm.relm.explorer.web.base.internal.util.cap(property);
			if(ac["int"+cap+"Accessor"]){ //$NON-NLS-1$ //$NON-NLS-2$
				var accessor = ac["int"+cap+"Accessor"]; //$NON-NLS-1$ //$NON-NLS-2$
				var getter = "get"+com.ibm.relm.explorer.web.base.internal.util.cap(accessor); //$NON-NLS-1$
				return this[getter] ? parseInt(this[getter]()) : parseInt(this[accessor]);
			}else if(ac[property+"Accessor"]){ //$NON-NLS-1$
				var accessor = ac[property+"Accessor"]; //$NON-NLS-1$
				var getter = "get"+com.ibm.relm.explorer.web.base.internal.util.cap(accessor); //$NON-NLS-1$
				return this[getter] ? this[getter]() : this[accessor];
			}else if(ac[property]){
				return parseInt(ac[property]);
			}
			// through
		}

		if(this.isGfxNodeWidget){
			return this["get"+com.ibm.relm.explorer.web.base.internal.util.cap(property)](); //$NON-NLS-1$
		}else{
			// To get the value of top and left, you need to use dojo.style.
			// On FF, widget.domNode.offsetTop/Left always returns zero when the EditArea is hidden.
			if(property == "top"){ //$NON-NLS-1$
				return parseInt(dojo.style(this.domNode, "top")); //$NON-NLS-1$
			}else if(property == "left"){ //$NON-NLS-1$
				return parseInt(dojo.style(this.domNode, "left")); //$NON-NLS-1$
			}else if(property == "width"){ //$NON-NLS-1$
				return dojo.marginBox(this.domNode).w;
			}else if(property == "height"){ //$NON-NLS-1$
				return dojo.marginBox(this.domNode).h;
			}
		}
		return null;
	},

	hlp_setStyle: function(/*String*/property, /*String*/value, /*Boolean*/update){
		// summary:
		//		Sets a style through an accessor if available or directly to domNode
		//		and performs necessary updates.
		// property:
		//		"top", "left", etc.
		// value:
		//		string value with "px". (Ex. "100px")
		// update:
		//		If true, calls some functions to update the canvas. Default is true.
		var props = {};
		props[property] = value;
		this.hlp_setStyles(props, update);
	},

	hlp_setStyles: function(/*Object*/props, /*Boolean*/update){
		// summary:
		//		Sets multiple styles through accessors or directly to domNode
		//		and performs necessary updates.
		// props:
		//		Object of styles and values.
		// update:
		//		If true, calls some functions to update the canvas. Default is true.
		if(!props){ return; }
		var p, value;

		var info = this.hlp_getComponentInfo();
		for(p in props){
			value = props[p];
			if(info && info.accessors){
				var ac = info.accessors;
				var cap = com.ibm.relm.explorer.web.base.internal.util.cap(p);
				if(ac["int"+cap+"Accessor"]){ //$NON-NLS-1$ //$NON-NLS-2$
					var accessor = ac["int"+cap+"Accessor"]; //$NON-NLS-1$ //$NON-NLS-2$
					var setter = "set"+com.ibm.relm.explorer.web.base.internal.util.cap(accessor); //$NON-NLS-1$
					if(this[setter]){
						this[setter](parseInt(value));
					}else{
						this[accessor] = parseInt(value);
					}
					return;
				}else if(ac[p+"Accessor"]){ //$NON-NLS-1$
					var accessor = ac[p+"Accessor"]; //$NON-NLS-1$
					var setter = "set"+com.ibm.relm.explorer.web.base.internal.util.cap(accessor); //$NON-NLS-1$
					if(this[setter]){
						this[setter](value);
					}else{
						this[accessor] = value;
					}
					return;
				}
				// through
			}
			if(this.isGfxNodeWidget){
				this["set"+com.ibm.relm.explorer.web.base.internal.util.cap(p)](parseInt(value)); //$NON-NLS-1$
			}else{
				this.domNode.style[p] = value;
			}
		}
		if(update == false){ return; }

		var info = this.hlp_getComponentInfo();
		if(info && info.resizable && info.resizable.refreshMethod){
			this[info.resizable.refreshMethod]();
		}
		var ed = this.hlp_getEditor();
		if(ed.adjustCanvasSize){
			ed.adjustCanvasSize(this);
		}
		if(this.isGfxWidget){
			this.update();
		}
		ed.refreshFocusFrame();
		if(ed.firePropertyChanged){
			ed.firePropertyChanged("@SetStyles", null, null, {widget:this,props:props}); //$NON-NLS-1$
		}
	},

	// summary: generates value for the style attribute.
	//   Ex. "position:absolute;top:25px;left:19px;width:429px;height:289px;"
	hlp_generateStyleValues: function(){
		var s = ""; //$NON-NLS-1$
		var position = dojo.style(this.domNode, "position"); //$NON-NLS-1$
		if(position && (position == "absolute" || position == "relative")){ //$NON-NLS-1$ //$NON-NLS-2$
			s += "position:"+position+";"; //$NON-NLS-1$ //$NON-NLS-2$

			// Top
			if(!this.hlp_hasAccessor("top")){ //$NON-NLS-1$
				var val = this.hlp_getStyle("top"); //$NON-NLS-1$
				if(val && val != "auto"){ //$NON-NLS-1$
					s += "top:" + val + "px;"; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}

			// Left
			if(!this.hlp_hasAccessor("left")){ //$NON-NLS-1$
				var val = this.hlp_getStyle("left"); //$NON-NLS-1$
				if(val && val != "auto"){ //$NON-NLS-1$
					s += "left:" + val + "px;"; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}

		var info = this.hlp_getComponentInfo();
		if(info && info.resizable){
			// Width
			if(!this.hlp_hasAccessor("width") && this.hlp_isResizableHorizontally()){ //$NON-NLS-1$
				var val = this.domNode.style.width;
				if(val && val != "auto"){ //$NON-NLS-1$
					s += "width:" + val + ";"; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}

			// Height
			if(!this.hlp_hasAccessor("height") && this.hlp_isResizableVertically()){ //$NON-NLS-1$
				var val = this.domNode.style.height;
				if(val && val != "auto"){ //$NON-NLS-1$
					s += "height:" + val + ";"; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		return s;
	},

	hlp_hasAccessor: function(property){
		var info = this.hlp_getComponentInfo();
		if(!info){ return false; }
		var cap = com.ibm.relm.explorer.web.base.internal.util.cap(property);
		if(info.accessors && (info.accessors["int"+cap+"Accessor"] || info.accessors[cap+"Accessor"])){ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return true;
		}else{
			return false;
		}
	},

	hlp_isResizableHorizontally: function(){
		var info = this.hlp_getComponentInfo();
		if(!info || !info.resizable || !info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("EAST") != -1 //$NON-NLS-1$
			|| info.resizable.directions.indexOf("WEST") != -1; //$NON-NLS-1$
	},

	hlp_isResizableVertically: function(){
		var info = this.hlp_getComponentInfo();
		if(!info || !info.resizable || !info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("NORTH") != -1 //$NON-NLS-1$
			|| info.resizable.directions.indexOf("SOUTH") != -1; //$NON-NLS-1$
	},


	hlp_getStyleDimensions: function(widget, parent){
		var dim = com.ibm.relm.explorer.web.base.internal.util.getDimensions(widget.domNode, parent.domNode);
		dim.width = widget.hlp_getStyle("width"); //$NON-NLS-1$
		dim.height = widget.hlp_getStyle("height"); //$NON-NLS-1$
		return dim;
	}
};
