/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights: Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.leeditor.page.FolderEditor"); //$NON-NLS-1$

dojo.require("com.ibm.relm.common.web.ui.RequiredMarker");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.ImagePool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$

dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.leeditor.internal", "FolderEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var LEServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
var ImagePool = com.ibm.relm.explorer.web.explorer.page.ImagePool;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var LELIST_URI = "#action=com.ibm.relm.le.leeditor.showLEList"; //$NON-NLS-1$
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ns = com.ibm.relm.le.ui.namespace;
var TYPE_FOLDER = ns.ae.Folder;
var RH_ETAG = "ETag"; //$NON-NLS-1$
var MAX_MESSAGE_LENGTH = 100;

/**
 * Widget to create a new Folder
 */
var FolderEditor = dojo.declare("com.ibm.relm.le.leeditor.page.FolderEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.relm.le", "leeditor/page/templates/FolderEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		_csPageHelpWidgets: null,
		csHelpNode: null,
		
		parentUri: null,
		folderUri: null,
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.internal", "FolderEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.dirty = false;
			this._csPageHelpWidgets= [];
			this._urlWhenClose = LELIST_URI;
			this._mode = args.mode;
		},
				
		postCreate: function() {
			this.inherited(arguments);
			this.typeIcon.src = dojo.moduleUrl("com.ibm.relm.le", "ui/images/folder.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if(this._mode == "create") { //$NON-NLS-1$
				this._saveButton.firstChild.nodeValue = this.messages.create;
			}
			// The folder. We may not have one initially.
			this._folder = null;
			
			if(this.folderUri) {
				// for le 
				LEServiceUtils.getElement(this.folderUri, dojo.hitch(this, this._setFolder), dojo.hitch(this, this._remoteError));
			} else {
				this._setEmptyFolder();
			}
			
			this._createContextSensitiveHelp();
			
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);
			}
		},
		
		_createContextSensitiveHelp: function() {
			dojo.empty(this.csHelpNode);
			this._createCSHWidget("com.ibm.relm.explorer.web.page_CreateFolder001"); //$NON-NLS-1$
		},
	
		_createCSHWidget: function(uaKey) {
			var parms = {};
			parms.uaKey = uaKey;
			var pageHelp = new PageHoverHelp(parms);
			this.csHelpNode.appendChild(pageHelp.domNode);
			this._csPageHelpWidgets.push(pageHelp);
		},
		
		/**
		 * Callback function to populate the page with the given folder.
		 */
		_setFolder: function(folder, ioArgs) {
			this._eTag = ioArgs.xhr.getResponseHeader(RH_ETAG);
			this._folder = folder;
			this._folder.folderUUID = folder.id;
			this._folder.name = folder.title;
			
			// Set the widgets up
			this._nameInput.value = folder.name;
			if (folder.description) {
				this._descriptionText.value = folder.description;
			}
			
		},
			
		/**
		 * Populate the page with no initial folder.
		 */
		_setEmptyFolder: function() {
			this._folder = new Object();
			// Check if a processArea was supplied
			if (this.processAreaUUID) {
				this._folder.processAreaUUID = this.processAreaUUID;
			} else {
				// Assume we want the project area for sharing
				var currentProjectArea = jazz.app.context.get();
				this._folder.processAreaUUID = currentProjectArea.itemId;
			}
			// Check if a parent folder was specified
			if (this.parentUUID) {
				this._folder.parentUUID = this.parentUUID;
			}
		},
					
		
		_handleSave: function() {
			var props = {
					type: TYPE_FOLDER,
					title: this._getName(),
					description: this._getDescription() || "" //$NON-NLS-1$
			};
			if(this._mode == "create") { //$NON-NLS-1$
				if(this.parentUri) {
					// create under folder
					LEServiceUtils.createChildElement(
							this.parentUri, 
							null,
							props,
							dojo.hitch(this, this._saveComplete),
							dojo.hitch(this, this._remoteError));
				} else {
					// create at root
					LEServiceUtils.createElement(
							null,
							props,
							dojo.hitch(this, this._saveComplete),
							dojo.hitch(this, this._remoteError));
				}
			} else {
				props.id = this._folder.folderUUID;
				var headers = {};
				headers[RH_ETAG] = this._eTag;
				LEServiceUtils.updateElement(
						this.folderUri,
						props,
						{headers: headers},
						dojo.hitch(this, this._saveComplete),
						dojo.hitch(this, this._remoteError));
			}
		},
		
		destroy: function() {
			if (this.navigateAwayHandle) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
			}
			if (this.onBeforeUnloadHandle) {
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
			}	
			dojo.forEach(this._csPageHelpWidgets, function(w) { w.destroy; });
			this.inherited(arguments);
		},
		
		_handleCancel: function() {
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) { //$NON-NLS-1$ //$NON-NLS-2$
					this._closeAndReturn();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this._closeAndReturn();
				return false;
			}
		},
		
		_closeAndReturn: function() {
			this.destroy();
			window.location.href = this._urlWhenClose;
		},
		
		_handleNavigateAway: function(url, e) {
			// Allow help links to open in a new window without ado
			if (e && e.target && (e.target.tagName == "A") && (e.target.target == "jazzhelp")) { //$NON-NLS-1$ //$NON-NLS-2$
				return false; // allow navigation
			}
			
			var currentLocation = window.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1); //$NON-NLS-1$
			if (url == badLink) {
				return true;
			}
			
			if (this.dirty) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) { //$NON-NLS-1$ //$NON-NLS-2$
					this.destroy();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				return false;
			}
		},
		
		_handleOnBeforeUnload: function() {
			return (this.dirty) ? this.messages.unsavedChanges + "\n\n" : false; //$NON-NLS-1$
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_saveComplete: function(args) {
			if(this._mode == "modify") { //$NON-NLS-1$
				LEServiceUtils.getElement(this.folderUri, dojo.hitch(this, this._setFolder), dojo.hitch(this, this._remoteError));
			}
			this._setDirty(false);
			if(this._mode == "create") { //$NON-NLS-1$
				this._closeAndReturn();
			}
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this.setHeaderMessage({
				message: error.message,
				showOnlyMessage: true,
				severity: 8
			});
		},
		
		_handleNameChange: function() {
			this._setDirty(true);
		},
		
		_handleIdChange: function() {
			this._setDirty(true);
		},
		
		_handleDescriptionChange: function() {
			this._setDirty(true);
		},
		
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && (name.length > 0));
			this._saveButton.disabled = (!this.dirty || !hasName);
		},
		
		_setDirty: function(isDirty) {
			this.dirty = isDirty;
			this._updateEnablement();
		},
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			this.validationMessageSpan.style.display = "";//$NON-NLS-1$
			if (!message) {
				return;
			}
			
			if (!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) { //$NON-NLS-1$
					message.message += "."; //$NON-NLS-1$
				}
			}
			
			if (!message.details) {
				message.details = ""; //$NON-NLS-1$
			} else {
				if (message.details.lastIndexOf(".") != (message.details.length - 1)) { //$NON-NLS-1$
					message.details += "."; //$NON-NLS-1$
				}
			}
			var messageText;
			if (message.message.length > MAX_MESSAGE_LENGTH) {
				messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "..."; //$NON-NLS-1$
			} else {
				if (message.showOnlyMessage == true) {
					messageText = message.message;
				} else {
					messageText = message.message + " " + message.details; //$NON-NLS-1$
				}
				if (messageText.length > MAX_MESSAGE_LENGTH) {
					messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "..."; //$NON-NLS-1$
				}
			}
			messageText = messageText.replace(/\.\.\.\./, "..."); //$NON-NLS-1$
			
			this.validationMessage.__jazzMessage = message;
			if (message.severity==8) {
				this.typeIcon.src = ImagePool.ERROR;
				this.validationMessageSpan.className = "validationMessageError"; //$NON-NLS-1$
			} else {
				if (message.severity==4) {
					this.typeIcon.src = ImagePool.WARNING;
					this.validationMessageSpan.className = "validationMessageWarning"; //$NON-NLS-1$
				} else {
					this.typeIcon.src = ImagePool.INFO;
					this.validationMessageSpan.className = "validationMessageInfo"; //$NON-NLS-1$
				}
			}
			this.validationMessage.title = message.details || message.message;
			dojox.xml.parser.replaceChildren(this.validationMessage, document.createTextNode(messageText));
			window.scrollTo(0,0); // show message
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n"; //$NON-NLS-1$
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.validationMessageSpan.className = "validationMessageEmpty"; //$NON-NLS-1$
			this.validationMessageSpan.style.display = "none";//$NON-NLS-1$
		}
	}
);

 FolderEditor.CSH_FOLDER_EDITOR = "com.ibm.jazz.reports.web.cshelp.folder_editor"; //$NON-NLS-1$

})();
