/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.le.leeditor.queryparams.SystemParameterUtil"); //$NON-NLS-1$

dojo.require('dojo.date.stamp'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$

(function(){
	// Format: $paramName$
	var PARAMETER_REGEXS = {
		USER: /\$system\.user\$/,
		DATE: /\$system\.date\$/,
		DATETIME: /\$system\.dateTime\$/,
		TIME: /\$system\.time\$/
	};

	var PARAMETER_NAMES = {
		USER: "user", //$NON-NLS-1$
		DATE: "date", //$NON-NLS-1$
		DATETIME: "dateTime", //$NON-NLS-1$
		TIME: "time" //$NON-NLS-1$
	};

	var SYSTEM_PARAMETER_NAMES = {
		USER: "system.user", //$NON-NLS-1$
		DATE: "system.date", //$NON-NLS-1$
		DATETIME: "system.dateTime", //$NON-NLS-1$
		TIME: "system.time" //$NON-NLS-1$
	};

	dojo.mixin(com.ibm.relm.le.leeditor.queryparams.SystemParameterUtil, {
		getUser: function() {
			var user = com.ibm.team.repository.web.client.session.getAuthenticatedContributor();
			var systemValue = {
				type: "user", //$NON-NLS-1$
				value: {
					uri: com.ibm.team.repository.web.client.session.getAuthenticatedContributorUri(),
					name: user.name,
					emailAddress: user.emailAddress,
					mboxUri: "mailto:" + user.emailAddress //$NON-NLS-1$
				}
			};

			return {
				displayValue: com.ibm.team.repository.web.client.session.getAuthenticatedContributor().name,
				systemValue: systemValue
			};
		},

		getDate: function(date) {
			return {
				displayValue: dojo.date.locale.format(date, {formatLength: "medium", selector: "date"}), //$NON-NLS-1$ //$NON-NLS-2$,
				systemValue: {
					type: "date", //$NON-NLS-1$
					value: dojo.date.stamp.toISOString(date/*, {selector: "date"}*/)
				}
			};
		},

		getDateTime: function(date) {
			return {
				displayValue: dojo.date.locale.format(date, {formatLength: "medium"}), //$NON-NLS-1$
				systemValue: {
					type: "dateTime", //$NON-NLS-1$
					value: dojo.date.stamp.toISOString(date)
				}
			};
		},

		getTime: function(date) {
			return {
				displayValue: dojo.date.locale.format(date, {formatLength: "medium", selector: "time"}), //$NON-NLS-1$ //$NON-NLS-2$
				systemValue: {
					type: "time", //$NON-NLS-1$
					value: dojo.date.stamp.toISOString(date/*, {selector: "time"}*/)
				}
			};
		},

		getValues: function(type) {
			var date = new Date();
			var values = {};
			//For current time or date you can pass "now" into template
			// <div dojoType=... when="now"></div>
			// https://dojotoolkit.org/reference-guide/1.6/dojo/parser.html		
			switch (type) {
				case 'user': //$NON-NLS-1$
					var user = this.getUser();
					values.displayValue = user.displayValue;
					values.systemValue = user.systemValue;
					break;
				case 'date': //$NON-NLS-1$
					var date = this.getDate(date);
					values.displayValue = date.displayValue;
					values.systemValue = date.systemValue;
					break;
				case 'dateTime': //$NON-NLS-1$
					var dateTime = this.getDateTime(date);
					values.displayValue = dateTime.displayValue;
					values.systemValue = dateTime.systemValue;
					break;
				case 'time': //$NON-NLS-1$
					var time = this.getTime(date);
					values.displayValue = time.displayValue;
					values.systemValue = time.systemValue;
					break;
				default:
					throw ("Unknown system paremter type"); //$NON-NLS-1$
			}
			return values;
		},

		getSystemValue: function(type) {
			return this.getValues(type).systemValue;
		},

		getSystemParameterValues: function() {
			var date = new Date();
			var parameters = {};
				parameters[PARAMETER_NAMES.USER] = this.getUser().systemValue;
				parameters[PARAMETER_NAMES.DATE] = this.getDate(date).systemValue;
				parameters[PARAMETER_NAMES.DATETIME] = this.getDateTime(date).systemValue;
				parameters[PARAMETER_NAMES.TIME] = this.getTime(date).systemValue;
			return {
				system: {
					type: "system", //$NON-NLS-1$
					value: parameters
				}
			};
		},

		getParametersFromQueryInfo: function(queryInfo, parametersOnQuery) {
			parametersOnQuery = parametersOnQuery || [];
			var namespaces = com.ibm.relm.le.ui.namespace;

			var parameters = [];

			dojo.forEach(parametersOnQuery, function(bnode, i) {
				var paramData = queryInfo[bnode.value];

				var nameObj = paramData[namespaces.parameter.parameterName];
				var parameterClassObj = paramData[namespaces.parameter.parameterClass];
				var typeObj = paramData[namespaces.parameter.parameterType];
				var labelObj = paramData[namespaces.parameter.parameterLabel];

				var parameter = {
					name: nameObj[0].value,
					parameterClass: parameterClassObj[0].value,
					type: typeObj[0].value,
					label: labelObj[0].value
				};
				parameters.push(parameter);
			});

			return parameters;
		}
	});
})();