/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.le.leeditor.queryparams.paramprompt.ParameterPromptDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.le.leeditor.queryparams.paramprompt", "ParameterPromptDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.form.ComboBox'); //$NON-NLS-1$
dojo.require('jazz.client.xhr'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.ArtifactPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.BooleanPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.DatePrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.NumberPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.EnumPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.ProjectAreaPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.SystemPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.paramprompt.StringPrompt"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.leeditor.queryparams.SystemParameterUtil"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.enums.EnumStore"); //$NON-NLS-1$

(function(){

	var ArtifactPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.ArtifactPrompt;
	var BooleanPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.BooleanPrompt;
	var DatePrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.DatePrompt;
	var NumberPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.NumberPrompt;
	var EnumPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.EnumPrompt;
	var ProjectAreaPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.ProjectAreaPrompt;
	var SystemPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.SystemPrompt;
	var StringPrompt = com.ibm.relm.le.leeditor.queryparams.paramprompt.StringPrompt;
	var EnumStore = com.ibm.relm.common.web.enums.EnumStore;

	dojo.declare(
		"com.ibm.relm.le.leeditor.queryparams.paramprompt.ParameterPromptDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.le','leeditor/queryparams/paramprompt/templates/ParameterPromptDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.le.leeditor.queryparams.paramprompt', 'ParameterPromptDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			width: 400,
			
			/**
			 * @param args
			 * 	Required fields:
			 * 		args.parameterValues - 
			 * 		args.queryName - The name of the query
			 * 		args.queryUri - The URI of the query
			 * 		args.isDirty - Whether or not the query has been modified (new or unsaved changes)
			 * 		args.parameters - Array of { name: x, parameterClass: y, category: z, label: n }
			 *		args._onSubmit - Run Query handler
			 *		args._onSaveAs(sparql) - Save as handler
			 *		args._onCancel - Cancel handler
			 * @param node Optional node to replace.
			*/
			constructor: function(args, node) {
				this._helpHeader = args.queryName
					? dojo.replace(this.messages.helpHeaderQueryX, [jazz.util.html.escape(args.queryName)])
					: this.messages.helpHeader;

				this.prompts = [];
			},

			postCreate: function() {
				this.parameters.sort(function(param1, param2) {
					var label1 = param1.label; var label2 = param2.label;
					if (label1 < label2)
						return -1;
					else if (label1 > label2)
						return 1;
					else return 0;
				});
				var containsEnums = dojo.some(this.parameters, function(parameter){ 
					return parameter.parameterClass == "enum"; //$NON-NLS-1$
				});
				var containtsNonSystemParameters = dojo.some(this.parameters, function(parameter){ 
					return parameter.parameterClass != "system"; //$NON-NLS-1$
				});

				if (!this.parameters || !this.parameters.length || !containtsNonSystemParameters) {
					this._initializePrompts();
					this._submit();
					this.destroyRecursive();
					return;
				}
				
				if (!this._onSaveAs || !this.queryUri) {
					this._saveAsButton.style.display = 'none'; //$NON-NLS-1$
				}
				else {
					// only one query can be edited at a time
					this._saveAsButton.style.display = this.isDirty ? 'none' : ''; //$NON-NLS-1$ //$NON-NLS-2$
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: this.width + 'px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					closable: false,
					resizeable: true,
					iconClass: 'explorer-queryparams-paramPrompt-icon' //$NON-NLS-1$
				});

				if (containsEnums) {
					this.enumStore = new EnumStore();
					this.loadingEnums = this.enumStore.load();
					this.loadingEnums.then(
						dojo.hitch(this, this._drawPrompts), 
						dojo.hitch(this, function(errObj) {
							this._loadingMessage.hide();
							this._messageBox.showErrorObject(errObj);
						})
					);
				} else {
					this._drawPrompts();
				}
			},

			_initializePrompts: function() {
				dojo.forEach(this.parameters, function(parameter) {
					var prompt;
					var parameterClass = parameter.parameterClass;

					var promptArgs = dojo.mixin({}, parameter);
					
					if (this.parameterValues && this.parameterValues[parameter.name]) {
						promptArgs.value = this.parameterValues[parameter.name];
					}
					
					if (!parameterClass || parameterClass == "simple") { //$NON-NLS-1$
						switch(parameter.type) {
							case "boolean": //$NON-NLS-1$
								prompt = new BooleanPrompt(promptArgs);
								break;
							case "date": //$NON-NLS-1$
								prompt = new DatePrompt(promptArgs);
								break;
							case "number": //$NON-NLS-1$
								prompt = new NumberPrompt(promptArgs);
								break;
							case "string": //$NON-NLS-1$
								prompt = new StringPrompt(promptArgs);
								break;
							case "artifact": //$NON-NLS-1$
								prompt = new ArtifactPrompt(promptArgs);
								break;
							case "projectArea": //$NON-NLS-1$
								prompt = new ProjectAreaPrompt(promptArgs);
								break;
							default:
								throw new Error("Unknown parameter type: " + parameter.type); //$NON-NLS-1$
						}
					} else if (parameterClass == "enum") { //$NON-NLS-1$
							promptArgs.enumStore = this.enumStore;
							promptArgs.messageBox = this._messageBox;
							prompt = new EnumPrompt(promptArgs);
					} else if (parameterClass == "system") { //$NON-NLS-1$
							prompt = new SystemPrompt(promptArgs); 
					} else
						throw new Error("Unknown parameter class: " + parameterClass); //$NON-NLS-1$
					prompt.startup();
					this.connect(prompt, 'onChange', this.monitorRunButton); //$NON-NLS-1$
					this.prompts.push(prompt);
					this.monitorRunButton();
				}, this);
			},

			_drawPrompts: function() {
				this._initializePrompts();
				var frag = document.createDocumentFragment();
				dojo.forEach(this.prompts, function(prompt) { 
					dojo.place(prompt.domNode, frag);
				});
				this.paramPrompts.appendChild(frag);
				this._loadingMessage.hide();
				this._dialog.updatePosition();
			},

			/**
			* This should connect to the onChange event of every row
			* 
			*/
			monitorRunButton: function() {
				this._submitButton.disabled = false;
				this._saveAsButton.disabled = false;
			},

			/**
			* This should be passed in by the caller
			*/
			_onSubmit: null,
			
			/**
			* This should be passed in by the caller
			*/
			_onSaveAs: null,
			
			/**
			* This can be passed in by the caller
			*/
			_onCancel: function() {
				//No-op
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				if (!this._dialog) return;
				this._dialog.close();
				delete this._dialog;
			},
			
			getParameterValues: function() {
				var values = {};
				dojo.forEach(this.prompts, function(dataPrompt) {
//					var passThroughIfNoValue = true;
					var passThroughIfNoValue = false;
					dataPrompt.getNameAndValue(values, passThroughIfNoValue);
				}, this);
				
				return values;
			},

			/**
			* Execute onSubmit with filled in parameters as args
			*/
			_submit: function(e) {
				e && dojo.stopEvent(e);
				var values = this.getParameterValues();
				this._onSubmit(values);
				//TODO: validation and enabling button
				this._closeDialog();
			},

			/**
			 * Save the specified parameter values into a new query.
			 */
			_saveAs: function() {
				this._messageBox.hideMessage();
				
				var parameterValues = this.getParameterValues();
				var systemParameterValues = com.ibm.relm.le.leeditor.queryparams.SystemParameterUtil.getSystemParameterValues();
				parameterValues = dojo.mixin(parameterValues, systemParameterValues);
				
				var self = this;
				this.getSubstitutedQuery(this.queryUri, parameterValues).then(function(sparql) {
					self._closeDialog();
					self._onSaveAs && self._onSaveAs(sparql);
				}, function(err) {
					self._messageBox.showErrorObject(err);
				});
			},
			
			/**
			 * Makes a server-side request to produce the final query with substitutions
			 * when provided the set of parameter values.
			 * 
			 * @param queryUri The URI of the query
			 * @param parameterValues The parameters for the query.
			 * @return {Deferred}
			 */
			getSubstitutedQuery: function(queryUri, parameterValues) {
				var deferred = new dojo.Deferred();
				
				jazz.client.xhrGet({
					url: com.ibm.relm.le.ui.queryServiceUri,
					handleAs: "text", //$NON-NLS-1$
					failOk: true,
					content: {
						method: "getSubstitutedQuery", //$NON-NLS-1$
						queryURI: queryUri,
						parameterValues: dojo.toJson(parameterValues)
					},

					load: function(sparql) {
						deferred.callback(sparql);
					},

					error: function(error, ioArgs) {
						deferred.errback(dojo.fromJson(ioArgs.xhr.responseText));
					}
				});
				
				return deferred;
			},
			
			/**
			 * Cancels the create and closes the dialog.
			 */
			cancel: function() {
				console.debug("Canceling dialog"); //$NON-NLS-1$
				// this calls back to destroy outer dialog too
				this._closeDialog();
				this._onCancel();
			},

			uninitialize: function() {
				dojo.forEach(this.prompts, function(w) { w.destroyRecursive(); });
			}
		});

	})();
