/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.productedit.RenameResourceDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.productedit', 'RenameResourceDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){
	var Site = com.ibm.relm.ple.web.ui.Site;

	dojo.declare(
		'com.ibm.relm.ple.web.productedit.RenameResourceDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','productedit/templates/RenameResourceDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.productedit', 'RenameResourceDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			site: null,
			globalConfiguration: null,

			constructor: function(args) {
				this._nameId = dijit.getUniqueId('RenameResourceDialog'); //$NON-NLS-1$
				this.nameLabel = (args.site.type == "configVersion") ?  //$NON-NLS-1$
					this.messages.configNameLabel : this.messages.productNameLabel;
			},

			save: function(e) {
				dojo.stopEvent(e);
				this._messageBox.hideMessage();

				var newName = dojo.trim(this._nameInput.value);
				if (!newName) return;

				this._nameInput.disabled = 
					this._okButton.disabled =
					this._cancelButton.disabled = true;
				var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.renameProductX, [this._originalName])); 

				var contextConfigVersionUri;
				var contextPartUri = null;

				if (this.site.configVersion) {
					contextConfigVersionUri = this.site.configVersionUri;
				}
				else {
					contextConfigVersionUri = this.site.contextConfigVersionUri;
					contextPartUri = this.site.contextPartUri;
				}

				var serviceUri = net.jazz.ajax._contextRoot + '/ple/update_properties'; //$NON-NLS-1$;
				var content = {
					saveProperties: dojo.toJson({
						'http:\/\/purl.org\/dc\/terms\/title': newName //$NON-NLS-1$
					}),
					deleteProperties: dojo.toJson([]),
					site: Site.siteToJsonSite(this.site)
				};

				jazz.client.xhrPost({
					url: serviceUri,
					content: content,

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function() {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectPath: this.site.path
						}).addBoth(dojo.hitch(this, function() {
							this._dialog.close();
						}));
					}),

					failOk: true,

					error: dojo.hitch(this, function(error, ioArgs) {
						// Refresh since we might have checked out the parent
						this.enableSave();
						this._nameInput.disabled =
							this._cancelButton.disabled = false;
						this._messageBox.showXhrError(ioArgs.xhr);
					})
				});
			},

			cancel: function() {
				this._dialog.close();
			},

			postCreate: function() {
				this.inherited(arguments);
				this.resource = this.site.itemVersion || this.site.configVersion;
				var resource = this.resource;
				this._originalName = resource.name;
				this._nameInput.value = this._originalName;

				var decorator = com.ibm.relm.ple.web.ui.formatting.formatDecoratorForItem(resource);
				if (!decorator) {
					dojo.destroy(this._decoratorRow);
				}
				else {
					this._decoratorNode.innerHTML = decorator;
				}

				this._nameHelp = new jazz.app.ua.TextHoverHelp({
					uaKey: 'com.ibm.relm.ple.web.text_CreateProductName001', //$NON-NLS-1$
					contentNode: this._nameSpan
				});

				this.connect(this._nameInput, 'onkeyup', this.enableSave); //$NON-NLS-1$

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: dojo.replace(this.messages.renameProductX, [resource.name]),
					width: '530px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconPath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/rename_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
				});
			},

			enableSave: function() {
				this._okButton.disabled = this._nameInput.value === this._originalName || !this._nameInput.value;
			},

			startup: function() {
				this._nameInput.select();
			},

			uninitialize: function() {
				this._nameHelp.destroyRecursive();
				this.inherited(arguments);
			}
		}
	);

})();