/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.rtc.CreateConfigurationDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){

	var Site = com.ibm.relm.ple.web.ui.Site;

	/**
	 * Same as Create Product dialog, but creates a special RTC "configuration"
	 * product, which is used to track conditions for deciding what with sync
	 * to RTC.
	 * This should be refactored to share code with the other create dialog.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.rtc.CreateConfigurationDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','rtc/templates/CreateConfigurationDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			site: null,

			constructor: function(args, node)
			{
				this._nameId = dijit.getUniqueId('CreateConfigurationDialog'); //$NON-NLS-1$				
			},

			postCreate: function() {
				this.inherited(arguments);

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.createConfigurationDialogTitle,
					width: '500px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-rtcConfiguration' //$NON-NLS-1$
				});
				
				this._description.innerHTML = this.messages.createConfigurationDialogDescription;
				this._dialog.startup();
				this._nameField.select();
			},

			cancel: function() {
				this._dialog.close();
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);

				this._messageBox.hideMessage();
				var title = dojo.trim(this._nameField.value);

				this._cancelButton.disabled = true;
				this._nameField.disabled = true;
				this._submitButton.disabled = true;

				var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.creatingConfigurationNamedX, [title]));
				
				var initialProperties = {
					// Empty string is all that's need to mark this as configuration.
					'http://jazz.net/ns/pd/extensions#conditionVariables': '' //$NON-NLS-1$ //$NON-NLS-2$
				};

				var self = this;
				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/create_item_version', //$NON-NLS-1$
					content: {
						title: title,
						configVersionUri: this.site.contextConfigVersionUri,
						itemVersionUri: this.site.itemVersionUri,
						initialProperties: dojo.toJson(initialProperties),
						dimensions: dojo.toJson([])
					},

					handle: function() {
						message.hide();
					},

					load: function(newConfiguration) {
						var newPath = self.site.path.concat(newConfiguration.pathSegment);
						
						var close = dojo.hitch(self._dialog, self._dialog.close);
						
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectPath: newPath,
							flashPath: true
						}).addBoth(close);
					},

					error: function(error, ioArgs) {
						self._messageBox.showXhrError(ioArgs.xhr);

						self._cancelButton.disabled = false;
						self._nameField.disabled = false;
						self._nameDidChange();
					}
				});
			},

			_nameDidChange: function() {
				var self = this;
				setTimeout(function() {
					// Paste event can fire BEFORE the paste is available in the text input.
					// Delay this one run loop to be sure the text is there.
					var title = dojo.trim(self._nameField.value);
					self._submitButton.disabled = title.length === 0;	
				}, 0);
			},


			uninitialize: function() {
			
				this.inherited(arguments);
			}
		}
	);

})();