/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.rtc.RtcPreviewDialog"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.ple.web.rtc.RtcPreviewDialogRow"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.rtc", "RtcPreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.ple.web.rtc", "ReceivePreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.ple.web.rtc", "SendPreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.Dialog');//$NON-NLS-1$
dojo.require('jazz.ui.StyledTable');//$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage');//$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcPreviewSummaryItem');//$NON-NLS-1$
dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$

(function(){

	var ICON_CLASS = {
		rolledback: 'ple-icon-rolledbackComponent', //$NON-NLS-1$
		updated: 'ple-icon-updatedComponent', //$NON-NLS-1$
		changed: 'ple-icon-changedComponent', //$NON-NLS-1$
		unmodifiable: 'ple-icon-unmodifiableComponent', //$NON-NLS-1$
		added: 'ple-icon-addedComponent', //$NON-NLS-1$
		removed: 'ple-icon-removedComponent', //$NON-NLS-1$
		unchanged: 'ple-icon-unchangedComponent' //$NON-NLS-1$
	};

	var receiveIcon_Large = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/previewreceive_obj32.png'); //$NON-NLS-1$ //$NON-NLS-2$
	var sendIcon_Large = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/previewsend_obj32.png'); //$NON-NLS-1$ //$NON-NLS-2$

	var messages;

	dojo.declare(
		"com.ibm.relm.ple.web.rtc.RtcPreviewDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','rtc/templates/RtcPreviewDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			BLANK_GIF: dojo.moduleUrl('dojo', 'resources/blank.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function(args, node) {
				var rtcPreviewDialogMessages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.rtc", "RtcPreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				var streamManagementMessages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
				messages = this.messages = dojo.mixin(rtcPreviewDialogMessages, streamManagementMessages);
				
				this.rows = [];
				this.firstRowOfDiffType = {};
				this.undrawnUnchanged = [];
				this.summaryItems = [];
				this.widgets = [];
			},

			postMixInProperties: function() {
				var receivePreviewDialogMessages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.rtc", "ReceivePreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				var sendPreviewDialogMessages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.rtc", "SendPreviewDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				if (this.operation == "send") { //$NON-NLS-1$
					this.iconClass = "ple-icon-reserveForRtc"; //$NON-NLS-1$
					this.largeIcon = sendIcon_Large;
					messages = this.messages = dojo.mixin(this.messages, sendPreviewDialogMessages);
				} else {
					this.iconClass = "ple-icon-reserveForRelm"; //$NON-NLS-1$
					this.largeIcon = receiveIcon_Large;
					messages = this.messages = dojo.mixin(this.messages, receivePreviewDialogMessages);
				}
			},

			postCreate: function() {
				this.generateSummary();

				var columns = [
					{
						label: this.messages.component,
						width: "40%" //$NON-NLS-1$
					},
					{
						label: this.messages.before,
						className: "ple-RtcPreviewDialog-csh", //$NON-NLS-1$
						width: "30%" //$NON-NLS-1$
					},
					{
						label: "", //$NON-NLS-1$
						width: "0" //$NON-NLS-1$
					},
					{
						label: this.messages.after,
						className: "ple-RtcPreviewDialog-csh", //$NON-NLS-1$
						width: "30%" //$NON-NLS-1$
					}
				];

				this.table = new jazz.ui.StyledTable({
					columns: columns
				}, this.table);
				this.widgets.push(this.table);
				this.initializeCSH();

				this.drawContent(this.data.diffs);

				this.dialog = new jazz.ui.Dialog({
					width: "700px", //$NON-NLS-1$
					contentNode: this.domNode,
					iconClass: this.iconClass,
					primaryTitle: this.messages.primaryTitle,
					destroyContents: true,
					closeOnEsc: false,
					closable: false
				});
			},
 
			initializeCSH: function() {
				var csh = new jazz.app.ua.PageHoverHelp({
					uaKey: (this.operation == "send") ? 'com.ibm.relm.ple.web.section_PrevSend001' : 'com.ibm.relm.ple.web.section_PrevRec001' //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}, this.summaryCSH);
				this.widgets.push(csh);

				var sendKeys = ["com.ibm.relm.ple.web.text_PrevSendB4001", "com.ibm.relm.ple.web.text_PrevSendAft001"]; //$NON-NLS-1$ //$NON-NLS-2$
				var receiveKeys = ["com.ibm.relm.ple.web.text_PrevRecB4001", "com.ibm.relm.ple.web.text_PrevRecAft001"]; //$NON-NLS-1$ //$NON-NLS-2$

				var titleNodes = dojo.query('.ple-RtcPreviewDialog-csh div', this.domNode).forEach(function(titleDiv, index) { //$NON-NLS-1$
					this.widgets.push(new jazz.app.ua.TextHoverHelp({
						uaKey: (this.operation == "send") ? sendKeys[index] : receiveKeys[index], //$NON-NLS-1$
						contentNode: titleDiv
					}));
				}, this);
			},

			generateSummary: function() {
				var col1 = this.summaryColumn1;
				var col2 = this.summaryColumn2;

				this._drawSummaryRow("removed", col2); //$NON-NLS-1$
				this._drawSummaryRow("added", col2); //$NON-NLS-1$

				if (this.operation == "receive") { //$NON-NLS-1$
					this._drawSummaryRow("unmodifiable", col1); //$NON-NLS-1$
				}
				if (this.creationTimesKnown) {
					this._drawSummaryRow("rolledback", col1); //$NON-NLS-1$ //$NON-NLS-2$
					this._drawSummaryRow("updated", col1); //$NON-NLS-1$ //$NON-NLS-2$
				} else {
					this._drawSummaryRow("changed", col1); //$NON-NLS-1$
				}

				var unchangedSummary;
				if (dojo.query('li', this.summaryColumn1).length < 3) { //$NON-NLS-1$
					unchangedSummary = this._drawSummaryRow("unchanged", col1); //$NON-NLS-1$
				} else {
					unchangedSummary = this._drawSummaryRow("unchanged", col2); //$NON-NLS-1$
				}
				unchangedSummary.set('showing', false); //$NON-NLS-1$
			},

			_drawSummaryRow: function(diffType, columnNode) {
				var showToggle = diffType === "unchanged"; //$NON-NLS-1$
				var numberOfComponents = this.data.summary[diffType];
				var summaryItem = new com.ibm.relm.ple.web.rtc.RtcPreviewSummaryItem(
					{type: diffType, number: numberOfComponents, showToggle: showToggle && numberOfComponents}
				);
				this.connect(summaryItem, 'snapDiffType', dojo.partial(this._snapDiffType, diffType)); //$NON-NLS-1$
				this.summaryItems[diffType] = summaryItem;
				showToggle && this.connect(summaryItem.toggleLink, 'click', dojo.partial(this.toggleDiffType, diffType)); //$NON-NLS-1$
				dojo.place(summaryItem.domNode, columnNode);
				return summaryItem;
			},

			_snapDiffType: function(diffType) {
				var firstRowOfType = this.firstRowOfDiffType[diffType];
				if (!firstRowOfType) return;
				firstRowOfType.domNode.scrollIntoView();
			},

			toggleDiffType: function(diffType, event) {
				event && dojo.stopEvent(event);
				var summaryItem = this.summaryItems[diffType];
				var currentlyShowing = summaryItem.get('showing'); //$NON-NLS-1$
				summaryItem.set('showing', !currentlyShowing); //$NON-NLS-1$
				
				if (diffType == "unchanged" && //$NON-NLS-1$
						this.undrawnUnchanged && this.undrawnUnchanged.length > 0) {
					var frag = document.createDocumentFragment();

					dojo.forEach(this.undrawnUnchanged, function(unchanged) {
						var row = new com.ibm.relm.ple.web.rtc.RtcPreviewDialogRow(unchanged);
						this.rows.push(row);
						if (!this.firstRowOfDiffType[diffType]) {
							this.firstRowOfDiffType[diffType] = row;
						}
						frag.appendChild(row.domNode);
					}, this);
					this.undrawnUnchanged = null;
					this.table.tbody.appendChild(frag);
				} else {
					dojo.query('tr[difftype="' + diffType + '"]', this.table.tbody).forEach(function(row) { //$NON-NLS-1$ //$NON-NLS-2$
						row.style.display = (currentlyShowing) ? 'none' : ''; //$NON-NLS-1$ //$NON-NLS-2$
					}, this);
				}

				var showingRows = dojo.query('tr', this.table.tbody).some(function(row) { //$NON-NLS-1$
					return row.style.display != "none"; //$NON-NLS-1$
				});
				this.noChanges.style.display = (showingRows) ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			 * Draws the component differences into the table
			 * 
			 * @param  {Array} diffs List of differences between component baseliens
			 */
			drawContent: function(diffs) {
				this.table.clearBody();
				this.destroyRows();
				var frag = document.createDocumentFragment();
				dojo.forEach(diffs, function(diff) {
					diff.operation = this.operation;
					if (diff.diffType == "unchanged") { //$NON-NLS-1$
						this.undrawnUnchanged.push(diff);
					} else {
						var row = new com.ibm.relm.ple.web.rtc.RtcPreviewDialogRow(diff);
						this.rows.push(row);
						if (!this.firstRowOfDiffType[diff.diffType]) {
							this.firstRowOfDiffType[diff.diffType] = row;
						}
						dojo.place(row.domNode, frag);
					}
				}, this);
				this.table.tbody.appendChild(frag);

				if (this.table.tbody.children.length === 0) {
					this.noChanges.style.display = ""; //$NON-NLS-1$
				}
			},

			onSubmit: function() {/*No-op*/},

			onCancel: function() {/*No-op*/},

			_onSubmit: function(args) {
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.updatingCmStream);
				this._closeDialog();
				message.hide();
				this.onSubmit();
			},

			_closeDialog: function() {
				this.dialog.close();
				delete this.dialog;
			},

			_onCancel: function(args) {
				this._closeDialog();
				this.onCancel();
			},

			destroyRows: function() {
				dojo.forEach(this.rows, function(w) { w.destroyRecursive; delete w;});
			},

			uninitialize: function() {
				this.destroyRows();
				dojo.forEach(this.widgets, function(w) { w.destroyRecursive; delete w;});
			}
		});

	dojo.declare(
		"com.ibm.relm.ple.web.rtc.RtcPreviewDialogRow", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templateString: ''+ //$NON-NLS-1$
			'<tr tabindex="-1">' + //$NON-NLS-1$    
			'	<td>' + //$NON-NLS-1$
			'		<img class="ple-icon ple-icon-rtcComponent" alt="" src="${_blankGif}">' + //$NON-NLS-1$
			'		<span dojoAttachPoint="componentNode"></span>' + //$NON-NLS-1$
			'	</td>' + //$NON-NLS-1$
			'	<td dojoAttachPoint="beforeNode"><div> - </div></td>' + //$NON-NLS-1$
			'	<td><img class="ple-icon" src="/relm/web/dojo/resources/blank.gif" dojoAttachPoint="iconNode"></td>' + //$NON-NLS-1$
			'	<td dojoAttachPoint="afterNode"><div> - </div></td>' + //$NON-NLS-1$
			'</tr>', //$NON-NLS-1$

			postCreate: function() {
				dojo.attr(this.domNode, 'difftype', this.diffType); //$NON-NLS-1$

				if (this.diffType == "unmodifiable") { //$NON-NLS-1$
					if (this.reason == "PINNED") { //$NON-NLS-1$
						this._drawReason(this.afterNode, messages.reasonPinned);
					} else if (this.reason == "PARENT_PINNED") { //$NON-NLS-1$
						this._drawReason(this.afterNode, messages.reasonParentPinned);
					} else {
						console.warn("unmodifiable component baseline with invalid reason", this.reason); //$NON-NLS-1$
					}
				} else if (this.diffType == "removed") { //$NON-NLS-1$
					if (this.reason == "VARIABILITY" || this.reason == "PARENT_VARIABILITY") { //$NON-NLS-1$ //$NON-NLS-2$
						this._drawReason(this.afterNode, messages.reasonVariability);
					} else {
						this._drawReason(this.afterNode, messages.reasonRemoved);
					}
				} else if (this.diffType == "unchanged" && this.operation == "receive") { //$NON-NLS-1$ //$NON-NLS-2$
					if (this.reason == "VARIABILITY" || this.reason == "PARENT_VARIABILITY") { //$NON-NLS-1$ //$NON-NLS-2$
						this._drawReason(this.afterNode, messages.unchangedVariability);
					}
				}
			},

			_drawReason: function(node, text) {
				node.innerHTML = '<i>' + text + '</i>'; //$NON-NLS-1$ //$NON-NLS-2$
			},

			_setDiffTypeAttr: function(diffType) {
				dojo.addClass(this.iconNode, ICON_CLASS[diffType]);
				dojo.attr(this.iconNode, 'alt', messages[diffType]); //$NON-NLS-1$
				dojo.attr(this.iconNode, 'title', messages[diffType]); //$NON-NLS-1$
			},

			_setComponentAttr: {
				node: 'componentNode', //$NON-NLS-1$
				type: 'innerHTML' //$NON-NLS-1$
			},

			_setBeforeAttr: {
				node: 'beforeNode', //$NON-NLS-1$
				type: 'innerHTML' //$NON-NLS-1$
			},

			_setAfterAttr: {
				node: 'afterNode', //$NON-NLS-1$
				type: 'innerHTML' //$NON-NLS-1$
			}
		}
	);
})();