/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$

(function() {

	var settingsLoaded = false;
	var deferredLoads = [];

	// Init on load of this file. Async, so code that requires these settings
	// to be 100% accurate or only runs once, should call .awaitSettings to wait
	// for this to complete.
	jazz.client.xhrGet({
		url: net.jazz.ajax._contextRoot + '/ple/rtc_settings', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		sync: false,

		load: function (settings) {
			dojo.mixin(com.ibm.relm.ple.web.rtc.RtcSettings, settings);

			for (var i = 0; i < deferredLoads.length; i++) {
				var d = deferredLoads[i];
				d.callback(com.ibm.relm.ple.web.rtc.RtcSettings);
			}

			settingsLoaded = true;
			deferredLoads = null;
		}.bind(this),

		error: function(error, ioArgs) {
			console.error('Could not determine RTC stream integration status.', error, ioArgs.xhr.responseText); //$NON-NLS-1$

			for (var i = 0; i < deferredLoads.length; i++) {
				var d = deferredLoads[i];
				d.errback('Could not determine RTC stream integration status.'); //$NON-NLS-1$

				// This only runs once, so even though the load failed, still consider this settingsLoaded.
				settingsLoaded = true;
				deferredLoads = null;
			}
		}
	});
	
	/**
	 * Awaits for the the RtcSetting to be fully loaded.
	 * @return {dojo.Deferred} fires .callback with the entire RtcSettings object
	 * once it is loaded.
	 */
	com.ibm.relm.ple.web.rtc.RtcSettings.awaitSettings = function() {
		var d = new dojo.Deferred();

		if (settingsLoaded) {
			d.callback(com.ibm.relm.ple.web.rtc.RtcSettings);
		}
		else {
			deferredLoads.push(d);
		}

		return d;	
	};
})();