/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.rtc.StreamManagement'); //$NON-NLS-1$

dojo.require('jazz.ui.Menu'); //$NON-NLS-1$
dojo.require('jazz.ui.menu.MenuItem'); //$NON-NLS-1$
dojo.require('dijit.MenuSeparator'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.RtcStreamComponentPicker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProgressDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.CreateConfigurationDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.advanced.AdvancedSettings'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcPreviewDialog');//$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var StatusMessage = com.ibm.relm.common.web.StatusMessage;
var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
var Permissions = relm.app.Permissions;
var Site = com.ibm.relm.ple.web.ui.Site;

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$

var rtcSettings_ = com.ibm.relm.ple.web.rtc.RtcSettings;
var advancedSettings_ = com.ibm.relm.ple.web.advanced.AdvancedSettings;

function updateRtcBaselines(rtcServer, streams) {
	var statusMessage = StatusMessage.show(messages.updatingCmStream); 

	var deferred = new dojo.Deferred();

	jazz.client.xhrPost({
		url: rtcServer + '/service/com.ibm.team.rtc.relm.IRelmService', //$NON-NLS-1$
		content: {
			method: 'updateStreams', //$NON-NLS-1$
			streams: dojo.toJson(streams)
		},
		handleAs: 'json', //$NON-NLS-1$

		handle: function() {
			statusMessage.hide();
		},

		load: function () {
			deferred.callback();
		},

		error: function(error, ioArgs) {
			callErrback(deferred, error, ioArgs);
		}
	});

	return deferred;
}

function reserveForRtcWithProgressDialog(site) {
	dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$

	var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
		primaryTitle: messages.reserveForRtc, 
		width: '400px', //$NON-NLS-1$
		explanation: '', //$NON-NLS-1$
		canCancel: false,
		mirrorStatusMessage: true,
		iconClass: 'ple-icon-reserveForRtc' //$NON-NLS-1$
	});
	progressDialog.startup();

	function onError(errorObject) {
		dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
		progressDialog.close();
	}

	var rtcServer = site.product.rtcServer;

	var checkForMultipleBaselines = true;
	getRelmBaselines(site, checkForMultipleBaselines)
	.then(function(result) {
		var relmStreams = result.currentBaselines;
		var lastBaselines = result.lastBaselines;

		var streamUuids = dojo.map(relmStreams, function(stream) {
			return stream.streamUuid;
		});

		getRtcBaselines(rtcServer, streamUuids)
		.then(function(rtcStreamsByUuid) {

			// New requirements to only update the top level
			// stream. Just ignore the rest for now.
			var firstRelmStream = relmStreams[0];
			if (firstRelmStream.baselines.length === 0) {
				dojo.publish('ple.product_tree_error', [{ //$NON-NLS-1$
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: messages.noBaselines,
					explanation: messages.noBaselines$explanation,
					userAction: messages.noBaselines$useraction
				}]);
				progressDialog.close();
				return;
			}

			var rtcStream = rtcStreamsByUuid[firstRelmStream.streamUuid];


			showConflictWarning(firstRelmStream.baselines, rtcStream.baselines, lastBaselines)
			.then(function() {

				updateRtcBaselines(rtcServer, [firstRelmStream]).then(function() {
					TreeRefresh.refresh({
						selectPath: site.path
					}).then(function() {
						progressDialog.close();
					});
				}, onError);
			})
			.otherwise(function() {
				// was cancelled
				progressDialog.close();
			});
	
		}, onError);

	}, onError);
}

function mapBaselineNamesByComponentUuid(baselineArray) {
	var baselineNames = {};
	for (var i = 0; i < baselineArray.length; i++) {
		var baseline = baselineArray[i];
		baselineNames[baseline.componentUuid] = baseline.baselineName;
	}
	return baselineNames;
}

/**
 * Find conclicts between the actual baselines in RTC that have been modified
 * since they were last synced into RELM. This is considered a conflict
 * (modified in both products at once).
 * 	
 * @param  {array} currentRelmBaselines the baselines that are in the RELM
 *                                      product tree and will be sent to RTC
 *                                      to the user proceeds.
 * @param  {array} currentRtcBaselines current baselines in RTC.
 * @param  {object} lastBaselinesMap saved map of component UUIDs to baseline
 * 	                                 UUIDs from the last receive from RTC.
 * 	                                 Saved as a JSON attribute on the stream
 * 	                                 product itself.
 * @return {array} found conclicts. Possibly empty. Each conflict has the
 *     componentName, newBaselineName RELM wants to send, and actualBaselineName
 *     that's in RTC currently.
 * 
 */
function getBaselineConflicts(currentRelmBaselines, currentRtcBaselines, lastBaselinesMap) {
	var baselinesConflicts = [];

	var currentRelmBaselineNames = mapBaselineNamesByComponentUuid(currentRelmBaselines);

	for (var i = 0; i < currentRtcBaselines.length; i++) {
		var currentRtcBaseline = currentRtcBaselines[i];
		var lastRelmBaselineUuid = lastBaselinesMap[currentRtcBaseline.componentUuid];
		var currentRelmBaselineName = currentRelmBaselineNames[currentRtcBaseline.componentUuid];

		//Current relm baselines excludes those that don't conform to the variability parameters. 
		//Checking if there is mapping for a component with the currentRtcBaseline ensures that we
		//only check for conflicts on components that conform to the variability parameters
		if (currentRelmBaselineName && lastRelmBaselineUuid !== currentRtcBaseline.baselineUuid) {

			baselinesConflicts.push({
				componentName: currentRtcBaseline.componentName,
				actualBaselineName: currentRtcBaseline.baselineName,
				newBaselineName: currentRelmBaselineName
			});
		}
	}

	return baselinesConflicts;
}

/**
 * Checks if the actual baselines in RTC have been modified since they
 * were last synced into RELM. This is considered a conflict (modified in both
 * products at once). If in conflict, a confirm dialog is shown with a warning
 * and list of baselines in conflict.
 * 	
 * @param  {array} currentRelmBaselines the baselines that are in the RELM product tree
 *                                      and will be sent to RTC to the user proceeds.
 * @param  {array} currentRtcBaselines current baselines in RTC.
 * @param  {object} lastBaselinesMap saved map of component UUIDs to baseline UUIDs from
 * 	                                 the last receive from RTC. Saved as a JSON attribute on the stream product itself.
 * @return {dojo.Deferred} if the user proceeds, the callback will be fired. Otherwise,
 *                            this will be canceled and the errback will be called.
 */
function showConflictWarning(currentRelmBaselines, currentRtcBaselines, lastBaselinesMap) {
	var deferred = new dojo.Deferred();

	if (!lastBaselinesMap) {
		deferred.callback();
		return deferred;
	}

	var baselinesConflicts = getBaselineConflicts(currentRelmBaselines, currentRtcBaselines, lastBaselinesMap);

	if (baselinesConflicts.length === 0) {
		deferred.callback();
		return deferred;
	}

	// Build up a nice HTML preview listing each conflict.
	var conflictList = dojo.create('ul'); //$NON-NLS-1$

	dojo.forEach(baselinesConflicts, function(baseline) {
		var li = dojo.create('li', null, conflictList); //$NON-NLS-1$
		var baselineMessage;
		if (!baseline.newBaselineName) {
			baselineMessage = dojo.replace(messages.componentNameXBaselineXWillBeRemoved,
				[baseline.componentName, baseline.actualBaselineName]);
		}
		else {
			baselineMessage = dojo.replace(messages.componentNameXBaselineXWillReplaceBaselineX,
				[baseline.componentName, baseline.newBaselineName, baseline.actualBaselineName]);
		}
		li.innerHTML = baselineMessage;

		dojo.create('img', { //$NON-NLS-1$
			src: dojo.moduleUrl('dojo','resources/blank.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			className: 'ple-icon ple-icon-rtcComponent' //$NON-NLS-1$
		}, li, 'first'); //$NON-NLS-1$
	});

	// Show the dialog. Add the preview after construction. Will .callback
	// on OK, and cancel the deferred on Cancel.
	var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
		iconClass: 'ple-icon-reserveForRtc', //$NON-NLS-1$
		width: '500px', //$NON-NLS-1$
		dialogTitle: messages.reserveForRtc,
		prompt: messages.confirmReplaceModifiedBaselinesPrompt,
		details: messages.confirmReplaceModifiedBaselinesDetails,
		onCancel: function() {
			deferred.cancel();
		}
	});

	dojo.place(conflictList, d.promptNode);

	d.addButton(messages.replaceButton, function() {
		deferred.callback();
		var okDeferred = new dojo.Deferred();
		okDeferred.callback();
		return okDeferred;
	});

	d.startup();
	return deferred;
}

function reserveForRelmWithPreview(site) {
	dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
	
	var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
		primaryTitle: messages.reserveForRelm, 
		width: '400px', //$NON-NLS-1$
		explanation: '', //$NON-NLS-1$
		canCancel: false,
		mirrorStatusMessage: true,
		iconClass: 'ple-icon-reserveForRelm' //$NON-NLS-1$
	});
	progressDialog.startup();

	function onError(errorObject) {
		dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
		progressDialog.close();
	}

	var rtcServer = site.product.rtcServer;

	getRelmBaselines(site, null, true).then(function(result) {
		var relmStreams = result.currentBaselines;
		var streamUuids = dojo.map(relmStreams, function(stream) {
			return stream.streamUuid;
		});

		getRtcBaselines(rtcServer, streamUuids).then(function(baselines) {
			// New requirements to only update the top level
			// stream. Just ignore the rest for now.
			var firstRelmStream = relmStreams[0];
			var rtcStream = baselines[firstRelmStream.streamUuid];
			rtcStream.streamUuid = firstRelmStream.streamUuid;
			rtcStream.name = rtcStream.streamName;

			showPreview('receive', firstRelmStream, rtcStream).then(function() { //$NON-NLS-1$
				updateRelmStreamFromRtcBaselines(site, baselines).then(function() {
					TreeRefresh.refresh({
						selectPath: site.path,
						openPath: true
					}).then(function() {
						progressDialog.close();
					});
				}, onError);
			})
			.otherwise(function() {
				progressDialog.close();
			});

		}, onError);

	}, onError);
}

function reserveForRelmWithProgressDialog(site) {
	dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
	
	var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
		primaryTitle: messages.reserveForRelm, 
		width: '400px', //$NON-NLS-1$
		explanation: '', //$NON-NLS-1$
		canCancel: false,
		mirrorStatusMessage: true,
		iconClass: 'ple-icon-reserveForRelm' //$NON-NLS-1$
	});
	progressDialog.startup();

	function onError(errorObject) {
		dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
		progressDialog.close();
	}

	var rtcServer = site.product.rtcServer;

	getRelmBaselines(site).then(function(result) {
		var streams = result.currentBaselines;
		var streamUuids = dojo.map(streams, function(stream) {
			return stream.streamUuid;
		});

		getRtcBaselines(rtcServer, streamUuids).then(function(baselines) {

			updateRelmStreamFromRtcBaselines(site, baselines).then(function() {
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
					selectPath: site.path,
					openPath: true
				}).then(function() {
					progressDialog.close();
				});
			}, onError);

		}, onError);

	}, onError);
}

function disconnectFromRtcWithProgressDialog(site) {
	var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
		dialogTitle: messages.disconnectTitle, 
		iconClass: 'ple-icon-disconnectStream', //$NON-NLS-1$
		prompt: messages.disconnectPrompt
	});

	// XXX: this is dumb since it opens up a confirm, then progress dialog.
	// Should just show the progress in the confirm dialog, but I'm taking a shortcut here.
	d.addButton(messages.ok, function() {
		var deferred = new dojo.Deferred();
		var statusMessage = StatusMessage.show(messages.disconnectingStatus);

		var buttons = dojo.query('button, input[type="button"], input[type="submit"', d.buttonArea); //$NON-NLS-1$
		buttons.attr('disabled', true); //$NON-NLS-1$

		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/ple/disconnect_from_rtc', //$NON-NLS-1$
			handleAs: 'json', //$NON-NLS-1$
			content: {
				streamSite: Site.siteToJsonSite(site)
			},

			handle: function() {
				statusMessage.hide();
			},

			load: function () {
				TreeRefresh.refresh({
					selectPath: site.path
				}).then(function() {
					buttons.attr('disabled', false); //$NON-NLS-1$
					deferred.callback();
				});
			},

			error: function(error, ioArgs) {
				buttons.attr('disabled', false); //$NON-NLS-1$

				try {
					var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
					dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
				}
				catch (e) {
					// In case it's not a JSON error.
					dojo.publish('ple.product_tree_error', [{ //$NON-NLS-1$
						summary: ioArgs.xhr.responseText
					}]);	
				}
				deferred.callback();
			}
		});

		return deferred;
	}, true);

	d.startup();
}

function generateLoadRules(site) {
	var product = site.product;

	if (product.rtcStream) {
		var streamInfo = {};
		var streamProduct = product;
	}
	else {
		streamInfo = getParentStreamProduct(site);
		if (!streamInfo) {
			dojo.publish('ple.product_tree_error', [getParentStreamErrorObject()]); //$NON-NLS-1$
			return;
		}
		streamProduct = streamInfo.streamProduct;
	}

	var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
		primaryTitle: messages.generateLoadRules, 
		width: '400px', //$NON-NLS-1$
		explanation: '', //$NON-NLS-1$
		canCancel: false,
		mirrorStatusMessage: true,
		iconClass: 'ple-icon-rtcLoadRules' //$NON-NLS-1$
	});
	progressDialog.startup();

	var message = StatusMessage.show(messages.generateLoadRules);

	jazz.client.xhrPost({
		url: net.jazz.ajax._contextRoot + '/ple/load_rules', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		content: {
			selectedSite: Site.siteToJsonSite(site),
			streamPartUri: streamProduct.contextPartUri,
			pathToItemVersion: streamInfo.pathToItem
		},
	
		handle: function() {
			message.hide();
			progressDialog.close();
		},

		load: function (result) {
			com.ibm.relm.common.web.ui.util.downloadFile('/ple/load_rules', 'id', [result.id], false, true); //$NON-NLS-2$ //$NON-NLS-1$
		},
	
		error: function(error, ioArgs) {
			var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
			dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
		}
	});
}

function generateLoadRulesMenuItemUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.generateLoadRules,
		iconClass: 'ple-icon-rtcLoadRules', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() { 
			generateLoadRules(site);
		}
	});
}


function connectMenuItemUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.connect,
		iconClass: 'ple-icon-connectStream', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			var picker = new com.ibm.relm.ple.web.ui.picker.RtcStreamComponentPicker({
				site: site
			});
			picker.startup();
		}
	});
}

function reserveForRelmMenuItemUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.reserveForRelm,
		iconClass: 'ple-icon-reserveForRelm', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			reserveForRelmWithPreview(site);
		}
	});
}

function reserveForRtcMenuItemUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.reserveForRtc,
		iconClass: 'ple-icon-reserveForRtc', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			reserveForRtcWithPreview(site);
		}
	});
}

function createConfigurationUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.createConfiguration,
		iconClass: 'ple-icon-rtcConfiguration', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			new com.ibm.relm.ple.web.rtc.CreateConfigurationDialog({
				site: site
			});
		}
	});
}

function disconnectMenuItemUnless(disabled, site) {
	return new jazz.ui.menu.MenuItem({
		label: messages.disconnect, 
		iconClass: 'ple-icon-disconnectStream', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			disconnectFromRtcWithProgressDialog(site);
		}
	});
}

function previewStreamMenuItem(disabled, streamSite) {
	return new jazz.ui.menu.MenuItem({
		label: messages.previewStream,
		iconClass: 'ple-icon-previewStream', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			com.ibm.relm.ple.web.ui.Goto.gotoCompare(streamSite, null, true);
		}
	});
}

function compareStreamMenuItem(disabled, product1, product2) {
	return new jazz.ui.menu.MenuItem({
		label: messages.compareConfigurations,
		iconClass: 'ple-icon-compareStream', //$NON-NLS-1$
		disabled: disabled,
		onClick: function() {
			launchStreamCompare(product1.uri, product2.uri);
		}
	});
}

/**
 * Brings up the compare screen to either preview a stream or compare two streams with 
 * variable inclusion applied. 
 * @param  {String} streamProduct1 required uri for the stream product
 * @param  {String} streamProduct2 optional uri for second stream product. If not provided
 *                                 then preview of single stream is assumed
 */
function launchStreamCompare(streamProduct1Uri, streamProduct2Uri) {
	window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
		action: 'ple.compare', //$NON-NLS-1$
		left: streamProduct1Uri,
		right: streamProduct2Uri,
		streamCompare: true
	});
}

function getRtcBaselines(rtcServer, streamUuids) {
	var deferred = new dojo.Deferred();
	var statusMessage = StatusMessage.show(messages.gettingBaselines); 

	var rtcUrl = rtcServer + '/service/com.ibm.team.rtc.relm.IRelmService'; //$NON-NLS-1$

	jazz.client.xhrPost({
		url: rtcUrl,
		handleAs: 'json', //$NON-NLS-1$
		content: {
			method: 'getBaselinesInStreams', //$NON-NLS-1$
			streamUuids: streamUuids
		},

		handle: function() {
			statusMessage.hide();
		},

		load: function (baselines) {
			deferred.callback(baselines);
		},

		error: function(error, ioArgs) {
			callErrback(deferred, error, ioArgs);
		}
	});

	return deferred;
}

/**
 * Safer way to call a deferred.errback. This will try 
 * and treat the error response as a JSON error object,
 * but can show the error directly if it cannot be parsed.
 * 
 * @param  {dojo.Deferred} deferred to call .errback on.
 * @param  {object} error    XHR error
 * @param  {object} ioArgs   XHR IO args
 */
function callErrback(deferred, error, ioArgs) {
	var message = ioArgs.xhr.responseText;

	try {
		var errorObject = dojo.fromJson(message);
		deferred.errback(errorObject);
	}
	catch (e) {
		// Probably not a JSON error. Likley a connection problem with the proxy.
		console.error(e);

		deferred.errback({
			summary: messages.failedToContactRtc,
			explanation: message,
			userAction: messages.failedToContactRtc$useraction
		});
	}
}

function getParentStreamProduct(site) {
	var path = site.itemPath;
	var pathFromStream = [];

	for (var i = path.length - 1; i >= 0; i--) {
		var item = path[i];
		if (item.rtcStream) {
			// First entry was the selected item, which we don't want
			pathFromStream.pop();

			return {
				streamProduct: item,
				pathToItem: '/' + pathFromStream.join('/') //$NON-NLS-1$ //$NON-NLS-2$
			};
		}

		pathFromStream.unshift(item.name);
	}

	return null;
}

function getParentStreamErrorObject() {
	return {
		summary: messages.actionRequiresStream,
		explanation: messages.actionRequiresStream$explanation,
		userAction: messages.actionRequiresStream$useraction
	};
}

function updateRelmStreamFromRtcBaselines(streamSite, baselines) {
	var statusMessage = StatusMessage.show(messages.updatingRelmStream); 

	var deferred = new dojo.Deferred();

	jazz.client.xhrPost({
		url: net.jazz.ajax._contextRoot + '/ple/reserve_for_relm', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		content: {
			method: "updateRelmStream", //$NON-NLS-1$
			streamSite: Site.siteToJsonSite(streamSite),
			snapshotName: '', //$NON-NLS-1$
			baselines: dojo.toJson(baselines)
		},

		handle: function(error, ioArgs) {
			statusMessage.hide();
			if (ioArgs.xhr && ioArgs.xhr.getResponseHeader("x-relm-pinned-component-skipped")) { //$NON-NLS-1$
				var errorObject = {
					summary: messages.pinnedComponentSkipped,
					explanation: messages.pinnedComponentSkipped$explanation,
					userAction: messages.pinnedComponentSkipped$useraction,
					type: "WARNING" //$NON-NLS-1$
				};
				dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			}
		},

		load: function () {
			deferred.callback();
		},

		error: function(error, ioArgs) {
			callErrback(deferred, error, ioArgs);
		}
	});

	return deferred;
}

/**
 * Gets all the relm baselines and substreams under the given product. If info about a component is given, 
 * then only information about the stream from that componenet and below is included
 * @param  {String} streamProduct 	 the JSON object representing the stream product
 * @param  {Object} componentInfo    optional information related to the selected component. Should include:
 *                                   	componentsParentUri, componentPartUri, componentUri
 * @param { boolean} checkForMultipleBaselines if true, the server will throw an error if it detects
 *                   mutliple baselines for the same component bound into the product tree. This
 *                   is done on Send SCM Components since we know RTC can't handle this case
 *                   and want to prevent a random outcome.
 * @param {boolean} includeExcludedBaselines if true, baselines that should be excluded based on 
 *                                           variability rules are included. 
 * @return {Object}	JSON representation of the selected stream with all child component and stream info,
 *                       even what isn't showing in the tree
 */
function getRelmBaselines(streamSite, checkForMultipleBaselines, includeExcludedBaselines) {
	var statusMessage = StatusMessage.show(messages.gettingBaselines); 
	var deferred = new dojo.Deferred();
	var content = {
		streamSite: Site.siteToJsonSite(streamSite),
		checkForMultipleBaselines: checkForMultipleBaselines,
		includeExcluded: true
	};

	jazz.client.xhrPost({
		url: net.jazz.ajax._contextRoot + '/ple/get_baselines', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		content: content,
		handle: function() {
			statusMessage.hide();
		},
		load: function (result) {
			//Filter out certain baselines from the result
			if (!includeExcludedBaselines) {
				result.currentBaselines = filterExcludedBaselines(result.currentBaselines);
			}
			deferred.callback(result);
		},

		error: function(error, ioArgs) {
			callErrback(deferred, error, ioArgs);
		}
	});

	return deferred;
}

function filterExcludedBaselines(streamsWithBaselines) {
	return dojo.map(streamsWithBaselines, function(stream) {
		stream.baselines = dojo.filter(stream.baselines, function(baseline) {
			return baseline.status != "VARIABILITY" && baseline.status != "PARENT_VARIABILITY"; //$NON-NLS-1$ //$NON-NLS-2$
		});
		return stream;
	});
}

function reserveForRtcWithPreview(site) {
	dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$

	var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
		primaryTitle: messages.reserveForRtc, 
		width: '400px', //$NON-NLS-1$
		explanation: '', //$NON-NLS-1$
		canCancel: false,
		mirrorStatusMessage: true,
		iconClass: 'ple-icon-reserveForRtc' //$NON-NLS-1$
	});
	progressDialog.startup();

	function onError(errorObject) {
		dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
		progressDialog.close();
	}

	var rtcServer = site.product.rtcServer;

	var checkForMultipleBaselines = true;
	getRelmBaselines(site, checkForMultipleBaselines, true)
	.then(function(result) {
		var relmStreams = result.currentBaselines;
		var lastBaselines = result.lastBaselines;

		var streamUuids = dojo.map(relmStreams, function(stream) {
			return stream.streamUuid;
		});

		getRtcBaselines(rtcServer, streamUuids)
		.then(function(rtcStreamsByUuid) {

			// New requirements to only update the top level
			// stream. Just ignore the rest for now.
			var firstRelmStream = relmStreams[0];
			var rtcStream = rtcStreamsByUuid[firstRelmStream.streamUuid];
			rtcStream.streamUuid = firstRelmStream.streamUuid;
			rtcStream.name = rtcStream.streamName;


			showPreview("send", firstRelmStream, rtcStream, lastBaselines) //$NON-NLS-1$
			.then(function() {
				firstRelmStream = filterExcludedBaselines([firstRelmStream])[0];
				if (firstRelmStream.baselines.length === 0) {
					dojo.publish('ple.product_tree_error', [{ //$NON-NLS-1$
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: messages.noBaselines,
						explanation: messages.noBaselines$explanation,
						userAction: messages.noBaselines$useraction
					}]);
					progressDialog.close();
					return;
				}
				updateRtcBaselines(rtcServer, [firstRelmStream]).then(function() {
					TreeRefresh.refresh({
						selectPath: site.path
					})
					.then(function() {
						progressDialog.close();
					});
				}, onError);
			})
			.otherwise(function() {
				// was cancelled
				progressDialog.close();
			});
	
		}, onError);

	}, onError);
}

function showPreview(operation, relmStream, rtcStream, lastBaselines) {
	var statusMessage = StatusMessage.show(messages.generatingPreview); 
	var deferred = new dojo.Deferred();
	var content = {
		operation: operation,
		relmStream: dojo.toJson(relmStream),
		rtcStream: dojo.toJson(rtcStream),
		lastBaselines: dojo.toJson(lastBaselines)
	};

	var relmBaselineCreationTimes = dojo.every(relmStream.baselines, function(baseline) {
		return baseline.creationTime;
	});
	var rtcBaselineCreationTimes = dojo.every(rtcStream.baselines, function(baseline) {
		return baseline.baselineCreationDateMillis;
	});
	var baselineCreationTimesKnown = relmBaselineCreationTimes && rtcBaselineCreationTimes;

	jazz.client.xhrPost({
		url: net.jazz.ajax._contextRoot + '/ple/preview_rtc', //$NON-NLS-1$
		handleAs: 'json', //$NON-NLS-1$
		content: content,
		handle: function() {
			statusMessage.hide();
		},
		load: function (result, ioArgs) {
			var dialog = new com.ibm.relm.ple.web.rtc.RtcPreviewDialog({
				creationTimesKnown: baselineCreationTimesKnown,
				operation: operation,
				data: result,
				onSubmit: deferred.callback,
				onCancel: deferred.cancel
			});
			dialog.startup();
		},

		error: function(error, ioArgs) {
			callErrback(deferred, error, ioArgs);
		}
	});

	return deferred;
}

function isRtcProduct(product) {
	return product.rtcBaseline ||
		product.rtcComponent;
}

dojo.mixin(com.ibm.relm.ple.web.rtc.StreamManagement, {

	rtcUrls: rtcSettings_.rtcUrls,

	isRtcProduct: isRtcProduct,

	advancedImport_enabled: !!advancedSettings_.advanced_import,
	variableInclusion_enabled: !!advancedSettings_.variable_include,

	getParentStreamProduct: getParentStreamProduct,

	isEnabled: function() {
		return !!rtcSettings_.enabled;
	},

	/**
	 * Awaits for the settings to be async loaded.
	 * Must be used by ProductsPage toolbar, since it only draws once,
	 * and if asks for the settings too early, might not draw
	 * the RTC integration icon. By using this, it will be called
	 * back once the settings are available and can add the RTC button
	 * at that time.
	 * @return {dojo.Deferred} will callback with the full RTC settings, specifically
	 * the .enabled property.
	 */
	awaitSettings: function() {
		return com.ibm.relm.ple.web.rtc.RtcSettings.awaitSettings();
	},


	createMenuForSelectedSites: function() {
		var sites = TreeSelection.getAllSelectedSites();
		var hasOneSelectedProduct = sites.length == 1 && TreeSelection.areAllSelectedProducts();
		var site = sites[0];
		var isInBaseline = site.isInBaseline;
		var itemVersion = site.itemVersion;
		
		var isStream = hasOneSelectedProduct && itemVersion.rtcStream;
		var isConnected = hasOneSelectedProduct && (itemVersion.rtcStream || itemVersion.rtcComponent || itemVersion.rtcBaseline);
		var isComponent = !!itemVersion.rtcComponent;
		var isConfiguration = site && dojo.isString(itemVersion.conditionVariables);
		var canModify = Permissions.canProductsModify();
		var canSnapshot = Permissions.canProductsCreateSnapshot();

		var menu = new jazz.ui.Menu();

		if (isStream) {
			menu.addChild(reserveForRtcMenuItemUnless(!isStream, site));
			menu.addChild(reserveForRelmMenuItemUnless(!isStream || !canModify || !canSnapshot || isInBaseline, site));
			menu.addChild(new dijit.MenuSeparator());
			if (this.variableInclusion_enabled){
				menu.addChild(createConfigurationUnless(!hasOneSelectedProduct || !isConnected || isConfiguration || !canModify || isInBaseline, site));
				menu.addChild(previewStreamMenuItem(!isStream, site));
			}
		}

		var needsSeparator = false;

		if (isStream || (rtcSettings_.showConnectToComponent && isComponent) || (!isConnected && !isComponent)) {
			menu.addChild(connectMenuItemUnless(!hasOneSelectedProduct || isConfiguration || !canModify || isInBaseline, site));
				needsSeparator = true;
		}

		if (isConnected && (isStream || (isComponent && rtcSettings_.showConnectToComponent))) {
			menu.addChild(disconnectMenuItemUnless(!canModify || isInBaseline, site));
			needsSeparator = true;
		}

		if (needsSeparator) {
			menu.addChild(new dijit.MenuSeparator());
			needsSeparator = false;
		}

		var streamInfo = getParentStreamProduct(site);
		menu.addChild(generateLoadRulesMenuItemUnless(!streamInfo, site));

		return menu;
	}
});

})();
