/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 * 
 * @author Hagop Hagopian
 *******************************************************************************/

dojo.provide("com.ibm.relm.common.web.ui.ActionPreview"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.common.web.ui", "ActionPreviewMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){

var _leftItem = null;
var _rightItem = null;

/**
 * Widget to show a preview of the action being performed
 */
dojo.declare(
	"com.ibm.relm.common.web.ui.ActionPreview", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/ActionPreview.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: false,
		
		_messages: dojo.i18n.getLocalization("com.ibm.relm.common.web.ui", "ActionPreviewMessages"), //$NON-NLS-1$ //$NON-NLS-2$

		/**
		 * args.leftItem and args.rightItem should contain the 'before' and 'after' items respectively.
		 * 
		 * @param args.leftItem - Required. The item the action will be performed on.
		 * @param args.rightItem - Optional. When constructing, you don't usually know what the desired
		 * 						   result will be. This can be set later.
		 * 
		 * item.displayName - Display name of item. 
		 * item.numChildren - Number of children this item has.
		 * item.parentDisplayName - Optional Display name of parent, otherwise null if it's a root item
		 */
		constructor: function(args) 
		{
			_leftItem = args.leftItem;
			_rightItem = args.rightItem; //can either be null or undefined
		},
		
		postCreate: function() {
			this.inherited(arguments);
			
			if (!_leftItem.parentDisplayName){
				//TODO: format the no-parent case, idk remove any dashes or arrows above the display item
				this._leftParentDiv.innerHtml = "";  //$NON-NLS-1$
			}
			else
				dojo.create("div", { innerHTML: _leftItem.parentDisplayName }, this._leftParentDiv); //$NON-NLS-1$
			
			dojo.create("div", { innerHTML: _leftItem.displayName }, this._leftItemDiv); //$NON-NLS-1$
			var numChildren = _leftItem.numChildren === 0 ? "" : _leftItem.numChildren; //$NON-NLS-1$
			dojo.create("div", { innerHTML: numChildren }, this._leftChildrenDiv); //$NON-NLS-1$
			
			this.formatRightItem();
		},

		/**
		 * set rightItem to null to hide right div
		 */		
		setRightItem: function(rightItem){
			_rightItem = rightItem;
			
			this.formatRightItem();				
		},
		
		formatRightItem: function(){
			if (!_rightItem){
				dojo.style(this._rightDiv, 'visibility', 'hidden'); //$NON-NLS-1$ //$NON-NLS-2$
				return;
			}
			dojo.removeAttr(this._rightDiv, 'style'); //$NON-NLS-1$
			
			if (!_rightItem.parentDisplayName){
				//TODO: format the no-parent case, idk remove any dashes or arrows above the display item
				this._rightParentDiv.innerHtml = "";  //$NON-NLS-1$
			}
			else{
				dojo.empty(this._rightParentDiv);
				dojo.create("div", { innerHTML: _rightItem.parentDisplayName }, this._rightParentDiv); //$NON-NLS-1$
			}

			dojo.empty(this._rightItemDiv);
			dojo.create("div", { innerHTML: _rightItem.displayName }, this._rightItemDiv); //$NON-NLS-1$
			
			dojo.empty(this._rightChildrenDiv);
			var numChildren = _rightItem.numChildren === 0 ? "" : _rightItem.numChildren; //$NON-NLS-1$
			dojo.create("div", { innerHTML: numChildren }, this._rightChildrenDiv); //$NON-NLS-1$
			
			dojo.style(this._rightDiv, 'visibility', 'visible'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
	
	});

})();
