/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.ComboButton'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$

/**
 * A button with an associated drop down menu. Similar to that
 * used in Work Items UI.
 */
(function(){
	
	var CLASS_NAME = 'relm-common-ComboButton'; //$NON-NLS-1$
	
	dojo.declare(
		'com.ibm.relm.common.web.ui.ComboButton', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/ComboButton.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,

			// zero-width space is used as empty string causes height issues
			text: '&#8203;', //$NON-NLS-1$
			
			dropDownIcon: dojo.moduleUrl('com.ibm.relm.common.web','ui/images/popup-down-arrow.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			/** CSS class for button icon. */
			iconClass: '', //$NON-NLS-1$

			/** Callback when main button is clicked. */
			onClick: null,

			/**
			 * Specify whether the button is enabled.  If false, button will be
			 * faded out, no hover, and unclickable.
			 * @default true
			 */
			enabled: true,
			
			/**
			 * @args can accept...
			 *	menuItems: An array of objects (not actual dijit.MenuItem
			 *		widgets, though) used to create the menu entries. Supports
			 *		the same properties as dijit.MenuItem, including label,
			 *		onClick, and iconClass. A 'null' entry will be shown
			 *		as a divider.
			 *	onClick: the click callback handler when the main button is clicked.
			 *	iconClass: CSS class with a background image to use as the main button icon.
			 */
			constructor: function(args, node) 
			{
				this.menuItems = args.menuItems;
			},
			
			postCreate: function() {
				this.inherited(arguments);
				dojo.addClass(this._iconNode, this.iconClass);
			},
			
			_setEnabledAttr: function(enabled) {
				dojo.toggleClass(this.domNode, CLASS_NAME+'-disabled', !enabled); //$NON-NLS-1$
				dijit.setWaiState(this.domNode, 'disabled', !enabled); //$NON-NLS-1$
				this.enabled = enabled;
			},
			
			/**
			 * Determines whether this widget is focusable or not.
			 * Used by dijit._KeyNavContainer to determine keyboard focus.
			 */
			isFocusable: function() {
				return this.get('enabled') && (dojo.style(this.domNode, 'display') != 'none'); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			},
			
			/**
			 * Focuses the this button.
			 * Used by dijit._KeyNavContainer to set keyboard focus.
			 */
			focus: function() {
				dijit.focus(this.domNode);
			},
			
			_onDownKeyPress: function(evt) {
				if (evt.charOrCode == dojo.keys['DOWN_ARROW']) { //$NON-NLS-1$
					this._onDropDown(evt);
					dojo.stopEvent(evt);
				}
			},
			
			_onClick: function(e) {
				dojo.stopEvent(e);
				if (this.get('enabled')) { //$NON-NLS-1$
					this.closeMenu();
					this.onClick && this.onClick();
				}
			},

			closeMenu: function() {
				dojo.removeClass(this.domNode, 'relm-common-ComboButton-open'); //$NON-NLS-1$
				this.disconnect(this._blurHandler);
				if (this._menu) {
					dijit.popup.close(this._menu);
					this._menu && this._menu.destroyRecursive();
					delete this._menu;
				}
				this.focus();
			},

			_onDropDown: function(e) {
				dojo.stopEvent(e);

				if (!this.get('enabled')) { //$NON-NLS-1$
					return;
				}
				
				dojo.addClass(this.domNode, 'relm-common-ComboButton-open'); //$NON-NLS-1$

				var menu = this._menu = new jazz.ui.Menu({
					onClose: function(e) {
						this.destroyRecursive();
					}
				});

				var self = this;
				dojo.forEach(this.menuItems, function(item) {
					if (!item) {
						menu.addChild(new dijit.MenuSeparator());
					}
					else {
						var oldHandler = item.onClick;
						if (oldHandler) {
							// Intercept the event and ensure we close the menu first
							item.onClick = function(event) {
								self.closeMenu();
								oldHandler(event);
							};
						}
						menu.addChild(new dijit.MenuItem(item));
					}
				});
				
				this.disconnect(this._blurHandler);
				this._blurHandler = this.connect(this._menu, '_onBlur', this.closeMenu); //$NON-NLS-1$

				dijit.popup.open({
					popup: this._menu,
					parent: this,
					around: this.domNode,
					orient: {BR:'TR', TR:'BR'}, //$NON-NLS-1$ //$NON-NLS-2$
					onCancel: dojo.hitch(this, this.closeMenu)
				});
				
				this._menu.focus();
			},

			uninitialize: function() {
				this.closeMenu();
				this.inherited(arguments);
			}
		}	
	);	// dojo.declare

})();