/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.Drawer'); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui", "DrawerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dojo.fx'); //$NON-NLS-1$

(function(){
	
	function pxToInt(cssPx)
	{
		return Number(cssPx.slice(0, -2));
	}
	
	var collapsedHandleWidthPx_ = 10;

	/**
	 * Collapsible sidebar on the right side of the product tree.
	 * Can be collapsed to a vertical bar and resized when visible.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.Drawer', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/templates/Drawer.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui", "DrawerMessages"), //$NON-NLS-1$ //$NON-NLS-2$
			
			// Trying to make this as small as we can't without scrolling when showing a work item.
			inititalDrawerWidth: 575,
			drawerMinWidth: 200,
			collapsedTooltip: '', //$NON-NLS-1$

			animationDurationMillis: 300,

			/**
			 * Can set during construction or latter with .set/.get.
			 * Controls whether or nor the drawer collapsed or not.
			 * @type {Boolean}
			 */
			collapsed: false,

			constructor: function(args, node)
			{
				// guess an initial width
				this.inititalDrawerWidth = args.inititalDrawerWidth || this.inititalDrawerWidth;
				this._drawerWidth = this.inititalDrawerWidth;
			},

			postCreate: function() {
				this.inherited(arguments);

				if (this.collapsed) {
					this.set('collapsed', true);	 //$NON-NLS-1$
				}
				else {
					this.set('collapsed', false);	 //$NON-NLS-1$
				}
			},

			setMainContent: function(newMainNode) {
				dojo.empty(this._mainContentNode);
				this._mainContentNode.appendChild(newMainNode);
			},

			setDrawerContent: function(newDrawerNode) {
				dojo.empty(this._drawerContentNode);
				this._drawerContentNode.appendChild(newDrawerNode);
			},

			_setCollapsedAttr: function(nowCollapsed) {
				if (nowCollapsed) {
					dojo.addClass(this.domNode, 'ple-Drawer-collapsed'); //$NON-NLS-1$
					// leave room for the drawer on right
					this._mainWrapper.style.marginRight = collapsedHandleWidthPx_ + 'px'; //$NON-NLS-1$
				}
				else {
					dojo.removeClass(this.domNode, 'ple-Drawer-collapsed'); //$NON-NLS-1$

					// fudge 1 px to compensate for borders, etc.
					this._mainWrapper.style.marginRight = this._drawerWidth + 1 + 'px'; //$NON-NLS-1$
					this._drawerWrapper.style.width = this._drawerWidth + 'px'; //$NON-NLS-1$
				}
				
				this._set('collapsed', nowCollapsed); //$NON-NLS-1$
			},

			/**
			* Function to manually set width of drawer
			*/
			setDrawerWidth: function(/*number*/ width) {
					// fudge 1 px to compensate for borders, etc.
					this._drawerWidth = width;
					this._mainWrapper.style.marginRight = this._drawerWidth + 1 + 'px'; //$NON-NLS-1$
					this._drawerWrapper.style.width = this._drawerWidth + 'px'; //$NON-NLS-1$
			},

			animateExpand: function(animate) {
				if (!this.collapsed) return;

				this.set('collapsed', false); //$NON-NLS-1$
				this._anim && this._anim.stop();

				var drawerAnim = dojo.animateProperty({
					node: this._drawerWrapper,
					duration: this.animationDurationMillis,
					properties: {
						width: {
							start: collapsedHandleWidthPx_
						}
					}
				});

				var mainAnim = dojo.animateProperty({
					node: this._mainWrapper,
					duration: this.animationDurationMillis,
					properties: {
						marginRight: {
							start: collapsedHandleWidthPx_
						}
					}
				});

				this._anim = dojo.fx.combine([drawerAnim, mainAnim]);
				this._anim.play();
				
				this._drawerWrapper.focus();
			},

			animateCollapse: function(animate) {
				if (this.collapsed) return;

				this._anim && this._anim.stop();

				var drawerAnim = dojo.animateProperty({
					node: this._drawerWrapper,
					duration: this.animationDurationMillis,
					properties: {
						width: {
							end: collapsedHandleWidthPx_
						}
					}
				});	

				var mainAnim = dojo.animateProperty({
					node: this._mainWrapper,
					duration: this.animationDurationMillis,
					properties: {
						marginRight: {
							end: collapsedHandleWidthPx_
						}
					}
				});

				this._anim = dojo.fx.combine([drawerAnim, mainAnim]);

				var handle = null;
				handle = dojo.connect(this._anim, 'onEnd', dojo.hitch(this, function() { //$NON-NLS-1$
					this.set('collapsed', true); //$NON-NLS-1$
					dojo.disconnect(handle);
				}));

				this._anim.play();
			},
			
			_startDrag: function(e) {
				if (this._isDragging) {
					this._endDrag();
					return;
				}

				dojo.addClass(this.domNode, 'ple-Drawer-duringResize'); //$NON-NLS-1$

				// This is an invisible element that covers the drawer context
				// during resizing. This is to ensure that this widget gets
				// all the mousemove events from that area. Otherwise,
				// artifact links that create an iframe in the drawer end up
				// capturing those events, making it hard to resize the
				// drawer.
				this._drawerContentOverlay = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-Drawer-drawerContentOverlay' //$NON-NLS-1$
				}, this._drawerContentNode);

				this._isDragging = true;
				this._startX = e.clientX;
				
				this._wasCollapsed = this.collapsed;

				this._widgetStartPosition = dojo.position(this.domNode, true);

				this._mainWrapperStartMarginRight = pxToInt(this._mainWrapper.style.marginRight);
				this._drawerWrapperStartPosition = dojo.position(this._drawerWrapper, true);

				var widgetRight = this._widgetStartPosition.x + this._widgetStartPosition.w;
				this._maxRight = widgetRight - this.drawerMinWidth;
				this._maxLeft = this._widgetStartPosition.x + 200;

				dojo.forEach(this._dragListeners, dojo.disconnect);
				this._dragListeners = [];
				this._dragListeners.push(dojo.connect(document, 'onmouseup', this, this._endDrag)); //$NON-NLS-1$
				this._dragListeners.push(dojo.connect(document, 'onmousemove', this, this._onDrag)); //$NON-NLS-1$

				dojo.stopEvent(e);
			},

			_onDrag: function(e) {
				dojo.stopEvent(e);
				if (!e.which && !e.button) {
					// We might miss the mouseup event if the mouse goes over an iframe.
					// Try to check the current state of buttons to prove one is still down.
					this._endDrag(e);
					return;
				}
				
				var diffX = e.clientX - this._startX;
				this._drawerWidth = this._drawerWrapperStartPosition.w - diffX;

				if (e.clientX <= this._maxLeft) {
					return;
				}

				if (this.collapsed) {
					if (e.clientX < this._maxRight) {
						// don't follwo the drag until above min size, then snap open
						this.set('collapsed', false); //$NON-NLS-1$
						return;
					}
				}
				else {
					if (e.clientX >= this._maxRight) {
						// snap to collapsed if over min size
						this.set('collapsed', true); //$NON-NLS-1$
						return;
					}

					this._mainWrapper.style.marginRight = this._mainWrapperStartMarginRight - diffX + 'px'; //$NON-NLS-1$
					this._drawerWrapper.style.width = this._drawerWidth + 'px'; //$NON-NLS-1$
				}
			},

			_endDrag: function(e) {
				dojo.stopEvent(e);
				
				dojo.destroy(this._drawerContentOverlay);
				dojo.removeClass(this.domNode, 'ple-Drawer-duringResize'); //$NON-NLS-1$

				var barelyMoved = Math.abs(e.clientX - this._startX) < 5;

				if (barelyMoved) {
					if (this._wasCollapsed && this.collapsed) {
						this._drawerWidth = Math.max(this.inititalDrawerWidth, this._drawerWidth);
						this.animateExpand();
					}
					else if (!this._wasCollapsed && !this.collapsed){
						this.animateCollapse();
					}
				}
				
				delete this._wasCollapsed;
				delete this._isDragging;
				delete this._startX;
				delete this._dividerPosition;
				delete this._drawerCellStartPosition;
				delete this._maxRight;
				delete this._maxLeft;

				dojo.forEach(this._dragListeners, dojo.disconnect);
				this._dragListeners = [];
			},

			uninitialize: function() {
				this.inherited(arguments);

				dojo.forEach(this._dragListeners, dojo.disconnect);
			}
		}
	);

})();