/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.ui', 'EasyMessageBoxMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.util.html'); //$NON-NLS-1$

(function(){

	jazz.ui.MessageBox.setDefaultPrefix('CRCPC'); //$NON-NLS-1$

	/**
	 * Extension to jazz.ui.MessageBox that adds convenience methods for showing errors formatted
	 * in Java by a MessageBoxException.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ui.EasyMessageBox', //$NON-NLS-1$
		[ jazz.ui.MessageBox ],
		{
			easyMessages: dojo.i18n.getLocalization('com.ibm.relm.common.web.ui', 'EasyMessageBoxMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			
			/**
			 * Show the parsed JSON object (typically obtained through 
			 * dojo.fromJson(ioArgs.xhr.responseText))
			 * returned from MessageBoxException.
			 */
			showErrorObject: function(errorObject, closable) {
				var details = null;
				
				if (!dojo.isObject(errorObject)) {
					console.trace();
					console.error('Invalid error object. Not an object: ', errorObject); //$NON-NLS-1$
					throw errorObject;
				}

				if (errorObject.explanation) {
					details = [{
						message: errorObject.explanation,
						code: jazz.ui.MessageBox.STATUS_TYPES.OK
						}];
				}

				if (errorObject.userAction) {
					details = details || [];
					details.push({
						message: errorObject.userAction,
						code: jazz.ui.MessageBox.STATUS_TYPES.OK
						});
				}

				if (!errorObject.summary || !dojo.isString(errorObject.summary)) {
					console.trace();
					console.debug('Invalid error object. Missing summary: ' + dojo.toJson(errorObject)); //$NON-NLS-1$
					throw errorObject;
				}

				this.showMessage({
					type: errorObject.type || jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: errorObject.summary || dojo.toJson(errorObject),
					details: details,
					closable: closable || errorObject.closable
				});
			},

			/** Synonym for .hideMessage */
			hide: jazz.ui.MessageBox.prototype.hideMessage,

			showXhrError: function(xhr, closable) {
				if (!xhr.responseText) {
					this.showErrorObject({
						summary: this.easyMessages.serverNotResponding,
						explanation: this.easyMessages.serverNotResponding$explanation,
						userAction: this.easyMessages.serverNotResponding$useraction
					}, true);
					return;
				}

				try {
					var errorObject = dojo.fromJson(xhr.responseText);
					if (!dojo.isObject(errorObject) || !errorObject.summary) {
						// Dojo may return something weird after a parsing error, so try to check that it's OK here.
						throw new Error('Internal error: response text after error was not expected JSON. Was: ' + xhr.responseText); //$NON-NLS-1$
					}
					this.showErrorObject(errorObject, closable);
				}
				catch (e) {
					console.error('Failed to parse the error object.', e); //$NON-NLS-1$

					this.showErrorObject({
						summary: this.easyMessages.unknownError,
						explanation: jazz.util.html.escape(xhr.responseText)
					}, true);
				}
			}
		}
	);

})();

