/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by Gs ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.EditOnClickField"); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

/**
 * A text field normally used for resource names that appears semi-read-only
 * in its normal state. Clicking or tabbing on the value will turn it into a
 * normal text field that can be edited (preview vs. edit modes). This allows for fields to still be
 * editable without drawing too much attention to themselves.
 */

dojo.declare(
	"com.ibm.relm.common.web.ui.EditOnClickField", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ], 
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/EditOnClickField.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: false,

		/**
		 * Optional initial value for the field.
		 */
		value: '', //$NON-NLS-1$

		/**
		 * Optional width of field in edit and preview mode.
		 */
		width: '10em', //$NON-NLS-1$

		/**
		 * Optional CSS class to be used to apply to the display font. Default to a page title size.
		 */
		fontClass: 'j-h1', //$NON-NLS-1$

		/**
		 * Optional tooltip for the control. Used as the HTML 'title'.
		 */
		tooltip: '', //$NON-NLS-1$

		/**
		 * When not enabled, shows as a normal span with no decoration.
		 * Used for forms that have an 'edit' vs. 'use' mode where you don't want a 
		 * title to be clickable at all, but want it to take up the same space
		 * as in 'edit' mode. Can modify after construction with .set/.get widget methods.
		 */
		enabled: true,

		_onPreviewFocus: function(e) {
			this.edit();
		},

		_onFieldBlur: function() {
			this.preview();
		},

		postCreate: function() {
			this.inherited(arguments);
			this._preview.style.width = this.width;
			this._field.style.width = this.width;
			this._lastValue = this.value;

			dojo.addClass(this._preview, this.fontClass);
			dojo.addClass(this._field, this.fontClass);

			if (!this.enabled) this.set('enabled', false); //$NON-NLS-1$
		},

		/**
		 * Goes into edit mode and selects the current text value.
		 * Only works if control is currently enabled.
		 */
		edit: function() {
			if (!this.enabled) {
				console.warn('trying to edit disabled edit-on-click field', this); //$NON-NLS-1$
				return;
			}

			this._preview.style.display = 'none'; //$NON-NLS-1$
			this._field.style.display = ''; //$NON-NLS-1$
			this._field.select();
		},

		_onKeyUp: function() {
			var newValue = this._field.value;	
			if (this._lastValue != newValue) {
				this._lastValue = newValue;
				this.onModify(newValue);
			}
		},

		onModify: function(newValue) {
		},

		_setEnabledAttr: function(enable) {
			if (enable) {
				this.enabled = true;
				dojo.addClass(this._preview, 'relm-common-EditOnClickField-enabled'); //$NON-NLS-1$
				this._preview.tabIndex = 0;
			}
			else {
				this.enabled = false;
				dojo.removeClass(this._preview, 'relm-common-EditOnClickField-enabled'); //$NON-NLS-1$
				dojo.removeAttr(this._preview, 'tabIndex'); //$NON-NLS-1$
			}
		},

		_getEnabledAttr: function() {
		 	return this.enabled;
		 }, 

		/**
		 * Goes into preview mode.
		 */
		preview: function() {
			this._setPreview(this._field.value);
			this._preview.style.display = ''; //$NON-NLS-1$
			this._field.style.display = 'none'; //$NON-NLS-1$
		},

		/**
		 * Gets the current value of the field.
		 */
		getValue: function() {
			return this._field.value;
		},

		/**
		 * Updates current value in either mode.
		 */
		setValue: function(newValue) {
			this._field.value = newValue;
			this._setPreview(newValue);
		},

		_setPreview: function(newValue) {
			if (newValue) {
				com.ibm.relm.common.web.ui.util.setText(this._preview, newValue);
			}
			else {
				// do this to ensure the height doesn't change even with empty content
				dojo.empty(this._preview);
				this._preview.innerHTML = '&nbsp;'; //$NON-NLS-1$
			}
		}
	}	
);