/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.FloatingMessageBox"); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$

(function(){
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	/**
	 * Extension to jazz.ui.MessageBox that absolutely positions itself in top-middle
	 * of its positioned parent. It hovers above any content below it.
	 *
	 * Will accommodate different widths and center them properly. Does not set an explicit width; resizes
	 * to content width, though an extra CSS class can be added to explicitly set a width.
	 *
	 * WARNING: This hasn't been tested very widely, it may need some tweaking to
	 * accommodate large messages.
	 *
	 * Also support an auto-delayed hiding effect, which should only be used for
	 * brief success message that are OK to auto-dismiss.
	 */
	dojo.declare(
		"com.ibm.relm.common.web.ui.FloatingMessageBox", //$NON-NLS-1$
		[ jazz.ui.MessageBox ],
		{
			/**
			 * This extra class will be applied to this object
			 * to potentially override the width the dialog.
			 * Unless large messages are being used, auto sizing is probably preferred.
			 */
			'class': '', //$NON-NLS-1$ //$NON-NLS-2$

			postCreate: function() {
				this.inherited(arguments);
				dojo.addClass(this.domNode, 'relm-FloatingMessageBox'); //$NON-NLS-1$
				if (this['class']) { //$NON-NLS-1$
					dojo.addClass(this.domNode, this['class']); //$NON-NLS-1$
				}
			},

			/**
			 * Similar to jazz.ui.MessageBox but auto centers the dialog.
			 * @return returns 'this' dialog to allow chaining of .hideLater.
			 */
			showMessage: function(args) {
				this.inherited(arguments);

				if (args.hideStatusMessage) {
					dojo.query('#content-cell').addClass('relm-FloatingMessageBox-suppressStausMessage'); //$NON-NLS-1$ //$NON-NLS-2$
				}

				window.clearTimeout(this._hideLaterTimeout);
				var w = dojo.position(this.domNode).w;
				this.domNode.style.marginLeft = -1 * w / 2 + 'px'; //$NON-NLS-1$

				return this;
			},

			hideMessage: function() {
				dojo.query('#content-cell').removeClass('relm-FloatingMessageBox-suppressStausMessage'); //$NON-NLS-1$ //$NON-NLS-2$
				this.inherited(arguments);
			},

			/**
			 * Schedules this box to hide itself soon. Useful for
			 * success messages like "Saved Thing" that are comforting to see,
			 * but should go away on there own instead of requiring the user to click X.
			 *
			 * @param delayMillis optional milliseconds to waiting until closing the box.
			 * Defaults to 2000.
			 */
			hideLater: function(delayMillis) {
				delayMillis = delayMillis || 2000;
				window.clearTimeout(this._hideLaterTimeout);

				this._hideLaterTimeout = window.setTimeout(dojo.hitch(this, function() {
					this.hideMessage(true);
					delete this._hideLaterTimeout;
				}), delayMillis);
			}
		}
	);

})();