dojo.provide('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$

(function() {

var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;

var treeFilter_ = 'recents'; //$NON-NLS-1$

dojo.subscribe('ple.tree_filter_did_change', function(newTreeFilter) { //$NON-NLS-1$
	treeFilter_ = newTreeFilter;
});

/**
 * Convenience functions for updating the browser URL to call actions and goto new pages.
 */
dojo.mixin(com.ibm.relm.ple.web.ui.Goto, {
	
	gotoRecentlyImportedBaselines: function(importedBaselinePathSegment) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.view_path', //$NON-NLS-1$
			show: 'recentlyImportedBaselines', //$NON-NLS-1$
			path: importedBaselinePathSegment
		});	},

	gotoRdfProperties: function(itemVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'com.ibm.relm.explorer.web.showExplorer', //$NON-NLS-1$
			resourceUri: itemVersionUri
		});
	},

	gotoShowHistory: function(itemVersionUri, partUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.history', //$NON-NLS-1$
			product: itemVersionUri,
			partUri: partUri,
			time: Date.now()
		});
	},

	gotoFindUse: function(itemVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.find_use', //$NON-NLS-1$
			product: itemVersionUri,
			time: Date.now()
		});
	},

	gotoFindArtifactUse: function(artifactUri, privateBaseline) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.find_use', //$NON-NLS-1$
			artifact: artifactUri,
			privateBaseline: privateBaseline,
			time: Date.now()
		});
	},

	gotoReadOnlyProductPage: function(resourceVersionUri) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'ple.read_only_product', //$NON-NLS-1$
			product: resourceVersionUri
		});
	},

	/**
	 * Update action URL to view a product tree path.
	 * @param  {array} path array of path segments to a tree node to view.
	 *     Does not have to be loaded already.
	 * @return { boolean } true if the path neede to be changed and was. false if the current location/hash
	 * was already set to the given path. This is used by tree refresh code to know whether
	 * the details view will automatically be loaded (due the the hashchange event and action it fires)
	 * if if an explicit refresh of the details is still needed.
	 */
	 gotoProductTreePath: function(path) {
	 	if (!dojo.isArray(path)) 
	 		throw new Error('"path" should be an array of tree IDs'); //$NON-NLS-1$

	 	var args = {
	 		action: 'ple.view_path', //$NON-NLS-1$
	 		path: path.join('/') //$NON-NLS-1$
	 	};

	 	if (treeFilter_) {
	 		args.show = treeFilter_;
	 	}

	 	var selectedCount = TreeSelection.getAllSelectedSites().length;
	 	if (selectedCount > 1) {
	 		// This just make multi-select URLs different from single.
	 		// We don't really want a URL specific for selections (otherwise we need to support refresh  on them).
	 		// So we just use the last selected item as the URL (refresh would show that). The problem
	 		// with that is it's possible to change the selection to one item and have teh URL not change,
	 		// thus the single product not know to load. This ensures a true single selection, and a mutli-select
	 		// with the same last product have different URLs.
	 		args.select = 'multiple'; //$NON-NLS-1$
	 		// XXX: This still seems to cause a problem going <Back after a mutli select where the selected node
	 		// is lost. Not sure why. Two backs will work always... Corner case.
	 	}


	 	// XXX: Firefox has a bug where it decodes the hash twice, so location.hash is not safe.
	 	// See http://stackoverflow.com/questions/1703552/encoding-of-window-location-hash
	 	var oldHash = location.href.split('#')[1]; //$NON-NLS-1$

	 	var newHash = dojo.objectToQuery(args); //$NON-NLS-1$

	 	if (oldHash === newHash) {
	 		return false;
	 	}
	 	else {
	 		window.location.hash = newHash;
	 		return true;
	 	}
	 },

	 gotoFindPaths: function(itemVersionUri) {
	 	window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
	 		action: 'ple.find_use', //$NON-NLS-1$
	 		product: itemVersionUri,
	 		time: Date.now()
	 	});
	 },

	/**
	 * Run product comparison
	 * @param  {[string]} rightItemVersionUri require right side of compare.
	 * @param  {[string]} leftItemVersionUri  optional left side of compare. If null, the last snapshot of the right is
	 * compared against.
	 */
	 gotoCompare: function(leftSite, rightSite, streamCompare) {

	 	var params = {
	 		leftConfigVersionUri: leftSite.configVersionUri || leftSite.contextConfigVersionUri,
	 		leftPartUri: leftSite.configVersion  ? null : leftSite.contextPartUri
	 	};

	 	if (rightSite) {
		 	params.rightConfigVersionUri = rightSite.configVersionUri || rightSite.contextConfigVersionUri;
		 	params.rightPartUri = rightSite.configVersion ? null : rightSite.contextPartUri;
	 	}

	 	if (streamCompare) params.streamCompare = 'true'; //$NON-NLS-1$

	 	params.action = 'ple.compare'; //$NON-NLS-1$
		
		window.location.href = '#' + dojo.objectToQuery(params); //$NON-NLS-1$
	}

	});

})();