/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.LabeledSelect'); //$NON-NLS-1$

// dojo.require('dojo.i18n');
// dojo.requireLocalization('com.ibm.relm.common.web.ui', 'LabeledSelectMessages');
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	/**
	 * Pairs a SELECT element with a LABEL for better accesibility. Since this
	 * is a widget and supports .focus(), it can be used in a jazz.ui.Toolbar with
	 * arrow key navigation. To prevent changing the selected option, you must first
	 * click ENTER or SPACE to actually focus on the SELECT and change it.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ui.LabeledSelect', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/LabeledSelect.html'), //$NON-NLS-1$ //$NON-NLS-2$

			label: '', //$NON-NLS-1$
			title: '', //$NON-NLS-1$

			constructor: function() {
				this._selectId = dijit.getUniqueId('relm-LabeledSelect'); //$NON-NLS-1$
				this._labelId = dijit.getUniqueId('relm-LabeledSelect'); //$NON-NLS-1$
			},

			/**
			 * Equivalent to select.onchange event. 
			 * @param  {event} e
			 */
			onChange: function(e) {
				// callback
			},

			postCreate: function() {
				this.inherited(arguments);
				this.select = this.containerNode;

				if (this.title) {
					dojo.attr(this.domNode, 'title', this.title); //$NON-NLS-1$
				}
			},

			/**
			 * Moves focus from the outer widget to the actual SELECT
			 * element when user clicks <Enter> or <Space>.
			 * @param  {event} e keydown event.
			 */
			_focusOnSelect: function(e) {
				switch (e.keyCode) {
				case dojo.keys.ENTER:
				case dojo.keys.SPACE:
				case dojo.keys.UP_ARROW:
				case dojo.keys.DOWN_ARROW:
					this.select.focus();
					break;
				default:
					return;
				}
			},

			/**
			 * Focuses on the outer node (not the SELECT).
			 * This is needed for proper arrow key navigation in toolbar.s
			 */
			focus: function() {
				this.domNode.focus();
			}
		}
	);

})();