/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.LoadingMessage"); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.common.web.ui", "LoadingMessageMessages"); //$NON-NLS-2$ //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	var messages_ = dojo.i18n.getLocalization("com.ibm.relm.common.web.ui", "LoadingMessageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	var html = jazz.util.html;

	/**
	 * Simple loading message with a spinner icon in front of it.
	 * Also supports showing placeholder text for things like "No Results"
	 * after something has loaded, but isn't found.
	 */
	dojo.declare(
		"com.ibm.relm.common.web.ui.LoadingMessage", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/LoadingMessage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			_spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web','ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			messages: messages_,

			absolute: false,
			showing: true,
			top: '', //$NON-NLS-1$
			marginTopBottom: '', //$NON-NLS-1$

			placeholder: messages_.noResultsDefaultPlaceholder,

			postCreate: function() {
				this.inherited(arguments);

				if (this.marginTopBottom && this.marginTopBottom.length) {
					this.domNode.style.marginTop = this.domNode.style.marginBottom = this.marginTopBottom;
				}

				if (this.absolute) {
					this.positionAbsolute();
				}

				if (!this.showing) {
					this.hide();
				}
			},

			positionAbsolute: function(fromTop) {
				dojo.addClass(this.domNode, 'relm-LoadingMessage-absolute'); //$NON-NLS-1$
				var top = fromTop || this.top;

				if (top && top.length > 0) {
					this.domNode.style.top = top;
				}
			},

			/**
			 * Hides both placeholder and loading message.
			 */
			hide: function() {
				clearTimeout(this._delayTimeout);
				this.domNode.style.display = 'none'; //$NON-NLS-1$
			},

			_setPlaceholderAttr: function(newPlaceholderText) {
				dojo.empty(this._placeholder);
				this._placeholder.innerHTML = html.escape(newPlaceholderText);
			},

			_setHtmlPlaceholderAttr: function(newPlaceholderHtml) {
				dojo.empty(this._placeholder);
				this._placeholder.innerHTML = newPlaceholderHtml;
			},

			/**
			 * Show non-animated placeholder text. Must set .placeholder
			 * string when this widget is constructed.
			 */
			showPlaceholder: function() {
				this._placeholder.style.display = ''; //$NON-NLS-1$
				this._spinner.style.display = 'none'; //$NON-NLS-1$
				this.domNode.style.display = ''; //$NON-NLS-1$
			},

			/**
			 * Show the loading message.
			 * @param  {number} delayMillis amount of time to wait before showing. Used
			 * to prevent flashing of the loading message when the results is expected to
			 * come back very quickly.
			 */
			show: function(delayMillis) {
				clearTimeout(this._delayTimeout);

				if (!delayMillis) {
					this._placeholder.style.display = 'none'; //$NON-NLS-1$
					this._spinner.style.display = ''; //$NON-NLS-1$
					this.domNode.style.display = ''; //$NON-NLS-1$
					return;
				}

				var self = this;
				this._delayTimeout = setTimeout(function() {
					if (!self || !self.domNode) return;
					self._placeholder.style.display = 'none'; //$NON-NLS-1$
					self._spinner.style.display = ''; //$NON-NLS-1$
					self.domNode.style.display = ''; //$NON-NLS-1$
					delete self._delayTimeout;
				}, delayMillis);
			}
		}
	);

})();