dojo.provide('com.ibm.relm.ple.web.ui.MoveReusePlaceState'); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'MoveReusePlaceStateMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProgressDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){
var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'MoveReusePlaceStateMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	
var NONE = "NONE"; //$NON-NLS-1$
var MOVE = "MOVE"; //$NON-NLS-1$
var REUSE = "REUSE"; //$NON-NLS-1$

var action_ = NONE;
var allSelectedSites_ = [];

var Site = com.ibm.relm.ple.web.ui.Site;
var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
var RemoveDialog = com.ibm.relm.ple.web.ui.pages.RemoveDialog;
	
dojo.mixin(com.ibm.relm.ple.web.ui.MoveReusePlaceState, {

	NONE: "NONE", //$NON-NLS-1$
	MOVE: "MOVE", //$NON-NLS-1$
	REUSE: "REUSE", //$NON-NLS-1$
	
	
	isPlaceReady: function()
	{
		return allSelectedSites_.length > 0 && (action_ == MOVE || action_ == REUSE);
	},
	
	onChosenChange: function() {
		// no-op callback for listening to changes. Used to enable Place button.
	},

	/**
	 * Gets a message to display to the user when a new action is occurring and it depends on
	 * what the previous action was.
	 * 
	 * @param newAction 	The new action which is in the process of being executed.
	 * @returns {String}	The message to display to the user, based on the new action and the
	 * 						previously executed action.
	 */
	getUserMessage: function(newAction){
		var userMessage = messages.errorMessage;

		if (newAction == MOVE){
			userMessage = allSelectedSites_.length == 1 ? messages.moveMessage : dojo.replace(messages.moveXMessage, [allSelectedSites_.length] );
		}
		else if (newAction == REUSE){
			userMessage = allSelectedSites_.length == 1 ? messages.reuseMessage : dojo.replace(messages.reuseXMessage, [allSelectedSites_.length] );
		}
		else {
			dojo.publish('ple.product_tree_show_message', [userMessage, jazz.ui.MessageBox.STATUS_TYPES.ERROR, true]); //$NON-NLS-1$
			throw new Error('MoveReusePlaceState: Unknown type: ' + action_); //$NON-NLS-1$
		}

		return userMessage;
	},

	_moveCurrentSelection: function() {
		this.moveAnything(allSelectedSites_, TreeSelection.getSelectedSite());
	},

	moveAnything: function(fromSites, toSite) {
		if (!fromSites.length) return;

		var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
			// primaryTitle: messages.moveDialogTitle,
			width: '300px', //$NON-NLS-1$
			explanation: messages.movingProgress,
			canCancel: false,
			mirrorStatusMessage: true,
			iconClass: 'ple-icon-move' //$NON-NLS-1$
		});
		progressDialog.startup();
		
		var message = com.ibm.relm.common.web.StatusMessage.show(messages.movingProgress);

		var self = this;
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/ple/move', //$NON-NLS-1$
			content: {
				toSite: Site.siteToJsonSite(toSite),
				fromSites: Site.sitesToJsonSites(fromSites)
			},

			handle: function() {
				message.hide();
			},
		
			load: dojo.hitch(this, function () {
				self.clearSavedSelection();

				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
					selectPath: toSite.path,
					openPath: true
				}).then(function() {
					progressDialog.close();													
				});
			}),
		
			error: function(error, ioArgs) {
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh()
					.addBoth(function() {
						progressDialog.close();								
					});
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			}
		});
	},

	_reuseCurrentSelection: function() {
		this.reuseAnything(allSelectedSites_, TreeSelection.getSelectedSite());
	},

	reuseAnything: function(fromSites, toSite) {
		if (!fromSites.length) return;

		var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
			// primaryTitle: messages.reuseDialogTitle,
			width: '300px', //$NON-NLS-1$
			canCancel: false,
			mirrorStatusMessage: true,
			iconClass: 'ple-icon-reuse' //$NON-NLS-1$
		});
		progressDialog.startup();
		
		var message = com.ibm.relm.common.web.StatusMessage.show(messages.reusingProgress);

		var self = this;
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/ple/reuse', //$NON-NLS-1$
			content: {
				toSite: Site.siteToJsonSite(toSite),
				fromSites:Site.sitesToJsonSites(fromSites)
			},

			handle: function() {
				message.hide();
			},
		
			load: function () {
				self.clearSavedSelection();

				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
					selectPath: toSite.path,
					openPath: true
				}).addBoth(function() {
					progressDialog.close();													
				});
			},
		
			error: function(error, ioArgs) {
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh()
					.addBoth(function() {
						progressDialog.close();								
					});
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			}
		});
	},

	_saveSelection: function(newAction) {
		action_ = newAction;
		allSelectedSites_ = TreeSelection.getAllSelectedSites();
		
		var userMessage = this.getUserMessage(newAction);
		dojo.publish('ple.product_tree_show_message', [userMessage, jazz.ui.MessageBox.STATUS_TYPES.INFO, true]); //$NON-NLS-1$
		this.onChosenChange();
	},

	clearSavedSelection: function(){
		action_ = NONE;
		allSelectedSites_ = [];
		this.onChosenChange();
		dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
	},

	saveSelectionForMove: function() {
		this._saveSelection(MOVE);
	},


	saveSelectionForReuse: function() {
		this._saveSelection(REUSE);
	},

	placeSavedSelection: function() {
		if (action_ == MOVE) {
			this._moveCurrentSelection();
		}
		else if (action_ == REUSE) {
			this._reuseCurrentSelection();
		}
		else {
			dojo.publish('ple.product_tree_error', [ {summary: "Must select to Move or Reuse first."}]); //TODO: nls//User should never see this. //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
	}
	
});	

})();