/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.OnOffButton"); //$NON-NLS-1$

/**
 * A jazz.ui.Toolbar compatible button that can be in an ON or OFF state. This
 * is similar to a TwoStateButton, but doesn't have a "toggle on" orange
 * appearance. This is used to toggle something between two states, but
 * shouldn't show either as visually enabled.
 *
 * Toggle between two CSS image classes (not images), and optionally two title
 * attributes.
 */
dojo.declare(
	'com.ibm.relm.ple.web.ui.OnOffButton', //$NON-NLS-1$
	[jazz.ui.toolbar.Button],
	{
		/**
		 * Must pass in extra imgClasses and optionally titles to use in the
		 * ON and OFF states.
		 * @param args.imgClassOn the on image class. Used as default imgClass
		 *     if none other specified.
		 * @param args.imgClassOff the off image class.
		 * @param args.titleOn the optional title for the on state. Used as
		 *     the default title if none other specified.
		 * @param args.titleOff the optional title for the off state.
		 */
		constructor: function(args) {
			if (!args.imgClass) {
				this.imgClass = args.imgClassOn;
			}

			if (!args.title) {
				this.title = args.titleOn;
			}
		},

		/**
		 * Checks if the button is currently toggled on.
		 */
		isToggledOn: function() {
			return this.get('imgClass') === this.imgClassOn; //$NON-NLS-1$
		},

		/**
		 * Toggles between on and off states for the button.
		 * @param optionalOnState if given as true or false, explicitly sets
		 *     the button state and image to match. If not given (undefined)
		 *     then toggles to the alternate state (ON->OFF, or OFF->ON).
		 */
		toggleOnOff: function(optionalOnState) {
			var nowOn = optionalOnState;

			if (typeof nowOn === 'undefined') { //$NON-NLS-1$
				var wasOn = this.isToggledOn();
				nowOn = !wasOn;
			}

			if (nowOn) {
				dojo.addClass(this.domNode, 'ple-OnOffButton-on'); //$NON-NLS-1$
				this.set('imgClass', this.imgClassOn); //$NON-NLS-1$
				if (this.titleOn) {
					this.set('title', this.titleOn); //$NON-NLS-1$
					// alt text is redundant here
					dojo.attr(this._img, 'alt', ''); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			else {
				dojo.removeClass(this.domNode, 'ple-OnOffButton-on'); //$NON-NLS-1$
				this.set('imgClass', this.imgClassOff); //$NON-NLS-1$
				if (this.titleOff) {
					this.set('title', this.titleOff); //$NON-NLS-1$
					// alt text is redundant here
					dojo.attr(this._img, 'alt', ''); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
	}
);
