/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.ProductBreadcrumbs"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui", "ProductBreadcrumbsMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.Breadcrumbs'); //$NON-NLS-1$

(function(){

	var treeFilter_ = 'recents'; //$NON-NLS-1$

	/**
	 * Breadcrumb widget with built-in knowledge of the Products pages. Always shows 'Products > '
	 * as the first item.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.ProductBreadcrumbs', //$NON-NLS-1$
		[ jazz.ui.Breadcrumbs ],
		{
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'ProductBreadcrumbsMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * If 'true' in a template, will automatically add a 'My Products' crumb.
			 * This should be set on all pages that can be loaded  from the My Products
			 * page.
			 * @type {Boolean} true to so 'My Products >'.
			 */
			fromMyProducts: false,

			constructor: function() {
				this.segments = [{
					label: this.messages.products,
					href : '#action=ple.welcome' //$NON-NLS-1$
				}];
			},

			postCreate: function() {
				this.inherited(arguments);

				this.segments = [{
					label: this.messages.products,
					href : '#action=ple.welcome' //$NON-NLS-1$
				}];

				if (this.fromMyProducts) {
					this.addFromMyProducts();
					this.refresh();
				}

				if (this.fromMyProducts) {
					this.subscribe('ple.breadcrumbs.treeFilterChanged', this._onTreeFilterChange); //$NON-NLS-1$
				}

			},

			_onTreeFilterChange: function() {
				this.segments.length = 1;

				if (this.fromMyProducts) {
					this.addFromMyProducts();
					this.refresh();
				}
			},

			addFromMyProducts: function() {
				if (treeFilter_ === 'favorites') { //$NON-NLS-1$
					this.segments.push({
						label: this.messages.favorites,
						href : '#action=ple.browse_products&show=favorites' //$NON-NLS-1$
					});
				}
				else if (treeFilter_ === 'recents') { //$NON-NLS-1$
					this.segments.push({
						label: this.messages.recentlyViewed,
						href : '#action=ple.browse_products&show=recents' //$NON-NLS-1$
					});
				}
				else {
					this.segments.push({
						label: this.messages.myProducts,
						href : '#action=ple.browse_products&show=' + encodeURIComponent(treeFilter_) //$NON-NLS-1$
					});
				}
			}
		}
	);
	
	/**
	 * Product breadcrumb text needs to match the current tree filter (Favorites and Recents vs. Favorites).
	 * This static function is used to set the current tree filter. This will update future
	 * breadcrumbs and any one already created (even if hidden).
	 * @param {string} newTreeFilter well-known name of new tree filter.
	 */
	dojo.subscribe('ple.tree_filter_did_change',function(newTreeFilter) { //$NON-NLS-1$
		treeFilter_ = newTreeFilter;
		dojo.publish('ple.breadcrumbs.treeFilterChanged'); //$NON-NLS-1$
	});
})();