/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.common.web.ui.ProductNameCache'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('jazz.client.xhr'); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.common.web.ui", "ProductNameCacheMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function(){
	var isDefaultName = true;
	var messages = dojo.i18n.getLocalization("com.ibm.relm.common.web.ui", "ProductNameCacheMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	// The values here must match the keyword names as defined in ProductNameUtil.ProductKeyword.
	var PROPERTIES = {
		Product: "Product", //$NON-NLS-1$
		Products: "Products", //$NON-NLS-1$
		product: "product", //$NON-NLS-1$
		products: "products", //$NON-NLS-1$
		Aproduct: "Aproduct", //$NON-NLS-1$
		aproduct: "aproduct", //$NON-NLS-1$
		Theproduct: "Theproduct", //$NON-NLS-1$
		theproduct: "theproduct", //$NON-NLS-1$
		Theproducts: 'Theproducts', //$NON-NLS-1$
		theProduct: 'theProduct', //$NON-NLS-1$
		theProducts: 'theProducts', //$NON-NLS-1$
		theproducts: 'theproducts', //$NON-NLS-1$
		aProduct: 'aProduct', //$NON-NLS-1$
		isDefault: "isDefault"  //$NON-NLS-1$
	};
	var PRODUCT_NAME_REGEX = /\{((?:a|the)?products?)\}/ig;

	/**
	* Takes a string and checks if it contains any product term keywords and substitutes them. 
	* 
	* @param {productNameString - String} A string that should be scanned or product name keywords and substituted
	* @return The string with product name keywords filled in
	*/
	com.ibm.relm.common.web.ui.ProductNameCache.substituteString = function(productNameString) {
		return dojo.replace(productNameString,  function(_,b) {
					 	return messages[b];
					}, PRODUCT_NAME_REGEX);
					return page;
	};

	jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + '/configure/productName', //$NON-NLS-1$
			handleAs: "json", //$NON-NLS-1$,
			sync: true,
			load: function(args) {
				isDefaultName = args[PROPERTIES.isDefault];
				if (isDefaultName === "false") { //$NON-NLS-1$
					delete args[PROPERTIES.isDefault];
					for( var i in args) {
						args[i] = jazz.util.html.escape(args[i]);
					}
					messages = args;
				}
			},

			error: function(error, args) {
				//Do nothing since default name will be used
			}
	});

	var orignialGetLocalization = dojo.i18n.getLocalization; 

	dojo.i18n.getLocalization = function(/*String*/ packageName, /*String*/ bundleName, /*String?*/ locale) {
		var bundle =  orignialGetLocalization(packageName, bundleName, locale);
		for(var i in bundle){
 		   var value = bundle[i];
 		   if (dojo.isString(value)) {
 		   		bundle[i] = dojo.replace(value,  function(_,b) {
 		   			var keyword = PROPERTIES[b];
 		   			var replacement = messages[keyword];
 		   			if (!replacement) {
 		   				//Should never happen but this makes it easier to track
 		   				console.error("Unknown product term keyword:", b); //$NON-NLS-1$
 		   			}
				 	return replacement;
				}, PRODUCT_NAME_REGEX);
 		   }
		}
		return bundle;
	};

	//Function to change the term product on pages. Results shown in Navigation menu and "Loading Page..." message
	(function() {
		var origPageRegistry = net.jazz.ajax.ui.internal.PageRegistry;
		var origGetPages = origPageRegistry.getPages;
		var origFindPageWithId = origPageRegistry.findPageWithId;
		
		var newGetPages = function(removeSuppressed) {
			var pageArray = origGetPages(removeSuppressed);
			dojo.forEach(pageArray, function(page, i){
				if (page.id == "com.ibm.relm.ple.web.ui.pages.ProductsWelcomePage") { //$NON-NLS-1$
					if (isDefaultName === false) {
						dojo.replace(page.name, function(_,b) {
							return messages[b];
						}, PRODUCT_NAME_REGEX);
					} 
				}
			});
			return pageArray;
		};

		var newFindPageWithId = function(pageId) {
			var page = origFindPageWithId(pageId);
			page.name = dojo.replace(page.name,  function(_,b) {
			 	return messages[b];
			}, PRODUCT_NAME_REGEX);
			return page;
		};
		
		origPageRegistry.getPages = newGetPages;
		origPageRegistry.findPageWithId = newFindPageWithId;
	})();
})();