/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.ProductPath'); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('jazz.ui.HoverPopup'); //$NON-NLS-1$

(function(){
	var formatting = com.ibm.relm.ple.web.ui.formatting;

	/**
	 * A clickable visual representation of a product path that acts 
	 * like a big button and goes to the My Products tree.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.ProductPath", //$NON-NLS-1$
		[ dijit._Widget ],
		{

			/**
			 * Array of JSON items. Children must have a pathSegment entry to form valid
			 * paths.
			 * @type {array}
			 */
			path: null,

			// Optional tooltip
			tooltip: '', //$NON-NLS-1$

			/**
			 * Read-only array access to the cell nodes used for path segments.
			 * Used to resize cells on Find Use page.
			 */
			cells: null,

			constructor: function() {
				this.cells = [];
				this.hoverPopups = [];
			},

			buildRendering: function() {
				this.domNode = dojo.create('a', { //$NON-NLS-1$
					className: 'ple-ProductPath', //$NON-NLS-1$
					title: this.tooltip
				});

				this.inherited(arguments);
			},

			postCreate: function() {
				this.inherited(arguments);

				// This uses a table to ensure the cells don't wrap--we always want the path to take up
				// just one line. Using floats or inline elements won't guarantee that.
				// var table = dojo.create('table', {role:'presentation'}); //$NON-NLS-1$ //$NON-NLS-2$
				// var tbody = dojo.create('tbody', {}, table); //$NON-NLS-1$
				// var row = dojo.create('tr', {}, tbody); //$NON-NLS-1$

				var frag = document.createDocumentFragment();

				// This image is a bit ugly right now. Will use > for now
				var path = [];

				var secondToLast = this.path.length - 2;
				var tempNode = dojo.create('span'); //$NON-NLS-1$

				var ariaLabels = [];

				dojo.forEach(this.path, function(pdItem, i) {
					path.push(pdItem.pathSegment);
					
					if (i > secondToLast) return;

					var needsHover = false;
					var segmentInnerHtml;

					var segmentNode;

					if (i === 0 || i === secondToLast) {
					var fullNameHtml = formatting.formatProductIconAndTitleForItem(pdItem);
						// First and last segments show full name
						segmentNode = dojo.create('span', { //$NON-NLS-1$
							innerHTML: fullNameHtml,
							className: 'ple-ProductPath-segment' //$NON-NLS-1$
						}, frag);
					}
					else {
						// Middle segments just show icon
						var iconOnlyHtml = formatting.formatProductIconAndTitleForItem(pdItem, true);

						segmentNode = dojo.create('span', { //$NON-NLS-1$
							innerHTML: iconOnlyHtml,
							className: 'ple-ProductPath-segment' //$NON-NLS-1$
						}, frag);

						tempNode.innerHTML = fullNameHtml;
						var fullNameText = tempNode.textContent;

						// Give them a rich hover though
						var hoverPopup = new jazz.ui.HoverPopup({
							around: segmentNode,
							orientation: 'T', //$NON-NLS-1$
							openDelay: 0,
							closeDelay: 0,
							persist: false,
							createContents: function() {
								return dojo.create('span', { //$NON-NLS-1$
									innerHTML: iconOnlyHtml
								});
							}
						});

						this.hoverPopups.push(hoverPopup);
						this.own(hoverPopup);
					}

					// if (pdItem.type === 'link') { //$NON-NLS-1$
					// 	dojo.addClass(segmentNode, 'ple-ProductPath-artifactLink'); //$NON-NLS-1$
					// }

					this.cells.push(segmentNode);

					if (i === 0) {
						segmentNode.className += ' ple-ProductPath-firstSegment'; //$NON-NLS-1$
					}
					else if (i === secondToLast) {
						segmentNode.className += ' ple-ProductPath-lastSegment'; //$NON-NLS-1$
					}
				}, this);

				var href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
					action: 'ple.view_path', //$NON-NLS-1$
					path: path.join('/') //$NON-NLS-1$
				});

				dojo.attr(this.domNode, 'href', href); //$NON-NLS-1$

				this.connect(this.domNode, 'onclick', this._onClick); //$NON-NLS-1$

				// var targetName = this.path[this.path.length - 1].name;
				// // var targetName = document.createTextNode(this.path[this.path.length - 1].name);
				// // 
				// var n = dojo.create('span', { //$NON-NLS-1$
				// 	innerHTML: targetName,
				// 	className: 'j-hint', //$NON-NLS-1$
				// 	style: 'padding-left: 15px;' //$NON-NLS-1$
				// });
				// frag.appendChild(n);
				
				this.domNode.appendChild(frag);
			},

			_onClick: function(e) {
				// Normally the browser does the right thing with the simple link.
				if (e.target.nodeName === 'A') return; //$NON-NLS-1$

				// But not IE always, so manually change the URL for its sake.
				dojo.stopEvent(e);
				window.location.href = dojo.attr(this.domNode, 'href'); //$NON-NLS-1$
			},

			uninitialize: function() {
				this.inherited(arguments);
			}
		}
	);

})();