/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.ProductSidebar"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui", "ProductSidebarMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.AutoHeightBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('jazz.util.i18n'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.AltProductTree'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.SimpleProductStore'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$

(function(){
	var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
	var util = com.ibm.relm.common.web.ui.util;
	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var Goto = com.ibm.relm.ple.web.ui.Goto;

	function pathsEqual(left, right) {
		if (left.length != right.length) return false;

		return dojo.every(left, function(leftPart, i) {
			return leftPart == right[i];
		});
	}

	/**
	 * Displays the product trees and associated toolbar in the left-side, master navigation pane.
	 * Defers much work to ProductsPage, which can see this (master) and the details view, which
	 * often need to be in sync.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.ProductSidebar", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/templates/ProductSidebar.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			treeFilter: 'recents', //$NON-NLS-1$
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'ProductSidebarMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * Creates a new product store and models to be used with the trees.
			 * Called during postCreate, but can be called again during a refresh.
			 * Cannot be called during construction, since event handling is only enabled during postCreate.
			 * Trees must be recreated (with the new models) after calling this.
			 */
			_createStoreAndModels: function() {
				this.store = new com.ibm.relm.ple.web.ui.tree.SimpleProductStore();

				TreeSelection.store = this.store;
			},

			_showEmptyTreeMessage: function(isEmpty) {
				this._emptyTreeNode.style.display = isEmpty ? '' : 'none' ; //$NON-NLS-1$ //$NON-NLS-2$
			},

			_setTreeFilterAttr: function(newTreeFilter) {
				if (this.treeFilter === newTreeFilter) return;

				this.treeFilter = newTreeFilter;
				dojo.publish('ple.tree_filter_did_change', [newTreeFilter]); //$NON-NLS-1$
				this._fullRefresh();
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this.subscribe('ple.select_product_in_tree', this._onTreeClick); //$NON-NLS-1$

				this._createStoreAndModels();

				this._autoScrollHeight = new com.ibm.relm.common.web.ui.AutoHeightBox(this._scrollBox, {
					listenOnPage: 'com.ibm.relm.ple.web.ui.pages.ProductsPage' //$NON-NLS-1$
				});

				this._productTree = this._newTree();
				this._productTree.placeAt(this._treeBoundary);

				this.subscribe('ple.refresh_favorites', this.refreshFavorites); //$NON-NLS-1$
				var self = this;

				this.subscribe('ple.delayed_favorites_refresh', function() { //$NON-NLS-1$
					if (this.treeFilter !== 'favorites') return; //$NON-NLS-1$
					self._needsDelayedRefresh = true;

					clearTimeout(self._refreshFavoritesTimeout);
					self._refreshFavoritesTimeout = setTimeout(function() {
						self._refreshFavoritesIfNeeded();
					}, 3500);
				});


				dojo.forEach(['onmouseleave', 'onmouseenter'], function(eventName) { //$NON-NLS-1$ //$NON-NLS-2$
					this.connect(this._treeBoundary, eventName, this._refreshFavoritesIfNeeded);
				}, this);

				this._emptyTreeNode.innerHTML = this.messages.noProductsInTreeHtml;
				// var links = jazz.util.i18n.addMessage(this._emptyTreeNode, this.messages.noProductsInTreeHtml);
				var links = dojo.query('.ple-mark-create, .ple-mark-findProduct', this._emptyTreeNode); //$NON-NLS-1$

				this.connect(links[0], 'onclick', function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					com.ibm.relm.ple.web.ui.productapi.createConfigVersion();
				});
				this.connect(links[1], 'onclick', this._showProductQuickSearch.bind(this)); //$NON-NLS-1$

				this.subscribe('ple.product_tree_error', this.showError); //$NON-NLS-1$
				this.subscribe('ple.product_tree_xhrError', this.showXhrError); //$NON-NLS-1$
				this.connect(this._messageBox, 'hideMessage', function() { //$NON-NLS-1$
					self._autoScrollHeight.resize();
				});

				this.subscribe('ple.product_tree_show_message', this.showMessage); //$NON-NLS-1$
				this.subscribe('ple.product_tree_hide_message', this.hideMessage); //$NON-NLS-1$

				/* Used when a new product is created, but not yet inserted into the tree. */
				this.subscribe('ple.new_product_created', this.addNewProductToTree); //$NON-NLS-1$
			},

			flashNodeAtPath: function(path, flashArgs) {
				this._productTree.flashNodeAtPath(path, flashArgs);
			},
			
			showError: function(errorObject) {
				this._messageBox.showErrorObject(errorObject, true);
				this._autoScrollHeight.resize();
			},

			showXhrError: function(error, closable) {
				this._messageBox.showXhrError(error, closable);
				this._autoScrollHeight.resize();
			},
			
			showMessage: function(summary, type, closable) {
				this._messageBox.showMessage({
					type: type,
					summary: summary,
					closable: closable
				});

				this._autoScrollHeight.resize();
			},

			hideMessage: function() {
				this._messageBox.hideMessage();
				this._autoScrollHeight.resize();
			},

			_showProductQuickSearch: function(e) {
				this._showQuickSearch(e);
			},

			_showQuickSearch: function(e) {
				dojo.stopEvent(e);
				var searchInput = dojo.query('.jazz-ui-BannerSearchBox .SearchInputText')[0]; //$NON-NLS-1$
				var searchBox = dijit.getEnclosingWidget(searchInput);

				// Find the product provider and use that to ensure we have the right scope.
				dojo.forEach(searchBox.providers, function(provider) {
					if (provider.declaredClass === 'com.ibm.relm.ple.web.ui.pages.ProductSearchProvider') { //$NON-NLS-1$
						searchBox._setScope({
							provider: provider,
							scope: provider.scopes[0],
							focusText: false
						});

						// Now focus on the box.
						searchInput.focus();
					}
				});
			},

			_refreshFavoritesIfNeeded: function() {
				if (this._needsDelayedRefresh) {
					delete this._needsDelayedRefresh;
					this.refreshFavorites();
				}
			},

			refreshFavorites: function() {
				this._productTree.queryRoots();
			},

			/**
			 * Returns the selected node in the last clicked tree, or `undefined` if nothing selected.
			 */
			getSelectedTreeNode: function() {
				return this._productTree.get('selectedNode'); //$NON-NLS-1$
			},

			/**
			 * Returns the path to the selected node as an array of tree IDs.
			 * The invisible root node is excluded.
			 */
			getSelectedTreePath: function() {
				if (!this._productTree) return [];

				var paths = this._productTree.getAllSelectedItemPaths();
				if (!paths.length) return [];

				var lastPath = paths[paths.length - 1];

				return dojo.map(lastPath, function(item) {
					return TreeSelection.store.getValue(item, 'pathSegment'); //$NON-NLS-1$
				});
			},

			/**
			 * Returns the number of selected artifacts or products in the last clicked tree, or `undefined` if nothing selected.
			 */
			getNumberOfSelected: function() {
				var nodes = this._productTree.get('selectedNodes'); //$NON-NLS-1$
				
				if (!nodes) 
					return 0;
				else
					return nodes.length;
			},
			
			/**
			 * Completely rebuilds the store, models, and visual tree. The tree widget is destroyed and rebuilt.
			 * Used for a last resort refresh that will always pull the latest information.
			 * If the tree doesn't exist yet, a new tree will be created with the current tree filter.
			 *
			 * @param preserveSelectedPath if true, will save the current path and attempt to reselect it 
			 * after refreshing the tree. Useful after a manual refresh so that you don't lose you place,
			 * but not wanted after setting a new tree filter.
			 */
			_fullRefresh: function(preserveSelectedPath) {
				var lastPath = this.getSelectedTreePath();
				dojo.publish('ple.clear_details'); //$NON-NLS-1$

				this._createStoreAndModels();

				this._productTree && this._productTree.destroyRecursive();
				this._emptyTreeNode.style.display = 'none'; //$NON-NLS-1$

				this._productTree = this._newTree();

				this._productTree.placeAt(this._treeBoundary);

				if (preserveSelectedPath) {
					this.selectProductTreePath(lastPath);
				}
			},

			_walkLoadedItems: function(item) {
				var store = this._productTree.store;

				if (store.getValue(item, 'type)' !== 'product')) { //$NON-NLS-1$ //$NON-NLS-2$
					return undefined;
				}

				var itemInfo = {
					itemVersionUri: store.getValue(item, 'uri') //$NON-NLS-1$
				};

				var children = store.getValue(item, 'children'); //$NON-NLS-1$

				if (children && dojo.isArray(children) && children.length > 0) {
					itemInfo.children = [];

					for (var i = 0; i < children.length; i++) {
						var child = children[i];
						var childInfo = this._walkLoadedItems(child);
						if (childInfo) itemInfo.children.push(childInfo);
					}
				}

				return itemInfo;
			},

			_getItemsToRefresh: function() {
				var toRefresh = [];
				// var rootChildren = this._productTree.rootNode.item.children;
				var rootChildren = this._productTree.rootTreeNodes;

				for (var i = 0; i < rootChildren.length; i++) {
					var topLevelItem = rootChildren[i].item;
					toRefresh.push(this._walkLoadedItems(topLevelItem));
				}

				return toRefresh;
			},

			/**
			 * Light-weight refresh that attempts to only update the tree with what has changed.
			 */
			refresh: function() {
				dojo.publish("ple.product_tree_hide_message"); //$NON-NLS-1$
				JazzUIWorkbench.showMessage(this.messages.refreshingProducts);
				var def = this._productTree.refresh();
				return def;
			},

			/**
			 * A light-weight refresh that attempts to select the node with the
			 * given URI after it's done. Useful after a create operation to
			 * select the new node.
			 *
			 * WARNING: there may be multiple places in the tree with the same product URI,
			 * so this isn't very reliable and should be replaced.
			 *
			 * @param uri the URI of the product that should be selected.
			 */
			refreshAndSelectProduct: function(uri) {

				var deferred = new dojo.Deferred();

				// Default to Recents for when a new top level product is created and nothing is selected yet.
				var tree = this._productTree;

				this.refresh().then(function() {
					return tree.selectTreeNodeWithPropertyAndValue('uri', uri); //$NON-NLS-1$
				}).then(function() {
					// Focus the keyboard on the new node
					var realPath = tree.get('path'); //$NON-NLS-1$
					var currentItem = null;
					if (realPath && realPath.length > 0 ) {
						currentItem = realPath[realPath.length - 1];
						var nodes = tree.getNodesByItem(currentItem);

						if (nodes && nodes.length > 0) {
							util.flashNode(nodes[nodes.length - 1].domNode);
							tree.focusNode(nodes[nodes.length - 1]);
						}
					}

					if (currentItem) deferred.callback(currentItem);
				});

				return deferred;
			},

			/**
			 * A light-weight refresh that attempts to select the node with the
			 * given URI after it's done. Useful after a create operation to
			 * select the new node.
			 *
			 * WARNING: there may be multiple places in the tree with the same product URI,
			 * so this isn't very reliable and should be replaced.
			 *
			 * @param uri the URI of the product that should be selected.
			 */
			// refreshAndSelectPath: function(path) {

			// 	var deferred = new dojo.Deferred();
			// 	var self = this;

			// 	// Default to Recents for when a new top level product is created and nothing is selected yet.
			// 	var tree = this._productTree;

			// 	this.refresh().then(function() {
			// 		return self.selectProductTreePath(path);
			// 	}).then(function() {
			// 		// Focus the keyboard on the new node
			// 		var realPath = tree.get('path');
			// 		var currentItem = null;
			// 		if (realPath && realPath.length > 0 ) {
			// 			currentItem = realPath[realPath.length - 1];
			// 			var nodes = tree.getNodesByItem(currentItem);

			// 			if (nodes && nodes.length > 0) {
			// 				util.flashNode(nodes[nodes.length - 1].domNode);
			// 				tree.focusNode(nodes[nodes.length - 1]);
			// 			}
			// 		}

			// 		if (currentItem) deferred.callback(currentItem);
			// 	});

			// 	return deferred;
			// },
			
			/**
			 * Sets the selected Tree Node to the uri, should only be called when something like the
			 * browser URL changes what is being viewed and the tree needs to reflect the change
			 * @param uri Uri of the item to select
			 */
			selectTreeNodeWithUri: function(uri) {
				var tree = this._productTree;
				tree.selectTreeNodeWithPropertyAndValue('uri', uri); //$NON-NLS-1$
			},
				
			/**
			 * Deselects from_productTree 
			 */
			deselectTrees: function() {
				this._productTree && this._productTree.set('path', ''); //$NON-NLS-1$ //$NON-NLS-2$
				TreeSelection.clearSelectedSites();
			},
			
			/**
			 * Cosntructs and returns a new product tree. Just a convenience
			 * factory method.
			 */
			_newTree: function(model, onLoad) {
				var tree = new com.ibm.relm.ple.web.ui.tree.AltProductTree({
					store: this.store,
					rootQuery: this.treeFilter,
					autoHeightBox: this._autoScrollHeight
				});

				this._emptyTreeHandle && dojo.disconnect(this._emptyTreeHandle);
				this._emptyTreeHandle = dojo.connect(tree, 'emptyStateDidChange', this, this._showEmptyTreeMessage); //$NON-NLS-1$

				this._onClickHandle && dojo.disconnect(this._onClickHandle);
				this._onClickHandle = dojo.connect(tree, 'onClickTreeNode', this, this._onTreeClick); //$NON-NLS-1$
				
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.setTree(tree);

				return tree;
			},

			/**
			 * Callback after a tree node is clicked. Changes the details view
			 * to reflect the now selected artifact of product.
			 */
			_onTreeClick: function(item, treeNode) {
				var type = TreeSelection.store.getValue(item, 'type'); //$NON-NLS-1$

				var allSelected = TreeSelection.getAllSelectedSites();

				if (allSelected.length === 0) {
					dojo.publish('ple.select_nothing'); //$NON-NLS-1$
					return;
				}

				var path = this.getSelectedTreePath();

				switch (type) {
				case 'product': //$NON-NLS-1$
				case 'configVersion': //$NON-NLS-1$
				case 'link': //$NON-NLS-1$
				case 'emptyPart': //$NON-NLS-1$
					Goto.gotoProductTreePath(path);
					return;

				default:
					console.error('_onTreeClick: Unknown type: ' + type); //$NON-NLS-1$
					break;
				}
			},

			/**
			 * Used when selecting a path in the tree, but the root
			 * node is not already in the tree. Will fetch the top-level node
			 * from the server and insert into the tree. Then the path will be
			 * expanded, assuming the path is valid.
			 * 
			 * This has the side-effect of adding the top-level product to the 
			 * Recently Viewed Products list for the user.
			 * 
			 * @param  {array} path path path segments to the expand and select.
			 * The first item will be a top-level item in the tree. The last item,
			 * if expanded to, will be loaded in the detalis view.
			 * 
			 */
			_fetchTopLevelConfigVersionAndRetrySelectProductTreePath: function(path, tree) {
				var store = tree.store;
				var rootPathSegment = path[0];

				store.fetchItemByRootPathSegment({
					pathSegment: rootPathSegment,
					onItem: dojo.hitch(this, function(topItem) {
						// Add TOP to Recently Viewed Products so the user
						// can open the path again later. It's more useful to get to the
						// leaf product, but then it would be out of context if edited
						// This is async.
						productapi.addToRecentlyViewed(store.getValue(topItem, 'uri')); //$NON-NLS-1$

						tree.appendRootItem(topItem);

						this.selectProductTreePath(path, true);
					}),
					onError: function() {
						// TODO: can't find it. SHow error?
						JazzUIWorkbench.hideMessageArea();
						dojo.publish('ple.clear_details', [{ clearTreeSelection: true }]); //$NON-NLS-1$
					}
				});
			},

			/**
			 * Selects and path in the product tree and load the
			 * leaf item. If the top-level product is not already in the tree
			 * is will be loaded and added as a Recently Viewed Product.
			 * 
			 * @param  {array} path        array of tree IDs that make up the path to select.
			 * @param  {boolean} secondAttempt if falsey, if the top-level item is not
			 * in the tree then it can be fetched from the server and added as a root item.
			 * That will cause this same method to be called again after the fetch, but with
			 * this set as true. When true, we just give up if not already loaded to prevent
			 * looping forever.
			 */
			selectProductTreePath: function(path, secondAttempt) {
				if (!path || path.length === 0) {
					dojo.publish('ple.clear_details'); //$NON-NLS-1$
					return;
				}

				var tree = this._productTree;

				var rootPathSegment = path[0];
				var targetProductId = path[path.length - 1];

				if (tree.get('loading')) { //$NON-NLS-1$
					// The tree hasn't finished loading yet. Poll until it's done.
					dojo.publish('ple.clear_details', [{showLoadingMessage: true}]); //$NON-NLS-1$

					clearTimeout(this._waitingForTreeTimeout);
					this._waitingForTreeTimeout = setTimeout(
						dojo.hitch(this, this.selectProductTreePath, path, secondAttempt),
						100);
					return;
				}

				clearTimeout(this._waitingForTreeTimeout);
				
				var topLevelProductAlreadyInTree = dojo.some(tree.rootTreeNodes, function(rootTreeNode) {
					return rootTreeNode.item.pathSegment === rootPathSegment;
				});

				if (!topLevelProductAlreadyInTree) {
					if (secondAttempt) {
						// There's no reason to believe this will work next time, so just give up.
						console.warn('Could not load first product in path'); //$NON-NLS-1$
						return;
					}

					this._fetchTopLevelConfigVersionAndRetrySelectProductTreePath(path, tree);
					return;
				}

				var oldPaths = tree.getAllSelectedItemPaths();

				var newPathIsAlreadyInTreePaths = dojo.some(oldPaths, function(oldPath) {
					oldPath = dojo.map(oldPath, function(item) {
							return item.pathSegment;
						});

					return pathsEqual(oldPath, path);
				});

				var deferredSetPath = null;
				
				if (!newPathIsAlreadyInTreePaths) {
					// Path is not set (probably loaded from URL, so load and wait
					// deferredSetPath = tree.set('path', newDijitPath);
					var andSelect = true;
					deferredSetPath = tree.openPath(path, andSelect);
				}

				dojo.when(
					// Deferred OR 'undefined' if this was synchronous
					deferredSetPath, 

					// callback
					dojo.hitch(this, function() {
						if (!this._validateSetPathAndLoadItem(tree, targetProductId)) {
							// This happens if we do a multi/checkbox select, then click Back in the browser.
							// The results is that the path in the URL probably is selected, but unlikely to
							// be the single path we validate against. We just give up in that case and clear the
							// whole selection. It's a pain to try an back out the entire selection process,
							// and isn't that useful. We just want to be sure the client is left in a good state
							// that the user can follow.
							TreeSelection.clearSelectedSites();
							this._productTree.clearAllSelectedPaths();
							dojo.publish('ple.clear_details', [{clearTreeSelection: true}]); //$NON-NLS-1$
						}
					}),

					// errback
					function(error) {
						// TODO: more error handling
						JazzUIWorkbench.hideMessageArea();
						TreeSelection.clearSelectedSites();
						dojo.publish('ple.clear_details', [{ clearTreeSelection: true}]); //$NON-NLS-1$
					}
				);

			},

			/**
			 * Validates that a just set 'path' in the tree points to a target //$NON-NLS-1$
			 * ID. This would not be the case if the path was stale in some way,
			 * and is no longer a valid path.
			 * 
			 * If the path is valid, and the target product is selected, loads
			 * the product in the details pane.
			 * 
			 * @param  {object} tree            the tree to check th ecurrent path of.
			 * @param  {string} targetProductId tree ID of the item we expect is selected. This can be
			 *                                  for any tree item: product or link.
			 * @return {boolean} true if it was loaded, otherwise false.
			 */
			_validateSetPathAndLoadItem: function(tree, targetProductId, deferred) {
				// Now that the tree is expanded and has a selected, make sure
				// it actually matches what we asked for. Otherwise, we might
				// havbe tried to load a stale path.
				// We only care about the last path in the case of multi selects.
				var truePaths = tree.getAllSelectedItemPaths();
				var truePath = truePaths[truePaths.length - 1];

				if (truePaths.length === 0 || truePath.length === 0) {
					console.warn('Tree has no selected path', truePaths); //$NON-NLS-1$
					return false;
				}

				var currentSelectedItem = truePath[truePath.length - 1];
				var parentItem = truePath[truePath.length - 2];
				var currentSelectedPathSegment = currentSelectedItem.pathSegment;

				if (currentSelectedPathSegment !== targetProductId) {
					console.warn('Path to product is stale. A product in the path was probably moved or deleted.'); //$NON-NLS-1$
					return false;
				}
				else {
					var type = currentSelectedItem.type;

					if (truePaths.length > 1) {
						dojo.publish('ple.multi_select_details'); //$NON-NLS-1$
						return true;
					}

					switch (type) {
					case 'product': //$NON-NLS-1$
						var contextConfigVersionUri = currentSelectedItem.contextConfigVersionUri;
						dojo.publish('ple.load_item_version_or_config_version_details', [currentSelectedItem]); //$NON-NLS-1$
						return true;
					
					case 'configVersion': //$NON-NLS-1$
						dojo.publish('ple.load_item_version_or_config_version_details', [currentSelectedItem]); //$NON-NLS-1$
						return true;
					
					case 'link': //$NON-NLS-1$
						dojo.publish('ple.load_link_details', [currentSelectedItem]); //$NON-NLS-1$
						return true;

					case 'emptyPart': //$NON-NLS-1$
						dojo.publish('ple.load_empty_part', [currentSelectedItem]); //$NON-NLS-1$
						return true;

					default:
						console.warn('Unknown type: ' + type); //$NON-NLS-1$
						return false;
					}
				}

			},

			uninitialize: function() {
				this._onClickHandle && dojo.disconnect(this._onClickHandle);
				this._emptyTreeHandle && dojo.disconnect(this._emptyTreeHandle);

				this._productTree.destroyRecursive();
				this._autoScrollHeight.destroy();

				this.inherited(arguments);
			}
		}
	);
})();