/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.ProgressDialog'); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'ProgressDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit.ProgressBar'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui", "ProgressDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.declare(
		'com.ibm.relm.ple.web.ui.ProgressDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/templates/ProgressDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			messages: messages,

			primaryTitle: '', //$NON-NLS-1$
			explanation: '', //$NON-NLS-1$
			width: '350px', //$NON-NLS-1$
			canCancel: true,
			mirrorStatusMessage: false,
			iconClass: null,
			
			postCreate: function() {
				this.inherited(arguments);

				if (!this.canCancel) {
					this._cancelButton.style.visibility = 'hidden'; //$NON-NLS-1$
				}

				if (this.explanation) {
					this.dialogStatusMessage.show(this.explanation);
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.primaryTitle || messages.defaultTitle,
					width: this.width,
					closable: false,
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: this.iconClass
				});
			},

			onCancel: function() {
			},

			cancel: function() {
				this.onCancel();
				this._dialog.close();
			},

			close: function() {
				this._dialog.close();
			},

			uninitialize: function() {
				this.inherited(arguments);
			}
		}
	);

	var ProgressDialog = com.ibm.relm.ple.web.ui.ProgressDialog;

	/**
	 * Runs a normal xhrGet method with standard callbacks, but if the wait time is too long
	 * for the callback, puts up a indeterminate progress dialog with the option to cancel the operation.
	 * After a cancel, normal callbacks will not be called, but an optional onCancel will (with no args).
	 * 
	 * @param  {object} args Normal dojo xhrArgs + args.primaryTitle for title of the progress
	 * dialog (defaults to 'Progress'), and args.explanation for the line describing what's happening.
	 * @return {Deferred}      from the XHR request.
	 */
	ProgressDialog.slowXhrGet = function(args) {

		var original = {
			handle: args.handle,
			load: args.load,
			error: args.error
		};

		delete args.handle;
		delete args.load;
		delete args.error;

		var dialog = null;
		var isCancelled = null;
		var onCancelHandle = null;
		var xhrDeferred = null;

		var showTimeout = setTimeout(function() {
			dialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
				primaryTitle: args.primaryTitle || messages.defaultTitle,
				explanation: args.explanation || '' //$NON-NLS-1$
			});

			onCancelHandle = dojo.connect(dialog, 'onCancel', function() { //$NON-NLS-1$
				isCancelled = true;
				xhrDeferred.cancel();
				args.onCancel && args.onCancel;
			});

			dialog.startup();

		}, args.showDialogInMillis || 400);

		args.load = function() {
			if (isCancelled) return;
			if (original.load) original.load.apply(null, arguments);
		};

		args.error = function() {
			if (isCancelled) return;
			if (original.error) original.error.apply(null, arguments);
		};

		args.handle = function() {
			clearTimeout(showTimeout);
			if (onCancelHandle) dojo.disconnect(onCancelHandle);
			dialog && dialog.close();
			if (original.handle) original.handle.apply(null, arguments);
		};

		xhrDeferred = jazz.client.xhrGet(args);

		// XXX: not intercepting the callbacks from this if the caller uses it.
		return xhrDeferred;
	};

})();