/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.VvcRefresh'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$

/**
 * Used to refresh configuration bindings, that is, sync the RELM configurations
 * with the VVC configurations and ensure RELM is using the configuration members
 * that VVC has selected. This is only really needed after a crash or some other
 * disaster where the two apps got out of sync.
 * 
 */
(function() {
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;

	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	var activeConfigVersionUris_ = {};

	/**
	 * Refreshes the configuration version 
	 * @param  {[type]} configVersion [description]
	 * @return {[type]}               [description]
	 */
	function refreshConfigVersion(configVersion) {
		if (!relm.app.Permissions.canProductsModify()) {
			// Just ignore if you can't modify.
			return;
		}

		if (activeConfigVersionUris_[configVersion.uri]) {
			console.info('Already waiting for refresh of configuration ' + configVersion.name); //$NON-NLS-1$
			return;
		}

		var statusMessage = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(messages.vvcRefreshProgressOfConfigurationX, [configVersion.name])); //$NON-NLS-1$

		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/ple/vvc_refresh', //$NON-NLS-1$
			content: {
				configVersionUri: configVersion.uri
			},

			handle: function() {
				statusMessage.hide();
				delete activeConfigVersionUris_[configVersion.uri];
			},

			load: function (result) {
				if (!result.needsRefresh) return;
				
				var site = TreeSelection.getSelectedSite();
				var currentPath = site ? site.path : null;

				TreeRefresh.refresh({
					selectPath: currentPath
				});
			},

			error: function(error, ioArgs) {
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			}
		});
	}

	com.ibm.relm.ple.web.ui.VvcRefresh.refreshConfigVersion = refreshConfigVersion;
})();