/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.audit.ProductAudit"); //$NON-NLS-1$

dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.audit", "AuditMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('jazz.ui.ResourceLink'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){

	var util = com.ibm.relm.common.web.ui.util;

	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.audit", "AuditMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	function newCell(nodeName, withText, inRow, className, isXmlValue) {
		var td = document.createElement(nodeName);
		if (isXmlValue) {
			// Only titles are saved as XML, and they should be clean and
			// safe. I don't see a need to call .stripScript here, but may
			// need to reevaluate in the future.
			td.innerHTML = withText;
		}
		else {
			util.setText(td, withText);
		}
		if (inRow) inRow.appendChild(td);
		if (className) td.className = className;
		return td;
	}

	function newTd(withText, inRow, className, isXmlValue) {
		return newCell('TD', withText, inRow, className, isXmlValue); //$NON-NLS-1$
	}

	function newRowTh(withText, inRow, className, isXmlValue) {
		var th = newCell('TH', withText, inRow, className, isXmlValue); //$NON-NLS-1$
		th.scope = 'row'; //$NON-NLS-1$
		return th;
	}

	function newHtmlTd(innerHtml, inRow, className) {
		var td = document.createElement('TD'); //$NON-NLS-1$
		td.innerHTML = innerHtml;
		if (inRow) inRow.appendChild(td);
		if (className) td.className = className;
		return td;
	}

	function artifactLinkHtmlString(auditEntry) {
		if (auditEntry.isDoorsBaseline) {
			return dojo.replace('<a class="jazz-ui-ResourceLink ple-ProductAudit-doorsBaseline" target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry); //$NON-NLS-1$ //$NON-NLS-2$	
		}
		return dojo.replace('<a class="jazz-ui-ResourceLink" target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * Formats the entry into a human readable string for the What or Change column.
	 * @return the human readable string. Can be HTML with links in it!
	 */
	function formatAuditChangeRow(auditEntry) {
		var type = auditEntry.type;

		var changeRow = dojo.create('tr'); //$NON-NLS-1$

		switch (type)
		{
			case 'addedProperty': //$NON-NLS-1$
			case 'changedProperty': //$NON-NLS-1$
			newRowTh(auditEntry.propertyName, changeRow, 'j-label'); //$NON-NLS-1$
			newTd(auditEntry.newValue, changeRow, null, auditEntry.isXmlValue);
			break;
		
		case 'removedProperty': //$NON-NLS-1$
			newRowTh(auditEntry.propertyName, changeRow, 'j-label'); //$NON-NLS-1$
			newTd(auditEntry.oldValue, changeRow, 'ple-ProductAudit-removed', auditEntry.isXmlValue); //$NON-NLS-1$
			break;
		
		case 'addedChild': //$NON-NLS-1$
			newRowTh(messages.addedChild, changeRow, 'j-label'); //$NON-NLS-1$
			var newChild = com.ibm.relm.ple.web.ui.formatting.formatDefaultProductIconAndTitle('product', auditEntry.childName, auditEntry.branchName); //$NON-NLS-1$
			newHtmlTd(newChild, changeRow);
			break;
		
		case 'removedChild': //$NON-NLS-1$
			newRowTh(messages.removedChild, changeRow, 'j-label'); //$NON-NLS-1$
			var removedChild = com.ibm.relm.ple.web.ui.formatting.formatDefaultProductIconAndTitle('product', auditEntry.childName, auditEntry.branchName); //$NON-NLS-1$
			newHtmlTd(removedChild, changeRow, 'ple-ProductAudit-removed'); //$NON-NLS-1$
			break;
		
		case 'addedHyperlink': //$NON-NLS-1$
			newRowTh(messages.addedHyperlink, changeRow, 'j-label'); //$NON-NLS-1$
			newTd('', changeRow).innerHTML = dojo.replace('<a target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry); //$NON-NLS-1$ //$NON-NLS-2$
			break;
		
		case 'removedHyperlink': //$NON-NLS-1$
			newRowTh(messages.removedHyperlink, changeRow, 'j-label'); //$NON-NLS-1$
			newTd('', changeRow, 'ple-ProductAudit-removed').innerHTML = dojo.replace('<a target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			break;
		
		case 'addedArtifact': //$NON-NLS-1$
			newRowTh(messages.addedArtifact, changeRow, 'j-label'); //$NON-NLS-1$
			newHtmlTd(artifactLinkHtmlString(auditEntry), changeRow); //$NON-NLS-1$
			break;
		
		case 'removedArtifact': //$NON-NLS-1$
			newRowTh(messages.removedArtifact, changeRow, 'j-label'); //$NON-NLS-1$
			newHtmlTd(artifactLinkHtmlString(auditEntry), changeRow,  'ple-ProductAudit-removed'); //$NON-NLS-1$
			break;
		
		case 'created': //$NON-NLS-1$
			if (auditEntry.isImported)
				newRowTh(messages.imported, changeRow, 'j-label'); //$NON-NLS-1$
			else
				newRowTh(messages.created, changeRow, 'j-label'); //$NON-NLS-1$
			
			var createdReason =
				dojo.replace(messages.byUserX, [jazz.util.html.escape(auditEntry.who)]);
			newHtmlTd(createdReason, changeRow);
			break;
		
		case 'snapshot': //$NON-NLS-1$
			newRowTh(messages.addedToSnapshot, changeRow, 'j-label'); //$NON-NLS-1$
			var byUserX = dojo.replace(messages.byUserX, [jazz.util.html.escape(auditEntry.who)]);
			newHtmlTd(byUserX, changeRow); //$NON-NLS-1$
			break;

		default:
			console.error('unknown audit entry', auditEntry); //$NON-NLS-1$
			return type;
		}

		return changeRow;
	}

	dojo.declare(
		"com.ibm.relm.ple.web.ui.audit.ProductAudit", //$NON-NLS-1$
		[ dijit._Widget ],
		{
			constructor: function(args, node)
			{
				this.auditData = args.auditData;
				this.resourceLinks = [];
				this._batchLoader = args.batchLoader;
			},

			postCreate: function() {
				this.inherited(arguments);
				dojo.addClass(this.domNode, 'ple-ProductAudit'); //$NON-NLS-1$

				// Must pre-sort the data. The table won't do it for us, it
				// just marks a column as what it's already sorted by .

				this.table = dojo.create('table', { //$NON-NLS-1$
					className: 'ple-ProductAudit-table', //$NON-NLS-1$
					 role:'presentation' //$NON-NLS-1$
				}, this.domNode);

				this.tbody = dojo.create('tbody', {}, this.table); //$NON-NLS-1$

				this._drawTable();
			},

			_destroyResourceLinks: function() {
				dojo.forEach(this.resourceLinks, function(link) {
					link.destroyRecursive();
				});

				this.resourceLinks.length = 0;
			},

			/**
			 * Empties the current table and recreates it from the current auditData.
			 * Will draw sorted in the order of the audit data.
			 */
			_drawTable: function() {
				this._destroyResourceLinks();
				dojo.empty(this.tbody);

				var frag = document.createDocumentFragment();

				var entriesLength = this.auditData.length;

				for (var i = 0; i < entriesLength; i++) {
					var auditEntry = this.auditData[i];

					var whoAndWhenRow = dojo.create('tr', {}, frag); //$NON-NLS-1$

					// Who
					dojo.create('th', { //$NON-NLS-1$
						innerHTML: auditEntry.who,
						className: 'ple-ProductAudit-header', //$NON-NLS-1$
						scope: 'col' //$NON-NLS-1$
					}, whoAndWhenRow);

					// When
					var dateString = dojo.date.locale.format(
						new Date(auditEntry.when),
						{ formatLength: 'medium' } //$NON-NLS-1$
					);

					var whenCell = dojo.create('th', { //$NON-NLS-1$
						className: 'ple-ProductAudit-header', //$NON-NLS-1$
						scope: 'col' //$NON-NLS-1$
					}, whoAndWhenRow);
					util.setText(whenCell, dateString);

					var changesLength = auditEntry.changes.length;

					// Walk backwards. This put the newest changes in each group at the top,
					// which aggrees with the rest of the audit layout (newest products at top in history,
					// newest groups of audit changes at top.)
					for (var j = changesLength - 1; j >= 0; j--) {
						var change = auditEntry.changes[j];

						var changeRow = formatAuditChangeRow(change);

						if (j === 0) {
							dojo.query('td', changeRow).addClass('ple-ProductAudit-lastInChunk'); //$NON-NLS-1$ //$NON-NLS-2$
						}

						frag.appendChild(changeRow);

					}
				}

				// Now look for all links tagged with the ResourceLink class.
				// Convert all these links into rich hover ResorceLinks.
				// We must destroy these ourselves.
				var self = this;
				dojo.query('a.jazz-ui-ResourceLink', frag).forEach(function(plainLink) { //$NON-NLS-1$
					
					var iconNode = null;
					
					// Skip user links. We don't need an icon on all of them.
					if (!dojo.hasClass(plainLink, 'ple-ProductAudit-who')) { //$NON-NLS-1$
						iconNode = dojo.create('img', { //$NON-NLS-1$
							height: '16', //$NON-NLS-1$
							width: '16', //$NON-NLS-1$
							src: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
						});
						dojo.place(iconNode, plainLink, 'before'); //$NON-NLS-1$
					}

					if (dojo.hasClass(plainLink, 'ple-ProductAudit-doorsBaseline')) { //$NON-NLS-1$
						iconNode.src = com.ibm.relm.ple.web.doors.DoorsUtils.DOORS_BASELINE_ICON;
						dojo.removeClass(plainLink, 'jazz-ui-ResourceLink'); //$NON-NLS-1$
					} else {
						var lazy = self._batchLoader !== null;

						var link = new jazz.ui.ResourceLink({
							lazyFetch: lazy,
							iconNode: iconNode
						}, plainLink);
						self.resourceLinks.push(link);

						if (self._batchLoader) {
							self._batchLoader.add(link);
						}
					}
				});

				this.tbody.appendChild(frag);
			},

			uninitialize: function() {
				this._destroyResourceLinks();
				this.inherited(arguments);
			}
		}
	);

})();