/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.branch.BranchDialogMixin"); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.branch.ValueOrderingRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.EditDimensionRow'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'BranchDialogMixinMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'BranchDialogMixinMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.declare(
		"com.ibm.relm.ple.web.ui.branch.BranchDialogMixin", //$NON-NLS-1$
		[],
		{
			constructor: function() {
				this._didDrawTable = false;
			},
			
			startup: function() {
				this._focusFirstField();
			},

			postCreate: function() {
				this.scopeId = this.id + "_scope"; //$NON-NLS-1$

				if (this._loadingMessage) {
				// Such a hack. Just hard code the typical size for "empty" dimensions here. Better than nothing...
					this._loadingMessage.contentNode.style.marginTop = '17px'; //$NON-NLS-1$
				} 
				this.initToolbar();
				this.initTable();
				this.subscribe("ple.dimension_data_loaded", this._updateDimensionForScope); //$NON-NLS-1$
			},

			initToolbar: function() {
				this._addButton = new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-add', //$NON-NLS-1$
					title: messages.addDimensionButton,
					label: messages.addDimensionButton,
					onClick: dojo.hitch(this, this._addOnClick),
					enabled: false
				});
				this._toolbar.addChild(this._addButton);
			},

			initTable: function() {
				var columns = [ 
					{
						label: messages.dimensionSharedColumnLabel,
						width: '100px', //$NON-NLS-1$
						className: "ple-BranchDialog-shared-label ple-BranchDialog-csh" //$NON-NLS-1$
					},
					{
						label: messages.dimensionNameColumnLabel,
						width: '999px', //$NON-NLS-1$
						className: "ple-BranchDialog-dimension-label ple-BranchDialog-csh" //$NON-NLS-1$
					},
					{
						label: messages.dimensionValueColumnLabel,
						width: '999px', //$NON-NLS-1$
						className: "ple-BranchDialog-value-label ple-BranchDialog-csh" //$NON-NLS-1$
					},
					{
						label: '', //$NON-NLS-1$
						width: '1px' //$NON-NLS-1$
					}
				];

				this._table = new jazz.ui.StyledTable({columns: columns});
				this._widgets.push(this._table);

				// Not strictly needed since each widget monitors onchange,
				// but ensures evaluation prior to focus change.
				this.connect(this._table.tbody, 'onkeyup', this._monitorCreateButton); //$NON-NLS-1$

				// Adding hover help into a styled table isn't really supported. We'll guess where
				// the title is and wrap it in a span to add the hover help to
				var uaKeys = ['com.ibm.relm.ple.web.text_Scope001', 'com.ibm.relm.ple.web.text_Dimension001', 'com.ibm.relm.ple.web.text_Value001']; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

				dojo.query('.ple-BranchDialog-shared-label div', this._table._headerRow)[0].id = this.scopeId; //$NON-NLS-1$
				dojo.query('.ple-BranchDialog-csh div', this._table._headerRow).forEach(function(titleDiv, i) { //$NON-NLS-1$
					
					var titleSpan = dojo.create('span', { //$NON-NLS-1$
						innerHTML: titleDiv.innerHTML
					});

					dojo.empty(titleDiv);
					dojo.place(titleSpan, titleDiv);

					this._widgets.push(new jazz.app.ua.TextHoverHelp({
						uaKey: uaKeys[i],
						contentNode: titleSpan
					}));
				}, this);

				this._table.placeAt(this._dimensionTable);

				this._loadAllScopesAndUsedDimensions();
			},

			/**
			 * Loads all possible dimensions to populate the pick lists with.
			 * Stores them in this._allDimensions.
			 * Once loaded, draws the table.
			 */
			_loadAllScopesAndUsedDimensions: function() {
				var usedScopes = this._oldBranchDimensions && this._oldBranchDimensions.map(function(dimension) {
					return dimension.scope;
				});

				var start = Date.now();
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/branch', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						//Most of the time time there will be no itemUri, but for create branch dialog there is one, and it is needed to show
						//the product local dimensions
						itemUri: this.itemUri,
						usedScopes: usedScopes,
						method: "loadScopes", //$NON-NLS-1$
						isConfigVersion: this.isConfigVersion
					},

					handle: function() {
						this._loadingMessage && this._loadingMessage.hide();
						this._addButton.set('enabled', true); //$NON-NLS-1$
					}.bind(this),

					load: function(allDimensions) {
						this._allDimensions = allDimensions;
						this._forceInScopesForUnusedDimensions(this._oldBranchDimensions);
						this._drawTable(this._oldBranchDimensions);
						console.debug("Time to load and draw scopes", Date.now() - start); //$NON-NLS-1$
					}.bind(this),

					error: function(error, ioArgs) {
						var messageBox = this.messageBox || this._messageBox;
						messageBox && messageBox.showXhrError(ioArgs.xhr);
						this._allDimensions = [];
						this._drawTable();
					}.bind(this)
				});
			},

			_updateDimensionForScope: function(scopeId, dimensions) {
				this._allDimensions.forEach(function(scope) {
					if (scope.id == scopeId) {
						scope.dimensions = dimensions;
					}
				});
			},

			/**
			 * It's possible for a configuration to still be using product-local
			 * dimensions from a deleted product. In that case, they won't be found in
			 * the list of ALL shared and product-local dimensions. This will
			 * force these extra dimensions and values into the list of scopes for this
			 * dialog so that they are visible. Note that their scopes from the server
			 * always start with "urn://unused". Each scope URI is unique based in the
			 * dimensions URI to ensure each is considered a different scope in the list.
			 * 
			 * @param  {[type]} currentDimensions [description]
			 * @return {[type]}                   [description]
			 */
			_forceInScopesForUnusedDimensions: function(currentDimensions) {
				if (!currentDimensions) return;

				for (var i = 0; i < currentDimensions.length; i++) {
					var dimension = currentDimensions[i];

					if (dimension.scope.indexOf('urn://unused') === 0) { //$NON-NLS-1$
						console.debug('Adding dimension for deleted product', dimension); //$NON-NLS-1$
						var fakeScopeForUnusedDimension = {
							name: dimension.itemTitle || dimension.name,
							id: dimension.scope,
							dimensions: [{
								name: dimension.name,
								id: dimension.uri,
								values: [{
									name: dimension.value,
									id: dimension.valueUri
								}]
							}]
						};

						this._allDimensions.push(fakeScopeForUnusedDimension);
					}
				}
			},

			/*
			 * @param {scopeId} (optional) ID for the aria-labelledby of the scope comboBox
			*/
			_drawTable: function(currentDimensions, scopeId) {
				this._loadingMessage && this._loadingMessage.hide();

				if (!currentDimensions || currentDimensions.length === 0) {
					this._addRow(null, this.focusFirstDimension, scopeId); // note that focusing occurs in startup 
					this._didDrawTable = true;
					this._monitorCreateButton();
					return;
				}

				var frag = document.createDocumentFragment();

				for (var i = 0; i < currentDimensions.length; i++) {
					var dimension = currentDimensions[i];
					var row = this._newRow(dimension);
					frag.appendChild(row.domNode);
				}

				this._table.tbody.appendChild(frag);

				this._didDrawTable = true;
				if (this.focusFirstDimension) this._focusFirstField();
				this._monitorCreateButton();
			},

			/**
			 * Creates a new dimension row widget and connects to the
			 * callbacks this dialog needs.
			 * @param  {object} dimensionModel details of the dimension for this row.
			 *                                 Can be null, which creates a new, blank row.
			 * @return {EditDimensionRow} the new row. Not in the DOM yet.
			 */
			_newRow: function(dimensionModel) {
				if (!dimensionModel) {
					// Default dimension for new row
					dimensionModel = {
						scope: this.isConfigVersion
							? 'shared' //$NON-NLS-1$
							: 'local' //$NON-NLS-1$
					};
				}

				//TODO: Have set of shared stores to be used between rows
				var row = new com.ibm.relm.ple.web.ui.branch.EditDimensionRow({
					messageBox: this.messageBox,
					scopeId: this.scopeId,
					scopesDimensionsValues: this._allDimensions,
					isConfigVersion: this.isConfigVersion,
					initialScopeId: dimensionModel.scope,
					initialNameId: dimensionModel.uri,
					initialValueId: dimensionModel.valueUri,
					//These two are used to show the existing dimensions disabled while lazy-loading values
					initialName: dimensionModel.name,
					initialValue: dimensionModel.value
				});



				this.connect(row, 'onChange', function() { //$NON-NLS-1$
					this._monitorCreateButton();
				}.bind(this));


				this.connect(row, 'didDelete', function() { //$NON-NLS-1$
					this._monitorCreateButton();
				}.bind(this));

				return row;
			},

			/**
			 * Creates a new dimension row and adds to the table.
			 */
			_addRow: function(dimension, focus) {
				var row = this._newRow(dimension);
				row.placeAt(this._table.tbody);
				
				if (focus) row.focus();

				return row;
			},

			_getDeleteCallback: function(hoverRow) {
				return function() {
					hoverRow.selectors.valueOrdering && hoverRow.selectors.valueOrdering.destroyRecursive();
					hoverRow.destroyRecursive();
					this._monitorCreateButton();
				};
			},

			/**
			 * If the first row is blank, focus on the first field in it.
			 */
			_focusFirstField: function() {
				var rows = this._getCurrentRows();
				if (rows.length > 0) {
					rows[0].focus();
				}
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				this._dialog.close();
				delete this._dialog;
			},

			/**
			 * Disables or enables all dimension rows and the Add Dimension
			 * button.
			 * @param {boolean} disabled true to disable.
			 */
			setDisabledOnAllDimensionRows: function(disabled) {
				this._getCurrentRows().forEach(function(row) {
					row.set('disabled', disabled); //$NON-NLS-1$
				});

				this._addButton.set('enabled', !disabled); //$NON-NLS-1$
			},

			/**
			 * Gets all the current row widgets.
			 * @return {array} array of all the row widgets.
			 */
			_getCurrentRows: function() {
				return dijit.findWidgets(this._table.tbody).filter(function(widget) {
					return widget.declaredClass === "com.ibm.relm.ple.web.ui.branch.EditDimensionRow"; //$NON-NLS-1$
				});
			},

			/**
			 * Checks if all the rows are valid and if Save can be enabled.
			 * @return {boolean} true if all rows are OK to be saved. False
			 * if there are missing or invalid fields.
			 */
			areAllRowsValid: function() {
				var rows = this._getCurrentRows();

				for (var i = 0; i < rows.length; i++) {
					var row = rows[i];
					if (!row.isValid()) return false;
				}

				return true;
			},

			/**
			 * Convert the dimension fields to an object of name/value pairs. This function is used for
			 * validation as well as retrieving values, hence the flags.
			 *
			 * @param {boolean} ignoreEmptyDimensions - If true, doesn't throw error for empty dimension rows. Default
			 *                                        	is throwing error for empty dimension rows.
			 * @param {boolean} ignoreUnorderedValues - If true, doesn't throw error for values that don't have a specified value ordering. 
			 *                                        	True would be used to enable save button, but default is to throw error so operation
			 *                                        	cannot proceed (check out branch, create config...) with unspecified value ordering
			 *
			 * @returns An array of dimension name/value pairs.
			 * @throws Error if a name or value is blank.
			 */
			_dimensionsToObject: function(ignoreUnorderedValues) {
				if (this._submitButton.disabled) throw new Error('Currently disabled, cannot submit values'); //$NON-NLS-1$

				var dimensions = [];

				var dimensionRows = this._getCurrentRows();

				for (var i = 0; i < dimensionRows.length; i++) {
					var row = dimensionRows[i];
					var dimension = row.getResult();
					if (!dimension) continue;
					dimensions.push(dimension);
				}

				return dimensions;
			},

			_addOnClick: function() {
				var hoverRow = this._addRow(null, true); 

				com.ibm.relm.common.web.ui.util.flashNode(hoverRow.domNode, {
					duration: 500,
					endColor: '#fff4e2' //$NON-NLS-1$
				});

				this._monitorCreateButton();
			},

			/**
			 * Cancels the create and closes the dialog.
			 */
			cancel: function() {
				// this calls back to destroy outer dialog too
				this._closeDialog();
			},
			
			_monitorCreateButton: function() {
				if (this._didDrawTable === false) {
					this._submitButton.disabled = true;
					return;
				}
				
				if (!this.areAllRowsValid()) {
					this._submitButton.disabled = true;
				}
				else {
					var disabled = false;
					if (this._nameField) {
						//Check that the name field isn't empty, if it exists
						disabled = !dojo.trim(this._nameField.value);
					}
					this._submitButton.disabled = disabled;
				}
			},
			
			uninitialize: function() {
				for (var i = 0; i < this._widgets.length; i++) {
					var w = this._widgets[i];
					w.destroyRecursive();
				}
			}
		});

})();