/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.CreateBranchDialog'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.form.ComboBox'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$
dojo.require('dojo.data.ItemFileReadStore'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateBranchDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

/**
 * Dialog to branch a product version.
 */
dojo.declare(
	"com.ibm.relm.ple.web.ui.branch.CreateBranchDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch.BranchDialogMixin ],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/CreateBranchDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateBranchDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		autoComplete: false,

		focusFirstDimension: true,

		/**
		 *
		 * @param args
		 * 	Required fields:
		 * 		itemVersion
		 * 	Optional fields:
		 * 		contextConfigVersionUri
		 * 		contextPartUri
		 */
		constructor: function(args)
		{
			this._helpHeader = dojo.replace(this._messages.helpHeaderProductX, [jazz.util.html.escape(args.itemVersion.name)]);
			this._oldBranchDimensions = args.itemVersion.branchDimensions || [];
			this.selectTreePath = true;
			this._widgets = [];

			//These properties are used by BranchDialogMixin			
			this.uri = args.itemVersion.uri;
			this.itemUri = args.itemVersion.conceptUri;
			this._massageOldBranchDimensionsData();
		},

		_massageOldBranchDimensionsData: function() {
			var itemUri = this.itemUri;
			
			this._oldBranchDimensions.forEach(function (dimension) {
				if (dimension.scope == "local") { //$NON-NLS-1$
					dimension.scope = itemUri;
				}
			});
		},

		postCreate: function() {
			this.inherited(arguments);

			this._width = 650;

			this._dialog = new jazz.ui.Dialog({
				primaryTitle: this._messages.dialogTitle,
				width: this._width + 'px', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode,
				iconClass: 'ple-icon-createBranch' //$NON-NLS-1$
			});

		},

		_monitorCreateButton: function() {
			if (this._createInProgress) {
				this._submitButton.disabled = true;
				return;
			}

			this.inherited(arguments);
		},

		/**
		 * Called when form is submitted (click of button or type 'Enter').
		 */
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var dimensions = null;

			try {
				dimensions = this._dimensionsToObject();
			} catch (error) {
				console.error(error);
				return;
			}

			this._createBranch(dimensions);
		},

		_createBranch: function(dimensionsObj) {
			this._messageBox.hideMessage();

			var serviceUrl = net.jazz.ajax._contextRoot + '/ple/branch'; //$NON-NLS-1$

			var payload = {
				itemVersionUri: this.itemVersion.uri,
				dimensions: dojo.toJson(dimensionsObj)
			};

			if (this.contextConfigVersionUri)
			{
				payload.contextConfigVersionUri = this.contextConfigVersionUri;
				payload.contextPartUri = this.contextPartUri;
			}

			var self = this;
			this._submitButton.disabled = this._cancelButton.disabled = true;
			this._createInProgress = true;

			var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.creatingBranch);
			this.setDisabledOnAllDimensionRows(true);

			jazz.client.xhrPost({
				url: serviceUrl,
				content: payload,
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,

				handle: function() {
					message.hide();
					self._createInProgress = false;
				},

				load: function(result) {
					self._createInProgress = false;
					dojo.publish('ple.didCheckOutItemVersionUri', [self.itemVersion.uri]); //$NON-NLS-1$

					if (self.path) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							//Change focus to new branch
							//pass in false for selectTreePath to avoid change in focus
							selectPath: self.path,
							flashPath: true
						}).addBoth(function() {
							self._closeDialog();
						});
					} else {
						//Probably coming from audit history page since only a product can be branched, which would necessitate a path being selected
						dojo.publish("ple.refresh_selected_path"); //$NON-NLS-1$
						self._closeDialog();
					}
				},
				
				error: function(_, args) {
					self.setDisabledOnAllDimensionRows(false);
					self._createInProgress = false;
					self._cancelButton.disabled = false;
					self._monitorCreateButton();
					self._messageBox.showXhrError(args.xhr);
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh();
				}
			});
		}
	});	// dojo.declare

})();
