/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.CreateDimensionDialog'); //$NON-NLS-1$

dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateDimensionDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$

(function(){
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.CreateDimensionDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/CreateDimensionDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'CreateDimensionDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			// Success callback passed into constructor.
			onDone: null,

			constructor: function(args, node)
			{
				
			},

			_createRow: function() {
				var tr = dojo.create('tr'); //$NON-NLS-1$

				var scopeTd = dojo.create('td', null, tr); //$NON-NLS-1$
				var dimensionTd = dojo.create('td', null, tr); //$NON-NLS-1$
				var valueTd = dojo.create('td', null, tr); //$NON-NLS-1$

				var options = [
					{
						label: '<img src="' + this._blankGif + '" class="ple-icon-dimensionShared"> ' + this.messages.dimensionScopeShared, //$NON-NLS-1$ //$NON-NLS-2$
						value: 'shared' //$NON-NLS-1$
					}
				];
				var initialScope = 'shared'; //$NON-NLS-1$

				if (!this.sharedOnly) {
					options.push({
						label: '<img src="' + this._blankGif + '" class="ple-icon-dimensionProductLocal"> ' + this.messages.dimensionScopeLocal, //$NON-NLS-1$ //$NON-NLS-2$
						value: 'local' //$NON-NLS-1$	
					});

					initialScope = 'local'; //$NON-NLS-1$
				}

				this._scopeSelect = new dijit.form.Select({
					maxHeight: 300,
					options: options,
					value: initialScope
				});

				this._scopeSelect.startup();
				this._scopeSelect.placeAt(scopeTd);

				this._dimensionInput = dojo.create('input', { //$NON-NLS-1$
					type: 'text', //$NON-NLS-1$
					classname: 'r-stretch', //$NON-NLS-1$
					placeholder: this.messages.dimension
				}, dimensionTd);

				this._valueInput = dojo.create('input', { //$NON-NLS-1$
					type: 'text', //$NON-NLS-1$
					classname: 'r-stretch', //$NON-NLS-1$
					placeholder: this.messages.firstDimensionValue
				}, valueTd);

				dojo.forEach([this._dimensionInput, this._valueInput], function(input) {
					this.connect(input, 'onkeyup', this._validate); //$NON-NLS-1$
				}, this);

				return tr;
			},

			_validate: function(e) {
				this._submitButton.disabled = dojo.trim(this._dimensionInput.value).length === 0 ||
					dojo.trim(this._valueInput.value).length === 0;
			},

			postCreate: function() {
				this.inherited(arguments);

				var columns = [
					{
						label: this.messages.dimensionScope,
						width: '0px' //$NON-NLS-1$
					},
					{
						label: this.messages.dimension,
						width: '49%' //$NON-NLS-1$
					},
					{
						label: this.messages.firstDimensionValue,
						width: '49%' //$NON-NLS-1$
					}
				];

				this._styledTable = new jazz.ui.StyledTable({
					columns: columns
				});
				this._styledTable.tbody.appendChild(this._createRow());
				this._styledTable.domNode.width = '100%'; //$NON-NLS-1$
				this._styledTable.placeAt(this._tableHolder);

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.createDimensionDialogTitle,
					width: '550px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-dimension' //$NON-NLS-1$
				});

				this._dialog.startup();
			},

			startup: function() {
				this._dimensionInput.focus();
			},

			onSubmit: function(e) {
				dojo.stopEvent(e);

				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$
				var dimension = dojo.trim(this._dimensionInput.value);
				var value = dojo.trim(this._valueInput.value);

				this._dialog.close();

				this.onDone && this.onDone(scope, dimension, value);
			},

			onCancel: function(e) {
				this._dialog.close();
			},

			uninitialize: function() {
				this._styledTable.destroyRecursive();		
				this.inherited(arguments);
			}
		}
	);

})();