/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.DimensionEditor'); //$NON-NLS-1$

dojo.require('jazz.ui.Tree'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.HeaderItem'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.DimensionItem'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.StatefulRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.DimensionValueRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.CreateDimensionDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.ValueOrderingRow'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionEditorMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	var State = com.ibm.relm.ple.web.ui.branch.StatefulRow.State;
	var html = jazz.util.html;

	function dimensionValueListItemFactoryFunction(dimensionValueRow) {
		// just use as is
		return dimensionValueRow.domNode;
	}

	/**
	 * Searches an "AbstractItem" in a jazz.ui.Tree for a direct child with
	 * a .model proeprty matching the given model. Used to find the tree node
	 * to remove based on a selected model.
	 */
	function findChildForModel(parentItem, model) {
		var children = parentItem.getChildren();

		for (var i = 0; i < children.length; i++) {
			var child = children[i];

			if (child.model === model) return child;
		}

		throw new Error('child not found for model ', model); //$NON-NLS-1$
	}

	function shouldSaveDimension(dimension) {
		var dirty = dimension.dirty;
		var state = dimension.state;

		if (state === State.NEW || (dirty && state !== State.DELETED)) return true;

		// Dimensions are implcitly dirty if their values are dirty.
		for (var i = 0; i < dimension.values.length; i++) {
			var value = dimension.values[i];

			if (value.state === State.NEW || value.dirty) return true;
		}

		return false;
	}

	function shouldSaveValue(value) {
		var dirty = value.dirty;
		var state = value.state;

		if (state === State.NEW || (dirty && state !== State.DELETED)) {
			return true;
		}

		return false;
	}

	function titleComparator(left, right) {
		var ltitle = left.title.toLowerCase();
		var rtitle = right.title.toLowerCase();
		if (ltitle < rtitle) {
			return -1;
		}
		else if (ltitle > rtitle) {
			return 1;
		}
		else {
			return 0;
		}
	}

	/**
	 * Given a list of value models, returns the models in sorted order based
	 * on their VVC greater-that order. An unordered node is sorted to the
	 * top, followed by the breadth-first search of its children. Then next
	 * unorderd node comes next, etc. Each "tree" rooted at an unordered node
	 * is essentially separate from the others. This results in a flat list
	 * that may be somewhat confusing of there are many confusing value
	 * dependencies; if they are all linear with not branches, the order will
	 * be obvious.
	 * 
	 * @param  {array} values array of value models. Not modified.
	 * @return {array}        a new, sorted array of the value models.
	 */
	function sortByOrder(values) {
		// Need to make two passes to build the tree. First, we map all the
		// URIs to the value models for easy lookup. Each model is given
		// a _children array to hold it children later.
		var urisToValues = [];

		values.forEach(function(value) {
			urisToValues[value.uri] = value;

			// Stuff in the temporary _children to build up a tree of values
			// with child nodes in place. The value is deleted at the end of
			// the sort.
			value._children = [];
		});

		// Now we have a lookup map, so we walk each one, track the roots when
		// we find them, and add each value as a child of it "greaterThan"
		// parent. The results in a tree for each unordered root. We'll just
		// walk it breadth first below to get the sorted valued.
		var unorderedRootValues = [];

		values.forEach(function(value) {
			if (!value.greaterThan) {
				unorderedRootValues.push(value);
			}
			else {
				var greaterThanValueModel = urisToValues[value.greaterThan];
				if (!greaterThanValueModel) {
					unorderedRootValues.push(value);
				}
				else {
					greaterThanValueModel._children.push(value);
				}
			}
		});

		// Now walk the trees and add them to sortedValues
		var sortedValues = [];

		function collectBreadthFirst(value) {
			// Collect all children before recursing to make this breadth-first
			value._children.forEach(function(child) {
				sortedValues.push(child);
			});

			// Now recurse
			value._children.forEach(collectBreadthFirst);

			// Clean up out sneaky child/tree pointer
			delete value._children;
		}

		// Sort the unordered/root nodes by name since their order isn't
		// really relevant.
		unorderedRootValues.sort(titleComparator);

		// Walk every unordered value and collect their sorted children now.
		unorderedRootValues.forEach(function(rootValue) {
			sortedValues.push(rootValue);
			collectBreadthFirst(rootValue);
		});

		return sortedValues;
	}

	/**
	 * Given a widget that supports `rename`, will call the rename function in
	 * the next event loop tick. This is a hack for IE 8 where a blur event
	 * may not fire at the right time, which results in the "rename" happening
	 * before the widget is in the dom, causing an error when focusing on the
	 * rename field. Delaying this ensures the widget is always in the dom
	 * before the rename happens.
	 * @param  {object} widget any widget with a `rename` method.
	 */
	function renameNextTick(widget) {
		if (widget.rename) {
			setTimeout(function() {
				widget.rename();
			}, 0);
		}
	}

	/**
	 * Converts from value models into the models needed to create
	 * Option nodes under the value ordering Select.
	 * 
	 * @param  {array} allValueModels all value models for the current 
	 * @param  {object} skipValueModel the model for a value. This will be skipped
	 * and not added to the result even if part of allValueModels. This skips the "current"
	 * value.
	 * @return {array}                option models like {name: 'Jay', value: 'jay'}.
	 */
	function valueModelsToOptionModels(allValueModels, skipValueModel) {
		var valueOptions = [];

		for (var i = 0; i < allValueModels.length; i++) {
			var valueModel = allValueModels[i];
			if (valueModel === skipValueModel) continue;

			valueOptions.push({
				name: valueModel.title,
				id: valueModel.uri || valueModel.title
			});
		}

		return valueOptions;
	}

	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.DimensionEditor', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/DimensionEditor.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionEditorMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			forceWatchCallbacksFor: {dirty: true},

			dirty: false,
			enabled: true,

			constructor: function() {
				this._untitledCount = 1;
				// just cache the hitched version to avoid creating so many closures.
				this._onDirtyItemCallbackHitched = dojo.hitch(this, this._onDirtyItemCallback);
			},

			postCreate: function() {
				this.inherited(arguments);

				this._loading.positionAbsolute('30px'); //$NON-NLS-1$

				this._valuesList._loadingMessage.set('placeholder', this.messages.noDimensionSelected); //$NON-NLS-1$
				this._valuesList.setListItemFactory(dimensionValueListItemFactoryFunction);
			},

			_onDirtyItemCallback: function(propName, oldDirty, newDirty) {
				this.set('dirty', newDirty); //$NON-NLS-1$

				// Update the values column to match the name of the renamed
				// dimension (if that's why we are dirty)
				this._updateValuesColumnHeaderForSelection();

				// Update the greater-than prompts after a rename or delete.
				// This is also called as soon as a new value is added.
				var dimensionItem = this._dimensionTree.getSelection()[0];
				if (dimensionItem && dimensionItem.model && dimensionItem.model.values){
					this._updateAllValueOrderingRows(dimensionItem.model.values);
				}
			},

			_showValues: function(dimensionItem, valueModels) {
				valueModels = sortByOrder(valueModels);

				var self = this;
				var dimensionValueRows = dojo.map(valueModels, function(dimensionValueModel) {
					// ownership is being handed to the result list. It will destroy these
					var row = new com.ibm.relm.ple.web.ui.branch.DimensionValueRow({
						model: dimensionValueModel,
						allValueModels: valueModels,
						shouldRemoveStatefulRow: dojo.hitch(self, self._onRemoveValueCallback)
					});

					if (dimensionValueModel.state === State.NEW && valueModels.length > 1) {
						this._addValueOrderingRow(row, valueModels, false);
					}

					row.stateful.watch('dirty', function(propName, oldDirty, newDirty) { //$NON-NLS-1$
						if (dimensionItem.model.state === State.OLD) {
							dimensionItem.model.state = State.MODIFIED_VALUES;
						}
						// dirty-ing the dimension will propagate to dirty this editor too.
						dimensionItem.stateful.set('dirty', true); //$NON-NLS-1$
					});

					return row;
				}, this);

				this._valuesList.showResults(dimensionValueRows);
			},

			_selectionChanged: function(sel) {
				this._valuesList.clearResults();

				var item = sel[0];
				if (!item) {
					this._valuesList._loadingMessage.set('placeholder', this.messages.noDimensionSelected); //$NON-NLS-1$

					this._addDimensionValueButton.set('enabled', false); //$NON-NLS-1$
					this._valuesHeader.innerHTML = this.messages.valuesColumn;
					return;
				}

				if (item === this._localFolder || item === this._sharedFolder) {
					this._valuesList._loadingMessage.set('placeholder', this.messages.noDimensionSelected); //$NON-NLS-1$

					this._addDimensionValueButton.set('enabled', false); //$NON-NLS-1$
					this._valuesHeader.innerHTML = this.messages.valuesColumn;

					return;
				}
				else {
					this._valuesList._loadingMessage.set('placeholder', this.messages.noValues); //$NON-NLS-1$

					this._addDimensionValueButton.set('enabled', true); //$NON-NLS-1$
					this._valuesHeader.innerHTML = dojo.replace(this.messages.valuesForXColumn, [html.escape(item.model.title)]);
					this._showValues(item, item.model.values);
				}
			},

			_updateValuesColumnHeaderForSelection: function() {
				var item = this._dimensionTree.getSelection()[0];

				if (!item || item === this._localFolder || item === this._sharedFolder) {
					this._valuesHeader.innerHTML = this.messages.valuesColumn;
				}
				else {
					this._valuesHeader.innerHTML = dojo.replace(this.messages.valuesForXColumn, [html.escape(item.model.title)]);
				}
			},


			/**
			 * Examines all the rows in the value list for new values.
			 * Any that needs a value-ordering (greater-than) prompt will
			 * have one added. Any that are already showing one will be updated
			 * with the current value names (assuming any were renamed or added).
			 * 
			 * @param  {array} allValueModels all value models for the
			 *     currently selected dimension.
			 */
			_updateAllValueOrderingRows: function(allValueModels) {
				if (allValueModels.length === 1) return;

				var valueRows = this._valuesList.getItems();

				for (var i = 0; i < valueRows.length; i++) {
					var valueRow = valueRows[i];
					var valueModel = valueRow.model;

					// Only prompt for precedence on new values.
					if (valueModel.state !== State.NEW) continue;

					if (valueRow.valueOrderingRow) {
						var optionModels = valueModelsToOptionModels(allValueModels, valueModel);
						valueRow.valueOrderingRow.updateOrderingOptions(null, optionModels);
					}
					else {
						this._addValueOrderingRow(valueRow, allValueModels, true);
					}
				}
			},

			/**
			 * Add the "more specific than" popover row. To collect the greater than value.
			 * Only used for new values, since this cannot be changed once sethis
			 * 
			 * @param {DimensionValueRow} valueRow                the row to
			 *     add the popover to.
			 * @param {array} allValueModels          all the models of the
			 *     other values so they can be listed as choices.
			 * @param {string} currentGreaterThanValue URI of the current
			 *     "greater than" value. This is used while a value is still
			 *     new, but you look at another dimension before saving, then
			 *     come back. The same value the user left it in should be
			 *     selected still.
			 * @param {booleam} animated                true to animate the
			 *     opening of the popover. Used when the user clicks "Add",
			 *     but is set to false when just redrawing the list.
			 */
			_addValueOrderingRow: function(valueRow, allValueModels, animated) {
				var valueOptions = [];

				allValueModels = sortByOrder(allValueModels);

				for (var i = 0; i < allValueModels.length; i++) {
					var valueModel = allValueModels[i];
					if (valueModel === valueRow.model) continue;

					valueOptions.push({
						name: valueModel.title,
						id: valueModel.uri || valueModel.title
					});
				}

				valueRow.valueOrderingRow = new com.ibm.relm.ple.web.ui.branch.ValueOrderingRow({
					value: valueRow.model.greaterThan,
					orderingOptions:valueOptions
				});

				valueRow.own(valueRow.valueOrderingRow);

				valueRow.connect(valueRow.valueOrderingRow, 'onChange', function(greaterThan) { //$NON-NLS-1$
					valueRow.model.greaterThan = greaterThan;
					// valueRow.stateful.set('dirty', true); //$NON-NLS-1$
				});

				dojo.addClass(valueRow.valueOrderingRow.domNode, 'ple-ValueOrderingRow-leftTail'); //$NON-NLS-1$
				dojo.addClass(valueRow.valueOrderingRow.orderingMessage, 'r-margin'); //$NON-NLS-1$
				
				dojo.place(valueRow.valueOrderingRow.domNode, valueRow.domNode, 'last'); //$NON-NLS-1$

				valueRow.valueOrderingRow.show(animated);
			},

			_addDimensionValue: function() {
				var dimensionItem = this._dimensionTree.getSelection()[0];
				if (!dimensionItem || !dimensionItem.model || !dimensionItem.model.values) return;

				var valueModel = {
					uri: null,
					title: dojo.replace(this.messages.untitledValueNumberX, [dimensionItem.untitledValueCount++]),
					state: State.NEW,
					greaterThan: null,
					originalState: State.NEW
				};

				// Update the dimension's data model with a new value model
				dimensionItem.model.values.push(valueModel);
				// Don't overwrite other state. Only show modified value if no other change
				if (dimensionItem.model.state === State.OLD) {
					dimensionItem.model.state = State.MODIFIED_VALUES;
				}

				// Update the UI from the new value.
				// Not sorted, just appended to the end
				var valueRow = new com.ibm.relm.ple.web.ui.branch.DimensionValueRow({
					model: valueModel,
					allValueModels: dimensionItem.model.values,
					shouldRemoveStatefulRow: dojo.hitch(this, this._onRemoveValueCallback)
				});
				valueRow.stateful.watch('dirty', this._onDirtyItemCallbackHitched); //$NON-NLS-1$

				this._valuesList.appendResult(valueRow);

				// This dimension is dirty since we just added a new value to
				// it. Only set the dirty state after adding the value row to
				// the DOM though, since the dirty callback will cause the
				// value list to be scanned for missing/ stale greater-then
				// prompts. If we do this too early, then the new row would be
				// missed.
				dimensionItem.stateful.set('dirty', true); //$NON-NLS-1$

				renameNextTick(valueRow);
			},

			_updateDimensionHeaders: function() {
				var sharedCount = this._sharedFolder.getChildren().length;
				
				if (this._localFolder) {
					var localCount = this._localFolder.getChildren().length;
					this._localFolder.setLabel(dojo.replace(this.messages.localCountXDimensions, [localCount]));

					this._sharedFolder.setLabel(dojo.replace(this.messages.sharedCountXDimensions, [sharedCount]));
				}
				else {
					this._sharedFolder.setLabel(dojo.replace(this.messages.allSharedCountXDimensions, [sharedCount]));
				}
			},

			getSelectedDimension: function() {
				var selectedItem = this._dimensionTree.getSelection()[0];
				return selectedItem ? selectedItem.model : null;
			},

			/**
			 * Sets a selected tree DimensionItem from a given dimension
			 * model. Will match the URI if there is one, otherwise, just
			 * matches the name (for new dimensions).
			 * @param {object} dimension dimension model.
			 */
			setSelectedDimension: function(dimension) {
				if (!dimension) {
					this._dimensionTree.setSelection(null);
					return;
				}

				// New dimensions don't have a URI to inspect. Since names
				// must be unique, this is safe, but still prefer URI.
				var identityProperty = dimension.uri ? 'uri' : 'title'; //$NON-NLS-1$ //$NON-NLS-2$

				var folders = [this._sharedFolder];
				if (this._localFolder) folders.push(this._localFolder);

				for (var i = 0; i < folders.length; i++) {
					var folder = folders[i];

					var dimensionItems = folder.getChildren();

					for (var j = 0; j < dimensionItems.length; j++) {
						var dimensionItem = dimensionItems[j];

						if (dimension[identityProperty] === dimensionItem.model[identityProperty]) {
							this._dimensionTree.setSelection(dimensionItem);
							break;
						}
					}
				}
			},

			_insertDimension: function(createAsShared, dimensionTitle, dimensionValue) {
				var valueModel = {
					uri: null,
					title: dimensionValue,
					state: State.NEW,
					originalState: State.NEW
				};

				var dimensionModel = {
					dirty: true,
					uri: null,
					title: dimensionTitle,
					values: [valueModel],
					state: State.NEW,
					originalState: State.NEW,
					shared: createAsShared
				};

				var item = new com.ibm.relm.ple.web.ui.branch.DimensionItem({
					model: dimensionModel,
					shouldRemoveStatefulRow: dojo.hitch(this, this._onRemoveDimensionCallback)
				});
				item.stateful.watch('dirty', this._onDirtyItemCallbackHitched); //$NON-NLS-1$

				var parent = createAsShared ? this._sharedFolder : this._localFolder;
				parent.addChild(item);
				this._updateDimensionHeaders();

				this._dimensionTree.setSelection(item);
				item.domNode.scrollIntoView(false);

				this.set('dirty', true); //$NON-NLS-1$
			},

			_addDimension: function() {
				var self = this;
				var d = new com.ibm.relm.ple.web.ui.branch.CreateDimensionDialog({
					sharedOnly: !this._localFolder,
					onDone: function(scope, dimension, value) {
						self._insertDimension(scope === 'shared', dimension, value); //$NON-NLS-1$
					}
				});
				d.startup();
				return;
			},

			_onRemoveValueCallback: function(statefulRow) {
				var valueModel = statefulRow.model;

				var dimensionItem = this._dimensionTree.getSelection()[0];
				if (!dimensionItem.model) return;

				// remove the old value
				var deadIndex = dojo.indexOf(dimensionItem.model.values, valueModel);
				dimensionItem.model.values.splice(deadIndex, 1);
				dimensionItem.stateful.set('dirty', true); //$NON-NLS-1$

				// redraw whole list
				this._showValues(dimensionItem, dimensionItem.model.values);

				this.set('dirty', true); //$NON-NLS-1$
			},

			/**
			 * Callback method called from StatefulRow when it should be destroyed.
			 * @param  {statefulRow} statefulRow the row to destroy.
			 */
			_onRemoveDimensionCallback: function(statefulRow) {
				var dimensionModel = statefulRow.model;

				// Should not need this, since it's going away. But in case debugger shows it
				dimensionModel.state = State.DELETED;

				var parentItem = dimensionModel.shared ? this._sharedFolder : this._localFolder;
				var dimensionItem = findChildForModel(parentItem, dimensionModel);

				this._selectionChanged([]);

				// this will .destroy the widget too
				parentItem.removeChild(dimensionItem);
				this._dimensionTree._focus = null;
				this._dimensionTree._selection = [];
				this._updateDimensionHeaders();

				this.set('dirty', true); //$NON-NLS-1$

			},

			showLoading: function() {
				if (this._dimensionTree) this._dimensionTree.domNode.style.display = 'none'; //$NON-NLS-1$
				this._loading.show();
			},

			/**
			 * Gets URIs of all dirty dimensions.
			 * Used when finding other affected products.
			 * @return {array} URIs of all the dirty (modified, not new) dimensions.
			 */
			getDirtyDimensionUris: function() {
				var dirtyUris = [];

				var dirty = this.getUpdatedCreatedDimensions();

				dojo.forEach(['local', 'shared'], function(scope) { //$NON-NLS-1$ //$NON-NLS-2$
					var dimensions = dirty[scope];
					for (var i = 0; i < dimensions.length; i++) {
						var dimension = dimensions[i];

						// Skip new dimensions... they have no URI
						// and can't be already used.
						// New dimensions have no URI
						if (dimension.state !== State.NEW &&
							dimension.uri) {
							dirtyUris.push(dimension.uri);
						}
					}
				});

				return dirtyUris;
			},

			/**
			 * Gets the models of the updated and newly created dimensions and
			 * their values.
			 * 
			 * @param  {boolean} validate true if check for invalid values,
			 *     which currently only means new values with new greater-than
			 *     set on them. If validation fails, throws an error with
			 *     details about the failure: .missingGreaterThan is the value
			 *     model that is missing a greaterThan property.
			 * @return {object}          object with .local and .shared
			 *     properties which are arrays of the dimensions.
			 */
			getUpdatedCreatedDimensions: function(validate) {

				function copyDimensionModel(dimension) {
					var copy = {};

					// Only copy properties relvant to saving
					copy.uri = dimension.uri;
					copy.title = dimension.title;

					// Only keep dirty enum values
					var dirtyValues = dojo.filter(dimension.values, shouldSaveValue);

					copy.values = dojo.map(dirtyValues, function(value) {

						if (validate) {
							if (value.state === State.NEW
								&& !value.greaterThan
								&& dirtyValues.length > 1) {
								var error = new Error('Missing precedence for new value'); //$NON-NLS-1$
								error.missingGreaterThan = value;
								throw error;
							}
						}

						// Defensive copy. Modify "unordered" on the fly to
						// save a null value to VVC. The underlying HTML
						// selects and models use "unordered" which is
						// distinct from "unassigned", but there is no
						// disinction like that in VVC. Don't modify the
						// original data (only what's going to the server)
						// since other JavaScript code still assumes that
						// 'unordered' means something.
						var valueCopy = dojo.mixin({}, value);
						if (valueCopy.greaterThan === 'unordered') { //$NON-NLS-1$
							valueCopy.greaterThan = null; //$NON-NLS-1$
						}
						return valueCopy;
					});

					return copy;
				}

				function copyDirtyDimensions(folderItem) {
					var dirtyModels = [];
					// local folder might be null if not loaded
					if (!folderItem) return [];

					var childItems = folderItem.getChildren();

					for (var i = 0; i < childItems.length; i++) {
						var item = childItems[i];
						var dimensionModel = item.model;

						if (shouldSaveDimension(dimensionModel)) {
							// make a copy of the model so we can mutate if we want without affecting real data.
							var modelCopy = copyDimensionModel(dimensionModel);
							dirtyModels.push(modelCopy);
						}
					}

					return dirtyModels;
				}

				var local = copyDirtyDimensions(this._localFolder);
				var shared = copyDirtyDimensions(this._sharedFolder);

				return {
					local: local,
					shared: shared
				};
			},

			/**
			 * Creates the tree widget as _dimensionTree. Will destroy the old one if needed
			 * and clear the values list. Can be used each time a new dimension
			 * is selected to ensure a clean slate to start with.
			 */
			_newDimensionTree: function() {
				this._selectionChanged([]);
				this._dimensionTree && this._dimensionTree.destroyRecursive();

				this._dimensionTree = new jazz.ui.Tree({
					indent: 0,
					style: 'display: none' //$NON-NLS-1$
				});
				dojo.addClass(this._dimensionTree.domNode, 'ple-hasHeaders'); //$NON-NLS-1$
				this._dimensionTree.placeAt(this._treeColumn, 'first'); //$NON-NLS-1$

				var superOnDblClick = this._dimensionTree.onDblClick;
				this._dimensionTree.onDblClick = function(item) {
					if (item.rename) {
						item.rename();
					}
					else {
						superOnDblClick.apply(this, arguments);
					}
				};

				// Hack around bug in tree where the focus can be kept on removed
				// children, which causes exceptions later when the focus node is accessed.
				var superRemoveChild = this._dimensionTree.removeChild;
				this._dimensionTree.removeChild = function(child, preserve) {
					if (child == this._focus) {
						this._focus = null;
						this._selection = [];
					}
					superRemoveChild.apply(this, arguments);
				};

				// destroyed by parent tree
				this._sharedFolder = new com.ibm.relm.ple.web.ui.branch.HeaderItem({
					label: '', //$NON-NLS-1$
					iconClass: 'ple-icon-dimensionShared' //$NON-NLS-1$
				});

				// destroyed by parent tree
				this._localFolder = new com.ibm.relm.ple.web.ui.branch.HeaderItem({
					label: '', //$NON-NLS-1$
					iconClass: 'ple-icon-dimensionProductLocal' //$NON-NLS-1$
				});

				// shared on top
				this._dimensionTree.addChild(this._sharedFolder);
				this._sharedFolder.setExpanded(true);
				this._dimensionTree.addChild(this._localFolder);
				this._localFolder.setExpanded(true);

				this.connect(this._dimensionTree, 'onSelectionChanged', this._selectionChanged); //$NON-NLS-1$
			},

			_setEnabledAttr: function(enabled) {
				this._set('enabled', enabled); //$NON-NLS-1$
				this._valuesList.set('enabled', enabled); //$NON-NLS-1$
				dojo.toggleClass(this.domNode, 'ple-DimensionEditor-disabled', !enabled); //$NON-NLS-1$

				this._addDimensionButton.set('enabled', enabled); //$NON-NLS-1$
				this._addDimensionValueButton.set('enabled', enabled); //$NON-NLS-1$
			},

			setDimensions: function(scopedDimensions) {
				this._newDimensionTree();

				this._loading.hide();

				var scopes = [
					{
						id: 'local', //$NON-NLS-1$
						folderItem: this._localFolder
					},
					{
						id: 'shared', //$NON-NLS-1$
						folderItem: this._sharedFolder
					}
				];

				for (var i = 0; i < scopes.length; i++) {
					var scope = scopes[i];

					// Clear out stale items first
					var folder = scope.folderItem;
					dojo.forEach(folder.getChildren(), function(childItem) {
						folder.removeChild(childItem, false);
					});

					var dimensions = scopedDimensions[scope.id];
					if (!dimensions) continue;

					dimensions.sort(titleComparator);

					for (var j = 0; j < dimensions.length; j++) {
						var dimensionModel = dimensions[j];

						var dimensionItem = new com.ibm.relm.ple.web.ui.branch.DimensionItem({
							model: dimensionModel,
							shouldRemoveStatefulRow: dojo.hitch(this, this._onRemoveDimensionCallback)
						});
						dimensionItem.stateful.watch('dirty', this._onDirtyItemCallbackHitched); //$NON-NLS-1$
						scope.folderItem.addChild(dimensionItem);
					}
				}

				if (this._localFolder.getChildren().length === 0 && !scopedDimensions.local) {
					this._dimensionTree.removeChild(this._localFolder, true);
					this._localFolder = null;
				}

				this._updateDimensionHeaders();
				this._addDimensionButton.set('enabled', true); //$NON-NLS-1$

				this._dimensionTree.domNode.style.display = ''; //$NON-NLS-1$

				this.set('dirty', false); //$NON-NLS-1$
			}
		}
	);

})();
