/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.DimensionValueRow'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionItemMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.branch.StatefulRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionItemMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	var html = jazz.util.html;
	var StatefulRow = com.ibm.relm.ple.web.ui.branch.StatefulRow;

	/**
	 * A wrapper around a StatefulRow that is the UI for editing a dimension value.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.DimensionValueRow', //$NON-NLS-1$
		[ jazz.ui.tree.AbstractItem, dijit._Widget ],
		{
			messages: messages,

			model: null,

			buildRendering: function() {
				this.domNode = dojo.create('li', { //$NON-NLS-1$
					className: 'ple-DimensionValueRow' //$NON-NLS-1$
				});

				this.stateful = new StatefulRow({
					model: this.model,
					shouldRemoveStatefulRow: this.shouldRemoveStatefulRow,
					labelFormatter: this._formatLabel.bind(this)

				});

				this.stateful.placeAt(this.domNode);
			},

			_formatLabel: function(model) {
				if (model.state !== StatefulRow.State.OLD) {
					return null;
				}

				if (!model.greaterThan) return null;

				for (var i = 0; i < this.allValueModels.length; i++) {
					var otherModel = this.allValueModels[i];
					if (otherModel.uri === model.greaterThan) {
						return dojo.replace(messages.dimensionValueXMoreSpecificThanValueX, [model.title, otherModel.title]);
					}
				}

				return null;
			},

			postCreate: function() {
				this.connect(this.domNode, 'ondblclick', function() { //$NON-NLS-1$
					this.stateful.rename();
				});
			},

			rename: function() {
				this.stateful.rename();
			},

			uninitialize: function() {
				this.stateful.destroyRecursive();
			}
		}
	);
})();
