/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.EditBranchDialog'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dijit.form.ComboBox'); //$NON-NLS-1$
dojo.require('dojo.data.ItemFileReadStore'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.BranchDialogMixin'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'EditBranchDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
var Site = com.ibm.relm.ple.web.ui.Site;

/**
 * Dialog to edit the dimensions of an existing branch.
 */
dojo.declare(
	"com.ibm.relm.ple.web.ui.branch.EditBranchDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch.BranchDialogMixin],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/EditBranchDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'EditBranchDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		focusFirstDimension: true,
		autoComplete: true,

		/**
		 *
		 * @param args
		 * 	Required fields:
		 * 		site
		 * 		servicePath
		 */
		constructor: function(args)
		{
			this._widgets = [];

			var site = args.site;
			this.isConfigVersion = !!site.configVersion;
			var resourceVersion = site.itemVersion || site.configVersion;

			if (this.isConfigVersion) //$NON-NLS-1$
			{
				this._helpHeader = dojo.replace(this._messages.helpHeaderForConfigurations, [jazz.util.html.escape(resourceVersion.name)]);
			}
			else
			{
				var branchName = resourceVersion.branchName;
				this._helpHeader = branchName ?
					dojo.replace(this._messages.helpHeaderBranchX, [jazz.util.html.escape(branchName)]) :
					this._messages.helpHeaderBranch;
			}


			this._oldBranchDimensions = resourceVersion.branchDimensions || [];

			//These property is used by BranchDialogMixin			
			this.uri = this._productVersionUri;
		},

		postCreate: function() {
			this.inherited(arguments);

			// Track all widgets that must be destroyed later.
			this._widgets = [];

			this._width = 600;
			
			var primaryTitle;
			var iconClass;

			if (this.isConfigVersion) {
				iconClass = 'ple-icon-dimension'; //$NON-NLS-1$
				primaryTitle = this._messages.dialogTitleForConfigurations;
			}
			else {
				iconClass = 'ple-icon-createBranch'; //$NON-NLS-1$
				primaryTitle = this._messages.dialogTitle;	
			}

			this._dialog = new jazz.ui.Dialog({
				primaryTitle: primaryTitle,
				width: this._width + 'px', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode,
				iconClass: iconClass
			});
		},

		/**
		 * Called when form is submitted (click of button or type 'Enter').
		 */
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var dimensions = null;

			try {
				dimensions = this._dimensionsToObject();
			}
			catch (error) {
				console.error(error);
				return;
			}

			this._updateBranch(dimensions);
		},

		_updateBranch: function(dimensionsObj) {
			this._messageBox.hideMessage();
			this._submitButton.disabled = true;
			this._cancelButton.disabled = true;

			var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.updatingBranch);
			this.setDisabledOnAllDimensionRows(true);

			var self = this;
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + this.servicePath,
				content: {
					site: Site.siteToJsonSite(this.site),
					dimensions: dojo.toJson(dimensionsObj)
				},
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,

				handle: function() {
					message.hide();
				},
				load: dojo.hitch(this, function(updatedBranchUri) {
//					this._closeDialog();

					// Different views respond in different ways. The tree will refresh. The focused product details will refresh too.
					// Everything does a full refresh. Maybe we should publish dimensions later so they can update in place.
//					dojo.publish('ple.branch_was_updated', [updatedBranchUri]); //$NON-NLS-1$
					
					TreeRefresh.refresh({
						selectPath: self.site.path,
						flashPath: false
					}).addBoth(dojo.hitch(self, function() {
						self._closeDialog();
					}));
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					this.setDisabledOnAllDimensionRows(false);
					this._monitorCreateButton();
					this._cancelButton.disabled = false;
					this._messageBox.showXhrError(ioArgs.xhr);
				})
			});
		},

		uninitialize: function() {
			this._widgets.forEach(function(w) { w.destroyRecursive(); });
		}
	});	// dojo.declare

})();
