/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.EditDimensionRow'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('dojo.store.Memory'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.ValueOrderingRow'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'BranchDialogMixinMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.StatusMessage');//$NON-NLS-1$

(function(){
	var html = jazz.util.html;

	function stretchDijitSelect(select) {
		dojo.query('.dijitButtonContents', select.domNode).addClass('r-stretch'); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.addClass(select.domNode, 'r-stretch'); //$NON-NLS-1$
	}

	function stretchDijitComboBox(comboBox) {
		dojo.addClass(comboBox.domNode, 'r-stretch'); //$NON-NLS-1$
	}

	 // Helpers used by ExistingDimensions class.
	function findById(inArray, value) {
		return findByProperty(inArray, 'id', value); //$NON-NLS-1$
	}

	function findByName(inArray, name) {
		return findByProperty(inArray, 'name', name); //$NON-NLS-1$
	}

	function findByProperty(inArray, propertyName, propertyValue) {
		for (var i = 0; i < inArray.length; i++) {
			var element = inArray[i];
			if (element[propertyName] === propertyValue) {
				return element;
			}
		}

		return null;
	}

	/**
	 * Helper class to more easily lookup dimensions and values.
	 * Relatively dumb, since it just iterates the arrays.
	 * This can be optimized later, but for most cases should be OK.
	 * @param {array} scopesDimensionsValues see EditDimensionRow properties for data format.
	 */
	function ExistingDimensions(scopesDimensionsValues) {
		this.scopesDimensionsValues = scopesDimensionsValues;
	}

	ExistingDimensions.prototype.getScopes = function(scope) {
		return this.scopesDimensionsValues;
	};

	ExistingDimensions.prototype.setDimensionsForScope = function(scopeId, dimensions) {
		var scopeData = findById(this.scopesDimensionsValues, scopeId);
		if (!scopeData) {
			//This happening would be programming error, so safe to non-nls
			console.error("No scope found with name:" + scopeId); //$NON-NLS-1$
		} else {
			scopeData.dimensions = dimensions;
		}
	};

	ExistingDimensions.prototype.getDimensionsForScope = function(scope) {
		var scopeData = findById(this.scopesDimensionsValues, scope);
		if (!scopeData) return [];

		return scopeData.dimensions || [];
	};

	ExistingDimensions.prototype.getValuesForDimensionId = function(scope, dimensionId) {
		var dimension = this.getDimensionForDimensionId(scope, dimensionId);
		
		return !dimension || !dimension.values
			? []
			: dimension.values;
	};

	ExistingDimensions.prototype.getDimensionForDimensionName = function(scope, dimensionName) {
		var dimensions = this.getDimensionsForScope(scope);
		return findByName(dimensions, dimensionName);
	};

	ExistingDimensions.prototype.getDimensionForDimensionId = function(scope, dimensionId) {
		var dimensions = this.getDimensionsForScope(scope);
		return findById(dimensions, dimensionId);
	};

	ExistingDimensions.prototype.getValuesForDimensionName = function(scope, dimensionName) {
		var dimension = this.getDimensionForDimensionName(scope, dimensionName);

		return !dimension || !dimension.values
			? []
			: dimension.values;
	};


	/**
	 * A row for a single dimension shown in create and edit dialog. A row can
	 * be set with initial values, but otherwise tracks its own values. Rows
	 * enforce known rules for dimensions related to configurations and
	 * product. For example, configurations cannot have product local
	 * dimensions.
	 *
	 * For gather value ordering information for new value a row can insert a
	 * new ValueOrderingRow below it. So while this widget is one row, and can
	 * result in two being added to a table. This widget cleans that up
	 * though, so the caller shouldn't care.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.EditDimensionRow', //$NON-NLS-1$
		[ com.ibm.relm.common.web.ui.table.HoverRow ],
		{
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'BranchDialogMixinMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			isConfigVersion: true,

			/**
			 * A JSON object of all known scopes, to dimensions, to value.
			 * This is used in place of a dojo store to track everything.
			 * It may be slightly slower, but is simple and predicatable.
			 *
			 * Required on construction.
			 *
			 * For one scope, one dimension and one value, it should look like
			 * the following. Note that any scopes can be used, and will end up
			 * being "shared" and either listed products or "product-local"
			 * that are known to have product-local dimensions:
			 * [
			 *		{
			 *			name: 'Shared', //$NON-NLS-1$
			 *			id: 'shared', //$NON-NLS-1$
			 *			dimensions: [
			 *				{
			 *					name: 'Geo', //$NON-NLS-1$
			 *					id: 'urn://geo', //$NON-NLS-1$
			 *					values: [
			 *						{
			 *							name: 'USA', //$NON-NLS-1$
			 *							id: 'urn://usa' //$NON-NLS-1$
			 *						}
			 *					]
			 *				}
			 *			]
			 *		}
			 *  ]
			 * @type {Array}
			 */
			scopesDimensionsValues: [],

			/** Initial scope ID if this is for an existing dimension. */
			initialScopeId: '', //$NON-NLS-1$

			/** Initial dimension ID or name if this is for an existing dimension.. FIXME: should always take ID */
			initialNameId: '', //$NON-NLS-1$

			/** Initial value ID if this is for an existing dimension. */
			initialValueId: '', //$NON-NLS-1$

			disabled: false,

			postMixInProperties: function() {
				// Wrap the incoming dimension data in a convencience class. This
				// is also an opportunity to optimize look ups eventually
				this.existingDimensions = new ExistingDimensions(this.scopesDimensionsValues);
			},

			postCreate: function() {
				this.inherited(arguments);

				this._initCells();
				this._initScope();
				this._refreshNameFromScope();

				// Delete button
				this.toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-delete', //$NON-NLS-1$
					title: this.messages.deleteButton,
					onClick: this._onDeleteClick.bind(this)
				}));
			},

			/**
			 * Toggles disabled for this row.
			 * When disabled, all the pick lists are disabled. Used
			 * during load and save.
			 * @param {boolean} disabled true to disable.
			 */
			_setDisabledAttr: function(disabled) {
				this._set('disabled', disabled); //$NON-NLS-1$

				if (disabled) {
					[this._scopeSelect, this._namePickList, this._valuePickList].forEach(function(list) {
						list.set('disabled', true); //$NON-NLS-1$
					});


					if (this._valueOrderingRow && this._newGreaterThan) {
						// we are showing the row, but a value is set, so just hide.
						this._hideValueOrderingRow();
					}
				}
				else {
					// value list is only condtionally enabled if the name is not invalid
					[this._scopeSelect, this._namePickList].forEach(function(list) {
						list.set('disabled', false); //$NON-NLS-1$
					});

					this._setValueDisabledIfNameInvalid();
				}
			},

			/**
			 * Focuses on the first list in this row.
			 */
			focus: function() {
				var lists = [this._namePickList, this._valuePickList, this._scopeSelect];
				
				for (var i = 0; i < lists.length; i++) {
					var list = lists[i];
					if (!list.get('disabled')) { //$NON-NLS-1$
						list.focus();
						return;
					}
				}
			},

			/**
			 * One time setup of all cells in this row.
			 */
			_initCells: function() {
				this._scopeCell = this.addCell(dojo.create('td')); //$NON-NLS-1$
				this._nameCell = this.addCell(dojo.create('td')); //$NON-NLS-1$
				this._valueCell = this.addCell(dojo.create('td')); //$NON-NLS-1$
			},

			_onDeleteClick: function(e) {
				this.destroyRecursive();
				this.didDelete(this);
			},

			/**
			 * Callback for when this row is deleted.
			 * @param {EditDimensionRow} this row.
			 */
			didDelete: function(deletedEditDimensionRow) {
				
			},

			/**
			 * Creates a new pick list style widget. This is can be a dijit
			 * Select, ComboBox, or FilteringSelect, depending on if this is
			 * editable or not. Does not add to the DOM, but does add some
			 * visually styling and associates the given data with it.
			 * 
			 * @param  {object} args 
			 * 
			 * 		args.destroyOldWidget - pass in an old pick list widget to
			 *      	have this method destroy it for you. This has the
			 *      	advantage that the focus can be preserved if that old
			 *      	widget had focus.
			 *      	
			 * 		args.data - array of objects with 'id' and 'name'
			 *      	properties to populate the list. Names are shown. IDs
			 *      	are used for values when possible. For ComboBoxes, the
			 *      	names come back as the values due to limiations with
			 *      	the dijit.ComboBox.
			 *
			 * 		args.isReadOnly - if the values can be selected in the list,
			 * 			but not changed.
			 *
			 * 		args.placeHolder - placeholder text when there is no
			 * 			selection.
			 *
			 *		args.initialValueId - the optional initial value to select
			 *			in the new list.
			 * 		
			 * @return {[type]}      [description]
			 */
			_newPickListWidget: function(args) {
				// if there is an existing widget to destroy and replace, see
				// if it has focus now. If so, set focus on the new one.
				var needsFocus = false;

				if (args.destroyOldWidget) {
					needsFocus = args.destroyOldWidget.get('focused'); //$NON-NLS-1$
					args.destroyOldWidget.destroyRecursive();
				}

				// Practically a no-op store, but using this makes Select and
				// ComboBox work similarly. 
				// TODO: Share store across widgets. 
				var store = new dojo.store.Memory({
					idProperty: 'id', //$NON-NLS-1$
					data: args.data
				});

				var pickListArgs = {
					maxHeight: 300,
					invalidMessage: this.messages.enteredInvalidValue,
					labelAttr: 'name', //$NON-NLS-1$
					searchAttr: 'name', //$NON-NLS-1$
					placeHolder: args.placeHolder,
					store: store,
					searchDelay: 0,
					autoComplete: 'off', //$NON-NLS-1$
					disabled: this.disabled
				};

				// If value is `undefined` then dijit.form.Select explodes, so
				// only set the property if we have one.
				if (args.initialValueId) {
					// ComboBoxes only know names, not IDs, so swap
					if (!args.isReadOnly) {
						var valueStruct = findById(args.data, args.initialValueId);
						if (valueStruct) pickListArgs.value = valueStruct.name;
					}
					else {
						pickListArgs.value = args.initialValueId;
					}
				} else if (args.initialValue) {
					//This is used if we want to prepopulate the picklist with a value before the store is loaded.
					args.isReadOnly = false;
					pickListArgs.disabled = true;
					pickListArgs.value = args.initialValue;
				}

				var PickList = args.isReadOnly ? dijit.form.FilteringSelect : dijit.form.ComboBox;
				var pickList = new PickList(pickListArgs);

				pickList.startup();

				if (args.isReadOnly) {
					stretchDijitSelect(pickList);
				}
				else {
					stretchDijitComboBox(pickList);	
				}

				if (needsFocus) {
					// Not added to DOM yet, so delay until next tick. Feels like a hack.
					setTimeout(function() {
						pickList.focus();
					}, 0);
				}

				return pickList;
			},

			/**
			 * Returns is this row is allowed to create new dimensions
			 * or values. This is _not_ true for product-local
			 * scoped rows on configurations. In this case, existing values
			 * can be chosen, but new ones cannot be created. The pick list
			 * widgets change to reflect this.
			 *
			 * Since this is tied to scope, this can change over time.
			 * 
			 * @return {Boolean} is new dimensions and values can be created
			 * on this rows.
			 */
			isCreationAllowed: function() {
				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$
				return !this.isConfigVersion || scope === 'shared'; //$NON-NLS-1$
			},

			/**
			 * Creates or recreates the dimesnion name pick list base on the
			 * current scope.
			 */
			_refreshNameFromScope: function() {
				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$
				var dimensions = this.existingDimensions.getDimensionsForScope(scope);
				// Don't try to load dimensions from server for 
				// 	1. A product local scope in the create product dialog (we know because there is no resource uri and scope is "local")
				// 	2. If the dimensions are already loaded
				if ((!this.uri && scope == "local") || dimensions && dimensions.length) {  //$NON-NLS-1$
					console.debug("Dimensions already loaded for scope:" + scope); //$NON-NLS-1$ //$NON-NLS-2$
					this._performRefreshName(dimensions);
				} else {
					console.debug("Loading dimensions and values for scope:", scope); //$NON-NLS-1$
					var initialNameId = this.initialNameId;
					var initialValueId = this.initialValueId;
					
					delete this.initialValueId;
					delete this.initialNameId;
					this._performRefreshName([], this.initialName, this.initialValue);
					delete this.initialName;
					delete this.initialValue;

					var statusMessage = com.ibm.relm.common.web.StatusMessage.show(this.messages.loadingDimensions);
					var startTime = Date.now();
					jazz.client.xhrGet({ //$NON-NLS-1$
						url: net.jazz.ajax._contextRoot + '/ple/branch', //$NON-NLS-1$
						content: {
							scope: scope,
							method: "loadDimension" //$NON-NLS-1$
						},
						handleAs: 'json', //$NON-NLS-1$
						load: function(dimensions) {
							statusMessage.hide();
							dojo.publish("ple.dimension_data_loaded", [scope, dimensions || []]); //$NON-NLS-1$

							this.initialNameId = initialNameId;
							this.initialValueId = initialValueId;

							this._performRefreshName(dimensions);
						}.bind(this),
						error: function(error, ioArgs) {
							statusMessage.hide();
							this.messageBox && this.messageBox.showXhrError(ioArgs.xhr);	
						}.bind(this)
					});
				}
			},

			/**
			 * Should only be called from _refreshNameFromScope after either loading dimensions or if
			 * dimensions are already loaded. 
			 * 	
			 * @param  {Array} dimensions Dimensions and values specific to the selected scope
			 */
			_performRefreshName: function(dimensions, initialName, initialValue) {
				var isNameReadOnly = !this.isCreationAllowed();

				this._namePickList = this._newPickListWidget({
					destroyOldWidget: this._namePickList,
					data: dimensions,
					placeHolder:this.messages.dimensionNamePlaceholder,
					initialValueId: this.initialNameId,
					initialValue: initialValue,
					isReadOnly: isNameReadOnly
				});
				
				// One time use
				delete this.initialNameId;

				dojo.disconnect(this._nameChangeHandle);
				this._nameChangeHandle = dojo.connect(this._namePickList, 'onChange', this, this._onNameChanged); //$NON-NLS-1$
				this.own(this._nameChangeHandle);

				this._namePickList.placeAt(this._nameCell);

				//If name was refreshed, then value needs to be refreshed too
				this._refreshValueFromName(initialValue);
			},

			/**
			 * If an invalid product-local dimension is selected for a
			 * configuration, then the row is invalid. A valid name must be
			 * selected. While an invalid name is shown, the value field is
			 * disabled.
			 *
			 * If the name field is disabled, then the name field is also disabled.
			 * 
			 * If the name is good, then the value is renabled when calling
			 * this.
			 */
			_setValueDisabledIfNameInvalid: function() {
				var disabled = false;
				
				if (this._namePickList.get('disabled')) { //$NON-NLS-1$
					disabled = true;
				} else if (this._namePickList.isValid && !this._namePickList.isValid()) {
					disabled = this._namePickList.isValid && !this._namePickList.isValid();
				}

				this._valuePickList.set('disabled', disabled); //$NON-NLS-1$
			},

			/**
			 * Creates or recreates the dimesnion value pick list base on the
			 * current dimension (name).
			 */
			_refreshValueFromName: function(initialValue) {
				var needsFocus = false;

				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$
				var isNameAndValueReadOnly = !this.isCreationAllowed();

				// If this is a Select (read-only) then this is the ID, if it's ComboBox
				// it's the name (since the user can type anything).
				var dimensionNameOrId = this._namePickList.get('value'); //$NON-NLS-1$

				// If the name is read-only, then IDs are returned. But ComboBox
				// only returns names since the user can type anything.
				var values = isNameAndValueReadOnly
					? this.existingDimensions.getValuesForDimensionId(scope, dimensionNameOrId)
					: this.existingDimensions.getValuesForDimensionName(scope, dimensionNameOrId);
				
				this._valuePickList = this._newPickListWidget({
					destroyOldWidget: this._valuePickList,
					data: values,
					placeHolder:this.messages.dimensionValuePlaceholder,
					initialValueId: this.initialValueId,
					initialValue: initialValue,
					isReadOnly: isNameAndValueReadOnly
				});

				this._setValueDisabledIfNameInvalid();
				
				// One time use
				delete this.initialValueId;

				dojo.disconnect(this._valueChangeHandle);
				this._valueChangeHandle = dojo.connect(this._valuePickList, 'onChange', this, this._onValueChanged); //$NON-NLS-1$
				this.own(this._valueChangeHandle);

				this._valuePickList.placeAt(this._valueCell);

				//If value was refreshed, then value ordering row needs to be hidden
				this._hideValueOrderingRow();
			},

			/**
			 * Creates the scope pick like. Only call once.
			 */
			_initScope: function() {
				var scopes = this.scopesDimensionsValues;

				var options = dojo.map(scopes, function(scope) {
					var iconClass = scope.id === 'shared' //$NON-NLS-1$
						? 'ple-icon-dimensionShared ' //$NON-NLS-1$
						: 'ple-icon-dimensionProductLocal'; //$NON-NLS-1$

					return {
						label: '<img src="' + this._blankGif + '" class="' + iconClass + '"> ' + html.escape(scope.name), //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						value: scope.id
					};
				}, this);

				this._scopeSelect = new dijit.form.Select({
					maxHeight: 300,
					options: options,
					value: this.initialScopeId
				});

				dojo.attr(this._scopeSelect.domNode, 'aria-labelledBy', this.scopeId); //$NON-NLS-1$

				this._scopeSelect.startup();
				stretchDijitSelect(this._scopeSelect);
				this.connect(this._scopeSelect, 'onChange', this._onScopeChanged); //$NON-NLS-1$

				this._scopeSelect.placeAt(this._scopeCell);
			},

			/**
			 * Checks if this row is using a new dimension value that requires
			 * setting a "greaterThan" for.
			 * @return {object} null if the value is existing or the first one.
			 * Otherwise, an object with the .newValueName, and list of existing .values.
			 */
			getNewDimensionValueInfo: function() {
				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$

				if (this.isConfigVersion && scope === 'local') { //$NON-NLS-1$
					return null;
				}

				var dimensionName = this._namePickList.get('value'); //$NON-NLS-1$
				var currentValueName = this._valuePickList.get('value'); //$NON-NLS-1$
				var values = this.existingDimensions.getValuesForDimensionName(scope, dimensionName);

				if (!values.length) return null;

				for (var i = 0; i < values.length; i++) {
					var value = values[i];
					if (value.name === currentValueName) return null;
				}

				return {
					existingValues: values,
					newValueName: currentValueName
				};
			},

			/**
			 * Shows the prompt for value ordering if not already showing.
			 * 
			 * @param  {string} newValueName   the new value that needs an order.
			 * @param  {array} existingValues all the old values this can come after.
			 */
			_showValueOrderRow: function(newValueName, existingValues) {
				if (!newValueName) return;

				if (this._valueOrderingRow) {
					this._valueOrderingRow.updateOrderingOptions(newValueName, existingValues);
					return;
				}

				this._valueOrderingRow = new com.ibm.relm.ple.web.ui.branch.ValueOrderingRow({
					orderingOptions: existingValues,
					valueName: newValueName 
				});
				this.own(this._valueOrderingRow);

				this._valueOrderingRow.placeAt(this.domNode, 'after'); //$NON-NLS-1$
				this._valueOrderingRow.show(true);

				dojo.disconnect(this._valueOrderChangedHandle);
				this._valueOrderChangedHandle = dojo.connect(this._valueOrderingRow, 'onChange', this, this._onValueOrderChanged); //$NON-NLS-1$
				this.own(this._valueOrderChangedHandle);
				dojo.addClass(this.domNode, 'ple-HoverRow-noBottom'); //$NON-NLS-1$
			},

			/**
			 * Hides the prmompt for value ordering and clears the current
			 * _newGreaterThan (order) value (assumed to not be needed if this
			 * is closed).
			 */
			_hideValueOrderingRow: function() {
				if (!this._valueOrderingRow) return;

				delete this._newGreaterThan;

				dojo.disconnect(this._valueOrderChangedHandle);
				delete this._valueOrderChangedHandle;

				var andDestroy = true;
				this._valueOrderingRow.hide(andDestroy);
				delete this._valueOrderingRow;

				dojo.removeClass(this.domNode, 'ple-HoverRow-noBottom'); //$NON-NLS-1$
			},

			_onValueOrderChanged: function(greaterThan) {
				this._newGreaterThan = greaterThan;

				this.onChange();
			},

			/**
			 * If an ordering value is required for this row, will show the
			 * slide out prompt for one. Otherwise, hide the prompt it no
			 * longer needed or just do nothing.
			 * 
			 * @return {boolean} true if the prompt was shown and a value was
			 *     required. Otherwise, false.
			 */
			_showOrderValueRowIfNewValue: function() {
				var info = this.getNewDimensionValueInfo();
				
				if (info) {
					this._showValueOrderRow(info.newValueName, info.existingValues);
					return true;
				}
				else {
					this._hideValueOrderingRow();
					return false;
				}
			},

			_onValueChanged: function(newValue) {
				this._showOrderValueRowIfNewValue();

				this.onChange();
			},

			_onNameChanged: function(newName) {
				this._refreshValueFromName();

				this.onChange();
			},

			_onScopeChanged: function(newScope) {
				this._refreshNameFromScope();

				this.onChange();
			},

			/**
			 * Callback if anything in this row changes.
			 */
			onChange: function() {
				// callback
			},

			/**
			 * Gets all filled in values in a single result object.
			 * Can only be called when this row isValid.
			 * The keys and values match the existing Java servlet
			 * and JavaScript code expectations.
			 * 
			 * @return {object} all filled in values. null if this row should be ignored,
			 * e.g., the first blank row or any row with not dimension or value.
			 * This will be treated as a delete.
			 */
			getResult: function() {
				if (!this.isValid(true)) throw new Error('Cannot get dimension data for row. Is invalid.'); //$NON-NLS-1$

				var scope = this._scopeSelect.get('value'); //$NON-NLS-1$
				var dimensionName;
				var dimensionUri;
				var valueName;
				var valueUri;
				var greaterThan;

				var canCreate = this.isCreationAllowed();
				
				var dimensionNameOrId = dojo.trim(this._namePickList.get('value')); //$NON-NLS-1$
				var valueNameOrId = dojo.trim(this._valuePickList.get('value')); //$NON-NLS-1$

				// this is a blank row, which is like a delete.
				if (!dimensionNameOrId && !valueNameOrId) {
					console.debug('Empty dimension row; ignore'); //$NON-NLS-1$
					return null;
				}

				if (canCreate) {
					// ComboBoxes only know the display names of their content,
					// not IDs or URIs, so we need to look up the existing
					// dims and values to get their URIs.
					dimensionName = dimensionNameOrId;
					var dimension = this.existingDimensions.getDimensionForDimensionName(scope, dimensionName);
					valueName = valueNameOrId;

					if (dimension) {
						dimensionUri = dimension.id; //$NON-NLS-1$
						var valueModel = findByName(dimension.values, valueName);
						if (valueModel) valueUri = valueModel.id;
					}
				}
				else {
					dimensionUri = dimensionNameOrId;
					dimension = this.existingDimensions.getDimensionForDimensionId(scope, dimensionUri);
					dimensionName = this.existingDimensions.getDimensionForDimensionId(scope, dimensionUri).name;

					valueUri = valueNameOrId;
					valueName = findById(dimension.values, valueUri).name;
				}

				// These key names match what the existing dialogs and
				// servlets expect.
				return {
					name: dimensionName, 
					value: valueName,
					shared: scope === 'shared' ? 'shared' : 'local', //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					uri: dimensionUri,
					valueUri: valueUri,
					valueOrdering: (this._newGreaterThan === "unordered") ? null : this._newGreaterThan //$NON-NLS-1$ //$NON-NLS-2$
				};
			},

			/**
			 * Checks if all required info for this row is entered and valid.
			 * If not, calls to .getResult will fail.
			 *
			 * @param {boolean} canShowOrderValueRow if true, calling this
			 *     function may result in the ordering prompt being show (and
			 *     this returning false). Otherwise, it's current state is
 			 *     checked, but no prompt is shown. We don't want to show the prompt when, say,
 			 *     checking just whether the save button should be enabled, but should before allowing
 			 *     the user to dismiss a dialog that requires an ordering.
 			 *     
 			 *      
			 * @return {Boolean} if data is filled in and can be read.
			 */
			isValid: function(canShowOrderValueRow) {
				// If there is an ordering prompt a greater than value must be set.
				if (this._valueOrderingRow) {
					if (!this._newGreaterThan) return false;
				}
				else if (canShowOrderValueRow) {
					if (this._showOrderValueRowIfNewValue()) return false;
				}

				// If either pick list know's it's invalid (FilterSelects do this) then fail
				if (!this._namePickList.isValid() || !this._valuePickList.isValid()) return false;

				var canCreate = this.isCreationAllowed();

				// If we can't create (and the lists are invalid) we're guaranteed to have valid choices.
				if (!canCreate) return true;

				// Check for any non-empty name and value. These can be for new or old dimensions
				var name = dojo.trim(this._namePickList.get('value')); //$NON-NLS-1$
				var value = dojo.trim(this._valuePickList.get('value')); //$NON-NLS-1$


				// valid to have neither, since this is the initial, blank row
				// or any row where both were deleted and the dimensions should be deleted...
				if (!name && !value) return true;

				// ...but if just one is missing, then fail.
				if (!name) return false;
				if (!value) return false;

				return true;
			}
		}
	);

})();