/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch.StatefulRow'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionItemMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.prop', 'PropertyNameMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.tree.AbstractItem'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin'); //$NON-NLS-1$

(function(){

	var html = jazz.util.html;

	/**
	 * Tracked on the model objects so we know if it's dirty
	 * and if it's ever been saved or not. Used to decide if the data
	 * will be saved and how to show it's label.
	 */
	var State = {
		OLD: 0,
		NEW: 1,
		DELETED: 2,
		MODIFIED: 3,

		// this is Dimension specific, since values don't have sub-values to modify
		MODIFIED_VALUES: 4
	};

	/**
	 * A generic UI for a data model that is renamable and undoable, plus
	 * some other hooks. Dimensions and dimension-values are almost the same,
	 * so they both use this to draw a UI with a name and toolbar for them.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.StatefulRow', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/StatefulRow.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.mixin(
				dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'DimensionItemMessages'), //$NON-NLS-1$ //$NON-NLS-2$
				dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.prop', 'PropertyNameMessages')), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * A format function that can optionally by used to override the formatting of the label.
			 * This is used when showing dimension values to force the "more specific than" decorator into
			 * the label. The function should return valid HTML to be used as the innerHTML for the label.
			 * It should return `null` if it the default label formatting should be used.
			 * @type {function}
			 */
			labelFormatter: null,

			forceWatchCallbacksFor: {dirty: true},
			
			/**
			 * Currently only used if the *name* is dirty,
			 * since that can be reverted.
			 * Setting this changes the font color and adds
			 * a revert button.
			 * @type {Boolean}
			 */
			dirty: false,

			model: null,

			imgClass: '', //$NON-NLS-1$

			/**
			 * Callback used to notify parent widget that this row should be removed.
			 */
			shouldRemoveStatefulRow: null,

			undo: function() {
				this.model.state = State.OLD;
				this.model.title = this.model.originalTitle || this.model.title;
				this.set('dirty', false); //$NON-NLS-1$
				this._updateDisplay();
			},

			onDelete: function(e) {
				dojo.stopEvent(e);

				var oldState = this.model.state;
				if (oldState === State.NEW) {
					// Since this was NEW, we don't want to update the state,
					// but actually delete it for good. Let the delegate
					// controller know to do this for us (since it needs to)
					// be taken out of its parent container widget.

					this.shouldRemoveStatefulRow(this);
				}
				else {
					this.model.state = State.DELETED;
					this.set('dirty', true); //$NON-NLS-1$
					this._updateDisplay();
				}
			},

			rename: function() {
				if (this.model.state === State.DELETED) return;
				this._renameField.value = this.model.title;
				this._toggleRename(true);
				this._renameField.select();
			},

			_toggleRename: function(show) {
				this._renameField.style.display = show ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
				this._labelNode.style.display = show ? 'none' : ''; //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			 * This only stops propagation so the jazz.ui.Tree won't catch the
			 * key presses and change focus.
			 */
			_cancelKeyPress: function(e) {
				if (e.stopPropagation) e.stopPropagation();
				if (window.event) window.event.cancelBubble = true;
			},

			_updateDisplay: function() {
				var state = this.model.state;
				
				var isDeleted = state === State.DELETED;
				this._renameButton.set('enabled', !isDeleted); //$NON-NLS-1$

				var showUndo = this.dirty && state !== State.NEW && state !== State.MODIFIED_VALUES;
				this._undoButton.domNode.style.display = showUndo ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$

				var showDelete = state === State.NEW;
				this._deleteButton.domNode.style.display = showDelete ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$

				this._updateLabel();
			},

			_updateLabel: function() {
				var newLabel = this.labelFormatter && this.labelFormatter(this.model);

				if (newLabel) {
					this._labelNode.innerHTML = newLabel;
					return;
				}
				
				switch (this.model.state) {
				case State.NEW:
					newLabel = dojo.replace(
						this.messages.nameXToAdd, [html.escape(this.model.title)]);
					break;

				case State.MODIFIED:
					newLabel = dojo.replace(
						this.messages.nameXWasX,
						[html.escape(this.model.title), html.escape(this.model.originalTitle)]);
					break;

				case State.DELETED:
					newLabel = dojo.replace(
						this.messages.nameXToDelete, [html.escape(this.model.title)]);
					break;

				case State.MODIFIED_VALUES:
					newLabel = dojo.replace(
						this.messages.nameXModifiedValues, [html.escape(this.model.title)]);
					break;

				default:
					if (this.defaultHtmlTitle) {
						newLabel = this.defaultHtmlTitle;
					}
					else {
						newLabel = html.escape(this.model.title);
					}
					break;
				}

				this._labelNode.innerHTML = newLabel;
				
			},

			_commitRename: function() {
				var newName = dojo.trim(this._renameField.value);
				this._toggleRename(false);
				
				if (newName.length === 0) {
					// don't set a new name to empty string. Will thrown an error in the API.
					return;
				}

				this.model.title = newName;

				var oldState = this.model.state;

				if ((oldState === State.OLD || oldState === State.MODIFIED_VALUES) && this.model.originalTitle !== newName) {
					// MODIFIED is "stronger" than MODIFIED_VALUES and  OLD
					this.model.state = State.MODIFIED;
				}
				else if(this.model.originalTitle === newName){
					this.model.title = this.model.originalTitle;
					if(oldState !==State.MODIFIED_VALUES){
						this.model.state = State.OLD;
						this.set('dirty', false); //$NON-NLS-1$
					}
					this._updateDisplay();
				}

				if (this.model.state === State.MODIFIED || this.model.state === State.NEW) {
					this.set('dirty', true); //$NON-NLS-1$
				}

				this._updateDisplay();
			},

			_onRenameKeyUp: function(e) {
				var c = e.keyCode;

				switch (c) {
				case dojo.keys.ENTER:
					this._commitRename();
					return;

				case dojo.keys.ESCAPE:
					dojo.stopEvent(e);
					this._renameField.value = this.model.title;
					this._toggleRename(false);

					// Hack. Setting the value while focused on this field
					// seems to have no affect. Printing it to console shows
					// the new value, but visually it never changes and is not
					// reliable later. This is not the usually behavior of an
					// INPUT, so there must be some bug here, but this works
					// around for now.
					var self = this;
					setTimeout(function() {
						self._renameField.value = self.model.title;
					}, 0);
					return;

				default:
					break;
				}
			},

			_setDirtyAttr: function(dirty) {
				this._set('dirty', dirty); //$NON-NLS-1$
				this.model.dirty = dirty;
				dojo.toggleClass(this.domNode, 'j-dirty', dirty); //$NON-NLS-1$
				this._updateLabel();
			},

			_onToolbarFocus: function() {
				dojo.addClass(this.domNode, 'ple-StatefulRow-focused'); //$NON-NLS-1$
			},

			_onToolbarBlur: function() {
				dojo.removeClass(this.domNode, 'ple-StatefulRow-focused'); //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);

				this.toolbar.startup();

				if (this.model.state !== State.OLD) {
					this.set('dirty', true); //$NON-NLS-1$
				}

				if (this.model.isExtra) {
					dojo.addClass(this._labelNode, 'ple-StatefulRow-isExtra'); //$NON-NLS-1$
				}

				this._updateDisplay();
			},

			uninitialize: function() {
				// probably not needed, but ensures no cycles
				delete this.shouldRemoveStatefulRow;
			}
		}
	);

	com.ibm.relm.ple.web.ui.branch.StatefulRow.State = State;
})();