/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.branch.UseBranchDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.branch', 'UseBranchDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.CreateBranchDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.ActionPreview'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

	var formatting = com.ibm.relm.ple.web.ui.formatting;
	
	/**
	 * Dialog to select which branch to use for a product.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch.UseBranchDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/branch/templates/UseBranchDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.branch', 'UseBranchDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * When not enabled, this list will appear grayed out and will not
			 * respond to most user action (keyboard nav, clicks etc.). This
			 * is used when a "submit" is in progress and we don't want the
			 * user to interact with the list, specifically we don't want them
			 * triggering two submits.
			 * @type {Boolean}
			 */
			enabled: true,

			/**
			 * 
			 * @param args - selected Site
			 * @param node Optional node to replace.
			 */
			constructor: function(args, node)
			{
				var escapedNames = {
					productName: jazz.util.html.escape(args.site.itemVersion.name),
					parentName: jazz.util.html.escape(args.site.parentResourceVersion.name)
				};
				
				this._headerMain = dojo.replace(this._messages.headerMainBranchXParentY,
					[escapedNames.productName, escapedNames.parentName]);
			},

			postCreate: function() {
				this.inherited(arguments);
				
				var leftItem = {
						'displayName': formatting.formatProductIconAndTitleForItem(this.site.itemVersion), //$NON-NLS-1$
						'numChildren': 0, //this.site.itemVersion.children.length //$NON-NLS-1$
						'parentDisplayName': formatting.formatProductIconAndTitleForItem(this.site.parentResourceVersion) //$NON-NLS-1$
				};
				
				this._actionPreview = new com.ibm.relm.common.web.ui.ActionPreview({ 'leftItem': leftItem }, this._header._actionPreviewDiv); //$NON-NLS-1$
				
				this._loadBranches();
				
				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this._messages.dialogTitle,
					width: '700px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-replaceBranch' //$NON-NLS-1$
				});
			},
			
			_loadBranches: function() {
				var self = this;

				jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/replace_branch', //$NON-NLS-1$
					content: {
						itemVersionUri: self.site.itemVersion.uri
					},
					
					handle: function(response, ioArgs) {
						self._loadingMessage.hide();
					},

					load: function(branchTips) {
						if (branchTips.length > 1) {
							dojo.addClass(self._tableContainer, 'ple-scrollingTable-containerBorder'); //$NON-NLS-1$
							self._drawTable(branchTips);
						}
						else { // no replacement branches available (tip was from initiating branch) - prompt to create branch
							self._messageBox.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
								summary: dojo.replace(self._messages.noBranchesForX, [jazz.util.html.escape(self.site.itemVersion.name)])
							});
							var a = dojo.query('.ple-ReplaceBranchDialog-create-branch-link', self._messageBox.domNode)[0]; //$NON-NLS-1$
							self.connect(a, 'onclick', self._createBranch); //$NON-NLS-1$
						}
					},

					error: function(_, args) {
						self._messageBox.showXhrError(args.xhr);
					}
				});

			},
			
			_createBranch: function(e) {
				dojo.stopEvent(e);
				this._closeDialog();
				
				var d = new com.ibm.relm.ple.web.ui.branch.CreateBranchDialog(this.site);
				d.startup();
			},
			
			_drawTable: function(branchTips) {
				var frag = document.createDocumentFragment();

				for (var i = 0; i < branchTips.length; i++) {
					var branchTip = branchTips[i];
					var tr = dojo.create('tr'); //$NON-NLS-1$
					tr.uri = branchTip.branchUri;

					// Branch
					dojo.create('td', { //$NON-NLS-1$
						innerHTML: formatting.formatBranchTitle(branchTip)
					}, tr);

					// Tip Product
					dojo.create('td', { //$NON-NLS-1$
						innerHTML: formatting.formatProductIconAndTitleForItem(branchTip)
					}, tr);
					
					// Created
					var createdMillis = branchTip['createdMillis']; //$NON-NLS-1$
					var createdDate = new Date(createdMillis);
					var formattedDate = dojo.date.locale.format(
						createdDate,
						{ formatLength: 'medium' } //$NON-NLS-1$
					);

					dojo.create('td', { //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(formattedDate)
					}, tr);

					var creator = branchTip.creatorName;

					dojo.create('td', { //$NON-NLS-1$
						innerHTML: formatting.formatCreator(creator)
					}, tr);

					this.connect(tr, 'onclick', dojo.hitch(this, this.selectRow, tr)); //$NON-NLS-1$
					this.connect(tr, 'ondblclick', dojo.hitch(this, this._onRowDblClick, tr)); //$NON-NLS-1$
					frag.appendChild(tr);
				}
				
				var columns = [
					{label: this._messages.branchColumnLabel, sortable: false},
					{label: this._messages.productColumnLabel, sortable: false},
					{label: this._messages.creationDateColumnLabel, sortable: false},
					{label: this._messages.creatorColumnLabel, sortable: false}
				];
				
				this._styledTable = new jazz.ui.StyledTable({columns: columns}, this._table);
				this._styledTable.tbody.appendChild(frag);
			},
			
			_onRowDblClick: function(row) {
				if (!this.enabled) return;

				this.selectRow(row);
				this._submit();
			},
			
			selectRow: function(row) {
				if (!this.enabled) return;

				// de-select the row that is already selected (if there is one)
				if (this.selectedRow) {
					dojo.removeClass(this.selectedRow, 'ple-ReplaceBranchDialog-selectedRow'); //$NON-NLS-1$
				}
				
				this.selectedRow = row;
				this.selectedBranchUri = row.uri;
				dojo.addClass(row, 'ple-ReplaceBranchDialog-selectedRow'); //$NON-NLS-1$
				this._submitButton.disabled = false;
				
				//TODO!: maaaaybe just don't show the parent or children of the right side, its ALWAYS duplicate right?
				var rightItem = {
					'displayName': dojo.query('td', row)[1].innerHTML, //Product is in the second column //$NON-NLS-1$ //$NON-NLS-2$
					'numChildren': 0, //this.site.itemVersion.children.length //$NON-NLS-1$
					'parentDisplayName': formatting.formatProductIconAndTitleForItem(this.site.parentResourceVersion) //$NON-NLS-1$
				};
				
				this._actionPreview.setRightItem(rightItem);
			},
			
			startup: function() {
				this._tableContainer.focus();
			},

			/**
			* Toggles the enabled property of this dialog. This is used when the action is being submitted so
			* nothing new is selected or a second submission made.
			*/
			toggleEnabled: function(enabled) {
				if (enabled !== undefined)
					this.set('enabled', enabled); //$NON-NLS-1$
				else
					this.set('enabled', !this.enabled); //$NON-NLS-1$
			},

			/**
			* Adjusts the state of the widget as the enabled attribute is set
			*/
			_setEnabledAttr: function(enabled) {
				this.enabled = enabled;
				this._submitButton.disabled = !enabled || !this.selectedRow;
				this._cancelButton.disabled = !enabled;
				dojo.toggleClass(this.domNode, 'ple-ReplaceBranchDialog-disabled', !enabled); //$NON-NLS-1$
			},

			_handleKeyPress: function(e) {
				if (!this.enabled) return;

				var key = e.keyCode;
				if (key === dojo.keys.UP_ARROW) {
					this.selectPrevious();
				} else if (key === dojo.keys.DOWN_ARROW) {
					this.selectNext();
				} else if (key == dojo.keys.ENTER) {
					this._submit();
				}
			},

			selectNext: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select first on arrow down if no selection
					var maybeRow = this._styledTable.tbody.firstChild;
					//Sometimes the firstChild of the tbody is a text node and the next child is the actual first row
					var row = (maybeRow.nodeName == "TR")? maybeRow : maybeRow.nextSibling; //$NON-NLS-1$
					this.selectRow(row);
				}
				else if (this.selectedRow.nextSibling) {
					this.selectRow(this.selectedRow.nextSibling);
				}
			},

			selectPrevious: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select last on arrow up if no selection
					this.selectRow(this._styledTable.tbody.lastChild);
				}
				//Sometimes the firstChild of the tbody is a text node so we don't want to try to select it
				else if (this.selectedRow.previousSibling && this.selectedRow.previousSibling.nodeName == "TR") { //$NON-NLS-1$
					this.selectRow(this.selectedRow.previousSibling);
				}
			},
			
			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				this._dialog.close();
				delete this._dialog;
			},
			
			/**
			 * Called when form is submitted (click of button or type 'Enter'). //$NON-NLS-1$
			 */
			_submit: function(e) {
				e && dojo.stopEvent(e);

				this._messageBox.hideMessage();

				this.toggleEnabled(false);

				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.replacingBranch);

				var self = this;

				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/replace_branch', //$NON-NLS-1$
					content: {
						contextConfigVersionUri: self.site.contextConfigVersionUri,
						contextPartUri: self.site.contextPartUri,
						branchUri: self.selectedBranchUri
					},

					handle: function() {
						message.hide();
					},

					load: function(result) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({selectPath: self.site.path}).addBoth(function() {
							self._closeDialog();
						});
					},

					error: function(_, args) {
						self.toggleEnabled(true);
						self._messageBox.showXhrError(args.xhr);
						self._messageBox.domNode.scrollIntoView();
					}
				});
			},
			
			/**
			 * Cancels the create and closes the dialog.
			 */
			_cancel: function() {
				this._closeDialog();
			},
			
			uninitialize: function() {
				this._actionPreview.destroyRecursive();
				if (this._styledTable)
					this._styledTable.destroyRecursive();
			}
			
		});	

	})();
