/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin'); //$NON-NLS-1$


(function(){
	/**
	 * Mixin that changes the behavior of .set for properties. Widgets that
	 * mix this in must set a property called `forceWatchCallbacksFor` that is
	 * a map of property names to `true`. All these property names will
	 * callback their `watch` methods even if the property is not different
	 * than the old value.
	 *
	 * This was mainly added to retrofit dimension related code that relies on broadcasting
	 * `dirty` notification to update the UI, but wants to update "Quick Information" after each
	 * change (not just transitioning to/from dirty).
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.branch._ForceWatchCallbacksMixin', //$NON-NLS-1$
		null,
		{
			/**
			 * Copied from Dojo except for a tweak to check forceWatchCallbacksFor
			 */
			_set: function(/*String*/ name, /*anything*/ value){
				// summary:
				//		Helper function to set new value for specified attribute, and call handlers
				//		registered with watch() if the value has changed.
				var oldValue = this[name];
				this[name] = value;

				// RELM: add the extra check
				if(this._watchCallbacks && this._created && 
					(this.forceWatchCallbacksFor[name] || value !== oldValue)) {

					this._watchCallbacks(name, oldValue, value);
				}
			}
		}
	);

})();