/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.history.HistoryPage'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.history', 'HistoryPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Separator'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.TwoStateButton'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.ui.AutoHeightBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.history.HistoryGrid'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.history.MiniMap'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('jazz.app.ua.PageHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LabeledSelect'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.CreateBranchDialog'); //$NON-NLS-1$

(function(){
	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var util = com.ibm.relm.common.web.ui.util;
	var Goto = com.ibm.relm.ple.web.ui.Goto;

	dojo.declare(
		"com.ibm.relm.ple.web.ui.history.HistoryPage", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/history/templates/HistoryPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.history", "HistoryPageMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			postCreate: function() {
				this.inherited(arguments);
				this._headerWidgets = [];

				var self = this;
				this.connect(this.domNode, 'onclick', function(e) { //$NON-NLS-1$
					// Ignore link clicks
					if (e.target.nodeName === 'A') return; //$NON-NLS-1$

					if (!self._skipClearingSelection) {
						self._clearSelected();
					}
					// Don't skip twice
					delete self._skipClearingSelection;
				});

				this.subscribe('ple.history.createBranch', this.showSelectedCreateBranchDialog); //$NON-NLS-1$
				this.subscribe('ple.didCheckOutItemVersionUri', this._refreshIfItemVersionUriChanged); //$NON-NLS-1$
				this.subscribe('ple.didCheckInItemVersionUri', this._refreshIfItemVersionUriChanged); //$NON-NLS-1$
				this.subscribe('ple.history.selectionDidChange', this._updateToolbar); //$NON-NLS-1$
				this.subscribe('ple.history.arrowsTooSlow', this._showSlowWarning); //$NON-NLS-1$
				this.subscribe('ple.history.viewInMyProducts', this._viewProduct); //$NON-NLS-1$
				this.subscribe('ple.history.compareProducts', this._compareProducts); //$NON-NLS-1$
				this.subscribe('ple.history.findUse', this._findUse); //$NON-NLS-1$
				this.subscribe('ple.history.disableFilterForUri', this._disableFilterAndFocus); //$NON-NLS-1$
				this.subscribe('ple.history.redact', this.redactEvent); //$NON-NLS-1$

				if (window.localStorage['ple.history.showMiniMap'] === 'false') { //$NON-NLS-1$ //$NON-NLS-2$
					this._miniMap.hide();
					this._toggleMiniMapButton.set('pressed', false); //$NON-NLS-1$
				}
				else {
					this._miniMap.hide();
					this._toggleMiniMapButton.set('pressed', true); //$NON-NLS-1$
				}

				this._autoScrollHeight = new com.ibm.relm.common.web.ui.AutoHeightBox(this._historyScroller, {
					listenOnPage: 'com.ibm.relm.ple.web.ui.history.HistoryPage', //$NON-NLS-1$
					fillParentWidth: true,
					priority: 10
				});

				// We need to manually position the mini-map's vertical
				// position whenever this is resized. This was done with
				// 'fixed' positioning before, but then we can't reposition
				// around the Mini Dashboard if it's on the right.
				this.connect(this._autoScrollHeight, 'didResize', function(newSize) { //$NON-NLS-1$
					// Move to the bottom of the scrolling view, minus some px
					// for the horizontal scrollbar if it's there and a little
					// padding.
					var bottom = '-' + (newSize.h - 25) + 'px'; //$NON-NLS-1$ //$NON-NLS-2$
					this._miniMap.domNode.style.bottom = bottom;
				});

				this.subscribe('ple.branch_was_updated', this.refresh); //$NON-NLS-1$

				this.connect(this._historyScroller, 'onscroll', this._moveHeaders); //$NON-NLS-1$
				this.connect(this._historyScroller, 'onmousedown', this._startPan); //$NON-NLS-1$

				// This is to be sure to abort a drag if it gets "stuck" somehow and the user clicks
				this.connect(this._historyScroller, 'onclick', this._endPan); //$NON-NLS-1$

				var ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
				ActionRegistry.registerAction('ple.history', this, '_loadHistory'); //$NON-NLS-1$ //$NON-NLS-2$

				this._toolbar.startup();

				this.enableRedaction();
			},

			onShow: function() {
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.messages.historyPageTitle);
			},

			/**
			 * Hide redaction toolbar item if not jazz admin
			 */
			enableRedaction: function() {
				if (!relm.app.Permissions.canProductsRedact()) {
					this._redactButton.domNode.style.display = "none"; //$NON-NLS-1$
					this._redactSeparator.domNode.style.display = "none"; //$NON-NLS-1$
				}
			},

			showSelectedCheckInDialog: function() {
				var itemVersions = this._historyGrid.getSelectedItemVersions();
				var itemVersion = itemVersions[itemVersions.length-1];

				com.ibm.relm.ple.web.ui.productapi.checkInProduct(itemVersion);
			},

			showSelectedCreateBranchDialog: function() {
				var itemVersions = this._historyGrid.getSelectedItemVersions();
				var itemVersion = itemVersions[itemVersions.length-1];
				var args = {
					itemVersion: itemVersion
				};				
				var d = new com.ibm.relm.ple.web.ui.branch.CreateBranchDialog(args);
				d.startup();
			},

			/**
			 * Start panning (scrolling) around. Adds event listeners to track mousemoves.
			 */
			_startPan: function(e) {
				var nodeName = e.target.nodeName;

				// Trying to guess if we clicked on something non-important and worthy 
				// of starting a pan from. Not perfect due to complex contents, but this is
				// OK. If the user doesn't actually mousemove he won't see it anyway.
				if (nodeName === 'A' || nodeName === 'IMG') return; //$NON-NLS-1$ //$NON-NLS-2$
				
				// Ignore right-clicks
				if (e.button > 1) return;

				dojo.stopEvent(e);

				this._endPanHandle && dojo.disconnect(this._endPanHandle);
				this._onPanHandle && dojo.disconnect(this._onPanHandle);
				this._endPanHandle = dojo.connect(document, 'onmouseup', this, this._endPan); //$NON-NLS-1$
				this._onPanHandle = dojo.connect(document, 'onmousemove', this, this._onPan); //$NON-NLS-1$

				this._startX = e.clientX;
				this._startY = e.clientY;

				this._startScrollLeft = this._historyScroller.scrollLeft;
				this._startScrollTop = this._historyScroller.scrollTop;
			},

			_onPan: function(e) {
				if (!this._isPanning) {
					this._isPanning = true;
					dojo.addClass(this.domNode, 'ple-HistoryPage-inDrag'); //$NON-NLS-1$
				}

				var diffX = e.clientX - this._startX;
				var diffY = e.clientY - this._startY;

				this._historyScroller.scrollLeft = this._startScrollLeft - diffX;
				this._historyScroller.scrollTop = this._startScrollTop - diffY;
			},

			_endPan: function(e) {
				if (this._isPanning) {
					// If we never moved, this isn't a pan, it's a click. In
					// that case we the onclick handler will clear the
					// selected product (since the user is not clicking a
					// product now). If we did pan though, we'll set this flag
					// to indicate that the onclick handler shouldn't clear
					// the selection. Either way, after a click, this flag is
					// cleared.
					this._skipClearingSelection = true;
				}

				delete this._isPanning;
				dojo.disconnect(this._endPanHandle);
				dojo.disconnect(this._onPanHandle);
				dojo.removeClass(this.domNode, 'ple-HistoryPage-inDrag'); //$NON-NLS-1$
			},
			
			_disableFilterAndFocus: function(productUri) {
				for (var i = 0; i < this._filter.select.length; i++) {
					var option = this._filter.select[i];
					if (option.value === 'all') { //$NON-NLS-1$
						this._filter.select.selectedIndex = i;
						break;
					}
				}

				// Note that we consider this a filter change, and don't change the URL or
				// add this to history.
				this._loadHistory({
					focusProduct: productUri
				});
			},

			_moveHeaders: function(e) {
				this._headersNode.style.left = - this._historyScroller.scrollLeft + 'px'; //$NON-NLS-1$
			},

			_showSlowWarning: function() {
				this._messageBox.showMessage({
					summary: this.messages.slowArrows,
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					closable: true
				});
			},

			redactEvent: function(e) {
				e && dojo.stopEvent(e);

				var products = this._historyGrid.getSelectedItemVersions();
				var refreshCallback = function() {
					this.refresh();
					dojo.publish("ple.refresh_selected_path"); //$NON-NLS-1$
				}.bind(this);
				
				productapi.redactProducts(products, refreshCallback);
			},

			/**
			 * Enables toolbar buttons based on the selected products.
			 */
			_updateToolbar: function() {
				var selected = this._historyGrid.getSelectedItemVersions();

				switch (selected.length){
 
				case 0:
					this._viewProductButton.set('enabled', false); //$NON-NLS-1$
					this._checkInButton.set('enabled', false); //$NON-NLS-1$
					this._createBranchButton.set('enabled', false); //$NON-NLS-1$
					this._findUseButton.set('enabled', false); //$NON-NLS-1$
					this._redactButton.set('enabled', false); //$NON-NLS-1$
					break;

				case 1:
					var itemVersion = selected[0];
					this._viewProductButton.set('enabled', true); //$NON-NLS-1$
					this._checkInButton.set('enabled', !itemVersion.isImmutable); //$NON-NLS-1$
					this._createBranchButton.set('enabled', true); //$NON-NLS-1$
					this._findUseButton.set('enabled', true); //$NON-NLS-1$
					this._redactButton.set('enabled', true); //$NON-NLS-1$
					break;

				case 2:
					this._viewProductButton.set('enabled', false); //$NON-NLS-1$
					this._checkInButton.set('enabled', false); //$NON-NLS-1$
					this._createBranchButton.set('enabled', false); //$NON-NLS-1$
					this._findUseButton.set('enabled', false); //$NON-NLS-1$
					this._redactButton.set('enabled', true); //$NON-NLS-1$
					break;

				default:
					this._viewProductButton.set('enabled', false); //$NON-NLS-1$
					this._checkInButton.set('enabled', false); //$NON-NLS-1$
					this._createBranchButton.set('enabled', false); //$NON-NLS-1$
					this._findUseButton.set('enabled', false); //$NON-NLS-1$
					this._redactButton.set('enabled', true); //$NON-NLS-1$
					break;
				}
			},

			/**
			 * Compares the selected products. If 1 is selected,
			 * it will be compared to its last snapshot. If 2 or more,
			 * then the first two, in order, will be compared as left and
			 * right respectively.
			 * @param  {even} e 
			 */
			_compareProducts: function(e) {
				e && dojo.stopEvent(e);
				var selected = this._historyGrid.getSelectedItemVersionUris();

				if (selected.length === 0) return;

				Goto.gotoCompare(selected[0], selected[1]);
			},

			/**
			 * View selected product in My Products.
			 * @param  {event} e 
			 */
			_viewProduct: function(e) {
				e && dojo.stopEvent(e);
				var selected = this._historyGrid.getSelectedItemVersionUris();
				if (selected.length < 1) return;
				productapi.findPathsToProductAndShow(selected[0]);
			},

			_clearSelected: function() {
				this._historyGrid.clearSelected();
			},

			_findUse: function(e) {
				e && dojo.stopEvent(e);
				var selected = this._historyGrid.getSelectedItemVersionUris();
				if (selected.length < 1) return;
				Goto.gotoFindUse(selected[0]);
			},

			_showDetails: function(e) {
				dojo.stopEvent(e);

				window.localStorage['ple.history.hideDetails'] = 'false'; //$NON-NLS-1$ //$NON-NLS-2$
				this._historyGrid.hideAllDetails(false);
			},

			_hideDetails: function(e) {
				dojo.stopEvent(e);

				window.localStorage['ple.history.hideDetails'] = 'true'; //$NON-NLS-1$ //$NON-NLS-2$
				this._historyGrid.hideAllDetails(true);
			},

			_toggleMiniMap: function(e) {
				dojo.stopEvent(e);

				if (this._toggleMiniMapButton.get('pressed')) { //$NON-NLS-1$
					window.localStorage['ple.history.showMiniMap'] = 'true'; //$NON-NLS-1$ //$NON-NLS-2$
					this._miniMap.show();
				}
				else {
					window.localStorage['ple.history.showMiniMap'] = 'false'; //$NON-NLS-1$ //$NON-NLS-2$
					this._miniMap.hide();
				}
			},

			_initHeaders: function(branchInfos, productName) {
				var columns = [];

				for (var i = 0; i < branchInfos.length; i++) {
					columns.push({
						label: branchInfos[i].name || this.messages.defaultBranchColumn,
						width: '320px', //$NON-NLS-1$
						className: 'ple-HistoryPage-branchHeader' //$NON-NLS-1$
					});
				}

				var headersTable = new jazz.ui.StyledTable({
					columns: columns
				});
				this._headerWidgets.push(headersTable);

				dojo.style(headersTable.domNode, {
					tableLayout: 'fixed', //$NON-NLS-1$
					width: 371 * branchInfos.length + 'px' //$NON-NLS-1$
				});

				// This isn't really "presentation", but it's hard to tie this with the separate
				// table used to layout the actual cells. Since cells are labeled with their branch
				// names anyway (on the products) then I'm considering this header as optional
				// and telling JAWS to ignore.
				dojo.attr(headersTable.domNode, 'role', 'presentation'); //$NON-NLS-1$ //$NON-NLS-2$

				// this._headersTable.domNode.style.position = 'absolute';
				headersTable.placeAt(this._headersNode);
				
				if (relm.app.Permissions.canProductsBranch()) {
					var ths = dojo.query('th.ple-HistoryPage-branchHeader > div'); //$NON-NLS-1$
					
					for (var j = 0; j < ths.length; j++) {
						var headerDiv = ths[j];
						headerDiv.title = branchInfos[j].name;
					}
				}
			},
			
			refreshEvent: function(e) {
				dojo.stopEvent(e);
				this.refresh();
			},

			/**
			 * Responds to check in/out notifications. Checks if the given
			 * URI is currently in the history view and refreshes the view
			 * if it is. 
			 * 
			 * @param  {string} itemVersionUri item version that may have been modified.
			 */
			_refreshIfItemVersionUriChanged: function(itemVersionUri) {
				this._historyGrid.isShowingItemVersionUri(itemVersionUri);
				this.refresh();
			},

			refresh: function() {
				var selected = this._historyGrid.getSelectedItemVersionUris();
				var selectedUri = null;

				if (selected.length > 0) {
					selectedUri = selected[0];
				}

				this._loadHistory({
					product: this._lastProduct,
					partUri: this._lastPartUri,
					focusProduct: selectedUri
				});
			},

			/**
			 * Clears out the last event, so the next time this page is pulled up,
			 * we're guaranteed to requery for paths instead of possibly showing
			 * old results.
			 */
			_cleanCache: function() {
				delete this._lastLoadTime;
			},

			/**
			 * Jazz action to load and draw the history.
			 * @param  {object} jazzEvent event with 'product' URL to get history for. //$NON-NLS-1$
			 */
			_loadHistory: function(jazzEvent) {

				if (!jazzEvent.product) jazzEvent.product = this._lastProduct;
				if (!jazzEvent.partUri) jazzEvent.partUri = this._lastPartUri;

				// Check the timestamp of the call. If it matches, just show again.
				if (jazzEvent.time && jazzEvent.time == this._lastLoadTime &&
					(
						(jazzEvent.product && jazzEvent.product == this._lastProduct)
						||
						(jazzEvent.partUri && jazzEvent.partUri == this._lastPartUri)
					)
				) {
					this._lastTitle && net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this._lastTitle);
					return undefined;
				}

				if (jazzEvent.product) {
					this._lastProduct = jazzEvent.product;
					delete this._lastPartUri;
				}
				if (jazzEvent.partUri) {
					this._lastPartUri = jazzEvent.partUri;
					delete this._lastProduct;
				}

				this._lastLoadTime = jazzEvent.time;

				this._clearSelected();
				this._refreshButton.set('enabled', false); //$NON-NLS-1$
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.messages.historyPageTitle);
				util.setText(this._titleNode, this.messages.historyPageTitle);
				this._messageBox.hideMessage();

				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.loadingHistory);
				
				// Hack to ensure the message animations finishes right away,
				// otherwise the half-faded in message is stuck while
				// drawing all the arrows.
				var pageTemplate = dijit.byId('jazz_ui_PageTemplate_0'); //$NON-NLS-1$
				pageTemplate && pageTemplate._fadeInAnim && pageTemplate._fadeInAnim.gotoPercent(1.0);

				this._destroyHeaderWidgets();
				this._miniMap.hide();
				this._historyGrid.clear();

				var doFilter = this._filter.select.options[this._filter.select.selectedIndex].value === 'filter'; //$NON-NLS-1$

				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/history', //$NON-NLS-1$
					content: {
						itemVersionUri: jazzEvent.product,
						partUri: jazzEvent.partUri,
						filter: doFilter,
						stressTest: jazzEvent.stressTest
					},
					handleAs: 'json', //$NON-NLS-1$
					handle: dojo.hitch(this, function() {
						message.hide();
						this._refreshButton.set('enabled', true); //$NON-NLS-1$
					}),
					load: dojo.hitch(this, function(jsonHistory) {
						try {
							var realTitle = dojo.replace(this.messages.historyPageTitleOfProductX, [jsonHistory.title]);
							this._productName = jsonHistory.title;
							this._lastTitle = realTitle;
							net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(realTitle);
							util.setText(this._titleNode, realTitle);
							this._initHeaders(jsonHistory.branchInfos, jsonHistory.title);
						
							// Must rest scroll position, otherwise the grid can jump around, and the arrows will draw at the initial
							// scroll offsets, which don't line up.
							// this._historyScroller.scrollTop = this._historyScroller.scrollLeft = 0;

							var self = this;
							this.connect(this._historyGrid, 'onArrowsDidDraw', function(layoutPositions) { //$NON-NLS-1$
								this._miniMap.draw(layoutPositions, self._historyGrid.getSelectedItemVersionUris());
								this._miniMap.trackScrolling(this._historyScroller);
								if (this._toggleMiniMapButton.get('pressed')) this._miniMap.show(); //$NON-NLS-1$
							});

							this._historyGrid.draw(jsonHistory.levels);

							var selectedNode = this._historyGrid.selectProductUri(jazzEvent.focusProduct, jazzEvent.product);

							if (selectedNode) {
								selectedNode.focus();
								util.centerInView({
									scrollView: this._historyScroller,
									node: selectedNode,
									horizontally: true,
									vertically: true,
									force: true
								});
							}

							var hideDetails = window.localStorage['ple.history.hideDetails']; //$NON-NLS-1$

							//Hide all details if preference already set to 'true' OR has never been set. Show All otherwise.
							if (!hideDetails) {
								this._historyGrid.hideAllDetails(false);
							}
							else {
								this._historyGrid.hideAllDetails(window.localStorage['ple.history.hideDetails'] == 'true'); //$NON-NLS-1$ //$NON-NLS-2$
							}
						}
						catch (e) {
							// This is a debugging aid. If there is a JavaScript, non xhr
							// error, it still went to this 'error' handler, which doesn't
							// know how to format a normal JavaScript error, so just catch it here.
							console.error(e);

							this._messageBox.showMessage({
								summary: e.toString(),
								type: jazz.ui.MessageBox.STATUS_TYPES.ERROR
							});
						}
					}),
					error: dojo.hitch(this, function(dojoError, ioArgs) {
						console.error(dojoError);
						this._messageBox.showXhrError(ioArgs.xhr);
					})
				});

				// To suppress jslint warning about not all paths returning a value, even though this is 'void'
				return undefined;
			},
			
			_destroyHeaderWidgets: function() {
				for (var i = 0; i < this._headerWidgets.length; i++) {
					var w = this._headerWidgets[i];
					w.destroyRecursive();
				}
			},

			uninitialize: function() {
				this._autoScrollHeight.destroy();
				this._destroyHeaderWidgets();
				this._endPanHandle && dojo.disconnect(this._endPanHandle);
				this._onPanHandle && dojo.disconnect(this._onPanHandle);
				this.inherited(arguments);
			}
		}
	);

})();
