/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.history.MiniMap"); //$NON-NLS-1$

// dojo.require("dojo.i18n");
// dojo.requireLocalization("com.ibm.relm.ple.ui.history", "MiniMapMessages");
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){
	var MAX_SIZE_PX = 150;
	
	dojo.declare(
		"com.ibm.relm.ple.web.ui.history.MiniMap", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/history/templates/MiniMap.html'), //$NON-NLS-1$ //$NON-NLS-2$
			//messages: dojo.i18n.getLocalization("com.ibm.relm.ple.ui.history", "MiniMapMessages"),

			constructor: function(args, node)
			{
				this._dragListeners = [];
				this._urisToBoxes = {};
			},

			postCreate: function() {
				this.inherited(arguments);
				dojo.style(this._viewFinderNode, 'opacity', 0.4); //$NON-NLS-1$

				this.subscribe('ple.history.selectionDidChange', this._selectBoxes); //$NON-NLS-1$
			},

			_selectBoxes: function(selectedUris) {
				dojo.query('.ple-MiniMap-selectedBox', this._mapNode).removeClass('ple-MiniMap-selectedBox'); //$NON-NLS-1$ //$NON-NLS-2$

				for (var i = 0; i < selectedUris.length; i++) {
					var uri = selectedUris[i];
					var box = this._urisToBoxes[uri];
					if (box) {
						dojo.addClass(box, 'ple-MiniMap-selectedBox'); //$NON-NLS-1$
					}
				}
			},

			_startDrag: function(e) {
				if (this._isDragging) {
					this._endDrag();
					return;
				}

				this._isDragging = true;
				this._mapPosition = dojo.position(this._mapNode);
				this._viewFinderPosition = dojo.position(this._viewFinderNode);
				this._startX = e.clientX;
				this._startY = e.clientY;

				dojo.forEach(this._dragListeners, dojo.disconnect);
				this._dragListeners = [];

				this._dragListeners.push(dojo.connect(document, 'onmouseup', this, this._endDrag)); //$NON-NLS-1$
				this._dragListeners.push(dojo.connect(document, 'onmousemove', this, this._onDrag)); //$NON-NLS-1$

				dojo.stopEvent(e);
			},

			_onClickViewfinder: function(e) {
				// Eat this event so it doesn't bubble. We handle mouse up/down separately.
				dojo.stopEvent(e);
			},

			_onDrag: function(e) {
				var diffX = e.clientX - this._startX;
				var diffY = e.clientY - this._startY;

				var origX = this._viewFinderPosition.x - this._mapPosition.x;
				var origY = this._viewFinderPosition.y - this._mapPosition.y;

				// A bit of a hack to compensate for the width of the margin.
				var mapMargin = 2;

				// Constrain position to be inside the map
				var newX = origX + diffX;
				newX = Math.max(0, newX);
				newX = Math.min(newX, this._mapPosition.w - this._viewFinderPosition.w + 2 + mapMargin);
				var newY = origY + diffY;
				newY = Math.max(0, newY);
				newY = Math.min(newY, this._mapPosition.h - this._viewFinderPosition.h + 2 + mapMargin);


				dojo.style(this._viewFinderNode, {
					top: newY + 'px', //$NON-NLS-1$
					left: newX + 'px' //$NON-NLS-1$
				});

				this._trackedScrollingNode.scrollLeft = Math.floor(newX / this._scale);
				this._trackedScrollingNode.scrollTop = Math.floor(newY / this._scale);

				dojo.stopEvent(e);
			},

			_endDrag: function(e) {
				dojo.stopEvent(e);

				delete this._isDragging;
				delete this._startX;
				delete this._startY;

				dojo.forEach(this._dragListeners, dojo.disconnect);
				this._dragListeners = [];

				delete this._mapPosition;
				delete this._viewFinderPosition;
			},

			_scaleInt: function(number) {
				return Math.floor(this._scale * number);
			},

			setViewFinderPosition: function(position) {
				dojo.style(this._viewFinderNode, {
					top: this._scaleInt(position.y) + 'px', //$NON-NLS-1$
					left: this._scaleInt(position.x) + 'px', //$NON-NLS-1$
					width: this._scaleInt(position.w) + 'px', //$NON-NLS-1$
					height: this._scaleInt(position.h) + 'px' //$NON-NLS-1$
				});
			},

			_updateViewFinder: function() {
				if (this._isDragging) return;

				var position = dojo.position(this._trackedScrollingNode);

				position.y = this._trackedScrollingNode.scrollTop;
				position.x = this._trackedScrollingNode.scrollLeft;
				this.setViewFinderPosition(position);
			},

			trackScrolling: function(domNode) {
				this._trackedScrollingNode = domNode;

				// Do an update right away
				this._updateViewFinder();

				this._scrollListener && dojo.disconnect(this._scrollListener);
				this._scrollListener = dojo.connect(domNode, 'onscroll', this, this._updateViewFinder); //$NON-NLS-1$

				this._resizeHandle && jazz.util.ResizeNotifier.unregister(this._resizeHandle);

				// FIXME: back to resize AFTER the AutoHeightBox we're in. That should notify this instead.
				this._resizeHandle = jazz.util.ResizeNotifier.register({
						context: this,
						force: true,
						callback: dojo.hitch(this, function() {
							clearTimeout(this._resizeTimeout);
							this._resizeTimeout = setTimeout(dojo.hitch(this, this._updateViewFinder), 250);
						})
					});
			},

			_newBox: function(position) {
				var box = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-MiniMap-box' //$NON-NLS-1$
				});

				dojo.style(box, {
					top: this._scale * position.y + 'px', //$NON-NLS-1$
					left: this._scale * position.x + 'px', //$NON-NLS-1$
					width: this._scale * position.w + 'px', //$NON-NLS-1$
					height: this._scale * position.h + 'px' //$NON-NLS-1$
				});

				return box;
			},

			hide: function() {
				this.domNode.style.display = 'none'; //$NON-NLS-1$
			},

			show: function() {
				this.domNode.style.display = ''; //$NON-NLS-1$
			},

			draw: function(layoutPositions, initiallySelectedUris) {
				var selectedUriSet = {};
				for (var i = 0; i < initiallySelectedUris.length; i++) {
					var initialUri = initiallySelectedUris[i];
					selectedUriSet[initialUri] = true;
				}

				this._urisToBoxes = {};
				var width = MAX_SIZE_PX;
				this._scale = width / layoutPositions.containerPosition.w;
				var height = Math.floor(this._scale * layoutPositions.containerPosition.h);

				if (height > MAX_SIZE_PX) {
					height = MAX_SIZE_PX;
					this._scale = MAX_SIZE_PX / layoutPositions.containerPosition.h;
					width = Math.floor(this._scale * layoutPositions.containerPosition.w);
				}

				dojo.empty(this._mapNode);

				dojo.style(this._mapNode, {
					height: height + 'px', //$NON-NLS-1$
					width: width + 'px' //$NON-NLS-1$
				});

				// draw all boxes
				var memberPositions = layoutPositions.memberPositions;
				try {
					for (i = 0; i < memberPositions.length; i++) {
						var pos = memberPositions[i];
						var uri = pos.uri;
						var box = this._newBox(pos);
						if (selectedUriSet[uri]) dojo.addClass(box, 'ple-MiniMap-selectedBox'); //$NON-NLS-1$
						this._urisToBoxes[uri] = box;
						this._mapNode.appendChild(box);
					}
				}
				catch (e) {
					// Happens if the load was too slow, and not all position were calculated in HistoryGrid.
					// The result is the mapo will be missing the rest of the boxes.
				}

				// path.setStroke({width: 1, color: '#ccc'});

			},

			uninitialize: function() {
				this._scrollListener && dojo.disconnect(this._scrollListener);
				dojo.forEach(this._dragListeners, dojo.disconnect);
				this._resizeHandle && jazz.util.ResizeNotifier.unregister(this._resizeHandle);
				// this._surface && this._surface.destroy();
				this.inherited(arguments);
			}
		}
	);

})();