/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.ConfigUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

(function(){

var newRefSyntax = com.ibm.relm.explorer.web.ui.internal.util.checkURLOption("newRefSyntax");//$NON-NLS-1$ 
	
// utilities
com.ibm.relm.explorer.web.ui.internal.ConfigUtils = {
	/* 
	 * template: template string which contains variables.
	 * params: {name1: value1, name2: value2, ...} 
	 */
	evalParams: function(template, params, getValueFunc, onlyClientSideExpression){
		if(!getValueFunc){
			getValueFunc = this._defaultGetValueFunc;
		} 
		if(!params){
			// return template as is
			return template;
		}
		for(var name in params) {
			var val = getValueFunc(name, params);
			if(newRefSyntax && !onlyClientSideExpression){
				template = this.replaceAll(template, "$" + name + "$", val); //$NON-NLS-1$ //$NON-NLS-2$
			}
			template = this.replaceAll(template, "${" + name + "}", val); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return template;
	},
	
	/* 
	 * template: template string which contains variables.
	 * paramsArray: [{name1: value1, name2: value2, ...}, ...] 
	 */
	evalParamsArray: function(template, paramsArray, getValueFunc, onlyClientSideExpression){
		dojo.forEach(paramsArray, function(params){
			if(!params) return;
			template = this.evalParams(template, params, getValueFunc, onlyClientSideExpression);
		}, this);
		return template;
	},

	
	/*
	 * template: template string which contains variables.
	 * params: {name1: value1, name2: value2, ...} 
	 */
	evalVariables: function(template, values, getValueFunc, onlyClientSideExpression){
		if(!getValueFunc){
			getValueFunc = this._defaultGetValueFunc;
		}
		if(newRefSyntax && !onlyClientSideExpression){
			template = this._evalValiablesByRegExp(template, values, getValueFunc, /\$[^\$ ]*\$/gm, 1, 1);
		}
		template = this._evalValiablesByRegExp(template, values, getValueFunc, /\$\{[^\}]*\}/gm, 2, 1);
		return template;
	},
	
	/*
	 * Generate a parameteValues structure which QueryService can handle
	 * paramsArray: [{name1: value1, name2: value2, ...}, ...]  
	 */
	getParameterValuesFromParamArray: function(paramsArray){
		var parameters = {};
		for(var i=paramsArray.length-1; i>=0; i--){
			var params = paramsArray[i];
			for(var pname in params){
				var pvalue = params[pname];
				if(dojo.isString(pvalue)){
					pvalue = {type: "string", value: pvalue}; //$NON-NLS-1$
				}
				parameters[pname] = pvalue;
			}
		};
		return parameters;
	},
	
	_evalValiablesByRegExp: function(template, values, getValueFunc, regexp, beginLen, endLen){
		var result = template.match(regexp);
		if(!result){
			return template;
		}
		for(var i=result.length-1; i>=0; i--){
			var target = result[i].slice(beginLen, -endLen); // "${val}" or "$val$"
			var val = getValueFunc(target, values);
			if(val !== undefined){
				template = this.replaceAll(template, result[i], val);
			}
		}
		return template;		
	},
	
	_defaultGetValueFunc: function(propName, params){
		return params[propName];
	},
	
	/*
	 * replace all "from" to "to" in "string" 
	 */
	replaceAll: function(string, from, to){
		return string.split(from).join(to);
	},
	
	/*
	 * generate a variable syntax for a name. e.g. ${name} or $name$ 
	 */
	generateVariableString: function(variableName){
		if(newRefSyntax){
			return "$" + variableName + "$"; //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			return "${" + variableName + "}"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	}	
};

})();

