/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$

(function(){
	
// URIs
com.ibm.relm.le.ui.serviceRoot = net.jazz.ajax._contextRoot + "/"; //$NON-NLS-1$
com.ibm.relm.le.ui.leServiceUri = com.ibm.relm.le.ui.serviceRoot + "relmle/le/"; //$NON-NLS-1$
	
com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils = {};

var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
var CT_APPLICATION_JSON = "application/json"; //$NON-NLS-1$

dojo.mixin(leServiceUtils, 
{
	getPermissions: function(callback, errorback) {
		var param = "?method=getPermissions"; //$NON-NLS-1$
		return this._doGet(null, param, callback, errorback);
	},
	
	getRootElements: function(callback, errorback) {
		return this._doGet(null, null, callback, errorback);
	},

	getAllElements: function(size, callback, errorback){
		var param = "?method=getAll" + (size > 0 ? "&size="+size : ""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		return this._doGet(null, param, callback, errorback);
	},

	getElements: function(uri, callback, errorback) {
		if (!uri) 
			return this.getRootElements(callback, errorback);
		
		var param = "?method=list"; //$NON-NLS-1$
		return this._doGet(uri, param, callback, errorback);
	},
	
	getElement: function(uri, callback, errorback) {
		return this._doGet(uri, null, callback, errorback);
	},
	
	getElementById: function(id, callback, errorback) {
		var param = "?method=retrieve&id=" + encodeURIComponent(id); //$NON-NLS-1$
		return this._doGet(null, param, callback, errorback);
	},
	
	getProperties: function(uri, properties, callback, errorback) {
		if (!properties) 
			return getElement(uri, callback, errorback); 
		
		var param = "?properties="; //$NON-NLS-1$
		if (dojo.isArray(properties)) {
			param += properties.join(","); //$NON-NLS-1$
		} else {
			param += properties;
		}
		return this._doGet(uri, param, callback, errorback);
	},
	
	getSparqlTemplate: function(uri, callback, errorback) {
		return getProperties(uri, ["sparqlTemplate"], callback, errorback); //$NON-NLS-1$
	},

	createElement: function(autoId, properties, callback, errorback){
		var uriParam = autoId ? "?autoId=true" : ""; //$NON-NLS-1$ //$NON-NLS-2$
		return this._doPost(null, uriParam, {postData: dojo.toJson(properties), contentType: CT_APPLICATION_JSON}, callback, errorback);
	},
	
	createChildElement: function(uri, autoId, properties, callback, errorback){
		var uriParam = autoId ? "?autoId=true" : ""; //$NON-NLS-1$ //$NON-NLS-2$
		return this._doPost(uri, uriParam, {postData: dojo.toJson(properties), contentType: CT_APPLICATION_JSON}, callback, errorback);
	},
	
	updateElement: function(uri, properties, param, callback, errorback){
		dojo.mixin(param, {putData: dojo.toJson(properties), contentType: CT_APPLICATION_JSON});
		return this._doPut(uri, null, param, callback, errorback);
	},

	deleteElement: function(uri, param, callback, errorback){
		return this._doDelete(uri, null, param, callback, errorback);
	},

	deleteElements: function(uris, callback, errorback){
		var uriParam = "?method=delete"; //$NON-NLS-1$
		return this._doPost(null, uriParam, {content: { target: uris }}, callback, errorback);
	},

	moveElements: function(uris, destination, callback, errorback){
		var uriParam = "?method=move"; //$NON-NLS-1$
		return this._doPost(null, uriParam, {content: { target: uris, destination: destination }}, callback, errorback);
	},

	copyElements: function(uris, destination, callback, errorback){
		var uriParam = "?method=copy"; //$NON-NLS-1$
		return this._doPost(null, uriParam, {content: { target: uris, destination: destination }}, callback, errorback);
	},

	deploySample: function(callback, errorback) {
		var param = "?method=deploySample"; //$NON-NLS-1$
		return this._doPost(null, param, null, callback, errorback);
	},
	
	executeQuery: function(uri, resources, parameters, callback, errorback) {
		var uriParam = "?method=execute"; //$NON-NLS-1$
		var arguments = {};
		if (resources) arguments.resources = dojo.toJson(resources);
		if (parameters) arguments.parameterValues = dojo.toJson(parameters);
		if (djConfig.isDebug) arguments.debug = true;
		return this._doPost(uri, uriParam, {content: arguments}, callback, errorback);
	},
	
	executeQueryById: function(id, resources, parameters, callback, errorback) {
		var uriParam = "?method=execute&id=" + encodeURIComponent(id); //$NON-NLS-1$
		var arguments = {};
		if (resources) arguments.resources = dojo.toJson(resources);
		if (parameters) arguments.parameterValues = dojo.toJson(parameters);
		if (djConfig.isDebug) arguments.debug = true;
		return this._doPost(null, uriParam, {content: arguments}, callback, errorback);
	},
	
	_doGet: function(uri, uriParam, callback, errorback) {
		return this._run(jazz.client.xhrGet, uri, uriParam, {preventCache: true}, callback, errorback);
	},
	_doPost: function(uri, uriParam, arguments, callback, errorback) {
		return this._run(jazz.client.xhrPost, uri, uriParam, arguments, callback, errorback);
	},
	_doPut: function(uri, uriParam, arguments, callback, errorback){
		return this._run(jazz.client.xhrPut, uri, uriParam, arguments, callback, errorback);
	},
	_doDelete: function(uri, uriParam, arguments, callback, errorback) {
		return this._run(jazz.client.xhrDelete, uri, uriParam, arguments, callback, errorback);
	},

	_run: function(method, requestUri, uriParam, arguments, callback, errorback) {
		var uri = requestUri ? requestUri : com.ibm.relm.le.ui.leServiceUri;
		if (uriParam) uri += uriParam;
		var param = {
				url: uri,
				headers: {"Accept": "application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
				handleAs: "json" //$NON-NLS-1$
		};
		if (arguments) dojo.mixin(param, arguments);
		var hasBody = arguments && (arguments.postData || arguments.putData) ? true : false;
		
		// use callback is specified when specified, otherwise return deferred object.
		if (callback || errorback) {
			param.load = callback;
			param.error = errorback;
			return method.call(this, param, hasBody);
		} else {
			var deferred = new dojo.Deferred();
			param.load = function(response, ioargs) {
				deferred.callback(response, ioargs);
			};
			param.error = function(response, ioargs) {
				deferred.errback(response);
			};
			method.call(this, param, hasBody);
			return deferred;
		}
	}

});
})();
