/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.dashboard.ui.internal.RELMViewViewletPreferenceProvider");//$NON-NLS-1$

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ViewSelect"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.dashboard.ui.internal", "RELMViewViewletMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() {

var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var ButtonDialog = com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;

var PREFERENCE_VIEWURI = "com.ibm.relm.explorer.web.dashboard.preference.viewuri";//$NON-NLS-1$
/*
 * The preference provider class.
 */
dojo.declare("com.ibm.relm.explorer.web.dashboard.ui.internal.RELMViewViewletPreferenceProvider",//$NON-NLS-1$  
	CustomPreferenceProvider, {
	label: null, 
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.dashboard.ui.internal", "RELMViewViewletMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	getLabel: function(value){
		try{
			return dojo.fromJson(value).title;
		}catch(e){
			return this.messages.notSpecified;
		}
	},
	createChooser: function(onOk, settings, viewlet){
		var content = new com.ibm.relm.explorer.web.dashboard.ui.internal.RELMViewViewletPreferenceSettingContent({}, dojo.create("div")); //$NON-NLS-1$
		new ButtonDialog({
			primaryTitle: this.messages.dialogTitle, 
			dialogContent: content,
			onOK: function(){
				var viewuri = this.dialogContent.getUri();
				var title = this.dialogContent.getTitle();
				var parameters = this.dialogContent.getParameters();
				onOk(dojo.toJson({title: title, uri: viewuri, params: parameters}), PREFERENCE_VIEWURI);
			}
		});
	}
});

dojo.declare("com.ibm.relm.explorer.web.dashboard.ui.internal.RELMViewViewletPreferenceSettingContent",  //$NON-NLS-1$
	[dijit._WidgetBase, dijit._Templated], {
	widgetsInTemplate: true, 
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web.dashboard","ui/internal/templates/RELMViewViewletPreferenceSettingContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

	_selectedView: null,
	_selectedProjectArea: null,
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.dashboard.ui.internal", "RELMViewViewletMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);

		// project area selection
		var serviceObject = {
			self: this,
			_deferredCallback: function() {
				if (this.deferred) {
					this._successCallback(this.deferred);
					this.deferred = false;
				}
			},
			_successCallback: function(/*array*/ projects) {
				if (!this.self.domNode) {
					return;
				}
				projects.sort(function(a,b) {
					return a.name < b.name ? -1 : 1;
				});
				var select = document.createElement("select");//$NON-NLS-1$
				// populate the projects
				dojo.forEach(projects, function(p, index){
					dojo.create("option", {value: p.itemId, innerHTML: p.name}, select);//$NON-NLS-1$					
				}, this);
				dojo.place(select, this.self._areaPicker, "only"); //$NON-NLS-1$
				this.self.connect(select, "onchange", this.self._areaPickerChanged);//$NON-NLS-1$ 
				this.self.areaSelect = select;
				this.self._areaPickerChanged();
			},
			_errorCallback: dojo.hitch(this, function(error) {
				this.onError(error);
			})
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");//$NON-NLS-1$ //$NON-NLS-2$
		ProcessClient.getAllProjectAreas(responseHandler);

		this._areaPicker.innerHTML = this._messages.loading;
		
		// view selection
		this.connect(this.viewSelect, "onChange", this._viewSelectionChanged);//$NON-NLS-1$
	},
	
	_areaPickerChanged: function(){
		this.viewSelect.setProjectArea(this.areaSelect.value);
	},
	
	_viewSelectionChanged: function(){
		var viewUri = this.getUri();
		jazz.client.xhrGet({
			url: viewUri, 
			preventCache: true,
			headers: {"Accept": "text/json"}, //$NON-NLS-1$ //$NON-NLS-2$
			handleAs: "json", //$NON-NLS-1$
			load: dojo.hitch(this, function(result){
				var parameters = [];
				var params = result.params;
				for(var name in params){
					var p = params[name];
					parameters.push({
						name: name,
						value: p.value,
						type: p.type,
						description: p.description
					});
				}
				dojo.style(this.paramTable.domNode, "display", parameters.length == 0 ? "none" : "");//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this.paramTable.setParameters(parameters);
			})
		});
	},
	
	onError: function(error){
		
	},
	
	getUri: function(){
		return this.viewSelect.getUri();
	},
	
	getTitle: function(){
		return this.viewSelect.getTitle();
	},
	
	getParameters: function(){
		return this.paramTable.parameters;
	}
});

})();
