/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.common.ImportQueryDialog"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ImportQueryDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	var ImportQueryWindowName = "_relmImportQueryWindow"; //$NON-NLS-1$
	var OpenWindowOptions = "width=1000,height=600,status=yes,resizable=yes,scrollbars=yes"; //$NON-NLS-1$
	
	dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.common.ImportQueryDialog", //$NON-NLS-1$
		[ com.ibm.relm.ple.web.ui.picker.BasePicker],
		{
			dialogWidth: "50%", //$NON-NLS-1$
			constructor: function (args)
			{
				this.dialogTitle = args.dialogTitle;
				this.iconClass = args.iconClass; 
			},
			
			postMixInProperties: function(){
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ImportQueryDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$ 
				this.dialogTitle = this.dialogTitle || this._messages.defaultTitle;
				this.submitButtonLabel = this._messages.importLabel;				
			},

			postCreate: function() {
				this.inherited(arguments);
				this.buildButton = dojo.create("button", {innerHTML: this._messages.buildLabel}, this.submitButton, "after"); //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(window, dojo.isIE <= 9 ? "onfocusin" : "onfocus", this._getList); //$NON-NLS-1$ //$NON-NLS-2$ 
				this.connect(this.buildButton, "onclick", this._onBuildButtonClick); //$NON-NLS-1$ 
				this._getList();
			},
			
			_openBuildPage: false,
			_onBuildButtonClick: function(e){
				var self = this;
				jazz.client.xhrGet({
					url: com.ibm.relm.explorer.web.ui.jrsServiceUri + "?buildUrl", //$NON-NLS-1$
					load: function(buildUrl) {
						self._openBuildPage = true;
						window.open(buildUrl, ImportQueryWindowName, OpenWindowOptions);
					},
					error: function(error, ioArgs) {
						self.buildButton.disabled = true;
					}
				});
				e && dojo.stopEvent(e);
			},
			
			_getList: function(){
				if(this.prevXhr){
					this.prevXhr.cancel(); //$NON-NLS-2$ //$NON-NLS-2$
				}

				var self = this;
				this.prevXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: com.ibm.relm.explorer.web.ui.jrsServiceUri, //$NON-NLS-1$
					load: function(searchResults) {
						self.resultList.clearResults();
						self.showResults(searchResults);
						self._listQueries = searchResults;
						//
						if(self._openBuildPage){
							var list = self.resultList;
							if(list._resultList.firstChild){
								list._selectRow(list._resultList.firstChild);
							}
							self._openBuildPage = false;
						}
					},
					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback.
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});

			},
			
			createListItem: function(item){
				var li = dojo.create("li"); //$NON-NLS-1$
				dojo.create("span", {innerHTML: item.title, style: {fontWeight: "bold"}}, li);	//$NON-NLS-1$//$NON-NLS-2$
				dojo.create("img", { //$NON-NLS-1$
					src: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/edit.gif"), //$NON-NLS-1$//$NON-NLS-2$ 
					onclick: function(){ window.open(item.uri, ImportQueryWindowName, OpenWindowOptions); }, 
					title: this._messages.openQuery, 
					style: {cssFloat: "right"} }, li); //$NON-NLS-1$
				dojo.create("div", {innerHTML: item.description}, li);//$NON-NLS-1$//$NON-NLS-2$
				return li;
			},
			
			doSearch: function(searchString){
				var filteredList = this._listQueries;
				var search = searchString.toLowerCase();
				filteredList = dojo.filter(filteredList, function(l){
					return l.title.toLowerCase().indexOf(search) >= 0 || (l.description && l.description.toLowerCase().indexOf(search) >= 0);  
				});
				this.showResults(filteredList);
			},
			
			clearResults: function(){
				this.inherited(arguments);
				if(this._listQueries){
					this.showResults(this._listQueries);
				}
			}
		});

})();