/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.common.ListPage"); //$NON-NLS-1$

dojo.require("dojo.date.stamp"); //$NON-NLS-1$
dojo.require("dojo.date.locale"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.explorer.page.MetaNav"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("dijit.form.ToggleButton"); //$NON-NLS-1$
dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.SimpleToolbar"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.Toolbar"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.list.FolderRow"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.list.ItemRow"); //$NON-NLS-1$

(function(){
	
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
var ROWTYPE_FOLDER = "folder"; //$NON-NLS-1$
var INDENT = 20;
var INDENT2 = 40;

var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;

var FolderRow = com.ibm.relm.explorer.web.ui.internal.list.FolderRow;
var ItemRow = com.ibm.relm.explorer.web.ui.internal.list.ItemRow;

/*
 * Abstract class to display list of resources like Views and Queries.
 * Resources can be organized by Folder.
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.common.ListPage",  //$NON-NLS-1$
		[dijit._Widget, dijit._Templated],
{
	page: null, // container of this list widget which must have setListDirty() 
	myResourcesTab: null, 		// tab for my resource
	sharedResourcesTab: null,	// tab for shared resource
	_childMap: null,			// [][] child array for folder array 
	_colexpState: null,	 
	_folders: null,				// folders with folderUUID 
	_callCount: 0,
	_previousSelectedResourceUri: null,
	_prevSelectedRow: null,
	_refreshRequired: false,
	
	// subclasses must override the following properties
	_serviceURL: null,
	_serviceClient: null, 
	
	_label_MyResources: "",//$NON-NLS-1$
	_label_SharedResources: "",//$NON-NLS-1$
	_label_createResource: "",//$NON-NLS-1$
	_label_deleteResource: "",//$NON-NLS-1$
	_label_moveResource: "",//$NON-NLS-1$
	
	_collapseIcon : null, 
	_expandIcon : null, 
	_resourceIcon : null, 

	_moveCopyDialog: null,		// must be subclass of AbstractMoveCopyDialog

	actionPrefix: null,
	selectAction: null,
	editFolderAction:null,

	ROWTYPE_RESOURCE: null,		//"view" or "query"
	_folderType: null,			//"view" or "query"

	// template
    widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/internal/common/templates/ListPage.html"), //$NON-NLS-1$ //$NON-NLS-2$

	/*
	 * Public method
	 */
	getTitleString: function(){
		if(this.myResourcesTab.checked){
			return this.myResourcesTab.label;
		} else {
			return this.sharedResourcesTab.label;
		}
	},

	/*
	 * 
	 */
	_addToolbarButtons: function(toolbar){
		// subclass can implement if necessary
	},
	
	_setCSHelp: function(){
		// subclass can implement if necessary
	},

	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._childMap = {};
		this._colexpState = {};
		this._folders = {};
	},

	postCreate: function(){
		this.inherited(arguments);
		var messages = this._messages;
		this.shared = (this.scope == "shared") ? true : false; //$NON-NLS-1$
		// set title
		this.myResourcesTab = new dijit.form.ToggleButton({label: this._label_MyResources});
		this.connect(this.myResourcesTab, "onClick", "onClickMyResources"); //$NON-NLS-1$ //$NON-NLS-2$
		this.sharedResourcesTab = new dijit.form.ToggleButton({label: this._label_SharedResources});
		this.connect(this.sharedResourcesTab, "onClick", "onClickSharedResources"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.place(this.myResourcesTab.domNode, this.titleArea);
		dojo.place(this.sharedResourcesTab.domNode, this.titleArea);

		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({columns: [{label: messages.Name, width: "30%", sortable: true}, //$NON-NLS-1$
		                                         {label: messages.CreatedBy, width: "14%", sortable: true}, //$NON-NLS-1$
		                                         {label: messages.LastUpdated, width: "14%", sortable: true}, //$NON-NLS-1$
		                                         {label: messages.Actions, width: "12%", className: "action-column", sortable: false}], //$NON-NLS-1$ //$NON-NLS-2$
		                                        trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, 
		                                        handleSort: dojo.hitch(this, this._handleSort),
		                                        sortBy: 0,
		                                        sortByAscending: true,
		                                        showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		this._populateHeader();
		// initialize permission
		this.initPermission();
	},

	initPermission: function(){
		var self = this;
		jazz.client.xhrGet({
			url: this._serviceURL + "?id=-1" + "&permission=", //$NON-NLS-1$ //$NON-NLS-2$
			headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
			preventCache: true,
			handleAs: "json", //$NON-NLS-1$
			load: dojo.hitch(this, function(response, ioArgs){				
				self.permissions = com.ibm.relm.explorer.web.ui.internal.util.parsePermission(response);
				self._updateActions();

				// get list of queries and display them
				self._refreshContent();
			}),
			error: function(error) {
				alert(error.message);
			}
		});
	},
	
	/**
	*	This is the function that is used to handle the sorting of the styled table. 
	*/
	_handleSort: function() {
		this._emptyTable();
		this._sortChildren(this._rootFolder.children, null, 9, this._rootFolder.folderUUID, true);
	},

	/**
	 * Function to recursively sort folders
	 */
	_sortChildren: function(children, parentDiv, parentPadding, parentUUID, fShow) {
		parentDiv && this._showExpando(parentDiv, children.folders.length || children.resources.length);
		var folders = children.folders;
		this._performSort(folders);
		dojo.forEach(folders, function(folder) {
			var folderDiv;
			var padding;
			if (parentDiv) {
				padding = parentPadding + INDENT;
				folderDiv = this._createFolderRow(folder, padding, fShow);
				this._insertAfter(folderDiv, this._getFolderInsertionPoint(parentDiv));
			}
			else {//In root folder 
				padding = 9;
				folderDiv = this._createFolderRow(folder, padding, true);
				this._table.tbody.appendChild(folderDiv);
			}
			this._sortChildren(folder.children, folderDiv, padding, folder.folderUUID,this._colexpState[folder.folderUUID]);
		}, this);
		//Sort
		this._sortAndPlace(children.resources, parentDiv, parentPadding, parentUUID, fShow);
	},

	_sortAndPlace: function(resources, parentDiv, padding, folderUUID, fShow) {
		var inFolder = !!parentDiv;

		this._performSort(resources);

		if (inFolder) {
			dojo.forEach(resources, function(res) {
				this._insertAfter(this._createResourceRow(res, padding + INDENT2, this._colexpState[folderUUID]), this._getResourceInsertionPoint(folderUUID));
			}, this);
		} else {//In root folder
			dojo.forEach(resources, function(res) {
				dojo.place(this._createResourceRow(res, padding + INDENT, true), this._table.tbody, "last"); //$NON-NLS-1$
			}, this);
		}
	},

	/**
	*	Calls the appropriate sorting function on this Array based on which columns off 
	*	the table needs to be sorted
	*
	*	There are three sorting functions (name, date and creator). Each is called with dojo.partial
	*	which sneeks in a parameter to tell the function which order to sort in (ascending/descending)
	*	@param {Array} foldersOrRows 
	*/
	_performSort: function(foldersOrRows) {
		var table = this._table;
		var sortBy = table.sortBy;
		var aToz = table.sortByAscending;

		switch (sortBy) {
			case 0:
				return foldersOrRows.sort(dojo.hitch(this, dojo.partial(this._sortByName, aToz)));
			case 1: 
				return foldersOrRows.sort(dojo.hitch(this, dojo.partial(this._sortByCreator, aToz)));
			case 2:
				return foldersOrRows.sort(dojo.hitch(this, dojo.partial(this._sortByDate, aToz)));
		}
	},

	_sortByName: function(asc, row1, row2) {
		var retVal;
		var name1 = row1.name.label;
		var name2 = row2.name.label;
		retVal = name1.localeCompare(name2);

		if(asc)
			return retVal;
		else
			return retVal * -1;
	},

	_sortByDate: function(asc, row1, row2) {
		var date1 = dojo.date.stamp.fromISOString(row1.modified);
		var date2 = dojo.date.stamp.fromISOString(row2.modified);
		var retVal = date1 - date2;

		if(asc)
			return retVal;
		else
			return retVal * -1;
	},

	_sortByCreator: function(asc, row1, row2) {
		var name1 = this._getDisplayValue(row1.creator);
		var name2 = this._getDisplayValue(row2.creator);
		
		var retVal;
		retVal = name1.localeCompare(name2);

		if(asc)
			return retVal;
		else
			return retVal * -1;
	},

	_getDisplayValue: function(valueObject) {
		var value = valueObject.value;
		if (valueObject.label)
			return valueObject.label;
		else {
			var displayValue = decodeURIComponent(value.substring(value.lastIndexOf('/') + 1)); //$NON-NLS-1$
			//Try to cache the label so it doesn't need to computed again when sorting
			valueObject.label = displayValue;
			return displayValue;
		}
	},
	
	/*
	 * Construct header area. Called from PostCreate
	 */
	_populateHeader: function(){
		var messages = this._messages;
		var title = this.shared ? this._label_SharedResources : this._label_MyResources;
		this.metanav.setTitle(jazz.util.html.escape(title));

		var toolbar = new jazz.ui.SimpleToolbar({});
		this.metanav._toolbar = toolbar;
		this.metanav._toolbarDiv.appendChild(toolbar.domNode);
		toolbar.addButton(this, this._newResource, {showLabel: true, imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web", "explorer/images/new_resource.gif"), label: this._label_createResource, requiresSelection:false, requireCreate:true}); //$NON-NLS-1$ //$NON-NLS-2$
		toolbar.addButton(this, this._newFolder, {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/new-folder-obj.gif"), label: this._messages.createFolder, requiresSelection:false, requireCreate:true}); //$NON-NLS-1$ //$NON-NLS-2$
		toolbar.addButton(this, this._reloadResources, {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/refresh16.gif"), label: messages.refresh, requiresSelection:false}); //$NON-NLS-1$ //$NON-NLS-2$
		toolbar.addButton(this, this._deleteItems, {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/delete_hov.gif"), label: this._label_deleteResources, requiresSelection:true, requireDelete:true}); //$NON-NLS-1$ //$NON-NLS-2$
		toolbar.addButton(this, this._moveItems, {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/movecopy.gif"), label: this._label_moveResources, requiresSelection:true}); //$NON-NLS-1$ //$NON-NLS-2$
		this._addToolbarButtons(toolbar);
		this.refreshButton = this.metanav._toolbar.getItem(2);

		this._setCSHelp();
		
		if(this.displayBreadcrumbs) {
			this.metanav.populateReportsBreadcrumbs();
		} else {
			this.metanav.handle.style.height = "30px"; //$NON-NLS-1$
			this.metanav.handle.style.minHeight = "30px"; //$NON-NLS-1$
		}
	},	
	
	/* 
	 * Event handlers
	 */
	onClickMyResources: function(){
		window.location = this.actionPrefix + "&scope=personal"; //$NON-NLS-1$
		JazzUIWorkbench.setTitle(this._label_MyResources);
		this.refreshTabs();
	},

	onClickSharedResources: function(){
		window.location = this.actionPrefix + "&scope=shared"; //$NON-NLS-1$
		JazzUIWorkbench.setTitle(this._label_SharedResources);
		this.refreshTabs();
	},
	
	_reloadResources: function() {
		if(this._callCount > 0) {
			return;
		}
		this._refreshContent();
	},

	/*
	 * Call count management
	 */
	// should be called before ServiceClient call
	_incCount: function() {
		if(this._callCount == 0) {
			this.metanav._toolbar.disableButton(this.refreshButton);
			JazzUIWorkbench.showMessage(this._messages.ExecutingQuery);
		}
		this._callCount++;
	},
	
	// should be called in ServiceClient callbacks
	_decCount: function() {
		this._callCount--;
		if(this._callCount <= 0) {
			this.metanav._toolbar.enableButton(this.refreshButton);
			JazzUIWorkbench.hideMessageArea();	            	
			this._callCount = 0;
			if(	this._prevSelectedRow && this._prevSelectedRow.domNode) {
				this._prevSelectedRow.domNode.scrollIntoView();
				dijit.focus(this._prevSelectedRow.domNode.children[0].lastChild);
				this._previousSelectedResourceUri = null;
				this._prevSelectedRow = null;
			}
		}
	},
	
	/*
	 * Render contents
	 */
	_refreshContent: function(pageUrl) { // was _reloadResources
		this.inherited(arguments);
		this.messageArea.hideMessage();
		this.showResourcesOutOfFolder = true;
		var self = this;

		this.refreshTabs();

		var title = this.shared ? this._label_SharedResources : this._label_MyResources;
		JazzUIWorkbench.setTitle(title);

		var responseHandler = new ServiceResponseHandler(
				self,
				"_renderRootFolders", //$NON-NLS-1$
		"_remoteError"); //$NON-NLS-1$
		this._incCount();
		this._serviceClient.getSystemRootFolderDescriptors(responseHandler);
	},	

	refreshTabs:function() {
		this.myResourcesTab.set('checked', !this.shared); //$NON-NLS-1$
		this.sharedResourcesTab.set('checked', this.shared); //$NON-NLS-1$
	},
	
	_emptyTable: function() {
		var widgets = dijit.findWidgets(this._table.domNode);
		dojo.forEach(widgets, function(widget) {
			widget.destroyRecursive();
		});
		this._table.clearBody();
	},

	getTr: function(guid) {
		return dojo.query('[data-guid=' + guid + ']')[0]; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	/*
	 * get resources from result of service call
	 */
	_getResourcesFromResult: function(response) {
		return dojo.map(response.results.bindings, function(r){
			return { 
				"name": {  //$NON-NLS-1$
					label: r.title.value, 
					type: "uri",  //$NON-NLS-1$
					value: r.uri.value,
					iconSrc: this._resourceIcon
				}, 
				"description": (r.description) ? r.description.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"creator": r.creator, //$NON-NLS-1$
				"modified": (r.modified) ? r.modified.value : "",  //$NON-NLS-1$ //$NON-NLS-2$
				"parentUUID": (r.parentUUID) ? r.parentUUID.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"parameterized": (r.parameterized) ? r.parameterized.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"ownedBy": (r.ownedBy) ? r.ownedBy.value : "" //$NON-NLS-1$ //$NON-NLS-2$
			};
		}, this);

	},

	/*
	 * get folders from result of service call
	 */
	_getFoldersFromResult: function(response){

		return dojo.map(response.results.bindings, function(r){
			return { 
				"name": {  //$NON-NLS-1$
					label: r.name.value, 
					type: "uri",  //$NON-NLS-1$
					value: r.uri.value
				}, 
				"children" : { folders: [], resources: []}, //$NON-NLS-1$
				"description": (r.description) ? r.description.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"creator": r.creator, //$NON-NLS-1$
				"modified": (r.modified) ? r.modified.value : "",            			 //$NON-NLS-2$ //$NON-NLS-1$
				"ownedBy": (r.ownedBy) ? r.ownedBy.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"parentUUID": (r.parentUuid) ? r.parentUuid.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"folderUUID": (r.folderUUID) ? r.folderUUID.value : "", //$NON-NLS-1$ //$NON-NLS-2$
				"shared": (r.scope) ? (r.scope.value === "shared") : false//$NON-NLS-1$ //$NON-NLS-2$
			};
		});
	},
	
	/*
	 * The root folders here are either "shared" or "my" resources
	 * callback of service call
	 */
	_renderRootFolders: function(response) {
		var folders = this._getFoldersFromResult(response);

		if (folders.length === 0) {
			// Something went wrong
			console.error("No root folder");//$NON-NLS-1$
			this._decCount();
			return;
		}
		for(var i=0; i<folders.length; i++){
			if(folders[i].shared === this.shared){
				this._rootFolder = folders[i];
				this._folders[this._rootFolder.folderUUID] = this._rootFolder;
				this._renderChildFoldersForParent(this._rootFolder.folderUUID, true);
				this._renderChildsWithoutProjectArea();
				break;
			}
		}
		this._decCount();
	},

	// get the children under root folder without project area property (for migration)
	_renderChildsWithoutProjectArea: function() {
		var args = {
				scope: (this.shared) ? "shared" : "personal", //$NON-NLS-1$ //$NON-NLS-2$
				sortByName: true
		};
		
		var responseHandler = new ServiceResponseHandler(
				this,
				"_addChildFoldersNonpa", //$NON-NLS-1$ 
		"_remoteError"); //$NON-NLS-1$
		this._incCount();
		this._serviceClient.getFolderDescriptorsUnderNonProjectAreaRootFolder(responseHandler, args);
		var args2 = {
				scope: (this.shared) ? "shared" : "personal", //$NON-NLS-1$ //$NON-NLS-2$
				sortByName: true
		};
		if (this._currentFilter) {
			args2.filter = this._currentFilter;
		}
		var responseHandler2 = new ServiceResponseHandler(
				this,
				"_addChildResourcesNonpa", //$NON-NLS-1$
		"_remoteError"); //$NON-NLS-1$
		this._incCount();
		this._serviceClient.getResourceDescriptorsUnderNonProjectAreaRootFolder(responseHandler2, args2);
	},

	// Gets the child folders for either "shared ***" or "my ***"
	_renderChildFoldersForParent: function(folderUUID, fShow){

		this._emptyTable();
		this._updateActions();
		if (this._loadTimeout) {
			clearTimeout(this._loadTimeout);
			delete this._loadTimeout;
		}
		if (!this._nextResourceId)
			this._nextResourceId = 1;
		var args = {
//				Project Area will be added by ServiceClient
//				Owner is intentionally empty
				parentUUID: folderUUID,
				sortByName: true
		};
		
		var responseHandler = new ServiceResponseHandler(
				this,
				(fShow) ? "_addChildFoldersShow" : "_addChildFoldersHide", //$NON-NLS-1$ //$NON-NLS-2$
		"_remoteError"); //$NON-NLS-1$
		this._incCount();
		this._serviceClient.getFolderDescriptors(responseHandler, args);
		var args2 = {
//				Project Area will be added by ServiceClient
//				Owner is intentionally empty
				parentUUID: folderUUID,
				sortByName: true
		};
		if (this._currentFilter) {
			args2.filter = this._currentFilter;
		}
		var responseHandler2 = new ServiceResponseHandler(
				this,
				(fShow) ? "_addChildResourcesAndShow" : "_addChildResourcesAndHide", //$NON-NLS-1$ //$NON-NLS-2$
		"_remoteError"); //$NON-NLS-1$
		this._incCount();
		this._serviceClient.getResourceDescriptors(responseHandler2, args2);
		// for queries which do not belong to any folder
		if(this.showResourcesOutOfFolder) {

			var args3 = {
					parentUUID: "", //$NON-NLS-1$
					scope: (this.shared) ? "shared" : "personal", //$NON-NLS-1$ //$NON-NLS-2$
							sortByName: true
			};
			if (this._currentFilter) {
			args3.filter = this._currentFilter;
			}
			var responseHandler3 = new ServiceResponseHandler(
					this,
					(fShow) ? "_addChildResourcesAndShow" : "_addChildResourcesAndHide", //$NON-NLS-1$ //$NON-NLS-2$
			"_remoteError"); //$NON-NLS-1$
			this._incCount();
			this._serviceClient.getResourceDescriptors(responseHandler3, args3);
		}
	},

	// callback
	_addChildFoldersNonpa: function(response) {
		response.nonpaFolder = true;
		this._addChildFolders(response, true);
		this._decCount();
	},

	// callback
	_addChildFoldersShow: function(response) {
		this._addChildFolders(response, true);
		this._decCount();
	},

	// callback
	_addChildFoldersHide: function(response) {
		this._addChildFolders(response, false);
		this._decCount();
	},

	_addChildFolders: function(response, fShow) {
		JazzUIWorkbench.hideMessageArea();
//		How many filters are already in the list
		var results = this._getFoldersFromResult(response);

		if (results != null) {
			this._performSort(results);
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				this._childMap[folder.folderUUID] = new Array();
				this._folders[folder.folderUUID] = folder;
//				Locate the parent to insert it under
				if (folder.parentUUID == this._rootFolder.folderUUID || folder.parentUUID == null || response.nonpaFolder) {
					var folderRow = this._createFolderRow(folder, 9, fShow);
					this._table.tbody.appendChild(folderRow);
					this._rootFolder.children.folders.push(folder);
				} else {
					var parentUUID = folder.parentUUID;
					var parentDiv = this.getTr(parentUUID);
					var parentObj = this._folders[parentUUID];
					if (parentDiv == null || parentObj == null) {
						console.warn("Parent not found for folder"); //$NON-NLS-1$
					} else {
						this._showExpando(parentDiv, true);
						this._childMap[folder.parentUUID].push(folder.folderUUID); //MFV added folders to the child map to handle click on parent folder checkbox
						this._insertAfter(this._createFolderRow(folder, parentDiv._indent + INDENT, fShow), this._getFolderInsertionPoint(parentDiv));
						parentObj.children.folders.push(folder);
					}
				}
			}
//			Recurse
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				var args = {
						parentUUID: folder.folderUUID,
						sortByName: true
				};
				var responseHandler = new ServiceResponseHandler(
						this,
						(this._colexpState[folder.folderUUID]) ? "_addChildFoldersShow" : "_addChildFoldersHide", //$NON-NLS-1$ //$NON-NLS-2$
				"_remoteError"); //$NON-NLS-1$
				this._incCount();
				this._serviceClient.getFolderDescriptors(responseHandler, args);
				var args2 = {
						parentUUID: folder.folderUUID,
						sortByName: true
				};
				if (this._currentFilter) {
					args2.filter = this._currentFilter;
				}
				var responseHandler2 = new ServiceResponseHandler(
						this,
						(this._colexpState[folder.folderUUID]) ? "_addChildResourcesAndShow" : "_addChildResourcesAndHide", //$NON-NLS-1$ //$NON-NLS-2$
				"_remoteError"); //$NON-NLS-1$
				this._incCount();
				this._serviceClient.getResourceDescriptors(responseHandler2, args2);
			}
		}
	},

	_getFolderInsertionPoint: function(folderTRNode) {
		var candidate = folderTRNode;
		var nextCandidate = candidate.nextSibling;
		while (true) {
			if (nextCandidate == null) {
				return candidate;
			}
			var tmp2 = folderTRNode._indent + (INDENT2 - INDENT);
			var tmp = nextCandidate._indent < tmp2;
			if (nextCandidate._rowType == this.ROWTYPE_RESOURCE || tmp){
				return candidate;
			}
			candidate = nextCandidate;
			nextCandidate = nextCandidate.nextSibling;
		} 
	},

	// callback
	_addChildResourcesNonpa: function(response) {
		response.nonpaResources = true;
		this._addChildResources(response, true);
		this._decCount();
	},
	// callback
	_addChildResourcesAndShow: function(response) {
		this._addChildResources(response, true);
		this._decCount();
	},
	// callback
	_addChildResourcesAndHide: function(response) {
		this._addChildResources(response, false);
		this._decCount();
	},

	_addChildResources: function(response, fShow) {
		var results = this._getResourcesFromResult(response);
		JazzUIWorkbench.hideMessageArea();
		if (results != null) {
			this._performSort(results);
			for (var i = 0; i < results.length; i++) {
				var resource = results[i];
				if (this._childMap[resource.parentUUID] == null)
					this._childMap[resource.parentUUID] = new Array();
				this._childMap[resource.parentUUID].push(resource.name.value);
//				Locate the parent to insert it under
				if (resource.parentUUID == this._rootFolder.folderUUID || resource.parentUUID == "" || response.nonpaResources) { //$NON-NLS-1$
					this._table.tbody.appendChild(this._createResourceRow(resource, 29, fShow));
					this._rootFolder.children.resources.push(resource);
				} else {
					var parentFolder = this.getTr(resource.parentUUID);
					if (parentFolder == null) {
//						Couldn't find the folder, so must be a root folder
						this._table.tbody.appendChild(this._createResourceRow(resource, 29, fShow));
						this._rootFolder.children.resources.push(resource);
					} else {
						this._showExpando(parentFolder, true);
						var parentUUID = resource.parentUUID;
						this._insertAfter(this._createResourceRow(resource, parentFolder._indent + INDENT2, fShow), this._getResourceInsertionPoint(parentUUID));
						this._folders[parentUUID].children.resources.push(resource);
					}
				}
			}
		}
	},

	_getResourceInsertionPoint: function(folderUUID) {
		var folderTRNode = this.getTr(folderUUID);
		var candidate = folderTRNode;
		var nextCandidate = candidate.nextSibling;
		while (true) {
			if (nextCandidate == null) {
				return candidate;
			}
			if (nextCandidate._rowType == ROWTYPE_FOLDER || nextCandidate._indent <= folderTRNode._indent + (INDENT2 - INDENT)){
				return candidate;
			}
			candidate = nextCandidate;
			nextCandidate = nextCandidate.nextSibling;
		} 
	},

	_createResourceRow: function(resource, padding, fShow) {
		var listRow = new ItemRow({
			rowType: this.ROWTYPE_RESOURCE,
			iconSrc: this._resourceIcon,
			item: resource, 
			padding: padding, 
			fShow: fShow,
			_handleCheckBoxClick: dojo.hitch(this, this._handleResourceCheckBoxClick),
			onResourceSelected: dojo.hitch(this, this.onResourceSelected)
		});
		if(resource.name.value == this._previousSelectedResourceUri) {
			this._prevSelectedRow = listRow;
		}
		this._initializeResourceActions(listRow.item, listRow.actionBar);
		return listRow.domNode;
	},

	_initializeResourceActions: function(resource, containerNode) {
		var messages = this._messages;
		var toolbar = new com.ibm.relm.explorer.web.explorer.page.Toolbar({});
		if(this.shared) {
			if(this.permissions.modifyPermission)
				toolbar.addButton(this, dojo.hitch(this, "_editResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/edit.gif"), label: messages.edit}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if(this.permissions.deletePermission)
				toolbar.addButton(this, dojo.hitch(this, "_deleteResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/delete.gif"), label: this._label_deleteResource}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			// always enable copy/move button
			toolbar.addButton(this, dojo.hitch(this, "_moveResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/movecopy.gif"), label: this._label_moveResource}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} else {
			toolbar.addButton(this, dojo.hitch(this, "_editResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/edit.gif"), label: messages.edit}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			toolbar.addButton(this, dojo.hitch(this, "_deleteResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/delete.gif"), label: this._label_deleteResource}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			toolbar.addButton(this, dojo.hitch(this, "_moveResource", resource), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/movecopy.gif"), label: this._label_moveResource}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		containerNode.appendChild(toolbar.domNode);
	},

	_createFolderRow: function(folder, padding, fShow) {
		var listRow = new FolderRow({
			item: folder, 
			padding: padding, 
			fShow: fShow,
			_handleFolderColExp: dojo.hitch(this, this._handleFolderColExp),
			_handleFolderCheckBoxClick: dojo.hitch(this, this._handleFolderCheckBoxClick),
			_expanded: (this._colexpState[folder.folderUUID]) ? "true" : "false" //$NON-NLS-1$ //$NON-NLS-2$,
		});
		this._initializeFolderActions(listRow.item, listRow.actionBar);
		return listRow.domNode;
	},

	_initializeFolderActions: function(folder, containerNode) {
		var messages = this._messages;
		var toolbar = new com.ibm.relm.explorer.web.explorer.page.Toolbar({});
		if(this.shared){
			if(this.permissions.createPermission)
				toolbar.addButton(this, dojo.hitch(this, "_createNewFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/new-folder-obj.gif"), label: messages.createFolder}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if(this.permissions.modifyPermission)
				toolbar.addButton(this, dojo.hitch(this, "_editFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/edit.gif"), label: messages.edit}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if(this.permissions.deletePermission) 
				toolbar.addButton(this, dojo.hitch(this, "_deleteFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/delete.gif"), label: messages.deleteMsg}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			// always enable copy/move button
			toolbar.addButton(this, dojo.hitch(this, "_moveFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/movecopy.gif"), label: messages.moveFolder}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} else {
			toolbar.addButton(this, dojo.hitch(this, "_createNewFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/new-folder-obj.gif"), label: messages.createFolder}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			toolbar.addButton(this, dojo.hitch(this, "_editFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/edit.gif"), label: messages.edit}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			toolbar.addButton(this, dojo.hitch(this, "_deleteFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/delete.gif"), label: messages.deleteMsg}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			toolbar.addButton(this, dojo.hitch(this, "_moveFolder", folder), {imgSrc: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/movecopy.gif"), label: messages.moveFolder}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		containerNode.appendChild(toolbar.domNode);
	},	

	_insertAfter: function(newNode, referenceNode) {
		referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
	},

	/**
	 * Toggles showing of the expando on a folder row
	 * @param  {Tr} folderRow       The folder row with the expando
	 * @param  {boolean} condition True indicates showing, False indicates hiding
	 */
	_showExpando: function(folderRow, condition) {
		dojo.toggleClass(folderRow, 'noChildren', !condition); //$NON-NLS-1$
	},

	_handleFolderColExp: function(args) {
		var folderRow = args.currentTarget.parentNode.parentNode;
		var fShow = (folderRow._expanded == "false") ? "true" : "false"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		folderRow._expanded = fShow;
		this._collapseExpandFolder(folderRow, fShow);
		args.currentTarget.src = (fShow == "true") ?  this._collapseIcon : this._expandIcon ; //$NON-NLS-1$
	},

	_collapseExpandFolder: function(folderRow, fShow) 
	{
		var self = this;
		self._colexpState[folderRow.getAttribute('data-guid')] = !self._colexpState[folderRow.getAttribute('data-guid')]; //$NON-NLS-1$ //$NON-NLS-2$
		dojo.forEach(this._childMap[folderRow.getAttribute('data-guid')], function(childID) { //$NON-NLS-1$
			var childRow = this.getTr(childID); 
			childRow.style.display = (fShow == "true") ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ 
			if(childRow._rowType == ROWTYPE_FOLDER) { 
				if(childRow._expanded == "true") { //$NON-NLS-1$
					self._collapseExpandFolder(childRow, fShow);
					
				}
			}
				
		}, this);
	},

	_handleResourceCheckBoxClick: function(args)
	{
		var checkedState = args.currentTarget.checked;

		//if we unchecked a report then make sure the folder is unchecked also
		if (checkedState == false)
			this._setRowCheckedState(args.currentTarget.parentNode.parentNode.folderID, false);

		this._updateActions();
	},
	
	_handleFolderCheckBoxClick: function(args)
	{
		var checkedState = args.currentTarget.checked;
		var folderID = args.currentTarget.parentNode.parentNode.getAttribute('data-guid'); //$NON-NLS-1$
		this._handleFolderCheck(folderID, checkedState);
		this._updateActions();
	},
	
	_handleFolderCheck: function(folderID, checkedState)
	{
		var childIDs = this._childMap[folderID];
		for (var i = 0; i < childIDs.length; i++)
		{
			var childID = childIDs[i];
			this._setRowCheckedState(childID, checkedState);
			if (this._childMap[childID] != null)   //Recurse child nodes
				this._handleFolderCheck(childID, checkedState);
		}
	},
	
	_getCheckboxFromTR: function(trNode)
	{
		if(trNode._rowType == "folder") { //$NON-NLS-1$
			return trNode.children[0].children[1];  //First child is the <td> second child of that is the input (checkbox);
		} else {
			return trNode.children[0].children[0];  //First child is the <td> first child of that is the input (checkbox);
		}
	},
	
	_isRowChecked: function(trNode)
	{
		return this._getCheckboxFromTR(trNode).checked;
	},
	
	_setRowCheckedState: function(rowID, checkedState)
	{
		var tr = this.getTr(rowID);
		if(!tr) return;
		var cb = this._getCheckboxFromTR(tr);
		cb.checked = checkedState;
	},
	
	_updateActions: function()
	{
		var buttonCounter = 0;
		var button = null;
		while((button = this.metanav._toolbar.getItem(buttonCounter++)) != null)
		{
			this._updateRequireSelection(button);

			if(this.shared) {
				if(button.args.requireCreate && !this.permissions.createPermission) {
					this.metanav._toolbar.disableButton(button);
				}
				if(button.args.requireModify && !this.permissions.modifyPermission) {
					this.metanav._toolbar.disableButton(button);
				}
				if(button.args.requireDelete && !this.permissions.deletePermission) {
					this.metanav._toolbar.disableButton(button);
				}
			}
		}
	},
	
	_updateRequireSelection: function(button) 
	{
		if (button.args.requiresSelection)
		{
			var rowsChecked = this._getCheckedCount() > 0;
			if (rowsChecked)
				this.metanav._toolbar.enableButton(button);
			else
				this.metanav._toolbar.disableButton(button);
		}
		
	},

	/*
	 * actions
	 */
	_confirmToDelete: function(folderItemIDs, resourceItemIDs){
		return true; // ok to delete
	},
	
	_deleteItems: function()
	{
		// subclass should implement
	},

	_moveItems: function()
	{
		var items = this._getCheckedItems(true);
		if (items.length <= 0)
			return;
			
		var resourceURIs = [];
		var resourceNames = [];
		var folderUUIDs = [];
		var folderNames = [];

		for (var i = 0; i < items.length; i++)
		{
			var item = items[i];
			if (item.rowType == this.ROWTYPE_RESOURCE) 
			{
				resourceURIs.push(item.id);
				resourceNames.push(item.name);
			}
			else if (item.rowType == ROWTYPE_FOLDER)
			{
				folderUUIDs.push(item.id);
				folderNames.push(item.name);
			} 
		}
		
		var args = {
			folderUUIDs: folderUUIDs,
			folderNames: folderNames,
			resourceURIs: resourceURIs,
			resourceNames: resourceNames,
			shared: this.shared,
			permissions: this.permissions,
			
			parentUUID: null,

			okCallback: dojo.hitch(this,this._okDialogCallback),
			cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		};
		var moveResource = new this._moveCopyDialog(args);
		
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveResource.mainDiv,
			primaryTitle: (items.length > 1) ? this._label_moveResources : this._label_moveResource,
			width: "300px", //$NON-NLS-1$
			isClosable: true
		});
		
		
	},

	_resourcesDeleted: function(args)
	{
		this._deletedTypes--;
		if (this._deletedTypes <= 0)
			this._reloadResources();
	},
	
	_newFolder: function() {
		this._createNewFolder(null);
	},

	_createNewFolder: function(folder) {
		var isShared = (this.shared == true) ? "true" : "false"; //$NON-NLS-1$ //$NON-NLS-2$
		var newLocation = this.editFolderAction + "&mode=create&folderType=" + this._folderType + /*"&processAreaUUID=" + folder.processAreaUUID +*/ "&shared=" + isShared; //$NON-NLS-1$ //$NON-NLS-2$
		if(folder){
			 newLocation = newLocation + "&parentUUID=" + folder.folderUUID;//$NON-NLS-1$ 
		}
		window.location.href = newLocation; 
	},
	
	_editFolder: function(folder) {
		var isShared = (this.shared == true) ? "true" : "false"; //$NON-NLS-1$ //$NON-NLS-2$
		window.location.href = this.editFolderAction + "&mode=modify&folderType=" + this._folderType + "&folderUUID=" + folder.folderUUID + "&shared=" + isShared; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
	},

	_newResource: function() {
		// subclass should implement
	},

	_editResource: function(r) {
		// subclass should implement
	},

	_filterDeletedFolder: function(deleted, root){
		if(deleted.parentUUID == root.folderUUID){
			root.children.folders = dojo.filter(root.children.folders, function(item){
		    	return item.name.value != deleted.name.value;
			});
		}
		else if(root.children.folders.length >0){
				var self = this;
				dojo.forEach(root.children.folders, function(entry,i){
    				entry = self._filterDeletedFolder(deleted,entry);
  				});
		}
		return root;
	},
	
	_deleteFolder: function(folder) {
		var messages = this._messages;
		if (!confirm(dojo.string.substitute(messages.confirmDelFolder, [folder.name.label]))) {
			return;
		}
		
		if(! this._confirmToDelete([folder.folderUUID], [])){
			return;
		}

		var args = {
				folderUUID: folder.folderUUID
		};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_folderDeleted", //$NON-NLS-1$
		"_remoteError"); //$NON-NLS-1$
		var rowNode = this.getTr(folder.folderUUID); 
		if(rowNode){
			this._pendingDeleteFolderNode = rowNode;
		}
//		find all of the child nodes
		this._pendingDeleteFolderChildrenNodes = new Array();
		this._addAllChildResourcesToFolderDelete(folder.folderUUID);
		this._serviceClient.postDeleteFolderDescriptor(responseHandler, folder.folderUUID, args);
		this._rootFolder = this._filterDeletedFolder(folder, this._rootFolder);
	},

	_addAllChildResourcesToFolderDelete: function(uuid){
		var folderChildren = this._childMap[uuid];
		if(folderChildren){
			for(var i = 0; i < folderChildren.length; i++){
				var child = this.getTr(folderChildren[i]); 
				if(child){
					this._pendingDeleteFolderChildrenNodes.push(child);
					this._addAllChildResourcesToFolderDelete(child.getAttribute('data-guid')); //$NON-NLS-1$
				}
			}
		}
	},

	_folderDeleted: function() {
		if(this._pendingDeleteFolderNode && this._pendingDeleteFolderNode != null){
			this._table.tbody.removeChild(this._pendingDeleteFolderNode);
			this._pendingDeleteFolderNode = null;
		}
		if(this._pendingDeleteFolderChildrenNodes && this._pendingDeleteFolderChildrenNodes != null){
			for(var i = 0; i < this._pendingDeleteFolderChildrenNodes.length; i++){
				this._table.tbody.removeChild(this._pendingDeleteFolderChildrenNodes[i]);
			}
			this._pendingDeleteFolderChildrenNodes = null;
		}
		console.debug("_folderDeleted"); //$NON-NLS-1$
		this._updateActions();
	},
	
	_filterDeletedResource: function(deleted, root){
		if(deleted.parentUUID == root.folderUUID){
			root.children.resources = dojo.filter(root.children.resources, function(item){
		    	return item.name.value != deleted.name.value;
			});
		}
		else if(root.children.folders.length >0){
				var self = this;
				dojo.forEach(root.children.folders, function(entry,i){
    				entry = self._filterDeletedResource(deleted,entry);
  				});
		}
		return root;
	},
	
	_deleteResource: function(r) {
		// subclass should implement
	},
	
	_resourceDeleted: function() {
		if(this._pendingDeleteResourceNode && this._pendingDeleteResourceNode != null){
			this._table.tbody.removeChild(this._pendingDeleteResourceNode);
			this._pendingDeleteResourceNode = null;
		}
		console.debug("_resourceDeleted"); //$NON-NLS-1$
		this._updateActions();
	},
	
	_moveResource: function(resource)
	{
		var resourceURIs = [resource.name.value];
		var resourceNames = [resource.name.label];
		var args = {
				resourceURIs: resourceURIs,
				resourceNames: resourceNames,
				shared: this.shared,
				permissions: this.permissions,
				parentUUID: (this.permissions.createPermission && (resource.parentUUID != null)) ? resource.parentUUID : "", //$NON-NLS-1$
						okCallback: dojo.hitch(this,this._okDialogCallback),
						cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		};
		var moveResource = new this._moveCopyDialog(args);
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveResource.mainDiv,
			primaryTitle: this._label_moveResource,
			width: "400px", //$NON-NLS-1$
			isClosable: true
		});
	},
	
	_moveFolder: function(folder)
	{
		var messages = this._messages;
		var folderUUIDs = [folder.folderUUID];
		var folderNames = [folder.name.label];
		var args = {
				folderUUIDs:folderUUIDs,
				folderNames: folderNames,
				shared: this.shared,
				permissions: this.permissions,
				parentUUID: (this.permissions.createPermission && (folder.parentUUID != null)) ? folder.parentUUID : "", //$NON-NLS-1$
						okCallback: dojo.hitch(this,this._okDialogCallback),
						cancelCallback: dojo.hitch(this,this._cancelDialogCallback)
		};
		var moveResource = new this._moveCopyDialog(args);
		this._dialog = new jazz.ui.Dialog({
			contentNode: moveResource.mainDiv,
			primaryTitle: messages.moveFolder,
			width: "300px", //$NON-NLS-1$
			isClosable: true
		});
	},

	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterState = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _reloadResources or setTimeout will pass some other value
		if (!this._loadTimeout) {
			this._loadTimeout = setTimeout(dojo.hitch(this,this._reloadResources,0,true),500);
		}
	},
	
	_okDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
		var needRefresh = false;
			
		if((args.targetShared && this.scope == "shared") || (!args.targetShared && this.scope == "personal")) { //$NON-NLS-1$ //$NON-NLS-2$
			// target is same scope  -  refresh the current resource list only
			needRefresh = true;
		} else {
			// target is different scope  -  refresh the current if the operation is move, and refresh target resource list when it is shown next
			needRefresh = args.isMove;
			// enable refresh flag for another scope view list
			this.page.setListDirty((args.targetShared)?"shared":"personal"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(needRefresh)	this._reloadResources();
	},
	
	_cancelDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
	},

	_remoteError: function(error) {
		JazzUIWorkbench.showMessage(error.message);
	},

	_getCheckedItems: function(excludeChildrenOfSelectedFolders)
	{
		var retVal = [];
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this._isRowChecked(row))
			{
				var selectedItem = {};
				selectedItem.rowType = row._rowType;
				selectedItem.id = row.getAttribute('data-guid'); //$NON-NLS-1$
				selectedItem.name = row._name;
				retVal.push(selectedItem);
				//If we want to exclude children of selected folders
				if (excludeChildrenOfSelectedFolders && row._rowType == ROWTYPE_FOLDER)
				{
					//skip all the subsequent rows until the next sibling
					var indent = row._indent;
					while ((i+1 < tableRows.length) && 
							(((tableRows[i+1]._rowType == ROWTYPE_FOLDER)? tableRows[i+1]._indent : tableRows[i+1]._indent - INDENT) > indent))
						i++;
				}
			}
		}
		return retVal;
	},
		
	_getCheckedCount: function()
	{
		var retVal = 0;
		var body = this._table.tbody;
		var tableRows = body.children;
		for (var i = 0; i < tableRows.length; i++)
		{
			var row = tableRows[i];
			if (this._isRowChecked(row))
				retVal++;
		}
		return retVal;
	},
	
	requestRefresh: function(){
		this._refreshRequired = true;
	},
	
	// tool-bar actions
    onShown: function() {
		var title = this.shared ? this._label_SharedResources : this._label_MyResources;
		JazzUIWorkbench.setTitle(title);
		
    	if(this._refreshRequired) {
    		this._emptyTable();
    		this._refreshContent();
    		this._refreshRequired = false;
    	} else {
    		// just change selected tab
    		this.refreshTabs();
    	}
    },

    onResourceSelected: function(target){
		// subclass should implement
	}
});
})();
