/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Textarea"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ViewSelect"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 
var ACTION = {
		COMMAND: "command", //$NON-NLS-1$
		LINK: "link" //$NON-NLS-1$
	};
var ARG_VIEW_ID = "viewId"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog", [dijit._Widget, dijit._Templated] , { //$NON-NLS-1$
	resourceNode: null, // target resource node

	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ActionEditDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	messages:null,
	
	_argsRow: null,
	
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		var actionCommands = [];
		if(this.resourceNode){
			actionCommands = this.resourceNode.actions; 
		}
		// retrieve action commands
		dojo.forEach(actionCommands, function(cmd){
			var options = this.commandNameField.options;
			options[options.length] = new Option(cmd, cmd);
		}, this);
		this._commandNameChanged();
		// set existing action definition
		if(this.actionDef){
			this._setActionDef(this.actionDef);
		}
	},
	
	destroy: function(){
		this._clearArgsRow();
		this.inherited(arguments);
	},
	
	_actionTypeChanged: function(value){
		if(this.actionTypeField.value == ACTION.COMMAND){
			this.linkEdit.style.display = "none"; //$NON-NLS-1$
			this.commandEdit.style.display = ""; //$NON-NLS-1$
		}else{
			this.linkEdit.style.display = ""; //$NON-NLS-1$
			this.commandEdit.style.display = "none"; //$NON-NLS-1$
		}
	},
	
	_clearArgsRow: function(){
		dojo.forEach(this._argsRow, function(r){ r.argValueNode.destroy();});
		this._argsRow = [];
	},
	
	_commandNameChanged: function(){
		dojo.forEach(this._argsRow, function(r){
			this.argsTable.removeChild(r.domNode);
		}, this);
		this._clearArgsRow();
		var commandName = this.commandNameField.value;
		if(this.resourceNode){
			var method = this.resourceNode[commandName];
			if(method){
				var args = null;
				var resourceNode = this.resourceNode;
				if(resourceNode && dojo.isFunction(resourceNode.getActionInfo)){
					args = resourceNode.getActionInfo(commandName);
				}
				if(!args){
					var methodStr = method.toString();
					methodStr = methodStr.replace(/((\/\/.*$)|(\/\*.*\*\/))/mg, ''); // remove comments //$NON-NLS-1$
					methodStr = methodStr.match(/^function\s*[^\(]*\(\s*([^\)]*)\)/m);
					args = dojo.map(methodStr[1].split(/,/), function(a, index){
						return this.messages.argumentPrefix + index;
					}); 
				}
				
				dojo.forEach(args, function(arg){
					arg = dojo.trim(arg);
					var argId = dijit.getUniqueId(arg+"TextBox"); //$NON-NLS-1$
					var tr = dojo.create("tr", null, this.argsTable); //$NON-NLS-1$
					var labelTd = dojo.create("td", { style: "white-space: nowrap; padding-bottom: 0.5em;", "class": "j-label"}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					dojo.create('label', {'for':argId, innerHTML: arg, "class": "j-label"},labelTd); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					var td = dojo.create("td", {style: "width: 100%; padding-left: 10px; padding-bottom: 0.5em;"}, tr); //$NON-NLS-1$ //$NON-NLS-2$
					var valueField = null;
					if (arg == ARG_VIEW_ID) {
						valueField = new com.ibm.relm.explorer.web.ui.internal.view.common.ViewSelect({'id':argId, showId: true, readOnly: false}, dojo.create("div", {style: "width: 100%;"}, td)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					} else {
						valueField = new dijit.form.Textarea({'id':argId, style: "width: 100%;"}, dojo.create("div", null, td)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
					this._argsRow.push({domNode: tr, argName: arg, argValueNode: valueField});
				}, this);
			}
		}
	},
	
	_setActionDef: function(actionDef){
		this.actionLabelField.value = actionDef.label;
		if(actionDef.action){
			this.actionTypeField.value = ACTION.COMMAND;
			this._actionTypeChanged();
			var methodStr = actionDef.action.match(/^\s*([^\(]*)\((.*)\)[^\)]*/m); 
			if(methodStr && methodStr.length > 2){
				var argVals = [];
				var commandName = methodStr[1]; 
				try {
					argVals = dojo.fromJson("[" + methodStr[2] + "]"); //$NON-NLS-1$ //$NON-NLS-2$
				} catch(e) {
					// open dialog even on error
					argVals[0] = methodStr[2];
				}
				this.commandNameField.value = commandName;
				this._commandNameChanged();
				var max = Math.min(this._argsRow.length, argVals.length);
				for(var i=0; i<max; i++){
					var val = argVals[i];
					if(val){
						if(!dojo.isString(val)) val = dojo.toJson(val);
						this._argsRow[i].argValueNode.set("value", val); //$NON-NLS-1$
					}
				}
			}
		}
		this.defaultActionField.checked = actionDef["default"]; //$NON-NLS-1$
	},
	
	getValues: function(){
		var values = {
			name: this.actionLabelField.value,
			type: this.actionTypeField.value, 
			"default": this.defaultActionField.checked //$NON-NLS-1$
		};

		if(values.type == ACTION.COMMAND){
			var commandName = this.commandNameField.value;
			var val = commandName + "("; //$NON-NLS-1$
			var args = dojo.map(this._argsRow, function(a){
				var argval = dojo.trim(a.argValueNode.get("value")); //$NON-NLS-1$
				if(!argval) return "null"; //$NON-NLS-1$
				var resourceNode = this.resourceNode;
				if(resourceNode && dojo.isFunction(resourceNode.getActionInfo)){
					var actionInfo = resourceNode.getActionInfo(commandName, a.argName);
					if(actionInfo && actionInfo.types){
						var length = actionInfo.types.length;
						var actionInfoTypes = dojo.clone(actionInfo.types);
						var found = false;
						var value = dojo.trim(argval).toLowerCase();
						var idx = dojo.indexOf(actionInfoTypes, "array"); //$NON-NLS-1$
						if (idx >= 0) {
							found = value.length != 0 && 
							value.charAt(0) == "[" && value.charAt(value.length-1) == "]"; //$NON-NLS-1$ //$NON-NLS-2$
							if (!found) length--;
						}	
						if (!found) {
							idx = dojo.indexOf(actionInfoTypes, "boolean"); //$NON-NLS-1$
							if (idx >= 0) {
								if (value == "true" || value == "false") { //$NON-NLS-1$ //$NON-NLS-2$
									found = true;
								}
								if (!found) length--;
							}
							if(length == 1 && dojo.indexOf(actionInfoTypes, "string") >= 0){ //$NON-NLS-1$
								// argument should be text which surrounded by double quote
								argval = dojo.toJson(argval);
							}
						}
					} 
				}
				return argval; 
			}, this);
			var argsStr = args.join(","); //$NON-NLS-1$
			val += argsStr + ")"; //$NON-NLS-1$
			values.value = val;
		}else{
			values.value = this.urlValueField.value;
		}
		return values;
	}
});

com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog.openDialog = function(actionDef, resourceNode, okCallback){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog({actionDef: actionDef, resourceNode: resourceNode});

	var dialogTitle = actionDef ? contentWidget.messages.configEditorEditAction : contentWidget.messages.AddAction;

	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: dialogTitle,
		onOpen: function(){
			dijit.focus(contentWidget.actionLabelField);
		},
		validate: function(values, callback){
			var actionLabel = values["name"]; //$NON-NLS-1$
			if(actionLabel && actionLabel.length>0){
				callback();
			}else{
				alert(contentWidget.messages.errActionLabelEmpty);
			}
		},
		onOK: function(values){
			if(!actionDef){
				actionDef = {};
			}
			actionDef.label = values["name"]; //$NON-NLS-1$
			var type = values["type"]; //$NON-NLS-1$
			var value = values["value"]; //$NON-NLS-1$
			if(type == ACTION.COMMAND) {
				actionDef.action = value;
			} else { // == ACTION.LINK
				actionDef.href = value;
			}
			actionDef["default"] = values["default"]; //$NON-NLS-1$ //$NON-NLS-2$
			okCallback(actionDef);
		}
	});
};

})();