/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 
	
dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent", [dijit._Widget] , { //$NON-NLS-1$

	messages:null,
	propTable: null,
	linkType: null,
	linkTypeDropdown: null,
	propContent: null,

	constructor: function(args) {

		this.propContent = this;
		
		this.linkType = args.linkType;
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

		var table = dojo.create("table", { //$NON-NLS-1$
			style: {
				padding: "5px", //$NON-NLS-1$
				width: "100%" //$NON-NLS-1$
			}
		});
		var trLinkType = dojo.create("tr", {}, table); //$NON-NLS-1$
		var tdLinkTypeHeading = dojo.create("td", { //$NON-NLS-1$
			style: {
				whiteSpace: "nowrap", //$NON-NLS-1$
				paddingBottom: "0.5em" //$NON-NLS-1$
			}
		}, trLinkType);
		var typeFieldId = "linkTypeDD"; //$NON-NLS-1$
		dojo.create("label", { //$NON-NLS-1$
			className: "j-label", //$NON-NLS-1$
			"for": typeFieldId, //$NON-NLS-1$
			innerHTML: this.messages.statusLinkLabel
		}, tdLinkTypeHeading);
		dojo.create("span", { //$NON-NLS-1$
			className: "j-required", //$NON-NLS-1$
			innerHTML: "*" //$NON-NLS-1$
		}, tdLinkTypeHeading);

		var tdLinkTypeValue = dojo.create("td", { //$NON-NLS-1$
			style: {
				width: "100%", //$NON-NLS-1$
				paddingLeft: "10px", //$NON-NLS-1$
				paddingBottom: "0.5em" //$NON-NLS-1$
			}
		}, trLinkType);
		
		this.linkTypeDropdown = new dijit.form.ComboBox({
			name: "linkType",         //$NON-NLS-1$
			searchAttr: "name",        //$NON-NLS-1$
			'aria-required':'true',		//$NON-NLS-1$ //$NON-NLS-2$
			style: {width: "100%"}        //$NON-NLS-1$
		}, dojo.create("select", null, tdLinkTypeValue));//$NON-NLS-1$

		this.loadLinkTypeToDropdown();

		dojo.place(table, this.domNode, "first"); //$NON-NLS-1$

		this.connect(this.linkTypeDropdown, "onChange", function() {  //$NON-NLS-1$
			var selectedVal = dojo.getObject("linkTypeDropdown.item.value", false, this);//$NON-NLS-1$
			if(selectedVal && selectedVal.length > 0){
				this.linkType = selectedVal[0];
			}else{
				this.linkType = this.linkTypeDropdown.get("value");//$NON-NLS-1$
			}
		});
	},
	
	loadLinkTypeToDropdown: function () {
		com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getConnectionTypeItems(dojo.hitch(this, function(linkTypes){
			var linkTypeStore = new dojo.data.ItemFileReadStore({
				data: {
	            	identifier: "value",//$NON-NLS-1$
	            	label: "name",//$NON-NLS-1$
					items: linkTypes
				}
			});

			this.linkTypeDropdown.set("store", linkTypeStore);//$NON-NLS-1$				
			this.linkTypeDropdown.set("value", this.linkType || ""); //$NON-NLS-1$//$NON-NLS-2$
		}), true /* usePrefixedUri */);
	},
	
	getValues: function(){
		return {
			linkType: this.linkType
		};
	}

});

com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent.openDialog = function(parent, index, callback2){

	var linkType = (index == -1) ? null : parent.statusLinks[index].type;
	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent(
			{ linkType: linkType});
	
	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.messages.statusLinkDialog,
		onOpen: function(){
			dijit.focus(contentWidget.linkTypeDropdown.domNode);
		},
		validate: function(values, callback){
			var type = values["linkType"]; //$NON-NLS-1$
			if(type && type.length>0) {
				callback();
			} else{
				alert(contentWidget.messages.errLinkTypeEmpty);
			}
		},
		onOK: function(values){
			var fModified = false;
			if(index >= 0) {
				// edit the status link type
				if(parent.statusLinks[index].type == values["linkType"]) {        //$NON-NLS-1$
					
					fModified = false;
				} else {
					fModified = true;
					parent.statusLinks[index].type = values["linkType"]; //$NON-NLS-1$
				}
				
			} else {
				// add new target props
				fModified = true;
				var link = {type: values["linkType"]};        //$NON-NLS-1$
				parent.statusLinks.push(link); 
			}
			
			if(callback2) {
				callback2(fModified);
			}
		}
	});
};

})();
