/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 
	
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_INLINE = "dijitInline";  //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent", [dijit._Widget] , { //$NON-NLS-1$

	messages:null,
	propTable: null,
	props: null,
	propContent: null,
	nameOptions: null,

	constructor: function(args) {

		this.props = [];
		this.propContent = this;
		
		var targetProps = args.target;
		for(var i = 0; i < targetProps.length; i++) {
			// copy props
			var prop = targetProps[i];
			this.props.push({name: prop.name,
				value: prop.value});
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		var self = this;

		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

		var div = dojo.create("div", { //$NON-NLS-1$
			"class" : "relm-config-form-editor-target-dialog" //$NON-NLS-1$ //$NON-NLS-2$
		}); 
		
		var divTable = dojo.create("div",   { //$NON-NLS-1$
			style: {
				padding: "5px", //$NON-NLS-1$
				width: "100%" //$NON-NLS-1$
			}
		}, div);
		
		var divHeading = dojo.create("div", { //$NON-NLS-1$
			"class": "heading"   //$NON-NLS-1$ //$NON-NLS-2$
		}, divTable);
		var divToolbar = dojo.create("div", { //$NON-NLS-1$
			"class": "toolbar"   //$NON-NLS-1$ //$NON-NLS-2$
		}, divHeading);
		var divHeadingTitle = dojo.create("div", { //$NON-NLS-1$
			role: "heading", //$NON-NLS-1$
			innerHTML: this.messages.targetPropertyTitle
		}, divHeading);
		divToolbar.appendChild(dojo.create("a", {  //$NON-NLS-1$
			href: "#",  //$NON-NLS-1$
			innerHTML: this.messages.Add,
			title: this.messages.AddTargetProperty,
			style: {
				verticalAlign: "middle",  //$NON-NLS-1$
				marginLeft: "8px"  //$NON-NLS-1$
			},
			onclick: function(e){
				dojo.stopEvent(e);
				self._addProperty();
			}
		}));
	
		this.propTable = new com.ibm.relm.explorer.web.ui.internal.view.common.Table({
			style: {width: "100%"},   //$NON-NLS-1$
			selectable: "true"        //$NON-NLS-1$
		});

		var typeItems = {items: this.nameOptions || []};
		var nameStore = new dojo.data.ItemFileReadStore({data: typeItems});
		this.propTable.setupColumns([
		                        {
		                        	id: "name",  //$NON-NLS-1$
		                        	name: this.messages.configEditorPropertyName,
		                        	getCellContent: function(cellData) {
		                        		var literalId = this.data.id;
		                        		var rowWidget = this;
		                        		var editBox = new dijit.InlineEditBox({
		                        			title : cellData,
		                        			width : "200px",  //$NON-NLS-1$
		                        			editor: self.nameOptions ? "dijit.form.ComboBox" : "dijit.form.TextBox", //$NON-NLS-1$ //$NON-NLS-2$
                        					editorParams: {
	                       						store : nameStore,
	                       						searchAttr : "name", //$NON-NLS-1$
	                       						labelAttr : "label" //$NON-NLS-1$
                        					},
                        					onChange : function(value) {
			            		    			if (!value || value.length === 0) {
			            		    				// field empty
			            		    				alert(this.messages.configEditorEmptyValue);
			            		    				this.set("value", rowWidget.data.name); //$NON-NLS-1$
			            		    					
			            		    			} else {
			            		    				self.updatePropertyName(rowWidget.data.name, value);
			            		    			}
		                        			}
		                        		}, dojo.create("div", {  //$NON-NLS-1$
		                        			className : CLASS_INLINE,
		                        			innerHTML : cellData
		                        		}));
		                        		dojo.style(editBox.domNode, {
		                        			width : "200px"  //$NON-NLS-1$
		                        		});
		                        		return editBox.domNode;
		                        	}

		                        },
		                        {
		                        	id: "value",  //$NON-NLS-1$
		                        	name: this.messages.configEditorPropertyValue,
		                        	getCellContent: function(cellData) {
		                        		var literalId = this.data.id;
		                        		var rowWidget = this;
		                        		var editBox = new dijit.InlineEditBox({
		                        			title : cellData,
		                        			width : "200px",  //$NON-NLS-1$
		                        			onChange : function(value) {
			            		    			if (!value || value.length === 0) {
			            		    				// field empty
			            		    				alert(this.messages.configEditorEmptyValue);
			            		    				this.set("value", rowWidget.data.value); //$NON-NLS-1$
			            		    					
			            		    			} else {
			            		    				self.updatePropertyValue(rowWidget.data.name, value);
			            		    			}

		                        			}
		                        		}, dojo.create("div", {  //$NON-NLS-1$
		                        			className : CLASS_INLINE,
		                        			innerHTML : cellData
		                        		}));
		                        		dojo.style(editBox.domNode, {
		                        			width : "200px"  //$NON-NLS-1$
		                        		});
		                        		return editBox.domNode;
		                        	}
		                        },
		                        {
		                        	id: "actions",  //$NON-NLS-1$
		                        	name: this.messages.configEditorAction,
		                        	isAction: true
		                        }

		                        ]);
		divTable.appendChild(this.propTable.domNode);

		dojo.place(div, this.domNode, "first"); //$NON-NLS-1$
		
		this.setParams(this.props);
	},
	
	setParams: function(params) {
		var propTable = this.propTable;
		propTable.clearItems();

		var self = this;
		if(params) {
			for(var i = 0; i < params.length; i++) {
				var param = params[i];
				var name = param.name;
				var value = param.value;
				propTable.addItem({name : name, 
									value: value,
									index: i,
									actions: [{
										iconClass: CLASS_DELETE_COMMAND,
										title: self.messages.configEditorRemoveConditions,
										onClick: function(e){
											if(name){
												if(confirm(dojo.string.substitute(self.messages.confirmRemoveConnectionParameter, [ this.data.name ]))){
													// remove parameter
													self.props.splice(this.data.index, 1);
													self.setParams(self.props);
												}
											}
										}
									}]
									
				});
			}
		}

	},

	_addProperty: function() {
		// add new property
		this.props.push({name: this.messages.NewPropertyName, 
			value: this.messages.NewPropertyValue});
		this.setParams(this.props);
	},
	
	updatePropertyName: function(oldName, newName) {
		for(var i = 0; i < this.props.length; i++) {
			if(this.props[i].name == oldName) {
				this.props[i].name = newName;
				break;
			}
		}
		this.propContent.setParams(this.props);

	},
	
	updatePropertyValue: function(name, value) {
		for(var i = 0; i < this.props.length; i++) {
			if(this.props[i].name == name) {
				this.props[i].value = value;
				break;
			}
		}
		this.propContent.setParams(this.props);

	},
	
	getValues: function(){
		return {
			props: this.props
		};
	}

});

com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent.openDialog = function(parent, index, callback2, contentParams){

	var targetProp = (index == -1) ? [] : parent.targetProps[index];
	var cp = dojo.mixin({ target: targetProp}, contentParams);

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent(cp);
	
	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		width: "",//$NON-NLS-1$
		dialogContent: contentWidget,
		primaryTitle: contentWidget.messages.targetPropertyDialog,
//		onOpen: function(){
//			dijit.focus(contentWidget.paramNameField);
//		},
		validate: function(values, callback){
			var fEmpty = false;
			var props = values["props"]; //$NON-NLS-1$
			dojo.forEach(props, function(prop) {
				if(prop.name == "") fEmpty = true; //$NON-NLS-1$
			});
			if(!fEmpty){
				callback();
			}else{
				alert(contentWidget.messages.errPropertyNameEmpty);
			}
		},
		onOK: function(values){
			var fModified = false;
			if(parent.targetProps == null) {
				parent.targetProps = [];
			}
			if(index >= 0) {
				// edit the target props
				fModified = this.checkModified(parent.targetProps[index], values["props"]); //$NON-NLS-1$
				parent.targetProps[index] = values["props"]; //$NON-NLS-1$
				
			} else {
				// add new target props
				fModified = true;
				parent.targetProps.push(values["props"]); //$NON-NLS-1$
			}
			
			if(callback2) {
				callback2(fModified);
			}
		},
		checkModified: function(oldProps, newProps) {
			if(oldProps.length != newProps.length) return true;
			for(var i = 0; i < oldProps.length; i++) {
				if(oldProps[i].name != newProps[i].name) return true;
				if(oldProps[i].value != newProps[i].value) return true;
			}
			return false;
		}
	});
};

})();
