/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.ConfigFormEditor"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("dijit.layout.BorderContainer"); //$NON-NLS-1$
dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
dojo.require("dijit.Dialog"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$

dojo.require("jazz.client.xhr"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.General"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Query"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Layout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Resource"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Connection"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.Background"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.loader.Config"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var JazzUIWorkbench = PlatformUI.getWorkbench();
var util = com.ibm.relm.explorer.web.ui.internal.util;
var messages = {};

var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_ADD_BUTTON = "add-button"; //$NON-NLS-1$
var CLASS_REMOVE_BUTTON = "remove-button"; //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.editor.ConfigFormEditor", //$NON-NLS-1$
	[ dijit.layout.ContentPane, dijit._Templated ],
{
	// parameters
	standAloneMode: false, // show title and tool-bar (save, close, and switch to source) if true
	
	// internal variables
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ConfigFormEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
	initialSource: "", //$NON-NLS-1$
	configUpdated: false,
	_switchToSource: false,
	
	editModes: {},
	permissions: {},
	editMode: null,
	returnToShared: false,
	
	_tabs: [
		"general", //$NON-NLS-1$
		"query", //$NON-NLS-1$
		"layout", //$NON-NLS-1$
		"resource", //$NON-NLS-1$
		"connection", //$NON-NLS-1$
		"background", //$NON-NLS-1$
		"componentInfo"//$NON-NLS-1$
	],
	
	// methods
	save: function(){
		this._onSave();
	},
	
	cancelEdit: function(){
		this._onDiscard();
	},
	
	getConfig: function(){
		return this.config;
	},
	
	getConfigSource: function(){
		return dojo.toJson(this.config);
	},

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.messages = messages;

		// set edit mode by user type
		// temporary hard coded
		this.editModes = [
							{
								name: "Parameter Only", //$NON-NLS-1$
								values: 
								{
									common: {
										disableNameEdit: true,
										disableActionButton: true,
										hideEmptyTable: true
									},
									general: {
										generalSection: true,
										idRow: false,
										nameRow: false,
//										sharedRow: false,
										paramRow: true,
										descriptionRow: false,
										nodeLockRow: false,
										folderRow: false
									},
									query: {
										querySection: true,
										querySelectRow: true,
										queryContentRow: false,
										queryParamRow: true
									},
									layout: {
										layoutSection: false,
										layoutDescriptionRow: false
									},
									componentInfo: {
										componentInfoSection: false,
										componentInfoClassRow:false,
										componentInfoAttributeRow: false
									},
									resource: {
										resourceSection: true,
										resourceSelectRow: true,
										targetRow: false,
										statusLinkRow: false,
										componentTypeRow: true,
										componentParameterRow: true,
										hoverRow: false,
										actionRow: false
									},
									connection: {
										connectionSection: true,
										connectionSelectRow: true,
										titleRow: false,
										idRow: false,
										connectionTypeRow: true,
										linkTypeRow: true,
										queryIdRow: false,
										targetRow: false,
										connectionParameterRow: true,
										componentTypeRow: true,
										componentParameterRow: true
									},
									background: {
										backgroundSection: false,
										backgroundContentRow: false
									}
								}
							},
							{
								name: "All Edit Mode", //$NON-NLS-1$
								values: 
								{
									common: {
										disableNameEdit: false,
										disableActionButton: false,
										hideEmptyTable: false
									},
									general: {
										generalSection: true,
										idRow: true,
										nameRow: true,
//										sharedRow: true,
										paramRow: true,
										descriptionRow: true,
										nodeLockRow: true,
										folderRow: true
									},
									query: {
										querySection: true,
										querySelectRow: true,
										queryContentRow: true,
										queryParamRow: true
									},
									layout: {
										layoutSection: true,
										layoutDescriptionRow: true
									},
									componentInfo: {
										componentInfoSection: true,
										componentInfoClassRow:true,
										componentInfoAttributeRow: true
									},
									resource: {
										resourceSection: true,
										resourceSelectRow: true,
										targetRow: true,
										statusLinkRow: true,
										componentTypeRow: true,
										componentParameterRow: true,
										hoverRow: true,
										actionRow: true
									},
									connection: {
										connectionSection: true,
										connectionSelectRow: true,
										titleRow: true,
										idRow: true,
										connectionTypeRow: true,
										linkTypeRow: true,
										queryIdRow: true,
										targetRow: true,
										connectionParameterRow: true,
										componentTypeRow: true,
										componentParameterRow: true
									},
									background: {
										backgroundSection: true,
										backgroundContentRow: true
									}
								}
							}
						];
	},

	postCreate: function(){
		if(this.standAloneMode){
			this.headerNode.domNode.style.display = "";//$NON-NLS-1$
			dojo.addClass(this.formNode.domNode, "keyline");//$NON-NLS-1$
			dojo.addClass(this.navigatorNode.domNode, "keyline");//$NON-NLS-1$
		}

		this.inherited(arguments);

		// add tabs
		this._selectedTabName = "general"; //$NON-NLS-1$
		this._tabNodes = {};
		dojo.forEach(this._tabs, function(tabName){
			var tabNode = dojo.create("div", { //$NON-NLS-1$
				"class": "sectionMenu", //$NON-NLS-1$ //$NON-NLS-2$
				innerHTML: messages[tabName + "Section"]}, //$NON-NLS-1$
				this.navigatorNode.domNode);
			this._tabNodes[tabName] = tabNode;
			this.connect(tabNode, "onclick", function(){//$NON-NLS-1$
				this._selectTab(tabName);
			});
		}, this);

		this.connect(this.sourceArea, "oninput", "_hideMessage"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this.sourceArea, "onkeydown", "_hideMessage"); //$NON-NLS-1$ //$NON-NLS-2$

		// initialize state
		this.saveButton.disabled = true;

		this.editModeDropdown.domNode.style.display = "none"; //$NON-NLS-1$
		this.switchButton.style.display = "none"; //$NON-NLS-1$

	},
	
	setEditorTitle: function(title) {
		if(this.titleNode) {
			this.titleNode.innerHTML = title;
		}
	},

	setSource: function(source){
		this.initialSource = source;

	},
	
	setPermissions: function(permissions) {
		this.permissions = permissions;
	},
	
	setMessageArea: function(messageArea) {
		this.messageArea = messageArea;
	},
	
	setDiagramView: function(diagramView){
		this.diagramView = diagramView;
	},
	
	getDiagramEditor: function(){
		return this.diagramView ? this.diagramView._getDiagramEditor() : null;
	},
	
	setSelectedTab: function(tabInfo){
		if(this._tabNodes[tabInfo.tabName]){
			this._selectTab(tabInfo.tabName);
			if(this.content){
				var sectionWidget = this.content.getSectionWidget(this._selectedTabName);
				if(sectionWidget && sectionWidget.setSelected){
					sectionWidget.setSelected(tabInfo);
				}
			}
		}
	},
	
	setConfig: function(config) {
		if(this.content){
			this.content.destroy();
		}
		this.config = config;
		this.setEditorTitle(this.config.diagramName);


		this.editModeDropdown.options.length = 0;
		for(var i = 0; i < this.editModes.length; i++) {
			this.editModeDropdown.addOption({value: i, label: this.editModes[i].name});
		}

		var PARAM_EDIT = this.editModes[0];
		var ALL_EDIT = this.editModes[1];
		if(this.permissions.modifyPermission) {
			this.content = new com.ibm.relm.explorer.web.ui.internal.editor.ConfigEditor._Content({editor: this, editMode: ALL_EDIT.values});
			this.formNode.set("content", this.content); //$NON-NLS-1$
			this.setEditMode(ALL_EDIT.values);
			var showSrc = util.checkURLOption("editSrc"); //$NON-NLS-1$
			this.switchButton.style.display = showSrc ? "" : "none"; //$NON-NLS-1$//$NON-NLS-2$
			this._tabNodes["layout"].style.display = showSrc ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
			this._tabNodes["background"].style.display = ""; //$NON-NLS-1$ //$NON-NLS-2$
			this._tabNodes["componentInfo"].style.display = ""; //$NON-NLS-1$ //$NON-NLS-2$
		}

		this._selectTab(this._selectedTabName);
		this._clearDirty();
	},
	
	setEditMode: function(mode) {
		if(this.content) {
			this.content.setEditMode(mode);
			this.editMode = mode;
		}
		
		// show/hide navigation enu item
	},
	
	getSource: function() {
		return this.sourceArea.value;
	},
	
	_showAllSection: function() {
		
	},
	
	_selectTab: function(tabName) {

		var mode = this.editMode;
		if(mode) {
			this._hideAll();
			mode[tabName][tabName + "Section"] = true;//$NON-NLS-1$
			this.setEditMode(mode);
		}
		var oldSelectedNode = this._tabNodes[this._selectedTabName];
		if(oldSelectedNode) {
			dojo.toggleClass(oldSelectedNode, "selectedSectionMenu", false); //$NON-NLS-1$
		}
		this._selectedTabName = tabName;
		var newSelectedNode = this._tabNodes[tabName];
		dojo.toggleClass(newSelectedNode, "selectedSectionMenu", true); //$NON-NLS-1$
	},
	
	_hideAll: function(){
		var mode = this.editMode;
		mode.general.generalSection = false;
		mode.query.querySection = false;
		mode.layout.layoutSection = false;
		mode.componentInfo.componentInfoSection = false;
		mode.resource.resourceSection = false;
		mode.connection.connectionSection = false;
		mode.background.backgroundSection = false;
	},
	
	onClose: function() {
		this._hideMessage();
		if(this.configUpdated) {
			// need reload or return to view list in which the view is saved
			this.returnToShared = this.config.shared;
		}
	},
	
	onSaveCompleted: function() {
		this.messageArea.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: messages.configEditorMessageSaveCompleted,
			closable: true});
		this._clearDirty();
		this.configUpdated = true;
		this.setEditorTitle(this.config.diagramName);
	},
	
	_onClose: function(ev){
		this._switchToSource = false;
		if(this.isDirty()) {
			// ask discard changes
			this.closeConfirmDialog.show(); // ask with dialog
		} else {
			this.onClose();
		}
	},

	_onSwitch: function() {
		this._switchToSource = true;
		if(this.isDirty()) {
			// ask discard changes
			this.closeConfirmDialog.show(); // ask with dialog
		} else {
			this.onSwitch();
		}
	},

	_onDiscard: function() {
		// discard changes and close
		this._clearDirty();
		
		this.closeConfirmDialog.hide();
		if(this._switchToSource){
			this.onSwitch();
		}else{
			this.onClose();
		}
		
		this.config = null;
	},
	_onCancelClose: function() {
		// cancel close operation
		this.closeConfirmDialog.hide();
	},
	
	_onSave: function(ev) {
		// when Save button pressed, save it and continue editing
		var self = this;
		// put save request to server
		JazzUIWorkbench.showMessage(messages.configEditorMessageSaving);
		jazz.client.xhrPut({
			url: this.config.viewUri,
			headers: {
				"Content-Type": "text/json" //$NON-NLS-1$ //$NON-NLS-2$
			},
			putData: this.getConfigSource(),
			handle: function() {
				JazzUIWorkbench.hideMessageArea();
			},
			load: function() {
//				self.initialSource = self.sourceArea.value;
				self.onSaveCompleted();
			},
			error: function(response, ioargs) {
				console.error(response.message);
				var message = util.getXhrErrorMessage(response, ioargs);
				self.messageArea.showMessage(message);
			}
		});
	},

	// control dirty state
	_isDirty: false,
	
	isDirty: function() {
		return this._isDirty;
	},

	_updateDirty: function(originalValue, newValue, dirtyKey) {
		if(!this._isDirty && (!originalValue || originalValue != newValue)){
			this._isDirty = true;
			this.dirtyStateChanged();
		}
	},
	
	_clearDirty: function() {
		if (this._isDirty) {
			this._isDirty = false;
			this.dirtyStateChanged();
		}
	},

	dirtyStateChanged: function() {
		// listeners should connect to this method
		if (this.saveButton) {
			this.saveButton.disabled = !this._isDirty;
		}
		if (this._dirtyIndicator) {
			this._dirtyIndicator.style.display = this._isDirty ? "" : "none"; //$NON-NLS-1$//$NON-NLS-2$
		}
	},

	destroy: function() {
		if(this.content){
			this.content.destroy();
		}
		this.inherited(arguments);
	},
	
	_hideMessage: function(){
		if(this.messageArea)
			this.messageArea.hideMessage();
	}
});

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.ConfigEditor._Content", dijit._Widget, { //$NON-NLS-1$
	// init properties
	editor: null,
	_sections: null,

	constructor: function() {
		this.messages = messages;
	},
	
	postCreate: function() {
		this.inherited(arguments);

		this._sections = {};
		this._sections["general"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.General({_configEditor: this.editor, editMode: this.editMode});//$NON-NLS-1$
		this._sections["query"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.Query({_configEditor: this.editor, editMode: this.editMode, isCollapsed: false});//$NON-NLS-1$
		this._sections["layout"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.Layout({_configEditor: this.editor, editMode: this.editMode, isCollapsed: false});//$NON-NLS-1$
		this._sections["componentInfo"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo({_configEditor: this.editor, editMode: this.editMode, isCollapsed: false});//$NON-NLS-1$
		this._sections["resource"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.Resource({_configEditor: this.editor, editMode: this.editMode, isCollapsed: false});//$NON-NLS-1$
		this._sections["connection"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.Connection({_configEditor: this.editor, editMode: this.editMode, isCollapsed: false});//$NON-NLS-1$
		this._sections["background"] = new com.ibm.relm.explorer.web.ui.internal.editor.sections.Background({_configEditor: this.editor, isCollapsed: false});//$NON-NLS-1$

		for(var tabName in this._sections){
			this.domNode.appendChild(this._sections[tabName].domNode);
		}
	},
	
	getSectionWidget: function(sectionName){
		return this._sections[sectionName];
	},
	
	destroy: function(){
		for(var tabName in this._sections){
			this._sections[tabName].destroy();
		}
		this.inherited(arguments);
	},
	
	setEditMode:function(mode) {
		for(var tabName in this._sections){
			this._sections[tabName].setEditMode(mode);
		}

		var displayType = mode.common.disableActionButton ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_ADD_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_EDIT_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_DELETE_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_ADD_BUTTON).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_REMOVE_BUTTON).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	startup: function(){
		this.inherited(arguments);
	}
});

})();
