/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.ConfigTextEditor"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("dijit.layout.BorderContainer"); //$NON-NLS-1$
dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.Dialog"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigTextEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.editor.ConfigTextEditor", //$NON-NLS-1$
	[ dijit.layout.ContentPane, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ConfigTextEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
	initialSource: null,
	configUpdated: false,
	_switchToForm: false,
	
	constructor: function() {
		this._dirtyGroups = {};
	},
	
	save: function(){
		this._onSave();
	},
	
	load: function(uri){
		this._loadSource(uri);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.sourceArea, "onChange", "onSourceEdit"); //$NON-NLS-1$ //$NON-NLS-2$
		
		// initialize state
		this.saveButton.disabled = true; 
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigTextEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	},

	setDiagramTitle: function(title) {
		this.diagramTitle = title;
	},
	
	clearContent: function(){
		this.initialSource = null;
		this.sourceArea.set("value", "");//$NON-NLS-1$ //$NON-NLS-2$
	},
	
	onSourceEdit: function() {
		this._hideMessage();
	},
	
	onClose: function() {
		this.messageArea.hideMessage();		
	},

	onSaveCompleted: function() {
		this.messageArea.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: this.messages.configSaved, 
			closable: true});
		this.configUpdated = true;
	},
	
	// StackContainer calls when this page is selected
	_loadSource: function(uri){
		this.diagramUri = uri;
		if(this.diagramUri){
			var url = this.diagramUri; 
			this.sourceArea.set("readOnly", true);//$NON-NLS-1$
			JazzUIWorkbench.showMessage(this.messages.configLoading);
			var self = this;
			jazz.client.xhrGet({
				url: url,
				handle: function() {
					JazzUIWorkbench.hideMessageArea();
				},
				load: function(response, ioargs){
					self.sourceArea.set("readOnly", false);//$NON-NLS-1$
					self.sourceArea.set("value", ioargs.xhr.responseText);//$NON-NLS-1$
					self.initialSource = self.sourceArea.get("value");//$NON-NLS-1$
				},
				error: function(ioargs){
					self.sourceArea.set("value", ioargs.message + "\n" + ioargs.responseText); //$NON-NLS-1$//$NON-NLS-2$
				},
				headers: {"Accept": "application/xml"}, //$NON-NLS-1$ //$NON-NLS-2$
				handleAs: "xml", //$NON-NLS-1$
				preventCache: true
			});			
		}
		return true;
	},

	_onClose: function(ev){
		this._switchToForm = false;
		if(this.isDirty()) {
			// ask discard changes
			this.closeConfirmDialog.show(); // ask with dialog
		} else {
			this.onClose();
		}
	},

	_onSwitch: function(ev){
		this._switchToForm = true;
		if(this.isDirty()) {
			// ask discard changes
			this.closeConfirmDialog.show(); // ask with dialog
		} else {
			this.onSwitch();
		}
	},
	
	_onDiscard: function() {
		// discard changes and close
		this.initialSource = null;
		
		this.closeConfirmDialog.hide();
		if(this._switchToForm){
			this.onSwitch();			
		}else{
			this.onClose();
		}
	},
	_onCancelClose: function() {
		// cancel close operation
		this.closeConfirmDialog.hide();
	},
	
	_onSave: function() {
		// when Save button pressed, save it and continue editing
		var self = this;		
		// put save request to server
		JazzUIWorkbench.showMessage(this.messages.configSaving);
		jazz.client.xhrPut({
			url: this.diagramUri,
			headers: {
				"Content-Type": "application/xml"  //$NON-NLS-1$ //$NON-NLS-2$
			},
			putData: this.sourceArea.get("value"), //$NON-NLS-1$
			handle: function() {
				JazzUIWorkbench.hideMessageArea();
			},
			load: function() {
				self.initialSource = self.sourceArea.get("value");//$NON-NLS-1$
				self.onSaveCompleted();
			},
			error: function(response, ioargs) {
				self._handleError(response, ioargs);
			}
		});
	
	},

	_onSaveAs: function(ev) {
		this._diagramName.value = this.diagramTitle;
		this.saveAsDialog.show();
	},
	
	_doSaveAs: function(ev) {
		var self = this;
		// put save request to server
		jazz.client.xhrPut({
			handleAs: 'json',  //$NON-NLS-1$

			url: this.diagramUri,
			content: {
				configContent: this.sourceArea.get("value"),//$NON-NLS-1$
				diagramName: this._diagramName.value
			},
			load: function() {
				// reflect changes to nav list
				self.saveAsDialog.hide();
				// TODO: need to retrieve new source and new ID from server if editing is continued.
//				self.initialSource = self.sourceArea.value;
//				self.modified = true;
				self.onSaveCompleted();
				self.onClose();
			},
			error: function(response, ioargs) {
				self._handleError(response, ioargs);
			}
		});
	
	},
	
	_cancelSaveAs: function() {
		this.saveAsDialog.hide();
		
	},
	
	_createDirtyIndicator: function(){
		this._dirtyIndicator = dojo.create("span", {className: "title dirtyIndicator"});   //$NON-NLS-1$   //$NON-NLS-2$
		this._dirtyIndicator.style.display = "none";   //$NON-NLS-1$
		this._dirtyIndicator.appendChild(dojo.doc.createTextNode(" *"));   //$NON-NLS-1$
	},

	isDirty: function() {
		return this.initialSource && (this.initialSource != this.sourceArea.get("value")); //$NON-NLS-1$
	},


	dirtyStateChanged: function(isDirty) {
		// listeners should connect to this method
		if (this.saveButton) {
			this.saveButton.disabled = !isDirty;   
		}

		if (this._dirtyIndicator) {
			if (isDirty) {
				this._dirtyIndicator.style.display = ""; //$NON-NLS-1$
			} else {
				this._dirtyIndicator.style.display = "none";   //$NON-NLS-1$
			}
		}
	},

	_handleError: function(response, ioargs){
        if(response.responseText){
        	var msg = response.message; 
        	if(msg){
        		var details = response.responseText;
        		this.messageArea.showMessage({
    				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,	
    				summary: msg, 
    				details:  [{message: details, code: jazz.ui.MessageBox.STATUS_TYPES.ERROR}],
    				closable: true
    			});
        		response.responseText = null; // don't show the message in error dialog
        	}
        }
	},
	
	_hideMessage: function(){
		if(this.messageArea)
			this.messageArea.hideMessage();	
	}
});

})();