/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent", [dijit._Widget, dijit._Templated,] , { //$NON-NLS-1$

	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ReorderResourceDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

	messages:null,
	resourceList: [],
	currentResource: null,
	
	selectedIndex: -1,
	numEntries: -1,
	
	constructor: function(args) {
		this.resourceList = args.resourceList;
		this.currentResource = args.currentResource;
	},
	
	postCreate: function(){
		this.inherited(arguments);

		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

		var self = this;
		this.list = new jazz.ui.ListBox({
			allowMultipleSelections :false,
			sortOrder: 2,
			onClick: function (entry) {
				if(entry) {
					var entries = self.list.getAllEntries();
					self.selectedIndex = dojo.indexOf(entries, entry);
					self.refreshButtonState();

				}
			}
		});

		this.listContainer.appendChild(this.list.domNode);
		
		this.initList();
	},
	
	initList: function() {
		var entries = [];
		var currentIndex = -1;
		this.numEntries = this.resourceList.length;
		for(var i = 0; i < this.resourceList.length; i ++) {
			var entry = {
					label: (this.resourceList[i].name) || this.messages.noName,
					value: this.resourceList[i],
					id: i,
					type: 0
			};
			entries.push(entry);
			if(this.currentResource == this.resourceList[i]) {
				currentIndex = i;
			}
		}
		this.list.addEntries(entries);
		if(currentIndex >= 0) {
			this.list.setSelection([entries[currentIndex]], true);
			this.selectedIndex = currentIndex;
			this.refreshButtonState();
		}
		
	},

	refreshButtonState: function() {
		if(this.selectedIndex > 0) {
			this.upButton.set('disabled', ""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.upButton.set('disabled', "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(this.selectedIndex < this.numEntries - 1){
			this.downButton.set('disabled', ""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.downButton.set('disabled', "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	upButtonClicked: function() {
		this.reorderEntries(this.selectedIndex - 1);
		this.selectedIndex -= 1;
		this.refreshButtonState();
	},
	
	downButtonClicked: function() {
		this.reorderEntries(this.selectedIndex + 1);
		this.selectedIndex += 1;
		this.refreshButtonState();
	},
	
	reorderEntries: function(moveToIndex) {

		var orgEntries = this.list.getAllEntries();
//		var tempEntry = null;
		var firstFound = false;
		var newEntries = [];
		for(var i = 0; i < orgEntries.length; i++) {
			if(i == moveToIndex || i == this.selectedIndex) {
				if(firstFound == false) {
					firstFound = true;
					continue;
				} else {
					if(moveToIndex < this.selectedIndex) {
						newEntries.push(orgEntries[this.selectedIndex]);
						newEntries.push(orgEntries[moveToIndex]);
					} else {
						newEntries.push(orgEntries[moveToIndex]);
						newEntries.push(orgEntries[this.selectedIndex]);
					}
				}
			} else {
				newEntries.push(orgEntries[i]);
			}
		}
		this.list.removeAllEntries();
		this.list.addEntries(newEntries);
		this.list.setSelection([newEntries[moveToIndex]], true);
		
	},
	
	getValues: function(){
		var entries = this.list.getAllEntries();
		var resourceList = dojo.map(entries, function(entry) {
			return entry.value;
		});
		return  resourceList;
	}


});

com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent.openDialog = function(resourceList, currentResource, callback2){
	// create edit icon dialog content
//	var params = config.params;
	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent({
		resourceList: resourceList,
		currentResource: currentResource
	});

	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.messages.ReorderResource,
		onOpen: function(){
			dijit.focus(contentWidget.list.domNode); 
		},
		validate: function(values, callback){
			callback();
		},
		onOK: function(values){
			callback2(values);
		}
	});
};

})();