/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.ActionEditTable"); //$NON-NLS-1$

dojo.require("jazz.core.loader"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var load_async = jazz.core.loader.load_async;
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_SET_DEFAULT = "setdefault-command"; //$NON-NLS-1$
var CLASS_UNSET_DEFAULT = "unsetdefault-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$

var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; //$NON-NLS-1$ 

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.ActionEditTable", com.ibm.relm.explorer.web.ui.internal.view.common.Table, { //$NON-NLS-1$
	_resourceInfo: null,
	_configEditor: null,
	
	setConfigEditor: function(editor){
		this._configEditor = editor;
	},
	
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$		
	},
	
	postCreate: function() {
		var messages = this.messages;
		var updateActionData = dojo.hitch(this, this.updateActionData);
		
		this.setupColumns(tableUtils.createTableColumns(
				[
				 {
       		    	id: "label",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceActionLabel,
       		    	editable: true, 
       		    	callback: updateActionData,
       		    	preventEmpty: true,
       		    	width: "160px"  //$NON-NLS-1$
				 },{
    		    	id: "actionType",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceActionType,
       		    	editable: true, 
       		    	callback: updateActionData,
       		    	preventEmpty: false,
       		    	options: [
       		    	   {label: messages.configEditorResourceActionType_Action, value: messages.configEditorResourceActionType_Action},
	    		       {label: messages.configEditorResourceActionType_Link, value: messages.configEditorResourceActionType_Link}
       		    	],
       		    	width: "120px"//$NON-NLS-1$
				 },{
       		    	id: "action",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceActionLink,
       		    	editable: true, 
       		    	callback: updateActionData,
       		    	preventEmpty: true,
       		    	width: "200px"  //$NON-NLS-1$
				 },{
	    		    id: "defaultAction",  //$NON-NLS-1$
	    		    name: messages.configEditorDefaultAction,
	    		    isAction: true
				 },{
    		    	id: "actions",  //$NON-NLS-1$
    		    	name: messages.configEditorAction,
    		    	isAction: true
				 }
		]));
	},

	setResourceInfo: function(resource) {
		this._resourceInfo = resource;
		
		this.clearItems();
		
		if(!resource) return;

		var messages = this.messages;
		var self = this;
		if(resource.actions.length>0) {
			dojo.forEach(resource.actions, function(action) {
				var actionType = (action.action && action.action.length > 0) ? messages.configEditorResourceActionType_Action
																			: messages.configEditorResourceActionType_Link;
				var commandOrLink = (action.action && action.action.length > 0) ? action.action : action.href;
				var isDefault = (action["default"]) ?  //$NON-NLS-1$
									{
										iconClass : CLASS_UNSET_DEFAULT, // checked and click to unset
										title: messages.configEditorUnsetDefaultAction,
										alwaysVisible: true,
										onClick: function (e) {
											if(action.label) {
												self.unsetDefaultAction(action);
											}
										}
									} 
									: {
										iconClass : CLASS_SET_DEFAULT, // unchecked and click to set
										title: messages.configEditorSetDefaultAction,
										alwaysVisible: true,
										onClick: function (e) {
											if(action.label) {
												self.setDefaultAction(action);
											}
										}

									};
				self.addItem({
					label: action.label,
					actionType: actionType,
					action: commandOrLink,
					defaultAction: [isDefault],
					actions: [{
						iconClass: CLASS_EDIT_COMMAND,
						title: messages.configEditorEditAction,
						onClick: function(e){
							self._editAction(action);
						}
					},{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.configEditorRemoveAction,
						onClick: function(e){
							if(action.label){
								if(confirm(dojo.string.substitute(messages.confirmRemoveAction, [ action.label ]))){
									// remove action
									self.removeAction(action);
								}
							}
						}
					}]
				});
			});
		}
	},
	
	setDefaultAction: function(targetAction) {
		var self = this;
		self.clearDefaultAction();
		targetAction["default"] = true; //$NON-NLS-1$
		self.setResourceInfo(self._resourceInfo);

	},
	
	unsetDefaultAction: function(action) {
		delete action["default"]; //$NON-NLS-1$
		var self = this;
		self.setResourceInfo(self._resourceInfo);
	},
	
	clearDefaultAction: function() {
		var self = this;
		var resource = self._resourceInfo;
		if(resource.actions.length>0) {
			dojo.forEach(resource.actions, function(action) {
				delete action["default"]; //$NON-NLS-1$
			});
		}
	},
	
	removeAction: function(action) {
		var resource = this._resourceInfo;
		if(resource) {
			for(var i = 0; i < resource.actions.length; i++) {
				if(resource.actions[i].label == action.label) {
					delete resource.actions.splice(i,1);
					this._configEditor._updateDirty();
					break;
				}
			}
			this.setResourceInfo(resource);
		}
	},
	
	updateActionData: function(rowData, propName, newValue, dirtyKey) {
		var label = rowData.label;
		if(label) {
			var orgProp = this.getActionDataByLabel(label);
			var oldValue = orgProp[propName];
			if(propName == "label"){//$NON-NLS-1$
				orgProp[propName] = newValue;
			}else{
				var actionType = propName == "actionType" ? newValue : rowData.actionType;//$NON-NLS-1$
				var action = propName == "action" ? newValue : rowData.action;//$NON-NLS-1$
				var messages = this.messages;
				if(actionType == messages.configEditorResourceActionType_Action) {
					orgProp.action = action;
					orgProp.href = ""; //$NON-NLS-1$
				}else if(actionType == messages.configEditorResourceActionType_Link) {
					orgProp.href = action;
					orgProp.action = ""; //$NON-NLS-1$					
				}
			}
			this._configEditor._updateDirty(oldValue, newValue, dirtyKey);
		}
	},
	
	getActionDataByLabel: function(label) {
		var resource = this._resourceInfo; 
		if(resource) {
			if(resource.actions && resource.actions.length > 0) {
				for(var i = 0; i < resource.actions.length; i++) {
					if(resource.actions[i].label == label) {
						return resource.actions[i];
					}
				}
			}
		}
	},

	refreshActionData: function() {
		var resource = this._resourceInfo;
		if(resource) {
			this._configEditor._updateDirty();
			this.setResourceInfo(resource);
		}
	},
	
	addAction: function(){
		this._editAction();
	},

	_editAction: function(actionDef) {
		var resource = this._resourceInfo;
		if(resource && resource.actions) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog", dojo.hitch(this, function() { //$NON-NLS-1$
				var componentType = resource.component ? resource.component.type: null;
				var resourceNode = this._configEditor.getDiagramEditor().createComponent(componentType || DEFAULT_COMPONENT_TYPE);
				
				com.ibm.relm.explorer.web.ui.internal.editor.ActionEditDialog.openDialog(actionDef, resourceNode, dojo.hitch(this, function(newActionDef){
					if(!actionDef){
						// newly created
						resource.actions.push(newActionDef);
					}
					if(newActionDef["default"] == true) { //$NON-NLS-1$
						this.clearDefaultAction();
						newActionDef["default"] = true; //$NON-NLS-1$
					}
					this.refreshActionData();
				}));
			}));
		}
	}
	
});

})();

