/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo"); //$NON-NLS-1$

dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore");//$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("jazz.ui.ListBox"); //$NON-NLS-1$
dojo.require("dijit.layout.BorderContainer"); //$NON-NLS-1$
dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
dojo.require("dijit.layout.TabContainer"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._ParameterEditMixin"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = {};

var load_async = jazz.core.loader.load_async;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;

var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentInfo",  //$NON-NLS-1$
		[dijit._Widget, dijit._Templated, 
		 com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin, 
		 com.ibm.relm.explorer.web.ui.internal.editor.sections._ParameterEditMixin], {
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormComponentInfo.html"), //$NON-NLS-1$ //$NON-NLS-2$

	constructor: function (){
		this._componentInfoDropdownId = dijit.getUniqueId("componentInfoDropdown");//$NON-NLS-1$
		this._classTypeDropdownId = dijit.getUniqueId("classTypeDropdown");//$NON-NLS-1$
	},

	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.acceptQueryVariables = false;
	},

	postCreate: function() {
		this.inherited(arguments);

		this.loadClassTypeToDropdown();
		
		this.addAttributeButton = this._createAddButton(messages.AddAttribute, this._handleAddAttribute);
		this.attributesToolbar.appendChild(this.addAttributeButton);
		this.attributesTable.setupColumns(this._createNameValueTableColumns(this.updateAttrName, this.updateAttrValue, false, "200px")); //$NON-NLS-1$

		this.shapeControlSet = this._createControlSet();
		this.shapesToolbar.appendChild(this.shapeControlSet);
		var updateShape = dojo.hitch(this, this.updateShape);
		this.shapesTable.setupColumns(this._createTableColumns(
				[{
					id: "type", //$NON-NLS-1$
					name: messages.configEditorComponentInfoShapeType,
					editable: false,
					width: "80px" //$NON-NLS-1$
				},{
					id: "shape", //$NON-NLS-1$
					name: messages.configEditorComponentInfoShapeValue,
					editable: false,
					callback: updateShape,
					preventEmpty: true
				},{
					id: "actions", //$NON-NLS-1$
					name: messages.configEditorAction,
					isAction: true
				}
		]));

		var selected = this._configEditor.getSelectedType && this._configEditor.getSelectedType();
		this.refreshComponentInfoDropdown(selected);
		this.connect(this.componentInfoList, "onChange", function(value) { //$NON-NLS-1$
			var selection = this.componentInfoList.getSelection();
			if(selection && selection.length > 0){
				var compId = selection[0].id;
				var componentInfoItem = this.getComponentInfoById(compId);
				this.setComponentInfoData(componentInfoItem.componentInfo, componentInfoItem.isSystem);
			}
		});
		this.connect(this.componentInfoList, "onClick", function(e){ //$NON-NLS-1$
			// Reselect last selected entry if no entry is selected because jazz.ui.ListBox doesn't provide prevent removing all selection.
			var selection = this.componentInfoList.getSelection();
			if(!selection || selection.length <= 0){
				this.componentInfoList.setSelection([{id: this.lastSelectedId}], true);
				var componentInfoItem = this.getComponentInfoById(this.lastSelectedId);
				this.setComponentInfoData(componentInfoItem.componentInfo, componentInfoItem.isSystem);
			}else{
				this.lastSelectedId = selection[0].id;
			}
		});
		
		this.connect(this.classTypeDropdown, "onChange", function(value) { //$NON-NLS-1$
			var componentInfo = this.getCurrentComponentInfo();
			if(componentInfo && !editorUtils.isEmptyObj(componentInfo)) {
				if(componentInfo.className != value) {
					componentInfo.className = value;
					this._updateDirty();
					this._updatePreview();
				}
			}
		});

		this.connect(this.previewArea, "onShow", function(e){ //$NON-NLS-1$
			this._updatePreview();
		});
	},

	_updatePreview: function(){
		if(this._uiPreviewComponent){
			this._uiPreviewComponent.destroy();
			this._uiPreviewComponent = null;
		}
		var uiPreview = this.uiPreview;
		var componentInfo = this.getCurrentComponentInfo();
		uiPreview._componentInfo = {};
		uiPreview._componentInfo[componentInfo.id] = componentInfo;
		var component = this._uiPreviewComponent = uiPreview.createComponent(componentInfo.id);
		if(!component){
			return;
		}
		var h = component.getHeight();
		uiPreview.domNode.style.height = h + 5 + "px"; //$NON-NLS-1$
		var w = component.getWidth();
		uiPreview.domNode.style.width = w + 5 + "px"; //$NON-NLS-1$
		uiPreview.addChild(this._uiPreviewComponent);
		uiPreview.adjustCanvasSize();

		this.configArea.layout();
	},

	getComponentInfoById: function(id) {
		return editorUtils.getComponentInfoById(id, this.getConfig(), this.getDiagramEditor());
	},
	
	getCurrentComponentInfo: function() {
		var selections = this.componentInfoList.getSelection();
		if(selections && selections.length > 0){
			return selections[0].value;
		}
	},
	
	refreshComponentInfoDropdown: function(selectedComponentInfoId) {
		// load dropdown
		var componentInfoIds = editorUtils.loadComponentTypeToListBox(this.componentInfoList, this.getConfig(), this.getDiagramEditor(), 
				null, messages.configEditorComponentInfoSystem);

		// select selected item
		if(this.componentInfoList.getEntry(selectedComponentInfoId)){
			this.componentInfoList.setSelection([{id: selectedComponentInfoId}], true);
			this.lastSelectedId = selectedComponentInfoId;
			var componentInfoItem = this.getComponentInfoById(selectedComponentInfoId);
			this.setComponentInfoData(componentInfoItem.componentInfo, componentInfoItem.isSystem);
		} else if(componentInfoIds.length > 0) {
			this.componentInfoList.setSelection([{id: componentInfoIds[0]}], true);
			this.lastSelectedId = componentInfoIds[0];
			var componentInfoItem = this.getComponentInfoById(componentInfoIds[0]);
			this.setComponentInfoData(componentInfoItem.componentInfo, componentInfoItem.isSystem);
		}
	},
	
	clearComponentInfoData: function() {
		this.setComponentInfoData({className: "", attributes: {}}); //$NON-NLS-1$
	},
	
	setComponentInfoData: function(item, isSystem) {
		if(item) {
			// disable editing for system components 
			this.classTypeDropdown.set("readOnly", isSystem);//$NON-NLS-1$
			var displayStyle = isSystem ? "none" : "";//$NON-NLS-1$//$NON-NLS-2$
			dojo.style(this.removeComponentInfoButton, "display", displayStyle);//$NON-NLS-1$
			dojo.style(this.addAttributeButton, "display", displayStyle);//$NON-NLS-1$
			dojo.style(this.shapeControlSet, "display", displayStyle);//$NON-NLS-1$
			//
			this.classTypeDropdown.set("value", item.className); //$NON-NLS-1$
			var attrTable = this.attributesTable;
			var shapeTable = this.shapesTable;
			attrTable.clearItems();
			shapeTable.clearItems();

			var addOrEditByPicker = dojo.hitch(this, this.addOrEditByPicker);
			var self = this;
			dojo.forEach(item.attributes, function(attr){
				if(attr.name == "targetShapes"){//$NON-NLS-1$
					self._updateShapesTable(attr.value, isSystem);
				}else{
					attrTable.addItem({
						name : attr.name,
						type : attr.type,
						value: attr.value,
						description: attr.description||"",//$NON-NLS-1$
						actions: isSystem ? null : [{
							iconClass: CLASS_EDIT_COMMAND,
							title: messages.configEditorEditParams,
							onClick: function(e){
								var rowData = this.data;
								var propName = rowData.name;
								if(propName){
									addOrEditByPicker(propName, rowData.type, rowData.value, rowData);
								}
							}
						},{
							iconClass: CLASS_DELETE_COMMAND,
							title: messages.configEditorRemoveParams,
							onClick: function(e){
								self._deleteItem(this.data.name, item.attributes, attr);
							}
						}],
						readonly: isSystem
					});
				}
			});
		}
		this._updatePreview();
	},

	_onNormalEditParam: function(propName, propType, propValue, data) {
		var componentInfo = this.getCurrentComponentInfo();
		if(!componentInfo) {
			return;
		}
		var targetParams = {};
		dojo.forEach(componentInfo.attributes, function(attr) {
			targetParams[attr.name] = {name: attr.name, value: attr.value, description: attr.description || ""}; //$NON-NLS-1$
		});
		var param = dojo.clone(targetParams[data.name]);
		
		this.addOrEditGeneralParameter(null, targetParams, data.name, dojo.hitch(this, function() {
			var attrNames = ["value", "description"]; //$NON-NLS-1$ //$NON-NLS-2$
			var updateParam = targetParams[data.name];
			dojo.forEach(attrNames, function(attrName) {
				if (param[attrName] != updateParam[attrName]) {
					this.updateAttrValue(param, attrName, updateParam[attrName]);
				}
			}, this);
			this.refreshComponentInfoData();
		}));
	},
	
	_onEditByPicker: function(data, propName, newValue) {
		this.updateAttrValue(data, "value", newValue); //$NON-NLS-1$
		this.refreshComponentInfoData();
	},

	_deleteItem: function(itemName, array, item){
		if(itemName){
			if(confirm(dojo.string.substitute(messages.confirmRemoveTargetProperty, [ itemName ]))){
				// remove parameter
				var index = dojo.indexOf(array, item);
				if(index < 0){
					// something wrong
					return;
				}
				array.splice(index, 1);
				this.setComponentInfoData(this.getCurrentComponentInfo());
				this._updateDirty();
			}
		}
	},
	
	_handleAddAttribute: function() {
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() { //$NON-NLS-1$
			var curr = this.getCurrentComponentInfo();
			if(!curr) {
				return;
			}
			com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(curr.attributes, dojo.hitch(this, this.refreshComponentInfoData));
		}));
	},
	
	_handleAddShape: function() {
		this._editShape(null);
	},
	
	_editShape: function(shapeDef){
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog", dojo.hitch(this, function() { //$NON-NLS-1$
			var curr = this.getCurrentComponentInfo();
			if(!curr) {
				return;
			}
			var shapes = this._findAttribute(curr.attributes, "targetShapes");//$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.ShapeEditDialog.openDialog(shapeDef, dojo.hitch(this, function(newShapeDef){
				if(!shapes){
					shapes = {name: "targetShapes", value: []};//$NON-NLS-1$
					curr.attributes.push(shapes);
				}
				if(!shapeDef){
					// newly created
					shapes.value.push(newShapeDef.value);
				}
				this.refreshComponentInfoData();
			}));
		}));
	},
	
	refreshComponentInfoData: function(){
		this.setComponentInfoData(this.getCurrentComponentInfo());
		this._updateDirty();
	},

	updateAttrName: function(rowData, propName, newValue, dirtyKey) {
		this._updateAttribute(true, rowData, propName, newValue, dirtyKey);
	},

	updateAttrValue: function(rowData, propName, newValue, dirtyKey) {
		this._updateAttribute(false, rowData, propName, newValue, dirtyKey);
	},
	
	_updateAttribute: function(/* boolean */ updateName, rowData, propName, newValue, dirtyKey) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var attr = this._findAttribute(componentInfo.attributes, rowData.name);
			if(attr){
				var oldValue = updateName ? attr.name : attr[propName];
				if(oldValue != newValue){
					if (updateName) {
						attr.name = newValue;
						rowData.name = newValue;
					}else{
						attr[propName] = newValue;
						rowData[propName] = newValue;
					}
					this._updatePreview();
					this._updateDirty(oldValue, newValue, dirtyKey);
				}
			}
		}
	},

	_findShape: function(shapes, attrJson){
		if(shapes){
			for(var i = shapes.length - 1; i >= 0; i--){
				var shape = shapes[i];
				if(dojo.toJson(shape) === attrJson) return shape;
			}
		}
		return null;
	},
	
	updateShape: function(rowData, propName, newValue, dirtyKey) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var shapes = this._findAttribute(componentInfo.attributes, "targetShapes");//$NON-NLS-1$
			if(shapes){
				var shape = this._findShape(shapes.value, rowData.shape);
				if(shape){
					var oldValue = dojo.toJson(shape);
					if(oldValue != newValue){
						// update data in componentInfo
						for(var i in shape){
							delete shape[i];
						}
						dojo.mixin(shape, dojo.fromJson(newValue)); 

						// keep the JSON value for the next time _findShape above.
						// the original value is filled in _updateShapesTable()
						rowData.shape = dojo.toJson(shape); 

						this._updatePreview();
						this._updateDirty(oldValue, newValue, dirtyKey);
					}
				}
			}
		}
	},

	loadClassTypeToDropdown: function () {
		// obtain resource class type list
		var items = editorUtils.getClassTypeItems();
		
		var types = [];
		dojo.forEach(items, function(candidate) {
			var item = {name: candidate.value,
						value: candidate.value};
			types.push(item);
		});

		var typeItems = {items: types};

		var classTypeStore = new dojo.data.ItemFileReadStore({data: typeItems});
		
		this.classTypeDropdown.set("store", classTypeStore); //$NON-NLS-1$
		this.classTypeDropdown.set("searchAttr", "value"); //$NON-NLS-1$ //$NON-NLS-2$
		this.classTypeDropdown.set("labelAttr", "name"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	setEditMode: function(editMode) {
		if(!editMode) return;

		var mode = editMode.componentInfo;
		
		this.componentInfoSection.style.display = mode.componentInfoSection ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoClassRow.style.display = mode.componentInfoClassRow ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoAttributeRow.style.display = mode.componentInfoAttributeRow ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		this.componentInfoShapeRow.style.display = this.componentInfoAttributeRow.style.display;
		if(this.componentInfoShapeRow.style.display === ""){//$NON-NLS-1$
			this.uiPreview.adjustCanvasSize();
		}
	},

	handleAddComponentInfo: function(e){
		this._addComponentInfo();
		dojo.stopEvent(e);
	},
	_addComponentInfo: function() {
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo", dojo.hitch(this, function() { //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo.openDialog(this.getConfig().componentInfo.items, dojo.hitch(this, this._createNewComponent));
		}));
	},
	
	_createNewComponent: function(newComponent) {
		var componentInfo = this.getConfig().componentInfo;
		if(!componentInfo.items || editorUtils.isEmptyObj(componentInfo.items)) {
			componentInfo.items = [];
		}
		
		componentInfo.items.push(newComponent);
		this._updateDirty();
		this.refreshComponentInfoDropdown(newComponent.id);
	},

	handleDuplicateComponentInfo: function(e){
		this._duplicateComponentInfo();
		dojo.stopEvent(e);
	},
	_duplicateComponentInfo: function() {
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo", dojo.hitch(this, function() { //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.AddComponentInfo.openDialog(this.getConfig().componentInfo.items, dojo.hitch(this, this._duplicateComponent));
		}));
	},
	
	_duplicateComponent: function(newComponent) {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			var dup = dojo.clone(componentInfo);
			// only id has been set by dialog
			dup.id = newComponent.id;
			newComponent = dup;
		}
		this._createNewComponent(newComponent);
	},

	handleRemoveComponentInfo: function(e){
		this._removeComponentInfo();
		dojo.stopEvent(e);
	},
	_removeComponentInfo: function() {
		var componentInfo = this.getCurrentComponentInfo();
		if(componentInfo) {
			if(confirm(dojo.string.substitute(messages.confirmRemoveComponentInfo, [componentInfo.id]))) {
				var items = this.getConfig().componentInfo.items;
				for(var i = 0; i < items.length; i++) {
					if(items[i].id == componentInfo.id) {
						this.getConfig().componentInfo.items.splice(i, 1);
						this._updateDirty();
						this.refreshComponentInfoDropdown();
						break;
					}
				}
			}
		}
	},

	_createControlSet: function() {
		var self = this;
		var controlSet = dojo.create("div", {}); //$NON-NLS-1$
		controlSet.appendChild(this._createArrowButton(function(){this._moveShape(1, self._findAttribute(self.getCurrentComponentInfo().attributes, "targetShapes"), self._updateShapesTable, self);},//$NON-NLS-1$
				this.messages.moveShapeDown,
				dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/down.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
		controlSet.appendChild(this._createArrowButton(function(){this._moveShape(-1, self._findAttribute(self.getCurrentComponentInfo().attributes, "targetShapes"), self._updateShapesTable, self);}, //$NON-NLS-1$
				this.messages.moveShapeUp,
				dojo.moduleUrl("com.ibm.relm.explorer.web","ui/images/up.gif"))); //$NON-NLS-1$ //$NON-NLS-2$
		controlSet.appendChild(this._createAddButton(messages.AddShape, this._handleAddShape));
		return controlSet;
	},
	_updateShapesTable: function(shapes, isSystem){
		var self = this;
		dojo.forEach(shapes, function(shape){
			self.shapesTable.addItem({
				type: shape.type,
				shape: dojo.toJson(shape),
				actions: isSystem ? null : [{
					iconClass: CLASS_EDIT_COMMAND,
					title: messages.configEditorEditShape,
					onClick: function(e){
						self._editShape(shape);
					}
				},{
					iconClass: CLASS_DELETE_COMMAND,
					title: messages.configEditorRemoveShape,
					onClick: function(e){
						self._deleteItem(this.data.type || this.data.shape, shapes, shape);
					}
				}],
				readonly: isSystem 
			});
		});
	},

	destroy: function() {
		this.inherited(arguments);
	}
});
})();