/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._ParameterEditMixin"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;

var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_RESTORE_COMMAND = "restore-command"; //$NON-NLS-1$

var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; // fallback //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable", //$NON-NLS-1$
		[com.ibm.relm.explorer.web.ui.internal.view.common.Table, com.ibm.relm.explorer.web.ui.internal.editor.sections._ParameterEditMixin], {
	defaultComponentType: null,
	
	_resourceInfo: null,
	_configEditor: null,
	acceptQueryVariables: true,
	targetqueryId: null,
	
	setConfigEditor: function(editor){
		this._configEditor = editor;
	},

	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		var queryMessages = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.ui.internal', 'QueryUtilsMessages'); //$NON-NLS-1$ //$NON-NLS-2$

		dojo.mixin(this.messages, queryMessages);
	},
	
	postCreate: function() {
		var updateComponentParamName = dojo.hitch(this, this.updateComponentParamName);
		var updateComponentParamValue = dojo.hitch(this, this.updateComponentParamValue);
		this.setupColumns(tableUtils.createNameValueTableColumns(updateComponentParamName, updateComponentParamValue, true, "400px")); //$NON-NLS-1$
	},
	
	setResourceInfo: function(resource) {
		this._resourceInfo = resource;
		
		this.clearItems();
		
		if(!resource) return;

		if(resource.component) {
			var params = {};
			var componentInfo = editorUtils.getComponentInfoById(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE, this._configEditor.getConfig(), this._configEditor.getDiagramEditor());
			if(!componentInfo || !componentInfo.componentInfo) return;
			dojo.forEach(componentInfo.componentInfo.attributes, function(a){
				if(a.name == "targetShapes") return; //$NON-NLS-1$
				var value = a.value || ""; //$NON-NLS-1$
				if(dojo.isObject(value)){ value = dojo.toJson(value); }
				params[a.name] = {value: value, description: a.description || "", type: a.type }; //$NON-NLS-1$
			});
			dojo.mixin(params, resource.component.params);
			var self = this;
			var addOrEditByPicker = dojo.hitch(this, this.addOrEditByPicker);
			var updateComponentParamValue = dojo.hitch(this, this.updateComponentParamValue);
			
			var messages = this.messages;
			for(var name in params) {
				var param = params[name];
				var value = param["value"]; //$NON-NLS-1$
				var desc = param["description"]; //$NON-NLS-1$
				var paramType = param["type"]; //$NON-NLS-1$
				var actions = [];
				actions.push({
					iconClass: CLASS_EDIT_COMMAND,
					title: messages.configEditorEditParams,
					onClick: function(e){
						var propName = this.data.name;
						if(propName){
							var rowData = this.data;
							addOrEditByPicker(propName, rowData.type, rowData.value, rowData);
						}
					}

				});
				actions.push({
					iconClass: CLASS_RESTORE_COMMAND,
					title: messages.RestoreDefault,
					onClick: function(e){
						var propName = this.data.name;
						if(propName){
							if(confirm(dojo.string.substitute(messages.confirmResetComponentParameter, [ propName ]))){
								// remove parameter
								delete resource.component.params[propName];
								self.refreshComponentParam();
							}
						}
					}
				});
				this.addItem({name : name, 
					value: value,
					description: desc,
					type: paramType,
					actions: actions
				});
			}
		}
	},
	
	_onEditByPicker: function(data, propName, newValue) {
		this.updateComponentParamValue(data, propName, newValue);
	},

	_onNormalEditParam: function(propName, propType, propValue, data) {
		this._editParameter(propName);
	},

	refreshComponentParam: function() {
		var resource = this._resourceInfo;
		if(resource) {
			this.setResourceInfo(resource);
			this._configEditor._updateDirty();
		}
	
	},
	
	updateComponentParamName: function(rowData, propName, newName, dirtyKey) {
		var oldName = rowData.name;
		if(oldName != newName) {
			var resource = this._resourceInfo;
			if(resource) {
				resource.component.params[newName] = resource.component.params[oldName];
				delete resource.component.params[oldName];
				this.setResourceInfo(resource);
				this._updateDirty(oldName, newName, dirtyKey);
			}
		}
	},

	updateComponentParamValue: function(rowData, propName, newValue, dirtyKey) {

		var resource = this._resourceInfo;
		if(resource) {
			// if the new value is the default value defined in component, then remove the property
			var type = null;
			var componentAttrs = this._configEditor.getDiagramEditor().getAttributes(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE);
			if(componentAttrs){
				for(var i=0; i<componentAttrs.length; i++){
					if(componentAttrs[i].name == rowData.name){
						type = componentAttrs[i].type;
						if(newValue == componentAttrs[i].value){
							var param = resource.component.params[rowData.name];
							var oldValue = param ? param[propName] : null;
							delete resource.component.params[rowData.name];
							this.setResourceInfo(resource);
							this._configEditor._updateDirty(oldValue, newValue, dirtyKey);
							return;
						}
						break;
					}
				}
			}
			// Otherwise update the value if the value is changed
			if(!resource.component.params[rowData.name]){
				resource.component.params[rowData.name] = {};
			}
			var param = resource.component.params[rowData.name];

			var oldValue = param[propName];
			if(oldValue != newValue){
				param["value"] = newValue; //$NON-NLS-1$
				if(type) param["type"] = type; //$NON-NLS-1$
				this.setResourceInfo(resource);
				this._configEditor._updateDirty(oldValue, newValue, dirtyKey);
			}
		}
	},
	addParameter: function(){
		this._editParameter();
	},

	_editParameter: function(targetParamName) {
		var resource = this._resourceInfo;
		if(resource && resource.component) {
			if(!resource.component.params) resource.component.params = {};
			var targetParams = resource.component.params;
			var componentParams = null;
			var componentInfo = editorUtils.getComponentInfoById(resource.component.type || this.defaultComponentType || DEFAULT_COMPONENT_TYPE, this._configEditor.getConfig(), this._configEditor.getDiagramEditor());
			if(componentInfo && componentInfo.componentInfo){
				var componentAttrs = componentInfo.componentInfo.attributes;
				if(componentAttrs) {
					if(targetParamName){
						if(!targetParams[targetParamName]){
							for(var i=0; i<componentAttrs.length; i++){
								var attr = componentAttrs[i];
								if(attr.name == targetParamName){
									targetParams[targetParamName] = {
										name: attr.name,
										value: dojo.isObject(attr.value) ? dojo.toJson(attr.value) : attr.value,
										type: attr.type	
									};
								}
							}
						}
					}else{
						componentParams = dojo.map(componentAttrs, function(attr){
							return {name: attr.name, value: attr.name};
						});
					}
				}
			}
			this.addOrEditGeneralParameter(componentParams, targetParams, targetParamName, dojo.hitch(this, this.refreshComponentParam));
		}
	}});

})();
