/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.Query"); //$NON-NLS-1$

dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.QueryUtils"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = {};

var load_async = jazz.core.loader.load_async;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var queryUtils = com.ibm.relm.explorer.web.ui.internal.view.common.QueryUtils;

var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.Query", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormQuery.html"), //$NON-NLS-1$ //$NON-NLS-2$

	editor: null,
	queries: null,
	editMode: null,
	
	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},
	
	postCreate: function() {
		var titleNode = dojo.create("span"); //$NON-NLS-1$
		titleNode.appendChild(dojo.doc.createTextNode(messages.LabelQuery));
	
		this.inherited(arguments);

		this.paramToolbar.appendChild(this._createAddButton(messages.AddParameter, this._handleAddParameter));
		var disableNameEdit = this.editMode && this.editMode.common.disableNameEdit;
		this.paramTable.setupColumns(this._createNameValueTableColumns(this.updateParamName, this.updateParamValue, disableNameEdit));
		
		this.sparqlToolbar.appendChild(this._createAddButton(messages.AddPrefixDefinition, this._handleAddPrefix, this, messages.AddPrefixButtonLabel));

		this.queries = this.getConfig().queryTemplates;
		dojo.forEach(this.queries, dojo.hitch(this, function(query) {
			this.queryDropdown.addOption({value: query.id, label: (query.id == "") ? messages.configEditorQueryDefault: query.id}); //$NON-NLS-1$
		}));

		if(this.queries[0]) {
			this.setSelectedIndex(0);
		}

		var self = this;
		var divSparqlToolbarBottom = dojo.create("div", { //$NON-NLS-1$
			"class" : "toolbar" //$NON-NLS-1$ //$NON-NLS-2$
		}, this.sparqlFooter);
		divSparqlToolbarBottom.appendChild(dojo.create("a", { //$NON-NLS-1$
			href : "#", //$NON-NLS-1$
			innerHTML : this.messages.previewQueryResult,
			title : this.messages.previewQueryResultTitle,
			style : {
				verticalAlign : "middle", //$NON-NLS-1$
				marginLeft : "8px" //$NON-NLS-1$
			},
			onclick : function(e) {
				dojo.stopEvent(e);
				self._handlePreviewQueryResult();
			}
		}));

		this.connect(this.queryDropdown,"onChange", function(value) {  //$NON-NLS-1$
			var key = value;
			var selected = this.queryDropdown.getOptions(key);
			if(selected) {
				var queryId = selected.value;
				var query = this.getQueryById(queryId);
				this.setQueryData(query);
			}
		});
		
		this.connect(this.queryTextArea, "onChange", function(newVal) {  //$NON-NLS-1$
			var query = this.getSelectedQuery();
			if(query) {
				if(query.sparql != newVal) { 
					query.sparql = newVal; 
					this._updateDirty(); 
				}
			}
		});

	},
	
	_handlePreviewQueryResult : function() {

		var queryTemplate = this.getSelectedQuery();
		var sparql = queryTemplate.sparql;

		var configParams = nodeUtils.getParamValue(this.getConfig().params);
		var queryParams = nodeUtils.getParamValue(queryTemplate.params);
		
		queryUtils.openPreviewQueryResultDialog(sparql, [configParams, queryParams]);
	},


	setQueryData: function(query) {
		if(query) {
			this.queryTextArea.set('value', query.sparql);  //$NON-NLS-1$
			this.setParamData(query.params);
		} else {
			this.queryTextArea.set('value', "");  //$NON-NLS-1$ //$NON-NLS-2$
			this.setParamData([]);
		}
		if(this.editMode) {
			this.setEditMode(this.editMode);
		}
		
	},
	
	getSelectedQuery:function() {
		var queryId = this.queryDropdown.value;
		return this.getQueryById(queryId);
	},
	
	getQueryById: function (queryId) {
		if(queryId || queryId == "") { //$NON-NLS-1$
			for(var i=0; i < this.queries.length; i++) {
				if(this.queries[i].id == queryId) {
					return this.queries[i];
				}
			}
			
		} 
		return null;
	},
	
	refreshQueryDropdown: function(selectQueryId) {
		var options = this.queryDropdown.options;
		options.length = 0;

		dojo.forEach(this.queries, dojo.hitch(this, function(query) {
			this.queryDropdown.addOption({value: query.id, label: (query.id == "") ? messages.configEditorQueryDefault: query.id}); //$NON-NLS-1$
		}));
		
		this._updateDirty();
		this.queryDropdown.set("value", selectQueryId);  //$NON-NLS-1$

		dojo.publish("/queryListChanged");  //$NON-NLS-1$
	},

	createQueryDropdownItem: function (query) {
		
		var optionChild = dojo.doc.createElement("option");  //$NON-NLS-1$
		optionChild.innerHTML = (query.id  == "" || query.id == null) ? messages.DefaultQuery: query.id; //$NON-NLS-1$
		optionChild.value=query.id;
		optionChild.selected = (query.id == ""|| query.id == null); //$NON-NLS-1$
		
		return optionChild;
	},

	setParamData: function(params) {

		var propTable = this.paramTable;
		propTable.clearItems();

		var self = this;
		for(var name in params) {
			var param = params[name];
			var value = param["value"]; //$NON-NLS-1$
			var description = param["description"]; //$NON-NLS-1$
			propTable.addItem({name : name, 
								value: value,
								description: description,
								actions: [{
									iconClass: CLASS_DELETE_COMMAND,
									title: messages.configEditorRemoveParams,
									onClick: function(e){
										var name = this.data.name;
										if(name){
											if(confirm(dojo.string.substitute(messages.confirmRemoveParameter, [ name ]))){
												// remove parameter
												delete params[name];
												self.refreshParamData();
											}
										}
									}
								}]
								
			});
		}

	},
	
	refreshParamData: function() {
		var queryId = this.queryDropdown.value;
		var query = this.getQueryById(queryId);
		if(query) {
			this._updateDirty();

			this.setParamData(query.params);
		}
		this.setEditMode(this.editMode);
	},
	
	updateParamName: function(rowData, propName, newName, dirtyKey) {
		var oldName = rowData.name;	
		if(oldName != newName) {
			// parameter name changed
			var queryId = this.queryDropdown.value;
			var query = this.getQueryById(queryId);
			if(query) {
				query.params[newName] = query.params[oldName];
				delete query.params[oldName];
				this.setParamData(query.params);
				this._updateDirty(oldName, newName, dirtyKey);
			}			
		}

	},
	
	updateParamValue: function(rowData, propName, newValue, dirtyKey) {
		var queryId = this.queryDropdown.value;
		var query = this.getQueryById(queryId);
		if(query) {
			var param = query.params[rowData.name];
			if(param) {
				var oldValue = param[propName];
				if(oldValue != newValue){
					param[propName] = newValue;
					this.setParamData(query.params);
					this._updateDirty(oldValue, newValue, dirtyKey);
				}
			}
		}
	},

	_handleAddParameter: function() {
		var query = this.getSelectedQuery();
		if(query) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() {  //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(query.params, dojo.hitch(this, this.refreshParamData));
			}));
		}
	},
	
	_addQuery: function() {
		load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddQuery", dojo.hitch(this, function() {  //$NON-NLS-1$
			com.ibm.relm.explorer.web.ui.internal.editor.AddQuery.openDialog(this.getConfig().queryTemplates, dojo.hitch(this, this.refreshQueryDropdown));
		}));
		
	},
	
	_removeQuery: function() {

		var query = this.getSelectedQuery();
		if(query) {
			if(confirm(dojo.string.substitute(messages.confirmRemoveQuery, [query.id]))) {
				var queries = this.getConfig().queryTemplates;
				for(var i = queries.length - 1; i >= 0; i--) {
					if(queries[i].id == query.id) {
						queries.splice(i, 1);
						this.refreshQueryDropdown();
						if(queries.length == 0) {
							this.setQueryData(null); // clear fields
						}
						break;
					}
				}
			}
		}
	},

	_handleAddPrefix : function() {
		var self = this;
		var prefixDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog({
			okCallback: dojo.hitch(this,this._okPrefixDialogCallback),
			cancelCallback: dojo.hitch(this,this._cancelPrefixDialogCallback)
		});

		self._prefixDialog = new jazz.ui.Dialog({
			contentNode: prefixDialog.domNode,
			primaryTitle: self.messages.prefixSelectionDialog,
			width: "600px", //$NON-NLS-1$
			isClosable: true
		});
	
	},
	_okPrefixDialogCallback: function(args)
	{
		
		this._prefixDialog.close();
		this._prefixDialog.destroy();
		// refresh sparql
		var sparql = this.queryTextArea.get("value"); //$NON-NLS-1$
		sparql = args.prefixText + sparql;
		this.queryTextArea.set("value", sparql); //$NON-NLS-1$

	},

	_cancelPrefixDialogCallback: function(args)
	{
		this._prefixDialog.close();
		this._prefixDialog.destroy();
	},

	

	setEditMode: function(editMode) {
		if(!editMode) return;
		
		this.editMode = editMode;
			
		var mode = editMode.query;
		
		this.querySection.style.display = mode.querySection ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.querySelectRow.style.display = mode.querySelectRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.queryContentRow.style.display = mode.queryContentRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.queryParamRow.style.display = mode.queryParamRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		
		if(editMode.common.hideEmptyTable) {
			var query = this.getSelectedQuery();
			if(query) {
				if (editorUtils.isEmptyObj(query.params)) {
					this.queryParamRow.style.display = "none"; //$NON-NLS-1$
				}
			}
		}

		var displayType = editMode.common.disableActionButton ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_ADD_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_EDIT_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_DELETE_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	startup: function() {
		
	},
	
	destroy: function() {
		this.inherited(arguments);
	},
	
	setSelectedIndex: function(index){
		if(index < 0){
			// create a new link
			this._addQuery();
		}else{
			var options = this.queryDropdown.options;
			this.queryDropdown.set('value', options[index].value);  //$NON-NLS-1$
		}		
	}
	
});

})();

