/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$

var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$

var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin", null, { //$NON-NLS-1$
	_configEditor : null,
	
	setConfigEditor: function(editor){
		this._configEditor = editor;
	},
	
	getConfigEditor: function(){
		return this._configEditor;
	},
	
	getConfig: function(){
		return this._configEditor.getConfig();
	},
	
	_updateDirty: function(oldvalue, newvalue, propName){
		return this._configEditor._updateDirty(oldvalue, newvalue, propName);
	},
	
	getDiagramEditor: function(){
		return this._configEditor.getDiagramEditor();
	},
	
	getSelectedNode: function(){
		if(dojo.isFunction(this._configEditor.getSelectedNode))
			return this._configEditor.getSelectedNode();
		return null;
	},
	
	_createAddButton: function(label, callback, context, buttonLabel){
		callback = dojo.hitch(context || this, callback);
		var a = dojo.create("a", {  //$NON-NLS-1$
			href: "#",  //$NON-NLS-1$
			innerHTML: buttonLabel || messages.Add,
			className: CLASS_ADD_COMMAND,
			title: label,
			style: {
				verticalAlign: "middle",  //$NON-NLS-1$
				marginLeft: "8px"  //$NON-NLS-1$
			}
		});
		this.connect(a, 'onclick', function(e){ //$NON-NLS-1$
			dojo.stopEvent(e);
			callback();
		});
		return a;
	},

	_createArrowButton: function(callback, title, imgUrl) {
		callback = dojo.hitch(this, callback);
		return new jazz.ui.toolbar.Button({
			title: title,
			imgSrc: imgUrl, //$NON-NLS-1$ //$NON-NLS-2$
			"class": "sectionButton", //$NON-NLS-1$ //$NON-NLS-2$
			onClick: dojo.hitch(this, function(e) { 
				dojo.stopEvent(e);
				callback();
			})
		}).domNode;
	},

	_moveShape: function(dir, shapes, callback, context){
		callback = dojo.hitch(context || this, callback);
		var config = this.getConfig();
		if(!config) return;

		var item = this.shapesTable.getSelectedItem();
		if(!item) return;

		if(!shapes || shapes.value.length < 2) return;

		var index = -1;
		for(var i=0; i<shapes.value.length; i++){
			var shape = shapes.value[i];
			if(dojo.toJson(shape) === item.data.shape){
				index = i;
				break;
			}
		}
		if(index < 0) return;

		var newIndex = -1; // -1 means don't move
		if(dir > 0 ) {
			// down 
			if((index < shapes.value.length -1) && this._swapShapes(shapes.value, index, index+1)) {
				newIndex = index + 1;
			}
		} else {
			// up
			if((index > 0) && this._swapShapes(shapes.value, index, index-1)) {
				newIndex = index - 1;
			}
		}
		if(newIndex != -1){
			if(this._updatePreview) {
				this._updatePreview();
			}
			this.shapesTable.clearItems();
			if(callback){
				callback(shapes.value);
			}
			this.shapesTable.selectItemByIndex(newIndex);
		}
	},
	_swapShapes: function (array, index1, index2){
		if( (index1 != index2) && (0 <= index1 < array.length) && (0 <= index2 < array.length)) {
			var temp = array[index1];
			array[index1] = array[index2];
			array[index2] = temp;
			return true; // if swap done
		}
		return false;
	},

	_findAttribute: function(attrs, name){
		if(attrs){
			for(var i = attrs.length - 1; i >= 0; i--){
				var attr = attrs[i];
				if(attr.name === name) return attr;
			}
		}
		return null;
	},

	_createParamTableColumnsEx: tableUtils.createParamTableColumnsEx,
	_createNameValueTableColumns: tableUtils.createNameValueTableColumns,
	_createTableColumns: tableUtils.createTableColumns, 
	_createTableColumnDescription: tableUtils.createTableColumnDescription
	
});


})();
