/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.hover._GfxRichHoverMixin"); //$NON-NLS-1$
dojo.require("dojox.gfx.shape"); //$NON-NLS-1$

(function(){

dojo.declare("com.ibm.relm.explorer.web.ui.internal.hover._GfxRichHoverMixin", null, { //$NON-NLS-1$
	_htmlNode: null,
	
	_getCursorPosition: function() {
		var editor = this.owner.hlp_getEditor();
		if (!editor)
			return null;
		var e = editor._mouseEvent;
		if (e && (e.layerX || e.layerY)) {
		    return {x: e.layerX, y: e.layerY};
		} 
		return null;
	},
	
	position: function(node){
		if(!this._htmlNode){
			var par = this.owner;
			while(!par.isEditor) par = par.getParent(); // find com.ibm.relm.explorer.web.base.internal.canvas.Viewer
			this._htmlNode = dojo.create("div", {style: {position: "absolute"}}, par.domNode); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		var l=Number.MAX_VALUE, t=Number.MAX_VALUE, w=0, h=0;
		var shape = this.shape;
		if(shape instanceof dojox.gfx.Group){
				var childShape = shape.children[0];
				var s = childShape.shape; 
				if(s.type == "polyline" && s.points && s.points.length > 2){ //$NON-NLS-1$
					var targetPoint = s.points[s.points.length/2-1]; // middle point
					var targetPoint2 = s.points[s.points.length/2]; // middle point 2
					var dx = Math.abs(targetPoint.x - targetPoint2.x);
					var dy = Math.abs(targetPoint.y - targetPoint2.y);
					w = h = 10;
					if(dx < w || dy < h){
						l = Math.min(targetPoint.x, targetPoint2.x);
						t = Math.min(targetPoint.y, targetPoint2.y);
						if(dx  < w){
							// vertical line
	 						h = dy;							
						} else {
							// horizontal line
	 						w = dx;														
						}
					} else {
						// use a point
						l = targetPoint.x - w/2;
						t = targetPoint.y - h/2;
					}
					var m = childShape._getRealMatrix();
					var gm = dojox.gfx.matrix;
					var topLeft = gm.multiplyPoint(m, l, t);
					l = topLeft.x;
					t = topLeft.y;
				} else {
					var boxes = childShape.getTransformedBoundingBox();
					if(boxes){ // boxes is null for Text shape
						var startIndex = 0;
						if (childShape.getLastPosition) {
							var last = childShape.getLastPosition();
							var bbox = childShape.getBoundingBox();
							if ((last.x == bbox.x + bbox.width && last.y == bbox.y) || (last.x == bbox.x && last.y == bbox.y + bbox.height)) {
								startIndex = 1;
							}
						}
						var topLeft = boxes[startIndex]; // top-left corner
						var bottomRight = boxes[startIndex + 2]; // bottom-right corner
						if(s.type == "path"){ //$NON-NLS-1$
							// round connection
							var pos = this._clipPosition(topLeft.x, topLeft.y, bottomRight.x, bottomRight.y);
							var cursorPos = this._getCursorPosition();
							var hoverPos = this._getLineHoverPosition(pos, cursorPos);
							l = hoverPos.x;
							t = hoverPos.y;
						}else{
							l = Math.min(l, topLeft.x);
							t = Math.min(t, topLeft.y);
							w = Math.max(w, bottomRight.x-topLeft.x);
							h = Math.max(h, bottomRight.y-topLeft.y);
						}
					}
				}
		}else{
			var boxes = shape.getTransformedBoundingBox();
			var topLeft = boxes[0]; // top-left corner
			var bottomRight = boxes[2]; // bottom-right corner
			l = topLeft.x, 
			t = topLeft.y;
			w = bottomRight.x-topLeft.x, 
			h = bottomRight.y-topLeft.y;
		}
		var aroundNode = this._htmlNode;
		dojo.style(aroundNode, {
			display: "", //$NON-NLS-1$
			left: l+"px", //$NON-NLS-1$
			top: t+"px", //$NON-NLS-1$
			width: w+"px", //$NON-NLS-1$
			height: h+"px" //$NON-NLS-1$
		});
	
		this.inherited(arguments, [aroundNode]);

		aroundNode.style.display = "none"; //$NON-NLS-1$
	},

	_clipPosition: function(x1, y1, x2, y2) {
		var editor = this.owner.hlp_getEditor();
		if (!editor)
			return {x1: x1, y1: y1, x2: x2, y2: y2};

		var viewX1 = editor.domNode.clientLeft;
		var viewY1 = editor.domNode.clientTop;
		var viewX2 = viewX1 + editor.domNode.clientWidth;
		var viewY2 = viewY1 + editor.domNode.clientHeight;

		var ratio = x2 != x1 ? (y2 - y1) / (x2 - x1) : 1;
		var pos1 = this._getCrossPoint(x1, y1, ratio, viewX1, viewY1, viewX2, viewY2);
		if (pos1) {
			var pos2 = this._getCrossPoint(x2, y2, ratio, viewX1, viewY1, viewX2, viewY2);
			if (pos2) {
				return {x1: pos1.x, y1: pos1.y, x2: pos2.x, y2: pos2.y};
			}
		}
		return {x1: x1, y1: y1, x2: x2, y2: y2};
	},
	
	/**
	 * calculate crossing point of line and view area
	 * 
	 *   (2)  |     (7)      |   (5) 
	 *  -------------------------------
	 *        |              |
	 *        |              |
	 *   (1)  |     (9)      |   (4)
	 *        |              |
	 *        |              |
	 *  -------------------------------
	 *   (3)  |     (8)      |   (6)
	 */
	_getCrossPoint: function(x, y, ratio, rx1, ry1, rx2, ry2) {
		// 
		if (x < rx1) {
			var y1 = Math.round(y + (rx1 - x) * ratio);
			if (ry1 <= y1 && y1 <= ry2) {
				// (1) adjust to left edge
				return {x: rx1, y: y1};
			} else if (ratio > 0 && y < ry1) {
				var x1 = Math.round(x + (y - ry1) / ratio);
				if (rx1 <= x1 && x1 <= rx2) {
					// (2) adjust to top edge
					return {x: x1, y: ry1};
				}				
			} else if (ratio < 0 && y > ry2) {
				var x1 = Math.round(x + (y - ry2) / ratio);
				if (rx1 <= x1 && x1 <= rx2) {
					// (3) adjust to bottom edge
					return {x: x1, y: ry2};
				}				
			}
		} else if (x > rx2) {
			var y1 = Math.round(y + (rx2 - x) * ratio);
			if (ry1 <= y1 && y1 <= ry2) {
				// (4) adjust to right edge
				return {x: rx2, y: y1};
			} else if (ratio < 0 && y < ry1) {
				var x1 = Math.round(x + (y - ry1) / ratio);
				if (rx1 <= x1 && x1 <= rx2) {
					// (5) adjust to top edge
					return {x: x1, y: ry1};
				}				
			} else if (ratio > 0 && y > ry2) {
				var x1 = Math.round(x + (y - ry2) / ratio);
				if (rx1 <= x1 && x1 <= rx2) {
					// (6) adjust to bottom edge
					return {x: x1, y: ry2};
				}				
			}
		} else if (y < ry1) {
			var x1 = Math.round(x + (ry1 - y) / ratio);
			if (rx1 <= x1 && x1 <= rx2) {
				// (7) adjust to top edge
				return {x: x1, y: ry1};
			}				
		} else if (y > ry2) {
			var x1 = Math.round(x + (ry2 - y) / ratio);
			if (rx1 <= x1 && x1 <= rx2) {
				// (8) adjust to bottom edge
				return {x: x1, y: ry2};
			}				
		} else {
			// (9) in area
			return {x: x, y: y};
		}
		return null;
	},
	
	_getLineHoverPosition: function(pos, cursorPos) {
		var l = 0;
		var t = 0;
		var ratio = (pos.y2 - pos.y1) / (pos.x2 - pos.x1);
		if (Math.abs(pos.x2 - pos.x1) > Math.abs(pos.y2 - pos.y1)) {
			var maxdiff = ratio < 0 ? 50 : 30;
			var x1 = pos.x1;
			var x2 = pos.x2;
			var y1 = pos.y1;
			if (pos.x1 > pos.x2) {
				x1 = pos.x2;
				x2 = pos.x1;
				y1 = pos.y2;
			}
			var diff = x2 - cursorPos.x - 10;
			if (diff < 3) diff = 3;
			l = cursorPos.x + Math.min(diff, maxdiff);
			if (x1 > l) l = x1;
			else if (x2 < l) l = x2;
			t = Math.round(y1 + (l - x1) * ratio);
		} else if (pos.y2 != pos.y1) {
			var maxdiff = 30;
			var x1 = pos.x1;
			var y1 = pos.y1;
			var y2 = pos.y2;
			if (pos.y1 > pos.y2) {
				x1 = pos.x2;
				y1 = pos.y2;
				y2 = pos.y1;
			}
			var diff = cursorPos.y - y1 - 10;
			if (diff < 3) diff = 3;
			t = cursorPos.y - Math.min(diff, maxdiff);
			if (y1 > t) t = y1;
			else if (y2 < t) t = y2;
			l = Math.round(x1 + (t - y1) / ratio);
		} else {
			l = (pos.x1 + pos.x2)/2;
			t = (pos.y1 + pos.y2)/2;
		}
		
		return {x: l, y: t};
	}
});

})();