/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.common.web.ui.internal.orion", "SparqlPluginMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin",null,{ //$NON-NLS-1$
	
	constructor: function(arguments) {
		var initializer = arguments.initializer;
		jazz.core.loader.load_async("com.ibm.relm.common.web.ui.internal.orion.OrionWrapper", dojo.hitch(this,function(){ //$NON-NLS-1$
			var orion = new com.ibm.relm.common.web.ui.internal.orion.OrionWrapper();
			try {
				this.initSparqlPlugin();
				if (initializer) initializer();
			} finally {
				orion.initialized();
			}
		}));
	},

	initSparqlPlugin: function() {
		
		var messages = dojo.i18n.getLocalization("com.ibm.relm.common.web.ui.internal.orion", "SparqlPluginMessages");  //$NON-NLS-1$ //$NON-NLS-2$
		
		define('orion/editor/sparqlContentAssist', [ //$NON-NLS-1$
		 	'orion/editor/templates' //$NON-NLS-1$
		 	 ], function(mTemplates) {
		 	
		 	 /*
		 	 * Provides support for content assist for SPARQL.
		 	 */
		 	 
		 	function SPARQLContentAssistProvider() {
		 	}
		 	
		 	var templates = [];
		 	
		 	/* SPARQL keywords should mandatorily be lowercase */
		 	var keywords = [ 
		 		"add", "as", "asc", "ask", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		 		"base", "bind", "by", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		 		"clear", "construct", "copy", "create", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		 		"data", "delete", "desc", "describe", "distinct", "drop", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
		 		"exists", //$NON-NLS-1$
		 		"filter", "from", //$NON-NLS-1$ //$NON-NLS-2$
		 		"graph", "group", //$NON-NLS-1$ //$NON-NLS-2$
		 		"having", //$NON-NLS-1$
		 		"in", "insert", //$NON-NLS-1$ //$NON-NLS-2$
		 		"limit", "load", //$NON-NLS-1$ //$NON-NLS-2$
		 		"minus", "move", //$NON-NLS-1$ //$NON-NLS-2$
		 		"named", "not", //$NON-NLS-1$ //$NON-NLS-2$
		 		"offset", "optional", "order", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		 		"prefix", //$NON-NLS-1$
		 		"reduced", //$NON-NLS-1$
		 		"select", "service", "silent", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		 		"to", //$NON-NLS-1$
		 		"undef", "union", "using", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		 		"values", //$NON-NLS-1$
		 		"where" //$NON-NLS-1$
		 	];
		 	
		 	// Replace argument placeholders with their corresponding values aka String.format
		 	var _stringFormat = function(str,args) {
		 		if(str === null || str === undefined || str === "" || args === null || args === undefined || args.length === 0) //$NON-NLS-1$
		 			return str;
		 		
		 		var subsStr = str;
		 		
		 		var placeHolders = str.match(/{[0-9]+}/g);
		 		for(var i=0;i<placeHolders.length;i++) {
		 			var argVal = parseInt(placeHolders[i].slice(1,-1));
		 			if(argVal < args.length)
		 				subsStr = subsStr.replace("{" + argVal + "}",args[argVal]); //$NON-NLS-1$ //$NON-NLS-2$
		 			else
		 				return str;
		 		}
		 		
		 		return subsStr;
		 	};
		 	
		 	var _xsdString = "xsd:string", //$NON-NLS-1$
		 		_xsdInteger = "xsd:integer", //$NON-NLS-1$
		 		_xsdBoolean = "xsd:boolean", //$NON-NLS-1$
		 		_xsdDouble = "xsd:double", //$NON-NLS-1$
		 		_xsdDateTime = "xsd:dateTime", //$NON-NLS-1$
		 		_xsdDayTimeDuration = "xsd:dayTimeDuration"; //$NON-NLS-1$
		 	
		 	/* SPARQL function names should mandatorily be lowercase */
		 	/*
		 	 * name: Name of the SPARQL function used for checking if this function has to be suggested
		 	 * description: The content shown to user when this function is added as suggestion (can be used for giving a small description about the function)
		 	 * proto: The prototype of the function which is to be added to the editor if the user selects this suggestion
		 	 * returnType: The return type of the function
		 	 */
		 	var functions = [
	 	        {
	 	       		name: "abs", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "avg", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "bnode", //$NON-NLS-1$
	 	       		description: " ()", //$NON-NLS-1$
	 	       		proto: " ()", //$NON-NLS-1$
	 	       		returnType: messages.blankNode
	 	       	},
	 	       	{
	 	       		name: "bnode", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		returnType: messages.blankNode
	 	       	},
	 	       	{
	 	       		name: "bnode", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[_xsdString]), //$NON-NLS-1$
	 	       		returnType: messages.blankNode
	 	       	},
	 	       	{
	 	       		name: "bound", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "ceil", //$NON-NLS-1$
		 	       	description: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "coalesce", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0}, ....)",[messages.expression]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0}, ....)",[messages.expression]), //$NON-NLS-1$
	 	       		returnType: messages.rdf_Term
	 	       	},
	 	       	{
	 	       		name: "concat", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} ltrl{1}, {0} ltrl{2}, ....)",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} ltrl{1}, {0} ltrl{2}, ....)",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "contains", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "count", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "datatype", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {0})",[messages.literal]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {0})",[messages.literal]), //$NON-NLS-1$
	 	       		returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "day", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "encode_for_uri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} ltrl)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} ltrl)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "floor", //$NON-NLS-1$
		 	       	description: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "hours", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "if", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0}{1}, {0}{2}, {0}{3})",[messages.expression, messages.one, messages.two, messages.three]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0}{1}, {0}{2}, {0}{3})",[messages.expression, messages.one, messages.two, messages.three]), //$NON-NLS-1$
					returnType: messages.rdf_Term
	 	       	},
	 	       	{
	 	       		name: "iri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "iri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[_xsdString]), //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "iri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[messages.IRI]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[messages.IRI]), //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "isblank", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "isiri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "isliteral", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "isnumeric", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "isuri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.rdfTerm, messages.term]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "lang", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} ltrl)",[messages.literal]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} ltrl)",[messages.literal]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "langmatches", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1}-{2}, {0} {1}-{3})",[messages.simpleLiteral, messages.language, messages.tag, messages.range]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1}-{2}, {0} {1}-{3})",[messages.simpleLiteral, messages.language, messages.tag, messages.range]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "lcase", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
					returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "max", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: messages.term
	 	       	},
	 	       	{
	 	       		name: "md5", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "md5", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "min", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: messages.term
	 	       	},
	 	       	{
	 	       		name: "minutes", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "month", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "now", //$NON-NLS-1$
	 	       		description: " ()", //$NON-NLS-1$
	 	       		proto: " ()", //$NON-NLS-1$
					returnType: _xsdDateTime
	 	       	},
	 	       	{
	 	       		name: "rand", //$NON-NLS-1$
	 	       		description: " ()", //$NON-NLS-1$
	 	       		proto: " ()", //$NON-NLS-1$
					returnType: _xsdDouble
	 	       	},
	 	       	{
	 	       		name: "regex", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1}, {2} {3})",[messages.stringLiteral, messages.text, messages.simpleLiteral, messages.pattern]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1}, {2} {3})",[messages.stringLiteral, messages.text, messages.simpleLiteral, messages.pattern]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "regex", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1}, {2} {3}, {2} {4})",[messages.stringLiteral, messages.text, messages.simpleLiteral, messages.pattern, messages.flags]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1}, {2} {3}, {2} {4})",[messages.stringLiteral, messages.text, messages.simpleLiteral, messages.pattern, messages.flags]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "replace", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg, {1} {2}, {1} {3})",[messages.stringLiteral, messages.simpleLiteral, messages.pattern, messages.replacement]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg, {1} {2}, {1} {3})",[messages.stringLiteral, messages.simpleLiteral, messages.pattern, messages.replacement]), //$NON-NLS-1$
	 	       		returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "replace", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg, {1} {2}, {1} {3}, {1} {4})",[messages.stringLiteral, messages.simpleLiteral, messages.pattern, messages.replacement, messages.flags]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg, {1} {2}, {1} {3}, {1} {4})",[messages.stringLiteral, messages.simpleLiteral, messages.pattern, messages.replacement, messages.flags]), //$NON-NLS-1$
	 	       		returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "round", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.numeric, messages.term]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "sameterm", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1}{2}, {0} {1}{3})",[messages.rdfTerm, messages.term, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1}{2}, {0} {1}{3})",[messages.rdfTerm, messages.term, messages.one, messages.two]), //$NON-NLS-1$
					returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "seconds", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "sha1", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha1", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha256", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha256", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha384", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha384", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha512", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "sha512", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdString]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "str", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} ltrl)",[messages.literal]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} ltrl)",[messages.literal]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
				{
	 	       		name: "str", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} rsrc)",[messages.IRI]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} rsrc)",[messages.IRI]), //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "strafter", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: messages.literal
	 	       	},
	 	       	{
	 	       		name: "strbefore", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: messages.literal
	 	       	},
	 	       	{
	 	       		name: "strdt", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} lexicalForm, {1} datatypeIRI)",[messages.simpleLiteral, messages.IRI]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} lexicalForm, {1} datatypeIRI)",[messages.simpleLiteral, messages.IRI]), //$NON-NLS-1$
					returnType: messages.literal
	 	       	},
	 	       	{
	 	       		name: "strends", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "strlang", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} lexicalForm, {0} langTag)",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} lexicalForm, {0} langTag)",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.literal
	 	       	},
	 	       	{
	 	       		name: "strlen", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
					returnType: _xsdInteger
	 	       	},
	 	       	{
	 	       		name: "strstarts", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg{1}, {0} arg{2})",[messages.stringLiteral, messages.one, messages.two]), //$NON-NLS-1$
	 	       		returnType: _xsdBoolean
	 	       	},
	 	       	{
	 	       		name: "struuid", //$NON-NLS-1$
	 	       		description: " ()", //$NON-NLS-1$
	 	       		proto: " ()", //$NON-NLS-1$
					returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "substr", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} source, xsd:integer startingLoc)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} source, xsd:integer startingLoc)",[messages.stringLiteral]), //$NON-NLS-1$
					returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "substr", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} source, xsd:integer startingLoc, xsd:integer {1})",[messages.stringLiteral, messages.len]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} source, xsd:integer startingLoc, xsd:integer {1})",[messages.stringLiteral, messages.len]), //$NON-NLS-1$
					returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "sum", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} {1})",[messages.variable, messages.varLabel]), //$NON-NLS-1$
	 	       		returnType: messages.numeric
	 	       	},
	 	       	{
	 	       		name: "timezone", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdDayTimeDuration
	 	       	},
	 	       	{
	 	       		name: "tz", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: messages.simpleLiteral
	 	       	},
	 	       	{
	 	       		name: "ucase", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} str)",[messages.stringLiteral]), //$NON-NLS-1$
					returnType: messages.stringLiteral
	 	       	},
	 	       	{
	 	       		name: "uri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[messages.simpleLiteral]), //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "uri", //$NON-NLS-1$
	 	       		description: " (xsd:string)", //$NON-NLS-1$
	 	       		proto: " (xsd:string)", //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "uri", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0})",[messages.IRI]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0})",[messages.IRI]), //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "uuid", //$NON-NLS-1$
	 	       		description: " ()", //$NON-NLS-1$
	 	       		proto: " ()", //$NON-NLS-1$
					returnType: messages.IRI
	 	       	},
	 	       	{
	 	       		name: "year", //$NON-NLS-1$
	 	       		description: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		proto: _stringFormat(" ({0} arg)",[_xsdDateTime]), //$NON-NLS-1$
	 	       		returnType: _xsdInteger
	 	       	}
		 	];
		 	
		 	SPARQLContentAssistProvider.prototype = new mTemplates.TemplateContentAssist(keywords, templates);
		 	
		 	SPARQLContentAssistProvider.prototype.computeProposals = function(buffer, offset, context) {
		 			
		 			var prefix = this.getPrefix(buffer, offset, context);
		 			var proposals = [];
		 			if (!this.isValid(prefix, buffer, offset, context)) {
		 				return proposals;
		 			}
		 			proposals = proposals.concat(this.getTemplateProposals(prefix, offset, context));
		 			proposals = proposals.concat(this.getKeywordProposals(buffer,offset,context));
		 			
		 			if(proposals.length == 0) {
			 			proposals.push({
			 				proposal: "", //$NON-NLS-1$
			 				description: messages.noProposalAvailable
			 			});
			 		}
		 			
		 			globalProposals = proposals.slice(0);
		 			return proposals;
		 	};
		 	
		 	SPARQLContentAssistProvider.prototype.getKeywordProposals = function(buffer, offset, context) {
		 			
		 		// buffer: content of the editor
		 		// offset: position at which content assist is required
		 		// context: additional contextual information about the content assist invocation
		 		
		 		var proposals = [];
		 		if(context.prefix === null || context.prefix === "" && buffer[offset - context.prefix.length - 1] !== '?') //$NON-NLS-1$ //$NON-NLS-2$
		 			return proposals;
		 		
		 		// extract variables from the SPARQL query
		 		var variables = new Array();
		 		variables = buffer.match(/[$?][a-zA-Z0-9_]+[ ;.]/gi); // regular expression for obtaining the variable names
		 		var variables_to_add = new Array();
		 		if(variables != null)
			 		for(var i=0; i < variables.length; i++)
			 			variables_to_add[variables[i].slice(1,-1)] = 1;
		 		
		 		if(buffer[offset - context.prefix.length - 1] !== '?') //$NON-NLS-1$
		 		{
			 		// Add keywords to proposals
			 		for (var i=0; i < keywords.length; i++) {
			 		   var keyword = keywords[i];
			 		   if (keyword.indexOf(context.prefix.toLowerCase()) === 0) {
			 				if(context.prefix.toUpperCase() === context.prefix) {
			 				   proposals.push({
			 					   proposal: keyword.substring(context.prefix.length).toUpperCase(),
			 					   description: keyword.toUpperCase() + "  --" + messages.keyword // suggest uppercase keywords if the context is uppercase  //$NON-NLS-1$
			 				   });
			 			   }
			 			   else {
			 					proposals.push({
			 					   proposal: keyword.substring(context.prefix.length),
			 					   description: keyword + "  --" + messages.keyword //$NON-NLS-1$
			 				   });
			 			   }
			 		   }
			 		}
			 		
			 		// Add functions to proposals
			 		for (var i=0; i < functions.length; i++) {
			 		   var keyword = functions[i];
			 		   if (keyword.name.indexOf(context.prefix.toLowerCase()) === 0) {
			 				if(context.prefix.toUpperCase() === context.prefix) {
			 				   proposals.push({
			 					   proposal: keyword.name.substring(context.prefix.length).toUpperCase() + keyword.proto,
			 					   description: keyword.returnType + " " + keyword.name.toUpperCase() + keyword.description + "  --" + messages.functions // suggest uppercase keywords if the context is uppercase  //$NON-NLS-1$  //$NON-NLS-2$
			 				   });
			 			   }
			 			   else {
			 					proposals.push({
			 					   proposal: keyword.name.substring(context.prefix.length) + keyword.proto,
			 					   description: keyword.returnType + " " + keyword.name.toUpperCase() + keyword.description + "  --" + messages.functions //$NON-NLS-1$ //$NON-NLS-2$
			 				   });
			 			   }
			 		   }
			 		}
		 		}
		 		else
		 		{
		 			// Add variable names to proposals
			 		for(var key in variables_to_add) {
			 			var keyword = key;
			 			if (keyword.indexOf(context.prefix) === 0) {
			 				proposals.push({
			 				   proposal: keyword.substring(context.prefix.length),
			 				   description: keyword + "  --" + messages.variable //$NON-NLS-1$
			 	 			});
			 	 		}
			 	 	}
		 		}
		 		
		 	 	return proposals;
		 	};
		 	 	
		  	return {
		  		SPARQLContentAssistProvider: SPARQLContentAssistProvider
		  	};
		 });

		define("orion/editor/sparqlGrammar", [], function() { //$NON-NLS-1$

			/*
			 * Provides a grammar that can do some very rough syntax highlighting for SPARQL.
			 */
			function SparqlGrammar() {
				/*
				 * Object containing the grammar rules.
				 */
				return {
					patterns: [
						{
							// Single Quotes
							name: "sparql.quotes.single", //$NON-NLS-1$
							match: "(')[^']+(\')" //$NON-NLS-1$
						},
						{
							// Double Quotes
							name: "sparql.quotes.double", //$NON-NLS-1$
							match: "(\")[^\"]+(\")" //$NON-NLS-1$
						},
						{
							// RELM Format Comment
							name: "sparql.comment.format", //$NON-NLS-1$
							match: "#!.*" //$NON-NLS-1$
						},
						{
							// Comment
							name: "sparql.comment.general", //$NON-NLS-1$
							match: "#.*" //$NON-NLS-1$
						},
						{
							// IRI
							name: "sparql.iri", //$NON-NLS-1$
							match: "<([^<>\"{}|^`\])*>" //$NON-NLS-1$
						},
						{
							// Parameter
							name: "sparql.parameter", //$NON-NLS-1$
							match: "[$][^$]+[$]" //$NON-NLS-1$
						},
						{ 
							// Variable
							name: "sparql.variable", //$NON-NLS-1$
							match: "[?][a-zA-Z0-9_]+" //$NON-NLS-1$
						},
						{
							// Number-Integer
							name: "sparql.number", //$NON-NLS-1$
							match: "(^|[ \t]+)[+\-]?([0-9]+)(?![a-zA-Z0-9])" //$NON-NLS-1$
						},
						{
							// Number-Decimal
							name: "sparql.number", //$NON-NLS-1$
							match: "(^|[ \t]+)[+\-]?([0-9]*[\.][0-9]+)(?![a-zA-Z0-9])" //$NON-NLS-1$
						},
						{
							// Number-Double
							name: "sparql.number", //$NON-NLS-1$
							match: "(^|[ \t]+)[+\-]?(([0-9]+)|([\.][0-9]+)|([0-9]+[\.][0-9]*))([eE][+-]?[0-9]+)(?![a-zA-Z0-9])" //$NON-NLS-1$
						},
						{
							// Prefix + Prefixed IRIs
							name: "sparql.prefix", //$NON-NLS-1$
							match: "([a-zA-Z0-9_.\-]+:[a-zA-Z0-9_.\-]*)|([ \t]a[ \t])" //$NON-NLS-1$
						},
						/*
						*	Keywords and functions may need to be handled in a more efficient and extensible manner
						*/
						{ 
							// Keywords
							name: "sparql.keyword", //$NON-NLS-1$
							match: "(?i)(ADD|ASC|ASK|AS|BASE|BIND|BY|CLEAR|CONSTRUCT|COPY|CREATE|DATA|DELETE|DESCRIBE|DESC|DISTINCT|DROP|EXISTS|FILTER|FROM|GRAPH|GROUP|HAVING|INSERT|IN|LIMIT|LOAD|MINUS|MOVE|NAMED|NOT|OFFSET|OPTIONAL|ORDER|PREFIX|REDUCED|SELECT|SERVICE|SILENT|TO|UNDEF|UNION|USING|VALUES|WHERE)(?![a-zA-Z0-9_])" //$NON-NLS-1$
						},
						{ 
							// Functions
							name: "sparql.function", //$NON-NLS-1$
							match: "(?i)(ABS|AVG|BNODE|BOUND|CEIL|COALESCE|CONCAT|CONTAINS|COUNT|DATATYPE|DAY|ENCODE_FOR_URI|FLOOR|HOURS|IF|IRI|ISBLANK|ISIRI|ISLITERAL|ISNUMERIC|ISURI|LANGMATCHES|LANG|LCASE|MAX|MD5|MINUTES|MIN|MONTH|NOW|RAND|REGEX|REPLACE|ROUND|SAMETERM|SECONDS|SHA1|SHA256|SHA384|SHA512|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|STRUUID|STR|SUBSTR|SUM|TIMEZONE|TZ|UCASE|URI|UUID|YEAR)(?=([ \t]*[\(]))" //$NON-NLS-1$
						}
					]
				};
			}

			return {
				SparqlGrammar: SparqlGrammar
			};
		});
	}
});