/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

(function()  {
dojo.provide("com.ibm.relm.explorer.web.ui.internal.reports.SpecifyParameterWizardStep");  //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
//dojo.require("dojox.grid.DataGrid");
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;

var CLASS_INLINE = "dijitInline";  //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.reports.SpecifyParameterWizardStep",  //$NON-NLS-1$
		[WizardStep, dijit._Widget, dijit._Templated] ,{
	
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web",  "ui/internal/reports/templates/SpecifyParameterWizardStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	
	parameters:[], // list of {name, value, queryUrl}
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this._infoIconSrc = dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/images/info_ovr.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		this.stepTitle = this.messages.parameterWizardStep;
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader ({stepTitle: ""}); //$NON-NLS-1$
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		this.initTable();
		this.startup();
	},

	initTable: function() {
		
		var comboStore = this._createDataStoreForQueryCombo();
		if(!comboStore){
			return;
		}
		
		var self = this;
		var propTable = this._paramTable;
		
		propTable.setupColumns([
		    {
		    	id: "name",  //$NON-NLS-1$
		    	name: this.messages.parameterName, 
		    	style: {width: "200px"}, //$NON-NLS-1$
		    	getCell: function(cellData){
		    		return dojo.create("td", {style: {whiteSpace: "nowrap"}}); //$NON-NLS-1$ //$NON-NLS-2$
		    	},
		    	getCellContent: function(cellData) {
		    		var div = dojo.create("span", {className : CLASS_INLINE}); //$NON-NLS-1$
		    		dojo.create("label", {innerHTML : cellData}, div); //$NON-NLS-1$
		    		return div;
		    	}
		    },
		    {
		    	id: "value", //$NON-NLS-1$
		    	name: this.messages.parameterValue, 
		    	style: {width: "200px"}, //$NON-NLS-1$
		    	getCellContent: function(cellData) {
		    		var rowWidget = this;
		    		var editBox = new dijit.InlineEditBox({
		    			editor: "dijit.form.ComboBox", //$NON-NLS-1$
		    			editorParams: { store: comboStore, searchAttr: "id", labelAttr: "name" },  //$NON-NLS-1$ //$NON-NLS-2$
		    			title : cellData,
		    			noValueIndicator: "<span class='noValueIndicator'>" + jazz.util.html.escape(self.messages.noValueIndicator) + "</span>", //$NON-NLS-1$ //$NON-NLS-2$
		    			style: {width : "100%"}, //$NON-NLS-1$
		    			onChange : function(value) {
		    				var paramName = rowWidget.data.name;
		    				dojo.forEach(self.parameters, function(param){
		    					if(param.name == paramName) {
		    						param.value = value;
		    					}
		    				});
		    			}
		    		}, dojo.create("div", { //$NON-NLS-1$
		    			className : CLASS_INLINE,
		    			innerHTML : cellData
		    		}));
		    		return editBox.domNode;
		    	}
		    },
		    {
		    	id: "queryUrl", //$NON-NLS-1$
		    	name: this.messages.parameterQueryUrl 
		    }
		]);
	},
	
	_warning: null,
	
	_createDataStoreForQueryCombo: function(){
		var idArray = dojo.map(this.wizard.queriesInView, function(q){
			return q.containerId;
		}).sort();
		var dupIds = [];
		for(var i=1; i<idArray.length; i++){
			if(idArray[i] == idArray[i-1]){
				// duplicated id
				dupIds.push(idArray[i]);
			}
		}
		var queriesInView = this.wizard.queriesInView;
		if(dupIds.length > 0){
			this._warning = {type: jazz.ui.MessageBox.STATUS_TYPES.WARNING, summary: dojo.string.substitute(this.messages.duplicatedId, [dupIds[0]])};
			queriesInView = dojo.filter(queriesInView, function(q){
				return dojo.indexOf(dupIds, q.containerId) < 0;
			});
		}
		var containers = {
				identifier: "id", //$NON-NLS-1$
				items: dojo.map(queriesInView, function(q){
					return {
						id: q.containerId,
						name: q.containerName || q.containerId
					};
				})
		};
		// add empty value to allow user to remove a pre-filled value
		containers.items.unshift({id:"", name: this.messages.noValueIndicator }); //$NON-NLS-1$

		return new dojo.data.ItemFileReadStore({ data: containers });		
	},
	
	focusStep: function() {
		var report = this.wizard._selectedReport;
		if(!report) {
			return;
		}
		if(!report.parameterNames){
			var serviceObject = {
					self: this,
					success: function(results) {
						var params = dojo.map(results, function(p){
							return {name: p.name, value: ""}; //$NON-NLS-1$
						});
						this.self.setParameters(params, true);						
						this.self.header.hideMessage();
						if(this.self._warning){
							this.self.header.showMessage(this.self._warning);
						}
					},
					failure: function(error) {
						this.self.header.showMessage(error.message);
					}
				};
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
			var args = {
				projectAreaUUID: jazz.app.context.get().itemId,
				reportUUID: report.reportUUID
			};
			this.header.showMessage({summary: this.messages.loadingParameters});
			ReportsClient.getParameters(srh, args);
		} else {
			var params = [];
			for(var i=0; i<report.parameterNames.length; i++){
				params.push({name: report.parameterNames[i], value: report.parameterValues[i]});
			}
			this.setParameters(params, true);
		}
	},
	
	setParameters: function(params, fill) {
		this.parameters = params;
		
		// fill previous parameters 
		if(this.wizard._reportParameters != null){
			for(var i=0; i<params.length; i++){
				if(!params[i].value){
					dojo.forEach(this.wizard._reportParameters, function(p){
						if(params[i].name == p.name){
							params[i].value = p.value;
							params[i].queryUrl = p.queryUrl;
							this._canFinish = true;
							this.triggerButtonStateUpdate();
						}
					}, this);
				}
			}
		}
		
		// fill empty cells if all cells are empty
		if(fill && this.wizard.queriesInView.length > 0){
			if(!dojo.some(this.parameters, function(p){
				return p.value;
			})){
				var idArray = dojo.map(this.wizard.queriesInView, function(q){
					return q.containerId;
				});
				var j = 0;
				for(var i=0; i<params.length; i++){
					if(!params[i].value){
						if(j < idArray.length){
							params[i].value = idArray[j];
							j++;
						}
					}
				}
			}
		}
		
		// clear current table content
		var propTable = this._paramTable;
		propTable.clearItems();

		// set parameters
		dojo.forEach(params, function(param) {
			propTable.addItem(param);
		});
		
	},
	
	onFinish: function(){
		this.wizard._reportParameters = this.parameters;
		this.inherited(arguments);  
	},
	
	onRegisterQueries: function(){
		this._registerQueriesToSparqlGateway();
	},
	
	canFinish: function() {
		return this._canFinish;
	},
	
	canCancel: function() {
		return true;
	},
	
	_findQueryByContainerId: function(containerId){
		var qs = this.wizard.queriesInView;
		for(var i=qs.length-1; i>=0; i--){
			var q = qs[i];
			if(q.containerId == containerId){
				return q;
			}
		}
		return null;
	},
	
	_registerQueriesToSparqlGateway: function(){
		var self = this;
		var viewId = this.wizard.viewId;
		var queries = [];
		dojo.forEach(this.parameters, function(p){
			var query = null;
			if(p.value){
				query = self._findQueryByContainerId(p.value);
			}
			queries.push(query);
		});
		var publishServiceUri = net.jazz.ajax._contextRoot + "/relmui/publish"; //$NON-NLS-1$
		this.registerButton.disabled = "true"; //$NON-NLS-1$
		this.header.showMessage({summary: this.messages.registeringQueries});
		jazz.client.xhrPost({
			url: publishServiceUri,
			headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
			content: {
				viewId: viewId,
				queries: dojo.toJson(queries)
			},
			handleAs: "json", //$NON-NLS-1$
			handle: function(){
				self.registerButton.disabled = ""; //$NON-NLS-1$
			},
			load: function(data, ioargs){
				// 
				var index = 0;
				dojo.forEach(self.parameters, function(p){
					if(index > data.length) return;
					p.queryUrl = data[index];
					index++;
				});
				self.setParameters(self.parameters);
				// enable finish button
				self._canFinish = true;
				self.triggerButtonStateUpdate();
				// test
//				window.open(publishServiceUri + "?queryEndpoints=" + dojo.toJson(data));
				self.header.hideMessage();
			},
			error: function(response, ioargs){
		    	var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs);
				self.header.showMessage(message);				
			}
		});			
	}
	
});

}) ();