/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("rmpw.jazz.ui.RootLayer"); //$NON-NLS-1$
dojo.require("rmpw.panner.ToolsLayer"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutline"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutlineDM"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramLayer"); //$NON-NLS-1$

/*
 * DiagramWidget is created in Layout class 
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	
templateString: '<div style="overflow: hidden; position: relative;" dojoAttachPoint="containerNode"></div>', //$NON-NLS-1$
	
	// mixin
	resourceUri: null,
	viewUri: null,
	
	// properties
	rootLayer: null,
	diagramLayer: null,
	toolsLayer: null,
	
	configuration: null,
	
	postCreate: function() {
		
		this.rootLayer = this._createRootLayer();
		this._createLayers(this.rootLayer);
		this.containerNode.appendChild(this.rootLayer.domNode);
		
		this.rootLayer._onMoveStop = function(){
			this._moving = false;
			this._setDefaultCursor();
		};
		this.subscribe("btp:KeyCommandInvoked", "scrollByKeyPress"); //$NON-NLS-1$ //$NON-NLS-2$

	},
	
	startup: function() {
		this.rootLayer.startup();
		this.inherited(arguments);
	    this.rootLayer.setLocation(0, 0);
	},
	
	resize: function(size) {
		if (!size) {
			return;
		}
		
		var rootSize = {};
		
		if (size.w) {
			rootSize.w = size.w;
		}
		if (size.h) {
			if(size.h <= 0) size.h = 1; // for IE
			this.domNode.style.height = size.h + "px"; //$NON-NLS-1$
			rootSize.h = size.h;
		}
		
		this.rootLayer.resize(rootSize);
		for (var i = 0, len = this.rootLayer._layers.length; i < len; ++i) {
			if (this.rootLayer._layers[i].resize) {
				this.rootLayer._layers[i].resize(rootSize);
			}
		}
	},
	
	_createRootLayer: function() {
		return new rmpw.jazz.ui.RootLayer({
			viewer: this
		});		
	},
	
	_createLayers: function(rootLayer) {
		// tools
		this.toolsLayer = new rmpw.panner.ToolsLayer();
		this.configuration.useDragPan = true;
		this.configuration.useWheelZoom = false;
		this.configuration.useWheelMove = true;
		this.diagramLayer = new com.ibm.relm.explorer.web.ui.internal.view.DiagramLayer( { configuration: this.configuration, viewUri: this.viewUri, webBaseUri: this.webBaseUri, title: this.title } );
		this.viewport = this.diagramLayer.viewer.viewport; 

		// outline
		var queryString = dojo.doc.location.search;
		if(queryString.length > 1){
			queryString = queryString.substring(1);
			if(dojo.queryToObject(queryString)["dmOutline"] === "true"){ //$NON-NLS-1$ //$NON-NLS-2$){
				this.outline = new com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutlineDM({pannableLayer: this.rootLayer});
			} else {
				this.outline = new com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutline({pannableLayer: this.rootLayer});
			}
		} else {
			this.outline = new com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutline({pannableLayer: this.rootLayer});
		}
		
		this._createTools(rootLayer, this.toolsLayer);
		
		rootLayer.addLayer("DIAGRAM_LAYER", this.diagramLayer); //$NON-NLS-1$
		rootLayer.addLayer("OUTLINE_LAYER", this.outline); //$NON-NLS-1$
		rootLayer.addLayer("TOOLS_LAYER", this.toolsLayer); //$NON-NLS-1$
		
        this.resize();
        this.rootLayer.setFocusController( this.diagramLayer.viewer.policies.focusScope );
        this._connectNodeMoved();
        
        this.connect(this.diagramLayer, "onViewerCreated", this._reconnect); //$NON-NLS-1$
 	},
	
	destroy: function() {
		if(this.diagramLayer){
			this.diagramLayer.destroy();
			this.diagramLayer = null;
		}
		if(this.outline){
			this.outline.destroy();
			this.outline = null;
		}
		this.inherited(arguments);
	},
	
	onCanvasUpdated: function(widget){
		this.outline.updateOutlineShape(widget);
		if (widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer)
			this.diagramLayer.viewer.editor.updateResorceNodes();
	},
	
	setScale: function(scale, cx, cy){
		this.rootLayer.setScale(scale, cx, cy, true);
		var val = Math.round( 100 * scale );				
		var label = val + "%"; //$NON-NLS-1$
		this.configuration.percent.set('label', label ); //$NON-NLS-1$		
	},
	
	_createTools: function(rootLayer, toolsLayer) {
		var valueChange = false;
		
		if( this.configuration.slider != null ){
		
			// catch when the slider changes the zoom
			this.configuration.slider.onChange = dojo.hitch(this, function(value) {
				valueChange = true;
				this.setScale(value, null, null);
				valueChange = false;			
			});
			
			// catch when something else like the WheelZoom edit policy changes the zoom
			this._connectOnViewportChanged(valueChange);

			this.connect(rootLayer, "setScale", function(scale) { //$NON-NLS-1$
				if (!valueChange) {
					this.configuration.slider.set("value", scale); //$NON-NLS-1$
				}
			});
		} 
	},
	
	getCommandController: function( attachPoint ){
		return this.diagramLayer.viewer.getCommandController( attachPoint );
	},

	setOutline: function( state ) {	
		if( state == false ){
			this.outline.domNode.style.visibility = "hidden"; //$NON-NLS-1$
			this.outlineVisibility = false;
		}else{
			this.outline.domNode.style.visibility = "visible"; //$NON-NLS-1$
			this.outlineVisibility = true;
		}	
	},
	
	getRenderer: function(){
	   return this.diagramLayer.viewer;	
	},
	
	refresh: function(){
		this.viewMap.policies.graphContainer.refresh();	
	},
	
	setEditMode: function(editMode){
		if(this.diagramLayer.setEditMode(editMode)){
			this.viewport = this.diagramLayer.viewer.viewport; 
			this.outline.viewPortRecreated();
			return true;
		}
		return false;
	},
	
	scrollByKeyPress: function(message) {
		
		var keyCode = message.keyCode;

		var scale = this.diagramLayer.getScale();
		var bounds = dojo.position(this.domNode);
		var diagramLoc = this.diagramLayer.getLocation();
		var diagramSize = this.diagramLayer._getSize();
		var dx = 0;
		var dy = 0;
		
		switch (keyCode) {
		case 33: // Page Up
			dy =  bounds.h;
			break;
		case 34: // Page Down
			dy = - bounds.h;
			break;
		case 35: // End
			dy = - diagramLoc.y + bounds.h - diagramSize.height * scale;
			break;
		case 36: // Home
			dy = - diagramLoc.y;
			break;
		case 37: // Left
			dx = bounds.w / 10;
			break;
		case 38: // Up
			dy = bounds.h / 10;
			break;
		case 39: // Right
			dx = - bounds.w / 10;
			break;
		case 40: // Down
			dy = - bounds.h / 10;
			break;
		}
		
		if(dx != 0 || dy != 0) {
			this.diagramLayer.viewer.viewport.pan(dx, dy);
		}

	},
	
	
	_reconnect: function() {
		if (this._nodeMovedHandle) {
			this.disconnect(this._nodeMovedHandle);
			this._connectNodeMoved();
		}
		
		if (this._onViewportChangedHandle) {
			this.disconnect(this._onViewportChangedHandle);
			this._connectOnViewportChanged(false);
		}
	},

	_connectNodeMoved: function() {
	    this._nodeMovedHandle = this.connect(this.diagramLayer.viewer.editor, "nodeMoved", "onCanvasUpdated"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	_connectOnViewportChanged: function(valueChange) {
		var percent = this.configuration.percent; 

		// catch when something else like the WheelZoom edit policy changes the zoom
		var viewport = this.diagramLayer.viewer.viewport;
		var timer = null;
		this._onViewportChangedHandle = this.connect(viewport, "onViewportChanged", function() { //$NON-NLS-1$
			var value = viewport.getZoom();
			var val = Math.round( 100 * value );				
			percent.set('label', val + "%"); //$NON-NLS-1$ //$NON-NLS-2$
			// update slider for mouseWheel
			if (!valueChange) {
				if(timer) clearTimeout(timer);
				var self = this;
				timer = setTimeout(function(){
					self.configuration.slider.set("value", value); //$NON-NLS-1$						
				}, 300);
			}
		}); 
	}

});

})();