/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutline"); //$NON-NLS-1$

dojo.require("dojox.gfx"); //$NON-NLS-1$
dojo.require("dojox.gfx.matrix"); //$NON-NLS-1$
dojo.require("rmpw.panner.Outline"); //$NON-NLS-1$
//dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ModelDiagramOutline"); //$NON-NLS-1$

/*
 * Created in DiagramWidget
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.RelmDiagramOutline", rmpw.panner.Outline, { //$NON-NLS-1$

	
	// private properties
	minResize : 0.05,
	maxResize : 2,
	_surface: null,
	_scalingGroup: null,
	_translateGroup: null,
	_contentGroup: null,
	_updatingShapes: null,
	_updateTimer: null,

	updateOutlineShape: function(mapping) {
		if (mapping.policies.outlineRenderer) {
			if (!this._updatingShapes) this._updatingShapes = [];
			this._updatingShapes.push(mapping);
			
			if (this._updateTimer) 
				clearTimeout(this._updateTimer);
			this._updateTimer = setTimeout(dojo.hitch(this, this._delayedUpdateOutlineShape), 50);
		}
	},

	_delayedUpdateOutlineShape: function() {
		this.resize();
		
		var matrix;
		var scale = this.pannableLayer.getScale();
		if (scale) {
			var offset = this.pannableLayer.viewer.viewport.getOffset();
			matrix = dojox.gfx.matrix.normalize([dojox.gfx.matrix.scale(1 / scale, 1 / scale), 
                                        dojox.gfx.matrix.translate(-offset.x, -offset.y)]);
		}
		
		dojo.forEach(this._updatingShapes, function(mapping) {
			mapping.policies && mapping.policies.outlineRenderer.updateOutlineShape(this._contentGroup, matrix);
		}, this);

		this._updateTimer = null;
		this._updatingShapes = null;
	},
	
  	_getDiagramEditor: function(){
		return dojo.getObject("pannableLayer.viewer.diagramLayer.viewer.editor", false, this); //$NON-NLS-1$
	},

	_getSurface: function(){
		return this._getDiagramEditor().surface;
	},

	postCreate: function(){
		var self = this;
		this.subscribe("com.ibm.relm.explorer.web.CONTAINER_LAYOUT_COMPLETED", function(message) { //$NON-NLS-1$
			if (message && message.owner == self._getDiagramEditor()) {
				self._updateOutlineTransform();
			}
		});
		
		this.isRealTime = true;
		var rootLayer = this.pannableLayer.viewer.rootLayer;
		if(rootLayer){
			this.pannableLayer.viewer.rootLayer = null;

			this.autoPanInterval=30;		
			this.autoPanStep=1;		
			this._maxedOut = false;
			this.inherited(arguments);

			this.connect(this._viewMoveable, "onMoving", this._onViewMoving); //$NON-NLS-1$
			
			this._surface = dojox.gfx.createSurface(this.panContainerNode, 1, 1);
			
			this._translateGroup = this._surface.createGroup();
			this._scalingGroup = this._translateGroup.createGroup();
			this._contentGroup = this._scalingGroup.createGroup();

			this.pannableLayer.viewer.rootLayer = rootLayer;
			// instead dojo.connect, use this.connect
			this.viewPortRecreated();
		}
	},
	
	destroy: function() {
		if (this._updateTimer) {
			clearTimeout(this._updateTimer);
			this._updatingShapes = null;
			this._updateTimer = null;
		}

		this.inherited(arguments);
	},
	
	setScale: function(scale, cx, cy, origin) {
		if(!this._viewSize){
			// not ready yet
			return;
		}
		
		this._cView.firstChild.style.width = this._view.firstChild.style.width = Math.floor(this._viewSize.w /scale) + "px"; //$NON-NLS-1$
		this._cView.firstChild.style.height = this._view.firstChild.style.height = Math.floor(this._viewSize.h /scale) + "px"; //$NON-NLS-1$
		if (!this._scaleUpdateHandle)
			this._scaleUpdateHandle = this.connect(this.pannableLayer.viewer.viewport, "onViewportFinished", dojo.hitch(this, this._updateView)); //$NON-NLS-1$
	},
	
	/*
	 * This method is called when update of view is completed after change of scale
	 */
	_updateView: function() {
		if (this._scaleUpdateHandle) {
			this.disconnect(this._scaleUpdateHandle);
			delete this._scaleUpdateHandle;
		}
		var loc = this.pannableLayer.viewer.viewport.getOffset();
		this._pLayerMoved(loc.x, loc.y);
	},
	
	/*
	 * Update transform for contents of outline view
	 */
	_updateOutlineTransform: function() {
		if (this._contentGroup) {
			this._contentGroup.setTransform(dojox.gfx.matrix.normalize(this._ratio));
		}
	},
	
	/*
	 * Update position of visible area indicator in outline view
	 */
	_pLayerMoved: function(x, y) {
		if (!this._movingPLayer && this._ratio) {
			if (this._relocateAnimation) {
				this._stopRelocateMove = true;
			}
			
			// locate content
			if (this._relocateAnimation || this._centerAnimation || this._centerPause) {
				var ol = this._view.offsetLeft + this._view.clientLeft;
				var ot = this._view.offsetTop + this._view.clientTop;
				this._setLocation(x * this._ratio + ol, y * this._ratio + ot);
			} else {
				// work-around for https://jazz.net/jazz04/resource/itemName/com.ibm.team.workitem.WorkItem/22603
				var loc = this.pannableLayer.viewer.viewport.getOffset();
				var scale = this.pannableLayer.getScale();
				this._view.style.left = this._cView.style.left =  - loc.x * this._ratio / scale + "px"; //$NON-NLS-1$
				this._view.style.top = this._cView.style.top =  - loc.y * this._ratio / scale + "px"; //$NON-NLS-1$
			}
		}
	},
	
	movePanContainerNode: function(x, y) {
		var trans = this._scalingGroup.getTransform();
		if(trans){
			x -= trans.dx;
			y -= trans.dy;
		}
	},
	
	restore: function(){
		this._pLayerMoved(0,0);
	},

	/*
	 * apply change of offset position to view
	 */
	_setPannableLayerLocation: function(x,y) {
		var scale = this.pannableLayer.getScale();
		x *= scale;
		y *= scale;
		
		// prevent diagram from panning to infinity and beyond
		if (this._diagram) {
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			if (box) {
				var bnds = this._diagram.policies.graphElement.getBounds();
				var min = (box.x - bnds.width -100) * scale;
				var max = (box.x + box.width +100) * scale;
				if (-x < min) {
					x = -min;
					this._maxedOut = true;
				} else if (-x > max) {
					x = -max;
					this._maxedOut = true;
				}
				min = (box.y - bnds.height-100) * scale;
				max = (box.y + box.height +100) * scale;
				if (-y < min) {
					y = -min;
					this._maxedOut = true;
				} else if (-y > max) {
					y = -max;
					this._maxedOut = true;
				}
			}
		}
		
		// pan the layers
		this._movingPLayer = true;
		var prevLocation = this.pannableLayer.viewer.viewport.getOffset();
		this.pannableLayer.setLocation(x,y,false,true);
		if (x != prevLocation.x || y != prevLocation.y) {
			this.pannableLayer.viewer.viewport.pan(x - prevLocation.x, y - prevLocation.y);
		}
		this._movingPLayer = false;
	},
	
	setDiagram: function(diagram) {
		this._diagram = diagram;
	},
	
	pan: function(dx, dy) {
		// pan diagram/sketch/etc
		var pt = this.pannableLayer.getLocation();
		this._setPannableLayerLocation(pt.x+Math.round(dx), pt.y+Math.round(dy));
		
		// pan diagram in outline shape
		this.resize();
	},
	
	viewPortRecreated: function(){
		this.restore();
		// reconnect to new viewport
		if (this._onEndPanHandle) this.disconnect(this._onEndPanHandle);
		this._onEndPanHandle = this.connect(this.pannableLayer.viewer.rootLayer.viewer.viewport, "onEndPan", "_pLayerMoved");//$NON-NLS-1$//$NON-NLS-2$
	},
	
	resize: function resize_$6(s) {
		var size = {
				w: this._overview.offsetWidth,
				h: this._overview.offsetHeight
		};
		var layers = this.getLayers();
		for (var i = 0; i < layers.length; ++i) {
			if (layers[i].resize) {
				layers[i].resize(size);
			}
		}
		/*
		 * Need to get the size of the diagram (represented by _cView) from the marginBox,
		 * this lets us use CSS styles and percentages instead of needing explicit sizes
		 * and requiring expensive calculations.
		 */
		
		if (!s) {
			s = dojo.marginBox(this.pannableLayer.domNode);
		}
		var w = s.w;
		var h = s.h;

		var surface = this._getSurface();
		if(!surface) return;
	 
		var diagram = surface.children[1].getBoundingBox();
		if(!diagram) return;
		
		var sw = diagram.width;
		var sh = diagram.height;
		var ow = size.w;
		var oh = size.h;
			var osrw = ow / sw;
			var osrh = oh / sh;
			this._ratios = {w: osrw, h: osrh};
		if (this._ratios.w < this._ratios.h) {
			var ratio = ow / sw;
		} else {
			var ratio = oh / sh;
		}
//		size of _cView
		var height = Math.floor(h * ratio);
		var width = Math.floor(w * ratio);
		if (ratio != this._ratio) {
			this._ratio = ratio;
			this._updateOutlineTransform();
		}
		this._viewSize = {
				w: width,
				h: height
		};
//		resize the view
		var scale = this.pannableLayer.getScale();
		this._cView.firstChild.style.width = this._view.firstChild.style.width = this._viewSize.w / scale + "px"; //$NON-NLS-1$
		this._cView.firstChild.style.height = this._view.firstChild.style.height = this._viewSize.h / scale + "px"; //$NON-NLS-1$

		var loc = this.pannableLayer.viewer.viewport.getOffset();

		this.panContainerNode.style.left = 0;
		this.panContainerNode.style.top = 0;
		this.panContainerNode.style.width = Math.floor(sw * ratio) + "px"; //$NON-NLS-1$
		this.panContainerNode.style.height = Math.floor(sh * ratio) + "px"; //$NON-NLS-1$
			
		this._pLayerMoved(loc.x, loc.y);

		if (this._resizeHandle) {
			var scale = this.getScale();
			this._resizeHandle.minSize = {
					w: Math.floor(Math.max(this._viewSize.w / this.maxResize, this._resizeHandle.domNode.offsetWidth)),
					h: Math.floor(Math.max(this._viewSize.h / this.maxResize, this._resizeHandle.domNode.offsetHeight))
			};
			this._resizeHandle.maxSize = {
					w: Math.ceil(this._viewSize.w / this.minResize),
					h: Math.ceil(this._viewSize.h / this.minResize)
			};
		}
		this._surface.setDimensions(this._overview.offsetWidth, this._overview.offsetHeight);

	},
	
	_onViewMoveStop: function() {
		this._moving = false;
		this._stopRelocateMove = false;
		
		var cvStartTop = this._cView.offsetTop;
		var cvStartLeft = this._cView.offsetLeft;
		
		var cvEndTop = this._view.offsetTop;
		var cvEndLeft = this._view.offsetLeft;
		
		if (cvStartTop == cvEndTop && cvStartLeft == cvEndLeft) {
			return;
		}
		
		this._cView.style.left =  this._view.style.left;
		this._cView.style.top =  this._view.style.top;
		this._resizeHandle.domNode.style.display = ""; //$NON-NLS-1$

	},
	
	_onViewMoveStart: function() {
		this._resizeHandle.domNode.style.display = "none"; //$NON-NLS-1$
		this._movingPLayer = true;
		this._moving = true;
	},
	
	_onViewMoving: function(mover, leftTop) {
		if (dojo.isFunction(this.pannableLayer.viewer.viewport.adjustPan)) {
			var scale = this.pannableLayer.getScale();
			var location = this.pannableLayer.viewer.viewport.adjustPan(
					{x: -Math.floor(leftTop.l / this._ratio * scale), y: -Math.floor(leftTop.t / this._ratio * scale)});
			leftTop.l = -Math.floor(location.x * this._ratio / scale);
			leftTop.t = -Math.floor(location.y * this._ratio / scale);
		}
	},

	_onViewMoved: function(mover, leftTop) {
		var loc = this.getLocation();
		var x = loc.x;
		var y = loc.y;
		
		this._setLocation(x, y);
		
		if (this.isRealTime) {
			this._moveView(leftTop.l, leftTop.t);
		}

	},

	_onRealTimeViewMoved: function() {
		this._moveView(this._view.offsetLeft, this._view.offsetTop);
	},

	_moveView: function(l, t) {
		this._setPannableLayerLocation(Math.floor(-l / this._ratio), Math.floor(-t / this._ratio));
	},
	
	_onResizeChange: function() {
		this._setScaleText(1 / (1 - ((this._viewSize.w - this._view.firstChild.offsetWidth) / this._viewSize.w)));
	},
	
	_onResize: function() {
		if (this._autoPanHandle) {
			clearInterval(this._autoPanHandle);
			delete this._autoPanHandle;
		}
		delete this._autoPan;
		
		this._setScaleText();
		
		var endSize = dojo.marginBox(this._view.firstChild);
		var scale = 1 / (1 - ((this._viewSize.w - endSize.w) / this._viewSize.w));
		
		if (scale == this.pannableLayer.getScale()) {
			return;
		}
		
		// maintain scale within bounds
		scale = Math.max(this.minResize, Math.min(scale, this.maxResize));
		
		this._resizeHandle.minSize = {
			w: Math.floor(Math.max(this._viewSize.w / this.maxResize, this._resizeHandle.domNode.offsetWidth)),
			h: Math.floor(Math.max(this._viewSize.h / this.maxResize, this._resizeHandle.domNode.offsetHeight))
		};
		this._resizeHandle.maxSize = {
			w: Math.ceil(this._viewSize.w / this.minResize),
			h: Math.ceil(this._viewSize.h / this.minResize)
		};
		
		// scale around a specific point
		var ratio = scale / this.pannableLayer.getScale();
		var loc = this.pannableLayer.getLocation();
		var viewBox = this.pannableLayer.getViewBox();
		var x = viewBox.w / 2 / ratio - loc.x;
		var y = viewBox.h / 2 / ratio - loc.y;
		
		var restore = this._offset;
		this._offset = {
			l: this._view.offsetLeft + this._view.clientLeft,
			t: this._view.offsetTop + this._view.clientTop
		};
		this.pannableLayer.viewer.rootLayer.setScale(scale, 0, 0, true );
		this._offset = restore;
	}


});

})();