/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$

dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox", //$NON-NLS-1$
		dijit.form.ComboBox,
{
	// attribute name which indicates cascade type. Only "group" is allowed for now.
	cascadeTypeAttr: "cascadeType",//$NON-NLS-1$
	
	// attribute name which indicates indent level. Positive number should be specified.
	// The result of "the number which is specified by this attribute" * "4ex" will be indent size;
	cascadeLevelAttr: "cascadeLevel",//$NON-NLS-1$
	
	openDropDown: function() {
		if (this.dropDown._setSelectedAttr) {
			var self = this;
			dojo.connect(this.dropDown, "_setSelectedAttr", function(node) {//$NON-NLS-1$
				// If category is selected, set the selection null
				if (self._isCascadeCategory(node)) {
					this._setSelectedAttr(null);
				}
			});
		}
		this.inherited(arguments);
	},

	closeDropDown: function() {
		if (this._categoryHandle) {
			dojo.disconnect(this._categoryHandle);
			delete this._categoryHandle;
		}
		this.inherited(arguments);
	},
	
	_isCascadeCategory: function(node) {
		return node && node.firstChild && node.firstChild.className == "com-ibm-relm-explorer-common-cascading-combo-category"; //$NON-NLS-1$
	},

	_getCascadeValue: function(item, attr) {
		var value = this.store._oldAPI ? this.store.getValue(item, attr) : item[attr];
		return value ? value.toString() : null;
	},
	
	_getMenuLabelFromItem: function(/*Item*/ item){
		var menuLabel = this.inherited(arguments);
		
		var itemType = this._getCascadeValue(item, this.cascadeTypeAttr);
		var cascadeLevel = this._getCascadeValue(item, this.cascadeLevelAttr);
		if (!itemType && !cascadeLevel) {
			return menuLabel;
		}
		
		var className = ""; //$NON-NLS-1$
		var style = ""; //$NON-NLS-1$
		if (itemType == "group") { //$NON-NLS-1$
			className = "com-ibm-relm-explorer-common-cascading-combo-category"; //$NON-NLS-1$
		}
		if (cascadeLevel > 0) {
			style += "padding-left: " + (4 * cascadeLevel) + "ex;"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	
		if (className.length > 0 || style.length > 0) {
			menuLabel.html = true;
			var label = "<div "; //$NON-NLS-1$
			if (className.length > 0) label += "class='" + className + "' "; //$NON-NLS-1$ //$NON-NLS-2$
			if (style.length > 0) label += "style='" + style + "' "; //$NON-NLS-1$ //$NON-NLS-2$
			label += ">" + menuLabel.label + "</div>"; //$NON-NLS-1$ //$NON-NLS-2$
			menuLabel.label = label;
		}		
		
		return menuLabel;
	},

	_onKey: function(/*Event*/ evt){
		try {
			// Set display style of items "none" because the node with display style "none" 
			// is not considered as selection target.
			this._enableDisableCascateCategory(false);
			this.inherited(arguments);
		} finally {
			this._enableDisableCascateCategory(true);
		}
	},
	
	_enableDisableCascateCategory: function(enable) {
		if (!this.dropDown) {
			return;
		}
		
		var node = this.dropDown.containerNode.firstChild;
		 while (node) {
			if (this._isCascadeCategory(node)) {
				 node.style.display = enable ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			 }
			 node = node.nextSibling;
		 }
	}
});