/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils"); //$NON-NLS-1$

(function(){
		
dojo.require("com.ibm.relm.common.web.enums.EnumManagerDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.enums.EnumStore"); //$NON-NLS-1$


var EnumManagerDialog = com.ibm.relm.common.web.enums.EnumManagerDialog;
var EnumStore = com.ibm.relm.common.web.enums.EnumStore;
	
com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils = {};

var enumUtils = com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils;

dojo.mixin(enumUtils, 
{
	isEnumType: function(type, enums) {
		if (type == "enum") { //$NON-NLS-1$
			return true;
		} else if (!enums) {
			return false;
		}
		
		return dojo.some(enums, function(e) {
			if (e.uri == type) {
				isEnum = true;
				return true;
			}
		});
	},
	
	getEnumCategory: function(type, enums) {
		return this.isEnumType(type, enums) ? type : ""; //$NON-NLS-1$
	},
	
	loadEnum: function(onLoadCallback, onErrCallback) {
		var enumStore = (new EnumStore()).load();
	 	//If there is no deferredLoading, then we can assume the enums are ready to use
		if (!enumStore) {
			return;
		}
		enumStore.then(onLoadCallback, onErrCallback);
	},

	makeEnumValueOptions: function(values, novalueLabel) {
		if (!novalueLabel) {
			return values;
		}
		var first = [{label: novalueLabel, value: ""}]; //$NON-NLS-1$
		return first.concat(values);
	},
	
	getEnumValueOptions: function(enums, type, novalueLabel) {
		return this.makeEnumValueOptions(this.getEnumValuesForType(enums, type), novalueLabel);
	},
	
	getEnumValuesForType: function(enums, type) {
		var target = null;
		for (var i = 0; i < enums.length; i++) {
			if (enums[i].uri == type) {
				target = enums[i];
				break;
			}
		}
		
		return this.getEnumValues(target);
	},

	/*
	 * Collect enumeration types 
	 */
	getEnumTypes: function(enums) {
		var values = [];
		dojo.forEach(enums, function(e) {
			values.push({label: e.label, value: e.uri});
		});
		return values;
	},
	
	/*
	 * Collect enumeration values corresponding to given enumeration type.
	 * This returns array in {label, value} manner.
	 */
	getEnumValues: function(e) {
		if (!e) {
			return [];
		}

		var self = this;
		var values = [];
		dojo.forEach(e.members, function(m) {
			values.push({label: m.label, value: self.getEnumValue(m), rawvalue: m});
		});
		return values;
	},
	
	getEnumValue: function(m) {
		if (!m) return null;
		return m.substitution || m.uri;
	},
	
	getEnumRawValue: function(type, e, value) {
		var values = this.getEnumValuesForType(e, type);
		if (!values) return null;
		
		for (var i = 0; i < values.length; i++) {
			var v = values[i];
			if (v.value == value) {
				return v.rawvalue;
			}
		}
		return null;
	},
	
	updateParam: function(param, substitution, uri) {
		param.value = {substitution: substitution, uri: uri};
	},
	
	updateParamByValue: function(param, rawvalue) {
		var substitution = rawvalue ? rawvalue.substitution : ""; //$NON-NLS-1$
		var uri = rawvalue ? rawvalue.uri : ""; //$NON-NLS-1$
		this.updateParam(param, substitution, uri);
	},
	
	equals: function(p1, p2) {
		return p1.enumtype == p2.enumtype && p1.value.uri == p2.value.uri && p1.value.substitution == p2.value.substitution;
	},
	
	openEnumerationManagerDialog : function(onSaveCallback, onCancelCallback) {
		var d = new EnumManagerDialog();
		dojo.connect(d, 'onSave', function(store) { //$NON-NLS-1$
			onSaveCallback(store.getEnums());
		});
		dojo.connect(d, 'onCancel', function() { //$NON-NLS-1$
			onCancelCallback();
		});
		d.startup();
	}
	
});
})();
