/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.ProjectAreaSelect"); //$NON-NLS-1$

dojo.require("dijit.form.ComboBox");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util");//$NON-NLS-1$
dojo.require("dojo.store.Memory");//$NON-NLS-1$
dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.SelectionDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ProjectAreaSelectMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function()  {
	
var SPARQL_FOR_PROJECT_AREA = "PREFIX oslc_cm: <http://open-services.net/ns/cm#> \n" +  //$NON-NLS-1$
		"PREFIX oslc_cm_x: <http://open-services.net/ns/cm-x#> \n" + //$NON-NLS-1$
		"SELECT distinct ?project \n" +  //$NON-NLS-1$
		"WHERE {  \n" +  //$NON-NLS-1$
		"?resource oslc_cm_x:project ?project .  \n" +  //$NON-NLS-1$
		"}  \n"; //$NON-NLS-1$

var SelectionDialog = com.ibm.relm.explorer.web.ui.internal.view.common.SelectionDialog;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.common.ProjectAreaSelect", //$NON-NLS-1$
	dijit.form.ComboBox,
{
	// labelAttr is used for labels in dropdown list
	// searchAttr is used for text in text box and value
	labelAttr: "title", //$NON-NLS-1$
	searchAttr: "title", //$NON-NLS-1$
	
	_projectAreas: null,
	_propertyRetrieved: false,
	
	postMixInProperties : function(args) {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ProjectAreaSelectMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	
	getTitle: function(){
		return  this.item ? this.item.title : null;
	},
	
	getUri: function(){
		return  this.item ? this.item.uri : null;
	},
	
	// not used now. need use FilteringSelect instead of ComboBox
	valueAttr: "uri", //$NON-NLS-1$
	
	postCreate: function(){
		this.inherited(arguments);
		this._populateProjectAreas();
	},

	_populateProjectAreas: function(){
		var self = this;
		self._projectAreas = [];
		self.uris = [];
		var xhrPost=jazz.client.xhrPost({
			url: com.ibm.relm.explorer.web.ui.queryServiceUri,
			headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
			content: {
				sparql: SPARQL_FOR_PROJECT_AREA
			},
			load: function(response, ioArgs){
				var projectAreaUris = response.results.bindings;
				if(projectAreaUris && projectAreaUris.length > 0) {
					dojo.forEach(projectAreaUris, function(p){
						self.uris.push(p.project.value);
					});
				}
			},
			handle: function() {
				self.retrieveServerList();
				if(self.numHost > 1) {
					// select server
					var servers = [];
					for(var server in self.urisByHost) {
						servers.push(server);
					}
					SelectionDialog.openServerSelectionDialog(self.messages.serverDialogTitle, servers, function(value){
						self.getProjectProperties(value.server);
					});
				} else if(self.numHost == 1){
					// 1 project area only : selection dialog is not shown
					self.getProjectProperties(self.checkedHostname);
				} else {
					// no project area found
					alert(self.messages.noProjectArea);
				}
			},
			handleAs: "json" //$NON-NLS-1$
		}, true);
	},
	
	retrieveServerList: function() {
		var self = this;
		self.numHost = 0;
		self.urisByHost = {};
		for(var i = 0; i < self.uris.length; i++) {
			var uri = self.uris[i];
			var hostTmp = (uri.indexOf("http://") == 0) ? uri.substr(7) : (uri.indexOf("https://") == 0) ? uri.substr(8) : uri; //$NON-NLS-1$ //$NON-NLS-2$
			hostTmp = hostTmp.substring(0, hostTmp.indexOf("/")); //$NON-NLS-1$
			if(hostTmp.indexOf(":") > 0 ) { //$NON-NLS-1$
				hostTmp = hostTmp.substring(0, hostTmp.indexOf(":")); //$NON-NLS-1$
			}
			var host = hostTmp;
			self.addUriByHost(host, uri);
			self.checkedHostname = host;
		}
	},
	
	addUriByHost: function(host, uri) {
		var self = this;
		if(!self.urisByHost[host]) {
			self.urisByHost[host] = [];
			self.numHost++;
		}
		self.urisByHost[host].push(uri);
	},
		
	getProjectProperties: function(hostname) {
		var self = this;
		var projectUris = self.urisByHost[hostname];
		
		for(var i = 0; i < projectUris.length; i++) {

			var xhrGet=jazz.client.xhrGet({
				url: com.ibm.relm.explorer.web.ui.serviceRoot + "proxy?uri=" + encodeURI(projectUris[i]), //$NON-NLS-1$
				headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
				handleAs: "json", //$NON-NLS-1$
				load: function(response, ioArgs) {
					self._propertyRetrieved = true;
					self._projectAreas.push({uri: response["rdf:resource"],  //$NON-NLS-1$, 
						title: response["dc:title"],//$NON-NLS-1$
						description: response["dc:description"], //$NON-NLS-1$
						archived: response["rtc_cm:archived"]});	 //$NON-NLS-1$ //$NON-NLS-2$
				},
				error: function(response, ioArgs) {
					self._propertyRetrieved = false;
					self.uris = projectUris;
				},
				handle: function(){
					self.refreshList(false); // default: not include archived projectarea
				}
			}, true);
		}
		
	},

	refreshList: function(fIncludeArchived) {
		var self = this;
		var items = null;
		if(!self._propertyRetrieved){
			// when project area label (and archived property) can not retrieved
			items = [];
			for(var i = 0; i < self.uris.length; i++) {
				items.push({uri: self.uris[i], title: self.uris[i]});
			}
		}else if(!fIncludeArchived) {
			// exclude archived project area (default)
			items = dojo.filter(self._projectAreas, function(item) {
				return item.archived == false;
			});
		}else {
			// list all project area
			items = self._projectAreas;
		}
		var tmplStore = new dojo.store.Memory({
			data: items,
			idProperty: self.valueAttr
		});
		self.set("store",tmplStore); //$NON-NLS-1$
		self.set("fetchProperties", {sort: [{attribute:"title", descending: false}]}); //$NON-NLS-1$) //$NON-NLS-2$

	}
});

}) ();