/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.RequestQueue"); //$NON-NLS-1$

(function(){
	var REQUEST_DELAY_TIME = 10;
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.common.RequestQueue", null, { //$NON-NLS-1$
		addToCacheCallback: null,	
		findInCacheCallback: null,	
		clearCache: null,	
		_requestQueue: null,
		_timerHandle: null,
		
		constructor : function(args) {
			dojo.mixin(this, args);
		},
		
		destroy: function() {
			this.cancelAll();
			if (dojo.isFunction(this.clearCache)) {
				this.clearCache();
			}
			this.addToCacheCallback = null;
			this.findInCacheCallback = null;
			this.clearCache = null;
			this.inherited(arguments);
		},
		
		request: function(request) {
			var deferred = new dojo.Deferred();
			if (this._findInCache(request)) {
				deferred.callback();
			} else {
				var requestObject = {deferred: deferred, request: request};
				if (!this._requestQueue) this._requestQueue = [];
				this._requestQueue.push(requestObject);
				if (this._requestQueue.length == 1)
					this._postRequest();
			}
			return deferred;
		},
		
		cancelAll: function() {
			if (this._timerHandle) {
				clearTimeout(this._timerHandle);
				this._timerHandle = null;
			}
			dojo.forEach(this._requestQueue, function(req) {
				req.deferred.cancel();
			});
			this._requestQueue = null;
		},
		
		_run: function() {
			this._timerHandle = null;
			if (this._requestQueue.length == 0) {
				return;
			}
			var requestObject = this._requestQueue[0];
			var deferred = requestObject.deferred;
			var request = requestObject.request;
			
			this._requestQueue.splice(0, 1);

			if (this._findInCache(request) || !dojo.isFunction(request.func)) {
				this._postRequest();
				deferred.callback();
				return;
			}
			
			var self = this;
			request.func(request.params).then(function(result) {
				if (result) self._addToCache(result);
				request.callback && request.callback(result);
				self._postRequest();
				deferred.callback();
			}, function(error) {
				request.errback && request.errback(error);
				self._postRequest();
				deferred.callback();
			});
		},
		
		_findInCache: function(request) {
			if (dojo.isFunction(this.findInCacheCallback)) {
				var result = this.findInCacheCallback(request);
				if (result) {
					request.callback(result);
					return true;
				}
			}
			return false;
		},
		
		_addToCache: function(result) {
			if (dojo.isFunction(this.addToCacheCallback))
				this.addToCacheCallback(result);
		},
		
		_postRequest: function() {
			if (this._requestQueue && this._requestQueue.length > 0) {
				this._timerHandle = setTimeout(dojo.hitch(this, this._run), REQUEST_DELAY_TIME);
			}
		}
});
})();
