/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.CommandProvider"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.CommandProvider"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.loader.Config"); //$NON-NLS-1$


// ----------------------------------------------------------------------------
// Compose multiple commands into a single command
// ----------------------------------------------------------------------------

(function(){
com.ibm.relm.explorer.web.ui.internal.view.edit.CommandProvider = {
		
	createClipbardData: function(dataFormat, data) {
		if (dataFormat == "config") { //$NON-NLS-1$
			var config = com.ibm.relm.explorer.web.ui.internal.view.loader.Config();
			config.viewUri = data.viewUri;
			config.queryTemplates = dojo.clone(data.queryTemplates);
			config.connectionLinks = dojo.clone(data.connectionLinks);
			return config;
		}
		return data;
	},
	
	createCommand: function(commandinfo) {
		var command = null;
		switch(commandinfo.type) {
		case "delete": //$NON-NLS-1$
			command = this.createDeleteCommand(commandinfo);
			break;
		case "paste": //$NON-NLS-1$
			command = this.createPasteCommand(commandinfo);
			break;
		default:
			break;
		}
		if (command) return command;
		return com.ibm.relm.explorer.web.base.internal.canvas.command.CommandProvider.createCommand(commandinfo);
	},
	
	createPasteCommand: function(commandinfo) {
		console.debug("paste: " + commandinfo.layout); //$NON-NLS-1$
		var compoundCommand = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		compoundCommand.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.PasteLayoutObjectCommand(
				commandinfo.layout, commandinfo.config, commandinfo.target, commandinfo.ref));
		return compoundCommand;
	},
	
	createDeleteCommand: function(commandinfo) {
		var widget = commandinfo.widget;
		if (!(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer)) {
			return null;
		}
			
		var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
		var compoundCommand = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();

		// remove connections
		var mgr = com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager;
		var connections = mgr.getTargetConnections(widget);
		dojo.forEach(connections, function(connection) {
			compoundCommand.append(new ec.RemoveChildCommand(connection));
		});

		// Remove all implicit containers defined for indirect link
		if (widget.isDownstream()) {
			dojo.forEach(this._getHiddenContainers(widget, false), function(container) {
				compoundCommand.append(new ec.RemoveChildCommand(container));
			});
		}
		if (widget.hasDownstream()) {
			dojo.forEach(this._getHiddenContainers(widget, true), function(container) {
				compoundCommand.append(new ec.RemoveChildCommand(container));
			});
		}
		
		// remove itself
		compoundCommand.append(new ec.RemoveChildCommand(widget));
		
		// Update all affected containers
		var lastCommand = null;
		dojo.forEach(this._getAffectedContainers(widget), function(container) {
			if (!lastCommand) lastCommand = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
			lastCommand.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(container));
		});
		if (lastCommand)
			compoundCommand.setLastCommand(lastCommand);
		
		return compoundCommand;
	},
	
	_getHiddenContainers: function(widget, downward, hiddenContainers) {
		var viewer = widget._viewer;
		dojo.forEach(downward ? widget.getDownstream() : widget.getUpstream(), function(link) {
			var container = viewer.findContainer(link.id);
			if (container && container.getHidden()) {
				if (!hiddenContainers) hiddenContainers = [];
				hiddenContainers.push(container);
				hiddenContainers = this._getHiddenContainers(container, downward, hiddenContainers);
			}
		}, this);
		return hiddenContainers;
	},
	
	_getAffectedContainers: function(widget, affectedContainers) {
		var viewer = widget._viewer;
		dojo.forEach(widget.getDownstream(), function(link) {
			var container = viewer.findContainer(link.id);
			if (container) {
				if (container.getHidden()) {
					affectedContainers = this._getAffectedContainers(container, affectedContainers);
				} else {
					if (!affectedContainers) affectedContainers = [];
					affectedContainers.push(container);
				}				
			}
		}, this);
		return affectedContainers;
	}
	
};
})();
