/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$

(function(){

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkPathFinder"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "NodesMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var _nodeMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "NodesMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
var _pageMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$

var SectionDialog = com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog;

com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions = {
	_defaultArtifactElementLayoutParam: {
			layoutManager: "GridLayout",//$NON-NLS-1$
			width: 100,
			height: 100,
			columns: 1, 
			spacing: 5, 
			margin: 5, 
			halign: "center", //$NON-NLS-1$
			fill: "#eeeeee", //$NON-NLS-1$
			pack: true
		},

	/*
	 * Actions
	 */
		
	/**
	 * Get link specific actions
	 */
	getLinkedLineActions: function(viewer, container, line) {
		if (!line.links || line.links.length == 0) {
			return [];
		}
		
		var self = this;
		var link = line.links[line.links.length - 2];
		var targetLink = line.targetLink;
		
		var hidelines = targetLink.hidelines;
		var unlimited = targetLink.unlimited;
		
		var actions = [];

		// show or hide connection lines
		 actions.push({
			 nodeWidget: line,
			 label: _nodeMessages.showLinesLabel,
			 selected: !hidelines,
			 action: function () {
				 self.changeLinkProperties(viewer, container, line, targetLink, {hidelines: !hidelines});
				 if (!container.getHidden())
					 viewer.setFocus(container);
			 }
		 });
		 
		 // show all nodes or linked nodes only
		 actions.push({
			 nodeWidget: line,
			 label: _nodeMessages.showLinkedNodeOnlyLabel,
			 selected: !unlimited,
			 action: function () {
				 self.changeLinkProperties(viewer, container, line, link, {unlimited: !unlimited});
				 if (!container.getHidden())
					 viewer.setFocus(container);
			 }
		 });
		 
		 // edit connection
		 actions.push({
			 nodeWidget: line,
			 label: _nodeMessages.editConnectionLabel,
			 iconClass: "edit-icon", //$NON-NLS-1$
			 action: function () {
				 var connectionId = targetLink ? targetLink.connectionId : null;
				 self.showAndEditConnection(viewer, container, line, connectionId);
			 },
		 });
		 
		actions.push("separator"); //$NON-NLS-1$
		
		// show hidden containers
		if (line.links.length > 2) {
			actions.push({
				 nodeWidget: line,
				 label: _pageMessages.showContainer,
				 action: function () {
					 self.showContainer(viewer, line.links);
				 }			
			 });
		}
		
		// remove link
		actions.push({
			 nodeWidget: line,
			 label: _nodeMessages.removeLinkLabel,
			 actionType: 'remove', //$NON-NLS-1$
			 iconClass: "delete-icon", //$NON-NLS-1$
			 action: function () {
				 var source = viewer.findContainer(link.id);
				 self.removeLinkToContainer(viewer, source, container, link.linktype, line.links);
				 if (!container.getHidden())
					 viewer.setFocus(container);
			 }
		 });
		 return actions;
	},

	/**
	 * Get linked container specific actions
	 */
	getLinkedContainerActions: function(viewer, parent, thisContainer, editmode) {
		if (!editmode)
			return [];
	
		var actions = [];

		var linkProperties = thisContainer._linkProperties;
		if (!linkProperties || linkProperties.length == 0) {
			return [];
		}
		
		var action = {
			actionType: 'childcontainer', //$NON-NLS-1$
			isGroup: true,
			label: _nodeMessages.showLinkLabel,
			iconClass: 'artifactelement-icon', //$NON-NLS-1$
		};
		actions.push(action);

		var showLinkToActions = [];
		var linkMap = this._getShowLinkToGroups(linkProperties);
		for (var id in linkMap) {
			var linkGroup = linkMap[id];
			if (linkGroup.links.length == 1) {
				var link = linkGroup.links[0];
				var linkLabel = link.label;
				var label = linkGroup.title + ' (' + linkLabel + ')';  //$NON-NLS-2$ //$NON-NLS-1$
				showLinkToActions.push(this._createShowLinkToAction(viewer, parent, thisContainer, link, label));
			} else if (linkGroup.links.length > 1) {
				// Collect "Show Links To" items which have other links for the same type of artifact element 
				var link = linkGroup.links[0];
				showLinkToActions.push({
					actionType: 'childcontainer', //$NON-NLS-1$
					isGroup: true,
					label: linkGroup.title,
					linkId: link.id,
					actions: dojo.map(linkGroup.links, function(link) {
						return this._createShowLinkToAction(viewer, parent, thisContainer, link);
					}, this)
				});
			}
		}
		showLinkToActions.sort(function(a, b) {
			return a.linkId.localeCompare(b.linkId);
		});

		action.actions = showLinkToActions;

		if (thisContainer.isDownstream() && thisContainer.hasDownstream()) {
			actions.push({
				label: _pageMessages.hideContainer,
				action: dojo.hitch(this, function() {
					this.hideContainer(viewer, thisContainer);
				})
			});
		}
		return actions;
	},
	
	_getShowLinkToGroups: function(linkProperties) {
		var linkMap = {};
		dojo.forEach(linkProperties, function(linkprop) {
			var linkGroup = linkMap[linkprop.id];
			if (!linkGroup)
				linkGroup = linkMap[linkprop.id] = {title: linkprop.title, links: []};
			linkGroup.links.push(linkprop);
		});
		return linkMap;
	},
	
	_createShowLinkToAction: function(viewer, parent, thisContainer, connection, label) {
		var name = connection.title || connection.label;
		return {
			actionType: 'childcontainer', //$NON-NLS-1$
//			iconClass: 'artifactelement-icon', //$NON-NLS-1$
			label: label || connection.label,
			linkId: connection.id,
			action: dojo.hitch(this, function() {
				var componentProp = this.getContainerParam(viewer, {
					_viewer: viewer,
					_containerId: viewer.createUniqueContainerId(name),
					elementId: connection.id,
					name: name,
					left: thisContainer.left + thisContainer.width + 50,
					top: thisContainer.top
				});
				this.createLinkedContainer(viewer, parent, thisContainer, connection.sourceProperty, componentProp);
			})
		};
	},
	
	/**
	 * Artifact Elements Container Properties
	 */
	getContainerParam: function(viewer, optionParam) {
		var param = dojo.clone(this._defaultArtifactElementLayoutParam);
		dojo.mixin(param, optionParam);
		param.onCreate = dojo.hitch(this, function(container) {
			this._createTitle(container, viewer, optionParam.name);
		});
		return param;
	},
	
	/**
	 * Artifact Elements Container Title Node
	 */
	_createTitle: function(container, diagramEditor, title) {
		var textNode = diagramEditor.createComponent('text', //$NON-NLS-1$ 
				{left: container.left, top: container.top - 40, text: title, _viewer: diagramEditor});
		container.addChild(textNode, 0);
	},
	
	showAndEditConnection: function(viewer, container, line, connectionId) {
		 var config = dojo.clone(viewer.getConfig());

		 // default value
		 var use = "containerlink"; //$NON-NLS-1$
		 var index = -1;
		 if (connectionId && config.connectionLinks) {
			 for (var i = 0; i < config.connectionLinks.length; i++) {
				 if (use == config.connectionLinks[i].use && config.connectionLinks[i].id == connectionId) {
					 index = i;
					 break;
				 }
			 }
		 } else {
			 connectionId = line.links[0].id + "__" + line.links[line.links.length - 1].id + "__connection"; //$NON-NLS-2$ //$NON-NLS-1$
		 }
		 
		 // open dialog
		 var additions = {id: connectionId, title: line.title, use: use}; //$NON-NLS-1$
		 
		 var self = this;
		 SectionDialog.openConnectionDialog({
			 additions: additions,
			 getConfig: function(){ return config; },
			 getDiagramEditor: function(){ return viewer; },
			 _updateDirty: function() {}
		 }, index, function(selected) {
			 self.updateConnection(viewer, container, line, line.targetLink, config, selected);
		 });
	},
	
	/*
	 * Link related command executions
	 */
	
	/**
	 * 
	 */
	changeParameters: function(viewer, container, prop) {
		viewer.removeHalfFocusFrame(container);
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
		for(var name in prop){
			var propName = name;
			var propValue = prop[name];
			command.append(new ec.SetPropertyValueCommand(container, propName, propValue));
		}
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(container));
		viewer.executeCommand(command);
	},
	
	createLinkPath: function(viewer, source, target, connections, linkparam) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();

		var parent = source.getParent();
		var upstream = source;
		var index = 0;
		for (; index < connections.length - 1; index++) {
			var connection = connections[index];
			if (!connection.sourceProperty) continue;

			var name = connection.target || connection.title || connection.label;
			var onCreate = dojo.hitch(this, function(container) {
				this._createTitle(container, viewer, container.name);
			});
			var componentparam = {
					_viewer: viewer,
					_containerId: viewer.createUniqueContainerId(name),
					layoutManager: "HiddenLayout",  //$NON-NLS-1$
					elementId: connection.id, 
					name: name, 
					hidden: true,
					onCreate: onCreate
			};
			var downstream = viewer.createComponent('container', componentparam);  //$NON-NLS-1$
			command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.AddNodeContainerLinkCommand(upstream, downstream, connection.sourceProperty));
			command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.AppendChildCommand(parent, downstream));
			upstream = downstream;
		}
		
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.AddNodeContainerLinkCommand(upstream, target, connections[index].sourceProperty, linkparam));
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(target));
		viewer.executeCommand(command);
		viewer.setFocus(target);
	},

	/**
	 * Add a link between source and target containers
	 */
	addLinkToContainer: function(viewer, source, target, linktype, linkparam) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.AddNodeContainerLinkCommand(source, target, linktype, linkparam));
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(target));
		viewer.executeCommand(command);
	},

	/**
	 * Remove a link between source and target containers
	 */
	removeLinkToContainer: function(viewer, source, target, linktype, linkpath) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.RemoveNodeContainerLinkCommand(source, target, linktype));
		if (linkpath) {
			for (var i = linkpath.length - 2; i > 0; i--) {
				var container = viewer.findContainer(linkpath[i].id);
				if (container)
					command.append(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.RemoveChildCommand(container));
			}
		}
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(target));
		viewer.executeCommand(command);
	},

	/**
	 * Show/Hide a link between source and target containers
	 */
	changeLinkProperties: function(viewer, container, line, link, linkparam) {
		var self = this;
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.SetLinkPropertyCommand(container, link, linkparam,
				function(oldProps, newProps) {
					self._firePropertyChanged(viewer, line, oldProps, newProps);
				}));
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(container));
		viewer.executeCommand(command);
	},
	
	/**
	 * Change appearance of connections
	 */
	updateConnection: function(viewer, container, line, link, config, connection) {
		var self = this;
		var propName = "connectionLinks"; //$NON-NLS-1$
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.SimpleUpdateConfigCommand(viewer.getConfig(), 
				propName, config[propName], function() { viewer.onConfigLoaded(viewer.getConfig()); }));
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.SetLinkPropertyCommand(container, link, {connectionId: connection.id}, 
				function(oldProps, newProps) {
					self._firePropertyChanged(viewer, line, oldProps, newProps);
				}));
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(container));
		viewer.executeCommand(command);
	},
	
	_firePropertyChanged: function(viewer, widget, oldProps, newProps) {
		var names = [];
		if (oldProps) {
			for (var name in oldProps) {
				if (dojo.indexOf(names, name) < 0)
					names.push(name);
			}
		}
		if (newProps) {
			for (var name in newProps) {
				if (dojo.indexOf(names, name) < 0)
					names.push(name);
			}
		}
		dojo.forEach(names, function(name) {
			viewer.firePropertyChanged(name, oldProps ? oldProps[name] : undefined, newProps ? newProps[name] : undefined, {widget:widget});
		});
	},
	
	/**
	 * Show hidden container
	 */
	showContainer: function(viewer, linkpath) {
		var updateContainer = null;
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		for (var i = 1; i < linkpath.length - 1; i++) {
			var container = viewer.findContainer(linkpath[i].id);
			if (!container) continue;
			if (!updateContainer) updateContainer = container;
			
			var layoutParam = null;
			if (container.columns === undefined) {
				layoutParam = this._defaultArtifactElementLayoutParam;
			} else {
				layoutParam = {layoutManager: "GridLayout"}; //$NON-NLS-1$
			}
			command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.ChangeLayoutCommand(container, layoutParam));
		}
		command.setLastCommand(new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.UpdateCommand(viewer, "reloadLocal")); //$NON-NLS-1$
		
		viewer.executeCommand(command);
	},
	
	/**
	 * Hide container
	 */
	hideContainer: function(viewer, container) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.ChangeLayoutCommand(container, {layoutManager: "HiddenLayout"}, function() { //$NON-NLS-1$
			viewer.reloadLocal();
		}));
		viewer.executeCommand(command);
	},
	
	createLinkedContainer: function(viewer, parent, source, linktype, componentparam, linkparam) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		var target = viewer.createComponent('container', componentparam);  //$NON-NLS-1$
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.AddNodeContainerLinkCommand(source, target, linktype, linkparam));
		command.append(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.AddLinkedContainerCommand(parent, target));
		viewer.executeCommand(command);
		if (!componentparam.hidden)
			viewer.setFocus(target);
	}
};

})();

		