/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.DecorationPropertyDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
dojo.require("dijit.form.RadioButton"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DecorationPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container

	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.DecorationPropertyDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 
		widgetsInTemplate: true,
		
		decorationProperty : null,
		
		fPreset: true,
		
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/DecorationPropertyDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor: function(){
			this._presetComboId = dijit.getUniqueId("presetCombo");//$NON-NLS-1$
			this._typeSelectionId = dijit.getUniqueId("typeSelection");//$NON-NLS-1$
			this._presetRadioId = dijit.getUniqueId("presetRadio");//$NON-NLS-1$
			this._customRadioId = dijit.getUniqueId("customRadio");//$NON-NLS-1$
			this._pathTextBoxId = dijit.getUniqueId("pathTextBox");//$NON-NLS-1$
			this._fillTextBoxId = dijit.getUniqueId("fillTextBox");//$NON-NLS-1$
			this._strokeTextBoxId = dijit.getUniqueId("strokeTextBox");//$NON-NLS-1$
			this._scaleTextBoxId = dijit.getUniqueId("scaleTextBox");//$NON-NLS-1$
		},

		postMixInProperties : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DecorationPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.inherited(arguments);
		},
		
		postCreate : function() {
			
			this.inherited(arguments);
			this.loadPresetSelectDropdown(this.comboBox_preset);
			this.setProperty(this.decorationProperty);

		},
		
		setProperty : function(value) {
			try {
				value = dojo.fromJson(value);
			} catch (e) {
				if(!value) {
					value = { preset: ""}; //$NON-NLS-1$
				} else {
					value = { preset: value};
				}
			}
			this._setDecorationPropertyValues(value);

		},
		
		_onHandlePreset: function() {
			this._setPresetActive(true);
		},
		
		_onHandleCustom: function() {
			this._setPresetActive(false);
		},
		
		_setPresetActive: function(fPreset) {
			this.fPreset = fPreset;
			if(fPreset) {
				this.presetRow.style.display = ""; //$NON-NLS-1$
				this.pathRow.style.display = "none"; //$NON-NLS-1$
				this.fillRow.style.display = "none"; //$NON-NLS-1$
				this.strokeRow.style.display = "none"; //$NON-NLS-1$
				this.scaleRow.style.display = "none"; //$NON-NLS-1$
				
				this.presetRadio.set("checked", true); //$NON-NLS-1$
				this.customRadio.set("checked", false); //$NON-NLS-1$
			} else {
				this.presetRow.style.display = "none"; //$NON-NLS-1$
				this.pathRow.style.display = ""; //$NON-NLS-1$
				this.fillRow.style.display = ""; //$NON-NLS-1$
				this.strokeRow.style.display = ""; //$NON-NLS-1$
				this.scaleRow.style.display = ""; //$NON-NLS-1$

				this.presetRadio.set("checked", false); //$NON-NLS-1$
				this.customRadio.set("checked", true); //$NON-NLS-1$
			}
		},
		
		_editProperty: function(type){
			var self = this;
			com.ibm.relm.explorer.web.ui.internal.util.openPicker(type, self["textBox_" + type].get("value"), null, function(value){//$NON-NLS-1$//$NON-NLS-2$
				self._okPropertyCallback(type, value); //$NON-NLS-1$				
			}, self);			
		},
		
		_editPath: function() {
			this._editProperty("path");//$NON-NLS-1$
		},
		
		_editFill: function() {
			this._editProperty("fill");//$NON-NLS-1$
		},
		
		_editStroke: function() {
			this._editProperty("stroke");//$NON-NLS-1$
		},

		_okPropertyCallback: function(attrName, value) {
			
			switch(attrName) {
			case "path": //$NON-NLS-1$
				this.textBox_path.set("value", value); //$NON-NLS-1$
				break;
			case "fill": //$NON-NLS-1$
				this.textBox_fill.set("value", value); //$NON-NLS-1$
				break;
			case "stroke": //$NON-NLS-1$
				this.textBox_stroke.set("value", value); //$NON-NLS-1$
				break;
			}
		},
		
		_setDecorationPropertyValues : function(value) {

			if(value.preset) {
				this.comboBox_preset.set("value", value.preset); //$NON-NLS-1$
			}
			if(value.path){
				this.textBox_path.set("value", value.path); //$NON-NLS-1$
			}
			var fill = value.fill;
			if(fill){
				try{
					fill = dojo.toJson(fill);
				} catch(e) {

				}
			}
			this.textBox_fill.set("value", fill); //$NON-NLS-1$
			var stroke = value.stroke;
			if(stroke) {
				try{
					stroke = dojo.toJson(stroke);
				} catch(e) {

				}
			}
			this.textBox_stroke.set("value", stroke); //$NON-NLS-1$
			if(!value.scale) value.scale = 1;
			this.textBox_scale.set("value", value.scale); //$NON-NLS-1$
			if(value.path && value.path != null) {
				this._setPresetActive(false);
			} else {
				this._setPresetActive(true);
			}
		},
		
		loadPresetSelectDropdown: function(comboBoxWidget) {
			var _preset = [];
			_preset.push({value: "", label: this.messages.noValueSpecified}); //$NON-NLS-1$
			_preset.push({value: "arrow", label: this.messages.arrow}); //$NON-NLS-1$
			_preset.push({value: "white_arrow", label: this.messages.whiteArrow}); //$NON-NLS-1$
			_preset.push({value: "filled_arrow", label: this.messages.filledArrow}); //$NON-NLS-1$
			_preset.push({value: "arrow_s", label: this.messages.arrows}); //$NON-NLS-1$
			_preset.push({value: "white_arrow_s", label: this.messages.whiteArrows}); //$NON-NLS-1$
			_preset.push({value: "filled_arrow_s", label: this.messages.filledArrows}); //$NON-NLS-1$
			_preset.push({value: "white_diamond", label: this.messages.whiteDiamond}); //$NON-NLS-1$
			_preset.push({value: "filled_diamond", label: this.messages.filledDiamond}); //$NON-NLS-1$
			var tmplStore = new dojo.store.Memory({
				data: _preset
				});
			comboBoxWidget.set("labelAttr", "label"); //$NON-NLS-1$ //$NON-NLS-2$
			comboBoxWidget.set("searchAttr", "value"); //$NON-NLS-1$ //$NON-NLS-2$
			comboBoxWidget.set("store", tmplStore); //$NON-NLS-1$
			
		},
		
		getValues: function() {

			var retValue = {};

			if(this.fPreset ) { 
				retValue.preset = this.comboBox_preset.get("value"); //$NON-NLS-1$
			} else {
				retValue.path = this.textBox_path.get("value"); //$NON-NLS-1$
				var fill = this.textBox_fill.get("value"); //$NON-NLS-1$
				try {
					fill = dojo.fromJson(fill);
				} catch(e) {
				}
				retValue.fill = fill; //$NON-NLS-1$
				var stroke = this.textBox_stroke.get("value"); //$NON-NLS-1$
				try {
					stroke = dojo.fromJson(stroke);
				} catch(e) {
				}
				retValue.stroke = stroke; //$NON-NLS-1$
				retValue.scale = this.textBox_scale.get("value"); //$NON-NLS-1$
			}
			
			return dojo.toJson(retValue);
		}

	});
})();
