/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand"); //$NON-NLS-1$

// ----------------------------------------------------------------------------
// Command classes which encapsulate a request as an object
// ----------------------------------------------------------------------------

var collectContainers = function(widgets, containers) {
	if (!containers) containers = [];
	dojo.forEach(widgets, function(w) {
		if (w instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer) {
			containers.push(w);
			containers = collectContainers(w.getChildren(), containers);
		}
	});
	return containers;
};

var createUniqueId = function(prefix, ids) {
	if (dojo.indexOf(ids, prefix) < 0)
		return prefix;
	
	var index = dojo.lastIndexOf(prefix, '_'); //$NON-NLS-1$
	if (index >= 0) {
		var suffix = prefix.substr(index + 1);
		if (suffix == "" || /^\d+$/.test(suffix)) //$NON-NLS-1$
			prefix = prefix.substring(0, index);
	}

	for (var i = 1; ; i++) {
		var id = prefix + '_' + i; //$NON-NLS-1$
		if (dojo.indexOf(ids, id) < 0)
			return id;
	}
	return null; //$NON-NLS-1$
};

com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand = {
	UpdateCommand: function(widget){
		this._widget = widget;

		this.execute = function(){
			var widget = this._widget;
			if(dojo.isFunction(widget.hlp_getEditor)){
				var editor = widget.hlp_getEditor();
//				editor.removeFocusFrame(widget); // children might hold the focus now. remove it.
				if(widget.setup) {
					widget.setup(true);
				}
				for(var n = widget; n; n =  n.getParent()){
					if(dojo.isFunction(n.layout)){
						n.layout();
					}
					if(!dojo.isFunction(n.getParent)) break;
				}
				editor.fireLoadResource(); // for reload connections
				widget.update();
			}	
		};
		this.undo = this.execute;
	},
	
	SimpleUpdateConfigCommand: function(config, name, value, onEnd){
		this._config = config;
		this._name  = name;
		this._value = value;
		this._onEnd = onEnd;
	
		this.execute = function(){
			var oldValue = this._config[this._name];
			this._config[this._name] = this._value;
			this._value = oldValue;
			if(dojo.isFunction(this._onEnd)){
				this._onEnd();
			}
		};
	
		this.undo = function(){
			this.execute();
		};
	},

	AddLinkedContainerCommand: function(parent, child){
		this._parent = parent;
		this._child = child;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._child);
		};
	
		this.undo = function(){
			this._child.clearConnections();
			this._parent.hlp_removeChild(this._child);
		};
	},
	
	AddNodeContainerLinkCommand: function(source, target, linktype, properties){
		this._editor = target.hlp_getEditor();
		this._sourceId = source.getId();
		this._targetId = target.getId();
		this._linktype = linktype;
		this._source = source;
		this._target = target;
	
		this.execute = function(){
			if (!this._sourceId && !this._targetId)
				return;
			
			this._editor.addLink(this._sourceId, this._targetId, this._linktype, properties);
			var container = this._editor.findContainer(this._targetId);
			if (container) container._query = null;
		};
	
		this.undo = function(){
			if (!this._sourceId && !this._targetId)
				return;

			this._editor.removeLink(this._sourceId, this._targetId, this._linktype);
			var container = this._editor.findContainer(this._targetId);
			if (container) container._query = null;
		};
	},
	
	RemoveNodeContainerLinkCommand: function(source, target, linktype){
		this._editor = target.hlp_getEditor();
		this._sourceId = source.getId();
		this._targetId = target.getId();
		this._linktype = linktype;
	
		this.execute = function(){
			if (!this._sourceId && !this._targetId)
				return;

			this._editor.removeLink(this._sourceId, this._targetId, this._linktype);
			var container = this._editor.findContainer(this._targetId);
			if (container) container._query = null;
		};
	
		this.undo = function(){
			if (!this._sourceId && !this._targetId)
				return;

			this._editor.addLink(this._sourceId, this._targetId, this._linktype);
			var container = this._editor.findContainer(this._targetId);
			if (container) container._query = null;
		};
	},
	
	SetLinkPropertyCommand: function(container, link, properties, onEnd){
		this._editor = container.hlp_getEditor();
		this._containerId = container.getId();
		this._link = link;
		this._properties = properties;
		this._onEnd = onEnd;

		this.execute = function(){
			var container = this._editor.findContainer(this._containerId);
			if (!container) return;

			var oldProperties = {};
			dojo.forEach(container.getUpstream(), function(link) {
				if (link.id == this._link.id && link.linktype == this._link.linktype) {
					for (key in this._properties)
						oldProperties[key] = link[key];
					dojo.mixin(link, this._properties);
				}
			}, this);
			container._query = null;
			this._properties = oldProperties;

			if(dojo.isFunction(this._onEnd)){
				this._onEnd(this._oldProperties, properties);
			}
		};
	
		this.undo = function(){
			this.execute();
		};
	},
	
	ChangeLayoutCommand: function(container, layoutParam, onEnd){
		this._editor = container.hlp_getEditor();
		this._containerId = container.getId();
		this._layoutParam = layoutParam;
		this._onEnd = onEnd;
	
		this.execute = function(){
			var container = this._editor.findContainer(this._containerId);
			if (!container) return;
			
			var oldParam = {};
			for (var key in this._layoutParam) {
				var value = container.hlp_getPropertyValue(key, true);
				oldParam[key] = value;
			}

			if (this._layoutParam.layoutManager && !this._layoutParam._layoutManager) {
				var layoutType = this._layoutParam.layoutManager;
				if(layoutType.indexOf('.') < 0){ //$NON-NLS-1$
					layoutType = "com.ibm.relm.explorer.web.ui.internal.view.layout." + layoutType; //$NON-NLS-1$
				}
				
				try {
		    		dojo.require(layoutType);
		    		var ctor = dojo.getObject(layoutType);
		    		if(ctor){
		    			this._layoutParam._layoutManager =  new ctor();
		    		}					
				}catch(e){
					console.error(e);
					return;
				}
			}
			
			var oldParam = {};
			for (var key in this._layoutParam) {
				if (key == "_layoutManager") { //$NON-NLS-1$
					container.setLayoutManager(this._layoutParam._layoutManager);
				} else if (key == "layoutManager") { //$NON-NLS-1$
					oldParam[key] = container.getLayout();
					container[key] = this._layoutParam[key];
				} else {
					var newValue = this._layoutParam[key];
					var oldValue = this._editor.hlp_getPropertyValue(key);
					if (oldValue !== undefined) oldParam[key] = oldValue;
					var func = "set" + key.charAt(0).toUpperCase() + key.substring(1); //$NON-NLS-1$
					if (container[func]) container[func](newValue);
					else container[key] = newValue;
				}
			}
			
			this._layoutParam = oldParam;
			container._query = null;

			if(dojo.isFunction(this._onEnd)){
				this._onEnd();
			}
		};
		
		this.undo = function(){
			this.execute();
		};
	},
	
	collectContainers: function(widgets, containers) {
		if (!containers) containers = [];
		dojo.forEach(widgets, function(w) {
			if (w instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer) {
				containers.push(w);
				containers = this.collectContainers(w.getChildren(), containers);
			}
		});
		return containers;
	},
	
	PasteLayoutObjectCommand: function(layoutText, config, container, refWidget){
		this._editor = container.hlp_getEditor();
		this.container = container;
		this.srcConfig = config;
		if ((this.container != this._editor) && !(this.container instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer)) {
			refWidget = this.container;
			this.container = this.container.getParent();
		}
		this._layoutText = layoutText;

		this.execute = function() {
			if (!this._editor || !dojo.isFunction(this._editor.loadLayout))
				return;
			
			var rootContainer = this._editor.loadLayout(this._layoutText);
			if (!rootContainer)
				return;
			rootContainer.setup();

			var dstConfig = this._editor.getConfig();
			var isLocal = dstConfig.viewUri == this.srcConfig.viewUri; // copied from another view or not

			this._insertChildren = rootContainer.getChildren();
			this._editor.removeContainer(rootContainer.id);

			var baseQueryIds = [];
			var connectionLinkIds = [];

			var insertContainers = collectContainers(this._insertChildren);
			dojo.forEach(insertContainers, function(container) {
				this._editor.removeContainer(container.id);
				var id = container._containerId;
				if (id) {
					// reassign container ids if already exists
					if (this._editor.findContainer(id)) {
						var prefix = id;
						var index = dojo.lastIndexOf(prefix, '_'); //$NON-NLS-1$
						if (index >= 0) {
							var suffix = prefix.substr(index + 1);
							if (suffix == "" || /^\d+$/.test(suffix)) //$NON-NLS-1$
								prefix = prefix.substring(0, index);
						}
						var newId = this._editor.createUniqueContainerId(prefix);
						container._containerId = newId;
					}
				}

				if (!isLocal) {
					// collect referred baseQueryIds and childLinkIds
					if (container.baseQueryId && dojo.indexOf(baseQueryIds, container.baseQueryId) < 0)
						baseQueryIds.push(container.baseQueryId);
					if (container.childLinkId) {
						childLinkIds = container.childLinkId.split(','); //$NON-NLS-1$
						dojo.forEach(childLinkIds, function(id) {
							if (id && dojo.indexOf(connectionLinkIds, id) < 0) {
								connectionLinkIds.push(id);
								dojo.forEach(this.srcConfig.connectionLinks, function(link) {
									if (id == link.id && link.baseQueryId && dojo.indexOf(baseQueryIds, link.baseQueryId) < 0) {
										baseQueryIds.push(link.baseQueryId);
									}
								});
							}
						}, this);
					}
				}
			}, this);

			// insert referred queryTemplates and connectionLinks
			var configChanged = false;
			if (!isLocal) {
				// baseQueryIds
				var queryIdMap = {};
				this._createdQueries = null;
				dojo.forEach(baseQueryIds, function(id) {
					var srcQuery = null;
					dojo.forEach(this.srcConfig.queryTemplates, function(query) {
						if (query.id == id)
							srcQuery = query;
					});
					if (srcQuery) {
						// reassign ids
						var newQuery = dojo.clone(srcQuery);
						var newId = createUniqueId(newQuery.id, dojo.map(dstConfig.queryTemplates, function(q) {return q.id;}));
						if (newQuery.id != newId) {
							dojo.forEach(insertContainers, function(container) {
								if (container.baseQueryId == newQuery.id)
									container.baseQueryId = newId;
								});
							queryIdMap[newQuery.id] = newId;
							newQuery.id = newId;
						}

						// add new query
						if (!this._createdQueries)
							this._createdQueries = [newQuery.id];
						else
							this._createdQueries.push(newQuery.id);
						dstConfig.queryTemplates.push(newQuery);
						configChanged = true;
					}
				}, this);

				// connectionIds
				this._createdLinks = null;
				dojo.forEach(connectionLinkIds, function(id) {
					var srcLink = null;
					dojo.forEach(this.srcConfig.connectionLinks, function(link) {
						if (link.id == id)
							srcLink = link;
					});
					if (srcLink) {
						// reassign ids
						var newLink = dojo.clone(srcLink);
						var newId = createUniqueId(newLink.id, dojo.map(dstConfig.connectionLinks, function(l) {return l.id;}));
						if (newLink.id != newId) {
							dojo.forEach(insertContainers, function(container) {
								var childIds = container.childLinkId ? container.childLinkId.split(',') : []; //$NON-NLS-1$
								childIds = dojo.map(childIds, function(id) {return id == newLink.id ? newId : newLink.id;});
								container.childLinkId = childIds.length > 0 ? childIds.join(',') : null; //$NON-NLS-1$
							});
							newLink.id = newId;
						}
						if (queryIdMap[newLink.baseQueryId])
							newLink.baseQueryId = queryIdMap[newLink.baseQueryId];
	
						// add new link
						if (!this._createdLinks)
							this._createdLinks = [newLink.id];
						else
							this._createdLinks.push(newLink.id);
						dstConfig.connectionLinks.push(newLink);
						configChanged = true;
					}
				}, this);
			}
			
			// insert children
			dojo.forEach(this._insertChildren, function(child) {
				this.container.hlp_addChild(child, refWidget);
			}, this);
			
			if (configChanged) {
				this._editor.onConfigLoaded(dstConfig);
			}
		};
		
		this.undo = function() {
			dojo.forEach(this._insertChildren, function(child) {
				this.container.hlp_removeChild(child);
			}, this);
			
			var self = this;
			var configChanged = false;
			var dstConfig = this._editor.getConfig();
			if (this._createdQueries) {
				dstConfig.queryTemplates = dojo.filter(dstConfig.queryTemplates, function(query) {
					return dojo.indexOf(self._createdQueries, query.id) < 0;
				});
				configChanged = true;
			}
			if (this._createdLinks) {
				dstConfig.connectionLinks = dojo.filter(dstConfig.connectionLinks, function(link) {
					return dojo.indexOf(self._createdLinks, link.id) < 0;
				});
				configChanged = true;
			}
			
			if (configChanged) {
				this._editor.onConfigLoaded(dstConfig);
			}
		};
	}
	
};
