/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.EditSideBar"); //$NON-NLS-1$

dojo.require("rmps.ui.SidebarContainer");//$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.PropertiesSideBarContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSideBarContent"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "SideBarContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
/**
 * Base class for a container of rightside bar
 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.EditSideBar", [rmps.ui.SidebarContainer], { //$NON-NLS-1$

	_sideBarControl: null, 
	
	postMixInProperties: function(){
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "SideBarContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.inherited(arguments);
	},

	setDiagramView: function(diagramView) {
		this._diagramView = diagramView;
	},
	
	showSidebar: function(show){
		if(show){
			var sideBar = this._createSidebarContent();
			this.addSidebar(sideBar);
			if(dojo.isIE < 9){
				dojo.addClass(sideBar.domNode.parentNode, "no-transition"); //$NON-NLS-1$
			}
			this._sideBarControl = sideBar;
			
			if (sideBar.setupContextSensitiveHelp) {
				sideBar.setupContextSensitiveHelp();
			} else {
				dojo.forEach(sideBar.getChildren(), function(content) {
					if (content.setupContextSensitiveHelp) content.setupContextSensitiveHelp();
				});
			}

			if(window.onresize){
				setTimeout(function(){
					window.onresize({});
				}, 10);
			}
		}else if(this._sideBarControl){
			var sideBar = this._sideBarControl;
			this._cleanSidebar();
			this.removeSidebar(sideBar);
		}
		if(window.onresize){
			this.display(show);
		}
	},

	_createSidebarContent: function(){
		// Show 'Palette' tab if there is no content, otherwise show 'Properties' tab
		var isEmpty = this._getOrMonitorIfViewIsEmpty();
		
		var sideBar = new dijit.layout.TabContainer();
		sideBar.addChild(new com.ibm.relm.explorer.web.ui.internal.view.edit.PropertiesSideBarContent({
			title: this._messages.propertiesTabTitle, 
			diagramView: this._diagramView,
			selected: isEmpty !== true
		}));
		sideBar.addChild(new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSideBarContent({
			title: this._messages.paletteTitle,
			diagramView: this._diagramView,
			selected: isEmpty === true
		}));

		// (#89590) Workaround for that rmps.ui.SidebarContainerSash doesn't update _sidebarWidth in _dragResize
			
		// (#92473) Layout of sidebar (rmps.ui.SidebarContainer) happens on view creation. This triggers window.onresize(), 
		// which is monitored by opened views. When an invisible view (hidden in backend) gets this event, 
		// this.domNode.parentNode.offsetWidth is zero. Then, width of contents of sidebar is set to zero in the result of the workaround for #89590.
		// Since the workaround is for the problem that happens during or after dragging sash of sidebar, 
		// there is no need for backend views to react to this.  
		dojo.mixin(sideBar, {
			resize: function(size){
				if (!this.domNode || this.domNode.parentNode.offsetWidth == 0) {
					return;
				}
				if(size && size.w && this.domNode.parentNode.offsetWidth > 0){
					size.w = this.domNode.parentNode.offsetWidth;
				}
				this.inherited("resize", arguments); //$NON-NLS-1$
			}
		});
		
		return sideBar;
	},
	
	_cleanSidebar: function(){
		if (this._sideBarControl) {
			this._sideBarControl.destroyRecursive();
			this._sideBarControl = null;
		}
	},

	_getOrMonitorIfViewIsEmpty: function() {
		if (!this._diagramView) return true;
		
		var editor = this._diagramView._getDiagramEditor();
		if (!editor.getConfig || !editor.getConfig()) {
			// monitor config being loaded when it has not been loaded 
			this._onConfigLoadHandle = this.connect(editor, "onConfigLoaded", function() { //$NON-NLS-1$
				this.disconnect(this._onConfigLoadHandle);
				if (this._sideBarControl && !this._checkEmpty(editor))
					this._sideBarControl.selectChild(this._sideBarControl.getChildren()[0]);
			});
			return true;
		}
		return this._checkEmpty(editor);
	},

	_checkEmpty: function(editor) {
		var children = editor.getChildren();
		for (var i = 0; i < children.length; i++) {
			var widget = children[0];
			
			// assume view is empty if there is no widget in the root container
			if (widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer) {
				return widget.getChildren().length == 0;
			}
		}
		return true;
	},
	
	destroy: function() {
		this._diagramView = null;
		this._sideBarControl = null;
		this.inherited(arguments);
	}
});

})();
