/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.FillPropertyDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ColorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.AddOffsetColorContent"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "FillPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container

	var colorUtils = com.ibm.relm.explorer.web.ui.internal.view.common.ColorUtils;

	var TYPE_SOLID = "solid"; //$NON-NLS-1$
	var TYPE_LINEAR_GRADIENT = "linear"; //$NON-NLS-1$
	var TYPE_RADIAL_GRADIENT = "radial"; //$NON-NLS-1$
	var TYPE_PATTERN = "pattern"; //$NON-NLS-1$
	
	var DEFAULT_COLOR = "white"; //$NON-NLS-1$
	var DEFAULT_GRADIENT = [{offset: 0, color: "black"}, {offset: 1, color: "white"}]; //$NON-NLS-1$//$NON-NLS-2$
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
	
	var _getIntValue = function(val){
		var ret = parseInt(val);
		if(isNaN(ret)){
			ret = undefined;
		}
		return ret;
	};
	/**
	 * Widget to Edit Fill properties
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.FillPropertyDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated , com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], 
		{ 
		widgetsInTemplate: true,
		
		type : "", //$NON-NLS-1$
		fillProperty : null,
		
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/FillPropertyDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor: function(){
			this._fillTypeId = dijit.getUniqueId("fillType");//$NON-NLS-1$
			this._solidColorValueId = dijit.getUniqueId("solidColorValue");//$NON-NLS-1$
			this._textBox_x1Id = dijit.getUniqueId("textBox_x1");//$NON-NLS-1$
			this._textBox_y1Id = dijit.getUniqueId("textBox_y1");//$NON-NLS-1$
			this._textBox_x2Id = dijit.getUniqueId("textBox_x2");//$NON-NLS-1$
			this._textBox_y2Id = dijit.getUniqueId("textBox_y2");//$NON-NLS-1$
			this._textBox_cxId = dijit.getUniqueId("textBox_cx");//$NON-NLS-1$
			this._textBox_cyId = dijit.getUniqueId("textBox_cy");//$NON-NLS-1$
			this._textBox_rId = dijit.getUniqueId("textBox_r");//$NON-NLS-1$
			this._textBox_xId = dijit.getUniqueId("textBox_x");//$NON-NLS-1$
			this._textBox_yId = dijit.getUniqueId("textBox_y");//$NON-NLS-1$
			this._textBox_widthId = dijit.getUniqueId("textBox_width");//$NON-NLS-1$
			this._textBox_heightId = dijit.getUniqueId("textBox_height");//$NON-NLS-1$
			this._textBox_srcId = dijit.getUniqueId("textBox_src");//$NON-NLS-1$
		},

		postMixInProperties : function(args) {
			this.inherited(arguments);
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "FillPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate : function() {
			this.inherited(arguments);

			this.linearColorsToolbar.appendChild(this._createAddButton(this.messages.AddColorsProperty, this._handleAddLinearColorProperty));
			this.radialColorsToolbar.appendChild(this._createAddButton(this.messages.AddColorsProperty, this._handleAddRadialColorProperty));

			this.linearColorsTable.setupColumns(this._createColorTableColumns());
			this.radialColorsTable.setupColumns(this._createColorTableColumns());
			this._setupColorNameDropdown();			
			var value = this.fillProperty;
			try {
				value = dojo.fromJson(value);
				if(dojo.isString(value)) {
					value = {type: TYPE_SOLID, color: value};
				}
			} catch (e){
				value = {type: TYPE_SOLID, color: value || DEFAULT_COLOR};
			}
			this.fillProperty = value;
			this._fillType.set("value", value.type); //$NON-NLS-1$
			this.type = value.type;
			this._switchType(this.type);

			this._setFillPropertyValues(value);
		},
		
		_setupColorNameDropdown: function() {

			colorUtils.loadColorSelectDropdown(this.solidColorValue);

		},
		
		/*
		 * Create columns for color tables (used by linear and radical gradient) which contains offset, color, and actions column
		 */
		_createColorTableColumns: function(){
			return this._createTableColumns(
					[
					 {
	       		    	id: "offset",   //$NON-NLS-1$
	       		    	name: this.messages.colorOffsetLabel,
	       		    	editable: true, 
	       		    	callback: this.updateColorsData,
	       		    	preventEmpty: true,
	       		    	width: "120px"  //$NON-NLS-1$
					 },{
	    		    	id: "color",   //$NON-NLS-1$
	       		    	name: this.messages.colorColorLabel,
	       		    	editable: true, 
	       		    	callback: this.updateColorsData,
	       		    	preventEmpty: true,
	       		    	width: "100px"//$NON-NLS-1$
					 },{
	     		    	id: "actions",  //$NON-NLS-1$
	    		    	name: this.messages.colorTableAction,
	    		    	isAction: true					 
					 }
			]);			
		},
		
		/*
		 * Called when offset or color value is changed in color table
		 */
		updateColorsData : function(rowData, propName, newValue, dirtyKey) {
			var self = this;
			var index = rowData.index;
			var orgProp = self.fillProperty.colors[index];
			var oldValue = orgProp[propName];
			if(oldValue != newValue) {
				if(propName == "offset" && dojo.isString(newValue)){//$NON-NLS-1$
					newValue = parseFloat(newValue);
					if(isNaN(newValue)){
						newValue = 0.5;
					}
				}
				orgProp[propName] = newValue;
				self._setFillPropertyValues(self.fillProperty);
			}

		},
		
		/*
		 * Create an edit button to open color picker
		 */
		_getEditColorButton: function(offsetColor) {
			var self = this;
			var button = dojo.create("a", { //$NON-NLS-1$
				href: "#", //$NON-NLS-1$
				title: this.messages.editColorButton,
				className: "dijitInline actionIcon" //$NON-NLS-1$
			});
			this.connect(button, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				colorUtils.openColorPickerDialog(offsetColor.color, function(color){
					offsetColor.color = color;
					self._setFillPropertyValues(self.fillProperty);
				}, self);
			});
			dojo.create("span", { //$NON-NLS-1$
				className: "dijitInline edit-command" //$NON-NLS-1$
			}, button);
			
			return button;
		},
		
		/*
		 * Map fill properties to UI 
		 */
		_setFillPropertyValues : function(value) {
			
			var self = this;
			
			switch (self.type) {
			case TYPE_SOLID:
				var color = value.color;
				if(!color && dojo.isString(value)){
					color = value;
				}
				self.solidColorValue.set("value", color); //$NON-NLS-1$
				break;
			case TYPE_LINEAR_GRADIENT:
				self.textBox_x1.set("value", value.x1 || 0); //$NON-NLS-1$
				self.textBox_y1.set("value", value.y1 || 0); //$NON-NLS-1$
				self.textBox_x2.set("value", value.x2 === undefined || value.x2 === null ? 100 : value.x2); //$NON-NLS-1$
				self.textBox_y2.set("value", value.y2 === undefined || value.y2 === null ? 100 : value.y2); //$NON-NLS-1$
				if(!value.colors) value.colors = dojo.clone(DEFAULT_GRADIENT);
				self._setupColorsTable(self.linearColorsTable, value.colors);
				break;
			case TYPE_RADIAL_GRADIENT:
				self.textBox_cx.set("value", value.cx || 0); //$NON-NLS-1$
				self.textBox_cy.set("value", value.cy || 0); //$NON-NLS-1$
				self.textBox_r.set("value", value.r); //$NON-NLS-1$
				if(!value.colors) value.colors = dojo.clone(DEFAULT_GRADIENT);
				self._setupColorsTable(self.radialColorsTable, value.colors);
				break;
			case TYPE_PATTERN:
				self.textBox_x.set("value", value.x || 0); //$NON-NLS-1$
				self.textBox_y.set("value", value.y || 0); //$NON-NLS-1$
				self.textBox_width.set("value", value.width || 0); //$NON-NLS-1$
				self.textBox_height.set("value", value.height || 0); //$NON-NLS-1$
				self.textBox_src.set("value", value.src); //$NON-NLS-1$
				break;
			}

		},
		
		/*
		 * Map fill properties to UI 
		 */
		_setupColorsTable : function(table, colors) {
			var propTable = table;
			propTable.clearItems();

			if(!colors) {
				return;
			}
			
			colors.sort(function(a, b){
				return a.offset - b.offset; 
			});

			var self = this;
			dojo.forEach(colors, function(offsetColor, index) {
				propTable.addItem({
					offset: offsetColor.offset,
					color: offsetColor.color,
					index: index,
					editButton: self._getEditColorButton(offsetColor),
					actions: [{
						iconClass: CLASS_DELETE_COMMAND,
						title: this.messages.deleteOffset,
						onClick: function(e){
							if(confirm(dojo.string.substitute(self.messages.confirmRemoveOffsetColor, [ this.data.offset, this.data.color ]))){
								// remove parameter
								colors.splice(this.data.index, 1);
								self._setupColorsTable(table, colors);
							}
						}
					}]
				});
			});

		},
		
		/*
		 * Event handler for fill type dropdown box
		 */
		_onFillTypeChanged : function(type) {
			this.type = type;
			this._switchType(this.type);
			this._setFillPropertyValues(this.fillProperty);
		},
		
		_switchType : function(type) {
			this.solidColorPane.style.display = (type == TYPE_SOLID) ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			this.linearPane.style.display = (type == TYPE_LINEAR_GRADIENT) ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			this.radialPane.style.display = (type == TYPE_RADIAL_GRADIENT) ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
			this.patternPane.style.display = (type == TYPE_PATTERN) ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/*
		 * Event handler for Add button of color table for linear gradient
		 */
		_handleAddLinearColorProperty : function() {
			this._handleAddColorProperty(this.linearColorsTable);
		},
		
		_handleAddColorProperty: function(table){
			var self = this;
			if(!self.fillProperty.colors) self.fillProperty.colors = [];
			com.ibm.relm.explorer.web.ui.internal.view.edit.AddOffsetColorContent.openDialog(self.fillProperty.colors, function(){
				self._setupColorsTable(table, self.fillProperty.colors);
			});
		},
		
		/*
		 * Event handler for Add button of color table for radical gradient
		 */
		_handleAddRadialColorProperty : function() {
			this._handleAddColorProperty(this.radialColorsTable);
		},
		
		/*
		 * Event handler for edit color button
		 */
		_editColor: function(evt) {
			var color = this.solidColorValue.get("value"); //$NON-NLS-1$
			colorUtils.openColorPickerDialog(color, dojo.hitch(this, function(color){
				this.solidColorValue.set("value", color);//$NON-NLS-1$
			}), this);
		},

		/*
		 * respond values
		 */
		getValues: function() {

			var retValue = null;
			
			switch(this.type) {
			case TYPE_SOLID:
				retValue = this.solidColorValue.get("value"); //$NON-NLS-1$
				return retValue;

			case TYPE_LINEAR_GRADIENT:
				retValue = {type: this.type,
							x1: _getIntValue(this.textBox_x1.get("value")), //$NON-NLS-1$
							y1: _getIntValue(this.textBox_y1.get("value")), //$NON-NLS-1$
							x2: _getIntValue(this.textBox_x2.get("value")), //$NON-NLS-1$
							y2: _getIntValue(this.textBox_y2.get("value")), //$NON-NLS-1$
							colors: this.fillProperty.colors 
							};
				break;
			case TYPE_RADIAL_GRADIENT:
				retValue = {type: this.type,
					cx: _getIntValue(this.textBox_cx.get("value")), //$NON-NLS-1$
					cy: _getIntValue(this.textBox_cy.get("value")), //$NON-NLS-1$
					r: _getIntValue(this.textBox_r.get("value")), //$NON-NLS-1$
					colors: this.fillProperty.colors
					};
				break;
			case TYPE_PATTERN:
				retValue = {type: this.type,
					x: _getIntValue(this.textBox_x.get("value")), //$NON-NLS-1$
					y: _getIntValue(this.textBox_y.get("value")), //$NON-NLS-1$
					width: _getIntValue(this.textBox_width.get("value")), //$NON-NLS-1$
					height: _getIntValue(this.textBox_height.get("value")), //$NON-NLS-1$
					src: this.textBox_src.get("value") //$NON-NLS-1$
					};
				break;
			
			}
			return dojo.toJson(retValue);
		}
	});
})();
