/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementPalette");//$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementEditUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementCreationTool"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementParameterContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container
	var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
	var leEditUtils = com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementEditUtils;
	var containerLinkActions = com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions;

	var ns = com.ibm.relm.le.ui.namespace;
	var TYPE_FOLDER = ns.ae.Folder;
	var TYPE_ELEMENT = ns.ae.Ae;

	var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$  //$NON-NLS-2$

	dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementPalette",//$NON-NLS-1$
		[com.ibm.relm.explorer.web.ui.internal.view.edit.Palette],
	{
		layout: "vertical",//$NON-NLS-1$
		defaultDropHandler: "com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementCreationTool",//$NON-NLS-1$
		defaultClickHandler: "com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementCreationTool",//$NON-NLS-1$
		defaultExpandDepth: 1,
		
		postCreate: function(){
			this.inherited(arguments);
		},

		startup: function(){
			var self = this;
			leServiceUtils.getPermissions().then(function(results){
				dojo.forEach(results, function(p){
					if(p == "modify") self.modifyPermission = true; //$NON-NLS-1$
				});
			});
			this._createLifecycleElementPaletteInfo().then(dojo.hitch(this, function(paletteInfo) {
				this.paletteInfo = paletteInfo;
				if(this._started){return;}
				this._started = true;
				if(this.autoRender){
					this.render();
				}
			}));
		},
		
		/**
		 * Create palette item information for building a palette
		 */
		_createLifecycleElementPaletteInfo: function() {
			var paletteInfo = {
					label: 'name',//$NON-NLS-1$
					rel: 'com.ibm.relm.explorer.web.ui.internal.view.edit',//$NON-NLS-1$
					items: []
				};

			var self = this;
			var deferred = new dojo.Deferred();
			leServiceUtils.getElements().then(function(items){
				if (self._destroyed) return;
				
				dojo.forEach(items, function(item) {
					self._insertPaletteItem(paletteInfo, item, -1, 0);
				});
				deferred.callback(paletteInfo);
			},
			function(error) {
				deferred.callback(paletteInfo);
			});
			
			return deferred;
		},
		
		_insertPaletteItem: function(paletteInfo, element, index, indent) {
			var item = this._createItem(null, element, indent);
			if (!item) return;
			if (index < 0) {
				paletteInfo.items.push(item);
			} else {
				paletteInfo.items.splice(index, 0, item);
			}
		},
		
		_delayedRender: function(id, folderItem, depth) {
			var deferred = new dojo.Deferred();
			leServiceUtils.getElements(folderItem.uri).then(dojo.hitch(this, function(items) {
				if (this._destroyed) return;
				
				var paletteItems = [];
				dojo.forEach(items, function(item) {
					paletteItems.push(this._createItem(folderItem.uri, item, depth + 1));
				}, this);
				deferred.callback(paletteItems);
			}));
			return deferred;
		},

		_createItem: function(parent, element, indent) {
			var item = null;
			if (element.type == TYPE_FOLDER) {
				item = {
						name: '',//'container', //$NON-NLS-1$
						icon: '../../../images/folder.gif', //$NON-NLS-1$
						uri: element.uri,
						description: element.title,	
						isFolder: true,
						folderId: element.id
				};
				
				if (element.numChildren > 0) {
					item.delayedRender = dojo.hitch(this, this._delayedRender);
					
					if (this.defaultExpandDepth > indent)
						item.initialState = "open"; //$NON-NLS-1$
				}
			} else if (element.type == TYPE_ELEMENT){
				var menuInfos = [];
				if (this.modifyPermission) {
					menuInfos.push({
						label: messages.editLabel,
						iconClass: 'edit-icon', //$NON-NLS-1$
						onClick: function(nodeWidget, e) {
							var message = messages.confirmExecuteMessage + "\n" + messages.reexecuteMessageFromPalette + "\n\n" + messages.askSaveMessage;  //$NON-NLS-2$ //$NON-NLS-1$
							if(!e.ctrlKey && !confirm(message)) return; 	
							leEditUtils.editArtifactElement(element.id, parent, e.ctrlKey ? "tab" : null); //$NON-NLS-1$
						}
					});
				}
				
				var editor = this.diagramView._getDiagramEditor();
				if (editor && editor.setElementItem)
					editor.setElementItem(element);
				item = {
						name: 'container', //$NON-NLS-1$
						icon: '../../../images/artifactelement.gif', //$NON-NLS-1$
						uri: element.uri,
						parentUri: parent,
						description: element.title,	
						menuInfos: menuInfos,
						props: containerLinkActions.getContainerParam(this.diagramView._getDiagramEditor(), {
								elementId: element.id,
								name: element.title
							}),
							onItemCreate: dojo.hitch(this, function(e) {
								var spec = e.componentSpec;
								if (spec) {
									return this._setInitParams(e, element.uri, spec);
								}
							})
				};
			}

			if (item && indent > 0) item.depth = indent;
			return item;
		},
		
		_setInitParams: function(e, elementUri, componentSpec) {
			var self = this;
			var editor = this.diagramView.getDiagramEditor();
			var deferred = new dojo.Deferred();
			leServiceUtils.getElement(elementUri, function(element) {
				if (self._destroyed) return;
				
				editor && editor.addToCache(element);
				componentSpec.props.elementId = element.id;
				delete componentSpec.props.containerParams;
				var openDialog = null;
				if (element.parameters)
					openDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementParameterContent.openDialog;
		
				if (openDialog) {
					openDialog(element, null, function(result) {
						if (result) {
							componentSpec.props.containerParams = result;
						}
						deferred.callback(e);
					}, e.clientX, e.clientY);
				} else {
					deferred.callback(e);
				}
			}, function(err, ioargs) {
				var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(err, ioargs);
				alert(message.summary);
			});
			
			return deferred;
		}
	});
})();