/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkAttributeContent"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$

(function() { // module container
	var containerLinkActions = com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions;

	/**
	 * Widget to show property pane
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkAttributeContent", [ com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent ], { //$NON-NLS-1$
		
		getAttributeOrder: function(){
			return [
			        "showLines",		//$NON-NLS-1$ 
			        "linkedNodeOnly",		//$NON-NLS-1$ 
			        "connectionId"	//$NON-NLS-1$ 
			];       
		},
		
		getOptions : function(attrName) {
			if (attrName == "showLines") { //$NON-NLS-1$
				return [{label: this.messages.True, value: true}, {label: this.messages.False, value: false}];
			} else if (attrName == "linkedNodeOnly") { //$NON-NLS-1$
				return [{label: this.messages.True, value: true}, {label: this.messages.False, value: false}];
			} else if (attrName == "connectionId") { //$NON-NLS-1$
				var viewer = this.diagramView._getDiagramEditor();
				var connectionLinks = viewer.getConfig().connectionLinks;
				var options = [];
				dojo.forEach(connectionLinks, function(link) {
					if (link.use == "containerlink") //$NON-NLS-1$
						options.push({label: link.title, value: link.id});
				});

				if (options.length == 0) {
					// If connection specified by connectionId does not exist, add the connectionId so that user can unselect it 
					var link = this.widget.targetLink;
					if (link.connectionId)
						options.push({label: link.connectionId, value: link.connectionId});
				}

				options.push({label: "-", value: ""}); //$NON-NLS-2$ //$NON-NLS-1$
				return options;
			}
		},
		
		_getAttributes: function(widget) {
			var attributes = [];
			attributes.push({name: "showLines"}); //$NON-NLS-1$
			attributes.push({name: "linkedNodeOnly"}); //$NON-NLS-1$
			attributes.push({name: "connectionId"}); //$NON-NLS-1$
			return attributes;
		},
		
		_getEditButtonInfo: function(name, value) {
			if(name != "connectionId") {//$NON-NLS-1$
				return this.inherited(arguments);
			}
			
			var viewer = this.diagramView._getDiagramEditor();
			var container = this.widget.targetAnchorObj.owner;
			var line = this.widget;
			var self = this;
			
			return {label: self.messages.editConnectionLabel, editFunc: dojo.hitch(this, function(value) {
				containerLinkActions.showAndEditConnection(viewer, container, line, value);
			})};
		},
		
		_updateWidget: function(prop, noReload){
			var viewer = this.diagramView._getDiagramEditor();
			var container = this.widget.targetAnchorObj.owner;
			var line = this.widget;
			var link = line.targetLink;
			
			var param = {};
			for (key in prop) {
				if (key == "showLines") //$NON-NLS-1$
					param.hidelines = prop[key] != "true"; //$NON-NLS-1$
				else if (key == "linkedNodeOnly") //$NON-NLS-1$
					param.unlimited = prop[key] != "true"; //$NON-NLS-1$
				else
					param[key] = prop[key];
			}
			containerLinkActions.changeLinkProperties(viewer, container, line, link, param);
		}
	});
})();