/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.NodeContainerLinkTracker"); //$NON-NLS-1$

(function(){

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
var containerLinkActions = com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions;
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.NodeContainerLinkTracker", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.canvas.tool.CreateNodeConnectionTracker,
{
	constructor : function(args) {
		this.pageMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramPageMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
		this.linktypeMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	createConnection: function(source, target) {
		var sourceId = source.getId();
		var targetId = target.getId();
		if (!sourceId || sourceId.length == 0 || !targetId || targetId.length == 0) {
			// both source and target should have id to connect
			return;
		}
		
		var viewer = this.getEditor();
		this._pickupLinkType(viewer, source, target, function(connections, linkparam) {
			containerLinkActions.createLinkPath(viewer, source, target, connections, linkparam);
		});
	},

	_hasConnectableType: function(target) {
		for (var widget = target; widget; widget = widget.getParent()) {
			if (widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer)
				return true;
		}
		return false;
	},
	
	_getConnectableWidget: function(source, target) {
		if (source == target)
			return null;
		
		for (var widget = target; widget; widget = widget.getParent()) {
			if (this._isConnectable(source, widget))
				return widget;
		}
		return null;
	},

	_isConnectable: function(source, target) {
		if (source == target || !target.connectable || 
				!(target instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer))
			return false;

		return true;
	},
	
	/**
	 * Create a link between existing containers
	 */
	createLink: function(viewer, source, target) {
		var sourceId = source.getId();
		var targetId = target.getId();
		if (!sourceId || sourceId.length == 0 || !targetId || targetId.length == 0) {
			// both source and target should have id to connect
			return;
		}
			
		var self = this;
		this._pickupLinkType(viewer, source, target, function(connections, linkparam) {
			self.createLinkPath(viewer, source, target, connections, linkparam);
		});
	},
	
	/**
	 * Select a link type between containers
	 */
	_pickupLinkType: function(viewer, source, target, callback) {
		var maxSearchDepth = 4;
		var self = this;
		var updator = null;
		var assessed = false;
		var previousUpdate = -1;
		JazzUIWorkbench.showMessage(this.pageMessages.loading);
		var finder = new com.ibm.relm.explorer.web.ui.internal.view.edit.LinkPathFinder({viewer: viewer, source: source, target: target});
		
		finder.getPossibleLinkPath(function(linkpath, progress) {
			if (progress >= 0 && (!linkpath || linkpath.length < 2)) {
				if (0 <= maxSearchDepth && maxSearchDepth <= progress) {
					// give up searching if no more than 2 paths have been found 
					finder.cancel();
					JazzUIWorkbench.hideMessageArea();
				} else {
					return;
				}
			}

			if (progress < 0) {
				JazzUIWorkbench.hideMessageArea();
			}

			if (updator) {
				if (progress - previousUpdate > 3 && dojo.isFunction(updator.updateLinkPathList))
					updator.updateLinkPathList(linkpath);
				return;
			}

			if (assessed)
				return;
			assessed = true;
			
			previousUpdate = progress;
			if (linkpath.length > 1) {
				// select from list
				var sourceTitle = source.getTitle();
				var targetTitle = target.getTitle();
				var openDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent.openDialog;
				updator = openDialog({type: "artifactElementType", sourceTitle: sourceTitle, targetTitle: targetTitle, linkpath: linkpath, index: 0, hidelines: false, unlimited: false}, function(result) { //$NON-NLS-1$
					if (!self._checkError(viewer, source, target, result.linkpath))
						callback(result.linkpath, {hidelines: result.hidelines, unlimited: result.unlimited});
				}, function() {
					finder.cancel();
					JazzUIWorkbench.hideMessageArea();
				});
			} else if (linkpath.length == 1) {
				if (!self._checkError(viewer, source, target, linkpath[0]))
					callback(linkpath[0]);
			} else {
				var sourceTitle = source.getTitle();
				var targetTitle = target.getTitle();
				var messageText = dojo.string.substitute(self.linktypeMessages.noLinkTypeMessage, [sourceTitle, targetTitle]);
				alert(messageText);
			}
		});
	},
	
	_checkError: function(viewer, source, target, linkpath) {
		linkpath[0].containerId = source.getId();
		linkpath[linkpath.length - 1].containerId = target.getId();

		if (this._checkAlreadyExists(viewer, source, target, linkpath)) {
			alert(this.linktypeMessages.alreadyExistsMessage);
			return true;
		} else if (this._checkCircularLnk(viewer, source, target, linkpath)) {
			var sourceTitle = source.getTitle();
			var targetTitle = target.getTitle();
			alert(dojo.string.substitute(this.linktypeMessages.circularLinkExistsMessage, [sourceTitle, targetTitle]));
			return true;
		}
		return false;
	},
	
	_checkAlreadyExists: function(viewer, source, target, linkpath, index) {
		var self = this;
		if (!index) {
			if (source.getId() != linkpath[0].containerId)
				return false;
			index = 1;
		}
		
		var targetId = linkpath[index].id;
		return dojo.some(source.getDownstream(), function(down) {
			var container = viewer.findContainer(down.id);
			if (container && container.elementId == targetId) {
				if (linkpath.length - 1 == index) {
					return container.getId() == linkpath[index].containerId;
				}
				return self._checkAlreadyExists(viewer, container, target, linkpath, index + 1);
			}
		});
		return false;
	},
	
	_checkCircularLnk: function(viewer, source, target, linkpath) {
		var self = this;
		var linkContainerId = linkpath[linkpath.length - 1].containerId;
		return dojo.some(source.getUpstream(), function(up) {
			if (up.id == linkContainerId) return true;
			var container = viewer.findContainer(up.id);
			if (container && self._checkCircularLnk(viewer, container, target, linkpath)) return true;
		});
	}
});
})();