/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2012, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.Palette");//$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget");//$NON-NLS-1$
dojo.require("dijit._Container");//$NON-NLS-1$
dojo.require("dijit._Contained");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragManager");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.dnd.DragSource");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.IconUtils");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.paletteInfo");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool");//$NON-NLS-1$//$NON-NLS-2$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

(function(){
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var paletteMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PaletteMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.Palette",//$NON-NLS-1$
	[dijit._Widget, dijit._Container],
{
	paletteInfo: com.ibm.relm.explorer.web.ui.internal.view.edit.paletteInfo.defaultPaletteInfo,
	autoRender: true,
	iconBase: "",//$NON-NLS-1$
	rel: "", // module name to calculate the icon path (The default is "lw") //$NON-NLS-1$
	useTool: true,
	layout: "vertical",//$NON-NLS-1$
    defaultDropHandler: "com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool",//$NON-NLS-1$
    defaultClickHandler: "com.ibm.relm.explorer.web.base.internal.canvas.tool.DefaultCreationTool",//$NON-NLS-1$

	dragSources: null,
	selectedEntry: null,
	toolMap: {},
	
	_editorProvider: null, // call getEditor() method
	
	setEditorProvider: function(editorProvider){
		this._editorProvider = editorProvider;
	},

	constructor: function(){
		this.dragSources = [];
	},
	
	postMixInProperties: function(){
		this.inherited(arguments);
		if (!this.contentTitle) this.contentTitle = paletteMessages.paletteTitle;
	},

	postCreate: function(){
		this.inherited(arguments);
		this.domNode.className = "relm-view-palette";//$NON-NLS-1$
		this.domNode.style.overflow = "auto"; // to suppress the unnecessary horizontal scrollbar //$NON-NLS-1$
		dojo.setSelectable(this.domNode, false); // now you can drag a blank area of an item on FF
	},

	startup: function(){
		if(this._started){return;}
		this._started = true;
		if(this.autoRender){
			this.render();
		}
	},

	destroy: function() {
		this.dragSources = null;
		this.selectedEntry = null;
		this.inherited(arguments);
	},

	render: function(){
		if(!this.paletteInfo || !this.paletteInfo.items){ return; }
		if(this.paletteInfo.rel){
			this.rel = this.paletteInfo.rel;
		}
		this.createPalette(this.paletteInfo.items);
	},
	
	createPalette: function(items, insertIndex){
		for(var i = 0; i < items.length; i++){
			if (insertIndex >= 0) insertIndex++;
			var item = items[i];
			if(item.display == "false"){ continue; }//$NON-NLS-1$
			if(this.layout == "horizontal" && this.hasChildren()){//$NON-NLS-1$
				this.addChild(new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSeparator({}), insertIndex);
			}
			var params;
			if(item.isFolder){
				params = {
					icon: item.icon,
					name: item.name,
					description: item.description,
					paletteId: this.id,
					item: item,
					menuInfos: item.menuInfos,
					delayedRender: item.delayedRender
				};
				params.depth = item.depth ? item.depth : 0;

				var folder = new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteFolder(params);
				this.addChild(folder, insertIndex);
				if (item.initialState == "closed" && !item.delayedRender) {//$NON-NLS-1$
					setTimeout(function(){ folder.folderClickHandler(); }, 0);
				} else if (item.initialState == "open" && item.delayedRender) { //$NON-NLS-1$
					if (!this._openFolderCache) this._openFolderCache = [];
					this._openFolderCache.push(folder);
				}
			}else{
				params = {
					icon: item.icon,
					name: item.name,
					description: item.description,
					props: item.props,
					paletteId: this.id,
					menuInfos: item.menuInfos,
					clickHandler: item.clickHandler || this.defaultClickHandler,
					dropHandler: item.dropHandler || this.defaultDropHandler,
				};
				if (item.onItemCreate) {
					params.onItemCreate = item.onItemCreate;
				}
				params.depth = item.depth ? item.depth : 0;
				var entry = new com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteEntry(params);
				this.addChild(entry, insertIndex);

				var ds = new com.ibm.relm.explorer.web.base.internal.dnd.DragSource(entry.domNode, "component", entry);//$NON-NLS-1$
				ds.targetShouldShowCaret = true;
				this.connect(ds, "onDragStart", "onDragStart"); // move start //$NON-NLS-1$ //$NON-NLS-2$
				this.connect(ds, "onDragEnd", "onDragEnd"); // move end //$NON-NLS-1$ //$NON-NLS-2$
				ds.offsetParent = dojo.doc.body;
				this.dragSources.push(ds);
			}
		}
		this._flushOpenFolderCache();
	},

	_flushOpenFolderCache: function() {
		if (!this._openFolderCache || this._openFolderCache.length == 0) return;
		
		var folder = this._openFolderCache.shift();
		if (this._openFolderCache.length == 0)
			delete this._openFolderCache;

		folder .folderClickHandler(function() {
			this._flushOpenFolderCache();
		});
	},
	
	onDragStart: function(e){
		dojo.addClass(document.body, 'relm-view-paletteMoving'); //$NON-NLS-1$
		this._connector = this.connect(com.ibm.relm.explorer.web.base.internal.dnd.dragManager, "onMouseMove", "onDragMove"); // moving //$NON-NLS-1$ //$NON-NLS-2$
		var item = dijit.byNode(e.dragSource.domNode);
		item.changeTool(item.dropHandler || this.defaultDropHandler);
	},

	onDragEnd: function(e){
		dojo.removeClass(document.body, 'relm-view-paletteMoving'); //$NON-NLS-1$
		this.disconnect(this._connector); // moving
		this.pushedEntry = null;
		if(e.dragStatus === "dropFailure"){//$NON-NLS-1$
			// set in DragManager
			this.entryDeselected();
		}
	},
	
	onDragMove: function(e){
		// someone may want to connect to this...
	},

	nop: function(){
		return false;
	},

	clearPaletteItems: function(){
		for(var i = 0, len = this.dragSources.length; i < len; i++){
			this.dragSources[i].unregister();
		}
		this.dragSources = [];
		this.destroyDescendants();
	},
	
	deselect: function(){
		if(this.selectedEntry){
			this.selectedEntry.deselect();
			this.selectedEntry = null;
		}
	},

	entrySelected: function(entryWidget, tool){
		if(!this._editorProvider) return;
		var message = {
			type: "PaletteSelectionChangedNotification",//$NON-NLS-1$
			src: this,
			selection: entryWidget,
			tool: tool,
			editor: this._editorProvider._getDiagramEditor()
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);//$NON-NLS-1$
	},

	entryDeselected: function(entryWidget){
		var message = {
			type: "PaletteSelectionChangedNotification",//$NON-NLS-1$
			src: this,
			selection: null,
			editor: this._editorProvider._getDiagramEditor()
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);//$NON-NLS-1$
	}
});


dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteSeparator",//$NON-NLS-1$
		[dijit._Widget, dijit._Contained, dijit._Templated],
{
	templateString: "<span style='font-size: 1.5em'> | </span>"//$NON-NLS-1$
});
		
dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase",//$NON-NLS-1$
	[dijit._Widget, dijit._Contained],
{
	icon: "",//$NON-NLS-1$
	description: "",//$NON-NLS-1$
	paletteId: "",//$NON-NLS-1$

	_palette: null,
	_menuNode: null,

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmouseover", "itemMouseOverHandler");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmouseout", "itemMouseOutHandler");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "oncontextmenu", "handleContextMenu");//$NON-NLS-1$//$NON-NLS-2$
	},

	buildRendering: function(){
		var p = this._palette = dijit.byId(this.paletteId);
		if (!p) return;

		// <DIV class="paletteFolder">
		//     <A href="javascript:void(0)"><IMG src="a.gif">label</A>
		// </DIV>
		var isVertical = p.layout == "vertical"; //$NON-NLS-1$
		var div = this.domNode = dojo.doc.createElement(isVertical ? "div" : "span");//$NON-NLS-1$//$NON-NLS-2$
		div.className = "paletteCommon paletteFolder paletteFolderLow";//$NON-NLS-1$
		div.style.whiteSpace = "nowrap";//$NON-NLS-1$
		if (isVertical && this.depth > 0) {
			div.style.paddingLeft = this.depth * 3.5 + "ex";//$NON-NLS-1$
		}

		var operation = dojo.doc.createElement("span");//$NON-NLS-1$
		dojo.addClass(operation, "relm-view-palette-collapsible"); //$NON-NLS-1$
		this.domNode.appendChild(operation);
	
		var a = dojo.doc.createElement("A");//$NON-NLS-1$
		a.href = "#";//$NON-NLS-1$
		a.onclick = p.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)

		var path = this.icon;
		var dim = null;
		if(path && path.match(/^\d+ \d+ \d+ \d+$/)){ // "dim"
			dim = path;
			path = p.iconBase;
		}else if(!path){
			path = p.iconBase;
		}
		var icon = this.iconImg = com.ibm.relm.explorer.web.ui.internal.view.edit.createIcon(path, dim, p.rel);
		if(icon){
			// need to sppecify style to use the same style on dnd
			icon.style.verticalAlign = "middle"; //$NON-NLS-1$
			icon.style.paddingRight = "4px"; //$NON-NLS-1$
			a.appendChild(icon);
		}
		
		div.appendChild(a);
		a.appendChild(dojo.doc.createTextNode(this.description));

		 if (this.menuInfos && this.menuInfos.length > 0) {
			 this._createContextMenuIcon(div);
		 }

	},

	itemMouseOverHandler: function(e){
		if (this._menuNode) {
			this.domNode.style.whiteSpace = "normal";//$NON-NLS-1$
			this._menuNode.style.display = ""; //$NON-NLS-1$
		}
	},

	itemMouseOutHandler: function(e){
		if (this._menuNode) {
			this.domNode.style.whiteSpace = "nowrap";//$NON-NLS-1$
			this._menuNode.style.display = "none"; //$NON-NLS-1$
		}
	},
	
	_createContextMenuIcon: function(div) {
		var a = dojo.doc.createElement("A");//$NON-NLS-1$
		a.href = "#";//$NON-NLS-1$
		this.connect(a, "onclick", "handleContextMenu");//$NON-NLS-1$ //$NON-NLS-2$ 
		this.connect(a, "onmousedown", function(e) {dojo.stopEvent(e);});//$NON-NLS-1$ 

		var icon = this.iconImg = com.ibm.relm.explorer.web.ui.internal.view.edit.createIcon('images/actionMenuWithArrow.gif', null, null, paletteMessages.actionMenu); //$NON-NLS-1$
		if(icon){
			// need to sppecify style to use the same style on dnd
			icon.style.verticalAlign = "middle"; //$NON-NLS-1$
			icon.style.paddingLeft = "4px"; //$NON-NLS-1$
			icon.tilte = paletteMessages.actionMenu;
			a.appendChild(icon);
		}
		a.style.cssFloat = "right"; //$NON-NLS-1$
		a.style.cursor = "pointer"; //$NON-NLS-1$
		a.style.display = "none"; //$NON-NLS-1$
		a.title = paletteMessages.actionMenu;
		
		div.appendChild(a);
		
		this._menuNode = a;
	},
	
	handleContextMenu: function(e) {
		 dojo.stopEvent(e);
		 
		 if (this.menuInfos) {
			 var x = e.clientX + 1;
			 var y = e.clientY + 1;
			 var dim = com.ibm.relm.explorer.web.base.internal.util.getDimensions(e.currentTarget);
			 if (dim) {
				 x = dim.left + dim.width;
				 y = dim.top + dim.height;
			 }
			this._palette.selectedEntry = null;
			this._palette.pushedEntry = null;
			 utils.createAndPopupMenu(this, this.menuInfos, x, y);
		 }
	}
});


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteFolder",//$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase,
{
	
	buildRendering: function(){
		this.inherited(arguments);
		this.domNode.className = "paletteCommon paletteFolder paletteFolderLow";//$NON-NLS-1$
		dojo.addClass(this.domNode.firstChild, "relm-view-palette-collapsed"); //$NON-NLS-1$
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onclick", "folderClickHandler");//$NON-NLS-1$ //$NON-NLS-2$ 
	},

	folderClickHandler: function(e){
		this._folderClickHandler();
	},

	_folderClickHandler: function(callback) {
		var self = this;
		var operation = this.domNode.firstChild;
		this._setupChildren().then(function(toggleShowHide) {
			if (toggleShowHide) {
				var collapsedDepth = -1;
				var expanded = dojo.hasClass(operation, "relm-view-palette-can-collapse"); //$NON-NLS-1$
				for(var w = self.getNextSibling(); w && self.depth < w.depth; w = w.getNextSibling()){
					var s = w.domNode.style;
					
					var show = !expanded;
					if (collapsedDepth < 0 && dojo.hasClass(w.domNode, "paletteFolder")) { //$NON-NLS-1$
						collapsedDepth = dojo.hasClass(w.domNode.firstChild, "relm-view-palette-collapsed") ? w.depth : -1; //$NON-NLS-1$
					} else if (collapsedDepth >= 0 && collapsedDepth < w.depth) {
						show = false;
					} else {
						collapsedDepth = -1;
					}
					s.display = show ? "" : "none";//$NON-NLS-1$//$NON-NLS-2$
				}
				if (expanded) {
					dojo.removeClass(operation, "relm-view-palette-can-collapse"); //$NON-NLS-1$
					dojo.addClass(operation, "relm-view-palette-collapsed"); //$NON-NLS-1$
				} else {
					dojo.removeClass(operation, "relm-view-palette-collapsed");//$NON-NLS-1$
					dojo.addClass(operation, "relm-view-palette-can-collapse");//$NON-NLS-1$
				}
			} else {
				dojo.removeClass(operation, "relm-view-palette-collapsed");//$NON-NLS-1$
				dojo.addClass(operation, "relm-view-palette-can-collapse");//$NON-NLS-1$
			}
			if (callback) callback();
		});
	},
	
	_setupChildren: function() {
		var deferred = new dojo.Deferred();
		if (this.delayedRender) {
			var delayedRender = this.delayedRender;
			delete this.delayedRender;
			delayedRender(this.id, this.item, this.depth).then(dojo.hitch(this, function(newItems) {
				if (this._destroyed) return;
				
				var index = dojo.indexOf(this._palette.getChildren(), this);
				this._palette.createPalette(newItems, index);
				
				for (var i = 0; i < newItems.length; i++)
					this._palette.paletteInfo.items.splice(index + 1 + i, 0, newItems[i]);
				deferred.callback(false);
			}));
		} else {
			deferred.callback(true);
		}
		return deferred;
	},

	_openFolder: function(id) {
	},
	
	folderMouseOverHandler: function(evt){
		this.inherited(arguments);
		dojo.removeClass(this.domNode, "paletteFolderLow");//$NON-NLS-1$
		dojo.addClass(this.domNode, "paletteFolderHi");//$NON-NLS-1$
	},

	folderMouseOutHandler: function(evt){
		this.inherited(arguments);
		dojo.removeClass(this.domNode, "paletteFolderHi");//$NON-NLS-1$
		dojo.addClass(this.domNode, "paletteFolderLow");//$NON-NLS-1$
	}
});


dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.edit.PaletteEntry",//$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.edit._PaletteEntryBase,
{
	name: "",//$NON-NLS-1$
	clickHandler: null,
	dropHandler: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
	},

	buildRendering: function(){
		this.inherited(arguments);
		this.domNode.className = "paletteCommon paletteEntry";//$NON-NLS-1$
		this.domNode.description = this.description;
		this.domNode.title = paletteMessages.hoverTips;
		this.domNode.componentClassName = this.name; // ex. "Hello"
		
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "itemMouseDownHander");//$NON-NLS-1$//$NON-NLS-2$
		this.connect(this.domNode, "onmouseup", "itemMouseUpHander");//$NON-NLS-1$//$NON-NLS-2$
	},
	
	deselect: function(){
		this.flat(this.domNode);
		this._palette.selectedEntry = null;
	},

	itemMouseOverHandler: function(e){
		this.inherited(arguments);
		var div = this.domNode;
		if(this._palette.selectedEntry == this){
			return;
		}else{
			this.raised(div);
		}
	},

	itemMouseOutHandler: function(e){
		this.inherited(arguments);
		var div = this.domNode;
		if(this._palette.selectedEntry == this){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	itemMouseDownHander: function(e){
		var div = this.domNode;
		this.sunken(div);
		this._palette.selectedEntry != this && this._palette.deselect();
		this._palette.pushedEntry = this;
	},

	itemMouseUpHander: function(e){
		if(this._palette.pushedEntry != this){
			this._palette.pushedEntry = null;
			this.raised(this.domNode);
			return;
		}
		if(this._palette.selectedEntry == this){
			this.raised(this.domNode);
			this._palette.selectedEntry = null;
			this._palette.entryDeselected(this);
			return;
		}
		if (e.button < 2) {
			this._palette.selectedEntry = this;
			this._palette.pushedEntry = null;
			this.changeTool();
		} else {
			this._palette.selectedEntry = null;
			this._palette.pushedEntry = null;
		}
	},

	changeTool: function(tool){
		if(!this._palette.useTool){ return; }

		// Change the current tool to DefaultCreationTool or
		// a tool specified in the component definition file.
		tool = tool || this.clickHandler;
		var self = this;
		jazz.core.loader.load_async(tool, function() {
			var myTool = self._palette.toolMap[tool];
			if(!myTool) {
				var obj = dojo.getObject(tool);
				myTool = self._palette.toolMap[tool] = new obj();
			}
			self._palette.entrySelected(self, myTool);
		});
	},

	flat: function(div){
		dojo.removeClass(div, "paletteEntryRaised");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntrySunken");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntryFlat");//$NON-NLS-1$
	},

	raised: function(div){
		dojo.removeClass(div, "paletteEntryFlat");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntrySunken");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntryRaised");//$NON-NLS-1$
	},

	sunken: function(div){
		dojo.removeClass(div, "paletteEntryFlat");//$NON-NLS-1$
		dojo.removeClass(div, "paletteEntryRaised");//$NON-NLS-1$
		dojo.addClass(div, "paletteEntrySunken");//$NON-NLS-1$
	}
});

})();
