/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.ParamsEditDialog"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
//
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.DialogHelpHeader"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ParamsEditDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
	
	var tUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
	var load_async = jazz.core.loader.load_async;
	
	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.ParamsEditDialog", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 

		widgetsInTemplate: true,

		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/ParamsEditDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor : function(args) {
			dojo.mixin(this, args);
			this.cloneParams = dojo.clone(this.params);
			this.messages = dojo.i18n.getLocalization(
					"com.ibm.relm.explorer.web.ui.internal", "ParamsEditDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

			this._helpHeader = this.messages.headerTitle;
			this._helpInfo = this.messages.headerText;

			this.propContent = this;
			if(!args) return;
			

		},
		
		postCreate : function() {
			this._saving = false;
			this._contentDirty = false;
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer
			.appendChild(this.messageBox.domNode);

			var div = dojo.create("div", { //$NON-NLS-1$
				"class" : "relm-params-edit-dialog" //$NON-NLS-1$ //$NON-NLS-2$
			});

			var divTable = dojo.create("div", { //$NON-NLS-1$
				style : {
					padding : "1%", //$NON-NLS-1$
					width : "98%" //$NON-NLS-1$
				}
			}, div);

			var divHeading = dojo.create("div", { //$NON-NLS-1$
				"class" : "heading" //$NON-NLS-1$ //$NON-NLS-2$
			}, divTable);
			var divToolbar = dojo.create("div", { //$NON-NLS-1$
				"class" : "toolbar" //$NON-NLS-1$ //$NON-NLS-2$
			}, divHeading);
//			var divHeadingTitle = dojo.create("div", { //$NON-NLS-1$
//			role : "heading", //$NON-NLS-1$
//			innerHTML : this.messages.queryParameterTitle
//			}, divHeading);
			var self = this;
			divToolbar.appendChild(dojo.create("a", { //$NON-NLS-1$
				href : "#", //$NON-NLS-1$
				innerHTML : this.messages.Add,
				title : this.messages.AddParameter,
				style : {
					verticalAlign : "middle", //$NON-NLS-1$
					marginLeft : "8px" //$NON-NLS-1$
				},
				onclick : function(e) {
					dojo.stopEvent(e);
					self._handleAddParameter();
				}
			}));

			this.propTable = new com.ibm.relm.explorer.web.ui.internal.view.common.Table(
					{
						 inlineEdit: false,
						 style : {
							width : "100%"}, //$NON-NLS-1$
							selectable : "true" //$NON-NLS-1$
					});
			this.propTable
			.setupColumns(tUtils.createTableColumns([
					{
						id : "name", //$NON-NLS-1$
						name : this.messages.parameterName,
						editable : true,
						callback : dojo.hitch(this, this.updatePropertyName),
						preventEmpty : true,
						width: "20%" //$NON-NLS-1$

					},
					{
						id : "value", //$NON-NLS-1$
						name : this.messages.parameterValue,
						editable : true,
						callback : dojo.hitch(this, this.updatePropertyValue),
						preventEmpty : false,
						width: "40%" //$NON-NLS-1$
					},
					{
						id : "description", //$NON-NLS-1$
						name : this.messages.parameterDescription,
						editable : true,
						callback : dojo.hitch(this, this.updatePropertyDescription),
						preventEmpty : false,
						width: "40%" //$NON-NLS-1$
					},
					{
						id : "actions", //$NON-NLS-1$
						isAction : true,
						width: "15px" //$NON-NLS-1$
					}
				]));
			divTable.appendChild(this.propTable.domNode);

			this.paramTable.appendChild(div);

			this._refreshing = true;
			this.setParams(this.cloneParams);

			this._refreshing = false;
		},

		updatePropertyName: function(rowData, propName, newName, dirtyKey) {
			var self = this;
			var oldName = rowData.name;
			if(oldName!=newName) {
				if(self.cloneParams[newName] != null) {
					// already exists
					alert(self.messages.errParamNameExists);
					self.setParams(self.cloneParams);
					return;
				}
				self.cloneParams[newName] = self.cloneParams[oldName];
				delete self.cloneParams[oldName];
				self.setParams(self.cloneParams);
				self._handleModified();
			}
		},

		updatePropertyValue: function(rowData, propName, newValue, dirtyKey) {
			var self = this;
			var param = self.cloneParams[rowData.name];
			if(param) {
				var oldValue = param[propName];
				if(oldValue != newValue){
					param[propName] = newValue;
					self.setParams(self.cloneParams);
					self._handleModified();
				}
			}
		},

		updatePropertyDescription: function(rowData, propName, newValue, dirtyKey) {
			var self = this;
			var param = self.cloneParams[rowData.name];
			if(param) {
				var oldValue = param[propName];
				if(oldValue != newValue){
					param[propName] = newValue;
					self.setParams(self.cloneParams);
					self._handleModified();
				}
			}
		},

		setParams : function(params) {
			var propTable = this.propTable;
			propTable.clearItems();

			var self = this;
			for(var name in params) {
				var param = params[name];
				var value = param["value"]; //$NON-NLS-1$
				var description = param["description"]; //$NON-NLS-1$
				propTable
				.addItem({
					name : name,
					value : value,
					description : description,
					actions : [ {
						iconClass : CLASS_DELETE_COMMAND,
						title : self.messages.removeParams,
						onClick : function(e) {
							if (this.data.name) {
								if (confirm(dojo.string
										.substitute(
												self.messages.confirmRemoveParameter,
												[ this.data.name ]))) {
									// remove
									// parameter
//									var query = self.getQuery(self.queryId,  self.cloneConfig);
									delete self.cloneParams[this.data.name];
									self._handleModified();
									self.setParams(self.cloneParams);
								}
							}
						}
					} ]

				});
			}

		},

		_handleAddParameter: function() {
			var self = this;
				load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", dojo.hitch(this, function() {  //$NON-NLS-1$
					com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(self.cloneParams, dojo.hitch(this, this.refreshParamData));
				}));
		},

		refreshParamData: function() {
			var self = this;
				self._handleModified();
				self.setParams(self.cloneParams);
		},

		_checkFields : function() {
			return true;
			
		},
		
		_handleSave : function() {
			if (this._saving)
				return;

			this._saving = true;
			
			if (!this._checkFields()) {
				this._saving = false;
				return;
			}
			
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer,
					"messageBoxVisible"); //$NON-NLS-1$

			this.params = this.cloneParams;
			this._saveComplete();

			if (this.okCallback != null) {
				var ret = {};
				ret[this.parameterName || "parameters"] = this.params; //$NON-NLS-1$
				this.okCallback(ret);
			}
		},

		_handleCancel : function() {
			if (this.cancelCallback != null)
				this.cancelCallback();
		},

		_handleModified : function() {
			if(!this._refreshing) {
				this._contentDirty = true;
				this._updateEnablement();
			}
		},

		_saveComplete : function(args) {

			this._saving = false;

		},

		_updateEnablement : function() {
			this._saveButton.disabled = !this._contentDirty;
		},

		_noCreatePermission : function(selectedFolder) {
		},

		_showErrorMessage : function(message) {
			var status = {
					type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary : message,
					closable : false
			};
			dojo.addClass(this._messageBoxContainer,
					"messageBoxVisible"); //$NON-NLS-1$
					this.messageBox.showMessage(status);
		}
		}
	);

})();
