/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.PathPropertyDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PathPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container

	var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
	
	// Path command types
	var PATH_TYPE_MOVETO = "moveto"; //$NON-NLS-1$
	var PATH_TYPE_LINETO = "lineto"; //$NON-NLS-1$
	var PATH_TYPE_HLINETO = "horizontalLineto"; //$NON-NLS-1$
	var PATH_TYPE_VLINETO = "verticalLineto"; //$NON-NLS-1$
	var PATH_TYPE_CUBICCURVETO = "curveto"; //$NON-NLS-1$
	var PATH_TYPE_SMOOTHCUBICCURVETO = "smoothCurveto"; //$NON-NLS-1$
	var PATH_TYPE_QUADRATICCURVETO = "quadraticBezierCurveto"; //$NON-NLS-1$
	var PATH_TYPE_SMOOTHQUADRATICCURVETO = "smoothQuadraticBezierCurveto"; //$NON-NLS-1$
	var PATH_TYPE_ARCTO = "ellipticalArc"; //$NON-NLS-1$
	var PATH_TYPE_CLOSEPATH = "closepath"; //$NON-NLS-1$
	var PATH_TYPE_UNKNOWN = "unknown"; //$NON-NLS-1$

	// action classes
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.PathPropertyDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin ], 
		{ 
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/PathPropertyDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

		widgetsInTemplate: true,
		
		type : "", //$NON-NLS-1$
		
		pathProperty : null,
		
		_pathTypeOptions : null,
		
		_pathCommandDefs: null,
		
		_currentPane: null,
		
		_currentPaneType: null,

		constructor: function(){
			this._resultValueId = dijit.getUniqueId("textBox_resultValue");//$NON-NLS-1$
			this._formatFieldId = dijit.getUniqueId("textBox_formatValue");//$NON-NLS-1$
		},

		postMixInProperties : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PathPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

			this._pathCommandDefs = [
					{type: PATH_TYPE_MOVETO, name: this.messages.pathNameMoveLabel, command: {type: "m", format: "x y"}},  //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_LINETO, name: this.messages.pathNameLineLabel, command: {type: "l", format: "x y"}},  //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_HLINETO, name: this.messages.pathNameHLineLabel, command: {type: "h", format: "length"}}, //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_VLINETO, name: this.messages.pathNameVLineLabel, command: {type: "v", format: "length"}}, //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_CUBICCURVETO, name: this.messages.pathNameCubicCurveLabel, command: {type: "c", format: "x1 y1 x2 y2 x y"}}, //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_SMOOTHCUBICCURVETO, name: this.messages.pathNameSmoothCubicCurveLabel, command: {type: "s", format: "x2 y2 x y"}}, //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_QUADRATICCURVETO, name: this.messages.pathNameQuadraticCurveLabel, command: {type: "q", format: "x1 y1 x y"}},  //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_SMOOTHQUADRATICCURVETO, name: this.messages.pathNameSmoothQuadraticCurveLabel, command: {type: "t", format: "x y"}},  //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_ARCTO, name: this.messages.pathNameArcLabel, command: {type: "a", format: "rx ry x-axis-rotation large-arc-flag sweep-flag x y"}},  //$NON-NLS-1$//$NON-NLS-2$
					{type: PATH_TYPE_CLOSEPATH, name: this.messages.pathNameCloseLabel, command: {type: "z", format: null}},  //$NON-NLS-1$
					{type: PATH_TYPE_UNKNOWN, name: this.messages.pathNameUnknownLabel, command: {type: "", format: null}}];  //$NON-NLS-1$

			// set up path type options
			var self = this;
			this._pathTypeOptions = [];
			dojo.forEach(this._pathCommandDefs, function(pathCommandDef) {
				var name = pathCommandDef.name;
				var type = pathCommandDef.type;
				self._pathTypeOptions.push({label: name, value: name, pathType: type});
			});

		},
		
		postCreate: function() {
			this._setupPanels();

			// setup path data
			var pathCommands = this._parsePathProperty(this.pathProperty);
			if (!pathCommands || pathCommands.length == 0) {
				var initialValue = this._getDefaultPathCommandValue(this._getPathCommandDef("type", PATH_TYPE_MOVETO)); //$NON-NLS-1$
				pathCommands = this._parsePathProperty(initialValue);
			}
			this._addPathCommands(pathCommands, 0);
		},

		/*
		 * Set up UI Controls
		 */
		_setupPanels: function() {
			
			// initialize toolbar buttons
			var beforeButton = this._createAddButton(this.messages.insertBeforeMessage, this._insertBeforePathCommand, 
					this, this.messages.insertBeforeLabel);
			this.pathEditToolbar.appendChild(beforeButton);

			var afterButton = this._createAddButton(this.messages.insertAfterMessage, this._insertAfterPathCommand, 
					this, this.messages.insertAfterLabel);
			this.pathEditToolbar.appendChild(afterButton);

			// initialize path table
			var updatePathCommand = dojo.hitch(this, this._updatePathCommand);
			this.pathEditTable.setupColumns(tableUtils.createTableColumns(
					[
					 {
	       		    	id: "name",   //$NON-NLS-1$
	       		    	name: this.messages.nameColumnLabel,
	       		    	editable: true, 
	       		    	width: "180px",  //$NON-NLS-1$
	       		    	className: "name",  //$NON-NLS-1$
					 },{
	       		    	id: "value",   //$NON-NLS-1$
	       		    	name: this.messages.valueColumnLabel,
	       		    	editable: true, 
	       		    	callback: updatePathCommand,
	       		    	preventEmpty: true,
	       		    	width: "280px",  //$NON-NLS-1$
		       		   	className: "value",  //$NON-NLS-1$
					 },{
	    		    	id: "actions",  //$NON-NLS-1$
	    		    	name: this.messages.actionColumnLabel,
	    		    	isAction: true
					 }
			]));
			this.connect(this.pathEditTable, "onSelected", this._itemSelected); //$NON-NLS-1$

			// initialize value
			this.textBox_formatValue.set("disabled", true);//$NON-NLS-1$
			this.textBox_formatValue.set("style", "border-style: hidden;");//$NON-NLS-1$//$NON-NLS-2$
			this.textBox_resultValue.set("readOnly", true);//$NON-NLS-1$
			this.textBox_resultValue.set("style", "border-style: hidden;");//$NON-NLS-1$//$NON-NLS-2$

			// initialize path edit controls
			dojo.forEach(this._pathCommandDefs, function(pathCommandDef) {
				var editNode = pathCommandDef.editNode;
				if (editNode) {
					editNode.style.display = "none"; //$NON-NLS-1$
				}
			});
		},

		_itemSelected: function(item) {
			var label = this.messages.formatFieldLabel;
			var format = ""; //$NON-NLS-1$

			if (item && item.data) {
				if (item.data.type === this._currentPaneType) {
					return;
				}
				this._currentPaneType = item.data.type;

				var pathCommandDef = this._getPathCommandDef("type", this._currentPaneType); //$NON-NLS-1$
				if (pathCommandDef && pathCommandDef.type != PATH_TYPE_UNKNOWN) {
					format += pathCommandDef.command.type;
					if (pathCommandDef.command.format) {
						format += "  " + pathCommandDef.command.format; //$NON-NLS-1$
					}
					label += " (" + pathCommandDef.name + ")"; //$NON-NLS-1$//$NON-NLS-2$
				}
			} else {
				this._currentPaneType = PATH_TYPE_UNKNOWN;
			}

			// Show path command format
			this.textBox_formatValue.set("value", format); //$NON-NLS-1$
			this.label_formatValue.innerHTML = label;

			// Hide edit fields for previous type
			if (this._currentPane) {
				this._currentPane.style.display = "none"; //$NON-NLS-1$
				this._currentPane = null;
			}

			// Show edit fields for current type
			this._currentPane = this._getPathCommandDef("type", this._currentPaneType).editNode; //$NON-NLS-1$
			if (this._currentPane) {
				this._currentPane.style.display = "block"; //$NON-NLS-1$
			}
		},

		_getPathCommandDef: function(attr, value) {
			if (!attr || !value) {
				return null;
			}
		
			for (var i = 0; i < this._pathCommandDefs.length; i++) {
				if (this._pathCommandDefs[i][attr] === value) {
					return this._pathCommandDefs[i];
				}
			}
			return null;
		},
		
		_getPathCommandDefByValue: function(value) {
			if (!value) {
				return null;
			}
			
			var command = dojo.trim(value).charAt(0).toLowerCase();
			for (var i = 0; i < this._pathCommandDefs.length; i++) {
				if (this._pathCommandDefs[i].command.type === command) {
					return this._pathCommandDefs[i];
				}
			}
			return this._pathCommandDefs[this._pathCommandDefs.length - 1];
		},
		
		_addPathCommand: function() {
			var defaultPathType = this.pathEditTable.getChildren().length == 0 ? PATH_TYPE_MOVETO : PATH_TYPE_LINETO;
			var value = this._getDefaultPathCommandValue(this._getPathCommandDef("type", defaultPathType)); //$NON-NLS-1$
			var pathCommands = this._parsePathProperty(value);
			this._addPathCommands(pathCommands, -1);
		},
		
		_insertAfterPathCommand: function() {
			var children = this.pathEditTable.getChildren();
			if (!children || children.length == 0) {
				this._addPathCommand();
				return;
			}
			
			var selectedItem = this.pathEditTable.getSelectedItem();
			if (!selectedItem) {
				this._addPathCommand();
				return;
			}

			var selectedIndex = dojo.indexOf(children, selectedItem);
			if (0 > selectedIndex || selectedIndex >= children.length - 1) {
				this._addPathCommand();
				return;
			}
			
			this._insertPathCommand(selectedIndex + 1);
		},
		
		_insertBeforePathCommand: function() {
			var children = this.pathEditTable.getChildren();
			if (!children || children.length == 0) {
				this._addPathCommand();
				return;
			}
			
			var selectedItem = this.pathEditTable.getSelectedItem();
			var selectedIndex = selectedItem != null ? dojo.indexOf(children, selectedItem) : 0;
			this._insertPathCommand(selectedIndex);
		},
		
		_insertPathCommand: function(index) {
			var pathCommands = [];
			dojo.forEach(this.pathEditTable.getChildren(), function(item) {
				pathCommands.push(item.data);
			});

			var defaultPathType = index == 0 ? PATH_TYPE_MOVETO : PATH_TYPE_LINETO;
			var value = this._getDefaultPathCommandValue(this._getPathCommandDef("type", defaultPathType)); //$NON-NLS-1$
			var newPathCommand = this._parsePathProperty(value)[0];

			pathCommands.splice(index, 0, newPathCommand);
			
			this.pathEditTable.clearItems();
			this._addPathCommands(pathCommands, index);
		},
		
		_removePathCommand: function(item) {
			var selectedIndex = dojo.indexOf(this.pathEditTable.getChildren(), item);
			this.pathEditTable.removeChild(item);
			
			var children = this.pathEditTable.getChildren();
			var newIndex = selectedIndex < children.length ? selectedIndex : children.length - 1;
			var item = children[newIndex];
			
			this.pathEditTable.selectItem(item);
			this._itemSelected(item);
			
			this._updatePathProperty();
		},
		
		_updatePathCommand: function(rowData, propName, newValue, row) {
			if (rowData[propName] === newValue) {
				return;
			}

			switch (propName) {
			case "name": //$NON-NLS-1$
				var value = null;
				
				var pathCommandDef = this._getPathCommandDef(propName, newValue);
				if (!pathCommandDef) {
					pathCommandDef = this._getPathCommandDef("type", PATH_TYPE_UNKNOWN); //$NON-NLS-1$
				}
				value = this._getDefaultPathCommandValue(pathCommandDef);

				rowData.type = pathCommandDef.type;
				rowData.name = pathCommandDef.name;
				rowData.value = value;
				break;
			case "value": //$NON-NLS-1$
				var value = newValue;
				if (value && value.length > 0) {
					value = dojo.trim(value);
				}

				var pathCommandDef =  this._getPathCommandDefByValue(newValue);
				if (!pathCommandDef) {
					pathCommandDef = this._getPathCommandDef("type", PATH_TYPE_UNKNOWN); //$NON-NLS-1$
				}
				
				rowData.type = pathCommandDef.type;
				rowData.name = pathCommandDef.name;
				rowData.value = value;
				break;
			default:
				return;
			}
			
			var selectedIndex = dojo.indexOf(this.pathEditTable.getChildren(), row);
			var pathCommands = [];
			dojo.forEach(this.pathEditTable.getChildren(), function(item) {
				pathCommands.push(item.data);
			});
			
			this.pathEditTable.clearItems();
			this._addPathCommands(pathCommands, selectedIndex);
		},

		_getDefaultPathCommandValue: function(command) {
			var value = ""; //$NON-NLS-1$
			
			if (!command || !command.command || !command.command.type) {
				return value;
			}
			
			value += command.command.type;
			
			if (!command.command.format) {
				return value;
			}
			
			// replace all elements with '0'
			var elems = command.command.format.split(" "); //$NON-NLS-1$
			for (var i = 0; i < elems.length; i++) {
				if (elems[i].length > 0) {
					value += " 0"; //$NON-NLS-1$
				}
			}
			return value;
		},
		
		/*
		 * Add path commands to table. 
		 * The item indicated by selectIndex will be selected if the value is valid. If not, the last item will be selected.
		 */
		_addPathCommands: function(pathCommands, selectIndex) {
			// Add commands
			var self = this;
			var updatePathCommand = dojo.hitch(this, this._updatePathCommand);
			dojo.forEach(pathCommands, function(pathCommand){
				self.pathEditTable.addItem(pathCommand, {name: self._pathTypeOptions}, updatePathCommand, this);
			});
			
			// Set selection
			if (0 > selectIndex || selectIndex >= pathCommands.length) {
				selectIndex = this.pathEditTable.getChildren().length - 1;
			}
			
			var item = this.pathEditTable.getChildren()[selectIndex];
			if (item !== this.pathEditTable.getSelectedItem()) {
				this.pathEditTable.selectItem(item);
				this._itemSelected(item);
			}

			// Reassemble path value
			this._updatePathProperty();
		},

		/*
		 * Parse path value and split it into path commands
		 */
		_parsePathProperty: function(value) {
			var pathCommands = [];
			if (!value || value.length == 0) {
				return pathCommands;
			}

			var command = null;
			var fragments = value.split(" "); //$NON-NLS-1$
			dojo.forEach(fragments, function(fragment) {
				if (fragment.match(/^[A-Za-z]/)) {
					this._createAndAddPathCommandObject(command, pathCommands);
					command = fragment;
				} else if (command) {
					command += " " + fragment; //$NON-NLS-1$
				} else {
					this._createAndAddPathCommandObject(command, pathCommands);
					command = fragment;
				}
			}, this);

			this._createAndAddPathCommandObject(command, pathCommands);
			return pathCommands;
		},

		_createAndAddPathCommandObject: function(command, pathCommands) {
			if (command != null) {
				var pathObject = this._createPathCommandObject(command);
				if (pathObject) pathCommands.push(pathObject);
			}
		},
		
		/*
		 * Create instance of path command to be set to table
		 */
		_createPathCommandObject: function(value) {
			var pathCommandDef = this._getPathCommandDefByValue(value);
			if (!pathCommandDef) {
				return null;
			}
			var type = pathCommandDef != null ? pathCommandDef.type : PATH_TYPE_UNKNOWN;
			var label = pathCommandDef.name;
			var removePathCommand = dojo.hitch(this, this._removePathCommand);
			
			return {
				type: type,
				name: label, 
				value: value, 
				actions: [{
					iconClass: CLASS_DELETE_COMMAND,
					title: this.messages.deletePathCommand,
					onClick: function(e){
						removePathCommand(this);
					}
				}]
			};
		},
		
		/*
		 * Assemble path value from path commands
		 */
		_updatePathProperty: function() {
			var pathProperty = "";//$NON-NLS-1$
			dojo.forEach(this.pathEditTable.getChildren(), function(row){
				if (row.data.value && row.data.value.length > 0) {
					pathProperty += row.data.value + " ";//$NON-NLS-1$
				}
			});
			
			// update preview
			this.textBox_resultValue.set("value", pathProperty);//$NON-NLS-1$
		},
		
		getValues: function() {
			return this.textBox_resultValue.value;//$NON-NLS-1$
		}

	});
})();
