/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PrefixSelectionDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container

	var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 
		checkBoxList: null,

		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/PrefixSelectionDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		postMixInProperties : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PrefixSelectionDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate : function() {
			// Table for prefixes
			var prefixTable = dojo.create("table", { //$NON-NLS-1$
				"class" : "prefix-table" //$NON-NLS-1$ //$NON-NLS-2$
			});

			// Header
			var headerRow = dojo.create("tr", null, prefixTable); //$NON-NLS-1$
			dojo.create("th", { //$NON-NLS-1$
				style: {width: "100px"}, //$NON-NLS-1$
				innerHTML: this.messages.checkBoxHeader,
				scope:'col'//$NON-NLS-1$
			}, headerRow); 
			dojo.create("th", { //$NON-NLS-1$
				innerHTML: this.messages.prefixHeader,
				scope:'col'//$NON-NLS-1$
			}, headerRow);
			
			// Show Loading messages while we wait
			this._loadingMessage.show();
			
			var self = this;
			// request to load prefixes asynchronously, 
			// to get the last loaded prefixes, 
			// or the defaults in case of error
			dojo.when(nodeUtils.loadPredefinedPrefixes(), function(prefixList){
				self.checkBoxList = [];
				dojo.forEach(prefixList, function(prefix){
					var row = dojo.create("tr", null, prefixTable); //$NON-NLS-1$
					var td1 = dojo.create("td", null, row); //$NON-NLS-1$
					this.checkBoxList.push(dojo.create("input", { //$NON-NLS-1$
						"type": "checkbox", //$NON-NLS-1$ //$NON-NLS-2$
						"value": prefix,//$NON-NLS-1$ 
						"aria-label": prefix,//$NON-NLS-1$ 
						onclick: function(e) {
							self._saveButton.disabled = false;
						}
					}, td1));
					dojo.create("td", { //$NON-NLS-1$
						style: {whiteSpace: "nowrap"}, //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(prefix)
					}, row);
				}, self);
	
				self.contentTable.appendChild(prefixTable);
				self._loadingMessage.hide();
			});			
		},

		
		_handleSave : function() {
			if (this._saving)
				return;
			this._saving = true;
			
			var prefixContent = ""; //$NON-NLS-1$
			dojo.forEach(this.checkBoxList, function(cbox){
				if(cbox.checked) {
					prefixContent += cbox.value + "\n"; //$NON-NLS-1$
				}
			});
			
			if (this.okCallback)
				this.okCallback({prefixText: prefixContent});

			this._saving = false;
		},

		_handleCancel : function() {
			if (this.cancelCallback)
				this.cancelCallback();
		}
	});
})();
