/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.view.edit.PrefixSelectionDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.ui.internal", "PrefixSelectionDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
	var prefixesLoaded = false;
	var PREDEFINED_PREFIXES_AND_NS = [
		{ prefix: "dcterms", namespace: "http://purl.org/dc/terms/"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "rdf", namespace: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "rdfs", namespace: "http://www.w3.org/2000/01/rdf-schema#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "xsd", namespace: "http://www.w3.org/2001/XMLSchema#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "ldp", namespace: "http://www.w3.org/ns/ldp#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc", namespace: "http://open-services.net/ns/core#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_am", namespace: "http://open-services.net/ns/am#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_asset", namespace: "http://open-services.net/ns/asset#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_cm", namespace: "http://open-services.net/ns/cm#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_qm", namespace: "http://open-services.net/ns/qm#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_rm", namespace: "http://open-services.net/ns/rm#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "oslc_config", namespace: "http://open-services.net/ns/config#"}, //$NON-NLS-1$ //$NON-NLS-2$
		{ prefix: "rtc_cm", namespace: "http://jazz.net/xmlns/prod/jazz/rtc/cm/1.0/"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "rqm_qm", namespace: "http://jazz.net/ns/qm/rqm#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "pd", namespace: "http://jazz.net/ns/pd#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "pd_ext", namespace: "http://jazz.net/ns/pd/extensions#"}, //$NON-NLS-1$ //$NON-NLS-2$
		{ prefix: "dm_diagram", namespace: "http://jazz.net/ns/dm/diagram#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "dm_document", namespace: "http://jazz.net/ns/dm/document#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "dm_linktypes", namespace: "http://jazz.net/ns/dm/linktypes#"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "foaf", namespace: "http://xmlns.com/foaf/0.1/"}, //$NON-NLS-1$//$NON-NLS-2$
		{ prefix: "prov", namespace: "http://www.w3.org/ns/prov#"} //$NON-NLS-1$ //$NON-NLS-2$
	];
	var PREDEFINED_PREFIXES = "";//$NON-NLS-1$
	var PREDEFINED_PREFIX = "";//$NON-NLS-1$

	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.le.ui.internal.view.edit.PrefixSelectionDialog", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 
		checkBoxList: null,

		templatePath : dojo.moduleUrl("com.ibm.relm.le", "ui/internal/view/edit/templates/PrefixSelectionDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		postMixInProperties : function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.le.ui.internal", "PrefixSelectionDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate : function() {
			// Table for prefixes
			var prefixTable = dojo.create("table", { //$NON-NLS-1$
				"class" : "prefix-table" //$NON-NLS-1$ //$NON-NLS-2$
			});

			// Header
			var headerRow = dojo.create("tr", null, prefixTable); //$NON-NLS-1$
			dojo.create("th", { //$NON-NLS-1$
				style: {width: "100px"}, //$NON-NLS-1$
				innerHTML: this.messages.checkBoxHeader,
				scope:'col'//$NON-NLS-1$
			}, headerRow); 
			dojo.create("th", { //$NON-NLS-1$
				innerHTML: this.messages.prefixHeader,
				scope:'col'//$NON-NLS-1$
			}, headerRow);
			
			// Show Loading messages while we wait
			this._loadingMessage.show();

			this.reset();
			var self = this;
			// request to load prefixes asynchronously, 
			// to get the last loaded prefixes, 
			// or the defaults in case of error
			dojo.when(this.getPredefinedPrefixes(), function(prefixList){
				self.checkBoxList = [];
				dojo.forEach(prefixList, function(prefix){
					var row = dojo.create("tr", null, prefixTable); //$NON-NLS-1$
					var td1 = dojo.create("td", null, row); //$NON-NLS-1$
					this.checkBoxList.push(dojo.create("input", { //$NON-NLS-1$
						"type": "checkbox", //$NON-NLS-1$ //$NON-NLS-2$
						"value": prefix,//$NON-NLS-1$ 
						"aria-label": prefix,//$NON-NLS-1$ 
						onclick: function(e) {
							self._saveButton.disabled = false;
						}
					}, td1));
					dojo.create("td", { //$NON-NLS-1$
						style: {whiteSpace: "nowrap"}, //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(prefix)
					}, row);
				}, self);
	
				self.contentTable.appendChild(prefixTable);
				self._loadingMessage.hide();
			});
		},

		_handleSave : function() {
			if (this._saving)
				return;
			this._saving = true;
			
			var prefixContent = ""; //$NON-NLS-1$
			dojo.forEach(this.checkBoxList, function(cbox){
				if(cbox.checked) {
					prefixContent += cbox.value + "\n"; //$NON-NLS-1$
				}
			});
			
			if (this.okCallback)
				this.okCallback({prefixText: prefixContent});

			this._saving = false;
		},

		_handleCancel : function() {
			if (this.cancelCallback)
				this.cancelCallback();
		},

		getPredefinedPrefixes: function(){
			return this.getPredefinedPrefixesAndNamespaces(false);
		},
		getPredefinedPrefixesAndNamespaces: function(includesNS){
			includeNS = includesNS || true;
			if(!prefixesLoaded){
				var deferred = new dojo.Deferred();
				var self = this;
				jazz.client.xhrGet({
					url: com.ibm.relm.le.ui.queryServiceUri + "?method=vocabularies", //$NON-NLS-1$
					handleAs: "json", //$NON-NLS-1$
					load: function(data){
						if(data){
							PREDEFINED_PREFIXES_AND_NS = data;
							self.reset();
						}
					},
					handle: function(){
						deferred.callback(includesNS ? PREDEFINED_PREFIXES_AND_NS : PREDEFINED_PREFIXES);
					}
				});
				prefixesLoaded = true;
				return deferred;
			}
			else 
				return (includesNS ? PREDEFINED_PREFIXES_AND_NS : PREDEFINED_PREFIXES);
		},
		reset: function(){
			PREDEFINED_PREFIXES = dojo.map(PREDEFINED_PREFIXES_AND_NS, function(ns){
				return "PREFIX " + ns.prefix + ": <" + ns.namespace + ">"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			});

			PREDEFINED_PREFIX = PREDEFINED_PREFIXES.join('\n') + '\n'; //$NON-NLS-1$ //$NON-NLS-2$

			this.PREDEFINED_PREFIX = PREDEFINED_PREFIX;
		}
	});
})();
