/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.PropertiesPane"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerAttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ResourceNodeAttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.TextAttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkAttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$

dojo.requireLocalization(
		"com.ibm.relm.explorer.web.ui.internal", "PropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
	/**
	 * Widget to show property pane
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.PropertiesPane", [ dijit._Widget, dijit._Templated ], { //$NON-NLS-1$
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/PropertiesPane.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		diagramView: null,

		constructor : function(args){
			this.messages = dojo.i18n.getLocalization(
					"com.ibm.relm.explorer.web.ui.internal", "PropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			if (!this.contentTitle) this.contentTitle = this.messages.propertiesPaneTitle;
		},
		
		postCreate : function() {
			this.inherited(arguments);
		
			this.titleNode.innerHTML = this.messages.nothingSelected;
			this._containerProperty = new com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerAttributeContent({diagramView: this.diagramView});
			this._resourceProperty = new com.ibm.relm.explorer.web.ui.internal.view.edit.ResourceNodeAttributeContent({diagramView: this.diagramView});
			this._textProperty = new com.ibm.relm.explorer.web.ui.internal.view.edit.TextAttributeContent({diagramView: this.diagramView});
			this._linkProperty = new com.ibm.relm.explorer.web.ui.internal.view.edit.LinkAttributeContent({diagramView: this.diagramView});
			this._defaultProperty = new com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent({diagramView: this.diagramView});
			// tentative
			this.subscribe("btp:SelectionChanged", this.onSelectionChanged); //$NON-NLS-1$

		},
		
		destroy: function(){
			this._containerProperty.destroy();
			this._resourceProperty.destroy();
			this._textProperty.destroy();
			this._defaultProperty.destroy();
			
			this.nodeSection.destroy();
			this.containerSection.destroy();

			this.inherited(arguments);
		},
		
		// Editor.fireSelectionChanged published this message
		onSelectionChanged: function(message){
			var selectedWidget = dojo.isArray(message.selection) ? null : message.selection;

			var attrContent = null;
			var containerContent = null;
			
			// find a target widget (node or container)
			if(selectedWidget){
				var viewer = selectedWidget.hlp_getEditor();
				if(this._getTargetEditor() != viewer){
					// do nothing
					return;
				}
				selectedWidget = viewer.getEditableNode(selectedWidget);
				if(selectedWidget){
					attrContent = this.getAttributeContent(selectedWidget);
					if (attrContent) {
						attrContent.setWidget(selectedWidget);
						this.nodeSection.setLabel(this.getTitle(selectedWidget));
					
						// find a container
						var selectedContainer = viewer.getEditableContainer(selectedWidget);			
						if(selectedContainer && selectedContainer != selectedWidget && selectedContainer != selectedContainer.hlp_getEditor()){
							containerContent = this.getAttributeContent(selectedContainer);
							containerContent.setWidget(selectedContainer);
							this.containerSection.setLabel(this.getTitle(selectedContainer));
						}
					}
				}
			}
			
			this.setContents(attrContent, containerContent);
		},
		
		_getTargetEditor: function(){
			if(this.diagramView){
				return this.diagramView.getDiagramEditor();
			}
			return null;
		},
		
		getTitle: function(widget){
			if(widget){
				if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer) {
					return this.messages.titleContainer;
				} else if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.Text) {
					return this.messages.titleText;
				} else if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.Image) {
					return this.messages.titleImage;
				} else if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode) {
					return this.messages.titleResourceNode;
				} else if (this._isLinkConnection(widget)) {
					return this.messages.linkTitle;
				} else if (this._isConnectionWidget(widget)) {
					return this.messages.nothingSelected;
				} else {
					return "\"" + widget.componentInfoId + "\"";//$NON-NLS-1$ //$NON-NLS-2$
				} 
			}else{
				return this.messages.nothingSelected;
			}
		},
		
		getAttributeContent : function(widget) {
			if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer) {
				return this._containerProperty;
			}
			if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode) {
				return this._resourceProperty;
			}
			if(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.Text) {
				return this._textProperty;
			}
			if (this._isLinkConnection(widget)) {
				return this._linkProperty;
			} else if (this._isConnectionWidget(widget)) {
				return null;
			}
			return this._defaultProperty;
		},

		_isConnectionWidget: function(widget) {
			if(djConfig.isDebug){
				return false;
			}
			return (widget instanceof com.ibm.relm.explorer.web.base.internal.gdc.PolylineConnection) ||
					(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.connections.RoundConnection);
		},
		
		_isLinkConnection: function(widget) {
			if (widget instanceof com.ibm.relm.explorer.web.base.internal.gdc.PolylineConnection) {
				return widget.links;
			}
		},
		
		setContents : function(node, container) {
			dojo.style(this.titleNode, "display", node || container ? "none" : "");//$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this.setSectionContent(this.nodeSection, node);
			this.setSectionContent(this.containerSection, container);
		},
		
		setSectionContent : function(container, content) {
			var containerNode = container.containerNode;
			var childNodes = containerNode.childNodes; 
			for(var i=childNodes.length-1; i >= 0; i--){
				var child = childNodes.item(i);
				if(child) {
					containerNode.removeChild(child);
				}
			}
			if(content && content.domNode) {
				containerNode.appendChild(content.domNode);
				if(container.isCollapsed()) container.collapse();
				dojo.style(container.domNode, "display", "");//$NON-NLS-1$ //$NON-NLS-2$
			}else{
				dojo.style(container.domNode, "display", "none");//$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		// called when selected node/container is changed
		setTarget : function(type) {
			
		}
	});
})();