/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.view.edit.PropertyEditDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.le.ui.internal", "PropertyEditDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
var tableUtils = com.ibm.relm.le.ui.internal.view.common.TableUtils;
var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;
var ns = com.ibm.relm.le.ui.namespace;

// action classes
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

/**
 * Widget to Edit parameter
 */
dojo.declare("com.ibm.relm.le.ui.internal.view.edit.PropertyEditDialogContent", //$NON-NLS-1$
[ dijit._Widget, dijit._Templated], 
{ 
	templatePath : dojo.moduleUrl("com.ibm.relm.le", "ui/internal/view/edit/templates/PropertyEditDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,

	property: null,
	originalName: null,

	constructor: function(args){
		this.property = args.data.property;
		this.originalName = args.data.name;
		this._les = [];
		this._lesIdToLe = {};
		this._loadAllElements(null);
	},
	postMixInProperties: function(args) {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.le.ui.internal", "PropertyEditDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._setupPanels();

		if(this.property){
			// setup property data
			this.propertyNameTextBox.set("value", this.property.name); //$NON-NLS-1$

			//TODO
			if(dojo.isArray(this.property.linkTo) && this.property.linkTo.length > 0){
				this._addLinkTos(this.property.linkTo, 0);
			}
		}
	},
	
	/*
	 * Set up UI Controls
	 */
	_setupPanels: function() {
		// initialize path table
		var updateLinkTo = dojo.hitch(this, this._updateLinkTo);
		this.linkTosEditTable.setupColumns(tableUtils.createTableColumns(
				[{
					id: "id", //$NON-NLS-1$
					name: this.messages.nameColumnLabel,
					editable: true,
					callback: updateLinkTo,
					preventEmpty: true,
					width: "280px" //$NON-NLS-1$
				},{
					id: "label", //$NON-NLS-1$
					name: this.messages.labelColumnLabel,
					callback: dojo.hitch(this, this.updateEnablement),
					editable: true,
					width: "300px" //$NON-NLS-1$
				},{
					id: "actions", //$NON-NLS-1$
					name: this.messages.actionColumnLabel,
					isAction: true
				}]
		));
	},

	_loadAllElements: function(uri){
		var self = this;
		leServiceUtils.getElements(uri).then(function(results){
			dojo.forEach(results, function(resource){
				switch(resource.type){
				case ns.ae.Folder:
					self._loadAllElements(resource.uri);
					break;
				case ns.ae.Ae:
					self._les.push(resource);
					self._lesIdToLe[resource.id] = resource;
					break;
				default:
					break;
				}
			});
			if(dojo.isArray(self.property.linkTo) && self.property.linkTo.length > 0){
				self.linkTosEditTable.clearItems();
				self._addLinkTos(self.property.linkTo, 0);
			}
		});
	},

	_handleUpdatePropertyName: function(name){
		this.updateEnablement();
	},
	updateEnablement: function() {
		var name = this.propertyNameTextBox.get("value"); //$NON-NLS-1$
		var empty = dojo.some(
				[{label: name}].concat(this.linkTosEditTable.getData()), //Check name and labels of Link To
				function(data){return !dojo.isString(data.label) || data.label.length <= 0;});
		this.setOkButtonDisabled(empty);
	},
	setOkButtonDisabled: function(){},

	_removeLinkTo: function(item) {
		var selectedIndex = dojo.indexOf(this.linkTosEditTable.getChildren(), item);
		this.linkTosEditTable.removeChild(item);
		item.destroy();

		var children = this.linkTosEditTable.getChildren();
		var newIndex = selectedIndex < children.length ? selectedIndex : children.length - 1;
		var item = children[newIndex];
		this.linkTosEditTable.selectItem(item);
	},

	/**
	 * For callback function of linkTosTable update
	 */
	_updateLinkTo: function(rowData, propName, newValue, row) {
		if (rowData[propName] === newValue) {
			return;
		}

		switch (propName) {
			case "id": //$NON-NLS-1$
				break;
			case "label": //$NON-NLS-1$
				if(newValue && newValue.length > 0){
					newValue = dojo.trim(newValue);
				}
				break;
			default:
				break;
		}
		rowData[propName] = newValue;

		var selectedIndex = dojo.indexOf(this.linkTosEditTable.getChildren(), row);
		var linkTos = [];
		dojo.forEach(this.linkTosEditTable.getChildren(), function(item) {
			linkTos.push(item.data);
		});

		this.linkTosEditTable.clearItems();
		this._addLinkTos(linkTos, selectedIndex);
		this.updateEnablement();
	},

	_handleAddLinkTo: function(e){
		dojo.stopEvent(e);
		this._addLinkTos([this._getDefaultLinkTo()], -1);
	},

	_getDefaultLinkTo: function(){
		if(!this._les || this._les.length <= 0){
			return;
		}
		var le = this._les[0];
		return {id: le.id, label: le.title};
	},
	/*
	 * Add Link Tos to table. 
	 * The item indicated by selectIndex will be selected if the value is valid. If not, the last item will be selected.
	 */
	_addLinkTos: function(linkTos, selectIndex) {
		// Add id
		var self = this;
		var updateLinkTo = dojo.hitch(this, this._updateLinkTo);
		dojo.forEach(linkTos, function(linkTo){
			var options = dojo.map(dojo.clone(self._les), function(le){
				return {label: le.title, value: le.id};
			});
			dojo.mixin(linkTo, {actions: [
											{
												iconClass: CLASS_DELETE_COMMAND,
												title: self.messages.removeLinkToAction,
												onClick: function(e){
													var label = this.data.label;
													if(label){
														if(confirm(dojo.string.substitute(self.messages.confirmRemoveLinkTo, [ label ]))){
															self._removeLinkTo(this);
														}
													}
												}
											}
										]});
			self.linkTosEditTable.addItem(linkTo, {id: options}, updateLinkTo, self);
		});

		// Set selection
		if (0 > selectIndex || selectIndex >= linkTos.length) {
			selectIndex = this.linkTosEditTable.getChildren().length - 1;
		}

		var item = this.linkTosEditTable.getChildren()[selectIndex];
		if (item !== this.linkTosEditTable.getSelectedItem()) {
			this.linkTosEditTable.selectItem(item);
		}
	},

	getValues: function() {
		var name = this.propertyNameTextBox.get("value"); //$NON-NLS-1$
		var linkTos = dojo.map(this.linkTosEditTable.getChildren(), function(row){
			return {id: row.data.id, label: row.data.label};
		});
		return {originalName: this.originalName, name: name, linkTos: linkTos};
	}
});
})();