/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
//
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dojox.layout.ResizeHandle"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.QueryUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.common.ImportQueryDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryEditDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
	var orionEditor = null, globalSetText = null;
	
	var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$
	
	var tUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
	var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
	var queryUtils = com.ibm.relm.explorer.web.ui.internal.view.common.QueryUtils;
	
	var BASE_QUERY_ID = "newquery"; //$NON-NLS-1$
	
	var _defaultSparql = nodeUtils.INITIAL_SPARQL_CONTENT;
	
	var queryUtilsMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "QueryUtilsMessages"); //$NON-NLS-1$  //$NON-NLS-2$

	/**
	 * Widget to Edit query
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 

		widgetsInTemplate: true,

		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/QueryEditDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$

		addNewQuery: false,
		
		constructor : function(args) {
			dojo.mixin(this, args);
			this.cloneConfig = dojo.clone(this.config);
			this.messages = dojo.i18n.getLocalization(
					"com.ibm.relm.explorer.web.ui.internal", "QueryEditDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

			this.propContent = this;
			if(!args) return;
			this._sparqlId = dijit.getUniqueId('sparqlTextareaId');//$NON-NLS-1$
			this._statusId = dijit.getUniqueId('relm-QueryEditorStatus'); //$NON-NLS-1$

			new com.ibm.relm.common.web.ui.internal.orion.SparqlPlugin({initializer: dojo.hitch(this, this.initOrionEditor)});
		},
		
		postCreate : function() {

			/* Override the original get and set methods to support Orion Editor */
			this.queryTextArea.origGet = this.queryTextArea.get;
			this.queryTextArea.origSet = this.queryTextArea.set;
			
			this.queryTextArea.get = function(property) {
				if(property.toUpperCase() === "VALUE") { //$NON-NLS-1$
					if(orionEditor === null) //$NON-NLS-1$
						return ""; //$NON-NLS-1$
					else
						return orionEditor.getText();
				}
				else
					return this.origGet(property);
			};
			
			this.queryTextArea.set = function(property,value) {
				if(property.toUpperCase() === "VALUE") { //$NON-NLS-1$
					if(orionEditor === null) //$NON-NLS-1$
						globalSetText = value.replace(/\s+$/g,''); //$NON-NLS-1$
					else {
						setTimeout(function() { // Defer setting of text to allow DOM to load completely
							orionEditor.setText(value); //$NON-NLS-1$
							orionEditor.getUndoStack().reset();
						},20);
					}
				}
				else
					this.origSet(property,value);
			};

			this.connect(this.queryTextArea, "onkeypress", function(ev){ //$NON-NLS-1$
				if(ev.keyCode === dojo.keys.ESCAPE){
					dojo.stopEvent(ev);
				}
			});

			this._contentDirty = false;

			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer
			.appendChild(this.messageBox.domNode);

			if(!this.queryId || !this.config.getQueryTemplate(this.queryId)) {
				this.addNewQuery = true;
				var newId = this.queryId || this._getNewQueryId();
				this.cloneConfig.queryTemplates.push({id: newId,
														params: {},
														sparql: _defaultSparql});
				this.queryId = newId;
				this._handleModified();
			}
			
			this.queryIdTextBox.set("value",  this.queryId); //$NON-NLS-1$

			this.propTable.setupColumns(tUtils.createTableColumns(
					[
					 {
						 id : "name", //$NON-NLS-1$
						 name : this.messages.parameterName,
						 editable : true,
						 callback : dojo.hitch(this, this._updateParamName),
						 preventEmpty : true

					 },
					 {
						 id : "value", //$NON-NLS-1$
						 name : this.messages.parameterValue,
						 editable : true,
						 callback : dojo.hitch(this, this._updateParamValue),
						 preventEmpty : false
					 },
					 {
						 id : "description", //$NON-NLS-1$
						 name : this.messages.parameterDescription,
						 editable : true,
						 callback : dojo.hitch(this, this._updateParamDescription),
						 preventEmpty : false
					 },
					 {
						 id : "actions", //$NON-NLS-1$
						 name : this.messages.action,
						 isAction : true
					 }

					 ]));

			nodeUtils.loadPredefinedPrefixes(); // pre-load prefixes 

			this._refreshing = true;
			var query = this.getQuery(this.queryId,  this.cloneConfig);
			if(query){
				this.setParams(query.params);
			}

			this.setSparql();
			this._refreshing = false;

			// allow resize
			this._resizeHandle = new dojox.layout.ResizeHandle({
				targetContainer: this.queryTextArea
			}).placeAt(this.queryTextAreaContainer);
			
		},

		initOrionEditor: function() {
			var self = this;
			
			// Instantiate an instance of the Orion Editor and initialize it
			require(["orion/editor/edit", "orion/editor/textMateStyler", "orion/editor/sparqlContentAssist", "orion/editor/sparqlGrammar"], function(edit, mTextMateStyler, mSPARQLContentAssist, mSparqlGrammar) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				
				// Line and column number status bar
				var status = ""; //$NON-NLS-1$
				var statusReporter = function(message, isError) {
					if (isError) {
						status =  "ERROR: " + message; //$NON-NLS-1$
					} else {
						status = message;
					}
					self.queryTextStatus.innerHTML = status;
				};
				
				// Instantiate the editor
				orionEditor = edit({
					parent: self.queryTextArea,
					statusReporter: statusReporter,
					autoPairParentheses: true,
					autoPairBraces: true,
					autoPairSquareBrackets: true,
					autoPairAngleBrackets: true,
					autoPairQuotations: true,
					smartIndentation: true
				});
				
				// Add SPARQL content assist to the editor
				var contentAssist = orionEditor.getContentAssist ? orionEditor.getContentAssist() : orionEditor._contentAssist;
				contentAssist.providers.push(new mSPARQLContentAssist.SPARQLContentAssistProvider());
				
				// Add Grammar based syntax highlighter for SPARQL to the editor
				var syntaxHighlighter = {
					styler: null,
					highlight: function(editor) {
						if (this.styler) {
							this.styler.destroy();
							this.styler = null;
						}
						var textView = editor.getTextView();
						this.styler = new mTextMateStyler.TextMateStyler(textView, mSparqlGrammar.SparqlGrammar());
					}
				};
				
				syntaxHighlighter.highlight(orionEditor);
				if(globalSetText != null)
				{
					orionEditor.setText(globalSetText);
					orionEditor.getUndoStack().reset();
					globalSetText = null;
				}
				
				var textView = orionEditor.getTextView();
				textView.addEventListener("ModelChanged", dojo.hitch(self,self._handleQueryEdit)); //$NON-NLS-1$
			});
		},

		// --- Query id edit  ---
		
		_getNewQueryId : function() {
			
			for(var i = 1; true ; i++) {
				var newId = BASE_QUERY_ID + i;
				if (this.getQuery(newId, this.config) == null) {
					return newId; 
				} 
			}
		},

		_handleUpdateQueryId: function(newId) {
			var self = this;
			var oldId = self.queryId;
			if(newId != oldId) {
				self._handleModified();
			}
			
		},
		
		// --- Parameter edit  ---
		/*
		 * 
		 */
		_handleAddParameter: function(ev) {
			var query = this.getQuery(this.queryId,  this.cloneConfig);
			if(query) {
				com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog(query.params, dojo.hitch(this, this.refreshParamData));
			}
			dojo.stopEvent(ev);
		},

		_updateParamName: function(rowData, propName, newName, dirtyKey) {
			var self = this;
			var oldName = rowData.name;
			if(oldName!=newName) {
				var query = self.getQuery(self.queryId,  self.cloneConfig);
				if(query.params[newName] != null) {
					// already exists
					alert(self.messages.errParamNameExists);
					self.setParams(query.params);
					return;
				}
				query.params[newName] = query.params[oldName];
				delete query.params[oldName];
				self.setParams(query.params);
				self._handleModified();
			}
		},

		_updateParamValue: function(rowData, propName, newValue, dirtyKey) {
			var self = this;
			var query = self.getQuery(self.queryId,  self.cloneConfig);
			if(query) {
				var param = query.params[rowData.name];
				if(param) {
					var oldValue = param[propName];
					if(oldValue != newValue){
						param[propName] = newValue;
						self.setParams(query.params);
						self._handleModified();
					}
				}
			}
		},

		_updateParamDescription: function(rowData, propName, newValue, dirtyKey) {
			var self = this;
			var query = self.getQuery(self.queryId,  self.cloneConfig);
			if(query) {
				var param = query.params[rowData.name];
				if(param) {
					var oldValue = param[propName];
					if(oldValue != newValue){
						param[propName] = newValue;
						self.setParams(query.params);
						self._handleModified();
					}
				}
			}
		},

		setParams : function(params) {
			var propTable = this.propTable;
			propTable.clearItems();

			var self = this;
			for(var name in params) {
				var param = params[name];
				var value = param["value"]; //$NON-NLS-1$
				var description = param["description"]; //$NON-NLS-1$
				propTable
				.addItem({
					name : name,
					value : value,
					description : description,
					actions : [ {
						iconClass : CLASS_DELETE_COMMAND,
						title : self.messages.removeParams,
						onClick : function(e) {
							if (name) {
								if (confirm(dojo.string
										.substitute(
												self.messages.confirmRemoveQueryParameter,
												[ this.data.name ]))) {
									// remove
									// parameter
									var query = self.getQuery(self.queryId,  self.cloneConfig);
									delete query.params[name];
									self._handleModified();
									self.setParams(query.params);
								}
							}
						}
					} ]

				});
			}

		},

		refreshParamData: function() {
			var query = this.getQuery(this.queryId,  this.cloneConfig);
			if(query) {

				this._handleModified();
				this.setParams(query.params);
			}
		},

		// --- SPARQL edit  ---

		setSparql: function() {
			var queryTemplate = this.getQuery(this.queryId, this.cloneConfig);
			if(queryTemplate) {
				this.queryTextArea.set("value", queryTemplate.sparql); //$NON-NLS-1$
			}
		},

		getQuery: function (id, config) {

			for(var i = 0; i < config.queryTemplates.length; i++) {
				if(id == config.queryTemplates[i].id) {
					return config.queryTemplates[i];
				}
			}
			return null;
		},

		_handleQueryEdit : function() {
			var queryTemplate = this.getQuery(this.queryId, this.cloneConfig);
			var sparql = this.queryTextArea.get("value"); //$NON-NLS-1$
			if(queryTemplate && queryTemplate.sparql != sparql) {
				queryTemplate.sparql = sparql;
				this._handleModified();
			}
			
		},
		
		// --- prefix dialog ---

		_handleAddPrefix : function(ev) {
			var self = this;
			var prefixDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.PrefixSelectionDialog({
				okCallback: dojo.hitch(this,this._okPrefixDialogCallback),
				cancelCallback: dojo.hitch(this,this._cancelPrefixDialogCallback)
			});

			self._prefixDialog = new jazz.ui.Dialog({
				contentNode: prefixDialog.domNode,
				primaryTitle: self.messages.prefixSelectionDialog,
				width: "600px", //$NON-NLS-1$
				isClosable: true,
				onClose: function(){
					// Disable spell check to avoid red squiggly lines in the editor in Firefox
					if(dojo.isFF !== undefined)
						dojo.body().setAttribute("spellCheck","false"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			});
			dojo.stopEvent(ev);
		},
		
		_okPrefixDialogCallback: function(args)
		{
			
			this._prefixDialog.close();
			this._prefixDialog.destroy();
			// refresh sparql
			var sparql = this.queryTextArea.get("value"); //$NON-NLS-1$
			sparql = args.prefixText + sparql;
			this.queryTextArea.set("value", sparql); //$NON-NLS-1$

		},

		_cancelPrefixDialogCallback: function(args)
		{
			this._prefixDialog.close();
			this._prefixDialog.destroy();
		},

		// --- import ---

		_handleImport : function(ev) {
			var dlg = new com.ibm.relm.explorer.web.ui.internal.common.ImportQueryDialog({
				onOk: function(item){
					var uri = item.uri;
					
					jazz.client.xhrGet({
			            url: com.ibm.relm.explorer.web.ui.jrsServiceUri + "?queryUri=" + uri, //$NON-NLS-1$
			            load: function(result){
			            	// queryText.set clear Undo stack, so call orion editor directly
			            	orionEditor.setText(result);
			            },
			            handle: function(){
			            	
			            }
			        });

			        this.close();
				}
			});
			dlg.startup();
			dojo.stopEvent(ev);
		},
		
		// --- preview dialog ---

		_handlePreviewQueryResult : function(ev) {

			var queryTemplate = this.getQuery(this.queryId, this.cloneConfig);
			var sparql = queryTemplate.sparql;

			var containerParams = nodeUtils.getParamValue(this.containerParams);
			var configParams = nodeUtils.getParamValue(this.cloneConfig.params);
			var queryParams = nodeUtils.getParamValue(queryTemplate.params);
			
			queryUtils.openPreviewQueryResultDialog(sparql, [containerParams, configParams, queryParams]);
			dojo.stopEvent(ev);
		},
		
		// --- editor functions ---

		_checkFields : function() {
			var self = this;
			var oldId = self.queryId;
			var newId = self.queryIdTextBox.get("value"); //$NON-NLS-1$
			
			if(newId == "") { //$NON-NLS-1$
				alert(self.messages.errEmptyQueryId);
				return false;
			}
			if(newId != oldId) {
				var query = self.getQuery(newId,  this.cloneConfig);
				if(query != null) {
					// the query id is already exists
					alert(self.messages.errQueryIdExists);
					return false;
				}
				if(!self.addNewQuery) {
					if(!confirm(this.messages.confirmChangingQueryId)) {
						return false;
					}
				}
			}
			return true;
			
		},
		
		_handleSave : function(callback) {
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer,	"messageBoxVisible"); //$NON-NLS-1$
			
			// save id 
			var query = this.getQuery(this.queryId,  this.cloneConfig);
			var newId = this.queryIdTextBox.get("value"); //$NON-NLS-1$
			query.id = newId;
			
			// save modification
			this.config.queryTemplates = this.cloneConfig.queryTemplates;

			if (callback != null)
				callback({baseQueryId: newId});
		},

		_handleModified : function() {
			if(!this._refreshing) {
				this._contentDirty = true;
			}
		},

		_noCreatePermission : function(selectedFolder) {
		},

		_showErrorMessage : function(message) {
			var status = {
					type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary : message,
					closable : false
			};
			dojo.addClass(this._messageBoxContainer,
			"messageBoxVisible"); //$NON-NLS-1$
			this.messageBox.showMessage(status);
		},
		
		destroy: function() {
			orionEditor && orionEditor.destroy();
			orionEditor = null;
			this._resizeHandle && this._resizeHandle.destroy();
			this._resizeHandle = null;
			this.inherited(arguments);
		}
		
		}
	);

	com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog.openQueryEditDialog = function (config, id, params, okCallback) {

		if(dojo.isString(params)) params = dojo.fromJson(params);
		var queryDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialogContent({
			queryId: id,
			config: config,
			containerParams : params
		});

		return new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
			dialogContent: queryDialog,
			primaryTitle: queryUtilsMessages.queryEditDialog,
			width: "", //$NON-NLS-1$
			allowResize: true,
			validate: function(values, callback) {
				if(queryDialog._checkFields()){
					callback();
				}
			},
			onOpen: function(){
				// Disable spell check to avoid red squiggly lines in the editor in Firefox
				if(dojo.isFF !== undefined)
					dojo.body().setAttribute("spellCheck","false"); //$NON-NLS-1$ //$NON-NLS-2$
				
//				dijit.focus(contentWidget.shapeLabelField);
			},
			onOK: function(values){
				queryDialog._handleSave(okCallback);
			},
			onClose: function(){
				// Remove spell check attribute
				if(dojo.isFF !== undefined)
					dojo.body().removeAttribute("spellCheck"); //$NON-NLS-1$
			}
		});
		
	};
	
})();
