/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.FilteringSelect"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container
	var linkPathUtils = com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils;

	/**
	 * Widget to Edit Fill properties
	 */
dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated], 
{ 
	widgetsInTemplate: true,
	templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/SelectLinkTypeContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

	constructor: function(){
		this._linkTypeId=dijit.getUniqueId('linkType');//$NON-NLS-1$
	},

	postMixInProperties : function(args) {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "SelectLinkTypeContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
		
	postCreate : function() {
		this.inherited(arguments);

		var linkpath = this.props.linkpath[0];
		var sourceTitle = this.props.sourceTitle || linkpath[0].title;
		var targetTitle = this.props.targetTitle || linkpath[linkpath.length-1].title;
		this.selectLinkTypeMessage.innerHTML = dojo.string.substitute(this.messages.selectLinkTypeMessage, [sourceTitle, targetTitle]);
		
		if (dojo.some(this.props.linkpath, function(link) { return dojo.isArray(link); })) {
			this.linkTypeLabel.innerHTML = this.messages.linkPathLabel;
		}
		
		this._updateOptions();
		this.setLinkValue(this.props.index);

		this.showLineCheck.setChecked(!this.props.hidelines);
		this.linkedNodesOnlyCheck.setChecked(!this.props.unlimited);
	},
	
	destroy: function() {
		if (dojo.isFunction(this.closeCallback)) this.closeCallback();
		this.inherited(arguments);
	},
	
	setLinkValue: function(index) {
		this.linkTypeDropdown.set("value", "option" + index); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	getLinkValue: function() {
		var value = this.linkTypeDropdown.get("value"); //$NON-NLS-1$
		if (value) {
			var index = parseInt(value.substring("option".length)); //$NON-NLS-1$
			return this.props.linkpath[index];
		}
		return null;
	},
	
	updateLinkPathList: function(linkpath) {
		this.props.linkpath = linkpath;
		this._updateOptions();
	},
	
	_updateOptions: function() {
		var options = this._createOptions();
		var store = new dojo.store.Memory({idProperty: "value", data: options}); //$NON-NLS-1$
		this.linkTypeDropdown.set("labelAttr", "label"); //$NON-NLS-1$ //$NON-NLS-2$
		this.linkTypeDropdown.set("searchAttr", "label"); //$NON-NLS-1$ //$NON-NLS-2$
		this.linkTypeDropdown.set("store", store); //$NON-NLS-1$
	},
	
	_createOptions: function() {
		var options = [];
		dojo.forEach(this.props.linkpath, function(link, index) {
			var label = linkPathUtils.getLinkPathText(link, "title"); //$NON-NLS-1$
			options.push({value: "option" + index, label: label}); //$NON-NLS-1$
		}, this);
		return options;
	}
});


com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent.openDialog = function(props, okCallback, closeCallback, left, top){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.view.edit.SelectLinkTypeContent({props: props, closeCallback: closeCallback});

	// show dialog
	var dialog = new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		primaryTitle: contentWidget.messages.selectLinkTypeTitle,
		dialogContent: contentWidget,
		width: "560px", //$NON-NLS-1$
		isClosable: true,
		validate: function(values, callback) {
			callback();					
		},
		onOK: dojo.hitch(this, function(){
			var linkpath = contentWidget.getLinkValue();
			var hidelines = !contentWidget.showLineCheck.get("checked"); //$NON-NLS-1$
			var unlimited = !contentWidget.linkedNodesOnlyCheck.get("checked"); //$NON-NLS-1$
			okCallback({linkpath: linkpath, hidelines: hidelines, unlimited: unlimited });
		})
	});
	
	if (left) dialog.domNode.style.left = left + "px"; //$NON-NLS-1$
	if (top) dialog.domNode.style.top = top + "px"; //$NON-NLS-1$
	
	return contentWidget;
};

})();
