/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.StrokePropertyDialogContent"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ColorUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "StrokePropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$


(function() { // module container

	var colorUtils = com.ibm.relm.explorer.web.ui.internal.view.common.ColorUtils;

	var DEFAULT_COLOR = "#FFFFFF"; //$NON-NLS-1$
	/**
	 * Widget to Edit parameter
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.StrokePropertyDialogContent", //$NON-NLS-1$
					[ dijit._Widget, dijit._Templated ], 
		{ 
		widgetsInTemplate: true,
		
		type : "", //$NON-NLS-1$
		strokeProperty : null,
		
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/StrokePropertyDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$

		constructor: function(){
			this._colorComboId = dijit.getUniqueId("colorCombo");//$NON-NLS-1$
			this._styleDropDownId = dijit.getUniqueId("styleDropDown");//$NON-NLS-1$
			this._widthTextBoxId = dijit.getUniqueId("widthTextBox");//$NON-NLS-1$
			this._capDropDownId = dijit.getUniqueId("capDropDown");//$NON-NLS-1$
			this._joinDropDownId = dijit.getUniqueId("joinDropDown");//$NON-NLS-1$
		},

		postMixInProperties : function(args) {
			this.inherited(arguments);
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "StrokePropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate : function() {
			
			this.inherited(arguments);
			this._setupColorNameDropdown();			
			this.setProperty(this.strokeProperty);

		},
		
		setProperty : function(value) {
			try {
				value = dojo.fromJson(value);
			} catch (e) {
				if(!value) {
					value = { color: DEFAULT_COLOR};
				} else {
					value = { color: value};
				}
			}
			this._setStrokePropertyValues(value);

		},
		
		_setupColorNameDropdown: function() {

			colorUtils.loadColorSelectDropdown(this.comboBox_color);

		},
		
		_editColor: function(evt) {
			var self = this;
			var color = self.comboBox_color.get("value"); //$NON-NLS-1$
			
			colorUtils.openColorPickerDialog(color, self._okColorPickerCallback, self);

		},
		
		_okColorPickerCallback: function(color)
		{
			var self = this;
			self.comboBox_color.set("value", color); //$NON-NLS-1$
		},

		
		_setStrokePropertyValues : function(value) {
			
				this.comboBox_color.set("value", value.color); //$NON-NLS-1$
				this.dropdown_style.set("value", value.style); //$NON-NLS-1$
				this.textBox_width.set("value", value.width); //$NON-NLS-1$
				this.dropdown_cap.set("value", value.cap); //$NON-NLS-1$
				this.dropdown_join.set("value", value.join); //$NON-NLS-1$
				
		},
		
		getValues: function() {

			var retValue = {};
			var color = this.comboBox_color.get("value"); //$NON-NLS-1$
			var style = this.dropdown_style.get("value"); //$NON-NLS-1$
			var width = this.textBox_width.get("value"); //$NON-NLS-1$
			var cap = this.dropdown_cap.get("value"); //$NON-NLS-1$
			var join = this.dropdown_join.get("value"); //$NON-NLS-1$

			if(color && color != "") { //$NON-NLS-1$
				retValue.color = color;
			}
			if(style && style != "") { //$NON-NLS-1$
				retValue.style = style;
			}
			if(width && width != "") { //$NON-NLS-1$
				retValue.width = parseInt(width);
			}
			if(cap && cap != "") { //$NON-NLS-1$
				retValue.cap = cap;
			}
			if(join && join != "") { //$NON-NLS-1$
				retValue.join = join;
			}
			
			return dojo.toJson(retValue);
		}

	});
})();
