/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.artifactElementCache"); //$NON-NLS-1$

dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$

(function(){
	
var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;

com.ibm.relm.explorer.web.ui.internal.view.extensions.artifactElementCache = {
	_elementCache: null,
	_elementTitleCache: null,
	_errorCache: null,
	_requeistQueue: null,
	
	getArtifactElement: function(id, callback, errback) {
		if (this._findInCache(id, callback, errback))
			return;
		
		this._request({id: id, callback: callback, errback: errback});
	},

	_request: function(request) {
		if (!this._requestQueue) this._requestQueue = [];
		this._requestQueue.push(request);
		if (this._requestQueue.length == 1)
			setTimeout(dojo.hitch(this, this._run), 10);
	},
	
	_run: function() {
		if (this._requestQueue.length == 0) {
			return;
		}
		var request = this._requestQueue[0];
		this._requestQueue.splice(0, 1);

		if (this._findInCache(request.id, request.callback, request.errback)) {
			setTimeout(dojo.hitch(this, this._run), 10);
			return;
		}

		var self = this;
		leServiceUtils.getElementById(request.id).then(function(element) {
			self.addToCache(element);
			request.callback && request.callback(element);
			setTimeout(dojo.hitch(self, self._run), 10);
		}, function(err) {
			self.addToErrorCache(request.id, err);
			request.errback && request.errback(err);
			setTimeout(dojo.hitch(self, self._run), 10);
		});
	},
	
	_findInCache: function(id, callback, errback) {
		var element = this._elementCache ? this._elementCache[id] : null;
		if (element) {
			callback && callback(element);
			return true;
		}
		var error = this._errorCache ? this._errorCache[id] : null;
		if (error) {
			errback && errback(error);
			return true;
		}
		return false;
	},
	
	getArtifactElementTitle: function(id, callback, errback) {
		if (this._elementTitleCache && this._elementTitleCache[id]) {
			callback(this._elementTitleCache[id].title);
			return;
		}
		this.getArtifactElement(id, function(element) {
			callback(element.title);
		}, function(error) {
			errback ? errback(error) : callback(id);
		});
	},

	setElementItem: function(item) {
		if (!this._elementTitleCache) this._elementTitleCache = {};
		if (dojo.isArray(item)) {
			dojo.forEach(item, function(e) {
				this._elementTitleCache[e.id] = {title: e.title, uri: e.uri};
			});
		} else {
			this._elementTitleCache[item.id] = {title: item.title, uri: item.uri};
		}
	},
	
	addToCache: function(element) {
		if (!this._elementCache) this._elementCache = {};
		this._elementCache[element.id] = element;
	},

	addToErrorCache: function(id, error) {
		if (!this._errorCache) this._errorCache = {};
		this._errorCache[id] = error;
	},

	clearCache: function() {
		this._elementCache = {};
		this._errorCache = {};
		this._elementTitleCache = {};
	}
};

})();