/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.connections"); //$NON-NLS-1$

dojo.require("dojo.DeferredList"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

(function(){
	
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
/*
 * Created in DiagramEditRoot
 */
com.ibm.relm.explorer.web.ui.internal.view.extensions.connections = {
	/*
	 * Manage connections
	 */
	refreshConnections: function(){
		console.debug("refreshConnections"); //$NON-NLS-1$
		var deferred = [];
		var config = this.config;
		if(config.connectionLinks){
			this._linksToBeRemoved = this._linksToBeRemoved.concat(this._links);
			this._links = [];
			deferred = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils.createConnections(config.connectionLinks, this);
		}else{
			this.clearConnections();			
		}
		return deferred;
	},
	
	reloadConnections: function(linkIds, params, reset){
		console.debug("reloadConnections"); //$NON-NLS-1$

		var self = this;
		
		if(!dojo.isArray(linkIds)) {
			linkIds = [linkIds];
		}

		JazzUIWorkbench.showMessage(self.messages.loadingConnections);
		var deferred = self._reloadConnections(linkIds, params, reset);
		dojo.when(new dojo.DeferredList(deferred), function(result){
			console.debug("connections were reloaded"); //$NON-NLS-1$
			JazzUIWorkbench.hideMessageArea();
		}, function(result){
			console.debug("failed to reloaded connections"); //$NON-NLS-1$
			JazzUIWorkbench.hideMessageArea();
			self.showMessage({summary: self.messages.loadConnectionsFailed, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, closable: true});
		});
	},
	
	_reloadConnections: function(linkIds, params, reset) {
		
		var self = this;
		var deferred = [];
		var config = self.config;
		dojo.forEach(linkIds, function(linkId) {
			if(reset) {
				// remove connections of linkId
				dojo.forEach(self.findConnectionsById(linkId), function(linkWidget){
					self.deleteConnection(linkWidget);
				});
			}
			if(config.connectionLinks) {
				dojo.forEach(config.connectionLinks, function(link) {
					if(link.id == linkId) {
						deferred = deferred.concat( com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils.createConnections([link], self, params));
					}
				});
			}
			
		});
		
		return deferred;
	},

	clearConnections: function(){
		this._removeConnections(this._links);
		this._links = [];
	},
	
	_removeConnections: function(connections){
		dojo.forEach(connections, function(l){
			l.removeShape();
			l.destroyRecursive(false);
		}, this );
	},
	
	addStaticConnection: function(linkWidget) {
		if (!this._staticLinks) this._staticLinks = [];
		this._staticLinks.push(linkWidget);
	},
	
	deleteConnection: function(linkWidget){
		var i = dojo.indexOf(this._links, linkWidget);
		if(i>=0){
			this._links.splice(i, 1);
		}
		i = dojo.indexOf(this._staticLinks, linkWidget);
		if(i>=0){
			this._staticLinks.splice(i, 1);
		}
		linkWidget.removeShape();
		linkWidget.destroyRecursive(false);
	},
	
	findConnections: function(sourceWidget, targetWidget){
		return dojo.filter(this._getLinks(), function(linkWidget){
			if(sourceWidget && sourceWidget != linkWidget.sourceAnchorObj.owner) return false;
			if(targetWidget && targetWidget != linkWidget.targetAnchorObj.owner) return false;
			return true;
		});
	},
	
	findConnectionsById: function(linkId) {
		return dojo.filter(this._getLinks(), function(linkWidget){
			if(linkId && linkId == linkWidget.linkId) return true;
			return false;
		});
	},
	
	_getLinks: function() {
		if (!this._links)
			return this._staticLinks;
		else if (!this._staticLinks)
			return this._links;
		return this._links.concat(this._staticLinks);
	}
};

})();