/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.containerlink"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$

(function(){
	var containerLinkActions = com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions;

	com.ibm.relm.explorer.web.ui.internal.view.extensions.containerlink = {
	
	getUpstream: function(id) {
		return this._upstreamMap ? this._upstreamMap[id] : null;
	},
	
	getDownstream: function(id) {
		return this._downstreamMap ? this._downstreamMap[id] : null;
	},
		
	initLinks: function(root, parent, keepValues) {
		if (this.__propertyChangedHandle) this.unsubscribe(this.__propertyChangedHandle);
		this.__propertyChangedHandle = this.subscribe("btp:PropertyChanged", this._onPropertyChanged); //$NON-NLS-1$
		
		if (!keepValues) {
			this._upstreamMap = {};
			this._downstreamMap = {};
		}
		
		var viewer = this.hlp_getEditor();
		this.connect(this, "addContainer", function(container) { //$NON-NLS-1$
			container.connect(container, "_startupConnections", function() { //$NON-NLS-1$
				// setup context menu
				dojo.forEach(container._staticConnections, function(line){
					dojo.mixin(line, com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper);
					line._viewer = viewer;
					line.primaryMenuTarget = true;
					line._getEditActions = function() {
						return containerLinkActions.getLinkedLineActions(viewer, container, line);
					};
				});
			});
		});

		if (!root) return;
		if (!parent) parent = root;
		
		for (var node = parent.firstChild; node; node = node.nextSibling) {
			if (node.nodeName == 'upstream') { //$NON-NLS-1$
				var downstreamId = parent.getAttribute('id'); //$NON-NLS-1$
				var upstreamId = node.getAttribute('id'); //$NON-NLS-1$
				var linktype = node.getAttribute('linktype'); //$NON-NLS-1$
				var unlimited = node.getAttribute('unlimited') == "true"; //$NON-NLS-1$ //$NON-NLS-2$
				var hidelines = node.getAttribute('hidelines') == "true"; //$NON-NLS-1$ //$NON-NLS-2$
				var connectionId = node.getAttribute('connectionId'); //$NON-NLS-1$
				this.addLink(upstreamId, downstreamId, linktype, {unlimited: unlimited, hidelines: hidelines, connectionId: connectionId});
			} else if (node.nodeName == 'container') { //$NON-NLS-1$
				this.initLinks(root, node, true);
			}
		}
	},

	_onPropertyChanged: function(message) {
		var widget = message.hint ? message.hint.widget : null;
		if (message.src != this || !(widget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer)) 
			return;
		
		if (message.propName == "@AddChild") //$NON-NLS-1$
			this._onAdded(widget);
		else if (message.propName == "@RemoveChild") //$NON-NLS-1$
			this._onRemoved(widget);
	},
	
	_onAdded: function(widget) {
		var thisId = widget.getId();
		if (!thisId) return;

		dojo.forEach(widget.__upstream, function(link) {
			this.addLink(link.id, thisId, link.linktype, {unlimited: link.unlimited, hidelines: link.hidelines});
		}, this);
		if (widget.__upstream) delete widget.__upstream;

		dojo.forEach(widget.__downstream, function(link) {
			this.addLink(thisId, link.id, link.linktype, {unlimited: link.unlimited, hidelines: link.hidelines});
		}, this);
		if (widget.__downstream) delete widget.__downstream;
	},

	_onRemoved: function(widget) {
		var id = widget.getId();
		if (!id) return;
		
		widget.__upstream = this.getUpstream(id);
		widget.__downstream = this.getDownstream(id);
		this.removeLinkNode(id);
	},
	
	addLink: function(upstreamId, downstreamId, linktype, props) {
		this._upstreamMap = this._addLink(downstreamId, upstreamId, linktype, props, this._upstreamMap);
		this._downstreamMap = this._addLink(upstreamId, downstreamId, linktype, props, this._downstreamMap);
	},

	removeLink: function(upstreamId, downstreamId, linktype) {
		this._upstreamMap = this._removeLink(downstreamId, upstreamId, linktype, this._upstreamMap);
		this._downstreamMap = this._removeLink(upstreamId, downstreamId, linktype, this._downstreamMap);
	},
	
	updateLinkNode: function(oldId, newId) {
		this._upstreamMap = this._updateLinkNode(oldId, newId, this._upstreamMap);
		this._downstreamMap = this._updateLinkNode(oldId, newId, this._downstreamMap);
	},
	
	removeLinkNode: function(id) {
		this._upstreamMap = this._removeLinkNode(id, this._upstreamMap);
		this._downstreamMap = this._removeLinkNode(id, this._downstreamMap);
	},

	_addLink: function(id1, id2, linktype, props, linkMap) {
		if (!linkMap[id1]) linkMap[id1] = [];
		if (!dojo.some(linkMap[id1], function(link) { return link.id == id2 && link.linktype == linktype; })) {
			linkMap[id1].push(dojo.mixin({id: id2, linktype: linktype}, props));
		}
		return linkMap;
	},

	_removeLink: function(id1, id2, linktype, linkMap) {
		if (!linkMap[id1])
			return linkMap;
		
		var links = dojo.filter(linkMap[id1], function(link) { return link.id != id2 || link.linktype != linktype; });
		if (links.length > 0)
			linkMap[id1] = links;
		else
			delete linkMap[id1];
		return linkMap;
	},
	
	_updateLinkNode: function(oldId, newId, linkMap) {
		if (!linkMap) return;
		
		var oldLinks = null;
		if (linkMap[oldId]) {
			oldLinks = linkMap[oldId];
			delete linkMap[oldId];
		}
		for (key in linkMap) {
			linkMap[key] = dojo.map(linkMap[key], function(link) { if (link.id == oldId) link.id = newId; return link; });
		}
		if (oldLinks) 
			linkMap[newId] = oldLinks;
		return linkMap;
	},
	
	_removeLinkNode: function(id, linkMap) {
		if (!linkMap) return;
		
		if (linkMap[id])
			delete linkMap[id];
		for (key in linkMap) {
			linkMap[key] = dojo.filter(linkMap[key], function(link) { return link.id != id; });
		}
		return linkMap;
	}
};

})();