/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.editor"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementEditUtils"); //$NON-NLS-1$


(function(){

var NodeContainer = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer;
var ResourceNode = com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode;
var openResourceDialog = com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog.openResourceDialog;
var openQueryEditDialog = com.ibm.relm.explorer.web.ui.internal.view.edit.QueryEditDialog.openQueryEditDialog;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var containerLinkActions = com.ibm.relm.explorer.web.ui.internal.view.edit.ContainerLinkActions;
var leEditUtils = com.ibm.relm.explorer.web.ui.internal.view.edit.LifecycleElementEditUtils;

var ICONCLASS_PASTE = "paste-icon"; //$NON-NLS-1$
var ICONCLASS_DELETE = "delete-icon"; //$NON-NLS-1$

var NoopCommand = function() {
	this.execute = function(){
		// do nothing
	};
	this.undo = function(){
		// do nothing
	};
};
/*
 * Created in DiagramEditRoot
 * "this" is com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor
 */
com.ibm.relm.explorer.web.ui.internal.view.extensions.editor = {
	/*
	 * Get editable widget which contains specified widget
	 */
	getEditableNode: function(selectedWidget){
		for(; selectedWidget && selectedWidget.preventSave && 
			!(selectedWidget instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode); 
			selectedWidget = selectedWidget.getParent());
		return selectedWidget;
	},
	/*
	 * Get editable container which contains specified widget
	 */
	getEditableContainer: function(selectedWidget){
		// find a container
		var selectedContainer = selectedWidget;
		if(selectedWidget && !(selectedWidget instanceof NodeContainer)){
			selectedContainer = selectedWidget.getParent();
			// Container which doesn't have attributes should be ignored
			// Such container is created in DefaultLoader
			for(; selectedContainer && (selectedContainer.preventSave || !selectedContainer.componentInfoId); selectedContainer = selectedContainer.getParent());
		}
		return selectedContainer;
	},

	_getEditActions: function() {
		var actions = [];
		var self = this;
		var clipboard = this.clipboard;
		if (!this.focusWidgets || this.focusWidgets.length == 0 || (this.focusWidgets.length == 1 && this.focusWidgets[0] == this)) {
			actions.push({
				nodeWidget: this,
				actionType: 'paste', //$NON-NLS-1$@
				label: self.messages.pasteLabel,
				iconClass: ICONCLASS_PASTE,
				action: function(){
					var message = {
						type: "ActionInvokedNotification", //$NON-NLS-1$
						src: self,
						command: "paste", //$NON-NLS-1$
						arg: null
					};
					dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
				},
				validate: function() {
					return clipboard && clipboard.getAvailableDataFormats(["layout"]).length > 0; //$NON-NLS-1$
				}
			});
		}
		if (this.focusWidgets && this.focusWidgets.length > 1) {
			actions.push({
				nodeWidget: this,
				actionType: 'remove', //$NON-NLS-1$@
				label: self.messages.removeSelectedItems,
				iconClass: ICONCLASS_DELETE,
				action: function(){
					if(confirm(dojo.string.substitute(self.messages.confirmRemoveItems, [self.focusWidgets.length]))){
						var message = {
							type: "ActionInvokedNotification", //$NON-NLS-1$
							src: self,
							command: "del", //$NON-NLS-1$
							arg: null
						};
						dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
					}
				}
			});
		}
		return actions;
	},

	_getAllActions: function() {
		return [];
	},
	
	/*
	 * Add context menu items.
	 * _ActionsMixin#showContextMenu calls viewer.getContextMenuItems when user clicks right mouse button.
	 */
	getContextMenuItems: function(widget){
		var actions = [];

		var editor = this.hlp_getEditor();
		if (!widget.primaryMenuTarget && (editor.focusWidgets && editor.focusWidgets.length > 1))
			return actions;
		
		if(editor && editor._editMode){
			// 
			widget = this.getEditableNode(widget);
			var self = this;
			// Edit Node
			if(widget instanceof ResourceNode){
				actions.push({
					nodeWidget: widget,
					label: self.messages.editNode,
					iconClass: null,
					action: function(){
						// open dialog
						var config = dojo.clone(self.config);
						openResourceDialog({
								getConfig: function(){ return config; },
								getDiagramEditor: function(){ return editor; },
								_updateDirty: function() { },
								getSelectedNode: function() {return widget; }
							}, 
							editorUtils.getResourceInfo(widget, config) || -1, 
							function(){
								dojo.mixin(self.config, config);
								self.reloadLocal();
								// just for making editor dirty
								self.executeCommand(new NoopCommand());
							}
						);
					}
				});
			}
			// Edit Query
			var container = this.getEditableContainer(widget);
			if(container && container != container.hlp_getEditor()){
				actions.push({
					label: self.messages.editParameters,
					action: function() {
						var params = container.getParameters();
						if (container.elementId) {
							leEditUtils.editLifecycleElementParams(container, params).then(function(result) {
								self._changeContainerParameters(editor, container, result);
							});
						} else {
							var paramDialog = new com.ibm.relm.explorer.web.ui.internal.view.edit.ParamsEditDialog({
								params: params,
								okCallback: function(result) {
									self._paramsDialog.close();
									self._paramsDialog.destroy();
									self._changeContainerParameters(editor, container, result.parameters);
								},
								cancelCallback: function(result) {
									self._paramsDialog.close();
									self._paramsDialog.destroy();
								}
							});

							self._paramsDialog = new jazz.ui.Dialog({
								contentNode: paramDialog.mainDiv,
								primaryTitle: paramDialog.messages.paramsEditDialog,
								width: "570px", //$NON-NLS-1$
								isClosable: true
							});
						}
					}
				});
				
				var baseQueryId = container.hlp_getPropertyValue("baseQueryId");//$NON-NLS-1$
				var elementId = container.hlp_getPropertyValue("elementId");//$NON-NLS-1$
				if (baseQueryId || !elementId) {
					if(actions.length == 0){
						actions.push("separator"); //$NON-NLS-1$
					}
					actions.push({
						nodeWidget: container,
						label: self.messages.editQuery,
						iconClass: null,
						action: function(){
							// open dialog
							var config = dojo.clone(self.config);
							var params = container.hlp_getPropertyValue("parameters"); //$NON-NLS-1$
							if(dojo.isString(params)) params = dojo.fromJson(params);
							openQueryEditDialog(
								config, 
								baseQueryId || "",//$NON-NLS-1$
								params, 
								function(value){
									dojo.safeMixin(container, value);
									dojo.mixin(self.config, config);
									self.reloadLocal();
									// just for making editor dirty
									self.executeCommand(new NoopCommand());
								}
							);
						}
					});
				}

				var linkActions = containerLinkActions.getLinkedContainerActions(editor, container.getParent(), container, editor._editMode);
				if (linkActions && linkActions.length > 0) {
					actions.push('separator'); //$NON-NLS-1$
					actions = actions.concat(linkActions);
				}
			}
		}
		return actions;
	},
	
	_changeContainerParameters: function(editor, container, values) {
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
		var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
		command.append(new ec.SetPropertyValueCommand(container, "parameters", values)); //$NON-NLS-1$
		command.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(container));
		editor.executeCommand(command);
	}
};

})();