/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.nodes"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

(function(){
	
var JazzUIWorkbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();

/*
 * Created in DiagramEditRoot
 */
com.ibm.relm.explorer.web.ui.internal.view.extensions.nodes = {
	_initNodeMap: function(){
		if(!this._resourceUriToWidgetId){
	    	// initialize map for resource uri to widget id
	    	this._resourceUriToWidgetId = {};
		}
	},
	
	_initStatusInfo: function(){
		if(!this._statusRequestQueue){
			// status query 
			this._statusRequestQueue = [];
			this._statusCallbacks = [];
		}
	},
	
	/*
	 * Maps between resource URI and resource node widget in this view 
	 */
	putUriAndId: function(uri, id){
		this._initNodeMap();
		var ids = this.uriToIds(uri);
		ids.push(id);
		this._resourceUriToWidgetId[uri] = ids;
	},
	
	removeId: function(uri, id){
		this._initNodeMap();
		var ids = this.uriToIds(uri);
		for(var i = ids.length - 1; i>=0; i--){
			if(ids[i] == id){
				ids.splice(i,1);
			}
		}
	},
	
	uriToIds: function(uri){
		this._initNodeMap();
		return this._resourceUriToWidgetId[uri] || [];
	},
	
	getAllResources: function(){
		this._initNodeMap();
		return this._resourceUriToWidgetId;
	},
	
	getAllResourceNodes: function(){
		var ids = [];
		var all = this.getAllResources();
		for(var uri in all){
			ids = ids.concat(this._resourceUriToWidgetId[uri]);
		}
		return dojo.map(ids, function(id){
			return dijit.byId(id);
		});
	},
	
	getCheckedResourceNodes: function(){
		var allResourceNodes = this.getAllResourceNodes();
		return dojo.filter(allResourceNodes, function(resourceNode){
			return resourceNode.nodeChecked;
		});
	},
	
	getCheckedResourceUris: function(){
		var checkedResourceNodes = this.getCheckedResourceNodes();
		return dojo.map(checkedResourceNodes, function(node){
			return node.uri;
		});
	},
	
	/*
	 * 
	 */
	findText: function(searchText){
		var textNodes = [];
		dojox.gfx.utils.forEach(this.surface, function(shape){
			var text = null;
			if (shape.shape && shape.shape.type == "text") { //$NON-NLS-1$
				text = shape.shape.text;
			} else if (shape.text && shape.text.type == "textpath") { //$NON-NLS-1$ 
				// 'path' is in shape.shape for textpath. text is in shape.text instead. 
				text = shape.text.text;
			} 
			
			if(text &&text.match(new RegExp(".*" + searchText + ".*", "i"))){ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				console.debug(text);
				textNodes.push(shape);
			}
		});
		return textNodes;
	},
	
	/*
	 * Status calculation
	 */
	queueStatusRequest: function(resourceUri, linkTypes, callback){
		this._initStatusInfo();
		this._statusRequestQueue.push({resourceUri: resourceUri, statusLinkTypes: linkTypes});
		this._statusCallbacks.push(callback);
	},
	
	refreshStatus: function(){
		this._initStatusInfo();
		if(this._statusRequestQueue.length == 0) return;
		var self = this;
		JazzUIWorkbench.showMessage(this.messages.updatingStatus);
		dojo.when(com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils.queryLinkedStatus(this._statusRequestQueue, this._statusCallbacks), 
			function(){
				// ok callback
				JazzUIWorkbench.hideMessageArea();			
			},
			function(result){
				// err callback
				JazzUIWorkbench.hideMessageArea();
				if(result.ioargs){
			    	var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(result, result.ioargs);
			    	message.summary = self.messages.updateStatusFailed;
			    	self.showMessage(message);
				}else{
					self.showMessage({
						summary: self.messages.updateStatusFailed,
						details: [{
							message: result.message,
							code: jazz.ui.MessageBox.STATUS_TYPES.ERROR
						}],
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, 
						closable: true});
				}
			}
		);
		this._statusRequestQueue = [];
		this._statusCallbacks = [];
	}
	
};

})();