/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.viewLoader"); //$NON-NLS-1$

dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("dojo.DeferredList"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

// for debug by firebug
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.loader.DefaultLoader"); //$NON-NLS-1$

(function(){
	
var JazzUIWorkbench = com.ibm.relm.explorer.web.ui.internal.util.getJazzUIWorkbench();
var DEFAULT_LOADER_CLASS = "com.ibm.relm.explorer.web.ui.internal.view.loader.DefaultLoader"; //$NON-NLS-1$

/*
 * viewLoader is created in DiagramEditRoot
 */
com.ibm.relm.explorer.web.ui.internal.view.extensions.viewLoader = {
	/*
	 * load method is called by Viewer.postCreate method
	 */
	load: function(url, forceLoad){
		// initialize load queue
		this._loadQueue = [];
		this._resourceLoadingCount = 0;
		
		// links
		this._links = [];
		this._linksToBeRemoved = [];
		
		// load diagram data
		JazzUIWorkbench.showMessage(this.messages.loading);
		var self = this;
		jazz.client.xhrGet({
			url: url,
			preventCache: true,
			load: function(response, ioArgs){
				JazzUIWorkbench.hideMessageArea();
				self._eTag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
				// reset component info
				if(dojo.isFunction(self.registerComponentInfo) && self.componentInfo){
					self._componentInfo = {};
					self.registerComponentInfo(self.componentInfo);
				}
				// load config
				var loaderType = response.loaderType || DEFAULT_LOADER_CLASS;

				jazz.core.loader.load_async(loaderType, function() {
					var ctor = dojo.getObject(loaderType);
					if(ctor){
						var layoutManager = new ctor({viewer:self, viewUri: url});
						JazzUIWorkbench.showMessage(self.messages.constructingView);
						self._loader = layoutManager;
						layoutManager.loadDiagram(response);
						self.refreshBackground(response);
						self.fireLoadResource();
						self.onConfigLoaded(response);
					}
				});
			},
			error: function(response, ioargs){
				JazzUIWorkbench.hideMessageArea();
				console.error(response.message);
				var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs);
				self.showMessage(message);
				response.responseText = null; // don't show the message in error dialog
			},
			headers: {"Accept": "text/json"}, //$NON-NLS-1$ //$NON-NLS-2$
			handleAs: "json" //$NON-NLS-1$ 
		});
	},
	
	reloadLocal: function(configSource){
		var config = configSource || this.getConfig();
		this._loadQueue = [];
		this._resourceLoadingCount = 0;
		this._loader.loadDiagram(config, this._editMode);
		this.refreshBackground(config);
		this.fireLoadResource();
		this.onConfigLoaded(config);
	},

	loadLayout: function(layoutText, initLinks) {
		return dojo.isFunction(this._loader.loadRootContainer) ? this._loader.loadRootContainer(layoutText, this, initLinks) : null;
	},
	
	getConfig: function(){
		if(this.config){
			this.config.layout = this.generate();
		}
		return this.config;
	},
	
	setMessageArea: function(messageArea){
		this._messageArea = messageArea;
	},
	
	showMessage: function(message){
		if(this._messageArea){
			this._messageArea.showMessage(message);
		}
	},
	
	onConfigLoaded: function(config, scopes){
	},
	
	refreshBackground: function(config){
		if(this.bgGroup) {
			// delete previous bg
			this.bgGroup.removeShape();
		}
		if(!config.backgroundshape) return;
		try{
			var backgroundshape = dojo.fromJson(config.backgroundshape);
			var numShapes = (backgroundshape && backgroundshape.length) || 0;
			if(numShapes > 0 ) {
				this.bgGroup = this.group.createGroup();
				this.bgGroup.moveToBack();
				for(var i=numShapes-1; i>=0; i--){
					var bgShape = backgroundshape[i];
					if(!bgShape) continue; // IE
					var shape = this.bgGroup.createShape(bgShape);
					shape.moveToBack(); // zindex
					if(bgShape.font){
						shape.setFont(bgShape.font);
					}
					if(bgShape.fill){
						shape.setFill(bgShape.fill);
					}
					if(bgShape.stroke){
						shape.setStroke(bgShape.stroke);
					}
				}
			}
		} catch(e){
			// something wrong
			console.error("something wrong in refreshBackground(): " + e); //$NON-NLS-1$
		}
	},
	
	/*
	 * load resources 
	 */
	queueLoadResource: function(deferred){
		this._loadQueue.push(deferred);
		console.debug("queueLoadResource: numQueued=" + this._loadQueue.length); //$NON-NLS-1$
	},
	
	_resourceLoadingCount: 0,
	_dummyDeferred: null,
	
	onResourceLoaded: function(){
		
	},
	
	fireLoadResource: function(force){
		if(this._loadQueue.length === 0) {
			if(!force){
				// no resource loading
				JazzUIWorkbench.hideMessageArea();
				return;
			}
			// support lqe auth case... to rerun loading connections and status, create dummy deferred
			if(this._dummyDeferred){
				return;
			}
			var deferred = new dojo.Deferred(dojo.hitch(this, function(){
				this._dummyDeferred = null;
			}));
			this._dummyDeferred = deferred;
			this.queueLoadResource(deferred);
			setTimeout(function(){
				deferred.resolve();
			}, 1000);
			this._resourceLoadingCount = 0;
		}
		var self = this;
		this._resourceLoadingCount++;
		dojo.when(new dojo.DeferredList(this._loadQueue), function(response){
			if(--self._resourceLoadingCount > 0){
				// another loader is still running
				console.debug("Number of queued loading: " + self._resourceLoadingCount); //$NON-NLS-1$
				return;
			}
			// loading resources was done
			self.onResourceLoaded();
			// now loading connections 
			JazzUIWorkbench.showMessage(self.messages.loadingConnections);
			var deferred = self.refreshConnections();
			dojo.when(new dojo.DeferredList(deferred), function(result){
				console.debug("connections were loaded"); //$NON-NLS-1$
				self._removeConnections(self._linksToBeRemoved);
				self._linksToBeRemoved = [];
				JazzUIWorkbench.hideMessageArea();
			}, function(result){
				console.debug("failed to loaded connections"); //$NON-NLS-1$
				JazzUIWorkbench.hideMessageArea();
				self.showMessage({summary: self.messages.loadConnectionsFailed, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, closable: true});
			}).then(function(){
				// loading connections was done
				// now refresh status of resources
				self.refreshStatus();
				dojo.publish("com.ibm.relm.explorer.web.VIEW_LAYOUT_COMPLETED", [self]); //$NON-NLS-1$
			});
		}, console.log);
		this._loadQueue = [];
	}
};

// patch for IE8 performance ... http://bugs.dojotoolkit.org/ticket/12456
if(dojo.isIE){
	var g = dojox.gfx, gs = g.shape, vml = g.vml;
	if (vml && dojox.gfx.vml.Group) { // Added extra check for .Group, since this fails sometimes on IE 10 without and doesn't seem to need it
		var C = gs.Container;
		dojox.gfx.vml.Group.prototype.add = function(shape){
			if(this != shape.getParent()){
				this.rawNode.appendChild(shape.rawNode);
				if(!shape.getParent()){
					// re-apply visual attributes
					shape.setFill(shape.getFill());
					shape.setStroke(shape.getStroke());
				}
				C.add.apply(this, arguments); 
			}
			return this; // self
		};
		dojox.gfx.vml.Surface.prototype.add = dojox.gfx.vml.Group.prototype.add;
	}
}

})();
