/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout"); //$NON-NLS-1$

(function(){

var xmlutils = com.ibm.relm.explorer.web.ui.internal.xmlutils;
	
dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout", //$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout,
{
	/*
	 * create children recursively 
	 */
	setup: function(keepChildren){
		var container = this._container;
		var viewer = container._viewer;
		var domNode = container._domNode;
		var children = xmlutils.getChildNodes(domNode);

		if(keepChildren){

		container.destroyQueryResultChildren();
			
		}else{
			
		container.destroyDescendants();
		
		for(var i=0; i<children.length; i++){
			var c = children[i];
			if(c.nodeType != 1 || c.nodeName == "param") continue; // handles only Element  //$NON-NLS-1$
			var nodeName = c.nodeName;
			var props = {};
			dojo.forEach(c.attributes, function(a){
				// ignore if the property (or method) already exists
				if(a.name != "id" && a.name != "layout" && a.name != "nodeType"){  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					props[a.name] = a.value;
				}
			});
			dojo.mixin(props, {
				_domNode: c,
				_viewer: viewer
			});

			var node = null;
			try {
				node = viewer.createComponent(nodeName, props);
			}catch(e){
				console.error(e);
				var errorMsg = "Failed to create component " + nodeName;  //$NON-NLS-1$
				console.error(errorMsg);
			}
			if(node){
				try {
					container.addChild(node);
				}catch(e){
					console.error(e);
					var errorMsg = "Failed to add component " + nodeName;  //$NON-NLS-1$
					console.error(errorMsg);
				}
				if(nodeName == "container"){ //$NON-NLS-1$
					if (!node.isDownstream()) {
						node.setup();
						node.layout();
					} else if (node.prepareSetup) {
						node.prepareSetup();
					}
				}
			}
		}
		
		} // !keepChildren

		this.createQueryChildren();
	},
	
	/*
	 * set dimension by attributes
	 */
	layout: function(){
	},
	
	_onAllResourceAdded: function() {
		this._updateContainerLinks(this._container);
	},
	
	_updateContainerLinks: function(container) {
		if (!container.hasDownstream()) {
			return;
		}
	
		var viewer = container._viewer;
		dojo.forEach(container.getDownstream(), function(child) {
			var childNode = viewer.findContainer(child.id);
			if (!childNode) {
				return;
			}
			
			if (childNode.readyForSetup) {
				childNode.readyForSetup(container.getId(), child.linktype);
			}
		});
		viewer.fireLoadResource();
	}
});
				
})();
