/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.xmlutils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.Text"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand"); //$NON-NLS-1$

(function(){
	
var xmlutils = com.ibm.relm.explorer.web.ui.internal.xmlutils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout", //$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout,
{
	columns: 1,
	margin: 0,
	defaultSpacing: 30,
	vspacing: 30,
	hspacing: 30,
	showColumnHeader: true,
	showRowHeader: true,
	pack: false,
	showGrid: false,

	_gridLines: null,
	
	_xprops: null,
	_yprops: null,
	
	setup: function() {
		var container = this._container;
		this.columns = this._getNumber(container.columns || this.column, this.columns); 
		this.margin = this._getNumber((container.margin !== undefined && container.margin !== "") ? container.margin : this.margin, this.margin); //$NON-NLS-1$
		var defaultSpacing =  (container.spacing !== undefined && container.spacing !== "") ? container.spacing : this.defaultSpacing; //$NON-NLS-1$
		this.vspacing = this._getNumber((container.vspacing !== undefined && container.vspacing !== "") ? container.vspacing : defaultSpacing, defaultSpacing); //$NON-NLS-1$
		this.hspacing = this._getNumber((container.hspacing !== undefined && container.hspacing !== "") ? container.hspacing : defaultSpacing, defaultSpacing); //$NON-NLS-1$
		this.showGrid = container.showGrid;
		
		var containerNode = container._domNode;
		this._gridLines = []; // grid lines will be destroyed in this.inherited() or container.destroyDescendants
		if(!containerNode) {
			this.inherited(arguments);
			return;
		}
		var columns = xmlutils.findNode("columns", containerNode); //$NON-NLS-1$
		var rows = xmlutils.findNode("rows", containerNode); //$NON-NLS-1$
		if(!columns || !rows) {
			this.inherited(arguments);
			return;
		}
		
		container.destroyDescendants();

		var xprops = this._xprops = xmlutils.getSubNodes("property", columns); //$NON-NLS-1$
		var numCols = this.columns = xprops.length;
		var yprops = this._yprops = xmlutils.getSubNodes("property", rows); //$NON-NLS-1$
		var numRows = yprops.length;
		
		var viewer = container._viewer;
		
		if(this.showColumnHeader){
			if(this.showRowHeader){
				var text = new com.ibm.relm.explorer.web.base.internal.gdc.Text();
				container.addChild(text);
			}
			for(var j=0; j<numCols; j++){
				var xprop = xprops[j];
				var text = viewer.createComponent("GridHeader", { //$NON-NLS-1$
					text: xprop.textContent || xprop.text 
				});
				container.addChild(text);
			}
		}		
		for(var i=0; i<numRows; i++){
			var yprop = yprops[i];
			if(this.showRowHeader){
				var text = viewer.createComponent("GridHeader", { //$NON-NLS-1$
					text: yprop.textContent || yprop.text
				});
				container.addChild(text);
			}
			for(var j=0; j<numCols; j++){
				var xprop = xprops[j];

				// create cell
				var cell = viewer.createComponent("container", { //$NON-NLS-1$
					_viewer: viewer,
					_queryParams: [xprop.getAttribute("value") || xprop.textContent || xprop.text,  //$NON-NLS-1$
					               yprop.getAttribute("value") || yprop.textContent || yprop.text], //$NON-NLS-1$
					baseQueryId: xprop.getAttribute("baseQueryId") || yprop.getAttribute("baseQueryId")  //$NON-NLS-1$ //$NON-NLS-2$
				});
				cell.setLayoutManager(new com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout({
					pack:true,
					showGrid: false
				}));
				container.addChild(cell);
				cell.setup();
			}
		}		
	},
	
	_getNumber: function(val, defaultVal){
		var n = Number(val);
		if(!isNaN(n)) return n;
		return defaultVal;
	},
	
	_getChildren: function(container) {
		return dojo.filter(container.getChildren(), function(c) {return !c.getHidden || !c.getHidden(); });
	},
	
	layout: function(){
		var container = this._container;
		var halign = container.halign;
		var valign = container.valign;
		var children = this._getChildren(container);
		if(this._gridLines && this._gridLines.length > 0){
			// never happen right now because gridlines are always cleared in setup
			children = dojo.filter(children, function(c){
				return c.getWidth; // if getWidth is undefined, it should be gridline
			});
		}
		var numCells = children.length;

		// calculate cell width and cell height
		var colSize = Math.max(Math.min(this.columns, numCells), 1);
		if(this.showRowHeader && this._xprops) colSize++;
		var gridWidth = new Array(colSize);
		var gridHeight = new Array(Math.ceil(numCells / colSize));		
		var index = 0;
		var row = 0;
		while(index < numCells){
			for(var i=0; i<colSize; i++){
				if(index >= numCells) break;
				var c = children[index];
				gridWidth[i] = Math.max(gridWidth[i] || 0, c.getWidth());   
				gridHeight[row] = Math.max(gridHeight[row] || 0, c.getHeight());
				index++;
			}
			row++;
		}

		// set my size
		var totalWidth = this.hspacing * (colSize - 1) + this.margin * 2;
		var totalHeight = (row > 0 ? this.vspacing * (row - 1) : 0) + this.margin * 2;
		dojo.forEach(gridWidth, function(w){ if(!isNaN(w)) totalWidth += w; });
		dojo.forEach(gridHeight, function(h){ if(!isNaN(h)) totalHeight += h; });
		var doPack = (children.length > 0 && (container.pack || this.pack));
		var specifiedWidth = container._specifiedDimension.width;
		if(doPack || !specifiedWidth || specifiedWidth < totalWidth){
			container.setWidth(totalWidth, true);
		}
		var specifiedHeight = container._specifiedDimension.height;
		if(doPack || !specifiedHeight || specifiedHeight < totalHeight){
			container.setHeight(totalHeight, true);
		}

		// locate children
		var offset = this._getOffsetPosition(specifiedWidth, specifiedHeight, totalWidth, totalHeight, halign, valign, doPack);
		var leftOffset = offset.x;
		var topOffset = offset.y;

		index = 0;
		row = 0;
		var top = this.margin + topOffset;
		while(index < numCells){
			var left = this.margin + leftOffset;
			var height = gridHeight[row];
			for(var i=0; i<colSize; i++){
				if(index >= numCells) break;
				var c = children[index];
				if(container.expandChild){
					c.setWidth(gridWidth[i]);
					c.setHeight(height);
				}
				var l, t;
				if(halign == "center"){ //$NON-NLS-1$
					l = left + (gridWidth[i] - c.getWidth()) / 2;
				}else if(halign == "right"){ //$NON-NLS-1$
						l = left + gridWidth[i] - c.getWidth();
				}else{
					l = left;
				}
				if(valign == "middle"){ //$NON-NLS-1$
					t = top + (height - c.getHeight()) / 2;
				}else if(valign == "bottom"){ //$NON-NLS-1$
						t = top + height - c.getHeight();
				}else{
					t = top;					
				}
				c.setTopLeft(t, l);
				left += gridWidth[i] + this.hspacing;
				index++;
			}
			top += height + this.vspacing;
			row++;
		}

		if(this.showGrid){
			// layout grid lines
			this._layoutGridLines(gridWidth, gridHeight, totalWidth, totalHeight);
		}

		// need update shape size which will be used as shadow border during drag and drop
		// but update() causes performance problem because it will call setPosition which causes nodeMoved...
		// container.update();
		if(container.domNode){
			container.domNode._offsetWidth = container.width; 
			container.domNode._offsetHeight = container.height;
		}
		container.setShape();
	},
	
	_getOffsetPosition: function(specifiedWidth, specifiedHeight, totalWidth, totalHeight, halign, valign, doPack) {
		// locate children
		var leftOffset = 0;
		if(!doPack && specifiedWidth && specifiedWidth > totalWidth){
			if(halign == "center"){ //$NON-NLS-1$
				leftOffset = (specifiedWidth - totalWidth) / 2;
			}else if(halign == "right"){ //$NON-NLS-1$
				leftOffset = specifiedWidth - totalWidth;
			}
		}
		var topOffset = 0;
		if(!doPack && specifiedHeight && specifiedHeight > totalHeight){
			if(valign == "middle"){ //$NON-NLS-1$
				topOffset = (specifiedHeight - totalHeight) / 2;
			}else if(valign == "bottom"){ //$NON-NLS-1$
				topOffset = specifiedHeight - totalHeight;
			}
		}
		return {x: leftOffset, y: topOffset};
	},
	
	_layoutGridLines: function(gridWidth, gridHeight, totalWidth, totalHeight){
		var container = this._container;
		var viewer = container._viewer;
		var gridLines = this._gridLines || [];
		if(gridLines.length < gridWidth.length + gridHeight.length - 2){
			for(var i = gridWidth.length + gridHeight.length - 2 - gridLines.length; i>=0; i--){
				// add separator
				var line = viewer.createComponent("GridLine", { //$NON-NLS-1$
					preventSave: true,
					points: [{ x:0, y:0 }, { x:0, y:0 }] // dummy
				});
				container.addChild(line);
				gridLines.push(line);				
			}
			this._gridLines = gridLines; 
		}

		// calculate offset
		var specifiedWidth = container._specifiedDimension.width ? parseInt(container._specifiedDimension.width, 10) : totalWidth;
		if (isNaN(specifiedWidth)) specifiedWidth = totalWidth;
		var specifiedHeight = container._specifiedDimension.height ? parseInt(container._specifiedDimension.height, 10) : totalHeight;
		if (isNaN(specifiedHeight)) specifiedHeight = totalHeight;
		var topLeft = this._getOffsetPosition(specifiedWidth, specifiedHeight, 
				totalWidth, totalHeight, container.halign, container.valign);
		
		// line location and size
		var index = 0;
		var offset=this.margin + topLeft.x;
		var spacing1 = this.hspacing/2;
		var spacing2 = this.hspacing-spacing1;
		for(var c = 0; c < gridWidth.length-1; c++){
			var line = gridLines[index]; 
			offset += gridWidth[c] + spacing1;
			line.setPoints([{x: offset, y: 0},{x: offset, y: specifiedHeight}]);
			offset += spacing2;
			line.update();
			index++;
		}

		offset=this.margin + topLeft.y;
		spacing1 = this.vspacing/2;
		spacing2 = this.vspacing-spacing1;
		for(var r = 0; r < gridHeight.length-1; r++){
			var line = gridLines[index]; 
			offset += gridHeight[r] + spacing1;
			line.setPoints([{x: 0, y: offset},{x: specifiedWidth, y: offset}]);
			offset += spacing2;
			line.update();
			index++;
		}
	},
	
	componentMoveHandler: function(e, tool, movedWidget){
		var cmd = this._createReparentComponentCommand(e, this._container, movedWidget);
		if(cmd){
			tool.getEditor().executeCommand(cmd);
		}else{
			this.layout();
		}
	},
	
	componentDropHandler: function(e, tool){
		var ret = null;
		if(tool.getEditor().componentSpec){
			var refWidget = this._findRefWidget(e);
			ret = tool.insertComponent(e, refWidget);
		}
		return ret;		
	},
	
	getCaretRect: function(e){
		var container = this._container;
		var children = this._getChildren(container);
		var refWidgetIndex = this._findRefWidgetIndex(e, children);
		if(refWidgetIndex < 0) 
			return null;
		var transform = this._getTransform();
		var spacing = Math.max(this.hspacing, 2);
		// calculate caret height
		var nextRowChildIndex = (Math.floor(refWidgetIndex/this.columns)+1)*this.columns;
		var bottom = nextRowChildIndex >= children.length ? (container.getHeight() - this.margin) 
				: children[nextRowChildIndex].getTop();
		var refWidget, left;
		if(refWidgetIndex < children.length){
			refWidget = children[refWidgetIndex];
			left = transform.x + refWidget.getLeft() - spacing/2;
		}else{
			refWidget = children[children.length - 1];
			left = transform.x + refWidget.getLeft() + refWidget.getWidth() + spacing/2;
		}
		return {top: transform.y + refWidget.getTop(), left: left, width: 2, height: bottom - refWidget.getTop()};
	},
	
	_createReparentComponentCommand: function(e, targetWidget, widget){
		var refWidget = this._findRefWidget(e);
		if(refWidget == widget){
			return null;
		}
		var command = new com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand.ReparentCommand(widget, targetWidget, refWidget);
		return command;
	},
	
	_getTransform: function(){
		var w = this._container; 
		var transformX = 0, transformY = 0;
		for(; w && w.group != null; w = w.getParent()){
			var transform = w.group.getTransform();
			if(transform){
				transformX += transform.dx;
				transformY += transform.dy;
			}
		}
		return {x: transformX, y: transformY};
	},
	
	_findRefWidgetIndex: function(e, children){
		if (!children) children = this._getChildren(this._container);
		if(!children || children.length == 0) 
			return -1;
		var transform = this._getTransform();
		var pos =
			dojo.isIE ?
					{x: e.offsetX, y: e.offsetY} :
					{x: e.layerX - transform.x, y: e.layerY - transform.y};
		// find col,row for ref widget
		var col, row;
		var numCols = Math.min(children.length, this.columns);
		for(col=0; col<numCols; col++){
			var c = children[col];
			var center_x = c.getLeft() + c.getWidth()/2;
			if(center_x > pos.x)
				break;
		}
		var numRows = children.length/numCols;
		for(row=0; row<numRows-1; row++){
			var c = children[(row+1)*numCols];
			if(c.getTop() > pos.y)
				break;
		}
		// then return the child index
		return Math.min(row*numCols + col, children.length);		
	},
	
	_findRefWidget: function(e){
		var children = this._getChildren(this._container);
		if(!children || children.length == 0) 
			return null;
		return children[this._findRefWidgetIndex(e, children)];
	}
});

})();