/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.layout.TreeLayout"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.Text"); //$NON-NLS-1$

(function(){

var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

var TREECHILD_QUERY_SPARQL = 
	nodeUtils.PREDEFINED_PREFIX + 
	"SELECT *\n" + //$NON-NLS-1$
	"WHERE {\n" + //$NON-NLS-1$
	"  {\n" + //$NON-NLS-1$
	"    ?source ?linkType ?resource " + //$NON-NLS-1$
	"  }\n" +  //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          dcterms:title ?title .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          oslc:shortTitle ?shortTitle .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          oslc_cm:status ?status .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          rdf:type ?type .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          dcterms:identifier ?identifier .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"  OPTIONAL { \n" + //$NON-NLS-1$
	"  ?resource \n" + //$NON-NLS-1$
	"          dcterms:subject ?tags .\n" + //$NON-NLS-1$
	"  }\n" + //$NON-NLS-1$
	"} ORDER BY ASC(?identifier)\n"; //$NON-NLS-1$

var EXPAND_ICON = dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/images/expand.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var COLLAPSE_ICON = dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/images/collapse.gif"); //$NON-NLS-1$ //$NON-NLS-2$

var TREE_HORIZONTAL = "horizontal"; //$NON-NLS-1$
//var TREE_VERTICAL = "vertical"; //$NON-NLS-1$
var TREE_EXPLORER = "explorer"; //$NON-NLS-1$
//var TREE_DIRECTION_NORMAL = "normal"; //$NON-NLS-1$
var TREE_DIRECTION_REVERSE = "reverse"; //$NON-NLS-1$

var DEFAULT_INDENT = 20;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.layout.TreeLayout", //$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout,
{
	defaultSpacing: 20,
	margin: 20,
	indent: 0,
	
	setup: function(){
		if(!this._rootTreeLayout) {
			// I'm tree root. _rootTreeLayout is propagated to child in _addTreeChild
			this._rootTreeLayout = this;
		}
		var container = this._container;
		if(container.treestyle == TREE_HORIZONTAL){
			this._hoverOrientation = "R"; //$NON-NLS-1$
		} else if (container.treestyle == TREE_EXPLORER ) {
			if(container.treedir == TREE_DIRECTION_REVERSE) {
				this._hoverOrientation = "L"; //$NON-NLS-1$
				this.columns = 1;
//				this.indent = 20;
				this._container.halign = "right"; //$NON-NLS-1$
			} else {
				this._hoverOrientation = "R"; //$NON-NLS-1$
				this.columns = 1;
				this._container.halign = "left"; //$NON-NLS-1$
//				this.indent = 20;
			}
		}else{
			// vertical
			if(container.treedir == TREE_DIRECTION_REVERSE) {
				// right to left
				this._hoverOrientation = "R"; //$NON-NLS-1$
				this.columns = 2;
			} else {
				//left to right
				this._hoverOrientation = "L"; //$NON-NLS-1$
				this.columns = 2;
			}
		}
		if(container.expandDepth !== undefined || this._expandDepth === undefined){
			this._expandDepth = Number(container.expandDepth || 0);
		}
		// _childLinkIdArray is set from container's childLinkId or from parent tree node
		if(this === this._rootTreeLayout || this._childLinkIdArray === undefined){
			var childLinkIdArray = (container.childLinkId || "").split(","); //$NON-NLS-1$ //$NON-NLS-2$
			if(childLinkIdArray && childLinkIdArray.length > 0){
				childLinkIdArray.unshift(""); // dummy first one for myself //$NON-NLS-1$
				this._childLinkIdArray = childLinkIdArray;
			}else{
				this._childLinkIdArray = null;
			}
		}
		// Skip layout support
		if(this._layerVariable === undefined && container.layerVariable){
			this._layerVariable = container.layerVariable;
		}
		if(this._layerOrders === undefined && container.layerOrder){
			var layerOrders = container.layerOrder.split(","); //$NON-NLS-1$
			this._layerOrders = dojo.map(layerOrders, function(lo){
				return lo.replace(/^\s+|\s+$/g, ""); //$NON-NLS-1$
			});
		}
		if(this._layerOrders){
			this._layerOrderMap = [];
			for(var i=0;i < this._layerOrders.length; i++){
				this._layerOrderMap[this._layerOrders[i]] = i;
			}
		}
		this.inherited(arguments);
	},
	
	layout: function(){
		var container = this._container;
		var halign = container.halign;
		var valign = container.valign;
		var children = container.getChildren();
		var numCells = children.length - (this._gridLines ? this._gridLines.length : 0);
		var doSkipLayout = this._layerOrders && this._layerOrders.length > 0;

		// calculate cell width and cell height
		var colSize = Math.max(Math.min(this.columns, numCells), 1);
		if(this.showRowHeader && this._xprops) colSize++;
		var gridWidth = new Array(colSize);
		var gridHeight = new Array(Math.ceil(numCells / colSize));
		var index = 0;
		var row = 0;
		var maxNodeWidth = 0;
		if(container.treedir == TREE_DIRECTION_REVERSE) {
			while(index < numCells){
				for(var i=colSize -1; i >= 0; i--){
					if(index >= numCells) break;
					var c = children[index];
					gridWidth[i] = Math.max(gridWidth[i] || 0, c.getWidth());
					gridHeight[row] = Math.max(gridHeight[row] || 0, c.getHeight());
					if(doSkipLayout && (c instanceof com.ibm.relm.explorer.web.ui.view.nodes.ResourceNode)){
						maxNodeWidth = Math.max(maxNodeWidth, c.getWidth());
					}
					index++;
				}
				row++;
			}
		} else {
			while(index < numCells){
				for(var i=0; i<colSize; i++){
					if(index >= numCells) break;
					var c = children[index];
					gridWidth[i] = Math.max(gridWidth[i] || 0, c.getWidth());
					gridHeight[row] = Math.max(gridHeight[row] || 0, c.getHeight());
					if(doSkipLayout && (c instanceof com.ibm.relm.explorer.web.ui.view.nodes.ResourceNode)){
						maxNodeWidth = Math.max(maxNodeWidth, c.getWidth());
					}
					index++;
				}
				row++;
			}
		}
		// set my size
		var totalWidth = 0;
		if(doSkipLayout){
			// skip layout
			totalWidth = this._layerOrders.length * (maxNodeWidth + this.hspacing) - this.hspacing + this.margin * 2 + this.indent;
		}else{
			totalWidth = this.hspacing * (colSize - 1) + this.margin * 2 + this.indent;
			dojo.forEach(gridWidth, function(w){ if(!isNaN(w)) totalWidth += w; });
		}
		var totalHeight = (row > 0 ? this.vspacing * (row - 1) : 0) + this.margin * 2;
		dojo.forEach(gridHeight, function(h){ if(!isNaN(h)) totalHeight += h; });
		var doPack = ((children.length > 0 || this._isTreeChild()) && (container.pack || this.pack));
		var specifiedWidth = container._specifiedDimension.width;
		if(doPack || !specifiedWidth || specifiedWidth < totalWidth){
			container.setWidth(totalWidth);
		}
		var specifiedHeight = container._specifiedDimension.height;
		if(doPack || !specifiedHeight || specifiedHeight < totalHeight){
			container.setHeight(totalHeight);
		}

		// locate children
		index = 0;
		row = 0;
		var top = this.margin ;
		while(index < numCells){
			var left = 0;
			if(container.treedir == TREE_DIRECTION_REVERSE){
				if(container.treestyle == TREE_EXPLORER){
					left = totalWidth - this.margin - this.indent;
				}else{
					left = totalWidth - this.margin;
				}
				if(doSkipLayout){
					var c = children[index];
					if(c.resource){
						var layerValue = c.resource[this._layerVariable];
						var layer = layerValue ? layerValue.value : this._layerOrders[this._layerOrders.length - 1];
						left -= this._layerOrderMap[layer] * (maxNodeWidth + this.hspacing);
					}
				}
			}else{
				if(container.treestyle == TREE_EXPLORER){
					left = this.margin + this.indent;
				}else{
					left = this.margin;
				}
				if(doSkipLayout){
					var c = children[index];
					if(c.resource){
						var layerValue = c.resource[this._layerVariable];
						var layer = layerValue ? layerValue.value : this._layerOrders[this._layerOrders.length - 1];
						left += this._layerOrderMap[layer] * (maxNodeWidth + this.hspacing);
					}
				}
			}
			var height = gridHeight[row];
			if(container.treedir == TREE_DIRECTION_REVERSE){
				for(var i=colSize-1; i>=0; i--){
					if(index >= numCells) break;
					var c = children[index];
					var l, t;
					if(halign == "center"){ //$NON-NLS-1$
						l = left - gridWidth[i] + (gridWidth[i] - c.getWidth()) / 2;
					} else if(halign == "right"){ //$NON-NLS-1$
						l = left - c.getWidth();
					}else{
						l = left - gridWidth[i];
					}
					if(valign == "middle"){ //$NON-NLS-1$
						t = top + (height - c.getHeight()) / 2;
					}else{
						t = top;
					}
					c.setTopLeft(t, l);
					//c.setWidth(gridWidth[i]);
					//c.setHeight(height);
					left -= (gridWidth[i] + this.hspacing);
					index++;
				}
			} else {
				for(var i=0; i<colSize; i++){
					if(index >= numCells) break;
					var c = children[index];
					var l, t;
					if(halign == "center"){ //$NON-NLS-1$
						l = left + (gridWidth[i] - c.getWidth()) / 2;
					} else if(halign == "right"){ //$NON-NLS-1$
						l = left - c.getWidth();
					}else{
						l = left;
					}
					if(valign == "middle"){ //$NON-NLS-1$
						t = top + (height - c.getHeight()) / 2;
					}else{
						t = top;
					}
					c.setTopLeft(t, l);
					//c.setWidth(gridWidth[i]);
					//c.setHeight(height);
					left += gridWidth[i] + this.hspacing;
					index++;
				}
			}
			top += height + this.vspacing;
			row++;
		}

		if(this.showGrid){
			// layout grid lines
			this._layoutGridLines(gridWidth, gridHeight, totalWidth, totalHeight);
		}

		// need update shape size which will be used as shadow border during drag and drop
		// but update() causes performance problem because it will call setPosition which causes nodeMoved...
		// container.update();
		if(container.domNode){
			container.domNode._offsetWidth = container.width;
			container.domNode._offsetHeight = container.height;
		}
		container.setShape();
	},
	
	createQueryChildren: function(){
		if(!this._isTreeChild()) {
			this.inherited(arguments);
		}
		if(this._expanded){
			this._rootTreeLayout._queueQuery(this);
		}
	},
	
	createChildrenFromQueryResult: function(){
		this.inherited(arguments);
		if(!this._isTreeChild()) {
			this._flashQueuedQueries();
		}
	},
	
	_getLinksForTreeChild: function(childTreeLayout){
		var config = this._container._viewer.config;
		var linkDefs = null;
		if(childTreeLayout._childLinkIdArray && childTreeLayout._childLinkIdArray.length > 0){
			// use the first one. pass second and later one to children
			// see _addTreeChild method.
			linkDefs = config.getLinks(childTreeLayout._childLinkIdArray[0]);
			dojo.forEach(linkDefs, function(l){
				if(l.treeOnly !== false && l.treeOnly !== "false"){ //$NON-NLS-1$
					// treeOnly is not specified explicitly false
					l.treeOnly = true;
				}
			});
		}
		if(!linkDefs){
			linkDefs = config.connectionLinks;
		}
		return linkDefs;
	},
	
	_queryQueue: null,
	
	_queueQuery: function(treeLayout){
		if(!this._queryQueue){
			this._queryQueue = [];
		}
		this._queryQueue.push(treeLayout);
	},
	
	_isEquivalentLinkDefs: function(linkDefs1, linkDefs2){
		if(linkDefs1.length != linkDefs2.length){
			return false;
		}
		for(var i=linkDefs1.length-1; i>=0; i--){
			var l1 = linkDefs1[i];
			var l2 = linkDefs2[i];
			if(l1.type && l1.type != l2.type){
				return false;
			}
			if(l1.baseQueryId && l1.baseQueryId != l2.baseQueryId){
				return false;
			}
		}
		return true;
	},
	
	_flashQueuedQueries: function(){
		var root = this._rootTreeLayout;
		if(!root || !root._queryQueue) return;
		var queue = root._queryQueue;
		root._queryQueue = null; // clear queue
		var bulkLinkDefs = {
			linkDefs : null,
			childTreeLayouts: []
		};
		// make a bulk query if the linkDefs is same as the referenced linkDefs
		var nobulk = root.nobulk || (root._container.nobulk === "true");//$NON-NLS-1$

		dojo.forEach(queue, function(childTreeLayout){
			var linkDefs = root._getLinksForTreeChild(childTreeLayout);
			childTreeLayout.numOfLinkDefs = linkDefs.length;
			if(linkDefs){
				if(!nobulk){
					if(!bulkLinkDefs.linkDefs){
						// keep the first linkDefs as a reference
						bulkLinkDefs.linkDefs = linkDefs;
						bulkLinkDefs.childTreeLayouts.push(childTreeLayout);
						return;
					}
					if(this._isEquivalentLinkDefs(bulkLinkDefs.linkDefs, linkDefs)){
						// if the linkDefs is same as the reference one, add this in the bulk query
						bulkLinkDefs.childTreeLayouts.push(childTreeLayout);
						return;
					}
				}
				// if not same as the reference one, execute query here
				// this should not happen...
				var queries = childTreeLayout._buildQuery(childTreeLayout, linkDefs);
				if(queries){
					childTreeLayout.nobulk = nobulk;
					childTreeLayout._rootTreeLayout = childTreeLayout;
					childTreeLayout.createChildrenFromQueries(queries);
					this._container._viewer.fireLoadResource();
				}
			}
		}, root);
		// execute bulk query
		var queries = root._buildQuery(bulkLinkDefs.childTreeLayouts, bulkLinkDefs.linkDefs);
		if(queries){
			var callback = function(resources, linkDef, delayed, head){
				var vars = head && head.vars;
				var sourceVar = vars && dojo.indexOf(vars,"source") >= 0;//$NON-NLS-1$
				dojo.forEach(bulkLinkDefs.childTreeLayouts, function(childTreeLayout, index){
					// if ?source is not defined, cannot get tree parent
					if(!sourceVar){
						var queries = childTreeLayout._buildQuery(childTreeLayout, bulkLinkDefs.linkDefs);
						if(queries){
							childTreeLayout._rootTreeLayout = childTreeLayout;
							childTreeLayout.createChildrenFromQueries(queries);
							root._container._viewer.fireLoadResource();
						}
						return;
					}
					// bulk query result contains results for other tree node. filter them.
					var myRes = dojo.filter(resources, function(r){
						var sourceResource = r["source"]; //$NON-NLS-1$
						return sourceResource && (sourceResource.value == childTreeLayout._treeParent.uri);
					});
					childTreeLayout._container.setLoading(false);
					// create children
					childTreeLayout.createChildrenFromQueryResult(myRes, linkDef, (index + 1) * root._container._viewer._delayedLayoutTimeout);
				});
				root._flashQueuedQueries();
			};
			var error = function(response, ioargs){
				dojo.forEach(bulkLinkDefs.childTreeLayouts, function(childTreeLayout, index){
					var container = childTreeLayout._container;
					if(container){
						container.setLoading(false);
					}
				});
			};
			dojo.forEach(bulkLinkDefs.childTreeLayouts, function(childTreeLayout){
				childTreeLayout._container.setLoading(true);
			});
			root.executeQuery(queries, callback, error, true);
			this._container._viewer.fireLoadResource();
		}
	},
	
	_isTreeChild: function(){
		return (this._rootTreeLayout && this._rootTreeLayout != this);
	},

	addResources: function(node, resources){
		var deferred = this.inherited(arguments);
		var container = this._container;
		if(container.treestyle == TREE_HORIZONTAL){
			var children = node.getChildren();
			var resNodeChildren = dojo.filter(children, function(child){return child instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode;});
			this.columns = Math.max(resNodeChildren.length, 1);
			var insertIndex = 0;
			if(this.columns < children.length){
				for(; insertIndex<children.length; insertIndex++){
					var child = children[insertIndex];
					if(!(child instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode)){
						break;
					}
				}
			}
			dojo.forEach(children, function(treeParent){
				if(treeParent instanceof com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode
				&& !treeParent._treeChildContainer){
					if(insertIndex > 0){
						node.removeChild(treeParent);
						node.addChild(treeParent, insertIndex++);
					}
					// add expand/collapse icon
					var bb = treeParent.group.children[0].getBoundingBox();
					var icon = treeParent.group.createImage({
						x: bb.width/2 - 6, y: bb.height,
						width: 12, height: 12,
						src: EXPAND_ICON
					});
					this._addTreeChild(treeParent, icon);
				}
			}, this);
			if(insertIndex > 0){
				this.layout();
			}
		}
		return deferred;
	},
	
	addChildResource: function(){
		var treeParent = this.inherited(arguments);
		if(!treeParent || !treeParent.group) return null;
		var container = this._container;
		if(container.treestyle == TREE_HORIZONTAL){
			treeParent.startDir = "bottom"; //$NON-NLS-1$
			treeParent.endDir = "top"; //$NON-NLS-1$
		}else if(container.treestyle == TREE_EXPLORER) {
			if(container.treedir == TREE_DIRECTION_REVERSE) {
				treeParent.startDir = "right"; //$NON-NLS-1$
				treeParent.endDir = "right"; //$NON-NLS-1$
				// add expand/collapse icon
				var bb = treeParent.group.children[0].getBoundingBox();
				var icon = treeParent.group.createImage({
					x: bb.width -9, y: bb.height/2 -6,
					width: 12, height: 12,
					src: EXPAND_ICON
				});
				this._addTreeChild(treeParent, icon);
			} else {
				treeParent.startDir = "left"; //$NON-NLS-1$
				treeParent.endDir = "left"; //$NON-NLS-1$
				// add expand/collapse icon
				var bb = treeParent.group.children[0].getBoundingBox();
				var icon = treeParent.group.createImage({
					x: -5, y: bb.height/2 -6,
					width: 12, height: 12,
					src: EXPAND_ICON
				});
				this._addTreeChild(treeParent, icon);
			}
		}else{
			if(container.treedir == TREE_DIRECTION_REVERSE) {
				// add expand/collapse icon
				treeParent.startDir = "left"; //$NON-NLS-1$
				treeParent.endDir = "right"; //$NON-NLS-1$
				var bb = treeParent.group.children[0].getBoundingBox();
				var icon = treeParent.group.createImage({
					x: -12, y: bb.height/2 - 6,
					width: 12, height: 12,
					src: EXPAND_ICON
				});
				this._addTreeChild(treeParent, icon);
			} else {
				// add expand/collapse icon
				var bb = treeParent.group.children[0].getBoundingBox();
				var icon = treeParent.group.createImage({
					x: bb.width, y: bb.height/2 - 6,
					width: 12, height: 12,
					src: EXPAND_ICON
				});
				this._addTreeChild(treeParent, icon);
			}
		}
		return treeParent;
	},

	_addTreeChild: function(treeParent, icon){
		// add container for tree children
		var container = this._container;
		var viewer = container._viewer;
		var vspacing = this.vspacing; var hspacing = this.hspacing;
		var treeChildContainer = viewer.createComponent("container", { //$NON-NLS-1$
			_viewer: viewer,
			_params: container._params, // reuse parent params
			queryTemplate: {sparql: TREECHILD_QUERY_SPARQL},
			treestyle: container.treestyle,
			treedir: container.treedir,
			vspacing: vspacing,
			hspacing: hspacing,
			halign: container.halign,
			valign: container.valign,
			width: 1, // for initial layout
			height: 1, // for initial layout
			preventSave: true // prevent from saving into souce code ... see xmlGenerator.js
		});
		
		// if childLinkId is comma separated value (e.g. "aaa,bbb,ccc"),
		// pass the 2nd and later value to child (e.g. "bbb,ccc")
		var childLinkIdArrayForNext = null;
		if(dojo.isArray(this._childLinkIdArray)){
			childLinkIdArrayForNext = [].concat(this._childLinkIdArray);
			if(this._childLinkIdArray.length > 1){
				childLinkIdArrayForNext.shift();
			}
		}
		var args = {
			margin: 0,
			indent: (container.treestyle == TREE_EXPLORER) ? DEFAULT_INDENT : 0,
			_expandDepth: this._expandDepth-1,
			_childLinkIdArray: childLinkIdArrayForNext,
			_treeParent: treeParent,
			_rootTreeLayout: this._rootTreeLayout, // propagate root of tree
			pack:true,
			numOfLinkDefs: 0
		};
		var doSkipLayout = this._layerOrders && this._layerOrders.length > 0;
		if(doSkipLayout){
			var layerValue = treeParent.resource[this._layerVariable];
			var layer = layerValue ? layerValue.value : this._layerOrders[this._layerOrders.length - 1];
			var layerIdx = this._layerOrderMap[layer];
			
			args._layerVariable = this._layerVariable;
			args._layerOrders = this._layerOrders.slice(layerIdx + 1);
		}
		var lm = new com.ibm.relm.explorer.web.ui.internal.view.layout.TreeLayout(args);
		if(container.treestyle == TREE_EXPLORER) {
			if(container.treedir == TREE_DIRECTION_REVERSE) {
				treeChildContainer.halign = "right"; //$NON-NLS-1$
			} else {
				treeChildContainer.halign = "left"; //$NON-NLS-1$
			}
		}
		treeChildContainer.setLayoutManager(lm);
		container.addChild(treeChildContainer);
		treeChildContainer.setup();
		treeParent._treeChildContainer = treeChildContainer;
		
		var self = this;
		treeParent.connect(treeParent, "hlp_setHalfFocusFrame", function(){ //$NON-NLS-1$
			icon.moveToFront();
		});
		treeParent.connect(treeParent, "hlp_setFocusFrame", function(){ //$NON-NLS-1$
			icon.moveToFront();
		});
		treeChildContainer.connect(icon.rawNode, "onclick", function(){ //$NON-NLS-1$
			self.expandCollapseChild(treeParent, icon, true);
		});
		treeChildContainer.connect(lm, "addResources", function(){ //$NON-NLS-1$
			lm.numOfLinkDefs--;
			if(lm.numOfLinkDefs <= 0){
				var c = treeChildContainer.getChildren();
				if(!c || c.length == 0) {
					icon.removeShape().destroy();
					delete icon;
				}
			}
		});
		dojo.style(icon.rawNode, "cursor", "pointer"); //$NON-NLS-1$//$NON-NLS-2$
		// auto expand child
		if(this._expandDepth > 0){
			this.expandCollapseChild(treeParent, icon, false);
		}
	},
	
	_buildQuery: function(childTreeLayout, linkDefs) {
		if(!linkDefs || linkDefs.length == 0) {
			return null;
		}
		
		var childTreeLayoutArray = dojo.isArray(childTreeLayout) ? childTreeLayout : [childTreeLayout];
		if(childTreeLayoutArray.length == 0){
			return null;
		}
		
		var queries = [];
		if(dojo.some(linkDefs, function(l){
			return l.type && !l.baseQueryId;
		})){
			// some of linkDefs have type attribute.
			var types = [];
			for(var idx = 0; idx < linkDefs.length; idx++){
				var type = linkDefs[idx].type;
				if(type && !linkDefs[idx].baseQueryId){
					var componentDef = linkDefs[idx].component;
					if(componentDef){
						var values = dojo.map(childTreeLayoutArray, function(ct){
							return "(<" + ct._treeParent.uri + ">)"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						});
						var q = TREECHILD_QUERY_SPARQL.replace("\?linkType", type) + " VALUES (?source) { " + values.join(" ") + "}"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-4$ //$NON-NLS-3$
						queries.push({sparql: q, linkDef: linkDefs[idx]}); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
					}else{
						types.push(type);
					}
				}
			}
			if(types.length > 0){
				var values = dojo.map(types, function(t){
					var ctv = dojo.map(childTreeLayoutArray, function(ct){
						return "(" + t + " <" + ct._treeParent.uri + ">)"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					});
					return ctv.join(" ");//$NON-NLS-1$
				});
				var q = TREECHILD_QUERY_SPARQL + " VALUES (?linkType ?source) { " + values.join(" ") + "}"; //$NON-NLS-1$ //$NON-NLS-3$ //$NON-NLS-2$
				queries.push({sparql: q}); //$NON-NLS-1$//$NON-NLS-2$
			}
		}

		if(dojo.some(linkDefs, function(l){
			return l.baseQueryId;
		})){
			// some of linkDefs have baseQueryId.
			for(var idx = 0; idx < linkDefs.length; idx++){
				var baseQueryId = linkDefs[idx].baseQueryId;
				if(baseQueryId){
					var query = nodeUtils.getLinkQuery(linkDefs[idx], this._container._viewer.config);
					// query template for connections has ?source and ?target.
					// children query should replace those variables to get child resources.
					query.sparql = nodeUtils.replaceAll(query.sparql, "?target", "?resource"); //$NON-NLS-1$ //$NON-NLS-2$
					var parentUris = dojo.map(childTreeLayoutArray, function(ct){
						return "(<" + ct._treeParent.uri + ">)"; //$NON-NLS-1$ //$NON-NLS-2$
					});
					query.sparql = query.sparql +" VALUES (?source) { " + parentUris.join(" ") + "}"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					queries.push({sparql: query.sparql, parameterValues: query.parameterValues, linkDef: linkDefs[idx]});
				}
			}
		}
		return queries;
	},
	
	expandCollapseChild: function(treeParent, icon, isUserAction){
		var treeChildContainer = treeParent._treeChildContainer;
		if(treeChildContainer){
			var container = this._container;
			if(!treeParent._treeChildExpanded){
				var childTreeLayout = treeChildContainer._layoutManager;
				childTreeLayout._expanded = true; // execute query in setup
				treeChildContainer.setLinks({sourceWidgets: [treeParent]});
				treeChildContainer.setup();
				if(isUserAction){
					childTreeLayout._flashQueuedQueries();
				}
				treeParent._treeChildExpanded = true;
				icon.setShape({src: COLLAPSE_ICON});
			}else{
				treeChildContainer._layoutManager._expanded = false; // don't execute query
				treeChildContainer.destroyDescendants(false);
				treeChildContainer.layout();
				treeParent._treeChildExpanded = false;
				icon.setShape({src: EXPAND_ICON});
				for(var n = container; n; n = n.getParent()){
					if(dojo.isFunction(n.layout)){
						n.layout();
					}
					if(!dojo.isFunction(n.getParent)) break;
				}
				// auto expand only at initial creation time
				this._expandDepth = 0;
				treeChildContainer._layoutManager._expandDepth = 0;
			}
		}
	}
});

})();