/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.loader.Config"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil"); //$NON-NLS-1$

(function(){

	var paramUtil = com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.loader.Config", //$NON-NLS-1$
	null, 
{
	diagramId: "", //$NON-NLS-1$
	diagramName: "", //$NON-NLS-1$
	description: "", //$NON-NLS-1$
	
	params: [],
	
	queryTemplates: [],
	resourceQueries: [], // obsolete
	linkQueries: [], // obsolete
	
	componentInfo: {},
	resourceInfo: [],
	_defaultResourceInfo: null,
	
	connectionLinks: [],
	layout: null,
	
	load: function(configJson, params){
		dojo.safeMixin(this, configJson);
		this.paramMixin(this.params, params);
		this.setSystemParamValues(this.params);
		// mixin default values into each resource info
		var defaultResourceInfo = this.getDefaultResourceInfo();
		// hale assumes component info is {items: [...]}
		if(this.componentInfo && !this.componentInfo.items){
			this.componentInfo = {items: this.componentInfo};			
		}
		// migrate old syntaxes 
		this._migrate10();
	},
	
	paramMixin: function(params, argParams) {
		for(p in argParams) {
			// replace parameter value only
			if(!params[p]) params[p] = {};
			params[p].value = argParams[p].value;
		}
	},
	
	setSystemParamValues: function(params) {

		for(var name in params) {
			
			var param = params[name];
			var type = param.type;
			if (!type) continue;
			if (type == "date" && param.value && dojo.isString(param.value)) { //$NON-NLS-1$
				var value = dojo.date.stamp.fromISOString(param.value);
				if (value && !isNaN(value.getTime())) param.value = value;
			} else if(type.indexOf("system:") == 0) { //$NON-NLS-1$
				param.value = paramUtil.getSystemValue(type.substr(7));
			}
			
		}

	},
	
	getAllQueryTemplates: function(){
		return this.queryTemplates;
	},
	
	getQueryTemplate: function(queryIdArg){
		var queryId = queryIdArg || ""; //$NON-NLS-1$
		var qt = this.queryTemplates;
		for(var i=0; i<qt.length; i++){
			var q = qt[i];
			if(q.id == queryId){
				return q;
			}
		}
		var q = this._getResourceQuery(queryId);
		if (q) return q;
		q = this._getLinkQuery(queryId);
		if (q) return q;
		return null;
	},
	
	getAllResourceInfo: function(){
		return this.resourceInfo;
	},
	
	getResourceInfoById: function(id){
		if(!id) return null;
		var resInfo = this.resourceInfo;
		for(var i=0; i<resInfo.length; i++){
			var info = resInfo[i];
			if(info.id == id){
				return info;
			}
		}
		return null;
	},
	
	getResourceInfo: function(resource){
		var resInfo = this.resourceInfo;
		for(var i=0; i<resInfo.length; i++){
			var info = resInfo[i];
			var targetProps = info.targetProps;
			for(var j=0; j<targetProps.length; j++){
				var target = targetProps[j];
				var found = true;
				for(var k=0; k<target.length; k++){
					var targetProp = target[k];
					var targetPropName = resource[targetProp.name];				
					if(!targetPropName || targetPropName.value != targetProp.value){
						found = false;
					}					
				}
				if(found) return info;
			}
		}
		return this.getDefaultResourceInfo();
	},
	
	getDefaultResourceInfo: function(){
		var defaultResourceInfo = null; 
		dojo.forEach(this.resourceInfo, function(r){
			if(!defaultResourceInfo && r.targetProps.length == 0){
				defaultResourceInfo = r;
			}
		});
		this._defaultResourceInfo = defaultResourceInfo;
		return this._defaultResourceInfo;
	},
	
	getStatusLinks: function(resourceInfo){
		return resourceInfo && resourceInfo["statusLinks"];//$NON-NLS-1$
	},
	
	getActions: function(resourceInfo){
		return resourceInfo && resourceInfo["actions"];//$NON-NLS-1$
	},
	
	getHoverProps: function(resourceInfo){
		return resourceInfo && resourceInfo["hoverProps"];//$NON-NLS-1$
	},
	
	getResourceInfoHasComponent: function(resource){
		return this._getResourceInfoHasProp(resource, "component") || {}; //$NON-NLS-1$
	},
	
	getResourceStatusLinks: function(resource){
		return this._getResourceInfoProp(resource, "statusLinks") || []; //$NON-NLS-1$
	},
	
	getResourceActions: function(resource){
		return this._getResourceInfoProp(resource, "actions") || []; //$NON-NLS-1$
	},
	
	getResourceHoverProps: function(resource){
		return this._getResourceInfoProp(resource, "hoverProps") || [];		 //$NON-NLS-1$
	},
	
	getLinks: function(linkId){
		if(!linkId) return null;
		if(!dojo.isArray(linkId)){
			linkId = [linkId];
		}
		return dojo.filter(this.connectionLinks, function(c){
			return (c.id && c.id == linkId);
		});
	},
	
	_getResourceInfoHasProp: function(resource, propName){
		var prop = null;
		var resourceInfo = this.getResourceInfo(resource);
		if(resourceInfo){
			prop = resourceInfo[propName];
		}
		if(!prop){
			resourceInfo = this.getDefaultResourceInfo();
		}		
		return resourceInfo;
	},
	
	_getResourceInfoProp: function(resource, propName){
		var resourceInfo = this._getResourceInfoHasProp(resource, propName);
		if(resourceInfo){
			return resourceInfo[propName];
		}
		return null;
	},
	
	// Just for old configurations ... to be removed
	_getResourceQuery: function(queryId){
		var resourceQueries = this.resourceQueries;
		for(var i=0; i<resourceQueries.length; i++){
			var q = resourceQueries[i];
			if(q.id == queryId){
				console.warn("DEPRECATED: " + QUERY_XPATH + " in configuration xml is obsoleted. Use " + QUERY_TEMPLATE_XPATH + " instead."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				return q;
			}
		}
		return null;
	},
	
	_getLinkQuery: function(queryId){
		var linkQueries = this.linkQueries;
		for(var i=0; i<linkQueries.length; i++){
			var q = linkQueries[i];
			if(q.id == queryId){
				console.warn("DEPRECATED: " + LINK_QUERY_XPATH + " in configuration xml is obsoleted. Use " + QUERY_TEMPLATE_XPATH + " instead."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				return q;
			}
		}
		return null; 
	},
	
	/*
	 * If queryTemplate dones't have id, add "defaultQuery" as id, and fix containers which don't have baseQueryId
	 */
	_migrate10: function(){
		dojo.forEach(this.queryTemplates, function(q){
			if(!q.id){
				q.id = "defaultQuery";//$NON-NLS-1$ 
				this._defaultQueryId = q.id;
			}
		}, this);
	}
}
);

})();