/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.loader.ConfigXMLLoader"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.xmlutils"); //$NON-NLS-1$


(function(){
	
var xmlutils = com.ibm.relm.explorer.web.ui.internal.xmlutils;
var PARAMS_XPATH = "view > param"; //$NON-NLS-1$
var QUERY_TEMPLATE_XPATH = "queryTemplate"; //$NON-NLS-1$
var QUERY_XPATH = "nodeQueryBase"; //$NON-NLS-1$
var LINK_QUERY_XPATH = "linkQueryBase"; //$NON-NLS-1$
var COMPONENT_XPATH = "components"; //$NON-NLS-1$
var RESOURCE_XPATH = "resource"; //$NON-NLS-1$
//var STATUS_LINKS_XPATH = "resource status link"; //$NON-NLS-1$
var CONNECTION_LINKS_XPATH = "connection link"; //$NON-NLS-1$
//var HOVER_LINKS_XPATH = "hover links"; //$NON-NLS-1$
var LAYOUT_XPATH = "layout"; //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.loader.ConfigXMLLoader", //$NON-NLS-1$
	null, 
{
	loadXml: function(configXml, configJson){
		// 
		configJson.diagramId = xmlutils.getContent("id", configXml); //$NON-NLS-1$
		configJson.diagramName = xmlutils.getContent("name", configXml); //$NON-NLS-1$
		configJson.description = xmlutils.getContent("description", configXml); //$NON-NLS-1$
		// params
		configJson.params = this._getParams(PARAMS_XPATH, configXml);
		// query templates
		configJson.queryTemplates = this._getQueryTemplates(QUERY_TEMPLATE_XPATH, configXml);
		// base SPARQL
		configJson.resourceQueries = this._getBaseQuery(QUERY_XPATH, configXml);
		// link SPARQL
		configJson.linkQueries = this._getBaseQuery(LINK_QUERY_XPATH, configXml);
		// component info
		configJson.componentInfo = this._getComponentInfo(COMPONENT_XPATH, configXml);
		// resource info
		configJson.resourceInfo = this._getResourceInfo(RESOURCE_XPATH, configXml, configJson);
		// connection
		configJson.connectionLinks = this._getLinks(CONNECTION_LINKS_XPATH, configXml);
		// layout
		configJson.layout = this._getLayout(LAYOUT_XPATH, configXml);
	},
	
	_getParams: function(xpath, configXml){
		return this._getParamsFor(xmlutils.findNode("view", configXml)); //$NON-NLS-1$
	},
	
	_getParamsFor: function(parent){
		var params = {};
		dojo.forEach(xmlutils.getChildNodesByName("param", parent), function(param){ //$NON-NLS-1$
			params[param.getAttribute("name")] = param.getAttribute("value");  //$NON-NLS-1$ //$NON-NLS-2$
		});
		return params;
	},
	
	_getLayout: function(xpath, configXml) {
		return xmlutils.findNode(xpath, configXml);
	}, 
	
	/*
	 * [{
	 * 	id,
	 * 	sparql
	 * }, ...]
	 */
	_getQueryTemplates: function(xpath, configXml){
		return dojo.map(xmlutils.getSubNodes(xpath, configXml), function(query){
			var sparql = xmlutils.findNode("query", query); //$NON-NLS-1$
			return {
				id: query.getAttribute("id"), //$NON-NLS-1$
				params: this._getParamsFor(query),
				sparql: sparql && (sparql.textContent || sparql.text) 
			};
		}, this);				
	},
	/*
	 * [{
	 * 	id,
	 * 	sparql
	 * }, ...]
	 */
	_getBaseQuery: function(xpath, configXml){
		return dojo.map(xmlutils.getSubNodes(xpath, configXml), function(query){
			return {
				id: query.getAttribute("id"), //$NON-NLS-1$
				sparql: query.textContent || query.text 
			};
		});		
	},
	
	_getComponentInfo: function(xpath, configXml){
		var compInfo = {};
		var json = xmlutils.getContent(xpath, configXml);
		compInfo.items = dojo.fromJson(json);
		return compInfo;
	},
	
	/*
	 * [{
	 * 	targetProps: [{
	 * 		name: name1
	 * 		value: val1
	 * 	}, ...],
	 * 	component: {
	 * 		type,
	 * 		params: {
	 * 			name1: val1,
	 * 			...
	 * 		}
	 * 	},
	 * 	hoverProps [{
	 * 		name, ... property name in query result
	 * 		label, 
	 * 		layout, ... "vertical" or "horizontal"
	 * 		baseQueryId
	 * 	}, ...],
	 * 	statusLinks [{
	 * 		type ... link type
	 * 	}, ...],
	 * 	actions [{
	 * 		label,
	 * 		action
	 * 	}, ...]
	 * }, ...]
	 */
	_getResourceInfo: function(xpath, configXml, configJson){
		var resourceInfo = dojo.map(xmlutils.getSubNodes(xpath, configXml), function(resource){
			var targetProps = dojo.map(xmlutils.getSubNodes("target", resource), function(target){ //$NON-NLS-1$
				return dojo.map(xmlutils.getSubNodes("prop", target), function(prop){ //$NON-NLS-1$
					return {
						name: prop.getAttribute("name"), //$NON-NLS-1$
						value: prop.getAttribute("value") //$NON-NLS-1$
					};
				});
			});
			var hoverProps = dojo.map(xmlutils.getSubNodes("hover prop", resource), function(prop){ //$NON-NLS-1$
				return {
						label: prop.getAttribute("label"), //$NON-NLS-1$
						layout: prop.getAttribute("layout"), //$NON-NLS-1$
						name: prop.getAttribute("name"), //$NON-NLS-1$
						baseQueryId: prop.getAttribute("baseQueryId") //$NON-NLS-1$
				};
			});
			var component = xmlutils.findNode("component", resource); //$NON-NLS-1$
			var statusLinks = dojo.map(xmlutils.getSubNodes("status link", resource), function(l){ //$NON-NLS-1$
				return {
					type: l.getAttribute("type") //$NON-NLS-1$
				};
			});
			var actions = dojo.map(xmlutils.getSubNodes("actions item", resource), function(item){ //$NON-NLS-1$
				return {
					label: item.getAttribute("label"), //$NON-NLS-1$
					href: item.getAttribute("href"), //$NON-NLS-1$
					action: item.getAttribute("action") //$NON-NLS-1$
				};
			});

			var resourceInfo = {
				targetProps: targetProps,
				component: component && {
					type: component.getAttribute("type"), //$NON-NLS-1$
					params: this._getParamsFor(component)
				},
				hoverProps: hoverProps,
				statusLinks: statusLinks,
				actions: actions
			};
			if(!configJson._defaultResourceInfo && targetProps.length == 0){
				configJson._defaultResourceInfo = resourceInfo;
			}
			return resourceInfo;
		}, this);
		
		return resourceInfo;
	},
	
	/*
	 * [{
	 * 	title,
	 * 	type,
	 * 	baseQueryId,
	 * 	targetProps: {
	 * 		name,
	 * 		value
	 * 	},
	 * 	params: {
	 * 		name,
	 * 		value
	 * 	},
	 * 	component: {
	 * 		type,
	 * 		params
	 * 	}
	 * }, ...]
	 */
	_getLinks: function(xpath, configXml){
		return dojo.map(xmlutils.getSubNodes(xpath, configXml), function(l){
			// parameters for query
			var params = this._getParamsFor(l);
			// target to filter 
			var targetProps = dojo.map(xmlutils.getSubNodes("target", resource), function(target){ //$NON-NLS-1$
				return dojo.map(xmlutils.getSubNodes("prop", target), function(prop){ //$NON-NLS-1$
					return {
						name: prop.getAttribute("name"), //$NON-NLS-1$
						value: prop.getAttribute("value") //$NON-NLS-1$
					};
				});
			});
			// component for this link
			var component = xmlutils.findNode("component", l); //$NON-NLS-1$
			
			// link object
			return {
				title: l.getAttribute("title"), //$NON-NLS-1$
				type: l.getAttribute("type"), //$NON-NLS-1$
				baseQueryId: l.getAttribute("baseQueryId"), //$NON-NLS-1$
				params: params,
				targetProps: targetProps,
				component: component && {
					type: component.getAttribute("type"), //$NON-NLS-1$
					params: this._getParamsFor(component)
				}
			};
		}, this);
	}
}
);

})();