/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.Image"); //$NON-NLS-1$
// summary:
//		A gfx link node widget

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes._ClickableMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.Image"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ICONCLASS_COPY = "copy-icon"; //$NON-NLS-1$
var ICONCLASS_DELETE = "delete-icon"; //$NON-NLS-1$
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes.Image", //$NON-NLS-1$
	[com.ibm.relm.explorer.web.base.internal.gdc.Image, com.ibm.relm.explorer.web.ui.internal.view.nodes._ClickableMixin],
{
	postCreate: function(){
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
		this.setLeft(this.left || this.x || 0);
		this.setTop(this.top || this.y || 0);
		this.setWidth(this.width);
		this.setHeight(this.height);
	},

	createTargetShape: function(){
		var shape = this.inherited(arguments);
		this.addEventHandlerToShape(shape);
		var self = this;
		if(this._getEditMode()){
			this._createActionBar([
			                       {
			                    	   src: dojo.moduleUrl("relm.app", "graphics/icons/view_menu.gif"), //$NON-NLS-1$ //$NON-NLS-2$
			                    	   title: this._messages.openMenu,
			                    	   width: 16,
			                    	   onclick: function(iconShape){
			                    		   var pos = utils.shapePosition(iconShape, self._viewer);
			                    		   self.showContextMenu(pos.x, pos.y + pos.h);
			                    	   }
			                       }
			                       ]);
		}
		return shape;
	},

	getShapeInfo: function(){
		var src = utils.evalSimpleNodeVariables(this.src, this);
		return {width:this.width, height:this.height, src:src};
	},
	
	setSrc: function(src){
		this.src = src;
		if(this.doRefresh && this.shape){
			this.shape.setShape({src:this.src});
		}
	},
	
	getOutputTagName: function(){
		return this.componentInfoId;
	},
	
	editModeChanged: function(editMode){
		if(!this.src && this.shape){
			var src = dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/blank.gif"); //$NON-NLS-1$ //$NON-NLS-2$
			this.shape.setShape({src: editMode ? src : ""});//$NON-NLS-1$ 
		}
	},
	
	_getEditActions: function() {
		var self = this;
		var actions = [];
		actions.push({
			nodeWidget: self,
			actionType: 'copy', //$NON-NLS-1$
			label: self._messages.copyImage,
			iconClass: ICONCLASS_COPY,
			action: function(){
				var message = {
					type: "ActionInvokedNotification", //$NON-NLS-1$
					src: self,
					command: "copy", //$NON-NLS-1$
					arg: null
				};
				dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
			}
		});
		if(!self.notDeletable) {
			actions.push({
				nodeWidget: self,
				actionType: 'remove', //$NON-NLS-1$
				label: self._messages.removeImage,
				iconClass: ICONCLASS_DELETE,
				action: function(){
					self.removeImage();
				}
			});
		}

		return actions;
	},
	
	removeImage: function() {
		var self = this;
//		var containerId = self.getId();
		if(confirm(self._messages.confirmRemoveImage)){
			var message = {
				type: "ActionInvokedNotification", //$NON-NLS-1$
				src: self,
				command: "del", //$NON-NLS-1$
				arg: null
			};
			dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
		}
	},
	
	_getEditMode: function(){
		if(this.hlp_getEditor){
			return this.hlp_getEditor()._editMode;
		}
		return false;
	}

	

});
