/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.Text"); //$NON-NLS-1$
// summary:
//		A gfx link node widget

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes._ClickableMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.Text"); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ICONCLASS_COPY = "copy-icon"; //$NON-NLS-1$
var ICONCLASS_DELETE = "delete-icon"; //$NON-NLS-1$
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes.Text", //$NON-NLS-1$
	[com.ibm.relm.explorer.web.base.internal.gdc.Text, com.ibm.relm.explorer.web.ui.internal.view.nodes._ClickableMixin],
{
	getFont: function(){
		if (dojo.isString(this.font))
			return this.font;
		return dojo.toJson(this.font);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this.setLeft(this.left || this.x || 0);
		this.setTop(this.top || this.y || 0);
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceNodeMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},

	createTargetShape: function(){
		var shape = this.inherited(arguments);
		this.addEventHandlerToShape(shape);
		var self = this;
		if(this._getEditMode()){
			this._createActionBar([
			                       {
			                    	   src: dojo.moduleUrl("relm.app", "graphics/icons/view_menu.gif"), //$NON-NLS-1$ //$NON-NLS-2$
			                    	   title: this._messages.openMenu,
			                    	   width: 16,
			                    	   onclick: function(iconShape){
			                    		   var pos = utils.shapePosition(iconShape, self._viewer);
			                    		   self.showContextMenu(pos.x, pos.y + pos.h);
			                    	   }
			                       }
			                       ]);
		}
		return shape;
	},

	getShapeInfo: function(){
		var shapeInfo = this.inherited(arguments);
		shapeInfo.text = utils.evalSimpleNodeVariables(shapeInfo.text, this);
		dojo.mixin(shapeInfo, {width:this.width, height:this.height});
		return shapeInfo;
	},
	
	getOutputTagName: function(){
		return this.componentInfoId;
	},
	
	editModeChanged: function(editMode){
		if(!this.text && this.shape){
			this.shape.setFill("#aaaaaa"); //$NON-NLS-1$
			this.shape.setShape({text: editMode ? this._messages.emptyText: ""});//$NON-NLS-1$
		}
	},
	
	getBB: function(){
		if(!this.text && this._getEditMode()){
			arguments[0] = this._messages.emptyText;
		} else {
			arguments[0] = utils.evalSimpleNodeVariables(arguments[0], this);
		}
		return this.inherited(arguments);
	},
	
	_getEditActions: function() {
		var self = this;
		var actions = [];
		actions.push({
			nodeWidget: self,
			actionType: 'copy', //$NON-NLS-1$
			label: self._messages.copyText,
			iconClass: ICONCLASS_COPY,
			action: function(){
				var message = {
					type: "ActionInvokedNotification", //$NON-NLS-1$
					src: self,
					command: "copy", //$NON-NLS-1$
					arg: null
				};
				dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
			}
		});
		actions.push({
			nodeWidget: self,
			actionType: 'remove', //$NON-NLS-1$
			label: self._messages.removeText,
			iconClass: ICONCLASS_DELETE,
			action: function(){
				self.removeText();
			}
		});

		return actions;
	},
	
	removeText: function() {
		var self = this;
		if(confirm(dojo.string.substitute(self._messages.confirmRemoveText, [self.text]))){
			var message = {
				type: "ActionInvokedNotification", //$NON-NLS-1$
				src: self,
				command: "del", //$NON-NLS-1$
				arg: null
			};
			dojo.publish("btp:ActionInvoked", [message]); //$NON-NLS-1$
		}
	},
	
	_getEditMode: function(){
		if(this.hlp_getEditor){
			return this.hlp_getEditor()._editMode;
		}
		return false;
	}


});
