/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes._ActionsMixin"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper"); //$NON-NLS-1$

(function(){
	
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var ARG_VIEW_ID = "viewId";//$NON-NLS-1$
var ARG_PARAMS = "params";//$NON-NLS-1$
var ARG_TARGET = "target";//$NON-NLS-1$
var ARG_CONTAINER_IDS = "containerIds";//$NON-NLS-1$
var ARG_CONNECTION_IDS = "connectionIds";//$NON-NLS-1$
var ARG_CLEAR = "clear";//$NON-NLS-1$

var actionHelper = {
	// The list of *custom* user-defined actions. Elsewhere 'actions' also refers
	// to the custom actions, plus built actions. Should probably clarify these names.
	actions: ["openView", "reloadContainer", "reloadConnections"], //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	// predefined actions
	getActionInfo: function(commandName, argument){
		if(arguments.length == 1){
			if(commandName == "openView"){//$NON-NLS-1$
				return [ARG_VIEW_ID, ARG_PARAMS, ARG_TARGET];
			}else if(commandName == "reloadContainer"){//$NON-NLS-1$
				return [ARG_CONTAINER_IDS, ARG_PARAMS];
			}else if(commandName == "reloadConnections"){//$NON-NLS-1$
				return [ARG_CONNECTION_IDS, ARG_PARAMS, ARG_CLEAR];
			}
			return null;
		}
		if(argument === ARG_VIEW_ID){ 
			return {
				types: ["string"]  //$NON-NLS-1$
			};
		}else if(argument === ARG_PARAMS){ 
			return {
				types: ["object"]  //$NON-NLS-1$
			};				
		}else if(argument === ARG_TARGET){ 
			return {
				types: ["string"]  //$NON-NLS-1$
			};
		}else if(argument === ARG_CONTAINER_IDS){ 
			return {
				types: ["string","array"]  //$NON-NLS-1$ //$NON-NLS-2$
			};
		}else if(argument === ARG_CONNECTION_IDS){ 
			return {
				types: ["string","array"]  //$NON-NLS-1$ //$NON-NLS-2$
			};
		}else if(argument === ARG_CLEAR){ 
			return {
				types: ["boolean"]  //$NON-NLS-1$
			};				
		}
	},
	
	openView: function(viewId, params, target){
		var viewer = this._viewer;
		utils.openView(viewId, params, this.resource, target, viewer.webBaseUri);
	},
	
	reloadContainer: function(containerIds, params){
		// evaluate parameters with resource properties
		var evaluatedParams = utils.evalParameters(params, this.resource);
		// 
		if(!dojo.isArray(containerIds)) {
			containerIds = [containerIds];
		}
		var viewer = this._viewer;
		dojo.forEach(containerIds, function(containerId){
			var container = viewer.findContainer(containerId);
			container.reload(evaluatedParams);
		});
		viewer.fireLoadResource();
	},
	
	reloadConnections: function(connectionIds, params, clear) {
		// evaluate parameters with resource properties
		var evaluatedParams = utils.evalParameters(params, this.resource);
		
		var viewer = this._viewer;
		viewer.reloadConnections(connectionIds, evaluatedParams, clear);
		
	},

	// mix-in methods
	_createActionBar: function(actionInfo){
		var params = dojo.getObject("_viewer.config.params", false, this); //$NON-NLS-1$
		if(!params || !params["_showActionBar"]){//$NON-NLS-1$
			// in default, don't show action bar.
			return;
		}
		var self = this;
		this.connect(self, "focus", function(){ //$NON-NLS-1$
			if(!self._actionGroup){
				var group = self.group.createGroup();
				utils.createHoverActions(group, actionInfo, self);				

				self._actionGroup = group;
				self._updateAction();
			}
			self._actionGroup.rawNode.style.display = ""; //$NON-NLS-1$
			self._actionGroup.moveToFront();
		});
		this.connect(self, "blur", function(){ //$NON-NLS-1$
			if(self._actionGroup){
				self._actionGroup.rawNode.style.display = "none"; //$NON-NLS-1$
			}
		});
	},
	
	/**
	 * Gets all the available actions for this node.
	 * This includes custom user-defined actions for the resource
	 * and built in action.
	 * Subclasses will override--there are no actions by defaults.
	 * Actions are used to create the right-click context menu, and that
	 * shown in hover action bar. Actions may be used for conditional toolbar
	 * buttons at some point.
	 *
	 * Each action is an object like:
	 * {
	 *		nodeWidget: The node widget (this) creating the action.
	 *		iconClass: CSS class for an icon.
	 *		label: Text label for the action.
	 *		action: A standalone function to be invoked when the action fires.
	 * }
	 * 
	 * @return {array} list of all actions available on this node.
	 */
	_getAllActions: function() {
		return [];
	},

	_getEditActions: function() {
		return [];
	},

	_destroyActions: function(){
		if(this._actionGroup){
			this._actionGroup.removeShape();
			this._actionGroup = null;
		}
	},
	
	_doAction: function(actionString, event){
		var isAllowedAction = dojo.some(this.actions, function(action){
			return (actionString.indexOf(action) === 0);
		});
		if(isAllowedAction){ 
			// supported method
			var f = "this." + actionString; //$NON-NLS-1$
			eval(f);
			return true;
		} 
		return false; // error
	},

	_updateAction: function(){
		// re-position action icons
		if(this._actionGroup){
			// Assume the first child is the rectangle of this action box itself (see. createHoverAction()) 
			var bb = this._actionGroup.children[0].getBoundingBox(); 
			this._actionGroup.setTransform({ dx:this.getWidth()-bb.width, dy:-bb.height-2}); // spacing = 2
		}
	}	
};
dojo.mixin(actionHelper, com.ibm.relm.explorer.web.ui.internal.view.nodes.contextMenuHelper);

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes._ActionsMixin", //$NON-NLS-1$
	null,
	actionHelper);

})();