/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes._ComponentNode"); //$NON-NLS-1$

(function(){
	
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.AnchorManager"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.FixedAnchor"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.anchor.CenterEdgeAnchor"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.ConfigUtils"); //$NON-NLS-1$

var configUtils = com.ibm.relm.explorer.web.ui.internal.ConfigUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes._ComponentNode", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode,
{
	sourceAnchorClass: "com.ibm.relm.explorer.web.base.internal.gdc.anchor.CenterEdgeAnchor", //$NON-NLS-1$
	targetAnchorClass: "com.ibm.relm.explorer.web.base.internal.gdc.anchor.CenterEdgeAnchor", //$NON-NLS-1$

	targetShapes: null,
	
	_outlineShape: null,
	
	_outlineShapeObj: null,
	
	_contentsCreated: false,
	
	_overflow: "clip", //$NON-NLS-1$

	createTargetShape: function(){
		if(!this.targetShapes){
			console.error("No 'targetShapes' has been specified"); //$NON-NLS-1$
			return;
		}
		
		// if this flag is on, node contents are created lately
		var delayedUpdate = false;
		if (this._viewer && this._viewer.delayedUpdate != undefined) {
			delayedUpdate = this._viewer.delayedUpdate;
		}
		
		var shapeObj = this._getOutlineShapeObject(this.targetShapes); 
		if (!shapeObj) {
			return;
		}
		var overflow = this._getContentsOverflowMode(shapeObj);
		var shape = this._createOutlineShape(shapeObj);
		this._outlineShapeObj = shapeObj;
		this._outlineShape = shape;
		this._overflow = overflow;
		if (!delayedUpdate || overflow == "expand") { //$NON-NLS-1$
			this.updateNodeContents(true);	
		}
		return shape;
	},

	getShapeInfo: function(){
		var info = {};
		dojo.forEach(this._targetShapeKeys, function(key){
			info[key] = this[key];
		}, this);
		return info;
	},
	
	getClipInfo: function(shapeInfo, targetShape) {
		var clipType = shapeInfo["type"];//$NON-NLS-1$
		if(dojo.isIE <= 8 && clipType !== "rect"){//$NON-NLS-1$
			// VML supports only rectangle clip
			return null;
		}
		switch(clipType) { //$NON-NLS-1$
		case "rect": { //$NON-NLS-1$
			var ret = {x: shapeInfo["x"]||0, //$NON-NLS-1$
					y: shapeInfo["y"]||0, //$NON-NLS-1$
					r: shapeInfo["r"]||0, //$NON-NLS-1$
					width: (shapeInfo["width"]||1), //$NON-NLS-1$
					height: (shapeInfo["height"]||1)}; //$NON-NLS-1$
			if(dojo.isIE <= 8 && targetShape){
				var targetShapeInfo = targetShape.getShape();
				if(targetShapeInfo && targetShapeInfo.type !== "mltext"){//$NON-NLS-1$
					// origin of text is parent one. 
					// y origin also should same but it has some maybe related to baseline so don't set 
					ret.x -= (targetShapeInfo.x || 0);
				}
			}
			return ret;
		}
		case "circle": //$NON-NLS-1$
			return {cx: shapeInfo["cx"]||0, //$NON-NLS-1$
					cy: shapeInfo["cy"]||0, //$NON-NLS-1$
					rx: shapeInfo["r"]||1, //$NON-NLS-1$
					ry: shapeInfo["r"]||1}; //$NON-NLS-1$
		case "ellipse": //$NON-NLS-1$
			return {cx: shapeInfo["cx"]||0, //$NON-NLS-1$
					cy: shapeInfo["cy"]||0, //$NON-NLS-1$
					rx: shapeInfo["rx"]||1, //$NON-NLS-1$
					ry: shapeInfo["ry"]||1}; //$NON-NLS-1$
		case "polyline": //$NON-NLS-1$
			return {points: shapeInfo["points"]}; //$NON-NLS-1$
		case "path": //$NON-NLS-1$
			return {d: shapeInfo["d"]}; //$NON-NLS-1$
		}
		
		return null;
	},

	update: function(props){
		this.inherited(arguments);
	},

	// restore the removed shape
	restoreShape: function(){
		this.inherited(arguments);
	},

	getBB: function(textShape){
		var w, h;
		if(dojo.isIE){
			w = textShape.rawNode.offsetWidth;
			h = parseInt(textShape.getFont().size)*1.3;
		}else{
			var bb = textShape.rawNode.getBBox();
			w = bb.width;
			h = parseInt(textShape.getFont().size)*1.3;
		}
		return {width:Math.round(w), height:Math.round(h)};
	},

	setText: function(text){
		this.text = text;
//		this.textShape.setShape({text:text});
	},

	hlp_showAllowConnectFeedback: function(){
		this.hlp_showComponentFrame("feedback", "#ffffaa", 2); //$NON-NLS-1$ //$NON-NLS-2$
	},

	_createOutlineShape: function(shapeObj) {
		var shapeProp = this._parseProperties(shapeObj);
		var shape = this._createShape(shapeProp.prop);
		if(shapeProp.dojoAttachPointName){
			this[shapeProp.dojoAttachPointName] = shape;
		}
						
		// AbstractNode needs "width", "height", "fill", "stroke" properties, so use the values of the first shape
		this._targetShapeKeys = [];
		for(var key in shapeProp.prop){
			this[key] = shapeProp.prop[key];
			this._targetShapeKeys.push(key);
		}

		shapeObj.created = true;
		return shape;
	},
	
	/*
	 * Create node contents if needed. The contents created once when node is revealed on canvas for the first time.
	 * This method returns true if the contents are created, otherwise it returns false.
	 */
	updateNodeContents: function() {
		if (this._contentsCreated) {
			return true;
		}
		
		this._contentsCreated = true;

		// set clip rectangle
		var shapeInfo = null;
		if(this._overflow === "clip"){//$NON-NLS-1$
			shapeInfo = this.getShapeInfo();
			if(dojo.isIE <= 8){
				// IE8 doesn't allow clip for image so set clip to this.group
				var clipRect = this.getClipInfo(shapeInfo);
				if(clipRect){
					// for focus frame
					clipRect.x -= 4; clipRect.y -= 4;
					clipRect.width += 9; clipRect.height += 9;
					this.group.setClip(clipRect);
				}
			}
		}

		var result = false;
		// Add shapes repeatedly using 'shapeInfo'
		dojo.forEach(this.targetShapes, function(shapeObj){
			if (shapeObj === this._outlineShapeObj) {
				return;
			}
			result = true;
			
			var shapeProp = this._parseProperties(shapeObj);
			var shape = null;
			try{
				shape = this._createShape(shapeProp.prop);
			} catch(e) {
				return;
			}
			if(shapeProp.dojoAttachPointName){
				this[shapeProp.dojoAttachPointName] = shape;
			}
			
			switch (this._overflow) {
			case "clip": //$NON-NLS-1$
				if(shapeInfo) {
					if(dojo.isIE <= 8){
						var type = shape.shape && shape.shape.type;
						if(type !== "text" && type !== "mltext"){//$NON-NLS-1$ //$NON-NLS-2$
							// Clip by this.group except for text
							break;
						}
					}
					shape.setClip(this.getClipInfo(shapeInfo, shape));
				}
				break;
			case "expand": //$NON-NLS-1$
				var outlineShape = this._outlineShape;
				var bb = shape.getBoundingBox();
				if (bb) { // bb can be null when text of text shape is empty
					this.width = outlineShape.shape.width = Math.floor(Math.max(outlineShape.shape.width, (bb.x + bb.width))); 
					this.height = outlineShape.shape.height = Math.floor(Math.max(outlineShape.shape.height, (bb.y + bb.height)));
				}
				break;
			default:
				break;
			}
		}, this);
		
		return result;
	},
	
	_getContentsOverflowMode: function(shapeObj) {
		var overflow = "clip"; //$NON-NLS-1$
		if(shapeObj && (this.overflow == "clip" || this.overflow == "expand" || this.overflow == "overflow")) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			overflow = this.overflow;
		} 
		return overflow;
	},
	
	_getOutlineShapeObject: function(shapeObjs) {
		if(!shapeObjs|| shapeObjs.length == 0) {
			return null;
		}
		return shapeObjs[0]; 
	},
	
	_parseProperties: function(shapeObj) {
		var prop = {};
		var dojoAttachPointName = null;
		for(var key in shapeObj){
			if(key == "dojoAttachPoint"){ //$NON-NLS-1$
				// treat "dojoAttachPoint" property specially
				dojoAttachPointName = shapeObj[key];
			}else{
				var val = shapeObj[key];
				if(dojo.isString(val)){
					val = configUtils.evalVariables(val, this);
				}
				if(key == "x" || key == "y" || key == "left" || key == "top" || key == "width" || key == "height" || key == "r"){ //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
					prop[key] = Number(val);
				}else{
					prop[key] = val;						
				}
			}
		}
		return {dojoAttachPointName: dojoAttachPointName, prop: prop};
	},

	_createShape: function(prop) {
		/*
		 * Workaround for #92656
		 * String for MultilineText should be set after a parent shape is created. Otherwise, the following problem occurs.
		 *  - text does not appear in IE8
		 *  - y position after second lines are miscalculated. (font size is not considered.)
		 */
		var mltextprop = null;
		if (dojo.isIE <= 8) {
			if (prop.type == "textpath" || prop.type == "mltext" || prop.type == "text") { //$NON-NLS-1$ //$NON-NLS-3$ //$NON-NLS-2$
				if(!prop.font) {
					prop.font = {};
				}
				/*
				 * Workaround for #92816
				 */
				if(prop.rotated === undefined){
					mltextprop = dojo.clone(prop);
					mltextprop.rotated = "false"; //$NON-NLS-1$
				}
			}
			if (prop.type == "mltext") { //$NON-NLS-1$
				if(mltextprop === null) mltextprop = dojo.clone(prop);
				delete mltextprop.text;
			}
		}
		
		/*
		 * Workaround for improving performance of text rendering
		 * This prevents dojo diagrammer from calculating text bounds
		 */
		var useGfxLayout = dojo.config.useGfxLayout;
		dojo.config.useGfxLayout = false;
		var shape = this.group.createShape(mltextprop || prop);
		try {
			if (shape.suspendInvalidate) {
				shape.suspendInvalidate();
			}
			
			// needed since somehow 'fill' and 'stroke' is ignored when passing it as an initialization parameter
			if(prop.fill != null){
				shape.setFill(this._strToG(prop.fill));
			}
			if(prop.stroke != null){
				shape.setStroke(this._strToG(prop.stroke));
			}
			if (prop.type == "textpath") { //$NON-NLS-1$
				if(prop.font) {
					shape.setFont(prop.font);
				}
				if(prop.path){
					shape.setShape(prop.path);
				}
			} else if (prop.type == "mltext") { //$NON-NLS-1$
				if (prop.font) {
					shape.setFont(prop.font);
				}
				if (dojo.isIE <= 8 && prop) {
					shape.setShape(prop);
				}
				// Below is workaround until mltext support setStroke. The mltext has text shapes for lines as its children. 
				// The setFill() method calls the method of children but the setStroke() does not. 
				if(prop.stroke != null){
					var stroke = this._strToG(prop.stroke);
					dojo.forEach(shape.children, function(c) {
						c.setStroke(stroke);
					});
				}
			} else if(prop.type == "text" && prop.font){ //$NON-NLS-1$
				shape.setFont(prop.font);
			}
		} finally {
			dojo.config.useGfxLayout = useGfxLayout;
			if (shape.resumeInvalidate) {
				shape.resumeInvalidate(true);
			}
		}
		return shape;
	}
});

})();