/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes._ConnectionLineMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils"); //$NON-NLS-1$

(function(){

var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;
var linkPathUtils = com.ibm.relm.explorer.web.ui.internal.view.common.LinkPathUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes._ConnectionLineMixin", //$NON-NLS-1$
	null,
{
		sourceAnchorClass: "com.ibm.relm.explorer.web.base.internal.gdc.anchor.CenterEdgeAnchor", //$NON-NLS-1$
		targetAnchorClass: "com.ibm.relm.explorer.web.base.internal.gdc.anchor.CenterEdgeAnchor", //$NON-NLS-1$
		_staticConnections: null,
		
		setupContainerConnectionLine: function(viewer) {
			if (this.getHidden()) return;
		
			this.linkComponentInfo = null;
			var linkInfo = this.getLinkInfo(viewer, this);
			if (linkInfo.length > 0) {
				this.linkDef = {linkDefType: "container", links: dojo.map(linkInfo, function(link) { //$NON-NLS-1$
					var targetLink = null;
					var linkpath = link.linkpath;
					var target = linkpath[linkpath.length - 2];
					var upstream = this.getUpstream();
					for (var i = 0; i < upstream.length; i++) {
						var up = upstream[i];
						if (up.id == target.id && up.linktype == target.linktype) {
							targetLink = up;
							break;
						}
					}
					
					var title = jazz.util.html.escape(linkPathUtils.getLinkPathTitle(link));
					var linktype = jazz.util.html.escape(linkPathUtils.getLinkPathText(link.linkpath, "title")); //$NON-NLS-1$
					return {sourceWidgets: [link.source], title: title, path: linktype, connections: link.linkpath, extensions: {
						targetLink: targetLink,
						getShowLines: function() {
							return !this.targetLink.hidelines;
						},
						getLinkedNodeOnly: function() {
							return !this.targetLink.unlimited;
						},
						getConnectionId: function() {
							return this.targetLink.connectionId;
						}
					}};
				}, this)};
				
				this.linkComponentInfo = {type: "lineConnection",  //$NON-NLS-1$
					params: {stroke: {value: {color:"#d0d0d0", style:"ShortDash",width:3, cap:"butt"}}} //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				};
				this._createStaticConnections();
				this._startupConnections();
			} else {
				this._clearStaticConnections();
			}
		},
		
		_createStaticConnections: function(){
			var uri = this.uri;
			if(this.linkDef){
				this._clearStaticConnections();
				if (this.linkDef.linkDefType == "multiSource") { //$NON-NLS-1$
					// [upstreamId: upstreamId1, [targetId1: resources: {sourceWidgets: [source1, ...]}, ...], ...]
					dojo.forEach(this.linkDef.links, function(link) {
						var l = link.resources[uri];
						if (l) {
							var defaultLineType = "lineConnection"; //$NON-NLS-1$
							l.title = link.title;
							l.component = link.component || {type: defaultLineType};
							
							// set default to lineConnection
							if (!l.component.type || l.component.type == "") //$NON-NLS-1$
								l.component.type = defaultLineType;
							this._createConnections(l, true);
						}
					}, this);
				} else if (this.linkDef.linkDefType == "container") { //$NON-NLS-1$
					dojo.forEach(this.linkDef.links, function(link) {
						this._createConnections(link);
					}, this);
				} else {
					this._createConnections(this.linkDef);
				}
			}
		},

		_createConnections: function(linkDef, traceRelations) {
			var self = this;
			dojo.forEach(linkDef.sourceWidgets, function(sourceWidget){
				var componentInfo = linkDef.component || self.linkComponentInfo || {};
				var lineWidget = utils.createConnectionWidget(sourceWidget, self, componentInfo.type, componentInfo.params, null);
				if(lineWidget) {
					self._viewer.addChild(lineWidget);
					self._staticConnections.push(lineWidget);
					lineWidget._richHover = utils._createRichHoverForConnection(lineWidget, linkDef);
					dojo.mixin(lineWidget, {title: linkDef.title, links: linkDef.connections});
					if (linkDef.extensions) dojo.mixin(lineWidget, linkDef.extensions);
					lineWidget.traceRelations = traceRelations;
				}
			});
		},
		
		_startupConnections: function(){
			var viewer = this._viewer;
			dojo.forEach(this._staticConnections, function(l){
				l.startup();
				if (l.traceRelations && viewer) viewer.addStaticConnection(l);
			});
		},
		
		clearConnections: function(){
			this._clearStaticConnections();
			var viewer = this._viewer;
			if(viewer) {
				// delete connections whose source is this
				dojo.forEach(viewer.findConnections(this), function(c) { viewer.deleteConnection(c); });
				// delete connections whose source is this
				dojo.forEach(viewer.findConnections(null, this), function(c) { viewer.deleteConnection(c); });
			}
			
		},
		
		updateStaticConnections: function() {
			this.clearConnections();
			this._createStaticConnections();
			this._startupConnections();
		},
		
		_clearStaticConnections: function(){
			var viewer = this._viewer;
			dojo.forEach(this._staticConnections, function(l){
				if (l.traceRelations && viewer) viewer.deleteConnection(l);
				l.removeShape();
				l.destroyRecursive(false);
			});
			this._staticConnections = [];		
		},
		
		onConnectableEndpointsAdded: function() {
			var messages = this.messages;
			dojo.forEach(this.__connectableEndpoints, function(knob) {
				knob.title = messages.connectionHandleTitle;
			});
		},
		
		
		onConnectableEndpointsRemoved: function() {
		}
});

})();