/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.internal.views.ContextConfigVersionHeader'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.internal.views', 'ContextConfigVersionHeaderMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){

	var Site = com.ibm.relm.ple.web.ui.Site;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;

	/**
	 * Given an item path, walks up the path until it finds the first context config version.
	 * If the path is to a root config version, then there is no context config version.
	 * If the path is to a an item version or nested config version, it will be the closest
	 * parent config version.
	 * 
	 * @param  {array of objects} itemPath the item path, that is, a JSON item for
	 *     each node in the tree to the selected item.
	 * @return {object}      an item path to the closest context config version.
	 *     `null` if the path was to a root config version.
	 */
	function findContextConfigVersionItemPath(itemPath) {
		// Walk backwards up the path until we find a context config version.
		for (var i = itemPath.length - 2; i >= 0; i--) {
			var tail = itemPath[i];

			if (tail.type === 'configVersion') { //$NON-NLS-1$
				return itemPath.slice(0, i + 1);
			}
		}

		return null;
	}

	dojo.declare(
		'com.ibm.relm.ple.web.ui.internal.views.ContextConfigVersionHeader', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/internal/views/templates/ContextConfigVersionHeader.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.internal.views', 'ContextConfigVersionHeaderMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			site: null,

			postCreate: function() {
				this.inherited(arguments);

				var links = [];
				var currentPath = this.site.itemPath;

				while (true) {
					var configVersionPath = findContextConfigVersionItemPath(currentPath);
					if (!configVersionPath) break;

					var configVersion = configVersionPath[configVersionPath.length - 1];

					var configVersionHref = '#' + dojo.objectToQuery({ //$NON-NLS-1$
						action: 'ple.view_path', //$NON-NLS-1$
						path: Site.itemPathToPath(configVersionPath).join('/') //$NON-NLS-1$
					});

					var link = dojo.create('a', { //$NON-NLS-1$
						href: configVersionHref,
						title: this.messages.gotoContextConfigVersion,
						innerHTML: jazz.util.html.escape(configVersion.name)
					});

					links.push(link);

					currentPath = configVersionPath;
				}

				var frag = document.createDocumentFragment();
				
				// We walked from closest to farthest, so they are backwards. Reverse!
				links.reverse();
				for (var i = 0; i < links.length; i++) {
					var currentLink = links[i];
					frag.appendChild(currentLink);
					frag.appendChild(document.createTextNode(' > ')); //$NON-NLS-1$
				}

				this._bodyNode.appendChild(frag);
			}
		}
	);

})();