/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.internal.views.MultiSelectView'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.internal.views', 'MultiSelectViewMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var util = com.ibm.relm.common.web.ui.util;
	
	/**
	 * A details view that shows a list of all selected items.
	 * The view updates itself in response to changes to TreeSelection.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.internal.views.MultiSelectView', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/internal/views/templates/MultiSelectView.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.internal.views', 'MultiSelectViewMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			isMultiselect: true,

			constructor: function(args, node)
			{
				/**
				 * Tracks all the DOM nodes in the list by their URIs.
				 * Used to destroy nodes after de-select.
				 */
				this._pathsToNodes = {};
			},

			postCreate: function() {
				this.inherited(arguments);

				if (relm.app.Permissions.canProductsModify()) {
					dojo.addClass(this.domNode, 'relm-products-modify'); //$NON-NLS-1$
				} 

				if (relm.app.Permissions.canProductsAddProperty()) {
					dojo.addClass(this.domNode, 'relm-products-addProperty'); //$NON-NLS-1$
				}

				// IE workaround
				this.subscribe('ple.product_tree.drag_did_end', this._hideDragWrapper); //$NON-NLS-1$

				this.updateListFromTreeSelection(true);
				
				this.connect(TreeSelection, 'onSelectionChange', function() { //$NON-NLS-1$
					this.updateListFromTreeSelection(true);
				});
			},

			/**
			 * Looks as current TreeSelection and redraws the list.
			 * Existing items already in the list are left where they are (not resorted).
			 * New items always go at the end.
			 */
			updateListFromTreeSelection: function() {
				var selectedSites = TreeSelection.getAllSelectedSites();

				if (selectedSites.length === 1) {
					// don't animate last removable
					return;
				}
				
				// Set the page title with the count
				util.setText(this._titleNode, dojo.replace(this.messages.xSelectedItems, [selectedSites.length]));

				// Copy the current URI to node map. We'll strip out current ones, the remaining should be destroyed
				// Copy this so that we don't edit the real map while nodes are being added and removed.
				var remainingUrisToNodes = dojo.mixin({}, this._pathsToNodes);

				for (var i = 0; i < selectedSites.length; i++) {
					var site = selectedSites[i];
					// All children have a part that can be used as their 
					var path = site.path.join('/'); //$NON-NLS-1$

					// Don't animate the first item shown, just those added later.
					this.addToList(path, site, i !== 0);

					delete remainingUrisToNodes[path];
				}

				var didRemove = false;

				// Anything in the map not handled is no longer selected, so remove
				for (var oldPath in remainingUrisToNodes) {
					// Don't animate last remove since the view will change to show the single product anyway
					this.removeFromList(oldPath, selectedSites.length !== 1);
					didRemove = true;
				}

				var self = this;
				if (didRemove) {
					setTimeout(function() {
						self._updateTitleIcon();
					}, 251);
				}

				this._updateTitleIcon();
			},

			_onDragStart: function(e) {
				var productTree = com.ibm.relm.ple.web.ui.tree.TreeRefresh.getTree();
				if (!productTree) return;

				var draggedSites = com.ibm.relm.ple.web.ui.TreeSelection.getAllSelectedSites();
				if (!draggedSites.length) return;

				var self = this;
				function avatarFactory() {
					var avatarPos = dojo.position(self._iconNode);

					// Fudge the position a bit to accommodate padding. Not really needed.
					avatarPos.x -= 4;
					avatarPos.y -= 4;

					var avatar = new com.ibm.relm.ple.web.dnd.Avatar({
						startX: avatarPos.x,
						startY: avatarPos.y
					});

					// var iconNode = self._iconNode.cloneNode(true);
					// dojo.removeAttr(iconNode, 'title'); //$NON-NLS-1$
					var labelNode = self._titleNode.cloneNode(true);

					var multiIcon = self._generateMultiIcon();
					dojo.place(multiIcon, avatar.contentNode);
					dojo.place(labelNode, avatar.contentNode);

					return avatar;
				}

				productTree.startDrag(e, draggedSites, avatarFactory);
			},

			_updateTitleIcon: function() {
				var icon = this._generateMultiIcon();
				dojo.empty(this._iconNode);
				dojo.place(icon, this._iconNode);
			},

			_generateMultiIcon: function() {
				var imgNodes = dojo.query('img', this._listNode); //$NON-NLS-1$
				return com.ibm.relm.ple.web.ui.internal.views.MultiSelectView.generateMultiIcon(imgNodes);
			},

			_hideDragWrapper: function() {
				dojo.removeClass(this._dragWrapper, 'ple-ProductView-dragWrapperVisible'); //$NON-NLS-1$
			},

			_showDragWrapper: function() {
				dojo.addClass(this._dragWrapper, 'ple-ProductView-dragWrapperVisible'); //$NON-NLS-1$
			},

			/**
			 * Adds a new item to the list if it's not already there.
			 * @param {string} uri     URI of the item. Can be link or resource.
			 * @param {object} item    item for teh given URI. Passed separately just to avoid looking up the URI from the store again.
			 * @param {boolean} animate true to animate the add with a slide in effect.
			 */
			addToList: function(path, site, animate) {

				// already selected?
				if (this._pathsToNodes[path]) return;

				var listItem = dojo.create('li'); //$NON-NLS-1$

				// Wrap the LI in an extra div. This is to allow the LI height to be separately from the div height, which
				// should always appear full height. The LI is overflow: visible to show the product through it.
				var div = dojo.create('div', null, listItem); //$NON-NLS-1$

				var item = site.link || site.emptyPart || site.itemVersion || site.configVersion;

				if (item.liveIcon && item.liveTitle) {
					div.innerHTML = '<img class="ple-icon" src="' + item.liveIcon + '">' + item.liveTitle; //$NON-NLS-1$ //$NON-NLS-2$
				}
				else if (item.liveIcon) {
					// This is for DOORS, which returns an icon, but no sometimes.
					div.innerHTML = '<img class="ple-icon" src="' + item.liveIcon + '">' + item.name; //$NON-NLS-1$ //$NON-NLS-2$
				}
				else {
					div.innerHTML = formatting.formatProductIconAndTitleForItem(item);
				}
				
				this._pathsToNodes[path] = listItem;
				this._listNode.appendChild(listItem);
				
				if (animate) {
					dojo.animateProperty({
						node: listItem,
						duration: 200,
						properties: {
							left: {
								start: -100,
								end: 0
							},
							opacity: {
								start: 0,
								end: 1
							}
						}
					}).play();
				}
			},

			/**
			 * Remove an item from the list.
			 * @param  {string} uri     URI item to remove.
			 * @param  {boolean} animate true to animate the remove with a slide out effect.
			 */
			removeFromList: function(path, animate) {
				var listItem = this._pathsToNodes[path];
				if (!listItem) return;
				delete this._pathsToNodes[path];
				
				if (!animate) {
					dojo.destroy(listItem);
					return;
				}
				
				dojo.animateProperty({
					node: listItem,
					duration: 200,
					onEnd: function() {
						dojo.destroy(listItem);
					},
					properties: {
						left: -100,
						height: 0,
						opacity: 0
					}
				}).play();
			},
			
			uninitialize: function() {
				// Clear out reference to DOM node.
				delete this._pathsToNodes;
			}
		}
	);
	
	/**
	 * Creates an inline-block element that's a composite
	 * of several given icon image nods. All are assumed to be
	 * 16x16.
	 * @param  {array of IMG} imgNodes images to create composite from.
	 * @param  {number} maxIcons max icons to include in composite. Optional,
	 *     defaults to 3.
	 * @return {DOM node} composite icon. Not in the DOM.
	 */
	com.ibm.relm.ple.web.ui.internal.views.MultiSelectView.generateMultiIcon = function(imgNodes, maxIcons) {
		maxIcons = maxIcons || 3;
		imgNodes = imgNodes.slice(-maxIcons);

		// How far to shift each icon so that they still overlap.
		var yStep = 3;
		var xStep = 6;
		var opacityStep = 1.0 / imgNodes.length;

		// Size big enough to old all the icons.
		var h = 16 + (imgNodes.length - 1) * yStep;
		var w = 16 + (imgNodes.length - 1) * xStep;

		// Position for the current icon. Shift after each loop.
		var x = 0;
		var y = 0;
		var opacity = 1.0 - opacityStep * (imgNodes.length - 1);

		var compositeNode = dojo.create('div'); //$NON-NLS-1$

		dojo.style(compositeNode, {
			position: 'relative', //$NON-NLS-1$
			paddingRight: '4px', //$NON-NLS-1$
			verticalAlign: 'top', //$NON-NLS-1$
			display: 'inline-block', //$NON-NLS-1$
			width: w + 'px', //$NON-NLS-1$
			height: h + 'px' //$NON-NLS-1$
		});

		for (var i = 0; i < imgNodes.length; i++) {
			var newImg = imgNodes[i].cloneNode(true);
			dojo.style(newImg, {
				position: 'absolute', //$NON-NLS-1$
				top: y + 'px', //$NON-NLS-1$
				left: x + 'px', //$NON-NLS-1$
				opacity: opacity
			});

			opacity += opacityStep;
			x += xStep;
			y += yStep;

			dojo.place(newImg, compositeNode);
		}

		return compositeNode;
	};

})();	
