/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.setup.ui.internal.wizard.WizardApplication"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui._Application"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.ui.Wizard"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.relm.setup.ui.internal.wizard.WizardUtil"); //$NON-NLS-1$

/*######################################### */
/*These imports are required to invoke auth dialogs*/
dojo.require("jazz.app.proxy"); //$NON-NLS-1$
dojo.require("jazz.app.auth"); //$NON-NLS-1$
dojo.require("jazz.app.events"); //$NON-NLS-1$
/*######################################### */

dojo.requireLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WizardUtil = com.ibm.relm.setup.ui.internal.wizard.WizardUtil;
var Wizard = jazz.ui.Wizard;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;

dojo.declare("com.ibm.relm.setup.ui.internal.wizard.WizardApplication", net.jazz.ajax.ui._Application, { //$NON-NLS-1$
	innerBoxTrim:  TrimStyles.EDITOR_YELLOW,
	
	steps: null,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardApplicationMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.steps= [];
		var extensions= jazz.core.RegistryFactory.getRegistry().getExtensionPoint("com.ibm.relm.setup.setupWizard").getExtensions(); //$NON-NLS-1$
		
		if(!dojo.isArray(extensions)){	/*Safe check */
			/*Start the application, but it will be empty */
//			this._startApp();	
			return;
		}
		
		var required = [], optional = [], modulesToLoad = [];
		for(var index=0; index<extensions.length; index++){
			var extension= extensions[index];
			var _elements= extension.getConfigurationElements();
			for (var i = 0; i < _elements.length; i++) {
				if ("step" == _elements[i].getName()) { //$NON-NLS-1$
					var jsModule = _elements[i].getAttribute("jsModule"); //$NON-NLS-1$
					if (jsModule) {
						modulesToLoad.push(jsModule);
						/*We have to make sure that required steps are coming together at the beginning */
						if(_elements[i].getAttribute("optional") == "true"){ //$NON-NLS-1$ //$NON-NLS-2$
							optional.push(jsModule);
						}else{
							required.push(jsModule);
						}
					}
				}
			}
		}
		
		jazz.core.loader.batch_load_async(modulesToLoad, dojo.hitch(this, function(){

			var index, jsModule, classConstructor;
			for(index=0; index<required.length; index++){
				jsModule= required[index];
				classConstructor = dojo.getObject(jsModule);
				if (classConstructor) {
					this.steps.push(classConstructor);
				}
			}	
			for(index=0; index<optional.length; index++){
				jsModule= optional[index];
				classConstructor = dojo.getObject(jsModule);
				if (classConstructor) {
					this.steps.push(classConstructor);
				}
			}	
//			this._startApp();		
		}));	

	},

	postCreate: function() {
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		 //$NON-NLS-1$ //$NON-NLS-2$
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId); //$NON-NLS-1$
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") { //$NON-NLS-1$
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function() {
		this._startApp();
	},
	
	_startApp: function () {
		/*Why do we make this call here? We have to make sure that all wizard steps have been created, so
		 * other clients will add their extensions after making sure that steps have been constructed
		 * properly */
		jazz.core.loader.load_async("rmps.core.setup.SetupInitializer", dojo.hitch(this, function(){ //$NON-NLS-1$
			rmps.core.setup.SetupInitializer.getInstance().initialize((dojo.hitch(this, function(){
				/*Now, render the wizard steps */
				document.body.style.background = "none"; //$NON-NLS-1$
				this._createUI();
				this.setName(this.messages.setupName);
				this.start();				
			})));
		}));
	},
	
	_createUI: function() {				
		this.content = dojo.create("div");		 //$NON-NLS-1$
		if (!WizardUtil.isEmbedded()) {
			var textHeader = document.createElement("div"); //$NON-NLS-1$
			textHeader.innerHTML= this.messages.setupHeading;
			textHeader.className = "setup-header"; //$NON-NLS-1$
			this.content.appendChild(textHeader);
		}

		var wizard = new Wizard({
			fullPageContainer: this.content,
			steps: this.initServerSetupSteps(),
			stepListLayout: "vertical" //$NON-NLS-1$
		});
		
		net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode().appendChild(this.content);
		
		/*This is a safe check in order to avoid the wizard UI failure if error occured */
		if(wizard.contentWidget){
			var contentWidget= wizard.contentWidget;
			if(dojo.isFunction(contentWidget.updateNavigationControls)){
				contentWidget.updateNavigationControls();				
			}
		}
	},

	initServerSetupSteps: function () {
		var steps= [];
		for(var index=0; index<this.steps.length; index++){
			var widget = new this.steps[index]();
			dojo.addClass(widget.domNode, "com-ibm-relm-setup-ui-internal-wizard-WizardApplication"); //$NON-NLS-1$
			steps.push(widget);
		}
		return steps;
	}
});

})();