/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.setup.ui.internal.wizard.WizardFinalizeStep"); //$NON-NLS-1$

dojo.require("com.ibm.relm.setup.ui.internal.wizard.Util"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var relmMessages = dojo.i18n.getLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var Util = com.ibm.relm.setup.ui.internal.wizard.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;

/*
 * The RELM "Finalize Application" in the JTS setup wizard. Augments the base
 * FinalizeStep (a la com.ibm.xtools.rmps.setup.ui.internal.wizard.FinalizeStep)
 * with additional application-specific steps. This involves a bit of a hack--
 * hijacking showCompletedMessage in the base class to make it call the next
 * step instead.
 * 
 * To re-run Finalize Application repeatedly during development test:
 * 1. In isFinalizationNeeded, replace "if (response === "true") {"
 * with "if (true) {". This will enable the button to initiate the
 * operations.
 * 2. Add 'this.connect(this.startButton, "onclick", "userSync");'
 * to postCreate. The first step, Finish Registration (start), will run and
 * fail, but the next step, Perform User Synchronization, will be initiated
 * nevertheless.
 */
dojo.declare("com.ibm.relm.setup.ui.internal.wizard.WizardFinalizeStep", com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep, { //$NON-NLS-1$
	
	postCreate: function() {
		// Base class method sets up the first 2 steps:
		// 1. Finish Registration
		// 2. Perform User Synchronization
		this.inherited(arguments);
		
		this.nowSetCompletedState(false);
		
		// Create DOM for the individual finalize operations (the RELM additions)
		this.domApplyLicense = this._addOperationDiv(relmMessages.finalizeApplyLicense, relmMessages.finalizeApplyLicenseDescription);
		this.domImportProcessTemplate = this._addOperationDiv(relmMessages.finalizeImportProcessTemplate, relmMessages.finalizeImportProcessTemplateDescription);
		this.domStorageAreasCreation = this._addOperationDiv(relmMessages.finalizeStorageAreasCreation, relmMessages.finalizeStorageAreasCreationDescription);
		
		// save off the list of steps for checking off when set-up is re-run
		this.domSteps = [
			this.domFinalize,
			this.domUserSync,
			this.domApplyLicense,
			this.domImportProcessTemplate,
			this.domStorageAreasCreation
		];
	},
	
	/*
	 * Overridden.
	 * Similar to base class method, but:
	 * 1. checks for pop-up blocker
	 * 2. calls nowSetCompletedState instead of _setCompletedState
	 * 3. displays previously completed steps as checked when re-running setup
	 */
	isFinalizationNeeded: function() {
		var self = this;
		var isPopupBlockerEnabled = this.detectPopupBlocker();
		if (isPopupBlockerEnabled) {
		    // A popup blocker was detected, we cannot continue
			self.header.showMessage({type: STATUS_TYPES.ERROR, summary: relmMessages.loginPopupBlocked });
			self.nowSetCompletedState(false);
			return null;
		} else {
			var dFinalize = this.finalizationIsNeeded();
			dFinalize.addCallback(function(response){
				self.needsFinalized = response;
				if (response === "true") { //$NON-NLS-1$
					//Finalize needed
					var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered"); //$NON-NLS-1$ //$NON-NLS-2$
					ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
				} else {
					self.header.showMessage({type: STATUS_TYPES.OK, summary: self.messages.noFinalizeNeeded });
					self.nowSetCompletedState(true); // call replacement to _setCompletedState
					//dojo.addClass(self.progressSection, "noDisplay"); //$NON-NLS-1$
					self.displayStepsAsChecked();
				}
			});
			dFinalize.addErrback(function(errObject){
				//Allow the user to try to finalize since we don't know if they have already done so.
				self._handleError(self.messages.finalizeCheckError, errObject);
				self.startButton.disabled = false;
				self.needsFinalized = "true"; //$NON-NLS-1$
			});
			return dFinalize;
		}
	},
	
	/*
	 * Marks all the steps as done. This may not be entirely accurate,
	 * as some of the steps may have succeeded while others failed,
	 * and there's only a single flag indicating whether finalization
	 * as a whole was previously performed.
	 */
	displayStepsAsChecked: function() {
		for (var i = 0; i < this.domSteps.length; i++) {
			var domStep = this.domSteps[i];
			dojo.addClass(domStep, "ok"); //$NON-NLS-1$
		}
	},
	
	detectPopupBlocker: function() {
	    // return true if there is a popup blocker enabled for this server
		var myTestWindow = window.open("about:blank","","directories=no,height=100,width=100,menubar=no,resizable=no,scrollbars=no,status=no,titlebar=no,top=0,location=no"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		var isPopupBlockerEnabled = true;
		if (myTestWindow) {
			myTestWindow.close();
			isPopupBlockerEnabled = false;
		}
		return isPopupBlockerEnabled;
	},
	
	/*
	 * Overridden.
	 * Replaced by nowShowCompletedMessage.
	 * userSync (last step of base class) calls this when it is successful.
	 * Tap into it to invoke the first RELM step.
	 * Note: it may partially be successful but that's still ok...
	 */
	showCompletedMessage: function() {
		this.applyInternalLicense();
	},
	
	/*
	 * Overridden.
	 * Replaced by nowSetCompletedState.
	 * userSync calls this when it completes.
	 * We don't want to do anything here.
	 */
	_setCompletedState: function(state) {
	},
	
	/*
	 * Does what _setCompletedState did.
	 */
	nowSetCompletedState: function(state) {
		this.isCompleted = state;
		this.triggerButtonStateUpdate();
	},
	
	/*
	 * Does what showCompletedMessage did.
	 */
	nowShowCompletedMessage: function() {
		var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
		this.header.showMessage({
			type: STATUS_TYPES.OK,
			summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
		});
		this.nowSetCompletedState(true);
	},
	
	/*
	 * Overridden.
	 * Calls nowSetCompletedState rather than _setCompletedState;
	 * compensates for bug that was causing the browser
	 * to become unresponsive executing script when an AJAX error
	 * on one of the operations occurred.
	 */
	_showInlineError: function(node, message, errObject) {
		dojo.addClass(node, "error"); //$NON-NLS-1$
		var errorDiv = node.getElementsByTagName("div"); //$NON-NLS-1$
		dojo.removeClass(errorDiv[0], "noDisplay"); //$NON-NLS-1$

		/*
		var errorObj = this._getErrorObject(errObject);
		node._errorMsgBox.showMessage({
			//The line below hangs the browser on script execution, but I don't know why.
			//Skipping it out means no message details, but no hung browser either.
			errorObject: errorObj,
			summary: message
		});
		this._setCompletedState(false);
		*/
		
		node._errorMsgBox.showMessage({
			summary: message,
			details: [{message: errObject.responseText, code: STATUS_TYPES.ERROR}],
			errorObject: errObject
		});
		this.nowSetCompletedState(false);
	},
	
	/*
	 * Overridden.
	 */
	_handleError: function(message, response) {
		this.inherited(arguments);
		this.nowSetCompletedState(false);
	},
	
	/*
	 * Step 3: Apply Internal User License
	 */
	applyInternalLicense: function() {
		var self = this;
		var domNode = this.domApplyLicense;
		dojo.addClass(domNode, "loading"); //$NON-NLS-1$
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?action=applyLicense", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			load: function(requestId) {
				Util.getAndProcessRequestDetails(requestId, domNode);
				self.importProcessTemplate();
			},
			error: function(error, ioArgs) {
				var messageBoxArgs = Util.buildMessageBoxArgs(error, ioArgs, relmMessages.licenseError);
				Util.showInlineStatus(domNode, messageBoxArgs);
			}
		});	
	},

	/*
	 * Step 4: Import Process Templates
	 */
	importProcessTemplate: function() {
		var self = this;
		var domNode = this.domImportProcessTemplate;
		dojo.addClass(domNode, "loading"); //$NON-NLS-1$
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?action=importProcessTemplates", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			load: function(requestId) {
				Util.getAndProcessRequestDetails(requestId, domNode);
				self.storageAreasCreation();
			},
			error: function(error, ioArgs) {
				var messageBoxArgs = Util.buildMessageBoxArgs(error, ioArgs, relmMessages.importProcessTemplatesError);
				Util.showInlineStatus(domNode, messageBoxArgs);
			}
		});
	},
	
	/*
	 * Step 5: Create Storage Areas
	 */
	storageAreasCreation: function() {
		var self = this;
		var domNode = this.domStorageAreasCreation;
		dojo.addClass(domNode, "loading"); //$NON-NLS-1$
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/relminstall?recreate=false", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			load: function(response) {
				dojo.removeClass(domNode, "loading"); //$NON-NLS-1$
				dojo.addClass(domNode, "ok"); //$NON-NLS-1$
				self.nowSetCompletedState(true);
				self.nowShowCompletedMessage();
			},
			error: function(error, ioArgs) {
				var messageBoxArgs = Util.buildMessageBoxArgs(error, ioArgs, relmMessages.createStorageError);
				Util.showInlineStatus(domNode, messageBoxArgs);
			}
		});	
	}
});

})();