/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.setup.ui.internal.wizard.WizardSampleStep"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.setup.ui.internal.wizard.Util"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
var Util = com.ibm.relm.setup.ui.internal.wizard.Util;

/*
 * The RELM "Create Sample Project" in the JTS setup wizard.
 * Responsible for creating a user and sample project.
 */
dojo.declare("com.ibm.relm.setup.ui.internal.wizard.WizardSampleStep", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.relm.setup", "ui/internal/wizard/templates/WizardSampleStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	widgetsInTemplate: true,
	innerBoxTrim: jazz.ui.StyledBox.TrimStyles.EDITOR,
	
	constructor: function() {
		// Track all widgets that must be destroyed later.
		this._widgets = [];
		
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.isInProgress = false;
	},

	postCreate: function() {
		this.domCreateUser = this._addOperationDiv(this.messages.configureProjectUser, this.messages.configureProjectUserDescription);
		this.domCreateProject = this._addOperationDiv(this.messages.configureProjectProject, this.messages.configureProjectProjectDescription);
		
		this.domSteps = [ this.domCreateUser, this.domCreateProject ];
		
		this.connect(this.startButton, "onclick", "resetInitialStatus"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this.startButton, "onclick", dojo.hitch(this, this.setInProgress, true)); //$NON-NLS-1$
		this.connect(this.startButton, "onclick", "createUser"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	canFlipToPreviousPage: function() {
		// disable only if a sample creation is in progress
		return !this.isInProgress;
	},
	
	canFlipToNextPage: function() {
		// disable only if a sample creation is in progress
		return !this.isInProgress;
	},
	
	canFinish: function() {
		// disable only if a sample creation is in progress
		return !this.isInProgress;
	},

	/*
	 * Sample project UIs can use this method to control
	 * the state of the Next and Finish buttons in this
	 * WizardStep.
	 */
	setInProgress: function(isInProgress) {
		this.startButton.disabled = isInProgress;
		this.isInProgress = isInProgress;
		this.triggerButtonStateUpdate();
	},
	
	resetInitialStatus: function() {
		for (var i = 0; i < this.domSteps.length; i++) {
			var step = this.domSteps[i];
			dojo.removeClass(step, "loading ok error warning info"); //$NON-NLS-1$
			dojo.query("> div.operationError", step).addClass("noDisplay"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},
	
	/*
	 * Creates a new DIV element for the operation and appends it to the DOM.
	 * Returns: a handle to the DIV element.
	 */
	_addOperationDiv: function(title, description, customClass) {
		var div = dojo.create("div", { className: "operation"}, this.operations); //$NON-NLS-1$ //$NON-NLS-2$
		if(customClass) {
			dojo.addClass(div, customClass);
		}
		dojo.create("h3", { className: "operationTitle", innerHTML: title}, div); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.create("p", { className: "operationDesc", innerHTML: description}, div); //$NON-NLS-1$ //$NON-NLS-2$
		var errDiv = dojo.create("div", { className: "operationError noDisplay"}, div); //$NON-NLS-1$ //$NON-NLS-2$
		div._errorMsgBox = new jazz.ui.MessageBox({
			onToggle: dojo.hitch(this, this._handleMsgBoxToggle)
		});
		this._widgets.push(div._errorMsgBox);
		errDiv.appendChild(div._errorMsgBox.domNode);
		return div;
	},

	_handleMsgBoxToggle: function() {
		this.triggerButtonStateUpdate();
	},
	
	/*
	 * Operation 1
	 */
	createUser: function() {
		var self = this;
		var domNode = this.domCreateUser;
		dojo.addClass(domNode, "loading"); //$NON-NLS-1$
		
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?action=createSampleUser", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			load: function(requestId) {
				Util.getAndProcessRequestDetails(requestId, domNode);
				self.createProject(); // move on to the next step
			},
			error: function(error, ioArgs) {
				var messageBoxArgs = Util.buildMessageBoxArgs(error, ioArgs, self.messages.configureProjectUserError);
				Util.showInlineStatus(domNode, messageBoxArgs);
				self.setInProgress(false);
			}
		});	
	},
	
	/*
	 * Operation 2
	 */
	createProject: function() {
		var self = this;
		var domNode = this.domCreateProject;
		dojo.addClass(domNode, "loading"); //$NON-NLS-1$
		
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + "/configure?action=createSampleProject", //$NON-NLS-1$
			headers: {
				"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
			},
			handle: function() {
				self.setInProgress(false);
			},
			load: function(requestId) {
				Util.getAndProcessRequestDetails(requestId, domNode);
			},
			error: function(error, ioArgs) {
				var messageBoxArgs = Util.buildMessageBoxArgs(error, ioArgs, self.messages.configureProjectProjectError);
				Util.showInlineStatus(domNode, messageBoxArgs);
			}
		});	
	},
	
	uninitialize: function() {
		for (var i = 0; i < this._widgets.length; i++) {
			var w = this._widgets[i];
			w.destroyRecursive();
		}
	}
});

})();