/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.explorer.web.ui.page.DiagramComponetMenu"); //$NON-NLS-1$

dojo.require("jazz.app._ComponentMenu"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.page.DiagramPage"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ViewClient"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.editor.CapabilityComponentMenu"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.editor.PageHelper"); //$NON-NLS-1$
dojo.require("com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils"); //$NON-NLS-1$

var PAGE_ID = "com.ibm.relm.explorer.web.ui.page.DiagramPage"; //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ViewClient = com.ibm.relm.explorer.web.ui.internal.view.ViewClient;
var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
var leServiceUtils = com.ibm.relm.le.ui.internal.LifecycleElementServiceUtils;

//This contains hints for how the menu should do wrapping.
var wrapHint = jazz.ui.internal.MegaMenu.wrapHint;

//This code was modelled off the code I found in "ModelingComponentMenu.js" from the DM code.
dojo.declare("com.ibm.relm.explorer.web.ui.page.DiagramComponetMenu", com.ibm.relm.common.web.ui.editor.CapabilityComponentMenu, { //$NON-NLS-1$
	
	_basicActionPrefix: "#action=com.ibm.relm.explorer.web.", //$NON-NLS-1$
	_leActionPrefix: "#action=com.ibm.relm.le.leeditor.", //$NON-NLS-1$
	_highlightedNode: null,	
	
	constructor: function() 
	{
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "DiagramComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		var self = this;
		var args = arguments;
		this.permissions = {};		
		leServiceUtils.getPermissions().then(function(results){
			dojo.forEach(results, function(p){
				if(p == "modify") self.permissions.modifyPermission = true; //$NON-NLS-1$
			});
			self.inherited(args);
		});
	},

	addCapabilityComponentMenuItems: function() {
		
		this.createMenuItems();
	},

	getPageId: function(){
		return PAGE_ID;
	},
	
	getPageHelperClass: function() {
		return com.ibm.relm.common.web.ui.editor.PageHelper;
	},

	createMenuItems: function() {
		var self = this;
		//Take a look at MegaMenu.js to get some decent documentation for what is allowed here or at least some documentation
		//for addItemByArgs.
		var welcomePage = this.addItemByArgs({
			href: "#action=com.ibm.relm.explorer.web.showDiagramWelcome", //$NON-NLS-1$
			label: this.messages.welcomeToDiagrams,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/view.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		
		welcomePage.onClick = dojo.hitch(this, "_highlightNode", welcomePage); //$NON-NLS-1$
		
		this.addTitle(this.messages.queries);

		var personalViews = this.addItemByArgs({
			href: this._basicActionPrefix + "showViewList&scope=personal", //$NON-NLS-1$
			label: this.messages.personalViews,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/myviews.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		
		personalViews.onClick = dojo.hitch(this, "_highlightNode", personalViews); //$NON-NLS-1$
		
		var sharedViews = this.addItemByArgs({
			href: this._basicActionPrefix + "showViewList&scope=shared", //$NON-NLS-1$
			label: this.messages.sharedViews,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/shared_views.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		
		sharedViews.onClick = dojo.hitch(this, "_highlightNode", sharedViews); //$NON-NLS-1$

		if(this.permissions.modifyPermission){
			var leList = this.addItemByArgs({
				href: this._leActionPrefix + "showLEList", //$NON-NLS-1$
				label: this.messages.leList,
				icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/artifactelement.gif") //$NON-NLS-1$ //$NON-NLS-2$
			});
			leList.onClick = dojo.hitch(this, "_highlightNode", leList); //$NON-NLS-1$
		}
/*
		var myFavs = this.addItemByArgs({
			label: this.messages.myFavDiagrams,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/search-icon-enabld.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		
		myFavs.onClick = dojo.hitch(this, "_highlightNode", myFavs); //$NON-NLS-1$
		//remove this once an 'href' is in place above.
		dojo.connect(myFavs, "onClick", function() { alert("tbd");}); //$NON-NLS-1$ //$NON-NLS-2$
		
		var everyonesFavs = this.addItemByArgs({
			label: this.messages.everyonesFavDiagrams,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/search-icon-enabld.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		
		everyonesFavs.onClick = dojo.hitch(this, "_highlightNode", everyonesFavs); //$NON-NLS-1$
		//remove this once an 'href' is in place above.
		dojo.connect(everyonesFavs, "onClick", function() { alert("tbd");}); //$NON-NLS-1$ //$NON-NLS-2$
*/

		this.addTitle(this.messages.create);
		
		var createDiagram = this.addItemByArgs({
			label: (this.permissions.modifyPermission) ? this.messages.createDiagram : this.messages.createDiagramSingle,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/create_view.gif"), //$NON-NLS-1$ //$NON-NLS-2$
			onClick: function() {
						window.location.href = self._basicActionPrefix + "showViewList&scope=personal"; //$NON-NLS-1$
						self.createNewView();
					}
		});

		if(this.permissions.modifyPermission){
			this.addItemByArgs({
				href: this._leActionPrefix + "showLEEditor", //$NON-NLS-1$
				label: this.messages.createLE,
				icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/create_artifactelement.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: function(){
					window.location.href = self._leActionPrefix + "showLEEditor"; //$NON-NLS-1$
				}
			});
		}
	},
	
	
	//The following two functions were taken from rmps.app.ui.internal.pages.SideNavigation.
	_clearHighlight: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode, "selected"); //$NON-NLS-1$
		}
	},
	
	_highlightNode: function(item, event){
		var node = item.domNode;
		if (this._highlightedNode != node) {
			this._clearHighlight();
			dojo.addClass(node, "selected"); //$NON-NLS-1$
			this._highlightedNode = node;
		}
	},
	
	createNewView: function() {
		var self = this;
		var responseHandler = new ServiceResponseHandler(
				self,
				"createNewViewWizard", //$NON-NLS-1$
		"_remoteError"); //$NON-NLS-1$
		ViewClient.getSystemRootFolderDescriptors(responseHandler);
	},
	
	createNewViewWizard: function(response) {
		com.ibm.relm.explorer.web.ui.internal.wizard.CreateNewDiagramWizard.openWizard();		
	},
	
	_remoteError: function(error) {
		PlatformUI.getWorkbench().showMessage(error.message);
	}

});

})();