/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.page.IndexSearchProvider"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("jazz.ui._SearchProvider"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.lqeclient"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "IndexSearchProviderMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("jazz.app.ua.internal.client"); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.ui.internal.SearchBox_Fuzzy'); //$NON-NLS-1$

(function(){

var attempts_ = 0;

// Update the banner search box to support fuzzy search totals from the
// index where the total results isn't well known.
var decorateInterval_ = window.setInterval(function() {
	var bannerSearchBoxNode = dojo.query('.navbar .search-area .jazz-ui-SearchBox')[0]; //$NON-NLS-1$
	if (!bannerSearchBoxNode) {
		attempts_++;
		if (attempts_ >= 20) {
			console.warn('Cannot find banner search box so support unbounded paging of index search results!'); //$NON-NLS-1$
		}
	}
	else {
		console.log('Found banner search box. Decorated.'); //$NON-NLS-1$
		var searchBox = dijit.getEnclosingWidget(bannerSearchBoxNode);
		com.ibm.relm.explorer.web.ui.internal.SearchBox_Fuzzy.decorate(searchBox);
		clearInterval(decorateInterval_);

		// workaround for #95499 (Search box doesn't fit to page)
		// Overwrite style specified by jazz. This may be able to removed once jazz platform is updated
		var searchArea = dojo.query('.navbar .search-area')[0]; //$NON-NLS-1$
		if (searchArea) {
			searchArea.style.position = "relative"; //$NON-NLS-1$
			searchArea.style.marginRight = "4px"; //$NON-NLS-1$
		}
	}
}, 250);

var getIconForArtifactType = com.ibm.relm.explorer.web.ui.internal.util.getIconForArtiactType;
	
dojo.declare(
	"com.ibm.relm.explorer.web.ui.page.IndexSearchProvider", //$NON-NLS-1$
	jazz.ui._SearchProvider, 
{
	scopes: [],
	name: null,
	defaultText: null,
	_prevXhr: null,
	
	constructor: function() {
		// obtain a reference to the messages set so we can use them
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "IndexSearchProviderMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.scopes = [
					{ id: "all", label: this.messages.All, icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/all-media_16.gif")} //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
//					,{ id: "rm", label: this.messages.RequirementManagement, icon: getIconForArtifactType("http://open-services.net/ns/rm#Requirement")}
//					,{ id: "cm", label: this.messages.ChangeManagement, icon: getIconForArtifactType("http://open-services.net/ns/cm#ChangeRequest")}
//					,{ id: "qm", label: this.messages.QualityManagement, icon: getIconForArtifactType("http://open-services.net/ns/qm#TestCase")}
				];
		
		this.name = this.messages.Index;
		this.defaultText = this.messages.TypeToSearch;
		this.timestamp = Date.now();
	 },
	
	_fnGetTopicPrefix: null,
	
	postCreate: function() {
		// Hack to change the info center for CLM topics
		this._fnGetTopicPrefix = jazz.app.ua.internal.client.getTopicPrefix; 
		var self = this;
		jazz.app.ua.internal.client.getTopicPrefix = function (id) {
			if (id.indexOf("com.ibm.team") !== -1 || id.indexOf("com.ibm.jazz") !== -1) { //$NON-NLS-1$ //$NON-NLS-2$
				// Return the public CLM info center
				var def = new dojo.Deferred();
				def.callback("http://www.ibm.com/support/knowledgecenter/SSYMRC_5.0.1"); //$NON-NLS-1$
				return def;
			}
			return self._fnGetTopicPrefix(id);
		}; 
		
		this.inherited(arguments);
	},
			
	onExecute: function(args) {
		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'relm.queries.search', //$NON-NLS-1$
			q: args.query,
			scope: args.scope,
			time: Date.now()
		});
	},
	
	_previousQuery: null,

	onUpdate: function(args) {
		var query = args.query;
		if(this._previousQuery != query) {
			this._previousQuery = query;
		}
		var scope = args.scope;
		var pageSize = args.maxResults;
		var pageNumber = args.page;
		var callback = args.callback;
		var errback = args.errback;
		
		if (pageNumber == 1) {
			// Assume page 1 is a new search and refresh the results.
			// This isn't perfect, since going "back" to 1 will cause a new search,
			// but that's OK.
			this.timestamp = Date.now();
		}

		var self = this;

		if (this._prevXhr){
			this._prevXhr.cancel();
		}

		this._prevXhr = jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + '/ple/lqe_full_text_search', //$NON-NLS-1$
			handleAs: 'json', //$NON-NLS-1$
			content: {
				q: query,
				pageSize: pageSize,
				pageNumber: pageNumber,
				decorateProductTitles: true,
				timestamp: this.timestamp
			},
			load: function(data, ioArgs){
				var queryResults = self._formatResultsForSearchProvider(data);
				callback(queryResults);
			},
			error: function(data, ioArgs){
				var errorSummary = dojo.fromJson(ioArgs.xhr.responseText).summary;
				errback(errorSummary);
			},
			handle: function(){
				self._prevXhr = null;
			}
		});
		
	},
	
	_formatResultsForSearchProvider: function(searchResults) {
		var formattedResults = [];
		var hits = searchResults.hits;
		var getOnClick = function(uri) {
			return function(e) {
				dojo.stopEvent(e);
				dojo.publish('ple.quick_search.show_result', [uri]); //$NON-NLS-1$
			};
		};
	
		for (var i = 0; i < hits.length; i++) {
			var hit = hits[i];
			
			var presentationUri = null;
			var label = hit.text;

			// Note that this uses the GRAPH URI. This is what is needed for versioned
			// resources like Products. For all other resources, it should be the same
			// as the subject (artifact) URI.
			var result = {
				uri: hit.uri,
				label: label,
				icon: hit.iconUrl
			};

			if (hit.isForProductTree) {
				// It's much faster to just change the `action` since this
				// doesn't require a full page reload, so use a special presentation URI.
				result.presentationUri = '#' + dojo.objectToQuery({ //$NON-NLS-1$
					action: 'ple.read_only_product', //$NON-NLS-1$
					product: hit.uri
				});

				result.onclick = getOnClick(hit.uri);
			}
			
			formattedResults.push(result);
		}
	
		return {
			results: formattedResults,
			paging: true,
			totalResults: searchResults.totalCount,
			isFuzzy: true,
			isFuzzyTotal: searchResults.isFuzzyTotal
		};
	}
});

})();