/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.relm.explorer.web.ui.page.QueriesComponentMenu"); //$NON-NLS-1$

dojo.require("jazz.app._ComponentMenu"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "QueriesComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("com.ibm.relm.common.web.ui.editor.CapabilityComponentMenu"); //$NON-NLS-1$

var PAGE_ID = "com.ibm.relm.explorer.web.ui.page.QueryPage"; //$NON-NLS-1$

//This contains hints for how the menu should do wrapping.
var wrapHint = jazz.ui.internal.MegaMenu.wrapHint;

//This code was modelled off the code I found in "ModelingComponentMenu.js" from the DM code.
dojo.declare("com.ibm.relm.explorer.web.ui.page.QueriesComponentMenu", com.ibm.relm.common.web.ui.editor.CapabilityComponentMenu, { //$NON-NLS-1$
	
	_basicActionPrefix: "#action=com.ibm.relm.explorer.web.", //$NON-NLS-1$
	_highlightedNode: null,	
	
	constructor: function() 
	{
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "QueriesComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	addCapabilityComponentMenuItems: function() {

		this.inherited(arguments);
		
		this.createMenuItems();
	},

	getPageId: function(){
		return PAGE_ID;
	},

	getPageHelperClass: function() {
		return com.ibm.relm.common.web.ui.editor.PageHelper;
	},

	createMenuItems: function() {
		//Take a look at MegaMenu.js to get some decent documentation for what is allowed here or at least some documentation
		//for addItemByArgs.
		var self = this;
		var welcomePage = this.addItemByArgs({
			href: "#action=com.ibm.relm.explorer.web.showQueryWelcome", //$NON-NLS-1$
			label: this.messages.welcomeToQueries,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/query.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		welcomePage.onClick = dojo.hitch(this, "_highlightNode", welcomePage);		 //$NON-NLS-1$

		this.addTitle(this.messages.queries);

		var personalQueries = this.addItemByArgs({
			href: this._basicActionPrefix + "showQueryList&scope=personal", //$NON-NLS-1$
			label: this.messages.personalQueries,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/myqueries.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		personalQueries.onClick = dojo.hitch(this, "_highlightNode", personalQueries); //$NON-NLS-1$

		var sharedQueries = this.addItemByArgs({
			href: this._basicActionPrefix + "showQueryList&scope=shared", //$NON-NLS-1$
			label: this.messages.sharedQueries,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/shared_queries.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		sharedQueries.onClick = dojo.hitch(this, "_highlightNode", sharedQueries); //$NON-NLS-1$

		var aeQueries = this.addItemByArgs({
			href: this._basicActionPrefix + "showQueryList&scope=ae", //$NON-NLS-1$
			label: this.messages.aeQueries,
			icon: dojo.moduleUrl("com.ibm.relm.le", "ui/images/artifactelement.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		aeQueries.onClick = dojo.hitch(this, "_highlightNode", aeQueries); //$NON-NLS-1$

		this.addTitle(this.messages.create);
		
		var createQuery = this.addItemByArgs({
			href: this._basicActionPrefix + "showExplorer&mode=create", //$NON-NLS-1$
			label: this.messages.createQuery,
			icon: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/images/create_query.gif") //$NON-NLS-1$ //$NON-NLS-2$
		});
		createQuery.onClick = dojo.hitch(this, "_highlightNode", createQuery); //$NON-NLS-1$

		this.addTitle(this.messages.searchGroup);

		var search = this.addItemByArgs({
			href: '#action=relm.queries.search', //$NON-NLS-1$
			label: this.messages.keywordSearch,
			icon: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/search-icon.gif') //$NON-NLS-1$ //$NON-NLS-2$
		});

		
		//These should show up dynamically, just making them show up now though.
		//In addition will need to have them auto update and have some kind of publish/subscribe
		//or notification that the user viewed a new item.
		
		//We use a wrap hint here as we want the recent items to appear in their own column in the menu.
		//this.addTitle(this.messages.recentDiagrams, {wrapHint: wrapHint.FORCE_WRAP});
	},
	
	
	//The following two functions were taken from rmps.app.ui.internal.pages.SideNavigation.
	_clearHighlight: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode, "selected"); //$NON-NLS-1$
		}
	},
	
	_highlightNode: function(item, event){
		var node = item.domNode;
		if (this._highlightedNode != node) {
			this._clearHighlight();
			dojo.addClass(node, "selected"); //$NON-NLS-1$
			this._highlightedNode = node;
		}
	}
});

})();