/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.ConfirmDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "ConfirmDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){
	function text(str)
	{
		return document.createTextNode(str);
	}

	function element(tagName)
	{
		return document.createElement(tagName);
	}

	var contextRoot = net.jazz.ajax._contextRoot;
	if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs'; //$NON-NLS-1$ //$NON-NLS-2$

	dojo.declare(
		"com.ibm.relm.ple.web.ui.pages.ConfirmDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/ConfirmDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			dialogTitle: '', //$NON-NLS-1$
			details: '', //$NON-NLS-1$
			width: '400px', //$NON-NLS-1$
			iconPath: '', //$NON-NLS-1$
			iconClass: '', //$NON-NLS-1$

			prompt: '', //$NON-NLS-1$
			htmlPrompt: '', //$NON-NLS-1$

			constructor: function(args, node)
			{
				this.iconSrc = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/question.gif'); //$NON-NLS-1$ //$NON-NLS-2$
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.pages", "ConfirmDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			},

			cancel: function() {
				this._dialog.close();
				delete this._dialog;
				this.onCancel();
			},

			/**
			 * Callback when the dialog is closed from the cancel button.
			 * Normally passed in as a property function when constructing the dialog.
			 */
			onCancel: function() {
				
			},

			/**
			*	This function is available to be overridden by implementers or
			*	passed into the constructor.
			*/
			onClose: function() {
			},

			/**
			*	This requires the callback to return a deffered so that it knows when to close the dialog
			*	
			*	@param [function] callback The callback will be called in the context of the confirm dialog so that it has access to the messagebox
			*/
			addButton: function(title, callback, isDefaultButton) {
				var button = dojo.create('button'); //$NON-NLS-1$
				button.appendChild(document.createTextNode(title));

				var self = this;
				this.connect(button, 'onclick', function(e) { //$NON-NLS-1$
					
					var buttons = dojo.query('button', self.domNode); //$NON-NLS-1$

					// Preserve button states to restore them is the action fails.
					var buttonStates = buttons.map(function(button) {
						return {
							button: button,
							disabled: !!button.disabled
						};
					});

					buttons.forEach(function(button) {
						button.disabled = true;
					});

					var deferred = dojo.hitch(this, callback)(e);

					deferred
					.then(function() {
						self._dialog.close();
						delete self._dialog;
					})
					.otherwise(function(xhrOrErrorObject) {
						dojo.forEach(buttonStates, function(state) {
							state.button.disabled = state.disabled;
						});

						if (xhrOrErrorObject.summary) {
							self.messageBox.showErrorObject(xhrOrErrorObject, true);
						}
						else {
							self.messageBox.showXhrError(xhrOrErrorObject, true);
						}
					});
				});

				if (isDefaultButton) {
					this._defaultButton = button;
				}

				dojo.place(button, this._customButtonsNode, 'last'); //$NON-NLS-1$
				
				//Don't know if this is safe memory leak wise
				return button;
			},

			postCreate: function() {
				this.inherited(arguments);

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.dialogTitle,
					width: this.width,
					destroyContents: true,
					contentNode: this.domNode,
					iconPath: this.iconPath,
					iconClass: this.iconClass,
					onClose: this.onClose
				});
			},

			startup: function() {
				if (this._defaultButton) {
					var self = this;
					this.connect(window.document, 'onkeydown', function(e) { //$NON-NLS-1$
						if (e.keyCode == dojo.keys.ENTER) {
							self._defaultButton.click();
						}
					});
				}
			}
		}
	);
})();

