/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.CreateBaselineDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateBaselineDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

(function(){

var formatting = com.ibm.relm.ple.web.ui.formatting;

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.CreateBaselineDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ], 
	{
		messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'CreateBaselineDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/CreateBaselineDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,

		site: null,

		constructor: function(args, node) 
		{
			this._nameId = dijit.getUniqueId('ple-CreateBaselineDialog'); //$NON-NLS-1$
		},
		
		postCreate: function() {
			this.inherited(arguments);

			this._helpHeader.set('mainHtmlMessage', formatting.formatProductIconAndTitleForItem(this.site.configVersion)); //$NON-NLS-1$
			var hoverHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: 'com.ibm.relm.ple.web.text_CreateBaselineName001', //$NON-NLS-1$
				contentNode: this._nameNode
			});
			this.own(hoverHelp);

			this._dialog = new jazz.ui.Dialog({
				width: '450px', //$NON-NLS-1$
				primaryTitle: this.messages.dialogTitle,
				iconClass: 'ple-icon-createBaseline', //$NON-NLS-1$
				destroyContents: true,
				contentNode: this.domNode
			});

			this._dialog.startup();
		},
		
		startup: function() {
			this._nameField.focus();
		},

		_nameDidChange: function() {
			var name = dojo.trim(this._nameField.value);
			this._submitButton.disabled = name.length === 0;
		},
		
		_onSubmit: function(e) {
			dojo.stopEvent(e);

			var baselineName = dojo.trim(this._nameField.value);

			this._submitButton.disabled = this._cancelButton.disabled = this._nameField.disabled = true;
			var dialog = this._dialog;

			var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.creatingBaselineX, [baselineName]));

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/create_baseline', //$NON-NLS-1$
				content: {
					configVersionUri: this.site.configVersionUri,
					baselineName: baselineName	
				},

				handle: function() {
					message.hide();
				},

				load: function(baseline) {
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						selectPath: [baseline.pathSegment],
						flashPath: true
					}).addBoth(function() {
						dialog.close();
					});
				},

				error: dojo.hitch(this, function(error, ioArgs) {
					this._nameDidChange();
					this._cancelButton.disabled = this._nameField.disabled = false;
					this._messageBox.showXhrError(ioArgs.xhr);
				})
			});
		},

		cancel: function() {
			this._dialog.close();
		}
		
	});	// dojo.declare

})();

