/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.CreateResourceDialog"); //$NON-NLS-1$

dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.PageTemplate"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('jazz.app.ua.TextHoverHelp'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.BranchDialogMixin');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting');//$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage');//$NON-NLS-1$

//Consumers of this dialog must specify the name of the message file and specify it overhere
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateProductDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateConfigurationDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "CreateCopyDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "PatchBaselineDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
var formatting = com.ibm.relm.ple.web.ui.formatting;
var Site = com.ibm.relm.ple.web.ui.Site;

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.CreateResourceDialog", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated, com.ibm.relm.ple.web.ui.branch.BranchDialogMixin],
	{
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/CreateResourceDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,

		_parentIconClass: '', //$NON-NLS-1$
		name: '', //$NON-NLS-1$

		/**
		 * Uri is used by the BranchDialogMixin to lookup local dimensions on a product, but it is never used by this dialog
		 * @type {String}
		 */
		uri: null,

		/**
		 * Used by BranchDialogMixin. Some dialogs require dimensions, but this dialogs is used for creations and it  should work with
		 * empty dimensions as well
		 * @type {Boolean}
		 */
		ignoreEmptyDimensions: true,

		focusFirstDimension: false,

		constructor: function(args, node)
		{
			this._widgets = [];
			this.messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.pages", args.messageFileName); //$NON-NLS-1$ //$NON-NLS-2$

			if (args.isConfigVersion) {
				this.uaKey = 'com.ibm.relm.ple.web.text_CreateConfigName001'; //$NON-NLS-1$
				this.iconClass = 'ple-icon-createConfigVersion'; //$NON-NLS-1$
			} else {
				this.uaKey = 'com.ibm.relm.ple.web.text_CreateProductName001'; //$NON-NLS-1$
				this.iconClass = 'ple-icon-createProduct'; //$NON-NLS-1$
			}

			if (args.site) {
				this.selectedResource = args.site.configVersion || args.site.itemVersion;

				this._parentTitle = formatting.formatProductTitleForItem(this.selectedResource);
				this._parentIconClass = formatting.getIconClassForItem(this.selectedResource);
			}

		},

		postCreate: function() {
            this.inherited(arguments);
            
            if (this.hideParent) {
            	var helpHeader = new com.ibm.relm.common.web.ui.DialogHelpHeader();
            	helpHeader.set('mainHtmlMessage',  formatting.formatProductIconAndTitleForItem(this.site.configVersion)); //$NON-NLS-1$
            	helpHeader.set('content', this.messages.helpText); //$NON-NLS-1$
            	helpHeader.placeAt(this.domNode, 'first'); //$NON-NLS-1$
            	this._widgets.push(helpHeader);
            }
            else if (this.site) {
            	if (!this.isConfigVersion) {
            		this._removeButton.domNode.style.display = "none"; //$NON-NLS-1$
            	}
            	this._hasParentNode.style.display = ''; //$NON-NLS-1$
            	this._parentTitleNode.innerHTML = this._parentTitle;
            }
            else {
            	this._noParentNode.style.display = ''; //$NON-NLS-1$
            }

			this._nameHelp = new jazz.app.ua.TextHoverHelp({
				uaKey: this.uaKey,
				contentNode: this._nameSpan
			});

            this._dialog = new jazz.ui.Dialog({
            	primaryTitle: this.messages.dialogTitle,
            	width: '600px', //$NON-NLS-1$
            	destroyContents: true,
            	contentNode: this.domNode,
            	iconClass: this.iconClass
            });

            this._dialog.startup();
		},

		startup: function() {
			this._nameField.select();
			this._monitorCreateButton();
		},

		_nameDidChange: function() {
			// Paste event can fire BEFORE the paste is available in the text input.
			// Delay this one run loop to be sure the text is there.
			setTimeout(dojo.hitch(this, this._monitorCreateButton), 0);
		},

		_onSubmit: function(e) {
			dojo.stopEvent(e);
			var name = dojo.trim(this._nameField.value);
			this.onOk(name);
		},
		
		onOk: function(newName) {
			this.messageBox.hideMessage();

			var dimensions;
			try {
				dimensions = this._dimensionsToObject();
			} catch(e) {
				return;
			}

			this._removeButton.set('enabled', false); //$NON-NLS-1$
			this._cancelButton.disabled = true;
			this._nameField.disabled = true;
			this._submitButton.disabled = true;

			var message = com.ibm.relm.common.web.StatusMessage.show(dojo.replace(this.messages.creatingX, [newName]));
			
			var site = this.site;

			var content = {
				title: newName,
				site: Site.siteToJsonSite(site),
				dimensions: dojo.toJson(dimensions)
			};

			if (site) {
				content.configVersionUri = site.configVersionUri || site.contextConfigVersionUri;
				content.itemVersionUri = site.itemVersionUri;
			}

			this.setDisabledOnAllDimensionRows(true);

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + this.servicePath,
				content: content,
				handle: function() {
					message.hide();
				},

				load: dojo.hitch(this, function(newResource) {
					// Hack: The APIs for creating copies and patches of
					// config versions don't return new parts created for
					// them, so there is no way to know where in the tree to
					// select them at. As a result, they get put at the top of
					// the tree. This normally assumes if there is a selected
					// site that new resources should be selected under that
					// site. This check ensure that new, top-level config
					// versions are selected there, not under the site. This
					// requires this code to know what top-level path segments
					// look like, though this has changed more than once in
					// the past, so is likely to break one day. It's just easy
					// at the end of 5.0 though.
					var isTopLevelConfigVersion = newResource.pathSegment.indexOf('configurationVersion_') !== -1; //$NON-NLS-1$
					var newPath;
					
					// Patch baseline is special, since it can work on a child, but
					// should select the patch in the root of the tree afterwards.
					if (this.site && !isTopLevelConfigVersion) {
						newPath = this.site.path.concat([newResource.pathSegment]);	
					}
					else {
						newPath = [newResource.pathSegment];	
					}
					
					var close = dojo.hitch(this._dialog, this._dialog.close);
					
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						selectPath: newPath,
						flashPath: true
					}).addBoth(close);
				}),

				error: dojo.hitch(this, function(error, ioArgs) {
					try {
						
										this.setDisabledOnAllDimensionRows(false);
										this.messageBox.showXhrError(ioArgs.xhr);
					
										this._cancelButton.disabled = false;
										this._nameField.disabled = false;
										this._nameDidChange();
					}
					catch (e) {
						console.debug(e);
					}
				})
			});
		},

		cancel: function() {
			this._dialog.close();
		},

		_removeParent: function() {
			if (!this.isConfigVersion) return;

			delete this.site;

			this._hasParentNode.style.display = 'none'; //$NON-NLS-1$
			this._noParentNode.style.display = ''; //$NON-NLS-1$
			this._nameField.focus();
		},

		uninitialize: function() {
			this._nameHelp.destroyRecursive();
			this._widgets.forEach(function(w) { w.destroyRecursive(); });
		}

	});	// dojo.declare

})();
