/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.FindUsePage'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'FindUsePageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require('jazz.ui.FilterBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductPath'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Separator'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.HiddenResultsMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('jazz.ui.ResourceLink');//$NON-NLS-1$
dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.doors.DoorsUtils');//$NON-NLS-1$

(function(){

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var formatting = com.ibm.relm.ple.web.ui.formatting;

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'FindUsePageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

var gCachedItemVersionUri = null;
var gCachedPathsData = null;

dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.FindUsePage', //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/FindUsePage.html'), //$NON-NLS-1$ //$NON-NLS-2$
		pageTitle: messages.pageTitle,
		resultsTitle: '', //$NON-NLS-1$
		baseClass: 'ple-FindUsePage', //$NON-NLS-1$

		_setPageTitleAttr: {
			node: '_pageTitleNode', //$NON-NLS-1$
			type: "innerHTML" //$NON-NLS-1$
		},

		_setResultsTitleAttr: {
			node: '_resultTitleNode', //$NON-NLS-1$
			type: "innerHTML" //$NON-NLS-1$
		},

		constructor: function() {
			// This is already in scope, but the template only access it though this instance field
			this._messages = messages;
			this._pathWidgets = [];
		},

		postCreate: function() {
			this.inherited(arguments);

			this._loadingMessage.positionAbsolute();

			this.subscribe('ple.clean_cache', this._cleanCache); //$NON-NLS-1$

			this.connect(this._hiddenResultsMessage, 'shouldShowAll', this.showAll); //$NON-NLS-1$

			var uaKey = "com.ibm.relm.ple.web.page_FindUse001"; //$NON-NLS-1$
			this._hoverHelp = new jazz.app.ua.PageHoverHelp({uaKey: uaKey}, this._helpHolder);

			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction('ple.find_use', this, 'findPaths'); //$NON-NLS-1$ //$NON-NLS-2$

			this._toggleShowBaselinesButton.set('titlePressed', messages.showBaselinesTogglePressedTooltip); //$NON-NLS-1$
		},

		/**
		 * Shows all filtered results. Clears the filter box.
		 */
		showAll: function() {
			this._filterBox.clear();
		},

		_onToggleShowBaselines: function(e) {
			var showBaselines = this._toggleShowBaselinesButton.get('pressed'); //$NON-NLS-1$
			dojo.toggleClass(this._resultsContainer, 'ple-FindUse-hideBaselines', !showBaselines); //$NON-NLS-1$

			// Realign what's visible
			this._alignPathsIfNeeded();

			// Refilter to update count
			this.filterPathsAgain();
		},

		/**
		 * Re-aligns the path lengths for the currently visible paths if
		 * needed. If full paths are showing, then clears any explicitly head
		 * widths to restore them to their auto widths.
		 */
		_alignPathsIfNeeded: function() {
			var isFullPaths = this._toggleFullPathsButton.get('pressed'); //$NON-NLS-1$
			
			this._resetExplicitHeadWidths();

			if (isFullPaths) {
				return false;
			}
			else {
				this._alignPaths();
				return true;
			}
		},

		_onToggleFullPaths: function() {
			var isFullPaths = this._toggleFullPathsButton.get('pressed'); //$NON-NLS-1$

			dojo.toggleClass(this._resultsContainer, 'ple-FindUse-hideTitles', !isFullPaths); //$NON-NLS-1$

			var enableHoverPopups = !isFullPaths;
			this._pathWidgets.forEach(function(pathWidget) {
				pathWidget.hoverPopups.forEach(function(hoverPopup) {
					hoverPopup.set('enabled', enableHoverPopups); //$NON-NLS-1$
				});
			});

			if (isFullPaths) {
				this._resetExplicitHeadWidths();
			}
			else {
				this._alignPaths();
			}
		},

		filterPathsAgain: function() {
			if (this._lastFilterString) {
				this.filterPaths(this._lastFilterString);
			}
		},

		/**
		 * Filters the shown results based on a search string.
		 *
		 * @param searchString terms to search form. Results can match any of the whitespace
		 * seperated words given here.
		 */
		filterPaths: function(searchString) {
			this._lastFilterString = searchString;

			var terms = null;
			if (searchString) {
				searchString = dojo.trim(searchString).toLowerCase();
				terms = searchString.split(/\s+/);
			}

			var showBaselines = this._toggleShowBaselinesButton.get('pressed'); //$NON-NLS-1$

			var pathQuery = showBaselines
				? '.ple-ProductPath' //$NON-NLS-1$
				: '.ple-FindUse-pathRow:not(.ple-FindUse-isBaseline) .ple-ProductPath'; //$NON-NLS-1$

			var paths = dojo.query(pathQuery, this._resultsContainer); //$NON-NLS-1$
			var hiddenCount = 0;

			paths.forEach(function(path) {
				if (!searchString) {
					path.style.display = ''; //$NON-NLS-1$
					return;
				}

				var textContent = path.textContent.toLowerCase();

				var matches = dojo.every(terms, function(term) {
					return textContent.search(term) !== -1;
				});

				if (matches) {
					path.style.display = ''; //$NON-NLS-1$
				}
				else {
					path.style.display = 'none'; //$NON-NLS-1$
					hiddenCount += 1;
				}
			});

			this._hiddenResultsMessage.setHiddenCount(hiddenCount);
			this._alignPaths();
		},

		_destroyPaths: function() {
			dojo.forEach(this._pathWidgets, function(w) {
				w.destroyRecursive();
			});

			dojo.query('.ple-FindUse-pathRow', this._resultsContainer).forEach(dojo.destroy); //$NON-NLS-1$

			this._pathWidgets = [];
		},

		/**
		 * Draws all path widgets into the results container.
		 * @param  {object} pathsData object with full .paths and the .target.
		 */
		_drawPaths: function(pathsData, privateBaseline) {
			this._destroyPaths();

			var target = pathsData.target;

			if (!pathsData.paths || pathsData.paths.length === 0) {
				this._noResultsNode.style.display = ''; //$NON-NLS-1$
				return;
			}

			var showBaselines = this._toggleShowBaselinesButton.get('pressed'); //$NON-NLS-1$
			var baselineCount = 0;

			var pathFrag = document.createDocumentFragment();

			dojo.forEach(pathsData.paths, function(currentPath) {
				var pathRow = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-FindUse-pathRow' //$NON-NLS-1$
				}, pathFrag);

				var configVersion = currentPath[0];

				if (configVersion.isImmutable) {
					pathRow.className += ' ple-FindUse-isBaseline'; //$NON-NLS-1$
					baselineCount += 1;
				}

				var pathWidget = new com.ibm.relm.ple.web.ui.ProductPath({
					path: currentPath
					// tooltip: messages.viewResult
				});
				this._pathWidgets.push(pathWidget);

				pathWidget.placeAt(pathRow);
			}, this);

			// Special handling for Doors baslines since those are not shown as ResourceLinks
			if (privateBaseline) {
				var firstPath = pathsData.paths[0];
				var lastPathSegment = firstPath[firstPath.length -1];
				var resultTitle = dojo.query('a', this._resultTitleNode)[0]; //$NON-NLS-1$
				if (resultTitle && lastPathSegment.name) {
					resultTitle.innerHTML = lastPathSegment.name;
				}
			}

			dojo.place(pathFrag, this._resultsContainer);

			this._updateShowBaselinesToggle(baselineCount);

			this._alignPathsIfNeeded();
		},

		_updateShowBaselinesToggle: function(baselineCount) {
			var baselineToggleLabel = dojo.replace(messages.showBaselinesToggleCountX, [baselineCount]);
			this._toggleShowBaselinesButton.set('label', baselineToggleLabel); //$NON-NLS-1$
			// XXX: Setting the 'label' resets the display title to the non-pressed title (tooltip).
			// Setting again will figure out the pressed state and do the right thing.
			this._toggleShowBaselinesButton.set('titlePressed', messages.showBaselinesTogglePressedTooltip); //$NON-NLS-1$

			var hasBaselines = baselineCount > 0;

			// The needs to toggle on an off when new data is loaded.
			this._toggleShowBaselinesButton.set('enabled', hasBaselines); //$NON-NLS-1$
			
			// This sould preserve the state already set unless disabled, then the "pressed" state looks dumb.
			if (!hasBaselines) this._toggleShowBaselinesButton.set('pressed', false); //$NON-NLS-1$
		},

		/**
		 * When paths are aligned vertiacally, an explicit `width` style is
		 * set on the head (product configuration) of each path to get them to
		 * line up. This removes all the explicit widths. This is needed when
		 * showing full (not aligned) paths or and before re-aligning paths so
		 * that the auto widths can be used in the alignment calculations.
		 */
		_resetExplicitHeadWidths: function() {
			this._pathWidgets.forEach(function(path) {
				var cells = path.cells;
				var headCell = cells[0];
				headCell.style.width = ''; //$NON-NLS-1$
			});
		},

		/**
		 * Aligns the "tails" of all the path vertically. This improves the
		 * scannability of the paths, since it results in a single column of
		 * "parent products" on the right side. If the target is directly
		 * contained by the product configuration, there is no tail segment,
		 * but the head segment is still lengthened for visual consistency.
		 *
		 * To align the paths, the path with the tail farthest to the left is
		 * found. All other paths need to be lengthened enough so that their
		 * tails start in the same position. This is done by padding the
		 * length of the head (product configuration) segment. The middle
		 * sections are not aligned.
		 *
		 * This method is related to showing/hiding full paths. Since lining
		 * up the tails makes some paths longer, generally the title of the
		 * "middle" parts of each path should be hidden to shorten the overall
		 * path lengths and lessens the chance of paths wrapping. Hiding is
		 * controlled just with a CSS class, not in this method, but when
		 * hidden: the head of each path is a product configuration and it's
		 * label is always show. The tail is either the parent product of the
		 * target, or the product configuration itself if the target is
		 * directly part of the product configuration. All other
		 */
		_alignPaths: function() {
			// First, clear all the explicit width to get everything back to
			// its natural size. Needed when toggling show/hide all names.
			// Bulk set up front to prevent read-write DOM thrashing.
			this._resetExplicitHeadWidths();

			// Fill this array with dicts about each path. Contains
			// the head node, the head width, and the "middle" width.
			var widthDicts = [];

			// All heads+middles will need to be made this wide to line up the
			// tails.
			var widestHeadAndMiddlePx = 0;

			// Collect a dict of info on each path. Also record which is the
			// longest path (really the greated .left position of the tail
			// segment) so that all the other paths can be aligned to that.
			this._pathWidgets.forEach(function(path) {
				var cells = path.cells;
				var headNode = cells[0];

				var widthDict = {
					headNode: headNode
				};

				var headCell = cells[0];

				var headWidthPx = headCell.offsetWidth;
				widthDict.headPx = headWidthPx;

				var middleWidthPx = 0;
				for (var i = 1; i < cells.length - 1; i++) {
					var middleCell = cells[i];
					var middleCellWidthPx = middleCell.offsetWidth;
					middleWidthPx += middleCellWidthPx;
				}
				widthDict.middlePx = middleWidthPx;
				widestHeadAndMiddlePx = Math.max(widestHeadAndMiddlePx, headWidthPx + middleWidthPx);

				widthDicts.push(widthDict);
			});

			// Now loop again over the info dicts we just gathered and compare
			// to the widestHeadAndMiddlePx. Resize all the heads to get their
			// tails aligned.
			widthDicts.forEach(function(widthDict) {
				var headAndMiddlePx = widthDict.headPx + widthDict.middlePx;

				if (headAndMiddlePx < widestHeadAndMiddlePx) {
					var bestHeadPx = widestHeadAndMiddlePx - widthDict.middlePx;
					widthDict.headNode.style.width = bestHeadPx + 'px'; //$NON-NLS-1$
				}
			});
		},

		/**
		 * Updates all places on the page that use the title: the window title, the page title, the
		 * help text, and the "not used" messages.
		 * @param   {string} shortTitle    a short title that is a plain string. Used in the window title bar.
		 * @param   {string} longTitleHtml a longer title that can contain HTML for styling.
		 * @private
		 */
		_updateTitle: function(shortTitle, longTitleHtml) {
			if (!longTitleHtml) {
				longTitleHtml = jazz.util.html.escape(shortTitle);
			}
			var safeTitle = jazz.util.html.removeStyleAndScript(shortTitle);

			if (!shortTitle) {
				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(messages.pageTitle);
				this.set('pageTitle', messages.pageTitle); //$NON-NLS-1$
			}
			else {
				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(
					dojo.replace(messages.pageTitleForX, [shortTitle]));

				this.set('pageTitle', dojo.replace(messages.pageTitleForX, [safeTitle])); //$NON-NLS-1$
			}

			// Results Title
			var resultsTitleHtml = dojo.replace(messages.resultsTitleForProductX, [longTitleHtml]);
			this.set('resultsTitle', resultsTitleHtml); //$NON-NLS-1$

			// No results message
			com.ibm.relm.common.web.ui.util.setText(
				this._noResultsNode,
				dojo.replace(messages.noProductsUseProductX, [shortTitle]));
		},

		/**
		 * A lame ajax call just to get the name of the product we're looking it.
		 * The page is only initialized with the URI, so the title needs to be looked up.
		 * This could come back with the find use results, but that can take a while, so yes,
		 * there is an extra call just to get some info on the product! Fills in missing titles once done.
		 */
		_loadItemVersionTitle: function(itemVersionUri) {
			var url = net.jazz.ajax._contextRoot + '/ple/get_resource_version'; //$NON-NLS-1$

			var get = jazz.client.xhrGet({
				url: url,
				handleAs: 'json', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri
				}
			});

			var self = this;
			get.then(function(itemVersion) {
				var shortTitle = itemVersion['name']; //$NON-NLS-1$

				if (shortTitle) {
					var escapedLongTitle = formatting.formatProductTitleForItem(itemVersion);
					self._updateTitle(shortTitle, escapedLongTitle);
				}
			});
		},

		/**
		 * Loads and updates the title for an artifact, if this is from a friend server.
		 * Creates a ResourceLink with a rich hover if possible.
		 * @param   {strign} artifactUri URI to the artifact.
		 * @private
		 */
		_loadArtifactTitle: function(artifactUri, privateBaseline) {
			var linkSrc = dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'); //$NON-NLS-1$ //$NON-NLS-2$
			var linkHtml = '<img class="ple-icon"src="' + linkSrc + '"><a href="' + artifactUri + '">' + artifactUri + '</a>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			// No name for the link unless we get it's compact preview later.
			this._updateTitle('', linkHtml); //$NON-NLS-1$

			 var link = dojo.query('a', this._resultTitleNode)[0]; //$NON-NLS-1$
			 var img = dojo.query('img', this._resultTitleNode)[0]; //$NON-NLS-1$

			if (privateBaseline) {
				//If this is a doors baseline, we don't want the normal OSLC preview
				img.src = com.ibm.relm.ple.web.doors.DoorsUtils.DOORS_BASELINE_ICON;
				return;
			}

			this._resourceLink = new jazz.ui.ResourceLink({
				iconNode: img
			}, link);
			
			jazz.ui.internal.resourceLinkAggreagtor().register(artifactUri, dojo.hitch(this, this._updateArtifactTitleFromCompactXml, artifactUri), function() {});						
		},

		/**
		 * The title is automatically updated for the ResourceLink on the page,
		 * but the title is used more than once. This is the callback that receives the compact
		 * rendering (piggy backs off of ResourceLink call) and get the raw, title string.
		 * Updates the window title and other places on the page that need updating.
		 * @param   {object} result compact XML results.
		 * @private
		 */
		_updateArtifactTitleFromCompactXml: function(artifactUri, result) {
			var oslcCompact = result.oslcCompact;
			var compactRendering = result.compactRendering;
			
			var title = null;
			this.artifactIconSrc = null;
			
			if (oslcCompact) {
				title = oslcCompact.title;
				this.artifactIconSrc = oslcCompact.icon;
			}
			else if (compactRendering) {
				title = compactRendering.title;	
				this.artifactIconSrc = compactRendering.icon;
			}

			if (title) {
				var safeTitle = jazz.util.html.removeStyleAndScript(title);

				// Window Title
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(
					dojo.replace(messages.pageTitleForX, [title]));

				this.set('pageTitle', dojo.replace(messages.pageTitleForX, [safeTitle])); //$NON-NLS-1$
			}

			com.ibm.relm.common.web.ui.util.setText(
				this._noResultsNode,
				dojo.replace(messages.noProductsUseProductX, [title || artifactUri]));
		},

		_loadPaths: function(itemVersionUri, artifactUri, privateBaseline) {
			this._messageBox.hideMessage();

			if (itemVersionUri) {
				this._loadItemVersionTitle(itemVersionUri);
			}
			else {
				this._loadArtifactTitle(artifactUri, privateBaseline);
			}
			
			if (gCachedItemVersionUri === itemVersionUri) {
				this._loadingMessage.hide();
				// We already cached the paths to this item for a one time use.
				// We'll use those those data instead of a new ajax request.
				var pathsData = gCachedPathsData;

				gCachedItemVersionUri = null;
				gCachedPathsData = null;

				this._drawPaths(pathsData, privateBaseline);
				return;
			}

			this._loadingMessage.show();
			this._destroyPaths();
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.findingPathsLoading);

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/find_use', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri,
					artifactUri: artifactUri,
					privateBaseline: privateBaseline
				},

				handle: dojo.hitch(this, function() {
					this._loadingMessage.hide();
					message.hide();
				}),
			
				load: dojo.hitch(this, function (pathsData) {
					this._drawPaths(pathsData, privateBaseline);
				}),
			
				failOk: true,
				error: dojo.hitch(this, function(error, ioArgs) {
					this._messageBox.showXhrError(ioArgs.xhr);
					this._loadingMessage.hide();
				})
			});
		},

		/**
		 * Clears out the last event, so the next time this page is pulled up,
		 * we're guaranteed to requery for paths instead of possibly showing
		 * old results.
		 */
		_cleanCache: function() {
			delete this._lastEvent;
		},

		_clearPage: function(jazzEvent) {
			if (jazzEvent) {
				if (this._lastEvent) {
					if (this._lastEvent.action == jazzEvent.action &&
						this._lastEvent.time == jazzEvent.time &&
						this._lastEvent.product == jazzEvent.product)
					{
						// Same request via the browser history, so don't re-run.
						// This is an optimzation when the use toggles between this page and My Products
						// with the browser back/forward buttons.
						// Note that the 'time' is a param and is always new for user initiated
						// actions. It will only be the same from the history.
						PlatformUI.getWorkbench().setTitle(this._lastPageTitle || messages.pageTitle);
						return false;
					}
				}
			}

			this._lastEvent = jazzEvent;

			// Clear old filter text
			this._hiddenResultsMessage.hide();
			this._filterBox.clear();
			this._filterBox.blur();

			this._noResultsNode.style.display = 'none'; //$NON-NLS-1$

			this._destroyPaths();

			return true;
		},

		_loadPathsAgain: function() {
			this._clearPage(this._lastEvent);
			this._loadPaths(this._lastEvent.product, this._lastEvent.artifact, this._lastEvent.privateBaseline);
		},

		/**
		 * Finds full paths, and displays them in a clickable format.
		 * @param  {object} jazzEvent
		 */
		findPaths: function(jazzEvent) {
			if (this._lastEvent) {
				if (this._lastEvent.time == jazzEvent.time &&
					this._lastEvent.product == jazzEvent.product) {
					// Same request via the browser history, so don't rerun.
					return;
				}
			}
			
			PlatformUI.getWorkbench().setTitle(messages.pageTitle);
			if (!this._clearPage(jazzEvent)) return;
			
			this._lastEvent = jazzEvent;

			this._toggleFullPathsButton.set('pressed', false); //$NON-NLS-1$
			dojo.addClass(this._resultsContainer, 'ple-FindUse-hideTitles'); //$NON-NLS-1$
			this._loadPaths(jazzEvent.product, jazzEvent.artifact, jazzEvent.privateBaseline);
		},

		uninitialize: function() {
			this._hoverHelp && this._hoverHelp.destroyRecursive();
			this._destroyPaths();
		}
	});
	



	com.ibm.relm.ple.web.ui.pages.FindUsePage.cachePathsData = function(itemVersionUri, pathsData) {
		gCachedItemVersionUri = itemVersionUri;
		gCachedPathsData = pathsData;
	};

})();