/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.pages.ProductComparison"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "ProductComparisonMessages"); //$NON-NLS-1$ //$NON-NLS-2$


dojo.require("dojo.data.ItemFileReadStore"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.tree.BaseProductTree"); //$NON-NLS-1$
dojo.require("dijit.layout._LayoutWidget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.ActionRegistry"); //$NON-NLS-1$
dojo.require("dojox.json.query"); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.ProductTreeModel'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$
dojo.require('jazz.app.ua.PageHoverHelp'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.formatting"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

(function() {

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var util = com.ibm.relm.common.web.ui.util;
var formatting = com.ibm.relm.ple.web.ui.formatting;
var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.pages", "ProductComparisonMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var DIFF_BELOW = "diffBelow"; //$NON-NLS-1$

/**
 * After an expand or collapse ANIMATION completes on this tree node,
 * publishes the 'ple.product_compare.charms_changed' event.
 * The page listens for this to know to refresh diff type charms.
 *
 * This is pretty heavyweight, but we need to know when the animations finish to
 * know the _final_ positions of the tree nodes, since the charms
 * are drawn relative to these positions.
 */
dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.DiffTypeCharmTreeNode', //$NON-NLS-1$
	[ com.ibm.relm.ple.web.ui.tree.BaseProductTreeNode ],
	{
		messages: messages,

		expand: function() {
			var deferred = this.inherited(arguments);
			deferred.then(function() {
				dojo.publish('ple.product_compare.charms_changed'); //$NON-NLS-1$
			});
			return deferred;
		},

		collapse: function() {
			var deferred = this.inherited(arguments);
			deferred.then(function() {
				dojo.publish('ple.product_compare.charms_changed'); //$NON-NLS-1$
			});
			return deferred;
		},

		postCreate: function() {
			this.inherited(arguments);

			var self = this;
			var expandAll = new jazz.ui.toolbar.Button({
					title: this.messages.hideDetails,
					imgClass: 'ple-ProductComparison-collapseAll', //$NON-NLS-1$
					onClick: function(e) {
						dojo.stopEvent(e);
						self.tree._collapseRecursive(self);
					}
			});
			this.toolbar.addChild(expandAll);

			var collapseAll = new jazz.ui.toolbar.Button({
					title: this.messages.showDetails,
					imgClass: 'ple-ProductComparison-expandAll', //$NON-NLS-1$
					onClick: function(e) {
						dojo.stopEvent(e);
						self.tree._expandRecursive(self);
					}
			});
			this.toolbar.addChild(collapseAll);

			var hasDiffInChildren = this.hasDiffInChildren(this.item);
			if (!hasDiffInChildren) {
				dojo.addClass(this.toolbar.domNode, 'hiddenToolbar'); //$NON-NLS-1$
				expandAll.set('enabled', false); //$NON-NLS-1$
				collapseAll.set('enabled', false); //$NON-NLS-1$
			}
		},

		/**
		 * Recursively walks down tree. Returns true if it finds differences anywhere in the children of the node.
		 * @param  {TreeNode}  item the tree item whose children should be examined
		 * @return {Boolean}      true means that there are differences in its children so expan all should be
		 *                             enabled
		 */
		hasDiffInChildren: function(item) {
			//Alread checked for differences in children so answer is cached
			if (item.hasDiffInChildren) return true;

			//If it doesn't have any children, then no differences
			var children = item.children;
			if (!children || !children.length) return false;
			
			//Check direct children for differences
			var directChildrenDiffs = dojo.some(children, function(child) {
				return child.hasDiffInChildren || (child.diffType && child.diffType[0]);
			});
			if (directChildrenDiffs)  {
				item.hasDiffInChildren = true; 
				if (!item.diffType || !item.diffType[0]) 
					item.diffType = [DIFF_BELOW];
				return true;
			}

			//Direct children didn't have differences, so recurse down
			var recursiveChildrenDiffs = dojo.some(children, function(child) {
				return this.hasDiffInChildren(child);
			}, this);
			if (recursiveChildrenDiffs) {
				item.hasDiffInChildren = true;
				if (!item.diffType || !item.diffType[0]) 
					item.diffType = [DIFF_BELOW];
				return true;
			}

			//Default case
			return false;
		}
	}
);

/**
 * A tree that publishes an event after an expand or collapse is complete
 * on its nodes. Really, this just creates special tree nodes that know how to do that.
 */
dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.DiffTypeCharmTree', //$NON-NLS-1$
	[ com.ibm.relm.ple.web.ui.tree.BaseProductTree ],
	{
		constructor: function(args, node) {
			this.inherited(arguments);
			this._diffIndicatorNode = args.diffIndicatorNode;
		},

		postCreate: function() {
			this.inherited(arguments);

			dojo.addClass(this.domNode, 'ple-DiffTypeCharmTree'); //$NON-NLS-1$
		},

		_expandRecursive: function(node){
			// summary:
			//		Expand all nodes in the tree
			// returns:
			//		Deferred that fires when all nodes have expanded
			var _this = this;

			function expand(node){
				var def = new dojo.Deferred();

				// Expand the node
				_this._expandNode(node).then(function(){
					// When node has expanded, call expand() recursively on each non-leaf child
					var childBranches = dojo.filter(node.getChildren() || [], function(node){
							return node.isExpandable && node.item.hasDiffInChildren;
						}),
						defs = dojo.map(childBranches, expand);

					// And when all those recursive calls finish, signal that I'm finished
					new dojo.DeferredList(defs).then(function(){
						def.resolve(true);
					});
				});

				return def;
			}

			return expand(node);
		},

		_collapseRecursive: function(node){
			// summary:
			//		Collapse all nodes in the tree
			// returns:
			//		Deferred that fires when all nodes have collapsed

			var _this = this;

			function collapse(node){
				var def = new dojo.Deferred();
				def.label = "collapseAllDeferred"; //$NON-NLS-1$

				// Collapse children first
				var childBranches = dojo.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					}),
					defs = dojo.map(childBranches, collapse);

				// And when all those recursive calls finish, collapse myself, unless I'm the invisible root node,
				// in which case collapseAll() is finished
				new dojo.DeferredList(defs).then(function(){
					if(!node.isExpanded || (node == _this.rootNode && !_this.showRoot)){
						def.resolve(true);
					}else{
						_this._collapseNode(node).then(function(){
							// When node has collapsed, signal that call is finished
							def.resolve(true);
						});
					}
				});


				return def;
			}

			return collapse(node);
		},

		_createTreeNode: function(args) {
			args.diffIndicatorNode = this._diffIndicatorNode;
			return new com.ibm.relm.ple.web.ui.tree.DiffTypeCharmTreeNode(args);
		}
	}
);

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.ProductComparison", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ], {
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl("com.ibm.relm.ple.web","ui/templates/ProductComparison.html"), //$NON-NLS-1$ //$NON-NLS-2$
		messages: messages,

		postMixInProperties: function() {
			
		},

		postCreate: function() {
			this.inherited(arguments);

			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction("ple.compare", this, "loadCompare"); //$NON-NLS-1$ //$NON-NLS-2$

			this.subscribe('ple.product_compare.charms_changed', function() { //$NON-NLS-1$
				this._indicatorAreaPosition = dojo.position(this.dAP_diffIndicatorArea, true);
				dojo.empty(this.dAP_diffIndicatorArea);
				this.walkOpenTree(this.productTree1.rootNode.item);
			});
		},

		initializeHoverHelp: function(isStreamCompare) {
			this._hoverHelp && this._hoverHelp.destroyRecursive();
			dojo.empty(this._helpHolder);
			var uaKey_compare = "com.ibm.relm.ple.web.page_Compare001"; //$NON-NLS-1$
			var uaKey_variablitiyResults = "com.ibm.relm.ple.web.page_Compare001"; //$NON-NLS-1$
			if (isStreamCompare) {
				this._hoverHelp = new jazz.app.ua.PageHoverHelp({
					uaKey: uaKey_variablitiyResults
				});
			} else {
				this._hoverHelp = new jazz.app.ua.PageHoverHelp({
					uaKey: uaKey_compare
				});
			}
			this._hoverHelp.placeAt(this._helpHolder);
		},

		uninitialize: function() {
			this.productTree1 && this.productTree1.destroyRecursive();
			this.productTree2 && this.productTree2.destroyRecursive();
			this._hoverHelp && this._hoverHelp.destroyRecursive();
		},


		_getCompareUrl: function(jazzEvent) {
			this.isStreamCompare = jazzEvent.streamCompare && jazzEvent.streamCompare === "true"; //$NON-NLS-1$
			var defaultUrl = net.jazz.ajax._contextRoot + '/ple/compare'; //$NON-NLS-1$
			var streamCompareUrl = net.jazz.ajax._contextRoot + '/ple/compare_stream'; //$NON-NLS-1$

			this.initializeHoverHelp(this.isStreamCompare);

			if (this.isStreamCompare) {
				this._isVariabilityResults = this.isStreamCompare;
				return streamCompareUrl;
			} else {
				this._isCompareWithLastSnapshot = !this.isStreamCompare;
				return defaultUrl;
			}
		},

		_updateTitle: function() {
			var pageTitle = this.isStreamCompare
				? this.messages.variabilityPreviewPageTitle
				: this.messages.pageTitle;

			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(pageTitle);
			this._pageTitle.innerHTML = jazz.util.html.escape(pageTitle);
		},

		loadCompare: function(jazzEvent) {
			this._messageBox.hideMessage();

			this._noResultsNode.style.display = 'none'; //$NON-NLS-1$

			if (this.productTree1) this.productTree1.destroyRecursive();
			if (this.productTree2) this.productTree2.destroyRecursive();

			dojo.empty(this.dAP_diffIndicatorArea);

			var compareUrl = this._getCompareUrl(jazzEvent);
			this._updateTitle();

			jazz.client.xhrGet({
				url: compareUrl,
				handleAs: 'json', //$NON-NLS-1$
				content: {
					rightConfigVersionUri: jazzEvent.rightConfigVersionUri,
					rightPartUri: jazzEvent.rightPartUri,
					leftConfigVersionUri: jazzEvent.leftConfigVersionUri,
					leftPartUri: jazzEvent.leftPartUri
				},

				load: function(response) {
					try {
						if (!response.tree1) {
							this.showNoPredecessor(response.noPredecessorTitle);
							return;
						}

						this.tree1data = response.tree1;
						this.tree2data = response.tree2;
						this.loadTrees();
					} catch (e) {
						console.error("Failed to parse product comparison data.", e); //$NON-NLS-1$
					}
				}.bind(this),
				error: function(dojoError, ioArgs) {
					this._messageBox.showXhrError(ioArgs.xhr);
				}.bind(this),

				handle: function() {
					this.loadingMessage.hide();
				}.bind(this)
			});

			this.startup();
	    },

		showNoPredecessor: function(itemVersionTitle) {
			util.setText(this._noResultsNode, dojo.replace(this.messages.noResultsForProductX, [itemVersionTitle]));
			this._noResultsNode.style.display = ''; //$NON-NLS-1$

			this.comparisonContainer.style.display = 'none'; //$NON-NLS-1$
		},

		getRowClass: function(item, opened) {
			if (!item.diffType) return undefined;

			var diffType = item.diffType[0];

			if (diffType === 'add') { //$NON-NLS-1$
				return 'ple-ProductComparison-add'; //$NON-NLS-1$
			} else if (diffType === 'remove') { //$NON-NLS-1$
				return 'ple-ProductComparison-remove'; //$NON-NLS-1$
			} else if (diffType === 'change') { //$NON-NLS-1$
				return 'ple-ProductComparison-change'; //$NON-NLS-1$
			} else
				return 'ple-ProductComparison-equal'; //$NON-NLS-1$

			if (!opened &&
				(item.rowType[0] == "container") && //$NON-NLS-1$
				(diffType == "hasChange")) { //$NON-NLS-1$
				return 'ple-ProductComparison-change'; //$NON-NLS-1$
			}

			return undefined;
		},

		doClick: function(event) {
			if (event.link !== undefined) {
				window.open(event.link);
			}
		},

		charmClick: function(charm, node) {
			if (!dojo.hasClass(charm, 'ple-ProductComparison-diffBelow')) return; //$NON-NLS-1$

			node.tree._expandRecursive(node);
		},

		/**
		 * Draws a diff type charm at the given, absolute location.
		 * @param  {string} diffType the type to draw.
		 * @param  {number} x        the x position to draw at, relative to whole page.
		 * @param  {number} y        the y position to draw at, relative to whole page.
		 * @param  {boolean} node    the left tree node this charm is associated with
		 */
		drawCharm: function(/*String*/ diffType, x, y, node) {
			// create diff image
			var img = dojo.create("IMG", { //$NON-NLS-1$
				width: '16px', //$NON-NLS-1$
				height: '16px', //$NON-NLS-1$
				src: this._blankGif
			});
			this.connect(img, 'onclick', function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				this.charmClick(img, node);
			});

			// Fudge the positions a little  to account for padding and borders of
			// the tree rows.

			dojo.style(img, {
				position: 'absolute', //$NON-NLS-1$
				left: x + 'px', //$NON-NLS-1$
				top: y - this._indicatorAreaPosition.y + 6 + 'px' //$NON-NLS-1$
			});

			switch (diffType) {
			case "add": //$NON-NLS-1$
				img.className = 'ple-ProductComparison-addition'; //$NON-NLS-1$
				img.title = this.messages.addTooltip;
				img.alt = this.messages.addAlt;
				break;
			case "change": //$NON-NLS-1$
				img.className = 'ple-ProductComparison-delta'; //$NON-NLS-1$
				img.title = this.messages.changeTooltip;
				img.alt = this.messages.changeAlt;
				break;
			case "remove": //$NON-NLS-1$
				img.className = 'ple-ProductComparison-subtraction'; //$NON-NLS-1$
				img.title = this.messages.removeTooltip;
				img.alt = this.messages.removeAlt;
				break;
			case "hasChange": //$NON-NLS-1$
				img.className = 'ple-ProductComparison-delta'; //$NON-NLS-1$
				img.title = this.messages.changeTooltip;
				img.alt = this.messages.changeAl;
				break;
			case DIFF_BELOW:
				if (!node.isExpanded) {
					img.className = 'ple-ProductComparison-diffBelow'; //$NON-NLS-1$
					img.title = this.messages.diffBelowTooltip;
					img.alt = this.messages.diffBelowAlt;
				}
				break;
			case "blank": //$NON-NLS-1$
			default:
				return;
			}

            // add image to area
            var area = this.dAP_diffIndicatorArea;
            area.appendChild(img);
        },

		walkOpenTree: function(/*Object*/ item)	{
			var nodes = this.productTree1.getNodesByItem(item);
			if (nodes.length > 1) {
				// TODO: possible issue? found multiple tree nodes for same data item, could
				// they have different types??
				console.warn("Found duplicate tree nodes for an item"); //$NON-NLS-1$
			}

			var node = nodes[0];

			// Root is hidden in Forest models, so don't draw
			if (!item.root) {
				var nodePos = dojo.position(node.domNode, true);

				if (item.diffType) {
					this.drawCharm(item.diffType[0], 0, nodePos.y, node);
				}
			}

			if (node.isExpanded) {
				var children = item.children;
				for (var child in children) {
					this.walkOpenTree(children[child]);
				}
			}
		},

		loadTrees: function() {
			this._noResultsNode.style.display = 'none'; //$NON-NLS-1$
			this.comparisonContainer.style.display = ''; //$NON-NLS-1$

			var store1 = new dojo.data.ItemFileReadStore({
				data: this.tree1data
			});

			var tree1root = dojox.json.query("?id='1'", this.tree1data.items); //$NON-NLS-1$

			this.dAP_tree1Label.innerHTML = formatting.formatProductTitleForItem(tree1root[0]);

			var sortFunction = function(a, b) {
				var typeA = this.store.getValue(a, 'type') || this.store.getValue(a, 'hiddenType'); //$NON-NLS-1$ //$NON-NLS-2$
				var typeB = this.store.getValue(b, 'type') || this.store.getValue(b, 'hiddenType'); //$NON-NLS-1$ //$NON-NLS-2$

				if (typeA === 'link' && typeB !== 'link') { //$NON-NLS-1$ //$NON-NLS-2$
					return 1;
				}
				else if (typeB === 'link' && typeA !== 'link') { //$NON-NLS-1$ //$NON-NLS-2$
					return -1;
				}
				else {
					var sortTitleA = this.store.getValue(a, 'sortTitle'); //$NON-NLS-1$
					var sortTitleB = this.store.getValue(b, 'sortTitle'); //$NON-NLS-1$
					if (sortTitleA < sortTitleB) {
						return -1;
					}
					else if (sortTitleA > sortTitleB) {
							return 1;
						}
					else {
						return 0;
					}
				}
			};

			var treeModel1 = new com.ibm.relm.ple.web.ui.tree.ProductTreeModel({
				store: store1,
				sort: true,
				sortFunction: sortFunction,
				query: {
					"rootChild": true //$NON-NLS-1$
				},
				childrenAttrs: ["children"] //$NON-NLS-1$
			});

			var allChildren = dojox.json.query("[=children]", this.tree1data.items); //$NON-NLS-1$
			var addCount = 0;
			for (var index in allChildren) {
				if (allChildren[index]) {
					var emptyChild =  dojox.json.query("[?_reference='empty']", allChildren[index]); //$NON-NLS-1$
					addCount += emptyChild.length;
				}
			}
			
			if (this.productTree1) this.productTree1.destroyRecursive();

			// Only difftype decorate the first tree since we only need one set of diff icons
			this.productTree1 = new com.ibm.relm.ple.web.ui.pages.DiffTypeCharmTree({
				model: treeModel1,
				showRoot: false,
				persist: false,
				getRowClass: this.getRowClass
			});
			this.productTree1.placeAt(this.dAP_productTree1);

			dojo.addClass(this.productTree1.domNode, 'com-ibm-relm-ple-web-ProductComparison'); //$NON-NLS-1$

			this.connect(this.productTree1, "onDblClick", this.doClick); //$NON-NLS-1$

			this.connect(this.productTree1, "onClick", function(item, node, event) { //$NON-NLS-1$
				if (node.isExpandable)
					node.tree._onExpandoClick({node: node});
			});

			var store2 = new dojo.data.ItemFileReadStore({
				data: this.tree2data
			});

			var tree2root = dojox.json.query("?id='1'", this.tree2data.items); //$NON-NLS-1$

			var tree2FormattedTitle = formatting.formatProductTitleForItem(tree2root[0]);

			if (this._isCompareWithLastSnapshot) {
				this.dAP_tree2Label.innerHTML = dojo.replace(this.messages.tree2label, [tree2FormattedTitle]);
	    	} else if (this._isVariabilityResults) {
	    		this.dAP_tree2Label.innerHTML = dojo.replace(this.messages.tree2labelStream, [tree2FormattedTitle]);
	    	} else {
	    		this.dAP_tree2Label.innerHTML = tree2FormattedTitle;
	    	}

			var treeModel2 = new com.ibm.relm.ple.web.ui.tree.ProductTreeModel({
				store: store2,
				sort: true,
				sortFunction: sortFunction,
				query: {
					"rootChild": true //$NON-NLS-1$
				},
				childrenAttrs: ["children"] //$NON-NLS-1$
			});

			if (this.productTree2) this.productTree2.destroyRecursive();

			this.productTree2 = new com.ibm.relm.ple.web.ui.tree.BaseProductTree({
				model: treeModel2,
				showRoot: false,
				persist: false,
				getRowClass: this.getRowClass
			});
			this.productTree2.placeAt(this.dAP_productTree2);
			dojo.addClass(this.productTree2.domNode, 'com-ibm-relm-ple-web-ProductComparison'); //$NON-NLS-1$

			this.connect(this.productTree2, "onDblClick", this.doClick); //$NON-NLS-1$

			var tree2ExpandoClick = dojo.hitch(this, function(tree2Node) {
				if (!tree2Node.isExpandable) return;
				var nodesInTree1 = this.productTree1.getNodesByItem(tree2Node.item);
				if (nodesInTree1.length > 0) {
					var node = nodesInTree1[0];
					if (node.isExpanded) {
						this.productTree1._collapseNode(node);
					} else {
						this.productTree1._expandNode(node);
					}
					//this.productTree2.focusNode(event.node);
			}});

			this.connect(this.productTree2, "onClick", function(item, node){ //$NON-NLS-1$
				return tree2ExpandoClick(node);
			});

			// replace expansion/collapse of nodes in tree2 to call tree1's _expandNode/_collapseNode
			// which will then in turn open the same nodes in tree2 linked via dojo.connect()
			// TODO: should we really be overridding private methods like this?
			// TODO: all this won't work if there are multiple nodes with same item (i.e. container)
			this.productTree2._onExpandoClick = function(event) {
				return tree2ExpandoClick(event.node);
			};
			this.productTree2._onLeftArrow = dojo.hitch(this, function(event) {
				var nodesInTree1 = this.productTree1.getNodesByItem(event.node.item);
				if (nodesInTree1.length > 0) {
					var node = nodesInTree1[0];
					this.productTree1._collapseNode(node);
				}
			});
			this.productTree2._onRightArrow = dojo.hitch(this, function(event) {
				var nodesInTree1 = this.productTree1.getNodesByItem(event.node.item);
				if (nodesInTree1.length > 0) {
					var node = nodesInTree1[0];
					this.productTree1._expandNode(node);
				}
			});

			this.productTree1.connect(this.productTree1, '_expandNode', dojo.hitch(this, function(item) { //$NON-NLS-1$
				expandTreeByItemsId(this.productTree2, item);
			}));

			this.productTree1.connect(this.productTree1, '_collapseNode', dojo.hitch(this, function(item) { //$NON-NLS-1$
				collapseTreeByItemsId(this.productTree2, item);
			}));

			this.productTree1.startup();
			this.productTree2.startup();
	    }
	});

function getDescendantNode(node, id) {
	var children = node.getChildren();
	for (var i = 0; i < children.length; i++) {
		var child = children[i];

		if (id == child.item.id) {
			return child;
		}
		var descendant = getDescendantNode(child, id);
		if (descendant) {
			return descendant;
		}
	}

	return undefined;
}

function expandTreeNodeById(tree, id) {
	var openNode = getDescendantNode(tree.rootNode, id);

	if (openNode) {
		tree._expandNode(openNode);
		var parentNode = openNode.getParent();
		while (parentNode) {
			if(parentNode.baseClass == 'dijitTreeNode') { //$NON-NLS-1$
				tree._expandNode(parentNode);
				parentNode = parentNode.getParent();
			}
			else{
				parentNode=null;
			}
		}
	}
}

function collapseTreeNodeById(tree, id) {
	var openNode = getDescendantNode(tree.rootNode, id);

	if (openNode) {
		tree._collapseNode(openNode);
	}
}

function expandTreeByItemsId(tree, item) {
	//TODO: this is working with items and not nodes
	var selectedItem = item.item;
	if (selectedItem) {
		if (selectedItem.id) {
			var id = selectedItem.id[0];
			expandTreeNodeById(tree, id);
		}
	}
}

function collapseTreeByItemsId(tree, item) {
	var selectedItem = item.item;
	if (selectedItem) {
		if (selectedItem.id) {
			var id = selectedItem.id[0];
			collapseTreeNodeById(tree, id);
		}
	}
}

})();