/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.pages.ProductSearchProvider'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "ProductPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.searching'); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('jazz.ui._SearchProvider'); //$NON-NLS-1$
dojo.require('jazz.util.ResizeNotifier'); //$NON-NLS-1$

(function(){

var searching = com.ibm.relm.ple.web.ui.searching;
var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'ProductPickerMessages'); //$NON-NLS-1$ //$NON-NLS-2$


dojo.declare(
	'com.ibm.relm.ple.web.ui.pages.ProductSearchProvider', //$NON-NLS-1$
	jazz.ui._SearchProvider,
{
	constructor: function() {
		this.scopes = searching.getProductScopes();
	},

	/**
	 * The server-side code still expects diffent scopes for "newest" vs "all"
	 * products. But the UI now shows "newest" as a check box in most cases
	 * ("Search" result full page still shows just scopes). This method
	 * updates the search args to change to the appropriate scope depending on
	 * the state of the check box.
	 * @param  {object} args normal search args from foundation.
	 */
	_updateScopeFromAdvancedOptions: function(args) {
		if (args.scope === 'newestProductsAndConfigurations' //$NON-NLS-1$
			|| args.scope === 'allProductsAndConfigurations') { //$NON-NLS-1$

			if (this._newestCheckbox) {
				args.scope = this._newestCheckbox.checked
					? 'newestProductsAndConfigurations' //$NON-NLS-1$
					: 'allProductsAndConfigurations'; //$NON-NLS-1$
			}
		}
	},

	onUpdate: function(args) {
		this._updateScopeFromAdvancedOptions(args);
		this._lastArgs = args;
		searching.searchOnUpdateForBannerQuickSearch(args);
	},

	onExecute: function(args) {
		this._updateScopeFromAdvancedOptions(args);

		window.location.href = '#' + dojo.objectToQuery({ //$NON-NLS-1$
			action: 'relm.queries.search', //$NON-NLS-1$
			q: args.query,
			scope: args.scope,
			time: new Date().getTime()
		});
	},

	postCreate: function() {
		// Check right now since we probably already missed the first _selectPage event.
		var pageId = net.jazz.ajax.ui.PlatformUI.getWorkbench().getCurrentPageId();
		this.highlightProductsIfPlePage(pageId);

		var pageList = dijit.byId('net_jazz_ajax_ui_PageList_0'); //$NON-NLS-1$
		this.connect(pageList, '_selectPage', function(page) { //$NON-NLS-1$
			this.highlightProductsIfPlePage(page.id);
		});

		/* This is a work around for foundation defect 
		https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=315935
		It can be removed once that is fixed, but should do no harm.
		*/
		setTimeout(function() {
			dojo.publish('jazz.util.ResizeNotifier.resizeEvent'); //$NON-NLS-1$
		}, 1000);
	},

	/**
	 * Always show the advanced options by default since they are small (only
	 * one checkbox) and give a clue to what the results are. If we showed
	 * "all" by default, we might choose to not open this, but since results
	 * are filtered to "newest" by default, we want it clear why some results
	 * are missing.
	 * 
	 * @type {Boolean}
	 */
	defaultAdvancedOptions: true,

	/**
	 * Foundation API that creates the advanced section with the "newest"
	 * checkbox.
	 * 
	 * @return {DOM node}
	 */
	createAdvancedOptions: function() {
		var optionsNode = dojo.create('div', { //$NON-NLS-1$
			className: 'r-small-margin-tb' //$NON-NLS-1$
		});

		var newestLabel = dojo.create('label', { //$NON-NLS-1$
			className: 'j-label  r-cursor-pointer', //$NON-NLS-1$
			title: messages.showNewestTooltip
		}, optionsNode);

		this._newestCheckbox = dojo.create('input', { //$NON-NLS-1$
			type: 'checkbox', //$NON-NLS-1$
			checked: true
		}, newestLabel);

		// Rerun the search as as soon as checkbox is clicked.
		this.connect(this._newestCheckbox, 'onclick', this.update.bind(this)); //$NON-NLS-1$

		newestLabel.appendChild(document.createTextNode(' ' + messages.showNewestCheckbox)); //$NON-NLS-1$
		return optionsNode;
	},

	/**
	 * Checks the page ID to see if it's a PLE page.
	 * If so, forces the Products component menu to highlight.
	 */
	highlightProductsIfPlePage: function(pageId) {
		if (!pageId) return;
		
		// By pure convention, all the relevant PLE pages start with the same ID prefix.
		if (pageId.indexOf('com.ibm.relm.ple.') === 0 && pageId != 'com.ibm.relm.ple.web.ui.pages.MainWelcomePage') { //$NON-NLS-1$ //$NON-NLS-2$
			var pageList = dijit.byId('net_jazz_ajax_ui_PageList_0'); //$NON-NLS-1$
			dojo.forEach(pageList.domNode.getElementsByTagName("A"), function(link) { //$NON-NLS-1$
				dojo.toggleClass(link, "selected", link.__jazzPageId === 'com.ibm.relm.ple.web.ui.pages.ProductsWelcomePage'); //$NON-NLS-1$ //$NON-NLS-2$
			});
		}
	}
});

})();
