/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.pages.ReadOnlyProductPage'); //$NON-NLS-1$

// dojo.require('dojo.i18n');
// dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'ReadOnlyProductPageMessages');
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.internal.views.ProductView'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProductBreadcrumbs'); //$NON-NLS-1$


(function(){
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	/**
	 * Page to show a product outside of the tree. Used in response to loading a frontside URI
	 * for a product with text/html as the Accept type.
	 *
	 * This is a read-only view since you should not modify products outside the tree (i.e., without knowing
	 * where it is bound before editing it).
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.pages.ReadOnlyProductPage', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/ReadOnlyProductPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			//messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'ReadOnlyProductPageMessages'),

			postCreate: function() {
				this.inherited(arguments);

				var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
				ActionRegistry.registerAction('ple.read_only_product', this, 'loadProductAction'); //$NON-NLS-1$ //$NON-NLS-2$
			},

			loadProductAction: function(jazzEvent) {
				var resourceVersionUri = jazzEvent.product;
				this._productView && this._productView.destroyRecursive();
				delete this._productView;

				this._messageBox.hideMessage();

				var self = this;
				jazz.client.xhrGet({
					url: net.jazz.ajax._contextRoot + '/ple/get_resource_version', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						itemVersionUri: resourceVersionUri,
						includeUserProperties: true,
						includeLinks: true
					},
				
					load: function (itemVersion) {
						self._productView && self._productView.destroyRecursive();
					
						self._productView = com.ibm.relm.ple.web.ui.internal.views.ProductView({
							readOnly: true,
							product: itemVersion
						});

						self._productView.placeAt(self._viewNode);
						// Since clicking the "Show in Product Tree" button adds it as recent, I don't think we need this.
						// com.ibm.relm.ple.web.ui.productapi.addToRecentlyViewed(itemVersionUri);
					},
				
					error: function(error, ioArgs) {
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});
				
			},

			uninitialize: function() {
				this._productView && this._productView.destroyRecursive();
				this.inherited(arguments);
			}
		}
	);

})();