/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.ReplaceVersionDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'ReplaceVersionDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.MessageBox'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('dojox.form.DropDownSelect'); //$NON-NLS-1$
dojo.require('dijit.form.Select'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.ActionPreview'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var html = jazz.util.html;
	var StatusMessage = com.ibm.relm.common.web.StatusMessage;

	/**
	 * Dialog to select which resourceVersion to use for a product.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.pages.ReplaceVersionDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/ReplaceVersionDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'ReplaceVersionDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * When not enabled, this list will appear grayed out and will not
			 * respond to most user action (keyboard nav, clicks etc.). This
			 * is used when a "submit" is in progress and we don't want the
			 * user to interact with the list, specifically we don't want them
			 * triggering two submits.
			 * @type {Boolean}
			 */
			enabled: true,

			site: null,

			iconClass: null,

			constructor: function(args, node)
			{
				var site = args.site;
				var resourceVersion = site.itemVersion || site.emptyPart || site.configVersion;
				this.resourceVersion = resourceVersion;
				this.forConfigVersions = !!site.configVersion || (site.emptyPart && site.emptyPart.isPartForConfigVersion);

				var escapedNames = {
					productName: html.escape(resourceVersion.name),
					parentName: html.escape(site.parentResourceVersion.name)
				};

				if (site.emptyPart) {
					this._headerMain = dojo.replace(this.messages.selectVersionNameXUnderX,
						[escapedNames.productName, escapedNames.parentName]);
				}
				else {
					this._headerMain = dojo.replace(this.messages.selectVersionNameXBranchXUnderX,
						[escapedNames.productName, this._getBranchLabelEscaped(), escapedNames.parentName]);
				}
				this._headerDetails = (this.forConfigVersions) ? this.messages.headerDetailsConfigs : this.messages.headerDetails;
			},

			postCreate: function() {
				this.inherited(arguments);
				
				var leftItem = {
					'displayName': formatting.formatProductIconAndTitleForItem(this.resourceVersion), //$NON-NLS-1$
					'numChildren': 0, //$NON-NLS-1$
					'parentDisplayName': formatting.formatProductIconAndTitleForItem(this.site.parentResourceVersion) //$NON-NLS-1$
				};
				
				this._actionPreview = new com.ibm.relm.common.web.ui.ActionPreview({ 'leftItem': leftItem }, this._header._actionPreviewDiv); //$NON-NLS-1$

				if (this.forConfigVersions) {
					this.servicePath = '/ple/replace_configuration'; //$NON-NLS-1$
				} else {
					this.servicePath = '/ple/replace_version'; //$NON-NLS-1$
				}
				this._loadVersions();

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.forConfigVersions ? this.messages.titleReplaceConfiguration : this.messages.titleReplaceVersion,
					width: '700px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: (this.forConfigVersions) ? 'ple-icon-replaceConfigVersion' : 'ple-icon-replaceItemVersion' //$NON-NLS-1$ //$NON-NLS-2$
				});
			},

			startup: function() {
				this._outerTable.focus();
			},

			/**
			* Toggles the enabled property of this dialog. This is used when the action is being submitted so
			* nothing new is selected or a second submission made.
			*/
			toggleEnabled: function(enabled) {
				if (enabled !== undefined) {
					this.set('enabled', enabled); //$NON-NLS-1$
				}
				else {
					this.set('enabled', !this.enabled); //$NON-NLS-1$
				}
			},

			/**
			* Adjusts the state of the widget as the enabled attribute is set
			*/
			_setEnabledAttr: function(enabled) {
				this.enabled = enabled;
				this._okButton.disabled = !enabled || !this.selectedRow;
				this._cancelButton.disabled = !enabled;
				dojo.toggleClass(this.domNode, 'relm-ple-ReplaceVersionDialog-disabled', !enabled); //$NON-NLS-1$
			},

			_handleKeyPress: function(e) {
				var key = e.keyCode;
				if (key === dojo.keys.UP_ARROW) {
					this.selectPrevious();
				} else if (key === dojo.keys.DOWN_ARROW) {
					this.selectNext();
				} else if (key == dojo.keys.ENTER) {
					this._submit();
				}
			},

			selectNext: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select first on arrow down if no selection
					var maybeRow = this._scrollingTable.tbody.firstChild;
					//Sometimes the firstChild of the tbody is a text node and the next child is the actual first row
					var row = (maybeRow.nodeName == "TR")? maybeRow : maybeRow.nextSibling; //$NON-NLS-1$
					this.selectRow(row);
				}
				else if (this.selectedRow.nextSibling) {
					this.selectRow(this.selectedRow.nextSibling);
				}
			},

			selectPrevious: function() {
				if (!this.enabled) return;

				if (!this.selectedRow) {
					// select last on arrow up if no selection
					this.selectRow(this._scrollingTable.tbody.lastChild);
				}
				//Sometimes the firstChild of the tbody is a text node so we don't want to try to select it
				else if (this.selectedRow.previousSibling && this.selectedRow.previousSibling.nodeName == "TR") { //$NON-NLS-1$
					this.selectRow(this.selectedRow.previousSibling);
				}
			},

			selectRow: function(row) {
				if (!this.enabled) return;

				if (this.selectedRow) {
					dojo.removeClass(this.selectedRow, 'relm-ple-ReplaceVersionDialog-selectedRow'); //$NON-NLS-1$
				}

				this.selectedRow = row;
				this._newItemVersionUri = row.uri;
				dojo.addClass(row, 'relm-ple-ReplaceVersionDialog-selectedRow'); //$NON-NLS-1$
				this._okButton.disabled = false;
				
				//TODO!: maybe just don't show the parent or children of the right side, its ALWAYS duplicate right?
				var rightItem = {
					'displayName': dojo.query('.td2', row)[0].innerHTML, //Product is in column td2 //$NON-NLS-1$ //$NON-NLS-2$
					'numChildren': 0, //$NON-NLS-1$
					'parentDisplayName': formatting.formatProductIconAndTitleForItem(this.site.parentResourceVersion) //$NON-NLS-1$
				};
				
				this._actionPreview.setRightItem(rightItem);
			},

			_getBranchLabelEscaped: function() {
				var branchLabel = this.resourceVersion.branchName;
				return branchLabel ? html.escape(branchLabel) : this.messages.noDimensionsLabel;
			},

			_loadVersions: function() {
				var self = this;

				var content = {
					contextPartUri: this.site.contextPartUri,
					conceptUri: this.resourceVersion.conceptUri
				};

				if (!this.forConfigVersions) {
					// We prefer the branch URI, if available, but for an empty part, 
					// we will fallback to the context part which will get all versions.
					content.branchUri = this.resourceVersion.branchUri;
				}

				jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + self.servicePath,
					content: content,
					
					handle: function(response, ioArgs) {
						self._loadingMessage.hide();
					},

					// all versions on the same branch are returned, whether mutable or immutable (resourceVersions)
					load: function(itemVersions) {
						if (itemVersions.length > 0) {
							self._drawTable(itemVersions);
						}
						else { // there are no replacement options, only self
							var isImmutable = itemVersions[0].isImmutable;
							var message = self.messages.noOtherVersionsOfXBranchX;
							self._messageBox.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
								summary: dojo.replace(message, [html.escape(self.resourceVersion.name), self._getBranchLabelEscaped()])
							});
						}
					},

					error: function(error, ioArgs) {
						console.error(error);
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});

			},

			_drawTable: function(resourceVersions) {
				dojo.addClass(this._outerTable, 'ple-scrollingTable-containerBorder'); //$NON-NLS-1$
				
				var columns;

				if (this.forConfigVersions) {
					columns = [
						{ label: this.messages.configColumnLabel },
						{ label: this.messages.versionColumnLabel },
						{ label: this.messages.creationDateColumnLabel },
						{ label: this.messages.creatorColumnLabel }
					];
				} else {
					columns = [
						{ label: this.messages.productVersionColumnLabel },
						{ label: this.messages.creationDateColumnLabel },
						{ label: this.messages.creatorColumnLabel }
					];
				}

				this._scrollingTable = new jazz.ui.StyledTable({
					columns: columns
				});

				for (var i = 0; i < resourceVersions.length; i++) {
					var resourceVersion = resourceVersions[i];
					var tr = (this.forConfigVersions) ? this._drawConfigVersionRow(resourceVersion) : this._drawItemVersionRow(resourceVersion);

					this.connect(tr, 'onclick', dojo.hitch(this, this.selectRow, tr)); //$NON-NLS-1$
					this.connect(tr, 'ondblclick', dojo.hitch(this, this._onRowDblClick, tr)); //$NON-NLS-1$
					this._scrollingTable.tbody.appendChild(tr);
				}

				this._scrollingTable.placeAt(this._tableHolder);
			},

			_drawItemVersionRow: function(resourceVersion) {
				var tr = dojo.create('tr'); //$NON-NLS-1$
				tr.uri = resourceVersion.uri;
				
				// Product
				dojo.create('td', { //$NON-NLS-1$
					innerHTML: formatting.formatProductIconAndTitleForItem(resourceVersion),
					'class': 'td2' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				// Created
				var createdMillis = resourceVersion['createdMillis']; //$NON-NLS-1$
				var createdDate = new Date(createdMillis);
				var formattedDate = dojo.date.locale.format(
					createdDate,
					{ formatLength: 'medium' } //$NON-NLS-1$
				);

				dojo.create('td', { //$NON-NLS-1$
					innerHTML: jazz.util.html.escape(formattedDate),
					'class': 'td3' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				var creator = resourceVersion.creatorName;
				var indirectlyCreated = resourceVersion.isIndirectlyCreated;

				dojo.create('td', { //$NON-NLS-1$
					innerHTML: indirectlyCreated ? formatting.formatIndirectCreator(creator) : formatting.formatCreator(creator),
					'class': 'td4' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				return tr;
			},

			_drawConfigVersionRow: function(resourceVersion) {
				var tr = dojo.create('tr'); //$NON-NLS-1$
				tr.uri = resourceVersion.uri;
				
				// Product
				dojo.create('td', { //$NON-NLS-1$
					innerHTML: formatting.formatProductIconAndTitleForItem(resourceVersion),
					'class': 'td2' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				//Version
				dojo.create('td', { //$NON-NLS-1$
					innerHTML: resourceVersion.version,
					'class': 'td2' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				// Created
				var createdMillis = resourceVersion['createdMillis']; //$NON-NLS-1$
				var createdDate = new Date(createdMillis);
				var formattedDate = dojo.date.locale.format(
					createdDate,
					{ formatLength: 'medium' } //$NON-NLS-1$
				);

				dojo.create('td', { //$NON-NLS-1$
					innerHTML: jazz.util.html.escape(formattedDate),
					'class': 'td3' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				var creator = resourceVersion.creatorName;
				var indirectlyCreated = resourceVersion.isIndirectlyCreated;

				dojo.create('td', { //$NON-NLS-1$
					innerHTML: indirectlyCreated ? formatting.formatIndirectCreator(creator) : formatting.formatCreator(creator),
					'class': 'td4' //$NON-NLS-1$ //$NON-NLS-2$
				}, tr);

				return tr;
			},

			_onRowDblClick: function(row) {
				if (!this.enabled) return;

				this.selectRow(row);
				this._submit();
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			_closeDialog: function() {
				this._dialog.close();
			},

			/**
			 * Called when form is submitted (click of button or type 'Enter').
			 */
			_submit: function(e) {
				e && dojo.stopEvent(e);

				this._messageBox.hideMessage();

				var self = this;

				var message = StatusMessage.show(this.messages.replacingVersion);
				this.toggleEnabled(false);

				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/replace_version', //$NON-NLS-1$
					content: {
						contextConfigVersionUri: this.site.contextConfigVersionUri,
						contextPartUri: this.site.contextPartUri,
						newItemVersionUri: this._newItemVersionUri,
						
						parentVersionUri: this.site.parentResourceVersion.uri,
						partUri: this.site.contextPartUri, //$NON-NLS-1$
						oldItemVersionUri: this.site.itemVersionUri
					},

					handle: function() {
						message.hide();
					},

					load: function(result) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectPath: self.site.path
						}).addBoth(function() {
							self._closeDialog();
						});
					},

					error: function(_, args) {
						self.toggleEnabled(true);
						self._messageBox.showXhrError(args.xhr);
					}
				});
			},

			/**
			 * Cancels the create and closes the dialog.
			 */
			_cancel: function() {
				// this callbacks to destroy outer dialog too
				this._closeDialog();
			},

			uninitialize: function() {
				this._actionPreview.destroyRecursive();
				if (this._scrollingTable)
					this._scrollingTable.destroyRecursive();
			}

		});

	})();