/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.ple.web.ui.pages.UpgradeInProgressPage'); //$NON-NLS-1$


dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require('net.jazz.ajax.ui.PlatformUI'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.FloatingMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox');//$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'UpgradeInProgressPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var upgradeStatus;
var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'UpgradeInProgressPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$

var STATUS_COMPLETE = 'http://jazz.net/ns/pd/extensions#upgradeCompletedSuccessfully'; //$NON-NLS-1$
var STATUS_ERROR = 'http://jazz.net/ns/pd/extensions#upgradeCompletedWithError'; //$NON-NLS-1$
var STATUS_IN_PROGRESS = 'http://jazz.net/ns/pd/extensions#upgradeInProgress'; //$NON-NLS-1$
var STATUS_NOT_STARTED = 'http://jazz.net/ns/pd/extensions#upgradeNotStarted'; //$NON-NLS-1$
var STATUS_TO_CLASS_AND_ALT_TEXT = {};
	STATUS_TO_CLASS_AND_ALT_TEXT[STATUS_COMPLETE] = ['complete', messages.statusComplete]; //$NON-NLS-1$
	STATUS_TO_CLASS_AND_ALT_TEXT[STATUS_ERROR] = ['error', messages.statusError]; //$NON-NLS-1$
	STATUS_TO_CLASS_AND_ALT_TEXT[STATUS_IN_PROGRESS] = ['inProgress', messages.statusInProgress]; //$NON-NLS-1$
	STATUS_TO_CLASS_AND_ALT_TEXT[STATUS_NOT_STARTED] = ['notStarted', '']; //$NON-NLS-1$ //$NON-NLS-2$
var STATUS_COL_ICON = 0;
var STATUS_COL_TITLE = 1;
var STATUS_COL_START = 2;
var STATUS_COL_COMPLETE = 3;
var STATUS_COL_TIMESPENT = 4;
	
jazz.client.xhrGet({
	url: net.jazz.ajax._contextRoot + "/ple/upgrade_status", //$NON-NLS-1$
	handleAs: 'json', //$NON-NLS-1$
	load: function(status) {
		upgradeStatus = status;
	},
	sync: true,
	failOk: true
});

dojo.declare(
	"com.ibm.relm.ple.web.ui.pages.UpgradeInProgressPage", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/UpgradeInProgressPage.html'), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,

		constructor: function() {
			dojo.mixin(this, com.ibm.relm.ple.web.ui.pages.UpgradeInProgressPage);
			this.messages = messages;
			if (!this.isUpgradeInProgress()) {
				// redirect to RELM Welcome page if user comes to this page while Upgrade is not in progress
				window.location.hash = "#"; //$NON-NLS-1$
			}
		},

		postCreate: function() {
			this.inherited(arguments);
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction('relm.upgrade_in_progress', this, 'init'); //$NON-NLS-1$ //$NON-NLS-2$
		},

		upgradeCompleted: function() {
			this.floatingMessageBox.showMessage({
				type: 'OK', //$NON-NLS-1$
				summary: this.messages.upgradeComplete
			}); 
			window.setTimeout(this.refreshPage, 3000);
		},

		refreshPage: function(e) {
			e && dojo.stopEvent(e);
			window.location.reload();
		},

		updateStatus: function() {
			jazz.client.xhrGet({
				url: net.jazz.ajax._contextRoot + "/ple/upgrade_status", //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				load: function(status) {
					upgradeStatus = status;
					this._drawUpdateSteps();
					switch (upgradeStatus.overallStatus) {
						case STATUS_NOT_STARTED:
						case STATUS_IN_PROGRESS:
						//No-op
						break;
						case STATUS_ERROR: 
						this.showErrorMessage();
						break;
						case STATUS_COMPLETE: 
						window.clearInterval(this.checkStatusInterval);
						this.upgradeCompleted();
						break;
						default: 
						console.error("No upgrade status received from server."); //$NON-NLS-1$
						this.showErrorMessage();
						break;
					}
				}.bind(this),
				error: function() {
					this.showErrorMessage();
				}.bind(this)
			});
		},

		showErrorMessage: function() {
			this.errorMessageBox.showErrorObject({
				summary: this.messages.upgradeStepError
			});
		},
		
		getStepStart: function(step) {
			if (step.startDate) {
				return dojo.date.locale.format(new Date(step.startDate), {formatLength: "medium"}); //$NON-NLS-1$
			} else {
				return '&nbsp;'; //$NON-NLS-1$
			}
		},

		getStepComplete: function(step) {
			if (step.completeDate) {
				return dojo.date.locale.format(new Date(step.completeDate), {formatLength: "medium"}); //$NON-NLS-1$
			} else {
				return '&nbsp;'; //$NON-NLS-1$
			}
		},
		
		getStepTimeSpent: function(step) {
			if (step.startDate && step.completeDate) {
				var timeSpentInSeconds = dojo.date.difference(new Date(step.startDate), new Date(step.completeDate), "second"); //$NON-NLS-1$
				return this.formatTimeInterval(timeSpentInSeconds); //$NON-NLS-1$
			} 
			else if (step.status == STATUS_IN_PROGRESS && step.startDate) {
				var timeSpentInSeconds = dojo.date.difference(new Date(step.startDate), new Date(), "second"); //$NON-NLS-1$
				return this.formatTimeInterval(timeSpentInSeconds); //$NON-NLS-1$				
			} 
			else {
				return '&nbsp;'; //$NON-NLS-1$
			}
		},
		
		getTotalTimeSpent: function(step) {
			if (upgradeStatus && upgradeStatus.allSteps[0].startDate) { 
				// if upgrade is still running just quickly use the current time minus the first step's start time
				if (upgradeStatus.overallStatus == STATUS_IN_PROGRESS) {
					var timeSpentInSeconds = dojo.date.difference(new Date(upgradeStatus.allSteps[0].startDate), new Date(), "second"); //$NON-NLS-1$
					return this.formatTimeInterval(timeSpentInSeconds); //$NON-NLS-1$
				}
				// in other cases find the last completed step and use that time minus first step's start time
				else {
					for (var i = upgradeStatus.allSteps.length-1; i >= 0; --i) {
						var compDate = upgradeStatus.allSteps[i].completeDate; 
						if (compDate && compDate > 0) {
							var timeSpentInSeconds = dojo.date.difference(new Date(upgradeStatus.allSteps[0].startDate), new Date(compDate), "second"); //$NON-NLS-1$
							return this.formatTimeInterval(timeSpentInSeconds); //$NON-NLS-1$
						}
					}
					// Nothing finished
					return '&nbsp;'; //$NON-NLS-1$
				}
			}
			else {
				return '&nbsp;'; //$NON-NLS-1$
			}
		},
		
		formatTimeInterval: function(totalSeconds) {
			var hours = Math.floor(totalSeconds / 3600);
			totalSeconds %= 3600;
			var minutes = Math.floor(totalSeconds / 60);
			var seconds = totalSeconds % 60;
			
			var sMinutes = dojo.number.format(minutes, {pattern: "00"}); //$NON-NLS-1$
			var sSeconds = dojo.number.format(seconds, {pattern: "00"}); //$NON-NLS-1$
			var templstr = this.messages.resultCellTimeSpent;
			var valarr = [hours, sMinutes, sSeconds];
			if (hours == 0 && minutes == 0) {
				templstr = this.messages.resultCellTimeSpentSecOnly;
				valarr = [sSeconds];
			}
			else if (hours == 0 && minutes > 0){
				templstr = this.messages.resultCellTimeSpentMinOnly;
				valarr = [sMinutes, sSeconds];
			}	
			return dojo.replace(templstr, valarr);
		},

		/**
		 * Draws the upgrade steps. If the upgrade steps are already drawn, then their status is updated.
		 */
		_drawUpdateSteps: function() {
			upgradeStatus && upgradeStatus.allSteps.forEach(function(step) {
				try {
					var existingNode = dojo.byId(step.id);
					var classNameAndAltText = STATUS_TO_CLASS_AND_ALT_TEXT[step.status];
					if (existingNode) {
						var cells = dojo.query('td', existingNode); //$NON-NLS-1$
						dojo.query('img', cells[STATUS_COL_ICON]) //$NON-NLS-1$
							.removeClass()
							.addClass(classNameAndAltText[0])
							.attr('title', classNameAndAltText[1]); //$NON-NLS-1$
						cells[STATUS_COL_TITLE].innerHTML = step.name;
						cells[STATUS_COL_START].innerHTML = this.getStepStart(step);
						cells[STATUS_COL_COMPLETE].innerHTML = this.getStepComplete(step);
						cells[STATUS_COL_TIMESPENT].innerHTML = this.getStepTimeSpent(step);
					} else {
						var listRow = dojo.create('tr', { //$NON-NLS-1$
							id: step.id
						}, this.stepsNode);
						dojo.create('img', { //$NON-NLS-1$
							className: classNameAndAltText[0],
							title: classNameAndAltText[1],
							id: step.id + "_image", //$NON-NLS-1$
							href: this._blankGif
						}, dojo.create('td', null, listRow)); //$NON-NLS-1$ //$NON-NLS-2$
						dojo.create('td', { //$NON-NLS-1$
							innerHTML: step.name,
							className: 'stepTitle' //$NON-NLS-1$
						}, listRow);
						dojo.create('td', { //$NON-NLS-1$
							innerHTML: this.getStepStart(step),
							className: 'datetime' //$NON-NLS-1$
						}, listRow);
						dojo.create('td', { //$NON-NLS-1$
							innerHTML: this.getStepComplete(step),
							className: 'datetime' //$NON-NLS-1$
						}, listRow);
						dojo.create('td', { //$NON-NLS-1$
							innerHTML: this.getStepTimeSpent(step),
							className: 'datetime' //$NON-NLS-1$
						}, listRow);
					}
				} catch (e) {
					console.error(e);
				}
			}, this);
			dojo.byId('totaltime').innerHTML = this.getTotalTimeSpent(); //$NON-NLS-1$
		},

		init: function(args) {
			if (this.isUpgradeInProgress()) {
				this.preventNavigation();
				this.checkStatusInterval = window.setInterval(dojo.hitch(this, this.updateStatus), 1000);
			}

			this._drawUpdateSteps();
		}
	});

dojo.mixin(com.ibm.relm.ple.web.ui.pages.UpgradeInProgressPage, {
	checkUpgradeStatus: function() {
		var upgradeInProgress = this.isUpgradeInProgress();
		this.setSuppressedPages(upgradeInProgress);
	},

	isUpgradeInProgress: function() {
		return upgradeStatus && upgradeStatus.overallStatus != STATUS_COMPLETE;
	},

	setSuppressedPages: function(upgradeInProgress) {
		var sp = dojo.getObject("com.ibm.team.repository.web.suppressedPages"); //$NON-NLS-1$
		if (sp) {
			var spIsString = dojo.isString(sp);
			if (spIsString) {
				sp = dojo.fromJson(sp);
			}
			var appId = PlatformUI._getApplicationId();
			var spa = sp[appId];
			if (!spa) {
				sp[appId] = spa = [];
			}
			if (upgradeInProgress) {
				//hard-coding because calls to net.jazz.ajax.ui.internal.PageRegistry.getPages() wasn't working correctly
				var filteredPages = [
					"com.ibm.relm.ple.web.ui.pages.MainWelcomePage", //$NON-NLS-1$
					"com.ibm.relm.ple.web.ui.pages.ProductsWelcomePage", //$NON-NLS-1$
					"com.ibm.relm.explorer.web.ui.page.DiagramPage", //$NON-NLS-1$
					"com.ibm.relm.explorer.web.ui.page.QueryPage", //$NON-NLS-1$
					"dm.app.analysis", //$NON-NLS-1$
					"com.ibm.team.reports" //$NON-NLS-1$
				];
				Array.prototype.push.apply(spa, filteredPages);
				window.location.hash = "action=relm.upgrade_in_progress"; //$NON-NLS-1$
			} else {
				spa.push("com.ibm.relm.ple.web.ui.pages.UpgradeInProgressPage"); //$NON-NLS-1$
			}
			dojo.setObject("com.ibm.team.repository.web.suppressedPages", spIsString ? dojo.toJson(sp) : sp); //$NON-NLS-1$
		}	
	},

	preventNavigation: function() {
		var workbench = net.jazz.ajax.ui.PlatformUI.getWorkbench();
		workbench._processActionUri = workbench.viewDefaultPage = workbench._processResourceUri = function() {
			console.warn("No URL changes will be processed until the upgrade is complete."); //$NON-NLS-1$
		};
	}
});

})();