/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.ConfigurationPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "ConfigurationPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.util.i18n'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$
dojo.require('jazz.ui.RemoteChooserContainer'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.prop.UserPropertyTable'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function ()
{

	var SYSTEM_PROPERTIES = com.ibm.relm.ple.web.ui.prop.UserPropertyTable.SYSTEM_PROPERTIES;
	var Site = com.ibm.relm.ple.web.ui.Site;
	
	/**
	 * Base class for ProductPicker and LqeArtifact picker.
	 * Both show a search box, with a list of results. Only the format
	 * of the results and the query they run is different.
	 *
	 * This class must be extended with doSearch and createListItem overridden.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.ConfigurationPicker', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated],
		{
			widgetsInTemplate: true,
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/picker/templates/ConfigurationPicker.html'), //$NON-NLS-1$ //$NON-NLS-2$
			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "ConfigurationPickerMessages"), //$NON-NLS-1$ //$NON-NLS-2$
			spinnerSrc: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/images/spinner.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			
			/**
			 * The OK callback called when an item in the list
			 * is selected. The passed argument will be the 'item' from
			 * the search results.
			 */
			onOk: null,
			site: null,
			
			userProperties: {},
			
			dialogTitle: '', //$NON-NLS-1$
			dialogIcon: '', //$NON-NLS-1$
	
			constructor: function (args)
			{
				this.itemVersion = args.product;
				this._parentPath = args.parentPath;
				this._parentVersionUri = args.parentVersionUri;
				this._parentViewUri = args.parentViewUri;
				this._parentPartUri = args.parentPartUri;
			},
	
	
			postCreate: function() {
				this.inherited(arguments);
				
				for (var name in this.itemVersion) {
					if (SYSTEM_PROPERTIES[name]) continue;

					this.userProperties[name] = this.itemVersion[name];
				}
				
				this.loadServiceProviderList();
			},
	
			loadServiceProviderList: function()
			{
				var self = this;
				// Init on load of this file. Synchronous, but should be very fast.
				jazz.client.xhrGet({
					url: com.ibm.relm.ple.web.rtc.RtcSettings.cmServiceProvider,
					handleAs: 'xml', //$NON-NLS-1$
					headers: {}, 
					sync: true,

					load: function (results) {
						var serviceProviders = results.getElementsByTagNameNS("http://open-services.net/ns/core#", "serviceProvider"); //$NON-NLS-1$ //$NON-NLS-2$
						self.setupServiceProviders(serviceProviders); //$NON-NLS-2$
					},

					error: function(error, ioArgs) {
						console.error('Could not load the service providers for the configuration picker.', error, ioArgs.xhr.responseText); //$NON-NLS-1$
					}
				});
			},
			
			setupServiceProviders: function(serviceProviders) {

				for (var i=0;i<serviceProviders.length;i++) { 
					var serviceProvider = serviceProviders[i].childNodes[0];
					var titleNode = serviceProvider.getElementsByTagNameNS("http://purl.org/dc/terms/", "title")[0]; //$NON-NLS-1$ //$NON-NLS-2$
					var url = serviceProvider.getAttributeNS("http://www.w3.org/1999/02/22-rdf-syntax-ns#", "about"); //$NON-NLS-1$ //$NON-NLS-2$
					
					var option = dojo.create("option", {value: url, innerHTML: titleNode.textContent}); //$NON-NLS-1$ //$NON-NLS-2$
//					var option = dojo.create("option", {value: url, innerHTML: title, selected: isSelected, defaultSelected: isSelected}); //$NON-NLS-1$ 
					dojo.place(option, this._select, 'last'); //$NON-NLS-1$
				}
			},
			
			uninitialize: function() {
			},
	
			startup: function() {
				this.inherited(arguments);
	
				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: '625px', //$NON-NLS-1$
					height: '555px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: this.iconClass
				});
				
				// Just looks nicer since there is a big white result list--easier to distinguish
				// the dialog parts.
				dojo.addClass(this._dialog.domNode, 'relm-dialog-gray'); //$NON-NLS-1$
	
				
			},
			
			_onData: function(data){
				console.log("_onData() data", data); //$NON-NLS-1$
				
				if (data.length !== 0) {
	                var response = dojo.fromJson(data);
	                var results = response["oslc:results"]; //$NON-NLS-1$
	                if (results.length === 0)
	                	this.cancel();
	                else
	                {
	                	var resource = results[0];
//	                	this._createProductWithConfiguration(resource["oslc:label"], resource["rdf:resource"]); //$NON-NLS-2$ //$NON-NLS-1$
	                	this._associateConfigurationWithProduct(resource["oslc:label"], resource["rdf:resource"]); //$NON-NLS-2$ //$NON-NLS-1$
	                }
	            }
	        },
	        
	        _showDefaultFrame: function()
	        {
	        	this._remoteLowerFrame.style.display = 'none'; //$NON-NLS-1$
				this._progressLowerFrame.style.display = 'none'; //$NON-NLS-1$
				this._defaultLowerFrame.style.display = 'block'; //$NON-NLS-1$
	        },
	        
	        _showProgressFrame: function()
	        {
	        	this._remoteLowerFrame.style.display = 'none'; //$NON-NLS-1$
				this._progressLowerFrame.style.display = 'block'; //$NON-NLS-1$
				this._defaultLowerFrame.style.display = 'none'; //$NON-NLS-1$
	        },
	        
	        _showRemoteFrame: function()
	        {
	        	this._defaultLowerFrame.style.display = 'none'; //$NON-NLS-1$
				this._progressLowerFrame.style.display = 'none'; //$NON-NLS-1$
				this._remoteLowerFrame.style.display = 'block'; //$NON-NLS-1$
	        },
	        
	        _associateConfigurationWithProduct: function(configurationLabel, configurationUri)
	        {
	        	this._messageBox.hideMessage();
	        	
				this._progressLowerFrameText.innerHTML = (dojo.replace(this.messages.associateConfigurationX, [configurationLabel]));
				this._showProgressFrame();
				
				var self = this;
				
				self.userProperties['http://jazz.net/ns/vvc#configuration_fake'] = configurationUri; //$NON-NLS-1$ //$NON-NLS-2$

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/update_properties', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						site: Site.siteToJsonSite(this.site),
						saveProperties: dojo.toJson(self.userProperties)
					},

					handle: function() {
						// message.hide();
					},

					load: dojo.hitch(this, function(newItemVersion) {
						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
							selectPath: this.site.path[0],
							flashPath: true
						}).addBoth(dojo.hitch(this, function() {
							self.cancel();
						}));
					}),

					error: dojo.hitch(this, function(error, ioArgs) {
						self._messageBox.showXhrError(ioArgs.xhr);
						self._updateLowerFrame();
					})
				});
	        },

			_onChange: function(e) {
				this._updateLowerFrame();
			},
			
			_updateLowerFrame: function()
			{
				var i = this._select.selectedIndex;
				if (i === -1) return;
				var option = this._select.options[i];

				if (this.remotePicker)
				{
					this.remotePicker.destroyRecursive();
					this.remotePicker = null;
				}
				if (option.value === 'default') { //$NON-NLS-1$
					this._showDefaultFrame();
				}
				else
				{
					
					var dialogDetails = this._fetchDialogDetails(option.value);
					
					var oslcDialogNode = dialogDetails.getElementsByTagNameNS("http://open-services.net/ns/core#", "dialog")[0]; //$NON-NLS-1$ //$NON-NLS-2$
					console.log('oslcDialogNode', oslcDialogNode); //$NON-NLS-1$
					var pickerUrl = oslcDialogNode.getAttributeNS("http://www.w3.org/1999/02/22-rdf-syntax-ns#", "resource");//$NON-NLS-1$ //$NON-NLS-2$
					var hintWidth = dialogDetails.getElementsByTagNameNS("http://open-services.net/ns/core#", "hintWidth")[0];//$NON-NLS-1$ //$NON-NLS-2$
					var hintHeight = dialogDetails.getElementsByTagNameNS("http://open-services.net/ns/core#", "hintHeight")[0];//$NON-NLS-1$ //$NON-NLS-2$
					
					this._remoteLowerFrame.style.width = hintWidth.textContent;
					this._remoteLowerFrame.style.height = hintHeight.textContent;
					
					if (pickerUrl.indexOf("?") != -1) //$NON-NLS-1$
						pickerUrl = pickerUrl+"&debug=true"; //$NON-NLS-1$
					else
						pickerUrl = pickerUrl+"?debug=true"; //$NON-NLS-1$
					
					this.remotePicker = new jazz.ui.RemoteChooserContainer({  
//		            	remoteLocation: "https://pcheng:9445/vvc/_ajax-modules/workspacePicker?debug=true&contextUri=https%3A%2F%2Fpcheng%3A9445%2Fvvc%2Fspaces%2F1_relm-20140203_153545",//$NON-NLS-1$
						remoteLocation: pickerUrl,  //$NON-NLS-1$
		            	mode: 'OSLC2', //$NON-NLS-1$
		            	onData: dojo.hitch(this, this._onData),
		            	onResize: dojo.hitch(this, this._onResize)
		            });
					this.remotePicker.iframe.style.height = "100%"; //$NON-NLS-1$
//					this.remotePicker.iframe.scrolling ='no'; //$NON-NLS-1$
					dojo.place(this.remotePicker.domNode, this._remoteLowerFrame); //$NON-NLS-1$
					this._showRemoteFrame();
				}
			},
			
			_fetchDialogDetails: function(serviceProviderUrl)
			{
				var self = this;
				var dialogDetails = null;
				// Init on load of this file. Synchronous, but should be very fast.
				jazz.client.xhrGet({
					url: serviceProviderUrl,
					handleAs: 'xml', //$NON-NLS-1$
					headers: {}, 
					sync: true,

					load: function (results) {
						console.log('results', results); //$NON-NLS-1$
						dialogDetails = results;
					},

					error: function(error, ioArgs) {
						console.error('Could not load the details of the oslc dialog.', error, ioArgs.xhr.responseText); //$NON-NLS-1$
					}
				});
				return dialogDetails;
			},
			
			cancel: function() {
				this._dialog.close();
				this._dialog.destroyRecursive();
				delete this._dialog;
			},
			
			_onResize: function(data) {
				console.log("_onResize() data", data); //$NON-NLS-1$
				if (data.length) {
	                var response = dojo.fromJson(data);
	                //TODO: Do the resize
	            }
			}
		}
	);
})();