dojo.provide('com.ibm.relm.ple.web.ui.picker.FilterList'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function ()
{
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.FilterListToken', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], 
		{
			widgetsInTemplate: false,
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/FilterListToken.html'), //$NON-NLS-1$ //$NON-NLS-2$ 
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			label: null,
			value: null,
			iconSrc: null,
			parentList: null,

			constructor: function(args) {
				this.checkboxId = dijit.getUniqueId('ple-FilterList'); //$NON-NLS-1$
			},

			postCreate: function()
			{
				this.set('enabled', this.enabled); //$NON-NLS-1$
				this.labelNode.innerHTML = this.label;
			},

			_getEnabledAttr: function() {
				return this.checkbox.checked;
			},

			_setEnabledAttr: function(on) {
				if (on) {
					this.checkbox.checked = true;
					dojo.attr(this.domNode, 'title', this.messages.hideResults); //$NON-NLS-1$
				}
				else {
					this.checkbox.checked = false;
					dojo.attr(this.domNode, 'title', this.messages.showResults); //$NON-NLS-1$
				}
			},

			updateState: function() {
				if (this.checkbox.checked) {
					dojo.attr(this.domNode, 'title', this.messages.hideResults); //$NON-NLS-1$
				}
				else {
					dojo.attr(this.domNode, 'title', this.messages.showResults); //$NON-NLS-1$
				}

				this.parentList.toggleApply(true);
			},

			_onClick: function (e)
			{
				this.updateState();
			}
		}
	);
	
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.FilterList', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/FilterList.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			_updatingImgSrc: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/updating.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'FilterListMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			label: '', //$NON-NLS-1$

			/**
			 * Set to true when showing in the LQE artifact picker in a popup menu.
			 * This changes some event behavior to get consistent cross browser keyboard handling
			 * when tabbing through checkboxes. Mostly a hack around some strange Chrome behavior.
			 * @type {Boolean}
			 */
			inPopup: false,

			filteredTypesCookieName: '', //$NON-NLS-1$

			constructor: function()
			{
				this.labelId = dijit.getUniqueId('ple-FilterList'); //$NON-NLS-1$
				this._tokens = [];
				this.label = this.messages.filtersLabel;
				this._lastEnabledStates = {};
			},
				
			postCreate: function() {
				this.connect(this.domNode, 'onmouseleave', this._focusOnApplyIfNeeded); //$NON-NLS-1$

				// Insanity: Using spacebar to click a checkbox
				// is not working in Chrome when this widget is in a popup (shift+space works!).
				// This appears to be a subtle Chrome bug. The only way to fix it and 
				// not break IE is to essnetially implements that defautlt behavior here.
				// If there is a SPACE or ENTER keyup on a check box, we'll stop that
				// event and do the toggling logic ourselves.
				if (this.inPopup) {
					this.connect(this._toggleArea, 'onkeyup', function(e) { //$NON-NLS-1$
						if (dojo.attr(e.target, 'type') !== 'checkbox') return; //$NON-NLS-1$ //$NON-NLS-2$

						var key = e.keyCode;

						switch (key) {
							case dojo.keys.SPACE:
							case dojo.keys.ENTER:
								dojo.stopEvent(e);
								var token = dijit.getEnclosingWidget(e.target);
								var currentEnabled = token.get('enabled'); //$NON-NLS-1$
								token.set('enabled', !currentEnabled); //$NON-NLS-1$
								token.updateState();
								return;

							default:
								return;
						}
					});
				}
			},

			_setEnabledAttr: function(enabled) {
				this._set('enabled', enabled); //$NON-NLS-1$

				for (var i = 0; i < this._tokens.length; i++) {
					var token = this._tokens[i];
					token.checkbox.disabled = !enabled;
				}
			},

			getCounts: function() {
				var counts = {
					total: 0,
					enabled: 0,
					enabledLabels: [],
					disabledLabels: []
				};

				for (var i = 0; i < this._tokens.length; i++) {
					var t = this._tokens[i];
					counts.total++;
					if (t.get('enabled')) { //$NON-NLS-1$
						counts.enabled++;
						counts.enabledLabels.push(t.get('label')); //$NON-NLS-1$
					}
					else {
						counts.disabledLabels.push(t.get('label')); //$NON-NLS-1$
					}
				}

				return counts;
			},

			toggleUpdating: function(show) {
				clearTimeout(this._updatingTimeout);

				var self = this;
				// this._updatingNode.style.display = show ? '' : 'none'; //$NON-NLS-1$ //$NON-NLS-2$
				if (show) {
					// Delay showing this a bit to avoid it flashing when the
					// results come back very quickly.
					this._updatingTimeout = setTimeout(function() {
						self._updatingNode.style.display = ''; //$NON-NLS-1$
					}, 300);
				}
				else {
					this._updatingNode.style.display = 'none'; //$NON-NLS-1$
				}
			},

			addToken: function(label, value, iconSrc, enabled) {
				var token = new com.ibm.relm.ple.web.ui.picker.FilterListToken({
					label: label,
					value: value,
					enabled: enabled,
					iconSrc: iconSrc,
					parentList: this
				});
				
				this._lastEnabledStates[value] = enabled;
				this._tokens.push(token);
				this._toggleArea.appendChild(token.domNode);

				this._clearButton.set('enabled', true); //$NON-NLS-1$


				return token;
			},

			updateTokens: function(tokenInfos) {
				var oldTokenSet = {};
				dojo.forEach(this._tokens, function(token) {
					oldTokenSet[token.value] = true;
				});

				this.clearTokens();

				function fadeInToken(token) {
					dojo.animateProperty({
						node: token.domNode,
						duration: 200,
						properties: {
							height: {
								start: 0,
								end: 17
							},
							opacity: {
								start: 0,
								end: 1
							}
						},
						onEnd: function() {
							// reset to the real height
							token.domNode.style.height = 'auto'; //$NON-NLS-1$
						}
					}).play();
				}

				for (var i = 0; i < tokenInfos.length; i++) {
					var info = tokenInfos[i];
					var token = this.addToken(info.label, info.value, info.iconSrc, info.enabled);

					var isNewToken = !oldTokenSet[info.value];

					if (isNewToken) {
						fadeInToken(token);
					}
				}
			},

			clearTokens: function()
			{
				for (var i = 0; i < this._tokens.length; i++)
				{
					var token = this._tokens[i];
					token.destroyRecursive();
				}
				
				this._tokens.length = 0;
				this.toggleApply(false);

				this._clearButton.set('enabled', false); //$NON-NLS-1$
			},

			/**
			 * To keep this list from closing when it's in a drop down, force
			 * the tabbing keyboard focus to wrap around to the top of the list
			 * instead of moving on to another widget. Esc can still be used to close
			 * this, or tabbing to a button an Entering.
			 * @param  {key event} e 
			 */
			interceptTabKey: function(e) {
				// Ignore on reverse tabbing
				if (e.shiftkey) return;

				if (e.keyCode === dojo.keys.TAB) {
					this._clearButton.focus();
					dojo.stopEvent(e);
				}
			},

			revertFilters: function() {
				for (var i = 0; i < this._tokens.length; i++)
				{
					var token = this._tokens[i];
					var value = token.get('value'); //$NON-NLS-1$
					if (value === undefined) continue;
					var enabled = this._lastEnabledStates[value];
					token.set('enabled', enabled); //$NON-NLS-1$
				}

				this.toggleApply(false);
				this.onRevert();
			},

			applyFilters: function() {
				this.onChange();
				this.toggleApply(false);
			},

			_focusOnApplyIfNeeded: function() {
				var isVisible = dojo.hasClass(this._applyArea, 'ple-FilterList-applyAreaVisible'); //$NON-NLS-1$
				if (this._needsFocusOnApply && !isVisible) {
					delete this._needsFocusOnApply;
					this._applyButton.focus();
				}
			},

			/**
			 * Move keyboard focus to the toggle/body area of the filter list.
			 * This is done when a filter list is shown in a popup control and
			 * lets the user click TAB once to get to the first type checkbox.
			 * The checkbox is not focused directly since this is called in response
			 * to keyboard events, and the checkbox might end up seeing a 'onkeyup'
			 * after being focused on, which could accidentally change its state.
			 *
			 * The visual focus outline is removed since the body can't be interacted with,
			 * we just want a better TAB order.
			 */
			focus: function() {
				this._toggleArea.focus();
			},

			toggleApply: function(show) {
				this._applyButton.disabled = !show;

				if (this._tokens.length < 1) {
					dojo.removeClass(this._applyArea, 'ple-FilterList-applyAreaVisible'); //$NON-NLS-1$
					return;
				}

				if (show) {
					var isVisible = dojo.hasClass(this._applyArea, 'ple-FilterList-applyAreaVisible'); //$NON-NLS-1$
					if (isVisible) {
						return;
					}

					this._needsFocusOnApply = true;
					dojo.addClass(this._applyArea, 'ple-FilterList-applyAreaVisible'); //$NON-NLS-1$
				}
				else {
					dojo.removeClass(this._applyArea, 'ple-FilterList-applyAreaVisible'); //$NON-NLS-1$
				}
			},

			onChange: function() {
				// callback when filters are changed and applied
			},

			onRevert: function() {
				// callback when close/revert is click
			},

			getFilters: function()
			{
				var filters = [];
				for (var i = 0; i < this._tokens.length; i++)
				{
					var t = this._tokens[i];
					filters.push({
						value: t.value,
						enabled: t.get('enabled') //$NON-NLS-1$
					});
				}

				return filters;
			},


			_clearAll: function() {
				for (var i = 0; i < this._tokens.length; i++)
				{
					var t = this._tokens[i];
					t.set('enabled', false); //$NON-NLS-1$
				}

				this.toggleApply(true);
			},

			/**
			 * This takes the currently disabled filters the user can see, as well
			 * as the ones saved in the cookie (or localStorage) and returns
			 * a marged array of their artifact types only (just string, not objects).
			 * The artifact type is a marshalled verison of the type URI + dcterms:type.
			 * This is used by the server to filter types out of the results.
			 * @return {array} array of strings. The artifact types to exclude.
			 */
			mergeCurrentAndSavedFilters: function()
			{
				var currentFilters = this.getFilters();

				if (!this.filteredTypesCookieName) {
					console.warn('Requesting saved type filter, but did not specify cookie name'); //$NON-NLS-1$
				}

				// Restore saved filter if there are any. We won't show them
				// until after the search run, but the will still affect the results.
				// We don't want to show filter for types that aren't found
				// by the search, so we'll only show the bar after the search runes.
				var filteredSet = {};
				
				var savedTypes = dojo.cookie(this.filteredTypesCookieName);
				if (savedTypes) {
					var saved = dojo.fromJson(savedTypes);
					for (var i = 0; i < saved.length; i++)
					{
						filteredSet[saved[i]] = true;
					}
				}

				for (i = 0; i < currentFilters.length; i++)
				{
					var current = currentFilters[i];
					if (current.enabled) {
						delete filteredSet[current.value];
					}
					else {
						filteredSet[current.value] = true;
					}
				}
				
				var filtered = [];
				for (var value in filteredSet) {
					filtered.push(value);
				}

				return filtered;
			},

			uninitialize: function()
			{
				this.clearTokens();
				this.inherited(arguments);
			}
		}
	);

})();
