/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.OslcPreview'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'OslcPreviewMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

(function(){
	var gator_ = jazz.ui.internal.resourceLinkAggreagtor();

	// TODO: code sharing with artifact view
	
	/**
	 * An HTML preview of a resource that supports OSLC or compact XML
	 * previews
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.OslcPreview', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/OslcPreview.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'OslcPreviewMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * The resource to preview. Can set this at any time to update the
			 * preview.
			 */
			resourceUri: '', //$NON-NLS-1$

			postCreate: function() {
				this.inherited(arguments);
				this._loadingMessage.hide();
				this._loadingMessage.positionAbsolute('10px'); //$NON-NLS-1$
				this._hintNode.style.display = ''; //$NON-NLS-1$
			},

			/**
			 * Updates the size of the iframe to match the current set height of this
			 * widget minus the header row which doesn't scroll.
			 */
			_resizeIframe: function()  {
				var headerHeightPx = dojo.marginBox(this._header).h;
				var iframeHeight = this.heightPx - headerHeightPx;
				if (this._iframe) {
					dojo.style(this._iframe, {
						display: '', //$NON-NLS-1$
						height: iframeHeight + 'px' //$NON-NLS-1$
					});
				}
				
				// If there's a local preview (DOORS baseline without an OSLC preview)
				// it occupies the same space.
				this._localPreview.style.height = iframeHeight + 'px'; //$NON-NLS-1$
			},

			/**
			 * Used to set the height of this preview. This also adjust the embedded iframe so that it
			 * is not clipped and will show it scroll bars. Can only be called after .postCreate
			 * when this widget is in the DOM.
			 * @param {number} heightPx height in pixels to be.
			 */
			_setHeightPxAttr: function(heightPx) {
				this.domNode.style.height = heightPx + 'px'; //$NON-NLS-1$
				this.heightPx = heightPx;
				this._resizeIframe();
			},

			_noPreview: function(error) {
				// check that we weren't unloaded
				if (!this._loadingMessage) return;

				this._header.style.display = 'none'; //$NON-NLS-1$
				this._loadingMessage.hide();
				this._hintNode.style.display = 'none'; //$NON-NLS-1$

				this._messageBox.showErrorObject({
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: this.messages.noPreview,
					explanation: error.message,
					userAction: this.messages.noPreview$useraction
				});
			},

			_doLogin: function(e) {
				dojo.stopEvent(e);

				// jazz.client is smart enough to inercept this request and do the login for us.
				// Just need to refresh previews once we're done.
				jazz.client.xhrGet({
						url: this.resourceUri,
						headers: {
							accept: "application/x-oslc-compact+xml" //$NON-NLS-1$
						},
					
						load: dojo.hitch(this, function (result) {
							gator_.resendOauth(this._oauthToken);
						}),
					
						error: dojo.hitch(this, function(error, ioArgs) {
							this._messageBox.showErrorObject({
								type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								summary: this.messages.artifactLoginFailed,
								explanation: this.messages.artifactLoginFailed$explanation,
								userAction: dojo.replace(this.messages.artifactLoginFailed$useraction, [this.artifactUri])
							});
						})
					});
			},

			_updatePreview: function(expectedUri, result) {
				if (this.resourceUri !== expectedUri) return;
				// check that we weren't destroyed while waiting
				if (!this._loadingMessage) return;

				this._hintNode.style.display = 'none'; //$NON-NLS-1$

				if (result.oauthRequired) {
					this._loadingMessage.hide();
					dojo.empty(this._loginNode);
					dojo.destroy(this._iframe);
					var loginLink = jazz.util.internal.login.insertLoginLink(this._loginNode, result.token);
					this._oauthToken = result.token;
					this.connect(loginLink, 'onclick', this._doLogin); //$NON-NLS-1$
					this._loginNode.style.display = ''; //$NON-NLS-1$
					return;
				}

				this._messageBox.hideMessage();
				this._loginNode.style.display = 'none'; //$NON-NLS-1$

				var title;
				var icon;
				var preview;

				if (result.oslcCompact) {
					title = result.oslcCompact.title || result.oslcCompact.shortTitle;
					icon = result.oslcCompact.icon;
					var previewData = result.oslcCompact.largePreview || result.oslcCompact.smallPreview;
					preview = previewData.document;
				}
				else if (result.compactRendering) {
					title = result.compactRendering.title || result.compactRendering.shortTitle;
					icon = result.compactRendering.icon;
					preview = result.compactRendering.largePreview || result.compactRendering.smallPreview;
				}
				else {
					console.warn('Do not know how to render this preview style:', result); //$NON-NLS-1$
					return;
				}

				this._titleNode.innerHTML = jazz.util.html.removeScript(title);
				this._titleNode.href = this.resourceUri;
				this._iconNode.src = icon;

				var self = this;
				if (preview) {
					dojo.destroy(this._iframe);
					this._iframe = dojo.create('iframe', { //$NON-NLS-1$
						src: preview,
						frameBorder: 0,
						style: 'display: none;', //$NON-NLS-1$
						'title': this.messages.artifactPreview, //$NON-NLS-1$
						className: 'ple-ResourceView-richPreview' //$NON-NLS-1$
					}, this._iframePreview);

					if (dojo.isIE) {
						// IE has problems with onload and iframes. There's a way to get this to work
						// but I'm just hiding early for now.
						this._loadingMessage.hide();
						this._resizeIframe();
						this._header.style.display = ''; //$NON-NLS-1$
					}
					else {
						// Wait until the iframe loads before hiding the spinner since that can take a while.
						this.connect(this._iframe, 'onload', function() { //$NON-NLS-1$
							self._header.style.display = ''; //$NON-NLS-1$
							self._resizeIframe();
							self._loadingMessage.hide();
						});
					}
				}
				else {
					this._loadingMessage.hide();
				}
			},

			/**
			 * Instead of an OSLC preview from a URL, this allows
			 * a preview to be constructed locally in JavaScript and shown
			 * as the preview. This was added to add previews for DOORS module
			 * baselines, since they don't have an OSLC preview.
			 * @param  {string} resourceUri URI of the resource. Used to provide a link to the resource.
			 * @param  {string} titleHtml   human readable title for the link in the header.
			 * @param  {string} iconSrc     URI to the icon of the resource to display.
			 * @return {node}             returns the DOM node used for the local preview. The caller
			 *                                    must empty and fill this node with preview HTML.
			 */
			showLocalPreview: function(resourceUri, titleHtml, iconSrc) {
				this._titleNode.href = resourceUri;
				this._titleNode.innerHTML = titleHtml;
				this._iconNode.src = iconSrc;

				this._iframePreview.style.display = 'none'; //$NON-NLS-1$
				this._localPreview.style.display = ''; //$NON-NLS-1$

				this._messageBox.hideMessage();
				this._loadingMessage.hide();
				this._hintNode.style.display = 'none'; //$NON-NLS-1$
				dojo.empty(this._loginNode);

				// show the header and resize around it
				this._header.style.display = ''; //$NON-NLS-1$
				this._resizeIframe();

				return this._localPreview;
			},

			_setResourceUriAttr: function(resourceUri) {
				if (this.resourceUri === resourceUri) return;

				this._iframePreview.style.display = ''; //$NON-NLS-1$
				this._localPreview.style.display = 'none'; //$NON-NLS-1$

				var oldIframe = this._iframe;
				if (oldIframe) {
					var self = this;
					dojo.animateProperty({
						node: this._iframe,
						properties: {
							opacity: 0
						},
						duration: 1000,
						onEnd: function() {
							if (self._iframe == oldIframe) {
								dojo.destroy(self._iframe);
								delete self._iframe;
							}
						}
					}).play();
					
				}
				dojo.destroy(this._iframe);

				//check that we weren't destroyed before we try to show or hide things
				if (!this || !this.domNode) { return; }
				
				this._messageBox.hideMessage();
				this._header.style.display = 'none'; //$NON-NLS-1$
				dojo.empty(this._loginNode);

				this.resourceUri = resourceUri;

				if (!this.resourceUri) {
					this._loadingMessage.hide();
					this._hintNode.style.display = ''; //$NON-NLS-1$
				}
				else {
					this._loadingMessage.show(200);
					this._hintNode.style.display = 'none'; //$NON-NLS-1$
					gator_.register(this.resourceUri, dojo.hitch(this, this._updatePreview, resourceUri), dojo.hitch(this, this._noPreview));
				}

			},

			uninitialize: function() {
				this.inherited(arguments);
			}
		}
	);

})();