/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.ProductPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "ProductPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.explorer.web.explorer.page', 'SearchResultsMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.PickerResult'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function ()
{
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var PickerResult = com.ibm.relm.ple.web.ui.picker.PickerResult;
	var Site = com.ibm.relm.ple.web.ui.Site;

	var LOCAL_STORAGE_SCOPE_KEY = 'ple.productPicker.scope'; //$NON-NLS-1$

	/**
	 * Product specific picker dialog.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.ProductPicker', //$NON-NLS-1$
		[com.ibm.relm.ple.web.ui.picker.BasePicker],
		{
			prevXhr: null,

			/**
			 * The currently selected product when this dialog was created.
			 * The chosen product will be added to this.
			 * Pass to constructor.
			 */
			selected: null,

			/**
			 * Set to true to pick product lines, not particular product versions.
			 * Used in dimensions since dimensions apply to all versions
			 * of a product.
			 */
			collapseVersions: false,

			_messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'ProductPickerMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function (args)
			{
				this.dialogTitle = args.dialogTitle || this._messages.dialogTitle;
				this.iconClass = args.iconClass || 'ple-icon-addProduct'; //$NON-NLS-1$
				this.searchLabel = this._messages.resourcePickerHelp; //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);

				if (this.collapseVersions) {
					this.liveSearchBox.set('placeholder', this._messages.typeToSearchProduct); //$NON-NLS-1$
				}
				else  {
					// We don't use this here, and it ends up taking up whitespace.
					// dojo.destroy(this.searchOptionsNode);
					dojo.addClass(this.searchBarNode, 'r-margin-tb'); //$NON-NLS-1$

					var newestLabel = dojo.create('label', { //$NON-NLS-1$
						className: 'j-label', //$NON-NLS-1$
						title: this._messages.showNewestTooltip
					}, this.searchOptionsNode);

					this._newestCheckbox = dojo.create('input', { //$NON-NLS-1$
						type: 'checkbox', //$NON-NLS-1$
						checked: true
					}, newestLabel);

					newestLabel.appendChild(document.createTextNode(' ' + this._messages.showNewestCheckbox)); //$NON-NLS-1$

					this.connect(this._newestCheckbox, 'onclick', function() { //$NON-NLS-1$
						this.liveSearchBox.runSearch();
					});
				}
			},

			onOk: function(resource){
				var message = com.ibm.relm.common.web.StatusMessage.show(this._messages.dialogTitle);
				this.submitButton.disabled = this.cancelButton.disabled = true;
				this.resultList.set('enabled', false); //$NON-NLS-1$

				var self = this;

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/add_resource_version', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						addResourceVersionUri: resource.uri,
						toSite: Site.siteToJsonSite(this.selected)
					},

					handle: function() {
						message.hide();
					},

					load: function(result) {
						var newPath = self.selected.path.concat(result.pathSegment);

						TreeRefresh.refresh({
							selectParentPath: newPath,
							flashPath: true
						}).addBoth(function() {
							self.close();
						});
					},

					error: function(error, ioArgs) {
						TreeRefresh.refresh();
						self.resultList.set('enabled', true); //$NON-NLS-1$
						self.submitButton.disabled = self.cancelButton.disabled = false;
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});
			},

			createListItem: function(item) {
				var text = this.collapseVersions ?
					formatting.formatProductItemTitleForItem(item) :
					formatting.formatProductTitleForItem(item);

				var iconInfo = formatting.getIconClassAndAltTextForItem(item);

				var pickerItem = {
					text: text,
					iconClass: iconInfo[0],
					artifactType: iconInfo[1],
					description: item.hitSummary,
					contributorName: item.creatorName,
					modifiedTime: item.modifiedMillis, //$NON-NLS-1$
					createdTime: item.createdMillis //$NON-NLS-1$
				};
				
				// XXX: Only safe since StyledList will use dijit.findWidgets to .destroy the PickerResult
				return new PickerResult({item: pickerItem}).domNode;
			},

			doSearch: function(searchString) {
				var deferred = new dojo.Deferred();

				var newestOnly = !!this._newestCheckbox && this._newestCheckbox.checked;
				var scopeId = newestOnly ? 'newestProductsAndConfigurations' : 'allProductsAndConfigurations'; //$NON-NLS-1$ //$NON-NLS-2$

				if(this.prevXhr){
					this.prevXhr.cancel();
				}

				var self = this;
				this.prevXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/product_search', //$NON-NLS-1$
					content: {
						q: searchString,
						scope: scopeId,
						collapseVersions: this.collapseVersions
					},

					load: function(searchResults) {
						var hits = searchResults.hits;
						self.showResults(hits);
					},

					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback.
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }

						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					}
				});

				return deferred;
			}
		}

	);

	var searchMessages = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.explorer.page', 'SearchResultsMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.mixin(
		com.ibm.relm.ple.web.ui.picker.ProductPicker.prototype.messages,
		searchMessages);

})();
