/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.RtcServerSelect'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'RtcServerSelectMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$

dojo.require('jazz.util.html'); //$NON-NLS-1$

(function(){
	var RtcSettings = com.ibm.relm.ple.web.rtc.RtcSettings;
	var LAST_SERVER_KEY = 'ple.RtcServerSelect.lastServer'; //$NON-NLS-1$

	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.RtcServerSelect', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/RtcServerSelect.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'RtcServerSelectMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function(args, node)
			{
				this._lastSelectedIndex = 0;	
			},

			postCreate: function() {
				this.inherited(arguments);

				var preferredServer = window.localStorage[LAST_SERVER_KEY];
				
				var foundPreferred = false;
				
				var html = jazz.util.html;

				for (var i = 0; i < RtcSettings.rtcUrlsWithName.length; i++) {
					var urlWithName = RtcSettings.rtcUrlsWithName[i];
					
					var isSelected = false;
					if (urlWithName.url === preferredServer) {
						isSelected = true;
						foundPreferred = true;
						this._lastSelectedIndex = i;						
					}
					
					var option = dojo.create('option', { //$NON-NLS-1$
						value: urlWithName.url,
						innerHTML: html.escape(urlWithName.name),
						selected: isSelected,
						defaultSelected: isSelected
					}, this._select); 
				}

				// Manage option
				if (relm.app.Permissions.isJazzAdmin()) {
					dojo.create('option', { //$NON-NLS-1$
						value: 'edit', //$NON-NLS-1$
						innerHTML: html.escape(this.messages.edit)
					}, this._select);
				}

				if (!foundPreferred) {
					this._select.selectedIndex = 0;
				}
				this.updateServerLabel();
			},

			updateServerLabel: function()
			{
				var i = this._select.selectedIndex;
				if (i === -1) 
					return;
				var option = this._select.options[i];
				
				if (option.value === 'edit') //$NON-NLS-1$
					this._serverUrl.innerHTML = ""; //$NON-NLS-1$
				else
					this._serverUrl.innerHTML = option.value;
			},
			
			/**
			 * Callack when changed.
			 */
			onChange: function() {
				
			},

			getServerUrl: function() {
				var i = this._select.selectedIndex;
				if (i === -1) return null;
				
				var option = this._select.options[i];
				return option.value;
			},

			_onChange: function(e) {
				var i = this._select.selectedIndex;
				if (i === -1) return;
				var option = this._select.options[i];

				if (option.value === 'edit') { //$NON-NLS-1$
					this._select.selectedIndex = this._lastSelectedIndex;
					window.location = net.jazz.ajax._contextRoot + '/admin#action=relm.admin.stream_management'; //$NON-NLS-1$
					return;
				}

				this._lastSelectedIndex = i;
				window.localStorage[LAST_SERVER_KEY] = option.value;
				
				this.updateServerLabel();
				this.onChange();
			}
		}
	);

})();