/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.RtcStreamComponentPicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "RtcStreamComponentPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.BasePicker'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.productapi"); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.StreamComponentRadios'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.picker.RtcServerSelect'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.RtcSettings'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function ()
{
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var Site = com.ibm.relm.ple.web.ui.Site;

	/**
	 * Product specific picker dialog.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.RtcStreamComponentPicker', //$NON-NLS-1$
		[com.ibm.relm.ple.web.ui.picker.BasePicker],
		{
			rtcMessages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'RtcStreamComponentPickerMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			prevXhr: null,
			subtractHeight: 300,
			minResultHeight: 50,

			site: null,

			constructor: function (args)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "ProductPickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				this.dialogTitle = this.rtcMessages.connectToStreamOrBaseline;
				this.iconClass = 'ple-icon-rtc'; //$NON-NLS-1$
				this.searchLabel = this.rtcMessages.searchStreamsOrBaselinesByKeyword; 
				this._resourceType = 'stream'; //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);

				if (com.ibm.relm.ple.web.rtc.RtcSettings.showConnectToComponent) {
					var radios = new com.ibm.relm.ple.web.ui.picker.StreamComponentRadios();

					radios.placeAt(this.searchOptionsNode);
					var self = this;
					this.connect(radios, 'onChange', function(newValue) { //$NON-NLS-1$
						self._resourceType = newValue;
						self.liveSearchBox.runSearch();
					});
				}

				this.submitButton.innerHTML = this.rtcMessages.connect;

				var serverSelectWrapper = dojo.create('div', {}, this._content, 'first'); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(serverSelectWrapper, {
					'float': 'right', //$NON-NLS-1$ //$NON-NLS-2$
					'width': '40%' //$NON-NLS-1$ //$NON-NLS-2$
				});

				this._serverSelect = new com.ibm.relm.ple.web.ui.picker.RtcServerSelect();
				this.own(this._serverSelect);
				this._serverSelect.placeAt(serverSelectWrapper); //$NON-NLS-1$

				dojo.style(this.searchBarNode, {
					'float': 'left', //$NON-NLS-1$ //$NON-NLS-2$
					'width': '60%' //$NON-NLS-1$ //$NON-NLS-2$
				});

				dojo.style(this._stackedListsNode, 'clear', 'both'); //$NON-NLS-1$ //$NON-NLS-2$

				this.resultList.set('htmlPlaceholder', this.rtcMessages.searchInstructions); //$NON-NLS-1$

				this.connect(this._serverSelect, 'onChange', this._onServerChange); //$NON-NLS-1$
			},

			clearResults: function() {
				this.resultList.set('htmlPlaceholder', this.rtcMessages.searchInstructions); //$NON-NLS-1$
				this.inherited(arguments);
			},

			_onServerChange: function() {
				this.liveSearchBox.runSearch();
			},

			onOk: function(streamResource){
				this.submitButton.disabled = this.cancelButton.disabled = true;
				this.resultList.set('enabled', false); //$NON-NLS-1$

				var statusMessage = com.ibm.relm.common.web.StatusMessage.show('Connecting to RTC...'); //$NON-NLS-1$

				var self = this;

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/connect_to_rtc', //$NON-NLS-1$
					content: {
						site: Site.siteToJsonSite(this.site),
						name: streamResource.name,
						streamUuid: streamResource.streamUuid,
						baselineUuid: streamResource.baselineUuid,
						componentUuid: streamResource.componentUuid,
						componentName: streamResource.componentName,
						rtcServer: streamResource.rtcServer
					},

					handle: function() {
						statusMessage.hide();
					},

					load: function () {
						TreeRefresh.refresh({
							selectPath: self.site.path
						})
						.then(function() {
							self.close();
						});
					},

					error: function(error, ioArgs) {
						TreeRefresh.refresh({
							selectPath: self.site.path
						});
						self.resultList.set('enabled', true); //$NON-NLS-1$
						self.submitButton.disabled = self.cancelButton.disabled = false;
						self.messageBox.showXhrError(ioArgs.xhr, true);
						var specialLink = dojo.query('a[data-relm-duplicateStream]')[0]; //$NON-NLS-1$
						if (specialLink) {
							self.connect(specialLink, 'onclick', function(e) { //$NON-NLS-1$
								dojo.stopEvent(e);
								var duplicateStreamProductUri = dojo.attr(specialLink, 'data-relm-duplicateStream'); //$NON-NLS-1$
								self.close();
								productapi.findPathsToProductAndShow(duplicateStreamProductUri);
							});
 						}
					}
				});
			},

			createListItem: function(item) {
				var li = dojo.create('LI', { //$NON-NLS-1$
					className: 'ple-LqeArtifactPicker-row' //$NON-NLS-1$
				});

				var iconClass = item['type'].toLowerCase() === 'workspace' ? //$NON-NLS-1$ //$NON-NLS-2$
					'ple-icon-rtcStream' : //$NON-NLS-1$
					'ple-icon-rtcComponent'; //$NON-NLS-1$

				dojo.create('img', { //$NON-NLS-1$
					src: this._blankGif,
					height: '16', //$NON-NLS-1$
					width: '16', //$NON-NLS-1$
					className: 'ple-icon ' + iconClass //$NON-NLS-1$
					}, li);

				li.appendChild(dojo.create('span', { //$NON-NLS-1$
					innerHTML: jazz.util.html.escape(item.componentName || item.name)
				}));

				var description = item['description']; //$NON-NLS-1$
				if (description) {
					li.appendChild(dojo.create('div', { //$NON-NLS-1$
						className: 'ple-RtcStreamComponentPicker-rowDescription', //$NON-NLS-1$
						innerHTML: jazz.util.html.escape(description)
					}));
				}

				return li;
			},

			doSearch: function(searchString, showAllResults) {
				var deferred = new dojo.Deferred();

				if(this.prevXhr){
					this.prevXhr.cancel();
				}

				var rtcServer = this._serverSelect.getServerUrl();

				var self = this;
				this.prevXhr = jazz.client.xhrGet({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/service/com.ibm.relm.ple.service.rtc.IRtcService', //$NON-NLS-1$
					content: {
						method: 'search', //$NON-NLS-1$
						name: searchString,
						type: this._resourceType,
						rtcServer: rtcServer
					},

					load: function(hits) {
						dojo.forEach(hits, function(hit) {
							hit.rtcServer = rtcServer;
						});

						deferred.callback(hits);
						self.showResults(hits);
					},

					error: function(error, ioArgs) {
						// This test is needed since an xhr cancel calls the error callback. 
						if (error.dojoType === 'cancel') { return; } //$NON-NLS-1$
						if (!ioArgs.xhr.responseText) { return; }
						
						self.messageBox.hideMessage();
						self.resultList.clearResults();
						self.clearPreview();
						self.didClearResults();
						self.messageBox.showXhrError(ioArgs.xhr);
					},

					handle: function(hits){
						if (hits.length === 0){
							self.resultList.set('htmlPlaceholder', self.rtcMessages.noResults); //$NON-NLS-1$
						}
					}
				});

				return deferred;
			}
		}
	);
})();
