/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.SortMenu'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'SortMenuMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.ui.Menu'); //$NON-NLS-1$

(function(){
	/**
	 * A menu to show sorting options. Only displays them doesn't do the sort.
	 * Will list a number of properties to sort on that a mutually exclusive and maintain
	 * a check mark on one.
	 *
	 * The sort direction is show at the bottom of the menu as a second set of mutually
	 * exclusive choices.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.SortMenu', //$NON-NLS-1$
		[ jazz.ui.Menu ],
		{
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'SortMenuMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			selectable: true,

			/**
			 * Properties to show as sortable; in sets of 
			 * property IDs, menu labels, and is/is not a date property. Like: [['priority', 'Priority', false], ...]
			 * Read-only after construction.
			 * @type {Array}
			 */
			properties: [],


			/**
			 * Sorts by ascending (true) or descending (false).
			 * @type {Boolean}
			 */
			sortByAscending: true,

			/**
			 * ID of property to sort on.
			 * Can be set later.
			 * @type {String}
			 */
			sortOnProperty: '', //$NON-NLS-1$

			disableProperties: function(propertyIds) {
				var currentSortOnPropertyIsDisabled;
				var disabledSet = {};

				for (var i = 0; i < propertyIds.length; i++) {
					var id = propertyIds[i];
					if (id === this.sortOnProperty) currentSortOnPropertyIsDisabled = true;
					disabledSet[id] = true;
				}

				for (var j = 0; j < this._propertyItems.length; j++) {
					var menuItem = this._propertyItems[j];
					var isDisabled = disabledSet[menuItem.propertyId];

					menuItem.set('disabled', isDisabled); //$NON-NLS-1$
					
					if (!isDisabled && currentSortOnPropertyIsDisabled) {
						this.set('sortOnProperty', menuItem.propertyId); //$NON-NLS-1$
						currentSortOnPropertyIsDisabled = false;
					}
				}
			},

			enableAll: function() {
				this.disableProperties([]);
			},			

			postCreate: function() {
				this.inherited(arguments);

				this._propertyItems = [];

				for (var i = 0; i < this.properties.length; i++) {
					var prop = this.properties[i];
					var propId = prop[0];
					var propLabel = prop[1];
					var propIsDate = prop[2];

					var item = new jazz.ui.menu.MenuItem({
						propertyId: propId,
						isDate: propIsDate,
						label: propLabel,
						onClick: dojo.hitch(this, this.set, 'sortOnProperty', propId) //$NON-NLS-1$
					});

					if (this.sortOnProperty === propId) {
						item.set('selected', true); //$NON-NLS-1$
						this._sortOnPropertyItem = item;
					}

					this._propertyItems.push(item);
					this.addChild(item);
				}

				this.addChild(new dijit.MenuSeparator());

				this._ascendingItem = new jazz.ui.menu.MenuItem({
					label: this.messages.ascending,
					imgClass: 'ple-icon-sortDescending', //$NON-NLS-1$
					selected: this.sortByAscending,
					onClick: dojo.hitch(this, this.set, 'sortByAscending', true) //$NON-NLS-1$
				});
				this.addChild(this._ascendingItem);

				this._descendingItem = (new jazz.ui.menu.MenuItem({
					label: this.messages.descending,
					imgClass: 'ple-icon-sortAscending', //$NON-NLS-1$
					selected: !this.sortByAscending,
					onClick: dojo.hitch(this, this.set, 'sortByAscending', false) //$NON-NLS-1$
				}));
				this.addChild(this._descendingItem);
			},

			_setSortOnPropertyAttr: function(propertyId) {
				this._set('sortOnProperty', propertyId); //$NON-NLS-1$
				
				if (!this._propertyItems) return;
				
				var wasDateProperty = this._sortOnPropertyItem && this._sortOnPropertyItem.isDate;
				var isDateProperty = false;

				for (var i = 0; i < this._propertyItems.length; i++) {
					var item = this._propertyItems[i];

					if (item.propertyId === propertyId) {
						item.set('selected', true); //$NON-NLS-1$
						this._sortOnPropertyItem = item;
					}
					else {
						item.set('selected', false); //$NON-NLS-1$
					}
				}

				if (this._sortOnPropertyItem.isDate) {
					this._ascendingItem.set('label', this.messages.ascendingDate); //$NON-NLS-1$
					this._descendingItem.set('label', this.messages.descendingDate); //$NON-NLS-1$
					
					// Default to newest-to-oldest for dates. This also causes a `didSortBy` callback, so don't
					// do it twice.
					if (!wasDateProperty) {
						// coming from a string or num, so change to reverse sorting
						this.set('sortByAscending', false); //$NON-NLS-1$
					}
					else {
						this.didSortBy(this.sortOnProperty, this.sortByAscending, this._sortOnPropertyItem.get('label')); //$NON-NLS-1$
					}
				}
				else{
					this._ascendingItem.set('label', this.messages.ascending); //$NON-NLS-1$
					this._descendingItem.set('label', this.messages.descending); //$NON-NLS-1$

					if (wasDateProperty) {
						// coming from a date, so change to natural sorting
						this.set('sortByAscending', true); //$NON-NLS-1$
					}
					else {
						this.didSortBy(this.sortOnProperty, this.sortByAscending, this._sortOnPropertyItem.get('label')); //$NON-NLS-1$
					}
				}
			},

			_setSortByAscendingAttr: function(ascending) {
				if (!this._ascendingItem) return;
				this._set('sortByAscending', ascending); //$NON-NLS-1$

				this._ascendingItem.set('selected', ascending); //$NON-NLS-1$
				this._descendingItem.set('selected', !ascending); //$NON-NLS-1$

				this.didSortBy(this.sortOnProperty, this.sortByAscending, this._sortOnPropertyItem.get('label')); //$NON-NLS-1$
			},

			/**
			 * Callback after a sort property or order is changed.
			 * @param  {string} propertyId      ID of property that was sorted
			 *     on.
			 * @param  {boolean} sortByAscending true if ascending.
			 * @param  {string} propertyLabel   human readable name of
			 *     property. Used to update GUI elements showing the sort
			 *     property.
			 */
			didSortBy: function(propertyId, sortByAscending, propertyLabel) { 
			}
		}
	);

})();