/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.picker.StyledList"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "StyledListMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){
	var util = com.ibm.relm.common.web.ui.util;

	dojo.declare(
		"com.ibm.relm.ple.web.ui.picker.StyledList", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/StyledList.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			/**
			 * When not enabled, this list will appear grayed out and will not
			 * respond to most user action (keyboard nav, clicks etc.). This
			 * is used when a "submit" is in progress and we don't want the
			 * user to interact with the list, specifically we don't want them
			 * triggering two submits.
			 * @type {Boolean}
			 */
			enabled: true,

			/**
			 * If items in the list can be selected, both through mouse clicks
			 * and keyboard control. This is used to disable selections in the RELM
			 * Welcome screen, which is just a jumping off point, not a picker.
			 * @type {Boolean}
			 */
			canSelect: true,

			_setEnabledAttr: function(enabled) {
				this._set('enabled', enabled); //$NON-NLS-1$
				dojo.toggleClass(this.domNode, 'relm-StyledList-disabled', !enabled); //$NON-NLS-1$
			},

			constructor: function(args, node)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "StyledListMessages"); //$NON-NLS-1$ //$NON-NLS-2$

				args.listItemFactory && this.setListItemFactory(args.listItemFactory);
				this._items = [];
			},

			setListItemFactory: function(listItemFactory) {
				if (!dojo.isFunction(listItemFactory)) {
					throw new Error('listItemFactory must be a function that creates <li> nodes.'); //$NON-NLS-1$
				}

				this._listItemFactory = listItemFactory;
			},

			focus: function() {
				this.domNode.focus();
			},

			// Callback for when a list item is selected
			onSelectionChanged: function(selectedResultsOrUndefined) {
			},

			onDoubleClickItem: function(item) {
			},

			onSubmit: function() {
			},

			getSelectedItem: function()
			{
				return this._selectedRow ? this._items[this._selectedRow.itemIndex] : undefined;
			},

			scrollSelectedIntoView: function() {
				if (!this._selectedRow) return;
				this._scrollIntoViewIfNeeded(this._selectedRow);
			},

			getItemForRow: function(row) {
				return this._items[row.itemIndex];
			},

			_selectRow: function(row) {
				if (!this.canSelect) return;

				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected'); //$NON-NLS-1$
				}

				dojo.addClass(row, 'selected'); //$NON-NLS-1$

				this._selectedRow = row;
				this._scrollIntoViewIfNeeded(row);
				this.onSelectionChanged(this._items[row.itemIndex]);
			},

			_scrollIntoViewIfNeeded: function(row) {
				if (row.offsetTop < this.resultListScoller.scrollTop) {
					row.scrollIntoView(true);
				}
				else if (row.offsetTop >= this.resultListScoller.scrollTop + this.resultListScoller.clientHeight) {
					row.scrollIntoView(false);
				}
			},

			selectNext: function() {
				if (!this.enabled) return;

				if (!this._selectedRow) {
					// select first on arrow down if no selection
					var nextRow = this._resultList.firstChild;
				}
				else  {
					nextRow = this._selectedRow.nextSibling;
				}

				while (nextRow) {
					if (nextRow.style.display !== 'none') { //$NON-NLS-1$
						this._selectRow(nextRow);
						return;
					}
					nextRow = nextRow.nextSibling;
				}
			},

			selectPrevious: function() {
				if (!this.enabled) return;

				if (!this._selectedRow) {
					// select last on arrow up if no selection
					var previousRow = this._resultList.lastChild;
				}
				else if (this._selectedRow.previousSibling) {
					previousRow = this._selectedRow.previousSibling;
				}

				while (previousRow) {
					if (previousRow.style.display !== 'none') { //$NON-NLS-1$
						this._selectRow(previousRow);
						return;
					}
					previousRow = previousRow.previousSibling;
				}
			},

			_onListKeyDown: function(e) {
				if (!this.enabled) return;

				var k = e.keyCode;

				// TODO: PAGE_UP/DOWN and HOME/END
				// FIXME: Enter key is not submitting form

				if (k === dojo.keys.UP_ARROW) {
					dojo.stopEvent(e);
					this.selectPrevious();
				}
				else if (k === dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this.selectNext(k);
				}
				else if (k === dojo.keys.ENTER) {
					this.onSubmit();
				}
			},

			_onDblClickResource: function(e) {
				if (!this.enabled) return;

				var row = e.target;

				if (e.target.nodeName != 'LI') { //$NON-NLS-1$
					row = dojo.NodeList(e.target).closest('LI')[0]; //$NON-NLS-1$
					if (!row) {
						console.warn('no list item found on click'); //$NON-NLS-1$
						return;
					}
				}

				dojo.stopEvent(e);
				this._selectRow(row);
				this.onDoubleClickItem(this._items[row.itemIndex]);
			},

			_onClickResource: function(e) {
				if (!this.enabled) return;

				var row = e.target;

				if (e.target.nodeName != 'LI') { //$NON-NLS-1$
					row = dojo.NodeList(e.target).closest('LI')[0]; // TODO: is there s faster way? //$NON-NLS-1$
					if (!row) {
						return;
					}
				}

				if (dojo.hasClass(row, 'selected')) { //$NON-NLS-1$
					return;
				}
				else {
					this._selectRow(row);
				}
			},

			deselectAll: function()
			{
				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected'); //$NON-NLS-1$
					this._selectedRow = null;
				}

				this.onSelectionChanged(undefined);
			},

			/**
			 * Empties out the result rows and footer. A dumb HTML clearing.
			 * Clears the selected row, since that row won't exist anymore.
			 * @return {[type]} [description]
			 */
			_emptyResults: function() {
				delete this._lastSearchString;
				dojo.empty(this._footerNode);
				dojo.empty(this._headerNode);

				// In case the rows were from widgets (Notably, PickerResult)
				// we clean them out here instead of waiting for the dialog to
				// close. These may end up getting .destroyed more than once
				// if the dialog does so so, but that's harmless, and I rather
				// be conservative with memory here.
				var deadRows = dijit.findWidgets(this._resultList);
				for (var i = 0; i < deadRows.length; i++) {
					deadRows[i].destroyRecursive();
				}

				dojo.empty(this._resultList);
				this._selectedRow = null;
			},

			showLoading: function(show)
			{
				this._hiddenResultsMessage && this._hiddenResultsMessage.hide();
				this._emptyResults();

				if (show) {
					this._loadingMessage.show();
				}
				else {
					this._loadingMessage.hide();
				}
			},

			clearResults: function() {
				this._destroyItems();
				this._items = [];
				this._emptyResults();
				this._loadingMessage.showPlaceholder();
				this.onSelectionChanged();
				this._hiddenResultsMessage && this._hiddenResultsMessage.hide();
			},

			_createListItem: function(item) {
				var li;

				if (!this._listItemFactory) throw new Error('listItemFactory not set to create list results'); //$NON-NLS-1$

				li = this._listItemFactory(item);
				// Mark these rows with a custom data attribute for easier querying later.
				// We don't want to confuse other LI elements (possibly nested in the rows or
				// even in the footer region) with the logical rows.
				dojo.attr(li, 'data-isRelmStyledListRow', 'yes'); //$NON-NLS-1$ //$NON-NLS-2$
				return li;
			},

			_drawItems: function() {
				var frag = document.createDocumentFragment();

				for (var i = 0; i < this._items.length; i++) {
					var item = this._items[i];
					var li = this._createListItem(item);
					// Cache the index of the item this row goes to for faster lookup.
					li.itemIndex = i;
					frag.appendChild(li);
				}

				this._resultList.appendChild(frag);
			},

			/**
			 * Sorts the rows. The sort function runs against the data backing the rows.
			 * Preserves the selected.
			 * @param  {function} compareFunction a compare function for the item (data) backing the rows.
			 * That is, the original results.
			 */
			sortResults: function(compareFunction) {
				var rows = dojo.query('li', this._resultList); //$NON-NLS-1$

				var self = this;
				rows.sort(function(leftRow, rightRow) {
					return compareFunction(self.getItemForRow(leftRow), self.getItemForRow(rightRow));
				});

				var frag = document.createDocumentFragment();
				for (var i = 0; i < rows.length; i++) {
					var li = rows[i];
					frag.appendChild(li);
				}				
				this._resultList.appendChild(frag);

				this._selectedRow && this._scrollIntoViewIfNeeded(this._selectedRow);
			},

			/**
			 * this._items may be object literal OR widgets.
			 * If they are widgets, we need to destroy them.
			 */
			_destroyItems: function() {
				for (var i = 0; i < this._items.length; i++) {
					var item = this._items[i];
					item.destroyRecursive && item.destroyRecursive();
				}

				this._items = [];
			},

			showResults: function(resultItems) {
				this._selectedRow = null;
				this._destroyItems();

				if (resultItems.length === 0) {
					this.clearResults();
					return;
				}

				this._loadingMessage.hide();

				this._items = resultItems;
				this._drawItems();
			},

			getItems: function() {
				return this._items;
			},

			/**
			 * Callbacl for when the "Show All" link is clicked in the "hidden results" filter message.
			 * Used to clear the FilterBox of the parent dialog and to trigger a new set of (all) results
			 * to be shown.
			 */
			shouldClearFilter: function() {
			},

			/**
			 * Filters the display hits by text. Check the visible text of each row,
			 * plus an extra properties handed in that are directly in the data item.
			 * @param  {string} searchString    the text to search for.
			 * @param  {array} checkProperties list of property IDs to search on the underlying item.
			 *                                 In addition to visible text on the row. Used for checking
			 *                                 artifact types, even though they aren't visible (just an icon).
			 */

			filterResults: function(searchString, checkProperties) {
				// This is just an optimization to speed to restoring of the
				// state when the artifact picker uses a column browser to move to
				// DOORS baseline (sub-artifacts). When we pop to the top level
				// again we want to restore the filter text, but not re-run
				// this filter since we know the results are already filtered for
				// that string.
				if (searchString === this._lastSearchString) {
					return;
				}

				this._lastSearchString = searchString;

				checkProperties = checkProperties || [];
				// Spit the search string into "words" we need to find each word to match.
				var words = [];

				if (searchString) {
					searchString = dojo.trim(searchString).toLowerCase();
					words = searchString.split(/\s+/);
				}

				var self = this;
				var hiddenCount = 0;

				// Check every row
				dojo.query('li[data-isRelmStyledListRow=yes]', this._resultList).forEach(function(row) { //$NON-NLS-1$
					var item = self.getItemForRow(row);

					// Will check the visible text of the row (scrapeText) and
					// any additional properties handed in that are on the rows' items.
					// Find all words of the search string (don't have to be next to each other).
					var allWordsAreInRow = dojo.every(words, function(word) {
						for (var i = 0; i < checkProperties.length; i++) {
							var property = checkProperties[i];
							var propertyValue = item[property];
							if (!propertyValue) continue;
							propertyValue = propertyValue.toLowerCase();
							if (propertyValue.search(word) !== -1) return true;
						}

						var text = row.textContent.toLowerCase();
						if (text.search(word) !== -1) return true;

						return false;
					});	

					if (!allWordsAreInRow) {
						// Can't find all words on this row, so hide it.
						hiddenCount++;
						row.style.display = 'none'; //$NON-NLS-1$
						if (row === self._selectedRow) {
							// Don't maintain the selection if the row is hidden.
							self.deselectAll();
						}
					}
					else {
						row.style.display = ''; //$NON-NLS-1$	
					}
				});

				if (hiddenCount === 0) {
					this._hiddenResultsMessage && this._hiddenResultsMessage.hide();
				}
				else {
					if (!this._hiddenResultsMessage) {
						// Create the "hidden results" message on demand. Reuse on future calls.
						this._hiddenResultsMessage = new com.ibm.relm.common.web.HiddenResultsMessage();
						this._hiddenResultsMessage.placeAt(this._footerNode, 'before'); //$NON-NLS-1$
						this.connect(this._hiddenResultsMessage, 'shouldShowAll', this.shouldClearFilter); //$NON-NLS-1$
					}
					this._hiddenResultsMessage.setHiddenCount(hiddenCount);
				}
			},

			/**
			 * Sets some content at the bottom of the list of results. Will 
			 * scroll with results. Used to additional messages (normally about filtered types).
			 * @param {DOM node} footerContentNode any DOM content. Not that this will be emptied
			 *                   as if it's dumb-HTML, which might not clear embedded widets, so 
			 *                   don't add widgets here unless you test it.
			 */
			 setFooterContent: function(footerContentNode) {
			 	dojo.empty(this._footerNode);
			 	if (footerContentNode) dojo.place(footerContentNode, this._footerNode);
			 },

			 setHeaderContent: function(headerContentNode) {
			 	dojo.empty(this._headerNode);
			 	if (headerContentNode) dojo.place(headerContentNode, this._headerNode);
			 },

			appendResult: function(resultItem) {
				this._loadingMessage.hide();

				this._items.push(resultItem);
				var li = this._createListItem(resultItem);
				this._resultList.appendChild(li);
			},

			_setPlaceholderAttr: function(newPlaceholderText) {
				this._loadingMessage.set('placeholder', newPlaceholderText); //$NON-NLS-1$
			},

			_setHtmlPlaceholderAttr: function(newPlaceholderText) {
				this._loadingMessage.set('htmlPlaceholder', newPlaceholderText); //$NON-NLS-1$
			},

			_getPlaceholderAttr: function() {
				this._loadingMessage.get('placeholder'); //$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);
				this._loadingMessage.showPlaceholder();
			},

			uninitialize: function() {
				this._destroyItems();
				this._emptyResults();
			}
		}
	);

})();

