/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('com.ibm.relm.ple.web.ui.pages.CreateResourceDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.ConfirmDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.ProgressDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.FindUsePage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.admin.ImportUploadDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.pages.CreateBaselineDialog'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function() {

	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$
	var contextRoot = net.jazz.ajax._contextRoot;
	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var Goto = com.ibm.relm.ple.web.ui.Goto;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var Site = com.ibm.relm.ple.web.ui.Site;
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var StatusMessage = com.ibm.relm.common.web.StatusMessage;

	// Matches the URI pattern for Product versions (ItemVersions).
	var ITEM_VERSION_RE = /.+\/itemVersion_\d+$/;
	var CONFIG_VERSION_RE = /.+\/configurationVersion_\d+$/;

	/**
	 * Given a full URI to a config version, figure out what it's path segment
	 * should be if it's to be displayed at the top of the product tree.
	 * 
	 * @param  {string} configVersionUri
	 * @return {string} the path segment string for this config version.
	 */
	function pathSegmentForTopLevelConfigVersionUri(configVersionUri) {
		// This must match the Java JsonItemVersions version of this EXACTLY.
		// It is here as a performance optimization since the algorithm is so
		// simple currently, but this is duplicating that logic.
		var lastSlash = configVersionUri.lastIndexOf('/'); //$NON-NLS-1$
		return configVersionUri.substring(lastSlash + 1);
	}

	/**
	 * New location for top-level product API operations so that they can be called from
	 * anywhere, even when ProductsPage is not showing, plus it helps off-load
	 * some of the code from that bloated page.
	 */
	dojo.mixin(com.ibm.relm.ple.web.ui.productapi, {
		/**
		 * Shows a dialog to create a new product. Can be top-level or
		 * a child. Creates the product on OK.
		 *
		 * @param optionalSite - If provided, new product will be a child of that site (confg or product)
		 */
		createProduct: function(optionalsite) {
			var dialog = new com.ibm.relm.ple.web.ui.pages.CreateResourceDialog({
				site: optionalsite,
				iconClass: 'ple-icon-createProduct', //$NON-NLS-1$
				messageFileName: "CreateProductDialogMessages", //$NON-NLS-1$
				isConfigVersion: false,
				servicePath: '/ple/create_item_version' //$NON-NLS-1$
			});

			dialog.startup();
		},

		/**
		 * Prompts for a name of a new config version then creates it.
		 * If there is no selected site given, a new top-level config version is
		 * created. Otherwise, it is created as a child of the selected config version
		 * of item version.
		 * 
		 * @param  {site} site optional site selected in tree to create the config version under.
		 */
		createConfigVersion: function(optionalSite) {
			var dialog = new com.ibm.relm.ple.web.ui.pages.CreateResourceDialog({
				site: optionalSite,
				iconClass: 'ple-icon-createConfigVersion', //$NON-NLS-1$
				messageFileName: "CreateConfigurationDialogMessages", //$NON-NLS-1$
				isConfigVersion: true,
				servicePath: '/ple/create_config_version' //$NON-NLS-1$
			});

			dialog.startup();
		},

		createCopyOfConfiguration: function(site) {
			var configVersion = site.configVersion;

			var args = {
				hideParent: true,
				isConfigVersion: true,
				iconClass: 'ple-icon-copyConfigVersion', //$NON-NLS-1$
				name: dojo.replace(messages.configurationXCopyName, [configVersion.name]),
				messageFileName: "CreateCopyDialogMessages", //$NON-NLS-1$
				_oldBranchDimensions: configVersion.branchDimensions,
				site: site,
				servicePath: '/ple/create_config_copy' //$NON-NLS-1$
			};

			var d = new com.ibm.relm.ple.web.ui.pages.CreateResourceDialog(args);
			d.startup();
		},

		patchBaselineOfConfiguration: function(site) {
			var configVersion = site.configVersion;

			var args = {
				hideParent: true,
				isConfigVersion: true,
				iconClass: 'ple-icon-patchBaseline', //$NON-NLS-1$
				name: dojo.replace(messages.baselineXPatchName, [configVersion.name]),
				messageFileName: "PatchBaselineDialogMessages", //$NON-NLS-1$
				_oldBranchDimensions: configVersion.branchDimensions,
				site: site,
				servicePath: '/ple/patch_baseline' //$NON-NLS-1$
			};

			var d = new com.ibm.relm.ple.web.ui.pages.CreateResourceDialog(args);
			d.startup();
		},

		useVersionOfConfiguration: function(site) {
			var d = new com.ibm.relm.ple.web.ui.pages.ReplaceVersionDialog({
					site: site
				});
			d.startup();
		},

		importProducts: function(site) {
			var args = {
				showImportProgress: true,
				site: site,
				onClose: function(importedBaselinePathSegment) {
					TreeRefresh.refresh({
						selectPath: importedBaselinePathSegment
					});
				}
			};
			var d = new com.ibm.relm.ple.web.admin.ImportUploadDialog(args);
			d.startup();
		},

		_doCheckOutProduct: function(site) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.checkingOutProduct);
			var deferred = new dojo.Deferred();

			var product = site.product;

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/check_out', //$NON-NLS-1$
				content: {
					contextConfigVersionUri: site.contextConfigVersionUri,
					contextPartUri: site.contextPartUri,
					itemVersionUri: site.itemVersionUri
				},

				load: function () {
					TreeRefresh.refresh({
						selectPath: site.path
					}).addBoth(function() {
						deferred.callback();
					});
				},
				error: function(error, ioArgs) {
					deferred.errback(ioArgs.xhr);
				},
				handle: function() {
					message.hide();
				}
			});

			return deferred;
		},

		/**
		 * Redacts product versions after a data spill. Can handle multiple products in one call.
		 * @param  {Array} product an array of json products or sites
		 * @param  {Function} refreshCallback a callback to be invoked when the redact operation completes
		 */
		redactProducts: function(sitesOrItemVersions, refreshCallback) {
			var formattedNames = []; var uris = [];
			dojo.forEach(sitesOrItemVersions, function(siteOrItemVersion) {
				var product = siteOrItemVersion.itemVersion || siteOrItemVersion;
				uris.push(product.uri);
				var formattedName = formatting.formatProductIconAndTitleForItem(product);
				var listedName = "<li>" + formattedName + "</li>"; //$NON-NLS-1$ //$NON-NLS-2$
				formattedNames.push(listedName);
			});

			var prompt = dojo.replace(messages.redactPromptX, [formattedNames.join('')]); //$NON-NLS-1$
			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.redactTitle,
				width: '500px', //$NON-NLS-1$
				htmlPrompt: prompt,
				details: messages.redactDetailMessage,
				iconClass: 'ple-icon-redact' //$NON-NLS-1$
			});

			var button = d.addButton(messages.redactAction, dojo.hitch(this, this._doRedact, sitesOrItemVersions, refreshCallback), true);
			button.disabled = true;

			this._checkIncompleteRedact(uris, button, d.messageBox);

			d.startup();
		},

		_checkIncompleteRedact: function(uris, button, messageBox) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.checkRedaction);
			//Should this be a get?
			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/redact', //$NON-NLS-1$
				content: {
					itemVersionUris: dojo.toJson(uris),
					checkIncompleteRedaction: true
				},
				load: function (missingVersions) {
					button.disabled = false;
					delete button;

					if (missingVersions && missingVersions.length) {
						var buffer = [];
						dojo.forEach(missingVersions, function(item) {
							var formattedTitle = formatting.formatProductTitleForItem(item);
							buffer.push(formattedTitle);
						});
						var missingVersionNames = buffer.join(", "); //$NON-NLS-1$
						var explanation = dojo.replace(messages.incompleteRedaction$explanation, [missingVersionNames]);
						messageBox.showErrorObject({
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							summary: messages.incompleteRedaction,
							explanation: explanation,
							userAction: messages.incompleteRedaction$userAction,
							closable: true
						});
					}
				},
				error: function(_, args) {
					messageBox.showXhrError(args.xhr);
				},
				handle: function() {
					message.hide();
				}
			});
		},

		_doRedact: function(sitesOrItemVersions, refreshCallback) {
			var message = com.ibm.relm.common.web.StatusMessage.show(messages.redacting);
			var deferred = new dojo.Deferred();

			var uris = dojo.map(sitesOrItemVersions, function(siteOrItemVersion) {
				return (siteOrItemVersion.itemVersion && siteOrItemVersion.itemVersion.uri) || siteOrItemVersion.uri;
			});

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: net.jazz.ajax._contextRoot + '/ple/redact', //$NON-NLS-1$
				content: {
					itemVersionUris: dojo.toJson(uris)
				},

				load: function () {
					deferred.callback();
					if (sitesOrItemVersions.length >= 1) {
						var firstSiteOrItemVersion = sitesOrItemVersions[0];
						//If this is an item version, then there is no path, but TreeRefresh can handle an undefined path
						var path = firstSiteOrItemVersion.path;
					}

					refreshCallback(path);
				},

				error: function(_, args) {
					deferred.errback(args.xhr);
				},

				handle: function() {
					message.hide();
				}
			});
			return deferred;
		},

		/**
		 * Determines if the given item is likely a normal product,
		 * as opposed to a pseudo-product like those used in the RTC
		 * integration where a product is used as a component, baseline, etc.
		 * @param  {object} product the product to check. Can be any object.
		 * @return {boolean}         true if the object appears to be a normal product.
		 */
		isNormalProduct: function(product) {
			var p = product;

			return !p.rtcBaseline &&
				p.conditionVariables === undefined &&
				!p.rtcComponent &&
				!p.link &&
				p.type === 'product'; //$NON-NLS-1$
		},

		/**
		* Performs an explicit check out of a product.
		*
		*/
		checkOutProduct: function(site) {
			this._doCheckOutProduct(site);			
		},

		createBaseline: function(site) {
			var name = formatting.formatProductTitleForItem(site.configVersion);

			var d = new com.ibm.relm.ple.web.ui.pages.CreateBaselineDialog({
				site: site
			});

			d.startup();
		},

		_doCreateBaseline: function(site) {
			var deferred = new dojo.Deferred();

			deferred.callback();

			return deferred;
		},

		checkInProduct: function(siteOrItemVersion) {
			var itemVersion = siteOrItemVersion.itemVersion || siteOrItemVersion;
			var name = formatting.formatProductTitleForItem(itemVersion);

			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.checkInTitle,
				prompt: dojo.replace(messages.checkInPrompt, [name]),
				details: messages.checkInDetailMessage,
				iconClass: 'ple-icon-checkIn' //$NON-NLS-1$
			});

			d.addButton(messages.checkInAction, dojo.hitch(this, this._doCheckInProduct, siteOrItemVersion), true);
			d.startup();
		},

		_doCheckInProduct: function(siteOrItemVersion) {
			var site;
			var itemVersion;

			if (siteOrItemVersion.itemVersion) {
				site = siteOrItemVersion;
				itemVersion = site.itemVersion;
			}
			else {
				site = {};
				itemVersion = siteOrItemVersion;
			}

			var message = com.ibm.relm.common.web.StatusMessage.show(messages.checkingInProduct);
			var deferred = new dojo.Deferred();

			var self = this;
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/check_in', //$NON-NLS-1$
				content: {
					contextConfigVersionUri: site.contextConfigVersionUri,
					contextPartUri: site.contextPartUri,
					itemVersionUri: itemVersion.uri
				},

				handle: function() {
					message.hide();
				},

				load: function() {
					dojo.publish('ple.didCheckInItemVersionUri', [itemVersion.uri]); //$NON-NLS-1$

					var path = site.path;

					if (!path) {
						// If the site has not path, then this is likely a check
						// in from Audit History outside the tree. We still want
						// to refresh the tree and the selected item in it though.
						dojo.publish("ple.refresh_selected_path"); //$NON-NLS-1$
						deferred.callback();
					} else {
						TreeRefresh.refresh({
							selectPath: path
						}).addBoth(function() {
							deferred.callback();
						});
					}

				},

				error: function(error, ioArgs) {
					deferred.errback(ioArgs.xhr);
				}
			});

			return deferred;
		},

		/**
		 * Given a path to an item version (product or product configuration
		 * really), will navigate to the product tree to show the item. That
		 * is, will change the current browser URL to the best URL to show the
		 * given item version.
		 *
		 * Find Use will be run to know which configuration it is in. If
		 * multiple uses are found, the Find Use page will be shown to allow
		 * the user to pick which path to show.
		 * 
		 * @param  {string} itemVersionUri URI of item version or config
		 *     version to show.
		 * @param {boolean} showPathsIfNotFound optional. If true, then when
		 *     there are no paths to a product, the "Find Paths" page will
		 *     still be shown, even though it will just have a message about
		 *     not finding it. Otherwise, the single page read-only view is
		 *     shown of the product outside of the tree. This is used with
		 *     `true` from the read-only products page itself, so clicking the
		 *     button on an unused product won't just show the same page.
		 */
		findPathsToProductAndShow: function(itemVersionUri, showPathsIfNotFound) {
			com.ibm.relm.ple.web.ui.ProgressDialog.slowXhrGet({
				explanation: messages.findingWhereUsed,
				showDialogInMillis: 500,
				url: net.jazz.ajax._contextRoot + '/ple/find_use', //$NON-NLS-1$
				content: {
					itemVersionUri: itemVersionUri
				},
				handleAs: 'json', //$NON-NLS-1$

				load: dojo.hitch(this, function (pathsData) {
					dojo.publish("ple.product_tree_hide_message"); //$NON-NLS-1$

					if (pathsData.paths.length === 1) {
						// Only 1 path, so show right away.
						var onePathOfItems = pathsData.paths[0];
						var path = dojo.map(onePathOfItems, function(item) {
							return item.pathSegment;
						});
						Goto.gotoProductTreePath(path);
						return;
					}
					else if (pathsData.paths.length === 0 && !showPathsIfNotFound) {
						if (CONFIG_VERSION_RE.exec(itemVersionUri)) {
							var pathSegment = pathSegmentForTopLevelConfigVersionUri(itemVersionUri);
							Goto.gotoProductTreePath([pathSegment]);
						}
						else if (ITEM_VERSION_RE.exec(itemVersionUri)) {
							Goto.gotoReadOnlyProductPage(itemVersionUri);
						}
					}
					else {
						// multiple paths, so must let user choose
						com.ibm.relm.ple.web.ui.pages.FindUsePage.cachePathsData(itemVersionUri, pathsData);
						Goto.gotoFindPaths(itemVersionUri);
						return;
					}
				}),

				error: function(error, ioArgs) {
					// TODO: handle error
					console.error(error);
				}
			});
		},

		/**
		 * Adds a product to the top of the tree by marking it as Recent and
		 * Favorite.
		 * @param  {site object} site the product to remove and add.
		 * @param {boolean} removeFromParent if true, this is like a Move
		 *     operation in that the product is first remove from the parent
		 *     it's under. If false, this is only a pure "add" operation, that
		 *     is, just a shortcut to add things to recently viewed and
		 *     favorites.
		 */
		addToTopOfTree: function(configVersionSites, removeFromParent) {
			var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
				canCancel: false,
				mirrorStatusMessage: true,
				iconClass: 'ple-icon-move' //$NON-NLS-1$
			});
			progressDialog.startup();

			var text = removeFromParent ? messages.movingToTopStatus : messages.addingToTopStatus;
			var message = com.ibm.relm.common.web.StatusMessage.show(text);

			jazz.client.xhrPost({
				handleAs: 'json', //$NON-NLS-1$
				url: contextRoot + '/ple/tree_top', //$NON-NLS-1$
				content: {
					fromSites: Site.sitesToJsonSites(configVersionSites),
					removeFromParent: removeFromParent
				},

				load: function(result) {
					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
						selectPath: [result.pathSegment],
						flashPath: true
					}).addBoth(function() {
						progressDialog.close();
					});
				},

				error: function(error, ioArgs) {
					progressDialog.close();
					var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
					dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
				},

				handle: function() {
					message.hide();
				}
			});
		},

		/**
		 * Silently adds to the Recently Viewed Products list, but does no refresh.
		 * @param {[type]} itemVersionUri [description]
		 */
		addToRecentlyViewed: function(itemVersionUri) {
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/recently_viewed', //$NON-NLS-1$
				content: {
					addItemVersionUri: itemVersionUri
				},
				error: function(error, ioArgs) {
					console.warn('Could not add product to recently viewed list: ' + itemVersionUri); //$NON-NLS-1$
					// Didn't add to Recently Viewed Products. Oh, well. Nothing to do about it.
				}
			});
		},

		getIconClassForItem: function(item){
			return formatting.getIconClassForItem(item);
		},

		formatDecoratorForItem: function(item) {
			return formatting.formatDecoratorForItem(item);
		},

		formatProductTitle: function(title, branch, snapshot) {
			return formatting.formatProductTitle(title, branch, snapshot);
		},

		formatProductTitleForItem: function(item) {
			return formatting.formatProductTitleForItem(item);
		},

		_doReplaceWithChildren: function(configVersionSite) {
			var deferred = new dojo.Deferred();

			var message = StatusMessage.show(messages.swappingStatus);

			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/replace_config_version_with_roots', //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				content: {
					site: Site.siteToJsonSite(configVersionSite)
				},
			
				load: function (pathSegmentItem) {
					TreeRefresh.refresh({
						selectPath: configVersionSite.pathForPeer(pathSegmentItem)
					}).addBoth(function() {
						deferred.callback();
					});
				},
			
				error: function(error, ioArgs) {
					deferred.errback(ioArgs.xhr);
				},

				handle: function() {
					message.hide();
				}
			});

			return deferred;
		},

		replaceWithChildren: function(configVersionSite) {
			var title = com.ibm.relm.ple.web.ui.formatting.formatProductTitleForItem(configVersionSite.configVersion);

			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.confirmReplaceWithProductsTitle,
				prompt: dojo.replace(messages.replaceXWithProductsPrompt, [title]),
				details:  messages.replaceWithChildrenDetails,
				iconClass: 'ple-icon-replaceWithChildren' //$NON-NLS-1$
			});

			var self = this;
			d.addButton(messages.swapButton, function() {
				return self._doReplaceWithChildren(configVersionSite);
			}, true);

			d.startup();
		},

		_doReplaceWithNewConfigVersion: function(itemVersionSite) {
			var deferred = new dojo.Deferred();

			var message = StatusMessage.show(messages.swappingStatus);

			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/ple/create_config_version', //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				content: {
					replaceItemVersionSite: Site.siteToJsonSite(itemVersionSite)
				},

				load: function (pathSegmentItem) {
					TreeRefresh.refresh({
						selectPath: itemVersionSite.pathForPeer(pathSegmentItem),
						openPath: true
					}).addBoth(function() {
						deferred.callback();
					});
				},

				error: function(error, ioArgs) {
					deferred.errback(ioArgs.xhr);
				},

				handle: function() {
					message.hide();
				}
			});

			return deferred;
		},

		replaceWithNewConfigVersion: function(itemVersionSite) {
			var title = com.ibm.relm.ple.web.ui.formatting.formatProductTitleForItem(itemVersionSite.itemVersion);

			var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
				dialogTitle: messages.confirmReplaceWithNewConfigurationTitle,
				prompt: dojo.replace(messages.replaceXWithNewProductConfigurationPrompt, [title]),
				details:  messages.replaceWithNewProductConfigurationDetails,
				iconClass: 'ple-icon-replaceWithConfiguration' //$NON-NLS-1$
			});

			var self = this;
			d.addButton(messages.swapButton, function() {
				return self._doReplaceWithNewConfigVersion(itemVersionSite);
			}, true);

			d.startup();
		}
	});

})();