/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.prop.PropertyName"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.prop", "PropertyNameMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){

	var DEFAULT_USER_NAMESPACE = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/property/'; //$NON-NLS-1$
	var DEFAULT_PREFIX = 'user'; //$NON-NLS-1$

	function text(str)
	{
		return document.createTextNode(str);
	}

	//TODO: Look into getting this directly from properties file instead of hardcoding
	//TODO: reverse order of map and logic using map
	var NAMESPACE_TO_PREFIXES = {
		"http://www.w3.org/1999/02/22-rdf-syntax-ns#" : "rdf", //$NON-NLS-1$ //$NON-NLS-2$
		"http://www.w3.org/2000/01/rdf-schema#" : "rdfs", //$NON-NLS-1$ //$NON-NLS-2$
		"http://purl.org/dc/elements/1.1/" : "dc", //$NON-NLS-1$ //$NON-NLS-2$
		"http://purl.org/dc/terms/" : "dcterms", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/core#" : "oslc", //$NON-NLS-1$ //$NON-NLS-2$
		//"http://relm.ibm.com/productdef/" : "pd",
		"http://xmlns.com/foaf/0.1/" : "foaf", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/asset#" : "oslc_asset", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/am#" : "oslc_am", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/qm#" : "oslc_qm", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/rm#": "oslc_rm", //$NON-NLS-1$ //$NON-NLS-2$
		"http://open-services.net/ns/cm#" : "oslc_cm" //$NON-NLS-1$ //$NON-NLS-2$
	};

	NAMESPACE_TO_PREFIXES[DEFAULT_USER_NAMESPACE] = DEFAULT_PREFIX;

	/**
	 * Converts a full predicate URI to a short name.
	 * This is similar to a QName in that it's normally
	 * a prefix:localName. But, the 'user' prefix is the default
	 * so it's not even shown (just 'localName').
	 * @param  {[type]} fullUri [description]
	 * @return {[type]}
	 */
	function getShortName(fullUri) {

		var lastSplit = Math.max(fullUri.lastIndexOf('#'), fullUri.lastIndexOf('/')); //$NON-NLS-1$ //$NON-NLS-2$
		if (lastSplit < 0) return fullUri;

		var strippedName = fullUri.substring(lastSplit + 1);
		var namespace = fullUri.substring(0, lastSplit + 1);

		var prefix = NAMESPACE_TO_PREFIXES[namespace];
		
		if (prefix) {
			var retval = prefix === DEFAULT_PREFIX ?
				decodeRelmLocalPredicateName(strippedName) :
				prefix + ":" + strippedName; //$NON-NLS-1$

			return retval;
		} else {
			return undefined;
		}
	}

	/**
	 * Encodes a short name so that it is safe to use in a URI for
	 * an RDF predicate. Notably names must not begin or end with certain
	 * characters (digits for sure) and must be all-ASCII.
	 *
	 * This method return the short name URL encoded, with leading and
	 * trailing underscores.
	 * 
	 * @param  {string} shortName human readable short name.
	 * @return {string}           the encoded, URI+RDF safe name. Not a full URI,
	 * just the tail.
	 */
	function encodeRelmLocalPredicateName(shortName) {
		return '_' + encodeURIComponent(shortName) + '_'; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * Decode a short name previously encoded with #encodeRelmLocalPredicateName.
	 * 
	 * @param  {string} encodedShortName the encoded short name.
	 * @return {string}           the human readable short name, decoded.
	 */
	function decodeRelmLocalPredicateName(encodedShortName) {
		// Backwards compatibility check. Not really needed.
		if (encodedShortName.charAt(0) == '_') { //$NON-NLS-1$
			// Trim _underscores_
			encodedShortName = encodedShortName.slice(1, encodedShortName.length - 1);
		}

		return decodeURIComponent(encodedShortName);
	}

	/**
	 * Given a short name
	 * returns the equivalent, full URI for a predicate.
	 * @param  shortName short name, one with a known prefix, or no prefix.
	 * @return full URI.
	 */
	function getFullUri(shortName) {
		var parts = shortName.split(/:/);

		if (parts.length < 2) {
			// No prefix means use default 'user' namespace
			return DEFAULT_USER_NAMESPACE + encodeRelmLocalPredicateName(shortName);
		}

		var shortPrefix = parts[0];
		var localName = parts[parts.length-1];

		for (var namespace in NAMESPACE_TO_PREFIXES) {
			if (NAMESPACE_TO_PREFIXES[namespace] == shortPrefix) {
				return namespace + encodeURIComponent(localName);
			}
		}

		// Couldn't find a known prefix. This is an error the user must correct.
		var error = new Error('unknown prefix'); //$NON-NLS-1$
		error.prefix = shortPrefix;
		throw error;
	}

	function removeNamespace(fullUri) {
		var parts = fullUri.split(/[#\/]/);
		return parts[parts.length-1];
	}

	/**
	 * Gets a property name for display in read-only mode.
	 * Has no prefix or namspace.
	 */
	function getReadOnlyDisplayName(fullUri) {
		var plainName = removeNamespace(fullUri);
		if (fullUri.indexOf(DEFAULT_USER_NAMESPACE) === 0) {
			return decodeRelmLocalPredicateName(plainName);
		}
		else {
			return decodeURIComponent(plainName);
		}
	}

	function isUri(maybeUri) {
		// TODO: there is a dojo function for this. Does it work right?
		return maybeUri.match(/^https?:\/\//);
	}

	/**
	 * The display name of the property shown while it is being
	 * edited. This will be the short name, but without any
	 * URL encoding.
	 * @param  {string} fullUri
	 * @return {string}
	 */
	function getModifiableDisplayName(fullUri) {
		var shortName = getShortName(fullUri);

		if (shortName) {
			return decodeURIComponent(shortName);
		} else  {
			return fullUri;
		}
	}

	/**
	 * Widget for the name of a property. Names appear to be read-only for a clean appearance, but can be modified
	 * by clicking on them or tabbing to them.
	 *
	 * The data associated with the widget always stores the property as a full, RDF predicate URI,
	 * but it may be shown in simpler forms to the user.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.prop.PropertyName", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templateString: '<div style="position: relative;"> <div dojoAttachPoint="_readOnlyNode" tabindex="0" dojoAttachEvent="onfocus: focusName"> </div> </div>', //$NON-NLS-1$
			widgetsInTemplate: false,

			/**
			 * Callback used when the edit of a name is done and the widget
			 * returns to a read-only view. The focus must be lost then (to stop editing).
			 * Normally a listener responds to this by focusing on the value portion of a property
			 * instead.
			 */
			onRevokeFocus: null,

			/**
			 * Full URI of the property name. Set to an initial value at constructionm
			 * but treat as read-only afterwards.
			 * @type {String}
			 */
			nameUri: '', //$NON-NLS-1$

			constructor: function(args, node)
			{
				this._messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.prop", "PropertyNameMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			},

			getNameUri: function() {
				return this.nameUri;
			},

			/**
			 * Draws the current read-only version of the name of this property.
			 */
			_updateReadOnlyLabel: function() {
				var readOnlyName = getReadOnlyDisplayName(this.nameUri);
				dojo.empty(this._readOnlyNode);
				this._readOnlyNode.appendChild(text(readOnlyName));
			},

			postCreate: function() {
				this.inherited(arguments);
				this._originalNameUri = this.nameUri;
				this._updateReadOnlyLabel();
			},

			/**
			 * Removes focus from the name. Returns to a read-only view.
			 *
			 * @param commitName if true, counts as a modifiaction of the name
			 * and the new name is kept. If false, the name is discarded and doesn't
			 * dirty the widget.
			 */
			blurName: function(commitName) {
				// XXX: blur is firing twice, but not sure why.
				if (!this._modifiableName) return;

				this.disconnect(this.blurHandle);
				this.disconnect(this.keyHandle);

				if (commitName) {
					//Need to convert back to full uri from short name
					var uriOrShortName = dojo.trim(this._modifiableName.value);

					if (!uriOrShortName) {
						dojo.publish('ple.details_view_error', [{ //$NON-NLS-1$
								type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								summary: this._messages.missingPropertyName,
								closable: true
							}]);
							return;
					}


					if (isUri(uriOrShortName)) {
						// Looks like it's already long
						this.nameUri = uriOrShortName;
					}
					else {
						try {
							this.nameUri = getFullUri(uriOrShortName);
						} catch (unknownPrefixError) {
							dojo.publish('ple.details_view_error', [{ //$NON-NLS-1$
								type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
								summary: dojo.replace(this._messages.unknownPrefixX, [unknownPrefixError.prefix]),
								explanation: this._messages.allowedPrefixesExplanation,
								userAction: this._messages.allowedPrefixesAction,
								closable: true
							}]);
							return;
						}
					}

					if (this.nameUri != this._originalNameUri) {
						this.onChange();
					}
				}

				dojo.destroy(this._modifiableName);
				delete this._modifiableName;

				this._updateReadOnlyLabel();
				this._readOnlyNode.style.display = ''; //$NON-NLS-1$
			},

			onChange: function() {
			},

			_onBlur: function(e) {
				// XXX: Hacky check to make sure this is an event. If a click
				// on another node by the mouse causes the event, sometimes the 'e' is just the
				// string 'mouse'. Not sure if this is a dojo thing or a bug.
				e && dojo.isObject(e) && dojo.stopEvent(e);

				this.blurName(true);
			},

			_onNameEditKeyDown: function(e) {
				switch(e.keyCode)
				{
				case dojo.keys.ENTER:
					this.blurName(true);
					this.onRevokeFocus();
					break;

				case dojo.keys.ESCAPE:
					this.blurName(false);
					this.onRevokeFocus();
					break;

				default:
					return;
				}
			},

			/**
			 * Focuses on the name and makes it modifiable.
			 */
			focusName: function() {
				this._readOnlyNode.style.display = 'none'; //$NON-NLS-1$

				this._currentModifiableDisplayName = getModifiableDisplayName(this.nameUri);

				this._modifiableName = dojo.create('input', { //$NON-NLS-1$
					value: this._currentModifiableDisplayName,
					size: 40,
					placeholder: this._messages.requiredNamePlaceholder
				});

				this.disconnect(this.blurHandle);
				this.blurHandle = this.connect(this._modifiableName, 'onblur', this._onBlur); //$NON-NLS-1$
				this.disconnect(this.keyHandle);
				this.keyHandle = this.connect(this._modifiableName, 'onkeydown', this._onNameEditKeyDown); //$NON-NLS-1$

				this.domNode.appendChild(this._modifiableName);
				this._modifiableName.focus();
				this._modifiableName.select();
			}
		}
	);

	dojo.mixin(com.ibm.relm.ple.web.ui.prop.PropertyName, {
		getReadOnlyDisplayName: getReadOnlyDisplayName
	});


})();