/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.prop.StringProperty"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.prop", "StringPropertyMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.prop.PropertyName"); //$NON-NLS-1$

(function() {

	/**
	 * Widget for a product property with a single string value.
	 * Assumes it is a <tr> row in a bigger table.
	 * TODO: Some of this should be extracted to a base property widget not
	 * bound to a string value so that other value types can derive from it.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.prop.StringProperty", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/prop/templates/StringProperty.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.prop', 'StringPropertyMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			initialNameUri: '', //$NON-NLS-1$
			initialValue: '', //$NON-NLS-1$
			onDelete: null,

			/**
			 * Gets the current, possibly modified, name and value as an object.
			 */
			getNameUriAndValue: function() {
				return {
					nameUri: this._nameWidget.getNameUri(),
					value: this._valueNode.value
				};
			},

			/**
			 * Puts keyboard focus on the name and makes if modifiable.
			 */
			focusName: function() {
				this._nameWidget.focusName();
			},

			postCreate: function() {
				this.inherited(arguments);

				this._toolbar.addChild(new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-delete', //$NON-NLS-1$
					title: this.messages.deleteLabel,
					enabled: relm.app.Permissions.canProductsDeleteProperty(),
					onClick: dojo.hitch(this, this._onDelete)
				}));

				this.connect(this._nameWidget, 'onChange', dojo.hitch(this, function() { //$NON-NLS-1$
					this.onChange();
				}));

				if (dojo.isIE) {
					// XXX: IE will not reliably fire the onkeyup event for
					// the value input if you mouseclick it first. If you tab
					// to it it works. If you click it, then click somewhere
					// else, then click again it works. But just clicking it
					// doesn't cause it to fire keyboard or focus events.
					// Seems like an outright bug that I can't see how to work
					// around. This is a lame hack to just poll for a change
					// in the value. This is needed to be sure the Save button
					// is enabled. We clear this as soon as it seems safe.
					var self = this;
					this._ieOnCheckInterval = setInterval(function() {
						if (self._valueNode.value != self.initialValue) {
							clearInterval(self._ieOnCheckInterval);
							self.onChange();
						}
					}, 500);
				}
			},

			_onDelete: function() {
				this.onDelete(this,this._nameWidget.getNameUri());
			},

			/**
			 * Puts the keyboard focus on the value.
			 */
			focusOnValue: function(e) {
				this._valueNode.select();
			},

			_onValueKeyUp: function(e) {
				// If this is ever called, then IE got its act together and is
				// seeing onkeyup events again. No need for polling anymore.
				clearInterval(this._ieOnCheckInterval);

				// Ignore Enter key when submiting the form
				if (e.keyCode == dojo.keys.ENTER) return;

				if (this._valueNode.value != this.initialValue) {
					this.onChange();
				}
			},

			onChange: function() {
			},

			_onToolbarFocus: function() {
				dojo.addClass(this.domNode, 'ple-HoverRow-focused'); //$NON-NLS-1$
			},

			_onToolbarBlur: function() {
				dojo.removeClass(this.domNode, 'ple-HoverRow-focused'); //$NON-NLS-1$
			},

			uninitialize: function() {
				clearInterval(this._ieOnCheckInterval);
				this.inherited(arguments);
			}
		}
	);	// dojo.declare

})();