/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.searching"); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.internal', 'ProductSearchProviderMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$

/**
 * Common functions for searching products.
 */
(function(){

var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.internal', 'ProductSearchProviderMessages'); //$NON-NLS-1$ //$NON-NLS-2$
var productapi = com.ibm.relm.ple.web.ui.productapi;
var formatting = com.ibm.relm.ple.web.ui.formatting;

var blankGif = dojo.moduleUrl('dojo','resources/blank.gif'); //$NON-NLS-1$ //$NON-NLS-2$

// Special callback used by Quick Search results. See below.
dojo.subscribe('ple.quick_search.show_result', function(itemVersionUri) { //$NON-NLS-1$
	productapi.findPathsToProductAndShow(itemVersionUri);
});

/**
 * Formats PLE search results into the format expected for the Quick Search banner.
 *
 * @searchResults normal PLE search results.
 */
function formatResultsForSearchProvider(searchResults) {
	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var formattedResults = [];
	var hits = searchResults.hits;
	var getOnClick = function(uri) {
		return function(e) {
				dojo.stopEvent(e);
				productapi.findPathsToProductAndShow(uri);
			};
	};

	for (var i = 0; i < hits.length; i++) {
		var hit = hits[i];

		var label = formatting.formatProductTitleForItem(hit);

		formattedResults.push({
			label: label,
			iconClass: productapi.getIconClassForItem(hit),
			description: hit['hitSummary'], //$NON-NLS-1$
			uri: hit.uri,
			onclick: getOnClick(hit.uri),
			presentationUri: '#' + dojo.objectToQuery({ //$NON-NLS-1$
				action: 'ple.read_only_product', //$NON-NLS-1$
				product: hit.uri
			})
		});
	}

	return {
		results: formattedResults,
		paging: true,
		totalResults: searchResults.totalResults
	};
}

/**
 * Formats normal PLE search result into the format expected for
 * a jazz.ui.SearchBox (used on Welcome to Products).
 *
 *
 * @searchResults normal PLE search results.
 */
function formatResultsForSearchBox(searchResults) {
	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var formattedResults = [];
	var hits = searchResults.hits;

	for (var i = 0; i < hits.length; i++) {
		var hit = hits[i];

		var label =
			'<span class="ple-ProductsWelcomePage-search-result">' + //$NON-NLS-1$
				'<img height="16" width="16" class="ple-icon ' + productapi.getIconClassForItem(hit) + '" src="' + blankGif +'">' + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				formatting.formatProductTitleForItem(hit) +
			'<span>'; //$NON-NLS-1$

		formattedResults.push({
			label: label,
			handler: dojo.hitch(productapi, productapi.findPathsToProductAndShow, hit.uri)
		});
	}

	return formattedResults;
}

dojo.mixin(com.ibm.relm.ple.web.ui.searching, {

	/**
	 * Returns the standard PLE product scopes as used by banner Quick Search
	 * and jazz.ui.SearchBoxes.
	 */
	getProductScopes: function() {
		return [
			{
				id: 'newestProductsAndConfigurations', //$NON-NLS-1$
				label: messages.productsScope,
				iconClass: 'ple-icon-product' //$NON-NLS-1$
			}
		];
	},

	getWelcomeProductScopes: function() {
		return [
			{
				id: 'newestProductsAndConfigurations', //$NON-NLS-1$
				label: messages.newestProductsScope,
				icon: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/product_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
			},
			{
				id: 'allProductsAndConfigurations', //$NON-NLS-1$
				label: messages.allProductsScope,
				icon: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/product_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
			}
		];
	},

	/**
	 * The "search on update" callback used as the user is typing to show
	 * drop down results from the banner Quick Search box.
	 * @args keyword args as passed from Quick Search.
	 */
	searchOnUpdateForBannerQuickSearch: function(args) {
		var query = args.query;
		var scope = args.scope;
		var pageSize = args.maxResults;
		var pageNumber = args.page;
		var callback = args.callback;
		var errback = args.errback;

		jazz.client.xhrGet({
			handleAs: 'json', //$NON-NLS-1$
			url: net.jazz.ajax._contextRoot + '/ple/product_search', //$NON-NLS-1$
			content: {
				q: query,
				pageSize: pageSize,
				pageNumber: pageNumber,
				scope: scope
			},

			load: function(searchResults) {
				callback(formatResultsForSearchProvider(searchResults));
			},

			error: function(_, args) {
				var errorSummary = dojo.fromJson(args.xhr.responseText).summary;
				errback(errorSummary);
			}
		});
	},

	/**
	 * The "search on update" callback used as the user is typing to show
	 * drop down results from the banner Quick Search box.
	 * @args keyword args as passed from Quick Search.
	 */
	searchOnUpdateForSearchBox: function(query, scope, callback, errback) {
		jazz.client.xhrGet({
			handleAs: 'json', //$NON-NLS-1$
			url: net.jazz.ajax._contextRoot + '/ple/product_search', //$NON-NLS-1$
			content: {
				q: query,
				scope: scope
			},

			load: function(searchResults) {
				callback(formatResultsForSearchBox(searchResults));
			},

			error: function(_, args) {
				var errorSummary = dojo.fromJson(args.xhr.responseText).summary;
				errback(errorSummary);
			}
		});
	}

});
})();