dojo.provide('com.ibm.relm.ple.web.ui.tree.AltTree'); //$NON-NLS-1$
dojo.provide('com.ibm.relm.ple.web.ui.tree.AltTreeNode'); //$NON-NLS-1$

dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LoadingMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.tree', 'ProductTreeMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeToolbarBatchDrawer');//$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$

(function() {

/**
 * This is just here to create site objects from paths. I don't like the
 * circular depenency betwee this tree and TreeSelection now. Possibly site
 * creation should be moved to another class independent of the selection,
 */
var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;

/**
 * Walks up the DOM from a node or event target to find the first
 * enclosing tree node widget. The DOM node itself can be the widget.
 * @param  {object} eventOrNode DOM event with target or DOM node to.
 * @return {AltTreeNode}             the tree node or null if not found.
 */
function getEnclosingTreeNode(eventOrNode) {
	var node = eventOrNode.target || eventOrNode;
	var treeNode = new dojo.NodeList(node).closest('.ple-AltTree-treeNode')[0]; //$NON-NLS-1$
	if (!treeNode) return null;
	return dijit.byNode(treeNode);
}

var EXPAND_ICON = dojo.moduleUrl("com.ibm.relm.ple.web", "ui/images/treeExpando_plus.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var COLLAPSE_ICON = dojo.moduleUrl("com.ibm.relm.ple.web", "ui/images/treeExpando_minus.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var BLANK_GIF = dojo.moduleUrl('dojo','resources/blank.gif'); //$NON-NLS-1$ //$NON-NLS-2$

var TreeToolbarBatchDrawer = com.ibm.relm.ple.web.ui.tree.TreeToolbarBatchDrawer;

dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.AltTreeNode', //$NON-NLS-1$
	[dijit._Widget, dijit._Templated],
	{
		depth: 0,
		baseClass: 'ple-AltTree-treeNode', //$NON-NLS-1$
		tree: null,
		item: null,

		/**
		 * Indicates if the nodes is loading or not. Access to get/set to ensure
		 * CSS classes are added/removed to show the loading state visually.
		 * @type {Boolean}
		 */
		loading: false,
		
		templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/tree/templates/AltTreeNode.html'), //$NON-NLS-1$ //$NON-NLS-2$

		constructor: function() {
			this._labelId = dijit.getUniqueId('relm-AltTree-label'); //$NON-NLS-1$
		},

		postCreate: function() {
			this.inherited(arguments);
			var store = this.tree.store;

			// We use margin-left here to indent the row. We don't want any leading toolbars to indent
			// which is why it's here, not on the row itself.
			this._expandoNode.style.marginLeft = this.tree.indentPx * this.depth + 'px'; //$NON-NLS-1$
			
			this.updateIcon();
			this.updateLabel();

			var childItems = store.getValue(this.item, 'children'); //$NON-NLS-1$
			this._setChildItems(childItems);

			this._checkHighContrast();
		},

		startup: function() {
			this.tree.toolbarBatchDrawer.addNode(this);
		},

		/**
		 * If this node should have a leading toolbar, subclasses should
		 * return it here.
		 * 
		 * @return {Toolbar} 
		 */
		createLeadingToolbar: function() {
			return null;
		},

		/**
		 * Checks if the leading toolbar for the tree node has been initialized. If it has not been initialized, 
		 * then it is initialized at this time. If the toolbar is already initialized, then nothing happens. 
		 */
		initToolbar: function() {
			if (this.leadingToolbar) return;

			var toolbar = this.createLeadingToolbar();
			if (toolbar) {
				toolbar.placeAt(this._leadingToolbarNode);
				this.leadingToolbar = toolbar;
			}
		},

		_checkHighContrast: function() {
			if (dojo.hasClass(document.body, "dijit_a11y")) { //$NON-NLS-1$
				if (dojo.hasClass(this.rowNode, "ple-AltTree-openChildren")) //$NON-NLS-1$
					this._expandoNode.src = COLLAPSE_ICON;
				else if (dojo.hasClass(this.rowNode, "ple-AltTree-closedChildren")) { //$NON-NLS-1$
					this._expandoNode.src = EXPAND_ICON;
				}
			}
		},

		_handleClickHighContrast: function() {
			if (dojo.hasClass(document.body, "dijit_a11y")) { //$NON-NLS-1$
				this._expandoNode.src = this.isOpen ? COLLAPSE_ICON : EXPAND_ICON;
			}
		},

		_resetBlankIcon: function() {
			this._expandoNode.src = BLANK_GIF; 
		},

		/**
		 * Returns the CSS class to provide an icon for this node. Subclasses must override.
		 * @return {string} CSS class name.
		 */
		getIconClassAndAltText: function()
		{
	    	return ['', '']; //$NON-NLS-1$ //$NON-NLS-2$
		},

		/**
		 * Updates the HTML label for this node from the current item.
		 * Defers to this.getLabelHtml for the actual contents.
		 */
		updateLabel: function() {
			dojo.empty(this.labelNode);

			this.labelNode.innerHTML = this.getLabelHtml();
		},

		updateIcon: function() {
			var classAndAltText = this.getIconClassAndAltText();
			if (classAndAltText) {
				dojo.attr(this._iconNode, {
					className: 'ple-AltTree-icon ' + classAndAltText[0], //$NON-NLS-1$
					src: this._blankGif,
					alt: classAndAltText[1]
				});
			}
			else {
				// Shouldn't hit this path anymore.
				throw new Error('Tree node doesn not support .getIconClassAndAltText'); //$NON-NLS-1$
			}
		},

		updateToolbars: function() {
			
		},

		/**
		 * Returns all the child AltTreeNodes of this node. @return {array} all children, or empty array.
		 */
		getChildTreeNodes: function() {
			if (!this.childrenNode) return [];


			var children = this.childrenNode.children;
			var childTreeNodes = [];

			for (var i = 0; i < children.length; i++) {
				var child = children[i];
				childTreeNodes.push(dijit.byNode(child));
			}

			return childTreeNodes;
		},

		/**
		 * Destroys all child AltTreeNodes and removes this.childrenNode
		 * if present.
		 * @return {[type]} [description]
		 */
		_destroyChildTreeNodes: function() {
			dojo.forEach(this.getChildTreeNodes(), function(childTreeNode) {
				childTreeNode.destroyRecursive();
			});

			this.childrenNode && dojo.destroy(this.childrenNode);
			delete this.childrenNode;
		},

		_setLoadingAttr: function(isLoading) {
			this.loading = isLoading;
			dojo.toggleClass(this.domNode, 'ple-AltTree-loading', isLoading); //$NON-NLS-1$
			// dojo[isLoading ? 'addClass' : 'removeClass'](this.domNode, 'ple-AltTree-loading');
			dijit.setWaiState(this.labelNode, 'busy', isLoading); //$NON-NLS-1$
		},

		/**
		 * Sets tab focus on this node. Really the label.
		 */
		focus: function() {
			this.initToolbar();
			this.labelNode.focus();
		},

		/**
		 * Gets the first child tree node.
		 * @return {AltTreeNode} tree node or null if no children.
		 */
		firstChildTreeNode: function() {
			return dijit.byNode(this.childrenNode.firstChild);
		},

		/**
		 * Gets the last child tree node.
		 * @return {AltTreeNode} tree node or null if no children.
		 */
		lastChildTreeNode: function() {
			return dijit.byNode(this.childrenNode.lastChild);
		},

		/**
		 * Gets this nodes parent AltTreeNode or null if none.
		 * @return {AltTreeNode} parent node or null.
		 */
		parentTreeNode: function() {
			var childList = this.domNode.parentNode;

			// We're either under a DIV of a TreeNode, or under the top level of the tree itself
			if (childList == this.tree.domNode) return null;

			var parentNode = childList.parentNode;
			return dijit.getEnclosingWidget(parentNode);
		},

		/**
		 * @return {AltTreeNode} next sibling AltTreeNode or null if none.
		 */
		nextTreeNode: function() {
			var next = this.domNode.nextSibling;
			return !!next  ? dijit.byNode(next) : null;
		},

		/**
		 * @return {AltTreeNode} previous sibling AltTreeNode or null if none.
		 */
		previousTreeNode: function() {
			var prev = this.domNode.previousSibling;
			return !!prev  ? dijit.byNode(prev) : null;
		},

		/**
		 * Gets the tree node visually above this node. This can be the previous sibling or parent.
		 * @return {AltTreeNode} node above or null.
		 */
		upTreeNode: function() {
			var prev = this.previousTreeNode();

			while (prev && prev.isOpen) {
				prev = prev.lastChildTreeNode();
			}

			return prev ? prev : this.parentTreeNode();
		},

		/**
		 * Gets the tree node visually below this node.
		 * @return {AltTreeNode} node above or null.
		 */
		downTreeNode: function() {
			if (this.isOpen) {
				var child = this.firstChildTreeNode();
				if (child) return child;
			}

			// Start with this node, then go up to parents while searching for next
			var current = this;

			do {
				var next = current.nextTreeNode();
				if (next) return next;

				current = current.parentTreeNode();
			} while (current);

			return null;
		},

		/**
		 * Closes an already expanded open node with an animation.
		 * Updates CSS as needed.
		 */
		close: function() {
			if (this.loading) return false;
			if (!dojo.hasClass(this.rowNode, 'ple-AltTree-openChildren')) return false; //$NON-NLS-1$

			this.isOpen = false;
			dojo.removeClass(this.rowNode, 'ple-AltTree-openChildren'); //$NON-NLS-1$
			dojo.addClass(this.rowNode, 'ple-AltTree-closedChildren'); //$NON-NLS-1$
			this._handleClickHighContrast();
			
			if (this.childrenNode) {
				this._currentAnimation && this._currentAnimation.stop();
				
				this._currentAnimation = dojo.animateProperty({
					node: this.childrenNode,
					duration: 200,
					properties: {
						height: {
							end: 0
						}
					},
					onEnd: dojo.hitch(this, function() {
						dojo.addClass(this.childrenNode, 'j-hide'); //$NON-NLS-1$
						dijit.setWaiState(this.labelNode, 'expanded', false); //$NON-NLS-1$
					})
				}).play();
			}

			return true;
		},

		/**
		 * Creates or updates this node's children to match the given child items.
		 * Doesn't touch CSS styles.
		 * @param  {array} newChildItems new child items.
		 */
		_createChildTreeNodes: function(newChildItems) {
			if (newChildItems && newChildItems.length) {
				if (!this.childrenNode) {
					this.childrenNode = dojo.create('div', { //$NON-NLS-1$
						className: 'ple-AltTree-children j-hide', //$NON-NLS-1$
						role: 'group' //$NON-NLS-1$
					}, this.domNode);
				}
			}

			this.tree._createChildTreeNodesInContainer(this.childrenNode, newChildItems, this.getChildTreeNodes(), this.depth + 1);
		},

		/**
		 * Sets the child items for this node. This will update CSS as well as create
		 * child nodes as needed.
		 * @param {array} childItems child Items.
		 */
		_setChildItems: function(childItems) {
			// Catch and ignore store notifications if we're already waiting for a load
			if (this.loading) return;

			if (childItems === true || (childItems && childItems.length)) {
				dojo.removeClass(this.rowNode, 'ple-AltTree-noChildren'); //$NON-NLS-1$

				// For existing nodes, we need to preserve their open/closed state
				dojo.toggleClass(this.rowNode, 'ple-AltTree-openChildren', !!this.isOpen); //$NON-NLS-1$
				dojo.toggleClass(this.rowNode, 'ple-AltTree-closedChildren', !this.isOpen); //$NON-NLS-1$
				this._handleClickHighContrast();

				if (dojo.isArray(childItems)) {
					this._createChildTreeNodes(childItems);
					this.loaded = true;
				}
				else {
					this.loaded = false;
				}
			}
			else {
				dojo.removeClass(this.rowNode, 'ple-AltTree-closedChildren'); //$NON-NLS-1$
				dojo.removeClass(this.rowNode, 'ple-AltTree-openChildren'); //$NON-NLS-1$
				dojo.addClass(this.rowNode, 'ple-AltTree-noChildren'); //$NON-NLS-1$
				this._resetBlankIcon();
				this.isOpen = false;

				this._destroyChildTreeNodes();
				this.loaded = true;
			}

			return;
		},

		/**
		 * Visually expands this node with an animation. The node mujst already be loaded.
		 */
		_expand: function() {
			if (!this.childrenNode) {
				// There are no children, so bail. We shouldn't get here
				dojo.removeClass(this.rowNode, 'ple-AltTree-closedChildren'); //$NON-NLS-1$
				dojo.addClass(this.rowNode, 'ple-AltTree-noChildren'); //$NON-NLS-1$
				this._resetBlankIcon();
				return;
			}

			this.isOpen = true;

			dojo.removeClass(this.rowNode, 'ple-AltTree-noChildren'); //$NON-NLS-1$
			dojo.removeClass(this.rowNode, 'ple-AltTree-closedChildren'); //$NON-NLS-1$
			dojo.addClass(this.rowNode, 'ple-AltTree-openChildren'); //$NON-NLS-1$
			this._handleClickHighContrast();

			dojo.removeClass(this.childrenNode, 'j-hide'); //$NON-NLS-1$
			this.childrenNode.style.height = 'auto'; //$NON-NLS-1$
			
			this._currentAnimation && this._currentAnimation.stop();
			
			this._currentAnimation = dojo.animateProperty({
				node: this.childrenNode,
				duration: 200,
				properties: {
					height: {
						start: 0
					}
				},
				onEnd: dojo.hitch(this, function() {
					this.childrenNode.style.height = 'auto'; //$NON-NLS-1$
					dijit.setWaiState(this.labelNode, 'expanded', true); //$NON-NLS-1$
				})
			}).play();
		},

		/**
		 * Loads an unloaded or partially loaded node then expands it once loaded.
		 * @return {[type]} [description]
		 */
		_loadAndExpand: function() {
			if (this.loading) return this._deferredOpen;
			this.set('loading', true); //$NON-NLS-1$

			var deferred = new dojo.Deferred();
			this._deferredOpen = deferred;

			this.tree.store.loadItem({
				item: this.item,
				onItem: dojo.hitch(this, function(fullItem) {
					this.loaded = true;
					delete this._deferredOpen;

					var childItems = fullItem.children;
					this.set('loading', false); //$NON-NLS-1$
					delete this._deferredOpen;
					this._setChildItems(childItems);
					this._expand();
					deferred.callback();
				}),
				onError: dojo.hitch(this, function() {
					this.set('loading', false); //$NON-NLS-1$
					delete this._deferredOpen;
					deferred.errback();
				})
			});

			return deferred;
		},

		/**
		 * Opens a node. Will load it first if needed. Either way, will expand the node visually.
		 */
		open: function() {
			var deferred = new dojo.Deferred();

			if (dojo.hasClass(this.rowNode, 'ple-AltTree-openChildren') || //$NON-NLS-1$
				dojo.hasClass(this.rowNode, 'ple-AltTree-noChildren') || //$NON-NLS-1$
				this.get('loading')) { //$NON-NLS-1$
				setTimeout(dojo.hitch(deferred, deferred.callback, true), 0);
				return deferred;		
			}

			if (this.loaded && this.tree.store.isItemLoaded(this.item)) {
				this._expand();
				deferred.callback();
			}
			else {
				this._loadAndExpand().then(function() {
					deferred.callback();
				});
			}

			return deferred;
		},

		/**
		 * Gets the path to this node in the tree.
		 * @return {array} array of parent Items in order from top to bottom.
		 */
		getItemPath: function() {
			var itemPath = [];
			var treeNode = this;

			while (treeNode) {
				itemPath.unshift(treeNode.item);
				treeNode = treeNode.parentTreeNode();
			}

			return itemPath;
		},

		/**
		 * Gets thes the site objec for this node. Most classes should use the
		 * TreeSelection class to get selected item. This is only here for the
		 * tree's internal use when calling other methods that want a site.
		 * @return {site} site description object for the this node.
		 */
		_getSite: function() {
			var path = this.getItemPath();	
			return TreeSelection.newSitesFromItemPaths([path])[0];
		},

		/**
		 * Redraws this noded based on the current item attributes
		 * and values. This updates the label, icon and toolbar if needed.
		 * @return {[type]} [description]
		 */
		updateDisplay: function() {
			this.updateLabel();
			this.updateIcon();
			this.updateToolbars();
		},

		/**
		 * Sets the data store item for this node. Updates the UI 
		 * of the new to match, including creating (recursive) children if needed.
		 * @param {Item} newItem the new item.
		 */
		_setItemAttr: function(newItem) {
			this.item = newItem;
			this.updateDisplay();			
			// TODO: get children from store
			this._setChildItems(newItem.children);
		},

		uninitialize: function() {
			this._leadingToolbar && this._leadingToolbar.destroyRecursive();
			this._destroyChildTreeNodes();

			this.tree._cleanUpTreeNode(this);
		}
	}
);

dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.AltTree', //$NON-NLS-1$
	[dijit._Widget],
	{
		baseClass: 'ple-AltTree-showGrid', //$NON-NLS-1$
		store: null,
		rootQuery: null,

		treeMessages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.tree', 'ProductTreeMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		/**
		 * How far to indent child nodes in pixels.
		 * @type {Number}
		 */
		indentPx: 22,
		
		constructor: function(args, domNode)
		{
			this._selectedTreeNodes = [];
			this._queryCount = 0;
			this.rootTreeNodes = [];
			this._focusOnTypeString = ''; //$NON-NLS-1$
			this.toolbarBatchDrawer = new TreeToolbarBatchDrawer();
		},
		
		buildRendering: function() 	{
			this.domNode = dojo.create('div', { //$NON-NLS-1$
				className: 'ple-AltTree' //$NON-NLS-1$
			});
			this.inherited(arguments);
		},
		
		postCreate: function()
		{
			dojo.attr(this.domNode, 'role', 'tree'); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.domNode, 'onclick', this._dispatchClick); //$NON-NLS-1$
			this.connect(this.domNode, 'ondblclick', this._onDlbClick); //$NON-NLS-1$
			
			this.connect(this.domNode, 'onkeydown', this._onKeyDown); //$NON-NLS-1$
		},

		/**
		 * Gets paths to all the deepest nodes that have been loaded already.
		 * This represent all the nodes (the leaves of the paths, plus the intermediate nodes)
		 * that would need to be reloaded during a refresh.
		 * 
		 * @return {array} forward slash separated paths to all all the lowest level, loaded nodes.
		 */
		getLoadedPaths: function() {
			function joinPath(path, component) {
				if (path === '') { //$NON-NLS-1$
					return component;
				}
				else {
					return path + '/' + component; //$NON-NLS-1$
				}
			}

			var openPathSet = {};

			function recurseLoadedPaths(parentPath, fromTreeNodes) {
				dojo.forEach(fromTreeNodes, function(treeNode) {
					// We check for a .childrenNode to know if the item is loaded or not already.
					// There is a .loaded property, but its giving bad results. I don't know if it's
					// not always being set at the right times, or there is something I'm missing. 
					// This is reliable though since the childrenNode is only created after loading
					// something
					if (!treeNode.childrenNode) {
						// Only return the parent paths. This node
						// will be looked up as part of loading the parent, but we don't
						// want to load it and its children, so don't include it directly.
						if (parentPath !== '') { //$NON-NLS-1$
							openPathSet[parentPath] = true;
						}
					}
					else {
						var nodePath = joinPath(parentPath, treeNode.item.pathSegment);
						recurseLoadedPaths(nodePath, treeNode.getChildTreeNodes());
					}
				}, this);
			}

			recurseLoadedPaths('', this.rootTreeNodes); //$NON-NLS-1$
			return openPathSet;
		},

		/**
		 * Return all nodes that a visible, that is the top-level ones, the
		 * children of the open nodes, and so on. Used to determine DnD
		 * targets.
		 *
		 * @param {function} excludeFunction optional function to exclude
		 *     nodes from the results. If present, will be called with the
		 *     `item` for the current tree node being examined. If the
		 *     function returns true, then that node AND all its recursive
		 *     children are also excluded and not returned in the results.
		 *     That is, it excludes entire branches.
		 *     
		 * @return {array} visible AltTreeNodes that are not exclude..
		 */
		getVisibleNodes: function(excludeFunction) {
			var visibleNodes = this.rootTreeNodes.filter(function(treeNode) {
				return !excludeFunction || !excludeFunction(treeNode.item);
			});

			function recurseVisibleNodes(fromNodes) {
				for (var i = 0; i < fromNodes.length; i++) {
					var treeNode = fromNodes[i];

					if (excludeFunction && excludeFunction(treeNode.item)) {
						continue;
					}

					if (treeNode.isOpen) {
						var childNodes = treeNode.getChildTreeNodes();
						// Extend the existing array by using 'apply' to unpack the child nodes
						visibleNodes.push.apply(visibleNodes, childNodes);
						recurseVisibleNodes(childNodes);
					}
				}
			}

			recurseVisibleNodes(visibleNodes);
			return visibleNodes;	
		},

		walkAllTreeNodes: function(onTreeNode) {
			function walk(treeNodes) {
				dojo.forEach(treeNodes, function(treeNode) {
					dojo.hitch(null, onTreeNode)(treeNode);

					var childNodes = treeNode.getChildTreeNodes();
					walk(childNodes);
				});
			}

			walk(this.rootTreeNodes);
		},

		/**
		 * Sets the data store for this tree. May start listening for updates from the store.
		 * @param {dojo.Store} newStore
		 */
		_setStoreAttr: function(newStore) {
			this.store = newStore;
		},
		
		/**
		 * Updates the root query and causes it to run again.
		 * @param {[type]} newRootQuery [description]
		 */
		_setRootQueryAttr: function(newRootQuery) {
			this.rootQuery = newRootQuery;
			this.clearAndQueryRoots();
		},

		_handleDownArrow: function(c) {
			var down = this.getFocusedTreeNode().downTreeNode();
			down && this.focusTreeNode(down);
		},
		
		_handleUpArrow: function(c) {
			var up = this.getFocusedTreeNode().upTreeNode();
			up && this.focusTreeNode(up);
		},
		
		/**
		 * Handles all keyboard events in this one handler. This includes
		 * navigation with arrows, and typing to focus on nodes.
		 * @param  {Event} e 
		 */
		_onKeyDown: function(e) {

			var widget = dijit.getEnclosingWidget(e.target);
			
			// These widgets don't seem to stopEvent, but we
			// don't want to double handle this, so specifically check.
			// This lets you stil navigate the toolbar with arrows
			//  without navigating the tree. 
			if (widget instanceof jazz.ui.toolbar.Button) return;

			var c = e.keyCode;

			switch (c) {
			case dojo.keys.DOWN_ARROW:
				dojo.stopEvent(e);
				this._clearFocusOnTypeString();
				this._handleDownArrow(c);
				return;

			case dojo.keys.UP_ARROW:
				dojo.stopEvent(e);
				this._clearFocusOnTypeString();
				this._handleUpArrow(c);
				return;

			case dojo.keys.RIGHT_ARROW:
				this._clearFocusOnTypeString();
				this._onRightArrow(e);
				return;

			case dojo.keys.LEFT_ARROW:
				this._clearFocusOnTypeString();
				this._onLeftArrow(e);
				return;

			case dojo.keys.ENTER:
				this._clearFocusOnTypeString();
				this._onEnter(e);
				return;

			default:
				if (e.ctrlKey || e.altKey) return;
				 
				// If a <space> char comes through here, it's not seen as a click of
				// a toolbar button. Since this doesn't .stopEvent, I can't figure out why.
				// <Enter> always works fine though.
				this._addToFocusOnTypeString(String.fromCharCode(c));
				return;
			}
		},

		/**
		 * Clears the current (invisible) search string used to select nodes as a user
		 * types in the tree.
		 */
		_clearFocusOnTypeString: function() {
			clearTimeout(this._focusOnTypeStringTimeout);
			this._focusOnTypeString = ''; //$NON-NLS-1$
		},

		/**
		 * Focuses on the next tree node at the same level of the currently focused node
		 * that "starts with" the given char. As characters are typed, this is called repeatedly
		 * and keeps appending characters to the current (invisible) search string. After a timeout
		 * (1 second) the search string is cleared and typing will start a new search. This allows
		 * users to focus on nodes by typing their names. As they type more characters, the search
		 * becomes more specific and a different node my be focused.
		 *
		 * There is a special case for when a single character is repeated. Typing "jjj" will go to
		 * the third product starting with "j" as opposed to something literally starting with "jjj".
		 * 
		 * Note that this only focuses the node. You still need to type <Enter> to select it.
		 * 
		 * @param  {string} character a single character as a string to search for. If we haven't reached the timeout,
		 *                    this is added to the current search string.
		 */
		_addToFocusOnTypeString: function(character) {
			clearTimeout(this._focusOnTypeStringTimeout);

			this._focusOnTypeStringTimeout = setTimeout(dojo.hitch(this, function() {
				this._focusOnTypeString = ''; //$NON-NLS-1$
			}), 1000);

			// Maybe not right for all languages, but generally we want to ignore case when searching.
			character = character.toLowerCase();

			// Ignore strings of the same character. This is to allow cycling
			// through products with the same first character by repeatedly
			// hitting that character. Matches Windows file selection.
			if (character !== this._focusOnTypeString) {
				this._focusOnTypeString += character.toLowerCase();
			}

			if (!this.rootTreeNodes.length) return;

			var visibleTreeNodes = [];

			// Just start from the first node and walk visually "down".
			var nextNode = this.rootTreeNodes[0];
			while (nextNode) {
				visibleTreeNodes.push(nextNode);
				nextNode = nextNode.downTreeNode();
			}

			// First we look for cycling when the same char is typed. For example, typing "jjj" should
			// go to the third product starting with "j". If you keep hitting "j", the focus will cycle
			// between all the products starting with "j".
			var focusedTreeNode = this.getFocusedTreeNode();
			if (this._focusOnTypeString.length === 1 && focusedTreeNode) {

				var visibleTreeNodesWithSameFirstChar = dojo.filter(visibleTreeNodes, function(treeNode) {
					return treeNode.item.name.charAt(0).toLowerCase() === this._focusOnTypeString;
				}, this);

				var currentIndex = dojo.indexOf(visibleTreeNodesWithSameFirstChar, focusedTreeNode);

				if (currentIndex != -1) {
					var nextIndex = (currentIndex + 1) % visibleTreeNodesWithSameFirstChar.length;
					var nextTreeNode = visibleTreeNodesWithSameFirstChar[nextIndex];
					this.focusTreeNode(nextTreeNode);
					return;
				}
			}

			// Cycling failed or was skipped, to move on to search for full prefixes.
			// Starting from the top node, look at all until the first with a
			// matching prefix is found. This matches a multi character search.
			for (var i = 0; i < visibleTreeNodes.length; i++) {
				var treeNode = visibleTreeNodes[i];
				var name = treeNode.item.name.toLowerCase();

				if (name.indexOf(this._focusOnTypeString) === 0) {
					this.focusTreeNode(treeNode);
					return;
				}
			}
		},

		_onEnter: function(e) {
			// Only "click" the node on enter if the label is what's in focus.
			// This prevents firing clicks when toolbar buttons are pressed, since they
			// need to decide for themselves if they need to change selection.
			var focusedTreeNode = this.getFocusedTreeNode();
			if (focusedTreeNode && focusedTreeNode.labelNode == e.target) {
				dojo.stopEvent(e);
				this.selectTreeNode(focusedTreeNode, dojo.isCopyKey(e), e.shiftKey);
				this.onClickTreeNode(focusedTreeNode.item, focusedTreeNode);
			}
		},

		/**
		 * Determines the current tree node with keyboard focus.
		 * Can be slow, so cache the result to a local var.
		 * @return {AlTreeNode} the focused tree node, or null if there is none.
		 */
		getFocusedTreeNode: function() {
			var node = dijit.getFocus().node;
			if (!node) return null;
			var widget = dijit.getEnclosingWidget(node);
			if (!widget || !widget instanceof com.ibm.relm.ple.web.ui.tree.AltTreeNode) return null;
			return widget;
		},

		_onRightArrow: function(e) {
			var focusedTreeNode = this.getFocusedTreeNode();
			if (!focusedTreeNode) return;
			if (e.altKey || e.ctrlKey) return;
			dojo.stopEvent(e);

			if (!focusedTreeNode.isOpen) {
				focusedTreeNode.open();
			}
			else {
				// Was already open, so go down a level
				var firstChild = focusedTreeNode.firstChildTreeNode();
				firstChild && this.focusTreeNode(firstChild);
			}
		},

		_onLeftArrow: function(e) {
			var focusedTreeNode = this.getFocusedTreeNode();
			if (!focusedTreeNode) return;
			if (e.altKey || e.ctrlKey) return;
			dojo.stopEvent(e);

			if (focusedTreeNode.isOpen) {
				focusedTreeNode.close();
			}
			else {
				var parent = focusedTreeNode.parentTreeNode();
				parent && this.focusTreeNode(parent);
			}
		},
		
		/**
		 * Sets tab focus on a given node.
		 * @param  {AltTreeNode} treeNode node to focus on.
		 */
		focusTreeNode: function(treeNode) {
			treeNode.focus();
		},

		/**
		 * Opens all the nodes in a tree for a path. Due to lazy loading,
		 * this returns a deferred and happens async.
		 * 
		 * @param  {array} path array of IDs of the nodes. This will change to equivalenceIds soon!
		 * @return {Deferred}        deferred that's called when entire path
		 *     is done opening. Passes the selected tree node to
		 *     the callback. Errback is called if the path doesn't correspond
		 *     to the tree structure. The path may be partially openeded in
		 *     this case.
		 */
		openPath: function(path, andSelect) {
			var remainingPath = [].concat(path);
			var deferred = new dojo.Deferred();
			var tree = this;

			function openPathRecursive(treeNodes) {
				 var currentPathSegment = remainingPath.shift();

				 if (!currentPathSegment) {
				 	deferred.errback('Exhausted path'); //$NON-NLS-1$
				 	return;
				 }

				for (var i = 0; i < treeNodes.length; i++) {
					var treeNode = treeNodes[i];

					if (currentPathSegment === treeNode.item.pathSegment) {

						if (!remainingPath.length) {
							// Nothing left in path, so this is the leaf node we want
							if (andSelect) {
								tree.focusTreeNode(treeNode);
								tree.selectTreeNode(treeNode);
							}

							deferred.callback(treeNode);
							return;
						}

						treeNode.open().then(function() {
							var childTreeNodes = treeNode.getChildTreeNodes();

							if (!childTreeNodes) {
								deferred.errback('Path not found. No children for node ' + currentPathSegment); //$NON-NLS-1$
								return;
							}

							openPathRecursive(childTreeNodes);
						});

						return;
					}
				}

				deferred.errback('Path not found. Cannot find node for ' + currentPathSegment); //$NON-NLS-1$
				return;
			}

			openPathRecursive(this.rootTreeNodes);
			return deferred;
		},

		/**
		 * Visually clears the selected nodes and clears the backing data. Nothing
		 * will be left selected.
		 */
		clearAllSelectedPaths: function() {
			for (var i = 0; i < this._selectedTreeNodes.length; i++) {
				var treeNode = this._selectedTreeNodes[i];

				// Check that we haven't been destroyed already 
				if (treeNode && treeNode.rowNode) {
					dojo.toggleClass(treeNode.rowNode, 'ple-AltTree-selected', false); //$NON-NLS-1$
					dijit.setWaiState(treeNode.labelNode, 'selected', false); //$NON-NLS-1$
				}
			}

			this._selectedTreeNodes = [];
		},

		/**
		 * Gets the paths to all the selected tree nodes, if any.
		 * @return {array} array of paths, which are themselves arrays of items leading to the selected nodes.
		 */
		getAllSelectedItemPaths: function() {
			return dojo.map(this._selectedTreeNodes, function(treeNode) {
				return treeNode.getItemPath();
			});
		},

		/**
		 * Visually selects the given tree node. Adds to the known selection.
		 * @param  {AltTreeNode} treeNode                        node to select.
		 * @param  {boolean} addOrRemoveFromCurrentSelection if true, then the current selection
		 * is not cleared. If this tree node is not already part of it it is added. If not part, it is removed.
		 * If false, then the selection is always clears first. This supports mult-select when ctrl-clicking.
		 * @param  {boolean} selectRange This supports selecting a range with shift+click. 
		 */
		selectTreeNode: function(treeNode, addOrRemoveFromCurrentSelection, selectRange) {
			dojo.toggleClass(this.domNode, 'ple-AltTree-multiSelect', !!addOrRemoveFromCurrentSelection || !!selectRange); //$NON-NLS-1$
			
			if (!addOrRemoveFromCurrentSelection) {
				this.clearAllSelectedPaths();
			}

			if (selectRange && this._selectAnchor) {
				var comparison = treeNode.domNode.compareDocumentPosition(this._selectAnchor.domNode);
				// Node.compareDocumentPosition returns a bitmask. Doing a bitwise and with 4 (000100) indicates 
				// this._selectAnchor comes after treeNode
				var firstNode, lastNode, currentNode;
				if (comparison & 4) {
					firstNode = treeNode;
					currentNode = treeNode;
					lastNode = this._selectAnchor;
				} else {
					firstNode = this._selectAnchor;
					currentNode = this._selectAnchor;
					lastNode = treeNode;
				}

				while (currentNode != lastNode) {
					this._addSelection(currentNode);
					currentNode = currentNode.downTreeNode();
				}
				this._addSelection(lastNode);

			} else {
				this._selectAnchor = treeNode;

				if (dijit.getWaiState(treeNode.labelNode, 'selected') === 'true') { //$NON-NLS-1$ //$NON-NLS-2$
					this._removeSelection(treeNode);
				}
				else {
					this._addSelection(treeNode);
				}
			}

			this.selectionChanged();
		},

		_addSelection: function(treeNode) {
			dojo.toggleClass(treeNode.rowNode, 'ple-AltTree-selected', true); //$NON-NLS-1$
			dijit.setWaiState(treeNode.labelNode, 'selected', true); //$NON-NLS-1$
			this._selectedTreeNodes.push(treeNode);
		},

		_removeSelection: function(treeNode) {
			dojo.toggleClass(treeNode.rowNode, 'ple-AltTree-selected', false); //$NON-NLS-1$
			dijit.setWaiState(treeNode.labelNode, 'selected', false); //$NON-NLS-1$
			this._selectedTreeNodes = dojo.filter(this._selectedTreeNodes, function(selectedNode) {
				return selectedNode != treeNode;
			});
		},

		/**
		 * Callback to notify listeners when the selection set changes.
		 */
		selectionChanged: function() {
			// callback
		},

		/**
		 * Expands a node on double click.
		 * @param  {Event} e 
		 */
		_onDlbClick: function(e) {
			dojo.stopEvent(e);
			var treeNode = getEnclosingTreeNode(e);
			if (!treeNode) return;

			this.onDblClickTreeNode(treeNode);
			this._toggleTreeNode(treeNode);
		},

		onDblClickTreeNode: function(treeNode) {
			// callback after a double click
		},

		/**
		 * Handles all mouse clicks. Defers to expando or node as needed.
		 * @param  {Event} e mouse event
		 */
		_dispatchClick: function(e) {
			var target = e.target;
			
			if (dojo.hasClass(e.target, 'ple-AltTree-expando')) { //$NON-NLS-1$
				dojo.stopEvent(e);
				this._expandoWasClicked(target);
				return;
			}

			var treeNode = getEnclosingTreeNode(e);

			if (treeNode) {
				this._treeNodeWasClicked(e, treeNode, false, false);
			}
		},
		
		/**
		 * Focus and selects clicked tree nodes.
		 * Calls the onClickTreeNode handler is implemented.
		 * @param  {event} e        mouse event
		 * @param  {AltTreeNode} treeNode clicked tree node.
		 */
		_treeNodeWasClicked: function(e, treeNode, isCopyKey, isShiftKey) {
			this.focusTreeNode(treeNode);
			this.clickTreeNode(treeNode, isCopyKey || dojo.isCopyKey(e), isShiftKey || e.shiftKey);
		},

		clickTreeNode: function(treeNode, isCopyKey, isShiftKey) {
			this.selectTreeNode(treeNode, isCopyKey, isShiftKey);
			this.onClickTreeNode(treeNode.item, treeNode);
		},

		onClickTreeNode: function(item, treeNode) {
			// public callback
		},

		_expandoWasClicked: function(expandoNode) {
			var treeNode = getEnclosingTreeNode(expandoNode);
			this.focusTreeNode(treeNode);
			this._toggleTreeNode(treeNode);
		},
		
		/**
		 * Opens and closes a tree node based on current state.
		 * @param  {AltTreeNode} treeNode 
		 */
		_toggleTreeNode: function(treeNode) {
			if (dojo.hasClass(treeNode.rowNode, 'ple-AltTree-openChildren')) { //$NON-NLS-1$
				treeNode.close();
			}
			else if (dojo.hasClass(treeNode.rowNode, 'ple-AltTree-closedChildren')) { //$NON-NLS-1$
				treeNode.open();
			}
		},

		/**
		 * Destroy all root nodes and their recursive children.
		 */
		_destroyRoots: function() {
			for (var i = 0; i < this.rootTreeNodes.length; i++) {
				var root = this.rootTreeNodes[i];
				root.destroyRecursive();
			}

			this.rootTreeNodes = [];
		},

		appendRootItem: function(newItem) {
			var rootItems = dojo.map(this.rootTreeNodes, function(treeNode) {
				return treeNode.item;
			});
			rootItems.push(newItem);
			this._createRootTreeNodes(rootItems);
		},

		/**
		 * Creates the root tree nodes after a query.
		 * @param  {array} rootItems root Items to create nodes for.
		 */
		_createRootTreeNodes: function(rootItems) {
			this.rootTreeNodes = this._createChildTreeNodesInContainer(this.domNode, rootItems, this.rootTreeNodes, 0);
			this.emptyStateDidChange(rootItems.length === 0);
		},

		/**
		 * Clears the current roots and runs the root query again.
		 * Used after changing the current root query.
		 */
		clearAndQueryRoots: function() {
			this.clearAllSelectedPaths();
			this._destroyRoots();

			this.queryRoots();
		},

		emptyStateDidChange: function(isEmpty) {
			
		},

		setRootItems: function(rootItems) {
			this._createRootTreeNodes(rootItems);
			this.selectionChanged();
		},

		/**
		 * Queries for the root nodes and creates them.
		 * Can be used to query for new roots and udpate them in place.
		 */
		queryRoots: function() {
			this._queryCount++;
			var queryCount = this._queryCount;
			var loading = null;
			this.set('loading', true); //$NON-NLS-1$

			if (!this.rootTreeNodes.length) {
				loading = new com.ibm.relm.common.web.ui.LoadingMessage({
					marginTopBottom: '100px' //$NON-NLS-1$
				});
				loading.placeAt(this.domNode);
			}

			var deferred = new dojo.Deferred();

			this.store.fetch({
				query: this.rootQuery,
				onComplete: dojo.hitch(this, function(rootItems) {
					// check for race-condition where the tree was destroyed while
					// this was running.
					if (!this.domNode) return;
					
					loading && loading.destroyRecursive();
					this.set('loading', false); //$NON-NLS-1$
					this._selectedTreeNodes = [];
					this.selectionChanged();

					if (this._queryCount !== queryCount) {
						// A newer query is running, so don't draw, let it win
						return;
					}

					this._createRootTreeNodes(rootItems);
					deferred.callback();
				}),
				onError:dojo.hitch(this,  function(error, ioArgs) {
					this.set('loading', false); //$NON-NLS-1$
					loading.destroyRecursive();
					dojo.publish("ple.product_tree_xhrError", [ioArgs.xhr, true]); //$NON-NLS-1$
					deferred.errback();
				})
			});

			return deferred;
		},

		/**
		 * Deprecated. Act erractically if a product is bound into multiple places.
		 * Should always set the full path to get expected results.
		 * Will find the FIRST item in the tree that is loaded and has a matching
		 * property with the given value. Selects that item and acts like it was clicked.
		 * 
		 * @param  {string} name  name of propert to find.
		 * @param  {object} value value of the 'name' property to find.
		 */
		selectTreeNodeWithPropertyAndValue: function(name, value) {
			console.warn('Deprecated: Selecting tree nodes based on URI is not reliable. Select based on path instead.'); //$NON-NLS-1$

			var tree = this;

			function recursiveSelect(currentTreeNode) {
				var currentValue = tree.store.getValue(currentTreeNode.item, name);

				if (currentValue == value) {
					tree.selectTreeNode(currentTreeNode);
					tree.onClickTreeNode(currentTreeNode.item, currentTreeNode);
					return true;
				}

				var childNodes = currentTreeNode.getChildTreeNodes();

				for (var i = 0; i < childNodes.length; i++) {
					var childNode = childNodes[i];
					if (recursiveSelect(childNode)) return true;
				}

				return false;
			}

			for (var i = 0; i < this.rootTreeNodes.length; i++) {
				var rootNode = this.rootTreeNodes[i];
				if (recursiveSelect(rootNode)) return;
			}
		},

		/**
		 * Creates new AltTreeNodes from a list of child items. If any of the old tree
		 * nodes are already present for a given item, then they are reused, preserving their
		 * expanded state. Otherwise, old tree nodes are destroyed.
		 *
		 * This method makes more sense on AltTreeNode except that it is also used
		 * for root nodes, which are not contained under an existing tree node.
		 * 	
		 * @param  {DOM node} containerNode the node the tree nodes should be added to.
		 * @param  {array} newChildItems list of store items for the new nodes.
		 * @param  {array} oldTreeNodes  list of AltTreeNodes that are already under the given container.
		 * They will be preseverd if the new items are the same, otherwise unused tree nodes are destroyed.
		 * @param  {number} depth         the depth to create the nodes at. The root is a 0.
		 * @return {array} all tree nodes added to the container, including old, existing ones.
		 */
		_createChildTreeNodesInContainer: function(containerNode, newChildItems, oldTreeNodes, depth) {
			var oldTreeNodeMap = {};

			// Remove and save any old tree nodes.
			// We'll sort an re-add ones that should still be there.
			// Optimization: might be faster if we were to leave them in the tree and sort in place... not sure.
			dojo.forEach(oldTreeNodes, function(oldTreeNode) {
				containerNode.removeChild(oldTreeNode.domNode);
				oldTreeNodeMap[oldTreeNode.item.pathSegment] = oldTreeNode;
			});

			var frag = document.createDocumentFragment();

			var newChildTreeNodes = [];

			dojo.forEach(newChildItems, function(newChildItem) {
				var pathSegment = newChildItem.pathSegment;
				var existingTreeNode = oldTreeNodeMap[pathSegment];

				if (existingTreeNode) {
					delete oldTreeNodeMap[pathSegment];
					newChildTreeNodes.push(existingTreeNode);
					// Dojo 1.8 fails using .placeAt here. Seems to have problem with a doc frag.
					// For now, use dojo.place
					// newChildTreeNode.placeAt(frag);
					dojo.place(existingTreeNode.domNode, frag);

					existingTreeNode.set('item', newChildItem); //$NON-NLS-1$
				}
				else {
					var newChildTreeNode = this.createTreeNode(newChildItem, depth);
					newChildTreeNodes.push(newChildTreeNode);
					dojo.place(newChildTreeNode.domNode, frag);
					newChildTreeNode.startup();
				}
			}, this);

			// Destroy any tree node widgets no longer in use
			for (var stalePathSegment in oldTreeNodeMap) {
				var staleTreeNode = oldTreeNodeMap[stalePathSegment];
				
				// Check if any of the stale node are selected and remove them
				// from the selection set before destroying.
				// Go backwards since we're modifying.
				for (var i = this._selectedTreeNodes.length - 1; i >= 0; i--) {
					var selected = this._selectedTreeNodes[i];
					if (selected === staleTreeNode) {
						this._selectedTreeNodes.splice(i, 1);
					}
				}

				staleTreeNode && staleTreeNode.destroyRecursive();
			}

			if (newChildItems.length) {
				if (containerNode) {
					containerNode.appendChild(frag);
				}
			}

			return newChildTreeNodes;
		},

		/**
		 * Removes any reference the tree might have to a tree node.
		 * Needed when a tree node is destroyed for proper garbage collections.
		 * 
		 * @param  {AltTreeNode} treeNode node to unmap.
		 */
		_cleanUpTreeNode: function(treeNode) {
			for (var i = 0; i < this._selectedTreeNodes.length; i++) {
				var selectedNode = this._selectedTreeNodes[i];
				if (selectedNode == treeNode) {
					// remove from selection
					this._selectedTreeNodes.splice(i, 1);
					this.selectionChanged();
					break;
				}
			}

			for (var j = 0; j < this.rootTreeNodes.length; j++) {
				var rootNode = this.rootTreeNodes[j];
				if (rootNode == treeNode) {
					this._selectedTreeNodes.splice(j, 1);
					break;
				}
			}
		},

		/**
		 * Gets and HTML label for the node. Subclasses more override. 
		 * @param  {Item} item the item for the node.
		 * @return {string}      an HTML label for the node. Be sure to escape special chars that are not tags.
		 */
		getLabelHtml: function(item) {
			return '&lt;label not implemented&gt;'; //$NON-NLS-1$
		},

		/**
		 * Create the tree node LI element with icon, expando,e tc.
		 * @param  {object} item         data store item for the node.
		 * @param  {number} depth        the indentiation depth.
		 * @return {DOM node}              new LI element.
		 */
		createTreeNode: function(item, depth) {

			return new com.ibm.relm.ple.web.ui.tree.AltTreeNode({
				depth: depth,
				tree: this,
				item: item
			});
		},

		/**
		 * Returns the tree node widget at the given ID path.
		 * @param  {array} path path of tree IDs.
		 * @return {AltTreeNode}        null if not found, otherwise the node.
		 */
		getNodeAtPath: function(path) {
			var currentNode = null;
			var currentChildren = this.rootTreeNodes;

			for (var i = 0; i < path.length; i++) {
				var currentPathSegment = path[i];

				// Ensure one of the children matches the path segment we want and update the currentNode if so

				var foundChild = dojo.some(currentChildren, function(childNode) {
					if (childNode.item.pathSegment === currentPathSegment) {
						currentNode = childNode;
						currentChildren = childNode.getChildTreeNodes();
						return true;
					}
				}, this);

				// If any part of the path can't be found, fail
				if (!foundChild) {
					return null;
				}
			}

			return currentNode;
		},

		/**
		 * Flashes a node orange to bring attention to it.
		 * @param  {array} path path of tree IDs.
		 */
		flashNodeAtPath: function(path, flashArgs) {
			var attemptNumber = 0;

			// If this fails, we'll keep retrying every 100 millis.
			// This happens after a new node was added, but the tree isn't quite up to date yet.
			// Gives up after a little while
			var getAndFlash = null;
			getAndFlash = dojo.hitch(this, function() {
				var treeNode = this.getNodeAtPath(path);
				if (treeNode) {
					var parentTreeNode = treeNode.parentTreeNode();
					parentTreeNode && parentTreeNode.open();
					com.ibm.relm.common.web.ui.util.flashNode(treeNode.rowNode, flashArgs);	

					// Since leading toolbar has different background color, we need to fade
					//  is separeately.
					treeNode.initToolbar();
					com.ibm.relm.common.web.ui.util.flashNode(treeNode._leadingToolbarNode, flashArgs);
				}
				else if (attemptNumber < 20) {
					attemptNumber += 1;
					setTimeout(getAndFlash, 100);
				} else {
		 			console.warn("Failed to find tree node to flash"); //$NON-NLS-1$
		 		}
			});

			getAndFlash();
		},

		uninitialize: function() {
			this._destroyRoots();
			this.toolbarBatchDrawer.destroy();
		}
	}
);

com.ibm.relm.ple.web.ui.tree.AltTree.getEnclosingTreeNode = getEnclosingTreeNode;

}());
