/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.tree.BaseProductTree'); //$NON-NLS-1$
dojo.provide('com.ibm.relm.ple.web.ui.tree.BaseProductTreeNode'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.tree', 'ProductTreeMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('dijit.Tree'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.ToolbarTreeNode'); //$NON-NLS-1$
dojo.require('jazz.ui.ResourceLink'); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.ui.formatting"); //$NON-NLS-1$
dojo.require("com.ibm.relm.ple.web.doors.DoorsUtils"); //$NON-NLS-1$

(function ()
{
	var formatting = com.ibm.relm.ple.web.ui.formatting;

	function removeFromStore(item) {
		if (item._removedFromStore) return item._removedFromStore;

		var plainItem = {};

		for (var p in item) {
			var rawValue = item[p];
			if (rawValue && rawValue.length === 1) {
				plainItem[p] = rawValue[0];
			}
		}

		item._removedFromStore = plainItem;
		return plainItem;
	}

	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.BaseProductTreeNode', //$NON-NLS-1$
		[ com.ibm.relm.ple.web.ui.tree.ToolbarTreeNode ],
		{
			postCreate: function() {
				this.inherited(arguments);
				this.updateHtmlLabel();

				if (!this.item.root) {
					var store = this.tree.model.store;
					var type = store.getValue(this.item, 'type'); //$NON-NLS-1$
					var isDoorsBaseline = store.getValue(this.item, 'isDoorsBaseline'); //$NON-NLS-1$

					if (type === 'link') { //$NON-NLS-1$
						// For links, we want to shown a rich hover, so don't
						// use dijit.Treee classes for icons. Start with
						// generic link icon. When a compact preview is
						// available, this will get replaced with the real
						// icon.
						dojo.removeClass(this.iconNode, 'ple-icon-artifactLink'); //$NON-NLS-1$
						if (!isDoorsBaseline) {
							this.iconNode.src = dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'); //$NON-NLS-1$ //$NON-NLS-2$ ;
						} else {
							this.iconNode.src = com.ibm.relm.ple.web.doors.DoorsUtils.DOORS_BASELINE_ICON;
						}

						var uri = store.getValue(this.item, 'uri'); //$NON-NLS-1$
						var isDeleted = store.getValue(this.item, '_isDeleted'); //$NON-NLS-1$

						// Deleted links don't exists, so don't try to preview them.
						if (!isDeleted && !isDoorsBaseline && uri) {
							var link = dojo.query('a', this.labelNode)[0]; //$NON-NLS-1$
							
							new jazz.ui.ResourceLink({
								iconNode: this.iconNode
							}, link);
						}
					}
				}
			},

			updateHtmlLabel: function() {
				// The default label from dijit.Tree is just text. We want a full HTML label
				// for grayed out branch and snapshot names. Assume this is a product.
				// TODO: this need to be updated if the store updates the branch, snapshot, or product title.
				// Current this is only created once.

				dojo.empty(this.labelNode);

				this.labelNode.innerHTML = this._formatHtmlLabel();
			},

			getLabel: function() {
				// We never use the default text label.
				return undefined;
			},

			/**
			 * Override when label is set (with .set method) and build the HTML label instead.
			 */
			_setLabelAttr: function(labelString) {
				// labelString should always be undefined, since it comes from getLabel above.
				// Instead, update the HTML label.
				this.updateHtmlLabel();
			},

			/**
			 * Returns an HTML label to use for this node's title.
			 * The default dijit.Tree uses only a simple string.
			 * Dojo examples actually recomend this hack for using HTML (i.e., replacing
			 * the innerHTML of the labelNode). This does break the binding between the 'name' property
			 * and the label. May need to fix that later. Also, a similar technique needs to be used in the
			 * My Products trees.
			 */
			_formatHtmlLabel: function() {
				if (this.item.root) {
					return "ROOT"; //$NON-NLS-1$
				}

				var store = this.tree.model.store;
				var title = store.getValue(this.item, 'name'); //$NON-NLS-1$
				var type = store.getValue(this.item, 'type'); //$NON-NLS-1$
				var isDeleted = store.getValue(this.item, '_isDeleted'); //$NON-NLS-1$
				var isDoorsBaseline = store.getValue(this.item, 'isDoorsBaseline'); //$NON-NLS-1$

				var titleString;
				if (isDoorsBaseline) {
					titleString = title;
				} else {
					titleString = jazz.util.html.escape(title);
				}

				if (isDeleted) {
					// special case for Compare Tree. Should probably move to special subclass for that view only.
					return '<span class="ple-deleted">' + titleString + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$
				}
				else if (type === 'link') { //$NON-NLS-1$
					var uri = store.getValue(this.item, 'uri'); //$NON-NLS-1$
					// Use a link directly to the artifact. Required for jazz.ui.ResourceLink.
					return '<a class="ple-valign-middle" targer="_blank" href="' + uri + '">' + titleString + '</a>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				else {
					return formatting.formatProductTitleForItem(removeFromStore(this.item));
				}
			}
		}
	);

	dijit.Tree.prototype.isExpandoNode = function(node, widget){
		// summary:
		//		check whether a dom node is the expandoNode for a particular TreeNode widget
		var isExpandoNode = dojo.isDescendant(node, widget.expandoNode) || dojo.isDescendant(node, widget.expandoNodeText); 
		return isExpandoNode;
	};

	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.BaseProductTree', //$NON-NLS-1$
		[dijit.Tree],
		{

			_createTreeNode: function(args) {
				return new com.ibm.relm.ple.web.ui.tree.BaseProductTreeNode(args);
			},

			_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){

				if (e.target.nodeName == 'A') { //$NON-NLS-1$
					// Do the default browser action when clicking a link.
					return;
				}

				this.inherited(arguments);
			},

			getIconClass: function(item) {
				if (!item || !item.type) {
					return 'ple-icon-blank'; //$NON-NLS-1$
				}
				var type = this.model.store.getValue(item, 'type'); //$NON-NLS-1$

				return formatting.getIconClassForItem(removeFromStore(item));

				// Cannot add class for disabled hierarchy here since the artifacts with icons don't come through this
				if (type === 'link') //$NON-NLS-1$
				{
					return 'ple-icon-artifactLink'; //$NON-NLS-1$
				}
				else
				{
					var rtcStream = this.model.store.getValue(item, 'rtcStream'); //$NON-NLS-1$
					var immutable = this.model.store.getValue(item, 'isImmutable'); //$NON-NLS-1$
					
					if (rtcStream) {
						if (immutable){
							return 'ple-icon-productStreamCheckedIn'; //$NON-NLS-1$
						} else {
							return 'ple-icon-productStream'; //$NON-NLS-1$
						}
					}

					if (item.rtcComponent) {
						if (immutable){
							return 'ple-icon-productComponentCheckedIn'; //$NON-NLS-1$
						} else {
						return 'ple-icon-productComponent'; //$NON-NLS-1$
						}
					}

					if (item.conditionVariables !== undefined) {
						return 'ple-icon-rtcConfiguration'; //$NON-NLS-1$
					}

					return immutable ?
						'ple-icon-productInSnapshot' : //$NON-NLS-1$
						'ple-icon-product'; //$NON-NLS-1$
				}
			}
		}
	);
})();
