/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.tree.SimpleProductStore'); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil"); //$NON-NLS-1$

(function() {

var SystemParameterUtil = com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil;

/**
 * A minimal data store that doesn't actually store anything. 
 * All items from this store can and should be used directly, since they 
 * are not cached here (i.e., no need for .getValue). This exists to provide compatability
 * with older code that assumes a store must be used to access product items, and as a way
 * to continue to support lazy loading.
 *
 * This does not claim to be a full dojo data store, and is only
 * usable by the product tree.
 */
dojo.declare(
	'com.ibm.relm.ple.web.ui.tree.SimpleProductStore', //$NON-NLS-1$
	null,

	{
		constructor: function(args)
		{
			this._productTreeService = net.jazz.ajax._contextRoot + '/ple/product_tree/'; //$NON-NLS-1$
		},

		getValue: function(item, attribute) {
			if (!item) { console.trace(); }
			return item[attribute];
		},

		isItemLoaded: function(item) {
			return item.children !== true;
		},

		loadItem: function(args) {
			var item = args.item;

			if (this.isItemLoaded(item) && !args.forceLoad) {
				if (args.onItem) {
					dojo.hitch(args.scope, args.onItem)(item);
				}
				return;
			}
			
			if (item.type === 'configVersion') { //$NON-NLS-1$
				var contextConfigVersionUri = item.uri;
				var contextPartUri = null;
			}
			else {
				contextConfigVersionUri = item.contextConfigVersionUri;
				contextPartUri = item.type === 'configVersion' ? null : item.contextPartUri; //$NON-NLS-1$
			}

			var parameters = {
				// 5.0 properties
				contextConfigVersionUri: contextConfigVersionUri,
				contextPartUri: contextPartUri,

				includeUserProperties: args.includeUserProperties,

				// 4.0.6 properties
				itemVersionUri: item.uri,
				viewMemberUri: item['pd:part'] //$NON-NLS-1$
			};

			// When dealing with a config version directly, use itself as the context.
			// This matches the API assumption.
			if (item.type === 'configVersion') { //$NON-NLS-1$
				parameters.contextConfigVersionUri = item.uri;
			}

			//TODO: Remove when prototype no longer needed
			if (item['http://jazz.net/ns/vvc#configuration_fake_inherited'] !== undefined) //$NON-NLS-1$
				parameters['http://jazz.net/ns/vvc#configuration_fake_inherited'] =  item['http://jazz.net/ns/vvc#configuration_fake_inherited']; //$NON-NLS-2$ //$NON-NLS-1$

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._productTreeService,
				content: parameters,
				
				load: dojo.hitch(this, function (fullItem) {
					// Lazy-load is only for children now. Don't mixin other properties anymore
					// since will override the pathSegment in some cases. Just grab the children exactly.
					delete item.children;
					item.children = fullItem.children;
					args.onItem && dojo.hitch(args.scope, args.onItem)(item);
				}),

				error: function(error, ioArgs) {
					args.onError && dojo.hitch(args.scope, args.onError)(error, ioArgs);
				}
			});
		},

		fetchItemByRootPathSegment: function(args) {
			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._productTreeService,
				content: {
					rootPathSegment: args.pathSegment
				},

				load: dojo.hitch(this, function (newItem) {
					if (args.onItem) {
						dojo.hitch(args.scope, args.onItem)(newItem);
					}
				}),

				error: function(error) {
					console.error(error);
					args.onError && dojo.hitch(args.scope, args.onError)(error);
				}
			});
		},

		fetch: function(args) {
			var startTime = new Date();

			var content = {};

			if (args.query) {
				content.filter = args.query;

				// Required if the `filter` is a product grouping URI.
				content.parameters = dojo.toJson(SystemParameterUtil.getSystemParameterValues());
			}

			jazz.client.xhrGet({
				handleAs: 'json', //$NON-NLS-1$
				url: this._productTreeService,
				content: content,

				load: dojo.hitch(this, function (newItems) {
					var fetchedTime = new Date();
					if (args.onItem) {
						dojo.forEach(newItems, function(item) {
							args.onItem(item);
						});
					}

					args.onComplete && args.onComplete(newItems, args);
					var renderedTime = new Date();

					var fetchTime = (fetchedTime - startTime);
					var renderTime = (renderedTime - fetchedTime);
					var totalTime = (renderedTime - startTime);

					if (window.djConfig.isDebug) {
						console.debug("Fetch Time:", fetchTime); //$NON-NLS-1$
						console.debug("Render Time:", renderTime); //$NON-NLS-1$
						console.debug("Total Time:", totalTime); //$NON-NLS-1$
					}
				}),

				error: function(error, ioArgs) {
					console.error(error);
					args.onError && dojo.hitch(args.scope, args.onError)(error, ioArgs);
				}
			});
		}
	}
);
})();
