/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.tree.ToolbarTreeNode"); //$NON-NLS-1$

dojo.require('dijit.Tree'); //$NON-NLS-1$

(function(){

	/**
	 * A tree node row with a Jazz toolbar embedded in it. Most of the work is 
	 * getting the CSS styles right to hide and show the toolbar at the right times
	 * and support tabbing to hidden toolbars (which will show them).
	 * 
	 * The toolbar is exposed as a public attribute named 'toolbar'. It has no
	 * buttons by default.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.ToolbarTreeNode', //$NON-NLS-1$
		[ dijit._TreeNode ], 	
		{
			postCreate: function() {
				this.inherited(arguments);
				
				dojo.addClass(this.domNode, 'ple-ToolbarTreeNode'); //$NON-NLS-1$

				// if keyboard focus moves to toolbar, but sure to make it visible
				this.connect(this.toolbar, 'onFocus', this._onToolbarFocus); //$NON-NLS-1$
				this.connect(this.toolbar, 'onBlur', this._onToolbarBlur); //$NON-NLS-1$

				this.toolbar = new jazz.ui.Toolbar();
				dojo.addClass(this.toolbar.domNode, 'ple-ToolbarTreeNode-toolbar'); //$NON-NLS-1$
				this.toolbar.placeAt(this.rowNode, 'left'); //$NON-NLS-1$
				this.toolbar.startup();

				// if keyboard focus moves to toolbar, but sure to make it visible
				this.connect(this.toolbar, 'onFocus', this._onToolbarFocus); //$NON-NLS-1$
				this.connect(this.toolbar, 'onBlur', this._onToolbarBlur); //$NON-NLS-1$
			},

			_onToolbarFocus: function() {
				dojo.addClass(this.toolbar.domNode, 'ple-ToolbarTreeNode-toolbar-focused'); //$NON-NLS-1$
			},

			_onToolbarBlur: function() {
				dojo.removeClass(this.toolbar.domNode, 'ple-ToolbarTreeNode-toolbar-focused'); //$NON-NLS-1$
			},

			unititialize: function() {
				this.toolbar.destroyRecursive();
			}
		}	
	);

})();