/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 

dojo.provide("com.ibm.relm.ple.web.ui.tree.TreeToolbarBatchDrawer");//$NON-NLS-1$

(function() {
	var BATCH_SIZE =  50;

	dojo.declare("com.ibm.relm.ple.web.ui.tree.TreeToolbarBatchDrawer", [], { //$NON-NLS-1$

		constructor: function() {
			this.remainingNodes = [];
			this.processed = 0;
			this.startTime = Date.now();
			this.scheduleCount = 0;
			this.startBatchCount = 0;
		},

		addNode: function(node) {
			this.remainingNodes.push(node);
			this.schedule();
		},

		/**
		 * This method should draw nodes in two cases:
		 * 	1. The number of remaining nodes is < BATCH_SIZE. A timeout is set to wait for more nodes to be added. If more 
		 * 	nodes are added and the number of remaining nodes to process is still < BATCH_SIZE, then the timeout is reset. After the timeout 
		 * 	is exhausted, this means that we haven't gotten a full batch yet for a little while, so rather than wait, we will start processing
		 * 	whatever nodes we have.
		 *
		 * 	2. The number of remaining nodes is > BATCH_SIZE. 
		 * 	
		 * 		a. If this is the first batch we are processing, then we immediately
		 *	 	start drawing the toolbars on those nodes. That way the first batch of nodes drawn always have a toolbar.
		 *
		 * 		b. If this isn't the first batch, then we have already started processing nodes. In this case, we set 
		 * 	 	a timeout and at the end of the timeout we start processing the next batch. This timeout leaves time for the browsers UI thread 
		 * 	 	to receive and process actions by the user and give the tree a responsive feel. 
		 *
		 * There are two cases where no nodes are drawn when schedule is called: 
		 * 	1. There is already a batch of nodes queued up to process. In this case, when the nodes finish processing, they automatically
		 * 	call schedule again
		 * 	
		 * 	2. There are no remaining nodes to process.
		 */
		schedule: function() {
			// console.debug("Schedule count", ++this.scheduleCount);
			if (this.batchTimeout) return;
			if (this.remainingNodes.length === 0) return;

			if (this.remainingNodes.length < BATCH_SIZE) {
				this.waitTimeout && window.clearTimeout(this.waitTimeout);
				this.waitTimeout = window.setTimeout(dojo.hitch(this, this._startBatch), 25);
			} else {
				this.waitTimeout && window.clearTimeout(this.waitTimeout);
				if (this.processed < BATCH_SIZE)
					this._startBatch();
				else 
					this.batchTimeout = window.setTimeout(dojo.hitch(this, this._startBatch), 50);
			}
		},

		_startBatch: function() {
//			console.debug("Start batch count", ++this.startBatchCount); 
			delete this.waitTimeout;
			delete this.batchTimeout;
			if (this.drawing) return;
			this.drawing = true;

			if (this.processed === 0) this.startTime = Date.now();

			var batch = this.remainingNodes.slice(0, BATCH_SIZE);
			this.remainingNodes.splice(0, BATCH_SIZE);

			dojo.forEach(batch, this._drawToolbar, this);

			this.drawing = false;
			if (this.remainingNodes.length > 0) {
				this.schedule();
			} 
			else {
				window.djConfig.isDebug && console.debug("Finished batch drawing of toolbars. Processed " + this.processed + " nodes in " + (Date.now() - this.startTime) + "ms."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		},

		_drawToolbar: function(node) {
			node.initToolbar();
			this.processed++;
		},

		destroy: function() {
			window.clearTimeout(this.waitTimeout);
		}
	});
})();