/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.variability.EditConfigurationDialog'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.variability', 'EditConfigurationDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var TreeRefresh = com.ibm.relm.ple.web.ui.tree.TreeRefresh;
	var Site = com.ibm.relm.ple.web.ui.Site;

	dojo.declare(
		'com.ibm.relm.ple.web.variability.EditConfigurationDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web', 'variability/templates/EditConfigurationDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.variability', 'EditConfigurationDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			site: null,

			constructor: function(args, node)
			{
				
			},

			postCreate: function() {
				this.inherited(arguments);

				var itemVersion = this.site.itemVersion;
				this._header.set('mainHtmlMessage', com.ibm.relm.ple.web.ui.formatting.formatProductIconAndTitleForItem(itemVersion)); //$NON-NLS-1$

				var viewport = dijit.getViewport();
				var dialogHeight = Math.max(viewport.h - 450, 200);

				var configurationText = itemVersion.conditionVariables;
				this._textarea.style.height = dialogHeight + 'px'; //$NON-NLS-1$
				this._textarea.value = configurationText;

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle, //$NON-NLS-1$
					width: '500px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode
				});

				this._dialog.startup();
			},

			onCancel: function(e) {
				this._dialog.close();
			},

			onSave: function(e) {
				this._messageBox.hideMessage();
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.saving);

				var saveProperties = {
					'http://jazz.net/ns/pd/extensions#conditionVariables': dojo.trim(this._textarea.value) //$NON-NLS-1$
				};

				this._okButton.disabled = true;
				this._cancelButton.disabled = true;
				this._textarea.disabled = true;

				var self = this;
				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/update_properties', //$NON-NLS-1$
					handleAs: 'json', //$NON-NLS-1$
					content: {
						site: Site.siteToJsonSite(this.site),
						deleteProperties: dojo.toJson([]),
						saveProperties: dojo.toJson(saveProperties)
					},

					handle: function() {
						message.hide();
					},

					load: function() {
						TreeRefresh.refresh({
							selectPath: self.site.path
						}).addBoth(function() {
							self._dialog.close();
						});
					},

					error: function(error, ioArgs) {
						self._okButton.disabled = false;
						self._cancelButton.disabled = false;
						self._textarea.disabled = false;
						self._messageBox.showXhrError(ioArgs.xhr);
					}
				});		
			},

			startup: function() {
				this._textarea.focus();
			},

			uninitialize: function() {
			
				this.inherited(arguments);
			}
		}
	);

})();