/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.explorer.web.ui.internal.PaginationControl_Fuzzy'); //$NON-NLS-1$

dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojo.NodeList-manipulate'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.explorer.web.ui.internal', 'PaginationControl_FuzzyMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

	var messages_ = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.ui.internal', 'PaginationControl_FuzzyMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	dojo.mixin(com.ibm.relm.explorer.web.ui.internal.PaginationControl_Fuzzy, {
		/**
		 * Decorates a pagination control to support showing 'of about' in the result
		 * count for fuzzy results where the total count changes as we page farther in.
		 * Hopefully, it's more accurate with time.
		 * Will also hide the "first" and "last" button, since we don't know where the
		 * last page is--only the next.
		 * @param  {jazz.ui.PaginationControl} paginationControl a plain pagination control.
		 */
		decorate: function(paginationControl) {
			var root = dojo.NodeList(paginationControl.domNode);

			var children = root.children();
			children.first().addClass('relm-PaginationControl-Fuzzy-hidden'); //$NON-NLS-1$
			children.last().addClass('relm-PaginationControl-Fuzzy-hidden'); //$NON-NLS-1$

			// between two .number there is the word ' of '. Make it vaguer
			paginationControl._relm_ofNode = root.children('.number').at(0).next(); //$NON-NLS-1$

			/**
			 * Hides and shows First and Last buttons which apply during normal results, but not fuzzy
			 * @param  {Boolean} isFuzzy true to hide controls not applicable to fuzzy results.
			 */
			paginationControl.relm_toggleFuzzyPresentation = function(isFuzzy) {
				dojo.toggleClass(this.domNode, 'relm-PaginationControl-Fuzzy', isFuzzy); //$NON-NLS-1$
			};

			// Override .setTotal to change text to '...on about...' when the result total is fuzzy.
			var oldSetTotal = paginationControl.setTotal;

			/**
			 * Allows updating the message about the number of results to be
			 * vague about the total.
			 * @param {number}  total        the total number of results to show.
			 * @param {Boolean} isFuzzyTotal true of the results is fuzzy (may
			 *     change during paging), which will change the message shown
			 *     to the user.
			 */
			 paginationControl.setTotal = function(total, isFuzzyTotal) {
				if (this._relm_ofNode) {
					this._relm_ofNode.text(' ' + //$NON-NLS-1$
						(isFuzzyTotal ? messages_.numberOfAboutNumberResults : messages_.numberOfNumberResults) +
						' '); //$NON-NLS-1$
				}

				oldSetTotal.apply(this, arguments);
			};

		}
	});

})();
