/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.explorer.web.ui.internal.SearchBox_Fuzzy'); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.ui.internal.PaginationControl_Fuzzy'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$

dojo.mixin(com.ibm.relm.explorer.web.ui.internal.SearchBox_Fuzzy, {
	/**
	 * Decorates a search box, specifically the banner search box, so that it will
	 * subsequently decorate it's pagination control to support fuzzy search
	 * (i.e., don't know the exact total number of results). This odd way is
	 * needed since we can't subclass SearchBox since we don't control where the
	 * instance is created in the banner.
	 */
	decorate: function(searchBox) {
		var oldShowResults = searchBox._showResults;

		searchBox._showResults = function(results, page, query) {
			// See if there's already a pagination control before showing results.
			var bannerSearchNode = this._bannerSearchArea && this._bannerSearchArea.domNode;

			if (bannerSearchNode) {
				var oldPaginationControlNode = dojo.query('.jazz-ui-PaginationControl ', this._bannerSearchArea.domNode)[0]; //$NON-NLS-1$
			}

			var returnValue = oldShowResults.apply(this, arguments);
			if (!bannerSearchNode) return returnValue;

			var newPaginationControlNode = dojo.query('.jazz-ui-PaginationControl ', this._bannerSearchArea.domNode)[0]; //$NON-NLS-1$

			if (oldPaginationControlNode !== newPaginationControlNode) {
				// If there is now a pagination control, it's brand new. So
				// decorate now to support fuzzy paging of index search
				// results.
				if (newPaginationControlNode) {
					var newPaginationControl = dijit.getEnclosingWidget(newPaginationControlNode);
					com.ibm.relm.explorer.web.ui.internal.PaginationControl_Fuzzy.decorate(newPaginationControl);
					this._relm_paginationControl = newPaginationControl;
				}
			}
			else if (newPaginationControlNode === null) {
				delete this._relm_paginationControl;
			}

			if (this._relm_paginationControl) {
				this._relm_paginationControl.relm_toggleFuzzyPresentation(!!results.isFuzzy);

				// Update the total results in a way that won't reset us to
				// page 1. Must set a proper 'start' page to correspond to the
				// current page first.
				var pager = this._relm_paginationControl;
				var startPageOffset = Math.floor(pager._start / pager._getPageSize());
				pager.startPage = startPageOffset;
				pager.setTotal(results.totalResults, results.isFuzzyTotal);	
			}

			return returnValue;
		};
	}
});
